/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.db;

import com.netapp.oci.platform.common.table.*;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.*;

/**
 * @author Al Velella
 *         Date: Nov 16, 2012
 *         Time: 10:16:52 AM
 */
final public class NetAppModel {
    public static final NetAppModel INSTANCE = new NetAppModel();

    private static final TableResultParser<Map<Long, Set<Long>>> containerIdToIdsParser = new TableResultParser<Map<Long, Set<Long>>>() {
        @Override
        public List<Map<Long, Set<Long>>> parseResult(ResultSet resultSet) throws SQLException {
            Map<Long, Set<Long>> containerIdToIds = new HashMap<Long, Set<Long>>();
            while (resultSet.next()) {
                Long id = resultSet.getLong(1);
                Long containerId = resultSet.getLong(2);
                Set<Long> ids = containerIdToIds.get(containerId);
                if (ids == null) {
                    ids = new HashSet<Long>();
                    containerIdToIds.put(containerId, ids);
                }
                ids.add(id);
            }
            return Collections.singletonList(containerIdToIds);
        }
    };

    private NetAppModel() {
    }

    public Map<Long, Set<Long>> getClusterIdByAggregateIds(Connection connection, Set<Long> aggregateIds) throws TableException {
        return getContainerIdToIds(connection, aggregateIds, "netapp_model.aggregate", "clusterid");
    }

    public Map<Long, Set<Long>> getClusterIdByDiskIds(Connection connection, Set<Long> diskIds) throws TableException {
        return getContainerIdToIds(connection, diskIds, "netapp_model.disk", "clusterid");
    }

    public Map<Long, Set<Long>> getClusterIdByFcPortIds(Connection connection, Set<Long> fcPortIds) throws TableException {
        return getContainerIdToIds(connection, fcPortIds, "netapp_model.fcp_port", "clusterid");
    }

    public Map<Long, Set<Long>> getClusterIdByNetworkPortIds(Connection connection, Set<Long> networkPortIds) throws TableException {
        return getContainerIdToIds(connection, networkPortIds, "netapp_model.network_port", "clusterid");
    }

    public Map<Long, Set<Long>> getClusterIdByNodeIds(Connection connection, Set<Long> nodeIds) throws TableException {
        return getContainerIdToIds(connection, nodeIds, "netapp_model.node", "clusterid");
    }

    public Map<Long, Set<Long>> getClusterIdByProcessorIds(Connection connection, Set<Long> processorIds) throws TableException {
        return getContainerIdToIds(connection, processorIds, "netapp_model.processor", "clusterid");
    }
    
    public Map<Long, Set<Long>> getClusterIdByQosVolumeWorkloadIds(Connection connection, Set<Long> qosVolumeWorkloadIds) throws TableException {
        return getContainerIdToIds(connection, qosVolumeWorkloadIds, "netapp_model.qos_volume_workload", "clusterid");
    }

    public Map<Long, Set<Long>> getClusterIdByQosWorkloadConstituentIds(Connection connection, Set<Long> qosWorkloadConstituentIds) throws TableException {
        return getContainerIdToIds(connection, qosWorkloadConstituentIds, "netapp_model.qos_workload_constituent", "clusterid");
    }

    public Map<Long, Set<Long>> getClusterIdByQosWorkloadNodeRelationshipIds(Connection connection, Set<Long> qosWorkloadNodeRelationshipIds) throws TableException {
        return getContainerIdToIds(connection, qosWorkloadNodeRelationshipIds, "netapp_model.qos_workload_node_relationship", "clusterid");
    }

    public Map<Long, Set<Long>> getClusterIdByQosWorkloadDetailIds(Connection connection, Set<Long> qosVolumeWorkloadDetailIds) throws TableException {
        return getContainerIdToIds(connection, qosVolumeWorkloadDetailIds, "netapp_model.qos_workload_detail", "clusterid");
    }

    public Map<Long, Set<Long>> getClusterIdByQosServiceCenterIds(Connection connection, Set<Long> qosServiceCenterIds) throws TableException {
        return getContainerIdToIds(connection, qosServiceCenterIds, "netapp_model.qos_service_center", "clusterid");
    }

    public Map<Long, Set<Long>> getClusterIdByVserverIds(Connection connection, Set<Long> vserverIds) throws TableException {
        return getContainerIdToIds(connection, vserverIds, "netapp_model.vserver", "clusterid");
    }

    public Map<Long, Set<Long>> getVserverIdByVolumeIds(Connection connection, Set<Long> volumeIds) throws TableException {
        return getContainerIdToIds(connection, volumeIds, "netapp_model.volume", "vserverid");
    }

    public Map<Long, Set<Long>> getVserverIdByFcpLifIds(Connection connection, Set<Long> networkLifIds) throws TableException {
        return getContainerIdToIds(connection, networkLifIds, "netapp_model.fcp_lif", "vserverid");
    }

    public Map<Long, Set<Long>> getVserverIdByNetworkLifIds(Connection connection, Set<Long> networkLifIds) throws TableException {
        return getContainerIdToIds(connection, networkLifIds, "netapp_model.network_lif", "vserverid");
    }

    public Map<Long, Set<Long>> getVserverIdByLunIds(Connection connection, Set<Long> lunIds) throws TableException {
        return getContainerIdToIds(connection, lunIds, "netapp_model.lun", "vserverid");
    }

    public Long getClusterIdByOriginatorId(Connection connection, long originId) throws TableException {
        return PreparedStatementWrapper.create(connection, "SELECT objid FROM netapp_model.cluster WHERE originId=?",
                TableResultParserDefault.LONG).setNext(originId).executeQueryEx().iterator().next();
    }
    
    public Map<Long, Set<Long>> getClusterIdByHostadapterIds(Connection connection, Set<Long> hostadapterIds) throws TableException {
        return getContainerIdToIds(connection, hostadapterIds, "netapp_model.storage_adapter", "clusterid");
    }

    public long getLastDatasourceUpdateTime(Connection connection, long datasourceId, long pollStartTime) throws TableException {
        // we assume there will only be one-to-one datasource/originator configuration and not one-to-many datasource-originator
        long originatorId = PreparedStatementWrapper.executeFunctionQuery(
                connection,
                "SELECT o.id FROM acquisition.originator o JOIN acquisition.ds ON o.ds_id=ds.id WHERE ds.id=" + datasourceId,
                -1L);

        // now find the next originator update since poll-starttime
        if (originatorId != -1) {
            return PreparedStatementWrapper.executeFunctionQuery(
                    connection,
                    " SELECT" +
                            "   createtime" +
                            " FROM" +
                            "   netapp_model.update_history" +
                            " WHERE" +
                            "   starttime>=" + pollStartTime + " AND originid=" + originatorId +
                            " LIMIT 1",
                    -1);
        }

        return -1;
    }

    /**
     * Get the Map of Volume Object ID to parent Vserver Name and Volume Name from the netapp_model.volume table for the given volume object IDs.
     * @param connection database connection
     * @param targetVolumeObjIDs volume object IDs to lookup UUIDs for
     * @return the Map of Volume Object ID to String array consisting of Vserver Name and Volume Name from the netapp_model.volume table for the given volume object IDs.
     * @throws TableException when there is a problem querying the database for the given volume UUIDs.
     */
    public Map<Long, String[]> getVolumeObjIDToNameMap(final Connection connection, final Set<Long> targetVolumeObjIDs) throws TableException {
        final Map<Long, String[]> volumeNamesByObjID = new HashMap<Long, String[]>();
        final String tempTableName = "volumeManagementTemp";
        Table.createIdsTemporaryTableForJoin(connection, tempTableName, targetVolumeObjIDs);
        // Lookup map of objid to parent VServer name and volume name
        final String sql = "SELECT DISTINCT v.objid, v.name, vs.name FROM netapp_model.volume v "
                + "JOIN " + tempTableName + " t ON v.objid = t.objid "
                + "JOIN netapp_model.vserver vs ON v.vserverId = vs.objid";
        PreparedStatementWrapper.create(connection, sql, new TableResultParserDefault<Object>() {
            @Override
            public Object parseObject(ResultSet resultSet) throws SQLException {
                final Long objID = resultSet.getLong(1);
                final String volumeName = StringCache.get(resultSet.getString(2));
                final String vServerName = StringCache.get(resultSet.getString(3));
                volumeNamesByObjID.put(objID, new String[] {vServerName, volumeName});
                return null;
            }
        }).executeQueryEx();
        return volumeNamesByObjID;
    }


    public Properties getClusterDatasourceProperties(final Connection connection, final long targetClusterObjectId) throws TableException {
        final Properties dsProperties = new Properties();
        final String sql = "SELECT dsa.name, dsa.value FROM acquisition.ds_attrs dsa JOIN "
                            + "netapp_model.cluster c ON dsa.ds_id = c.datasourceId WHERE "
                            + "c.objid = " + targetClusterObjectId;
        PreparedStatementWrapper.create(connection, sql, new TableResultParserDefault<Object>() {
            @Override
            public Object parseObject(ResultSet resultSet) throws SQLException {
                final String propertyName = StringCache.get(resultSet.getString(1));
                final String propertyValue = StringCache.get(resultSet.getString(2));
                dsProperties.setProperty(propertyName, propertyValue);
                return null;
            }
        }).executeQueryEx();
        return dsProperties;
    }

    public String getQosPolicyGroupNameByObjId(final Connection connection, final long qosPolicyGroupObjId) throws TableException {
        return PreparedStatementWrapper.create(connection, "SELECT policyGroup FROM netapp_model.qos_policy_group WHERE objid=?",
                TableResultParserDefault.STRING).setNext(qosPolicyGroupObjId).executeQueryEx().iterator().next();
    }

    private Map<Long, Set<Long>> getContainerIdToIds(Connection connection, Set<Long> volumeIds, String table, String containerIdColName) throws TableException {
        Table.createIdsTemporaryTableForJoin(connection, "id_temp", volumeIds);
        return PreparedStatementWrapper.create(connection,
                " SELECT" +
                        "   objid, " + containerIdColName +
                        " FROM" +
                        "   " + table +
                        " JOIN" +
                        "   id_temp" +
                        " USING" +
                        "   (objid)", containerIdToIdsParser).executeQueryEx().iterator().next();
    }
}
