package com.netapp.oci.netapp.performance.hostadapter.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.hostadapter.interfaces.data.HostadapterMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.SampleInfo;
import com.netapp.oci.platform.framework.performance.data.Summary;

import javax.ejb.Remote;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author dpadmana
 */
@Remote
public interface HostadapterPerformanceQueryRemote {
    String JNDI_NAME = "HostadapterPerformanceQueryRemote";

    /**
     * Gets Hostadapter samples for period of time
     *
     * @param hostadapterIds set of Hostadapter ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of Hostadapter id to list ordered by time of Hostadapter samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<HostadapterMeasurements>>> getSamples(Set<Long> hostadapterIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets Hostadapter samples for period of time
     *
     * @param containerId container id
     * @param hostadapterIds set of Hostadapter ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of Hostadapter id to list ordered by time of Hostadapter samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<SampleInfo<HostadapterMeasurements>>> getSamples(Long containerId, Set<Long> hostadapterIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets hostadapter samples for period of time
     *
     * @param containerId container id
     * @param hostadapterId Hostadapter id
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return list ordered by time of Hostadapter samples data
     * @throws NetappInterfaceException if anything went wrong
     */
    List<SampleInfo<HostadapterMeasurements>> getSamples(Long containerId, Long hostadapterId, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Retrieve the latest sample for the given Hostadapter id's (hostadapterIds).  The map returned will contain only Hostadapter ids that has a sample.
     * Hostadapter ids with no samples will not be returned in the map. An empty map will be returned if no samples are found for all the Hostadapter ids passed.
     *
     * @param clusterId cluster id
     * @param hostadapterIds set of Hostadapter ids
     * @return map of Hostadapter id to latest sample SampleInfo<HostadapterMeasurements> Hostadapter ids with no samples will not be included in this map.
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, SampleInfo<HostadapterMeasurements>> getLatestSample(Long clusterId, Set<Long> hostadapterIds) throws NetappInterfaceException;

    /**
     * Gets Hostadapter summaries for Hostadapter ids
     *
     * @param hostadapterIds set of Hostadapter ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of Hostadapter id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<HostadapterMeasurements>>> getSummaries(Set<Long> hostadapterIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * Gets Hostadapter summaries for Hostadapter ids
     *
     * @param hostadapterIds set of Hostadapter ids
     * @param fromTime begining of the time period
     * @param toTime end of the time period
     * @return map of Hostadapter id to summary data
     * @throws NetappInterfaceException if anything went wrong
     */
    Map<Long, List<Summary<HostadapterMeasurements>>> getSummaries(Long clusterId, Set<Long> hostadapterIds, Date fromTime, Date toTime) throws NetappInterfaceException;

    /**
     * returns true if samples exist over the time period
     *
     * @param containerId
     * @param objId
     * @param fromTime
     * @param toTime
     * @return true if samples exist
     * @throws com.netapp.oci.platform.common.interfaces.data.performance.PerformanceException
     *          if anything went wrong
     */
    Boolean areStatisticsAvailable(Long containerId, Long objId, Date fromTime, Date toTime) throws NetappInterfaceException;
}
