/*
 * Copyright (c) 2012 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.iscsilif.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.iscsilif.interfaces.data.IscsiLifMeasurements;
import com.netapp.oci.netapp.performance.iscsilif.interfaces.data.IscsiLifRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;

import javax.ejb.Remote;
import java.util.Collection;

/**
 * @author Al Velella
 *         Date: Jul 31, 2012
 *         Time: 5:31:26 PM
 */
@Remote
public interface IscsiLifPerformanceRemote {
    String JNDI_NAME = "IscsiLifPerformanceRemote";

    /**
     * The Acq use this method to report on IscsiLif performance
     *
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey  -- uuid of cluster
     * @param vserverKey - the vserver Key Identifier
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw iSCSI LIF samples
     * @param leaseTime    the time for datasource to be registered in the in the registry (in milliseconds)
     *
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if failed to report
     */
    void reportIscsiLifSamples(Long datasourceId, String clusterKey, String vserverKey, long time, Collection<RawSample<IscsiLifMeasurements>> rawSamples, long leaseTime) throws NetappInterfaceException;

    /**
     * The Acq use this method to report on IscsiLif performance
     *
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey  -- uuid of cluster
     * @param vserverKey - the vserver Key Identifier
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw iSCSI LIF samples
     * @param leaseTime    the time for datasource to be registered in the in the registry (in milliseconds)
     *
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if failed to report
     */
    void reportIscsiLifRawSamples(Long datasourceId, String clusterKey, String vserverKey, long time, Collection<RawSample<IscsiLifRawMeasurements>> rawSamples, long leaseTime) throws NetappInterfaceException;
}
