package com.netapp.oci.netapp.performance.nic;

import java.sql.ResultSet;
import java.sql.SQLException;

import com.netapp.oci.netapp.performance.nic.interfaces.data.NicMeasurements;
import com.netapp.oci.netapp.performance.nic.interfaces.data.NicRawMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.RatioMethod;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;

public class NicMetaData {
    public static final Column[] RAW_COLUMNS = new Column[]{
        new Column("rx_bytes_rate", "bigint unsigned"),
        new Column("rx_errors_rate", "bigint unsigned"),
        new Column("tx_bytes_rate", "bigint unsigned"),
        new Column("tx_errors_rate", "bigint unsigned"),
        new Column("opm_nic_total_byte_per_sec", "bigint unsigned"),
        new Column("opm_nic_total_error_rate", "bigint unsigned")
    };
    

    public static final Column[] SAMPLE_COLUMNS = new Column[]{
        new Column("rxBytesRate", "float"),
        new Column("rxErrorsRate", "float"),
        new Column("txBytesRate", "float"),
        new Column("txErrorsRate", "float"),
        new Column("opmNicTotalBytePerSec", "float"),
        new Column("opmNicTotalErrorRate", "float"),
        new Column("opmNicUtilization", "float")
    };
    
    public static final Column[] SUMMARY_COLUMNS = new Column[]{
        new Column("rxBytesRate", "float"),
        new Column("rxErrorsRate", "float"),
        new Column("txBytesRate", "float"),
        new Column("txErrorsRate", "float"),
        new Column("opmNicTotalBytePerSec", "float"),
        new Column("opmNicTotalErrorRate", "float"),
        new Column("opmNicUtilization", "float")
    };
    
   
    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG},
        {SqlMethod.AVG}
    };

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG,
        SqlMethod.AVG
    };

    private static ObjectParser<NicRawMeasurements> rawParser;
    private static ObjectParser<NicMeasurements> sampleParser;

    static {
        rawParser = new ObjectParser<NicRawMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, NicRawMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getRxBytesRate());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getRxErrorsRate());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTxBytesRate());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTxErrorsRate());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmNicTotalBytePersec());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmNicTotalErrorRate());
            }

            public NicRawMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new NicRawMeasurements()
                .setRxBytesRate(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                .setRxErrorsRate(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                .setTxBytesRate(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                .setTxErrorsRate(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                .setOpmNicTotalBytePersec(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i))
                .setOpmNicTotalErrorRate(PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i));
            }
        };
        
        sampleParser = new ObjectParser<NicMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, NicMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getRxBytesRate());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getRxErrorsRate());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTxBytesRate());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTxErrorsRate());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmNicTotalBytePerSec());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmNicTotalErrorRate());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmNicUtilization());
            }

            public NicMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new NicMeasurements()
                .setRxBytesRate(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                .setRxErrorsRate(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                .setTxBytesRate(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                .setTxErrorsRate(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                .setOpmNicTotalBytePerSec(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                .setOpmNicTotalErrorRate(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                .setOpmNicUtilization(PerformanceTableUtils.getFloatNullable(resultSet, ++i));
                
            }
        };
    }
    
    public static ObjectParser<NicRawMeasurements> getRawParser() {
        return rawParser;
    }
    
    public static ObjectParser<NicMeasurements> getSampleParser() {
        return sampleParser;
    }
    
    public static ObjectParser<NicMeasurements> getSummaryParser() {
        return sampleParser;
    }
}
