package com.netapp.oci.netapp.performance.nic.interfaces.remote;

import java.util.Collection;

import javax.ejb.Remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.nic.interfaces.data.NicMeasurements;
import com.netapp.oci.netapp.performance.nic.interfaces.data.NicRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;

@Remote
public interface NicPerformanceRemote {
    String JNDI_NAME = "NicPerformanceRemote";
    
    /**
     * To report raw samples for nfsv3 performance
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey - the cluster Key Identifier
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw aggregate samples
     * @param leaseTime - the time for datasource to be registered in the in the registry (in milliseconds)
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if failed to report
     */
    void reportNicRawData(Long datasourceId, String clusterKey, long time, Collection<RawSample<NicRawMeasurements>> rawSamples, long leaseTime) throws NetappInterfaceException;
    
    /**
     * To report summary samples for nfsv3 performance
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey - the cluster Key Identifier
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw aggregate samples
     * @param leaseTime - the time for datasource to be registered in the in the registry (in milliseconds)
     * @throws com.netapp.oci.netapp.interfaces.data.NetappInterfaceException if failed to report
     */
    void reportNicData(Long datasourceId, String clusterKey, long time, Collection<RawSample<NicMeasurements>> rawSamples, long leaseTime) throws NetappInterfaceException;
}
