/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.qosworkload;

import com.netapp.oci.netapp.performance.qosworkload.interfaces.data.QosVolumeWorkloadMeasurements;
import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.RatioMethod;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;

import java.sql.ResultSet;
import java.sql.SQLException;

public class QosVolumeWorkloadMetaData {
    public static final Column[] SAMPLE_COLUMNS = new Column[] {
        new Column("latency", "float"),
        new Column("ops", "float"),
        new Column("totalData", "float"),
        new Column("readData", "float"),
        new Column("readLatency", "float"),
        new Column("readOps", "float"),
        new Column("writeData", "float"),
        new Column("writeLatency", "float"),
        new Column("writeOps", "float"),
        new Column("cacheMissRate", "float"),
        new Column("cacheMissRateBase", "bigint(20)")};

    public static final Column[] SUMMARY_COLUMNS = new Column[] {
        new Column("latency", "float"),
        new Column("ops", "float"),
        new Column("totalData", "float"),
        new Column("readData", "float"),
        new Column("readLatency", "float"),
        new Column("readOps", "float"),
        new Column("writeData", "float"),
        new Column("writeLatency", "float"),
        new Column("writeOps", "float"),
        new Column("cacheMissRate", "float"),
        new Column("cacheMissRateBase", "bigint(20)")};

public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][] {
        {new RatioMethod("ops")}, //latency
        {SqlMethod.AVG}, // ops
        {SqlMethod.AVG}, // totalData
        {SqlMethod.AVG}, // readData
        {new RatioMethod("readOps")}, // readLatency
        {SqlMethod.AVG}, // readOps
        {SqlMethod.AVG}, // writeData
        {new RatioMethod("writeOps")}, // writeLatency
        {SqlMethod.AVG}, // writeOps
        {new RatioMethod("cacheMissRateBase")}, // cacheMissRate
        {SqlMethod.SUM}, // cacheMissRateBase
};

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[] {
        new RatioMethod("ops"), //latency
        SqlMethod.AVG, // ops
        SqlMethod.AVG, // totalData
        SqlMethod.AVG, // readData
        new RatioMethod("readOps"), // readLatency
        SqlMethod.AVG, // readOps
        SqlMethod.AVG, // writeData
        new RatioMethod("writeOps"), // writeLatency
        SqlMethod.AVG, // writeOps
        new RatioMethod("cacheMissRateBase"), // cacheMissRate
        SqlMethod.SUM, // cacheMissRateBase
    };

    private static ObjectParser<QosVolumeWorkloadMeasurements> sampleParser;

    static {
        sampleParser = new ObjectParser<QosVolumeWorkloadMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement,
                    QosVolumeWorkloadMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCacheMissRate());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCacheMissRateBase());
            }

            public QosVolumeWorkloadMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new QosVolumeWorkloadMeasurements()
                        .setLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setTotalData(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setReadData(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setReadLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setReadOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setWriteData(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setWriteLatency(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setWriteOps(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setCacheMissRate(PerformanceTableUtils.getFloatNullable(resultSet, ++i))
                        .setCacheMissRateBase(PerformanceTableUtils.getLongNullable(resultSet, ++i));
            }
        };
    }

    public static ObjectParser<QosVolumeWorkloadMeasurements> getSummaryParser() {
        return sampleParser;
    }

    public static ObjectParser<QosVolumeWorkloadMeasurements> getSampleParser() {
        return sampleParser;
    }
}
