package com.netapp.oci.netapp.performance.shared;

import com.netapp.oci.netapp.Param;
import com.netapp.oci.netapp.performance.PerformanceConstants;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.framework.performance.tables.*;
import java.sql.Connection;
import java.util.regex.Pattern;
import org.apache.log4j.Logger;

/**
 * @author Anton Sergeev
 */
public class NonPartitionedPerformanceComponentsFactory<C, S> extends AbstractPerformanceComponentsFactory<C, S> {
    private static final Logger logger = Logger.getLogger(NonPartitionedPerformanceComponentsFactory.class);
    
    public NonPartitionedPerformanceComponentsFactory(String packageName, Column[] sampleColumns, Column[] summaryColumns,
                                                   ObjectParser<C> sampleParser, ObjectParser<S> summaryParser,
                                                   SqlMethod[] summaryCommonMethods, SqlMethod[][] sampleToSummaryMethods) {
        super(packageName, sampleColumns, summaryColumns, sampleParser, summaryParser, summaryCommonMethods, sampleToSummaryMethods);
    }
    
    public NonPartitionedPerformanceComponentsFactory(String packageName, Column[] sampleColumns, ObjectParser<C> sampleParser) {
    	super(packageName, sampleColumns, null, sampleParser, null, null, null);
    }


    public SampleTable<C> getSampleTable(Connection connection, Long containerId) {
        int maxPartitions = Pattern.matches(PerformanceConstants.SAMPLE_RAW_PACKAGE_PATTERN, packageName) ? PerformanceConstants
                .getTablesLifeSpan(Param.Key.SAMPLE_RAW_TABLES_LIFESPAN_DAYS) : PerformanceConstants
                .getTablesLifeSpan(Param.Key.SAMPLE_TABLES_LIFESPAN_DAYS);
        if (logger.isDebugEnabled()) {        
            logger.debug("getSampleTable: packageName = " + packageName + ", maxPartitions = " + maxPartitions);
        }
        SampleTable<C> table = new PartitionedSampleTable<C>(connection, sampleParser, getPackagePrefix(),
                sampleColumns, maxPartitions);
        table.setDatabase(PerformanceConstants.NETAPP_PERFORMANCE_DATABASE);
        return table;
    }

    public SummaryTable<S> getSummaryTable(Connection connection, Long containerId) {
        SummaryTable<S> table = new InsertSelectSummaryTable<S>(connection, getPackagePrefix(),
                summaryColumns, summaryParser);
        table.setDatabase(PerformanceConstants.NETAPP_PERFORMANCE_DATABASE);
        return table;
    }

    public String getPackagePrefix() {
        return packageName;
    }

    public String getPerformancePackageId(Long containerId) {
        return getPackagePrefix();
    }
}
