/*
 * Copyright (c) 2011 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.performance.volume;

import com.netapp.oci.netapp.performance.shared.PerformanceTableUtils;
import com.netapp.oci.netapp.performance.volume.interfaces.data.VolumeMeasurements;
import com.netapp.oci.netapp.performance.volume.interfaces.data.VolumeRawMeasurements;
import com.netapp.oci.platform.common.table.ObjectParser;
import com.netapp.oci.platform.common.table.PreparedStatementWrapper;
import com.netapp.oci.platform.common.table.TableException;
import com.netapp.oci.platform.framework.performance.tables.Column;
import com.netapp.oci.platform.framework.performance.tables.SqlMethod;
import com.netapp.oci.platform.framework.performance.tables.RatioMethod;

import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * @author Al Velella
 *         Date: Oct 14, 2011
 *         Time: 11:33:18 AM
 */
public class VolumeMetaData {
    public static final Column[] RAW_COLUMNS = new Column[]{
        new Column("avg_latency", "bigint unsigned"),
        new Column("cifs_other_latency", "bigint unsigned"),
        new Column("cifs_other_ops", "bigint unsigned"),
        new Column("cifs_read_latency", "bigint unsigned"),
        new Column("cifs_read_ops", "bigint unsigned"),
        new Column("cifs_write_latency", "bigint unsigned"),
        new Column("cifs_write_ops", "bigint unsigned"),
        new Column("fcp_other_latency", "bigint unsigned"),
        new Column("fcp_other_ops", "bigint unsigned"),
        new Column("fcp_read_latency", "bigint unsigned"),
        new Column("fcp_read_ops", "bigint unsigned"),
        new Column("fcp_write_latency", "bigint unsigned"),
        new Column("fcp_write_ops", "bigint unsigned"),
        new Column("iscsi_other_latency", "bigint unsigned"),
        new Column("iscsi_other_ops", "bigint unsigned"),
        new Column("iscsi_read_latency", "bigint unsigned"),
        new Column("iscsi_read_ops", "bigint unsigned"),
        new Column("iscsi_write_latency", "bigint unsigned"),
        new Column("iscsi_write_ops", "bigint unsigned"),
        new Column("nfs_other_latency", "bigint unsigned"),
        new Column("nfs_other_ops", "bigint unsigned"),
        new Column("nfs_read_latency", "bigint unsigned"),
        new Column("nfs_read_ops", "bigint unsigned"),
        new Column("nfs_write_latency", "bigint unsigned"),
        new Column("nfs_write_ops", "bigint unsigned"),
        new Column("other_latency", "bigint unsigned"),
        new Column("other_ops", "bigint unsigned"),
        new Column("read_latency", "bigint unsigned"),
        new Column("read_ops", "bigint unsigned"),
        new Column("total_ops", "bigint unsigned"),
        new Column("total_protocol_other_latency", "varchar"),
        new Column("total_protocol_read_latency", "varchar"),
        new Column("total_protocol_write_latency", "varchar"),
        new Column("write_latency", "bigint unsigned"),
        new Column("write_ops", "bigint unsigned"),
        new Column("opm_vol_nfs_ops", "bigint unsigned"),
        new Column("opm_vol_cifs_ops", "bigint unsigned"),
        new Column("opm_vol_iscsi_ops", "bigint unsigned"),
        new Column("opm_vol_fcp_ops", "bigint unsigned"),
    };
	
    public static final Column[] SAMPLE_COLUMNS = new Column[]{
            new Column("avgLatency", "float"),
            new Column("cifsOtherLatency", "float"),
            new Column("cifsOtherOps", "float"),
            new Column("cifsReadData", "float"),
            new Column("cifsReadLatency", "float"),
            new Column("cifsReadOps", "float"),
            new Column("cifsWriteData", "float"),
            new Column("cifsWriteLatency", "float"),
            new Column("cifsWriteOps", "float"),
            new Column("fcpOtherLatency", "float"),
            new Column("fcpOtherOps", "float"),
            new Column("fcpReadData", "float"),
            new Column("fcpReadLatency", "float"),
            new Column("fcpReadOps", "float"),
            new Column("fcpWriteData", "float"),
            new Column("fcpWriteLatency", "float"),
            new Column("fcpWriteOps", "float"),
            new Column("iscsiOtherLatency", "float"),
            new Column("iscsiOtherOps", "float"),
            new Column("iscsiReadData", "float"),
            new Column("iscsiReadLatency", "float"),
            new Column("iscsiReadOps", "float"),
            new Column("iscsiWriteData", "float"),
            new Column("iscsiWriteLatency", "float"),
            new Column("iscsiWriteOps", "float"),
            new Column("nfsOtherLatency", "float"),
            new Column("nfsOtherOps", "float"),
            new Column("nfsReadData", "float"),
            new Column("nfsReadLatency", "float"),
            new Column("nfsReadOps", "float"),
            new Column("nfsWriteData", "float"),
            new Column("nfsWriteLatency", "float"),
            new Column("nfsWriteOps", "float"),
            new Column("otherLatency", "float"),
            new Column("otherOps", "float"),
            new Column("readData", "float"),
            new Column("readLatency", "float"),
            new Column("readOps", "float"),
            new Column("sanOtherLatency", "float"),
            new Column("sanOtherOps", "float"),
            new Column("sanReadData", "float"),
            new Column("sanReadLatency", "float"),
            new Column("sanReadOps", "float"),
            new Column("sanWriteData", "float"),
            new Column("sanWriteLatency", "float"),
            new Column("sanWriteOps", "float"),
            new Column("totalOps", "float"),
            new Column("writeData", "float"),
            new Column("writeLatency", "float"),
            new Column("writeOps", "float"),
            new Column("OpmVolNfsOps", "float"),
            new Column("OpmVolCifsOps", "float"),
            new Column("OpmVolIscsiOps", "float"),
            new Column("OpmVolFcpOps", "float"),
    };

    public static final Column[] SUMMARY_COLUMNS = new Column[]{
            new Column("avgLatency", "float"),
            new Column("cifsOtherLatency", "float"),
            new Column("cifsOtherOps", "float"),
            new Column("cifsReadData", "float"),
            new Column("cifsReadLatency", "float"),
            new Column("cifsReadOps", "float"),
            new Column("cifsWriteData", "float"),
            new Column("cifsWriteLatency", "float"),
            new Column("cifsWriteOps", "float"),
            new Column("fcpOtherLatency", "float"),
            new Column("fcpOtherOps", "float"),
            new Column("fcpReadData", "float"),
            new Column("fcpReadLatency", "float"),
            new Column("fcpReadOps", "float"),
            new Column("fcpWriteData", "float"),
            new Column("fcpWriteLatency", "float"),
            new Column("fcpWriteOps", "float"),
            new Column("iscsiOtherLatency", "float"),
            new Column("iscsiOtherOps", "float"),
            new Column("iscsiReadData", "float"),
            new Column("iscsiReadLatency", "float"),
            new Column("iscsiReadOps", "float"),
            new Column("iscsiWriteData", "float"),
            new Column("iscsiWriteLatency", "float"),
            new Column("iscsiWriteOps", "float"),
            new Column("nfsOtherLatency", "float"),
            new Column("nfsOtherOps", "float"),
            new Column("nfsReadData", "float"),
            new Column("nfsReadLatency", "float"),
            new Column("nfsReadOps", "float"),
            new Column("nfsWriteData", "float"),
            new Column("nfsWriteLatency", "float"),
            new Column("nfsWriteOps", "float"),
            new Column("otherLatency", "float"),
            new Column("otherOps", "float"),
            new Column("readData", "float"),
            new Column("readLatency", "float"),
            new Column("readOps", "float"),
            new Column("sanOtherLatency", "float"),
            new Column("sanOtherOps", "float"),
            new Column("sanReadData", "float"),
            new Column("sanReadLatency", "float"),
            new Column("sanReadOps", "float"),
            new Column("sanWriteData", "float"),
            new Column("sanWriteLatency", "float"),
            new Column("sanWriteOps", "float"),
            new Column("totalOps", "float"),
            new Column("writeData", "float"),
            new Column("writeLatency", "float"),
            new Column("writeOps", "float"),
            new Column("OpmVolNfsOps", "float"),
            new Column("OpmVolCifsOps", "float"),
            new Column("OpmVolIscsiOps", "float"),
            new Column("OpmVolFcpOps", "float"),
           
    };

    public static final SqlMethod[][] SAMPLE_TO_SUMMARY_METHODS = new SqlMethod[][]{
            {new RatioMethod("totalOps")},
            {new RatioMethod("cifsOtherOps")},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {new RatioMethod("cifsReadOps")},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {new RatioMethod("cifsWriteOps")},
            {SqlMethod.AVG},
            {new RatioMethod("fcpOtherOps")},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {new RatioMethod("fcpReadOps")},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {new RatioMethod("fcpWriteOps")},
            {SqlMethod.AVG},
            {new RatioMethod("iscsiOtherOps")},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {new RatioMethod("iscsiReadOps")},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {new RatioMethod("iscsiWriteOps")},
            {SqlMethod.AVG},
            {new RatioMethod("nfsOtherOps")},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {new RatioMethod("nfsReadOps")},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {new RatioMethod("nfsWriteOps")},
            {SqlMethod.AVG},
            {new RatioMethod("otherOps")},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {new RatioMethod("readOps")},
            {SqlMethod.AVG},
            {new RatioMethod("sanOtherOps")},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {new RatioMethod("sanReadOps")},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {new RatioMethod("sanWriteOps")},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {new RatioMethod("writeOps")},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
            {SqlMethod.AVG},
    };

    public static final SqlMethod[] SUMMARY_COMMON_METHODS = new SqlMethod[]{
            new RatioMethod("totalOps"),
            new RatioMethod("cifsOtherOps"),
            SqlMethod.AVG,
            SqlMethod.AVG,
            new RatioMethod("cifsReadOps"),
            SqlMethod.AVG,
            SqlMethod.AVG,
            new RatioMethod("cifsWriteOps"),
            SqlMethod.AVG,
            new RatioMethod("fcpOtherOps"),
            SqlMethod.AVG,
            SqlMethod.AVG,
            new RatioMethod("fcpReadOps"),
            SqlMethod.AVG,
            SqlMethod.AVG,
            new RatioMethod("fcpWriteOps"),
            SqlMethod.AVG,
            new RatioMethod("iscsiOtherOps"),
            SqlMethod.AVG,
            SqlMethod.AVG,
            new RatioMethod("iscsiReadOps"),
            SqlMethod.AVG,
            SqlMethod.AVG,
            new RatioMethod("iscsiWriteOps"),
            SqlMethod.AVG,
            new RatioMethod("nfsOtherOps"),
            SqlMethod.AVG,
            SqlMethod.AVG,
            new RatioMethod("nfsReadOps"),
            SqlMethod.AVG,
            SqlMethod.AVG,
            new RatioMethod("nfsWriteOps"),
            SqlMethod.AVG,
            new RatioMethod("otherOps"),
            SqlMethod.AVG,
            SqlMethod.AVG,
            new RatioMethod("readOps"),
            SqlMethod.AVG,
            new RatioMethod("sanOtherOps"),
            SqlMethod.AVG,
            SqlMethod.AVG,
            new RatioMethod("sanReadOps"),
            SqlMethod.AVG,
            SqlMethod.AVG,
            new RatioMethod("sanWriteOps"),
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            new RatioMethod("writeOps"),
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
            SqlMethod.AVG,
    };
    
    private static ObjectParser<VolumeRawMeasurements> rawParser;
    private static ObjectParser<VolumeMeasurements> sampleParser;

    static {
        rawParser = new ObjectParser<VolumeRawMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, VolumeRawMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsOtherLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getFcpOtherLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getFcpOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getFcpReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getFcpReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getFcpWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getFcpWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getIscsiOtherLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getIscsiOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getIscsiReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getIscsiReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getIscsiWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getIscsiWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsOtherLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalProtocolOtherLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalProtocolReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalProtocolWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmVolNfsOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmVolCifsOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmVolIscsiOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmVolFcpOps());
            }

            public VolumeRawMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new VolumeRawMeasurements(
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getStringNullable(resultSet, ++i),
                        PerformanceTableUtils.getStringNullable(resultSet, ++i),
                        PerformanceTableUtils.getStringNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i),
                        PerformanceTableUtils.getBigIntegerNullable(resultSet, ++i));
            }
        };
    
        sampleParser = new ObjectParser<VolumeMeasurements>() {
            public void prepareObjectDataForInsert(PreparedStatementWrapper preparedStatement, VolumeMeasurements objectData) throws TableException {
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getAvgLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsOtherLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getCifsWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getFcpOtherLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getFcpOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getFcpReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getFcpReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getFcpReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getFcpWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getFcpWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getFcpWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getIscsiOtherLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getIscsiOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getIscsiReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getIscsiReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getIscsiReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getIscsiWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getIscsiWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getIscsiWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsOtherLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getNfsWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSanOtherLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSanOtherOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSanReadData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSanReadLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSanReadOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSanWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSanWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getSanWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getTotalOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteData());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteLatency());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getWriteOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmVolNfsOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmVolCifsOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmVolIscsiOps());
                PerformanceTableUtils.setNextNullable(preparedStatement, objectData.getOpmVolFcpOps());
            }

            public VolumeMeasurements parseObjectData(ResultSet resultSet, int startIndex) throws SQLException {
                int i = startIndex - 1;
                return new VolumeMeasurements(
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i),
                        PerformanceTableUtils.getFloatNullable(resultSet, ++i));
            }
        };
    }

    public static ObjectParser<VolumeMeasurements> getSummaryParser() {
        return sampleParser;
    }

    public static ObjectParser<VolumeMeasurements> getSampleParser() {
        return sampleParser;
    }
    
    public static ObjectParser<VolumeRawMeasurements> getRawParser() {
        return rawParser;
    }

}
