package com.netapp.oci.netapp.performance.volume.interfaces.remote;

import com.netapp.oci.netapp.interfaces.data.NetappInterfaceException;
import com.netapp.oci.netapp.performance.volume.interfaces.data.VolumeMeasurements;
import com.netapp.oci.netapp.performance.volume.interfaces.data.VolumeRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;

import javax.ejb.Remote;
import java.util.Collection;

/**
 * @author Anton Sergeev
 */
@Remote
public interface VolumePerformanceRemote {
    String JNDI_NAME = "VolumePerformanceRemote";

    /**
     * The Acq use this method to report on Volume performance
     *
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey  -- uuid of cluster
     * @param vserverKey - the Vserver Key Identifier
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw volume samples
     * @param leaseTime    the time for datasource to be registered in the in the registry (in milliseconds)
     *
     * @throws NetappInterfaceException if failed to report
     */
    void reportVolumeSamples(Long datasourceId, String clusterKey, String vserverKey, long time, Collection<RawSample<VolumeMeasurements>> rawSamples, long leaseTime) throws NetappInterfaceException;
    
    /**
     * The Acq use this method to report on Volume raw performance
     *
     * @param datasourceId - id of the datasource reporting performance data
     * @param clusterKey  -- uuid of cluster
     * @param vserverKey - the Vserver Key Identifier
     * @param time - the report time, it is the datasource responsibility to make sure that the earlier sample and the
     *               latest sample time difference is less then X minute (1 by default),
     * @param rawSamples - raw volume samples
     * @param leaseTime    the time for datasource to be registered in the in the registry (in milliseconds)
     *
     * @throws NetappInterfaceException if failed to report
     */
    void reportVolumeRawSamples(Long datasourceId, String clusterKey, String vserverKey, long time, Collection<RawSample<VolumeRawMeasurements>> rawSamples, long leaseTime) throws NetappInterfaceException;
    
}
