/*
 * Copyright (c) 2011 NetApp
 * All rights reserved
 */

package com.netapp.oci.netapp.upgrade;

import com.netapp.oci.netapp.startup.NetAppPlugin;
import com.netapp.oci.netapp.upgrade.version_1.Upgrade_1_0_0_103;
import com.netapp.oci.netapp.upgrade.version_1.Upgrade_1_0_0_69;
import com.netapp.oci.netapp.upgrade.version_1.Upgrade_1_0_0_95;
import com.netapp.oci.netapp.upgrade.version_1.Upgrade_1_0_0_96;
import com.netapp.oci.netapp.upgrade.version_1_1.Upgrade_1_1_0_10;
import com.netapp.oci.netapp.upgrade.version_1_1.Upgrade_1_1_0_31;
import com.netapp.oci.netapp.upgrade.version_1_1.Upgrade_1_1_0_6;
import com.netapp.oci.netapp.upgrade.version_1_2.*;
import com.netapp.oci.netapp.upgrade.version_1_3.Upgrade_1_3_0_13;
import com.netapp.oci.platform.upgrade.UpgradeRegistry;

/**
 * @author Al Velella
 *         Date: Nov 17, 2011
 *         Time: 2:39:30 PM
 */
public class NetAppUpgradeRegistry extends UpgradeRegistry {
    public static final UpgradeRegistry INSTANCE = new NetAppUpgradeRegistry();

    private NetAppUpgradeRegistry() {
        super(NetAppPlugin.CONFIG_DB, NetAppPlugin.PERFORMANCE_DB);

        createDB("com/netapp/oci/netapp/upgrade/create_db_netapp_model.sql");
        createDB("com/netapp/oci/netapp/upgrade/create_db_netapp_performance.sql");

        //All the older upgraders(version 0-21) have been merged to upgrader 21.
        //The starting version in the database will be 21.
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_21.sql", "Initial database schema");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_22.sql", "Add columns to vserver and node");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_23.sql", "Convert to case-insensitive strings");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_24.sql", "Updating attribute list for Cluster, Aggregate, Node, Vserver, Volume");
        //25 reserved for QoS work should it be delivered.
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_25.sql", "Add QoS configuration tables");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_26.sql", "Adding raw value columns for Enums on Cluster, Aggregate, Node, Vserver, Volume");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_27.sql", "Change table");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_28.sql", "iSCSI LIF performance");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_29.sql", "Snapshot table and Volume.oldestSnapshotTimestamp");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_30.sql", "Volume Move Info table");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_31.sql", "Param table");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_32.sql", "Adding raw value for Aggregate raidType");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_33.sql", "Update network LIF attributes");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_34.sql", "Converting TIMESTAMP columns");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_35.sql", "Network port relationship");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_36.sql", "Removing obsolete aggregate columns");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_37.sql", "Adding 'ObjectType' column to 'changes' table primary key");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_38.sql", "Improved SnapMirror support");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_39.sql", "Adding 'changed_attribute' table");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_40.sql", "More 'Disk' attributes");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_41.sql", "More SnapMirror attributes");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_42.sql", "Job_Schedule, snapshot_policy and sis_policy tables");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_43.sql", "Fix Vserver state enum");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_44.sql", "'change' table primary key and index changes");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_45.sql", "'vserver_to_aggregate' table");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_46.sql", "'snapshot_policy_vserver_relationship' table");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_48.sql", "Moved 'ownerNodeId' into 'disk' table");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_49.sql", "Increase summary data to 30 days");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_50.sql", "QoS support");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_51.sql", "Support for keeping history");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_52.sql", "Add QoS relationship");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_53.sql", "Infinivol and constituent volumes support");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_54.sql", "Resize resource key to 1024");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_55.sql", "Convert snap_mirror attributes");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_56.sql", "snap_mirror_policy and snap_mirror_policy_rule");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_57.sql", "Added new attributes to Node, Disk and SnapMirror");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_58.sql", "Adding 'ResourceKey' and raw value string columns to FCPLif and FCPPort");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_59.sql", "Increased disk's capacitySectors, rightSizeSectors column's size to BIGINT");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_60.sql", "Adding 'InterconnectType' attribute to Node; adding 'IsInFDR' and 'FailedReason' attributes to Disk");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_61.sql", "Adding derived volume style");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_62.sql", "Support volume workloads with no vserver or volume association");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_63.sql", "Adding 'ldapClientEnabled' to VServer and 'fcPortName' to StorageAdapter");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_64.sql", "updating Node nvRAMBatteryStatus enums");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_65.sql", "Add resource key and raw enum values to disk_path");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_66.sql", "Add policy foreign key to snap mirror");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_68.sql", "Updating Qtree and making resource keys 512 length");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_69.sql", new Upgrade_1_0_0_69(), "Migrate QoS data to new performance container");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_70.sql", "Add ServiceProcessor table, add attrs to VServer and Aggregate views.  Rename Change table");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_71.sql", "Add storage service support");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_72.sql", "Add datasourceId and lastupdatetime to cluster view");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_73.sql", "Add resourceKey to snap mirror policy rule");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_74.sql", "Add qos policy group support");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_75.sql", "Add routing group and route information of LIFs");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_76.sql", "Add Volume Junction parent relationship, add jobSchedule info to Snapmirror");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_77.sql", "Add remaining resource keys for acquisition objects");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_78.sql", "snap-mirror attributes for protection workflows, initiator group attributes");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_79.sql", "Change column type of snap_mirror.progressLastUpdated from VARCHAR to TIMESTAMP");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_80.sql", "Add flash_device entity, add attributes to lun and snapshotpolicyschedule entities");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_81.sql", "Resource key fixes");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_83.sql", "Add quota-type index to quota table");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_84.sql", "Add more performance statistics to update_history");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_85.sql", "Add read-only, read-write access attr to exportRule, add raw values to Lun and Portset enum attributes");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_86.sql", "Add delete_detail");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_87.sql", "Add cloneParentId to VolumeSelfRelationship, diagnosisStatus to Cluster");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_88.sql", "Add storage shelf");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_89.sql", "Add param to control recording cluster adds/deletes changes");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_90.sql", "Add routing_group_destination table to allow for multiple destinations");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_91.sql", "Add UNKNOWN as a valid ownerType for license");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_92.sql", "Node SFO attributes");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_93.sql", "Add access related attributes to volume");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_94.sql", "Add raid group isReconstructing, adjust volume security styles");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_95.sql", new Upgrade_1_0_0_95(), "Add new performance counters to qos-volume-workload");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_96.sql", new Upgrade_1_0_0_96(), "Remove unused performance counters");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_97.sql", "Don't update current sample tables");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_98.sql", "Preserve offline objects");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_99.sql", "Add timezone information to cluster");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_100.sql", "Treat Vserver NIS domains as separate objects");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_101.sql", "Revert vserver NIS columns; refer to as 'active'");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_102.sql", "New attributes on CIFS share and export policy");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_103.sql", new Upgrade_1_0_0_103(), "Remove unused performance counters again");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_104.sql", "Add FK constraint to rootVolumeId of vserver table");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_105.sql", "Correct update_history key");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_106.sql", "Correct Volume workload UUID key");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_107.sql", "Enforce unique UUIDs in key-cache");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_108.sql", "Set partnerNodeId to null if node is removed");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_109.sql", "Correct SnapshotPolicy UUID key");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_110.sql", "Correct SnapshotPolicySchedule UUID key, reset netappfoundation ds password");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_111.sql", "Remove duplicate qos workload detail");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_112.sql", "Update snaphotpolicyvserver relationship UUID");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_113.sql", "Add more processor performance counters");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_114.sql", "Add destination volume derived style to snap_mirror");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_115.sql", "Remove source location from snap mirror UUID and ResourceKey");
        register("com/netapp/oci/netapp/upgrade/version_1_0/upgrade_1_0_0_116.sql", "Increase length of snap mirror error fields");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_0.sql", "Add VServer name mapping");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_1.sql", "Add CIFS Share ACL");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_3.sql", "Add Service Center metrics");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_4.sql", "Restore user quotas");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_5.sql", "Remove source_location from snap_mirror resourceKey and uuid");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_6.sql", new Upgrade_1_1_0_6(), "Upgrade old processor tables");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_7.sql",  "Track objState for all objects");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_8.sql",  "Update performance table lifespan default");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_9.sql",  "Add volume blocks footprint");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_10.sql", new Upgrade_1_1_0_10(), "Upgrade User Quota UUIDs");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_11.sql",  "Make qtree_quota and user_quota tables case sensitive");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_12.sql",  "collect meta-data about which applications are monitoring the cluster");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_13.sql",  "Add CIFS options for CIFS share");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_14.sql",  "Add quota status to volume");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_15.sql",  "New objState index");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_16.sql",  "Add StorageShelfCable and new attributes for StorageAdapter");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_17.sql",  "To fix duplicate records with same UUID but different objStates");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_18.sql",  "Add user summary to user_quota schema, add additional attributes to Cluster and Node schema");

        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_20.sql",  "Add export policy ID to qtree");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_21.sql",  "Add createtime");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_22.sql",  "Adjust modeling of LIFs to optimize UM/OCF reconciliation queries");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_23.sql",  "Add createtime to LIF");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_24.sql",  "Add attachedSerialNumber column to storage_adapter and storage_shelf_cable");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_25.sql",  "Add partnerNode column to storage_shelf");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_26.sql",  "Make QOS Workload Detail primary keys be globally unique");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_27.sql",  "Make QOS Resource primary keys be globally unique");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_28.sql",  "Add fabricEstablished, firmwareRev & infoName columns to fcp_port");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_29.sql",  "Make QOS Workload Details and Resource primary keys large enough to be globally unique");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_30.sql",  "Change ENUM value in aggregate.raidType from MIXED to MIXED_RAID_TYPE");
        register("com/netapp/oci/netapp/upgrade/version_1_1/upgrade_1_1_0_31.sql",  new Upgrade_1_1_0_31(), "Re-attributing Deferred Workloads");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_0.sql",  new Upgrade_1_2_0_0(), "Fix for upgrade_1_1_0_25.sql");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_1.sql",  new Upgrade_1_2_0_1(), "Entry Efficiency Upgrade");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_2.sql", new Upgrade_1_2_0_2(), "Clean old databases");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_3.sql", "Storage switch");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_4.sql", "Add 2 columns(fcpLifId, iSCSIPortalGroupId) to portset_port");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_5.sql", "Update comments of uuid in 3 tables:storage_shelf_cable, network_lif and fcp_lif");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_6.sql", "Support for entry efficiency fields (i.e., usableDataSize, usableRootSize).");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_7.sql", "Support for System health alert. creating new table.");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_8.sql", "Support for vServer Snap Mirror and new Snap Mirror attributes.");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_9.sql", "Support for Metrocluster attributes.");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_10.sql", "Add alertingResourcename into system_health_alert table. ");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_11.sql", "Add RAID4 and RAID0 into raidType enum. ");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_12.sql", "Increase the size of the UUID column in job_schedule table to 512.");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_13.sql", "Add VMDISK into interfaceType and effectiveInterfaceType in disk table, add comments for ha_pair table.");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_14.sql", "Added isHealthy in storage_pool and raidType, raidTypeRaw in raid_group");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_15.sql", "Added homeNodeName, ownerNodeName and activeNodeName in disk ");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_16.sql",  "Add a column stackId to storage_shelf table");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_17.sql", "Support for IP Spaces, Broadcast Domains and Failover Groups.");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_18.sql", "Added subtype column to vserver table.");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_19.sql", "Added operationalState column to vserver table.");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_20.sql", new Upgrade_1_2_0_20(), "Created sample, current & summary tables for qos_workload_queue_dblade,qos_workload_queue_nblade, " +
                "qos_workload_detail, qos_service_center, qos_volume_workload along with qos_workload_node_relationship config table with config data population based on cluster version, " +
                "qos_workload_detail with  old sample & specific config tables backed up");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_21.sql", new Upgrade_1_2_0_21(), "Add column delayNvlogTransferWaitTime to all qos_workload_queue_dblade acquisition tables");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_22.sql", new Upgrade_1_2_0_22(), "Add cacheMissRate to qos_volume_workload and qos_workload_queue_dblade samples");    
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_23.sql", "Added isLocallyAttachedShelf column to storge_shelf table.");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_24.sql", "Added isLocalAttach column to disk table.");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_25.sql", new Upgrade_1_2_0_25(), "Add new columns to include _WAFL_CP, _BACKGROUND_DELETE and _RA_STRIPE_X contributions for cpuDBladeBackgroundServiceTime, diskHDDBackgroundServiceTime & diskSSDBackgroundServiceTime");     
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_26.sql", "Added owningNodeId column to lun_map table, and create lun_map_reporting_node table.");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_27.sql", "Rename isLocallyAttachedShelf column to isLocalAttach in storage shelf table.");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_30.sql", "Rename enum values in mtMode column--partially_switchover to partial_switchover, partially_switchback to partial_switchback");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_31.sql", new Upgrade_1_2_0_31(), "Added in new Data Source attributes to indicate whether or not to store raw and cooked statistics.");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_32.sql", "Added is-config-locked-for-changes column to the vserver table");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_33.sql", "Change table name storage_pool_spare to storage_pool_available and column spare_size to available_size");
        
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_34.sql", new Upgrade_1_2_0_34(), "Added read_partial_blocks and write_partial_blocks columns to sample_lun_raw_xxx tables.");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_35.sql", "Upgrade script for adding the ip space uuid to the broadcast domain uuid.");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_36.sql", "Adding Autovolume and Undefined enums for qos_volume_workload table, workloadclass column");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_37.sql", "Deleting qos workloads from root volumes");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_38.sql", new Upgrade_1_2_0_38(), "Added a doatsource attribute to indicate if need run dyanmic UUID conversion during a poll.");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_39.sql", "Graphite counter additions, updates to sample* tables.");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_40.sql", new Upgrade_1_2_0_40(), "Add missing counters for Graphite, updates to sample* tables.");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_41.sql", "Added percentSparesConsumed, partitioningType, partitioningTypeRaw columns to disk table; Added aggregateType column to aggregate table.");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_42.sql", "Added isCache column to disk_aggregate_relationship table.");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_43.sql", "Added enum value 'NOT_MAPPED' to 'storageType' column types of 'storage_pool' and 'storage_pool_available' tables.");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_44.sql", "Modify column type of dataInBlocks/dataOutBlocks/cmdTransfered to be bigint.");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_45.sql", "drop performance tables for nfsv4, nfsv41 and cifsvserver objects so that they are dynamically recreated with updated table attributes");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_46.sql", "added rootVolumeName for vserver");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_47.sql", "drop performance tables for nfsv3, nfsv4 and nfsv41 objects so that they are dynamically recreated with updated table attributes");       
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_48.sql", new Upgrade_1_2_0_48(), "Add base counters required to summarize data");
	    register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_49.sql", new Upgrade_1_2_0_49(), "added sysemOps to node");
        register("com/netapp/oci/netapp/upgrade/version_1_2/upgrade_1_2_0_50.sql", "Add AFF related attributes in the disk and node tables");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_1.sql", "created 'lun_import' table for supporting WFA foreign lun import");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_2.sql", "upgrade failover groups and add the failover group relationship table");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_3.sql", "upgrade license table");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_4.sql", "upgrade plex table with resync percentage column");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_5.sql", "added isExportPolicyInherited for qtree table");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_6.sql", "created two new performance data retention parameters");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_7.sql", "upgrade volume table with cachingPolicy and cachingPolicyRaw column");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_8.sql", "added mtRemoteConfigurationState, mtRemoteConfigurationStateRaw, mtRemoteMode, mtRemoteModeRaw,mtConfigurationType for cluster table.");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_9.sql", "upgrade disk and vserver table with percentRatedLifeUsed and isSmbEncryptionRequired columns");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_10.sql", "Flag whether automatic switchover has been enabled or disabled for the cluster");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_11.sql", "run the procedures for performance data migration");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_12.sql", "reset SUMMARY_TABLES_LIFESPAN_DAYS param to 390");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_13.sql", new Upgrade_1_3_0_13(), "Add column processorElapsedTime to all processor performance tables");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_14.sql", "Remove QtreeUUID foreign key constraint from netapp_model.lun table");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_15.sql", "Stored procedure for adding new columns for newly added counters in OPM 2.0 ");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_16.sql", "Add columns for newly added counters in OPM 2.0 for Aggregate from netapp_performance.sample_aggregate_%, netapp_performance.sample_aggregate_raw_% and netapp_performance.summary_aggregate_% tables");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_17.sql", "Add AFF related attributes in the disk and node tables");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_18.sql", "Add columns for newly added counters in OPM 2.0 for LUN");        
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_19.sql", "Add new columns for synthetic counters for fcp_Lif, iscsi_lif and fcp_port which are not dependant on other objects");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_20.sql", "Add new columns for synthetic counters for volume");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_21.sql", "Create metrocluster_operation table");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_22.sql", "Add targetWWNN column to disk_path table");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_23.sql", "Stored procedure for dropping existing columns ");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_24.sql", "Add sslFipsEnabled column to cluster table");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_25.sql", "Adding support for cluster peer");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_26.sql", "Stored procedure for changing existing column names");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_27.sql", "Renaming sample_nic and summary_nic columns to cammelCase & Add columns for newly added counters in OPM 2.0 for NIC Common from netapp_performance.sample_nic$, netapp_performance.sample_nic_raw$ and netapp_performance.summary_nic$ tables");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_28.sql", "Add columns for iscslifVserver");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_29.sql", "Adding, Removing and renaming new columns for synthetic counters for lif and node");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_30.sql", "Adding synthetic counters for lifVserver");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_31.sql", "nfsv3, nfsv4 and nfsv41");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_32.sql", "Adding new columns for synthetic counters for disk");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_33.sql", "Removing synthetic utilization counters for nic common raw table.");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_34.sql", "Make nodeID column in metrocluster_operation table nullable");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_35.sql", "added fcpLifServer and cifsVserver counters");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_36.sql", "fcpPort Derived counters");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_37.sql", "Stored procedure to calculate node Utilization in the summary table");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_38.sql", "Adding new columns for synthetic counters for node");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_39.sql", "Adding new columns to network_lif and vserver tables");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_40.sql", "Stored Procedure to rename table");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_41.sql", "renaming (sample/summary)_vserver table to (sample/summary)_volumeVserver");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_42.sql", "Adding new columns for synthetic counters for aggregate.");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_43.sql", "Adding stored procedure for summary of Node table");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_44.sql", "Adding new TCI columns for synthetic counters for node");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_45.sql", "Create vserver_peer table");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_46.sql", "fcp port derived counters added to summary_fcpport");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_47.sql", "Create network_subnet table.");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_48.sql", "Fixed a misspelled enum peerState value in vserver_peer table");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_49.sql", "Changed the type of vserver#activeNisServers column to TEXT");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_50.sql", "Changed stored procedure call, correcting the parens to compute node utlilization");
        register("com/netapp/oci/netapp/upgrade/version_1_3/upgrade_1_3_0_51.sql", "Create snap_mirror_history table");
    }

    public Entry getDropViewsUpgrader() {
        return new Entry("com/netapp/oci/netapp/upgrade/drop_views.sql", null, "Drop OCI Essentials views");
    }

    public Entry getCreateViewsUpgrader() {
        return new Entry("com/netapp/oci/netapp/upgrade/create_views.sql", null, "Create OCI Essentials views");
    }

    public String getSqlDirBase() {
        return "com/netapp/oci/netapp/upgrade/";
    }
}
