CREATE TABLE netapp_model.qos_policy_group (
	objid BIGINT(20) NOT NULL,
	originid BIGINT(20) NOT NULL,
	UUID VARCHAR(255) NOT NULL COMMENT 'Policy group UUID.  ZAPIs: qos-policy-group-get-iter.qos-policy-group-info.uuid',
	resourceKey VARCHAR(512) NOT NULL COMMENT 'DERIVED.  A structured key that may be used to uniquely identify this resource globally.',
    clusterId BIGINT(20) NOT NULL,
    policyGroup VARCHAR(255) DEFAULT NULL COMMENT 'Name of the policy group.  ZAPIs: qos-policy-group-get-iter.qos-policy-group-info.policy-group',
    pgIdentifier BIGINT(20) DEFAULT NULL COMMENT 'The ID used to identify the policy group. ZAPIs: qos-policy-group-get-iter.qos-policy-group-info.pgid ',
    policyGroupClass ENUM('NOT_MAPPED','PRESET','USER_DEFINED','SYSTEM_DEFINED') DEFAULT NULL COMMENT 'Class for the policy group. ZAPIs: qos-policy-group-get-iter.qos-policy-group-info.policy-group-class',
    policyGroupClassRaw VARCHAR(255) DEFAULT NULL,
    maxThroughput VARCHAR(255) DEFAULT NULL COMMENT 'Maximum throughput defined by this policy. It is specified in terms of IOPS or MB/s, and the range is zero to infinity.  ZAPIs: qos-policy-group-get-iter.qos-policy-group-info.max-throughput ',
    checksum INT NOT NULL,
	changeDetailChecksum INT NOT NULL,
	updatetime BIGINT(20) NOT NULL,
	KEY updatetime_index (updatetime),
    KEY originId_index (originid),
	CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
	PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;

ALTER TABLE netapp_model.vserver ADD COLUMN qosPolicyGroupId BIGINT(20) DEFAULT NULL AFTER snapshotPolicyId;
ALTER TABLE netapp_model.vserver ADD CONSTRAINT FOREIGN KEY FK_qosPolicyGroupId (qosPolicyGroupId) REFERENCES netapp_model.qos_policy_group (objid);

ALTER TABLE netapp_model.volume ADD COLUMN qosPolicyGroupId BIGINT(20) DEFAULT NULL AFTER storageServiceId;
ALTER TABLE netapp_model.volume ADD CONSTRAINT FOREIGN KEY FK_qosPolicyGroupId (qosPolicyGroupId) REFERENCES netapp_model.qos_policy_group (objid);

ALTER TABLE netapp_model.lun ADD COLUMN qosPolicyGroupId BIGINT(20) DEFAULT NULL AFTER clusterId;
ALTER TABLE netapp_model.lun ADD CONSTRAINT FOREIGN KEY FK_qosPolicyGroupId (qosPolicyGroupId) REFERENCES netapp_model.qos_policy_group (objid);

CREATE TABLE netapp_model.qos_policy_group_vserver_relationship (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the referenced QoS policy groupI',
    vserverId BIGINT(20) NULL COMMENT 'ID of the vserver the QoS Policy Group references',
    checksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    changeDetailChecksum INT NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    PRIMARY KEY (objid),
    CONSTRAINT FOREIGN KEY FK_objid (objid) REFERENCES netapp_model.qos_policy_group (objid),
    CONSTRAINT FOREIGN KEY FK_vserverid (vserverId) REFERENCES netapp_model.vserver (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;