-- Copyright (c) 2013 NetApp. All rights reserved

CREATE TABLE netapp_model.change_delete_detail (
  id BIGINT(20) NOT NULL AUTO_INCREMENT,
  deleteTime BIGINT(20) NOT NULL COMMENT 'The time that the object was deleted',
  objectType VARCHAR(255) NOT NULL COMMENT 'Type of the object that was deleted',
  objId BIGINT(20) NOT NULL COMMENT 'ID of the object that was deleted',
  clusterId BIGINT(20) NOT NULL COMMENT 'ID of the cluster associated to the deleted object',
  nodeId BIGINT(20) NULL COMMENT 'ID of the node associated to the object or NULL if object is not a child of node',
  vserverId BIGINT(20) NULL COMMENT 'ID of the vserver associated to the object or NULL if object is not a child of vserver',
  aggregateId BIGINT(20) NULL COMMENT 'ID of the aggregate associated to the object or NULL if object is not a child of aggregate',
  volumeId BIGINT(20) NULL COMMENT 'ID of the volume associated to the object or NULL if object is not a child of volume',
  name VARCHAR(255) NULL COMMENT 'Name of deleted object or NULL if object does not have a name',
  ancestorDeleted BOOL NULL COMMENT 'True if the object\'s node, vserver, aggregate or volume was also removed in the same transaction',
  PRIMARY KEY (id),
  KEY deletetime_index (deleteTime),
  KEY deletetime_objid_index (deleteTime, objId),
  KEY objid_index (objId)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;

ALTER TABLE netapp_model.update_history ADD COLUMN createTime BIGINT(20) NOT NULL AFTER starttime;
UPDATE netapp_model.update_history set createTime=startTime;

-- update uuid comment
ALTER TABLE netapp_model.snapshot MODIFY COLUMN uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the snapshot. Not available in ZAPI. Uses {VolumeUUID}.{SnapshotInstanceUUID}';

-- normalize clusterId,vserverId,volumeId,nodeId columns so they can be easily used in delete_details

-- drop the keys and constraints, but note that destinationNodeId is missing a foreign key
ALTER TABLE netapp_model.snap_mirror DROP FOREIGN KEY snap_mirror_ibfk_1;
ALTER TABLE netapp_model.snap_mirror DROP FOREIGN KEY snap_mirror_ibfk_2;
ALTER TABLE netapp_model.snap_mirror DROP FOREIGN KEY snap_mirror_ibfk_3;

ALTER TABLE netapp_model.snap_mirror DROP KEY FK_destinationClusterId;
ALTER TABLE netapp_model.snap_mirror DROP KEY FK_destinationVserverId;
ALTER TABLE netapp_model.snap_mirror DROP KEY FK_destinationVolumeId;

-- rename the columns
ALTER TABLE netapp_model.snap_mirror CHANGE COLUMN destinationClusterId clusterId BIGINT(20) NOT NULL;
ALTER TABLE netapp_model.snap_mirror CHANGE COLUMN destinationVserverId vserverId BIGINT(20) NOT NULL;
ALTER TABLE netapp_model.snap_mirror CHANGE COLUMN destinationVolumeId volumeId BIGINT(20) NOT NULL;
ALTER TABLE netapp_model.snap_mirror CHANGE COLUMN destinationNodeId nodeId BIGINT(20) DEFAULT NULL;

-- recreate the foreign key and constraints
ALTER TABLE netapp_model.snap_mirror ADD CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid);
ALTER TABLE netapp_model.snap_mirror ADD CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid);
ALTER TABLE netapp_model.snap_mirror ADD CONSTRAINT FOREIGN KEY FK_volumeId(volumeId) REFERENCES netapp_model.volume (objid);
ALTER TABLE netapp_model.snap_mirror ADD CONSTRAINT FOREIGN KEY FK_nodeId (nodeId) REFERENCES netapp_model.node (objid);
