-- Copyright (c) 2013 NetApp. All rights reserved

CREATE TABLE netapp_model.storage_shelf (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the storage shelf. Derived from ShelfUUID. Not from ZAPI',
    resourceKey VARCHAR(512) NOT NULL COMMENT 'DERIVED. A structured key that may be used to uniquely identify this resource globally',
    clusterId BIGINT(20) NOT NULL,
    nodeId BIGINT(20) NOT NULL,
    shelfUuid VARCHAR(255) DEFAULT NULL COMMENT 'Shelf unique identifier that distinguishes it from other shelves manufactured',
    shelfIdentifier INT DEFAULT NULL COMMENT 'The shelf id switch setting',
    sesProductId VARCHAR(255) DEFAULT NULL COMMENT 'Enclosure product identifier',
    firmwareRevisionA VARCHAR(255) DEFAULT NULL COMMENT 'Shelf module A firmware revision',
    firmwareRevisionB VARCHAR(255) DEFAULT NULL COMMENT 'Shelf module B firmware revision',
    module VARCHAR(255) DEFAULT NULL COMMENT 'The shelf module attachment',
    channelName VARCHAR(255) DEFAULT NULL COMMENT 'The channel/port-number the shelf (hub) is attached to',
    shelfType VARCHAR(255) DEFAULT NULL COMMENT 'Shelf module type',
    state ENUM('NOT_MAPPED', 'NO_STATUS', 'INIT_REQUIRED', 'ONLINE', 'OFFLINE', 'MISSING', 'UNKNOWN') DEFAULT NULL COMMENT 'The current state of the shelf',
    stateRaw VARCHAR(255) DEFAULT NULL COMMENT 'The current state of the shelf - ZAPI raw value',
    temperatureSensorCount INT DEFAULT NULL COMMENT 'Count of temperature sensors on this shelf',
    temperatureSensorFailedCount INT DEFAULT NULL COMMENT 'Count of failed temperature sensors on this shelf. (temp-sensor-info.temp-sensor-is-error)',
    coolingElementCount INT DEFAULT NULL COMMENT 'Count of cooling elements on this shelf',
    coolingElementFailedCount INT DEFAULT NULL COMMENT 'Count of failed cooling elements on this shelf. (cooling-element-info.cooling-element-is-error)',
    powerSupplyCount INT DEFAULT NULL COMMENT 'Count of power supplies on this shelf',
    powerSupplyFailedCount INT DEFAULT NULL COMMENT 'Count of failed power supplies on this shelf (power-supply-info.power-supply-is-error)',
    currentSensorCount INT DEFAULT NULL COMMENT 'Count of current sensors on this shelf',
    currentSensorFailedCount INT DEFAULT NULL COMMENT 'Current of failed current sensors on this shelf (current-sensor-info.is-sensor-error)',
    voltageSensorCount INT DEFAULT NULL COMMENT 'Count of current sensors on this shelf',
    voltageSensorFailedCount INT DEFAULT NULL COMMENT 'Current of failed current sensors on this shelf (current-sensor-info.is-sensor-error)',
    checksum INT NOT NULL,
    changeDetailChecksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_nodeId (nodeId) REFERENCES netapp_model.node (objid),
    PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;

-- fix case
ALTER TABLE netapp_model.flash_device CHANGE COLUMN UUID uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of FlashDevice';
