CREATE TABLE netapp_model.vserver_name_mapping (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the VServer Name Mapping. Equal to {VserverUUID}.{direction}.{position} Not returned from ZAPI',
    resourceKey VARCHAR(512) NOT NULL COMMENT 'DERIVED. A structured key that may be used to uniquely identify this resource globally',
    clusterId BIGINT(20) NOT NULL,
    vserverId BIGINT(20) NOT NULL,
    position INT NOT NULL COMMENT 'Position of an existing name mapping in the list of name mappings for this Vserver',
    direction ENUM('NOT_MAPPED', 'KRB_UNIX', 'WIN_UNIX', 'UNIX_WIN') DEFAULT NULL COMMENT 'The vserver name mapping direction',
    directionRaw VARCHAR(255) DEFAULT NULL COMMENT 'The vserver name mapping direction - ZAPI raw value',
    pattern VARCHAR(255) DEFAULT NULL COMMENT 'Pattern to use to match the name while searching for a name that can be used as a replacement',
    replacement VARCHAR(255) DEFAULT NULL COMMENT 'The name that is to be used as a replacement if the pattern associated with this entry matches',
    checksum INT NOT NULL,
    changeDetailChecksum INT NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid),
    PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;
