ALTER TABLE netapp_model.disk ADD COLUMN isShared tinyint(1) AFTER isReplacing;
ALTER TABLE netapp_model.aggregate ADD COLUMN usesSharedDisks tinyint(1) AFTER isHybridEnabled;
ALTER TABLE netapp_model.disk MODIFY COLUMN containerType enum('NOT_MAPPED','AGGREGATE','BROKEN','LABELMAINT','MAINTENANCE','SPARE','SHARED','UNASSIGNED','VOLUME','UNKNOWN') COLLATE utf8_bin DEFAULT NULL COMMENT 'Type of overlying disk container' AFTER vendor;
ALTER TABLE netapp_model.disk MODIFY COLUMN raidPosition enum('NOT_MAPPED','COPY','DATA','DPARITY','ORPHAN','PARITY','PENDING','PRESENT', 'SHARED') COLLATE utf8_bin DEFAULT NULL COMMENT 'Position of disk relative to its container-type' AFTER bytesPerSector;

CREATE TABLE netapp_model.storage_pool (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPI:  storage-pool-get-iter.storage-pool-info',
    originid BIGINT(20) NOT NULL,
	uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the storage pool. DERIVED: {ClusterUUID}.{StoragePoolName}',
	resourceKey VARCHAR(512)  NOT NULL,
    name VARCHAR(255) NOT NULL COMMENT 'The cluster-wide unique name of the storage pool.',
    clusterId BIGINT(20) NOT NULL,
	allocationUnitSize BIGINT(20) NOT NULL COMMENT 'Minimum size that can be allocated from the storage pool.',
    diskCount BIGINT(20) NOT NULL COMMENT 'The number of disks that are part of the  storage pool',    
    poolUsableSize BIGINT(20) DEFAULT NULL COMMENT 'The amount of usable size in bytes in the  storage pool.',	
    poolTotalSize BIGINT(20) DEFAULT NULL COMMENT 'The total size of the  storage pool.',
    storageType ENUM('ATA','BSAS','FCAL','FSAS','LUN','MSATA','SAS','SATA','SSD') DEFAULT NULL COMMENT 'The type of the disks that constitute the storage pool',
    storageTypeRaw VARCHAR(255) COMMENT 'The type of the disks that constitute the storage pool',
    checksum INT(11) NOT NULL,
    changeDetailChecksum INT(11) NOT NULL,
    createtime BIGINT(20) NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    objState ENUM('LIVE','DEAD'),
	KEY FK_clusterId (clusterId),
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
    KEY objState_index (objState, objid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    PRIMARY KEY (objid)
    ) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;

ALTER TABLE netapp_model.disk ADD COLUMN storagePoolId BIGINT(20) AFTER raidGroupId;
ALTER TABLE netapp_model.disk ADD FOREIGN KEY FK_storagePoolId (storagePoolId) REFERENCES netapp_model.storage_pool (objid);

CREATE TABLE netapp_model.storage_pool_aggregate_relationship (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique relationship identifier. ZAPI:storage-pool-aggregate-get-iter.storage-pool-aggregate-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'The uuid of this relationship. DERIVED:{ClusterUUID}.{StoragePoolName}.{AggregateUUID}.',
	resourceKey VARCHAR(512)  NOT NULL,
	clusterId BIGINT(20) NOT NULL,
	storagePoolId BIGINT(20) NOT NULL,	
	aggregateId BIGINT(20) NOT NULL,
	usedCapacity BIGINT(20) DEFAULT NULL COMMENT 'Capacity provisioned by aggregate from  storage pool',
	allocatedUnitCount BIGINT(20) DEFAULT NULL COMMENT 'No of memory units provisioned by aggregate from  storage pool',
    checksum INT NOT NULL,
	changeDetailChecksum INT NOT NULL,
	createtime BIGINT(20) NOT NULL,
	updatetime BIGINT(20) NOT NULL,
	objState ENUM('LIVE','DEAD'),
	KEY FK_clusterId (clusterId),
	KEY FK_storagePoolId (storagePoolId),
	KEY FK_aggregateId (aggregateId),
	KEY updatetime_index (updatetime),
    KEY originId_index (originid),
	KEY objState_index (objState, objid),
	CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
	CONSTRAINT FOREIGN KEY FK_storagePoolId (storagePoolId) REFERENCES netapp_model.storage_pool (objid),
	CONSTRAINT FOREIGN KEY FK_aggregateId (aggregateId) REFERENCES netapp_model.aggregate (objid),
	PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;

CREATE TABLE netapp_model.storage_pool_spare (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique relationship identifier.ZAPI: storage-pool-spare-capacity-get-iter.storage-pool-spare-capacity-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'The uuid of this relationship.  DERIVED:{ClusterUUID}.{NodeUUID}.{StoragePoolName}.',
	resourceKey VARCHAR(512)  NOT NULL,
	clusterId BIGINT(20) NOT NULL,
	nodeId BIGINT(20) NOT NULL,
	storagePoolId BIGINT(20) NOT NULL,
	allocationUnitCount BIGINT(20) NOT NULL COMMENT 'The total number of units which can be allocated as spare capacity for aggregates',
	allocationUnitSize BIGINT(20) NOT NULL COMMENT 'Minimum size that can be allocated from the  storage pool',
	spareUsableSize BIGINT(20) NOT NULL COMMENT 'The total spare capacity in bytes available on a given node in the  storage pool',
	storageType ENUM('ATA','BSAS','FCAL','FSAS','LUN','MSATA','SAS','SATA','SSD') DEFAULT NULL COMMENT 'The type of the disks that constitute the  storage pool',
	storageTypeRaw VARCHAR(255) COMMENT 'The type of the disks that constitute the storage pool',
	syncmirrorPool	VARCHAR(255) COMMENT 'The name of the syncmirror pool.',
    checksum INT NOT NULL,
	changeDetailChecksum INT NOT NULL,
	createtime BIGINT(20) NOT NULL,
	updatetime BIGINT(20) NOT NULL,
	objState ENUM('LIVE','DEAD'),
	KEY FK_clusterId (clusterId),
	KEY FK_storagePoolId (storagePoolId),
	KEY FK_nodeId (nodeId),
	KEY updatetime_index (updatetime),
    KEY originId_index (originid),
	KEY objState_index (objState, objid),
	CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
	CONSTRAINT FOREIGN KEY FK_storagePoolId (storagePoolId) REFERENCES netapp_model.storage_pool (objid),
	CONSTRAINT FOREIGN KEY FK_nodeId (nodeId) REFERENCES netapp_model.node(objid),
	PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;

CREATE TABLE netapp_model.disk_aggregate_relationship (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique relationship identifier. ZAPI: aggr-status-get-iter aggr-status-info.aggr-plex-info.aggr-raidgroup-info',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'The uuid of this relationship.  DERIVED:{DiskUUID}.{AggregateUUID}',
	resourceKey VARCHAR(512)  NOT NULL,
	clusterId BIGINT(20) NOT NULL,
	diskId BIGINT(20) NOT NULL,	
	aggregateId BIGINT(20) NOT NULL,	
    checksum INT NOT NULL,
	changeDetailChecksum INT NOT NULL,
	createtime BIGINT(20) NOT NULL,
	updatetime BIGINT(20) NOT NULL,
	objState ENUM('LIVE','DEAD'),
	KEY FK_clusterId (clusterId),
	KEY FK_diskId (diskId),
	KEY FK_aggregateId (aggregateId),
	KEY updatetime_index (updatetime),
    KEY originId_index (originid),
	KEY objState_index (objState, objid),
	CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
	CONSTRAINT FOREIGN KEY FK_diskId (diskId) REFERENCES netapp_model.disk(objid),
	CONSTRAINT FOREIGN KEY FK_aggregateId (aggregateId) REFERENCES netapp_model.aggregate (objid),
	PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;