CREATE TABLE netapp_model.`network_ip_space` (
  `objid` bigint(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPI:  net-ipspaces-get-iter.net-ipspaces-info',
  `originid` bigint(20) NOT NULL,
  `uuid` varchar(255) COLLATE utf8_bin NOT NULL COMMENT 'Globally unique ID of the ip space.',
  `resourceKey` varchar(512) NOT NULL,
  `name` varchar(255) COLLATE utf8_bin COMMENT 'The cluster-wide unique name of the ip space.',
  `clusterId` bigint(20) NOT NULL,
  `networkIpSpaceId` bigint(20) COMMENT 'The ID of the IPspace.',
  `checksum` int(11) NOT NULL,
  `changeDetailChecksum` int(11) NOT NULL,
  `createtime` bigint(20) NOT NULL,
  `updatetime` bigint(20) NOT NULL,
  `objState` enum('LIVE','DEAD') DEFAULT NULL,
  PRIMARY KEY (`objid`),
  KEY `FK_clusterId` (`clusterId`),
  KEY `updatetime_index` (`updatetime`),
  KEY `originId_index` (`originid`),
  KEY `objState_index` (`objState`,`objid`),
  CONSTRAINT `network_ip_space_ibfk_1` FOREIGN KEY (`clusterId`) REFERENCES netapp_model.`cluster` (`objid`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;


CREATE TABLE netapp_model.`network_port_broadcast_domain` (
  `objid` bigint(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPI:  net-port-broadcast-domain-get-iter.net-port-broadcast-domain-info',
  `originid` bigint(20) NOT NULL,
  `uuid` varchar(255) COLLATE utf8_bin NOT NULL COMMENT 'Globally unique ID of the broadcast domain DERIVED: cluster_uuid.broadcast_domain_name.',
  `resourceKey` varchar(512) NOT NULL,
  `name` varchar(255) COLLATE utf8_bin COMMENT 'The cluster-wide unique name of the broadcast domain.',
  `clusterId` bigint(20) NOT NULL,
  `mtu` bigint(20) COMMENT 'Maximum Transmission Unit (MTU) of the broadcast domain',
  `networkIpSpaceId` bigint(20) COMMENT 'The Object Id of the IPspace associated with the broadcast domain.',
  `checksum` int(11) NOT NULL,
  `changeDetailChecksum` int(11) NOT NULL,
  `createtime` bigint(20) NOT NULL,
  `updatetime` bigint(20) NOT NULL,
  `objState` enum('LIVE','DEAD') DEFAULT NULL,
  PRIMARY KEY (`objid`),
  KEY `FK_clusterId` (`clusterId`),
  KEY `FK_networkIPSpaceId` (`networkIpSpaceId`),
  KEY `updatetime_index` (`updatetime`),
  KEY `originId_index` (`originid`),
  KEY `objState_index` (`objState`,`objid`),
  CONSTRAINT `network_port_broadcast_domain_ibfk_1` FOREIGN KEY (`clusterId`) REFERENCES netapp_model.`cluster` (`objid`),
  CONSTRAINT `network_port_broadcast_domain_ibfk_2` FOREIGN KEY (`networkIpSpaceId`) REFERENCES netapp_model.`network_ip_space` (`objid`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;

CREATE TABLE netapp_model.`network_failover_group` (
  `objid` bigint(20) NOT NULL COMMENT 'Locally unique object identifier.  ZAPI: net-failover-group-get-iter.net-failover-group-info',
  `originid` bigint(20) NOT NULL,
  `uuid` varchar(255) COLLATE utf8_bin NOT NULL COMMENT 'Globally unique ID of the broadcast domain DERIVED: cluster_uuid.failover_group_name.',
  `resourceKey` varchar(512) NOT NULL,
  `name` varchar(255) COLLATE utf8_bin COMMENT 'The cluster-wide unique name of the failover group.',
  `clusterId` bigint(20) NOT NULL,
  `networkPortBroadcastDomainId` bigint(20) COMMENT 'The Object Id of the network port broadcast domain associated with the network failover group.',
  `checksum` int(11) NOT NULL,
  `changeDetailChecksum` int(11) NOT NULL,
  `createtime` bigint(20) NOT NULL,
  `updatetime` bigint(20) NOT NULL,
  `objState` enum('LIVE','DEAD') DEFAULT NULL,
  PRIMARY KEY (`objid`),
  KEY `FK_clusterId` (`clusterId`),
  KEY `FK_networkPortBroadcastDomainId` (`networkPortBroadcastDomainId`),
  KEY `updatetime_index` (`updatetime`),
  KEY `originId_index` (`originid`),
  KEY `objState_index` (`objState`,`objid`),
  CONSTRAINT `network_failover_group_domain_ibfk_1` FOREIGN KEY (`clusterId`) REFERENCES netapp_model.`cluster` (`objid`),
  CONSTRAINT `network_failover_group_domain_ibfk_2` FOREIGN KEY (`networkPortBroadcastDomainId`) REFERENCES netapp_model.`network_port_broadcast_domain` (`objid`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;

ALTER TABLE netapp_model.network_port ADD COLUMN networkIpSpaceId bigint(20) DEFAULT NULL COMMENT 'IP Space associated with the network port.' AFTER operationalDuplexRaw;
ALTER TABLE netapp_model.network_port ADD KEY `FK_networkIpSpaceId` (`networkIpSpaceId`);
ALTER TABLE netapp_model.network_port ADD CONSTRAINT `network_port_ibfk_3` FOREIGN KEY (`networkIpSpaceId`) REFERENCES netapp_model.`network_ip_space` (`objid`);

ALTER TABLE netapp_model.network_port ADD COLUMN networkPortBroadcastDomainId bigint(20) DEFAULT NULL COMMENT 'Broadcast domain associated with the network port.' AFTER networkIpSpaceId;
ALTER TABLE netapp_model.network_port ADD KEY `FK_networkPortBroadcastDomainId` (`networkPortBroadcastDomainId`);
ALTER TABLE netapp_model.network_port ADD CONSTRAINT `network_port_ibfk_4` FOREIGN KEY (`networkPortBroadcastDomainId`) REFERENCES netapp_model.`network_port_broadcast_domain` (`objid`);

ALTER TABLE netapp_model.vserver ADD COLUMN networkIpSpaceId bigint(20) DEFAULT NULL COMMENT 'IP Space associated with the vServer.' AFTER disallowedProtocols;
ALTER TABLE netapp_model.vserver ADD KEY `FK_networkIpSpaceId` (`networkIpSpaceId`);
ALTER TABLE netapp_model.vserver ADD CONSTRAINT `vserver_ibfk_4` FOREIGN KEY (`networkIpSpaceId`) REFERENCES netapp_model.`network_ip_space` (`objid`);
