ALTER TABLE netapp_model.snap_mirror CHANGE COLUMN volumeId volumeId BIGINT(20);
ALTER TABLE netapp_model.snap_mirror ADD COLUMN networkCompressionRatio VARCHAR(255) COMMENT 'The compression ratio achieved for the data sent over the wire with network compression enabled' AFTER newestSnapshotId;
ALTER TABLE netapp_model.snap_mirror ADD COLUMN operationId VARCHAR(255) COMMENT 'Operation unique identifier of the currently executing SnapMirror operation.' AFTER networkCompressionRatio;
ALTER TABLE netapp_model.snap_mirror ADD COLUMN lastTransferNetworkCompressionRatio VARCHAR(255) COMMENT 'The compression ratio achieved for the data sent over the wire as a part of the last transfer' AFTER lastTransferErrorCodes;
ALTER TABLE netapp_model.snap_mirror ADD COLUMN relationshipGroupType enum('NOT_MAPPED','NONE','VSERVER','INFINITEVOL') COLLATE utf8_bin DEFAULT NULL COMMENT 'For FlexVol relationships, specifies the type of the group relationship that includes this FlexVol. For group relationships, specifies the type of the group relationship.' AFTER relationshipControlPlane;
ALTER TABLE netapp_model.snap_mirror ADD COLUMN relationshipGroupTypeRaw VARCHAR(255) COMMENT 'For FlexVol relationships, specifies the type of the group relationship that includes this FlexVol. For group relationships, specifies the type of the group relationship.' AFTER relationshipGroupType;

ALTER TABLE netapp_model.snap_mirror_policy ADD COLUMN createSnapshot tinyint(1) COMMENT 'Specifies whether a new Snapshot Copy is created at the time of a SnapMirror transfer.' AFTER comment;
ALTER TABLE netapp_model.snap_mirror_policy ADD COLUMN ignoreAccessTime tinyint(1) COMMENT 'Specifies whether incremental transfers will ignore files which have only their access time changed.' AFTER createSnapshot;
ALTER TABLE netapp_model.snap_mirror_policy ADD COLUMN isNetworkCompressionEnabled tinyint(1) COMMENT 'Specifies whether network compression is enabled for transfers.' AFTER ignoreAccessTime;
ALTER TABLE netapp_model.snap_mirror_policy ADD COLUMN restart enum('NOT_MAPPED','ALWAYS','NEVER','DEFAULT') COLLATE utf8_bin DEFAULT NULL COMMENT 'Defines the behavior of SnapMirror if an interrupted transfer exists - "always" , "never" , "default".' AFTER isNetworkCompressionEnabled;
ALTER TABLE netapp_model.snap_mirror_policy ADD COLUMN restartRaw VARCHAR(255) COMMENT 'Defines the behavior of SnapMirror if an interrupted transfer exists - "always" , "never" , "default".' AFTER restart;
ALTER TABLE netapp_model.snap_mirror_policy ADD COLUMN tries VARCHAR(255)  COMMENT 'Specifies the maximum number of times to attempt each manual or scheduled transfer for a SnapMirror relationship.' AFTER transferPriorityRaw;
ALTER TABLE netapp_model.snap_mirror_policy ADD COLUMN type enum('NOT_MAPPED','VAULT','ASYNC_MIRROR','MIRROR_VAULT', 'SYNC_MIRROR') COLLATE utf8_bin DEFAULT NULL COMMENT 'The type of the Snapmirror policy - "vault" , "async_mirror" , "mirror_vault" ,"sync_mirror"' AFTER tries;
ALTER TABLE netapp_model.snap_mirror_policy ADD COLUMN typeRaw VARCHAR(255) COMMENT 'The type of the Snapmirror policy - "vault" , "async_mirror" , "mirror_vault" ,"sync_mirror"' AFTER type;

CREATE TABLE netapp_model.snap_mirror_self_relationship (
  objid bigint(20) NOT NULL COMMENT 'Locally unique object identifier.',
  originid bigint(20) NOT NULL,
  uuid varchar(255) COLLATE utf8_bin NOT NULL COMMENT 'UUID of the Volume Snap Mirror',
  vserverSnapMirrorId bigint(20) DEFAULT NULL COMMENT 'ID of the parent vserver snap mirror',
  checksum int(11) NOT NULL,
  updatetime bigint(20) NOT NULL,
  changeDetailChecksum int(11) NOT NULL,
  createtime bigint(20) NOT NULL,
  objState enum('LIVE','DEAD') COLLATE utf8_bin NOT NULL DEFAULT 'LIVE' COMMENT 'Internal object state',
  PRIMARY KEY (objid),
  KEY updatetime_index (updatetime),
  KEY originId_index (originid),
  KEY FK_vserverSnapMirrorId (vserverSnapMirrorId),
  KEY objState_index (objState,objid),
  CONSTRAINT snap_mirror_self_relationship_ibfk_1 FOREIGN KEY (objid) REFERENCES netapp_model.snap_mirror (objid),
  CONSTRAINT snap_mirror_self_relationship_ibfk_2 FOREIGN KEY (vserverSnapMirrorId) REFERENCES netapp_model.snap_mirror (objid)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;