CREATE TABLE netapp_model.lun_import (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the LUN import. DERIVED from {vserver-uuid}.{foreign_disk}.',
    resourceKey VARCHAR(255) NOT NULL,
    clusterId BIGINT(20) NOT NULL,
    vserverId BIGINT(20) NOT NULL,
    foreignDiskId BIGINT(20) DEFAULT NULL COMMENT 'Reference to a locally unique object identifier of DISK table', 
    administrativeStatus ENUM('STARTED','STOPPED', 'PAUSED', 'NOT_MAPPED') DEFAULT NULL COMMENT 'Administrative status of the import operation',
    administrativeStatusRaw VARCHAR(255) DEFAULT NULL COMMENT 'Administrative status of the import operation',
    comparedBlocks BIGINT(20) DEFAULT NULL COMMENT 'Number of compared blocks',
    estimatedRemainingDuration BIGINT(20) DEFAULT NULL,
    failureReason VARCHAR(255) DEFAULT NULL,
    importCurrentNodeId BIGINT(20) DEFAULT NULL COMMENT 'Reference to a locally unique object identifier of NODE table',
    importHomeNodeId BIGINT(20) DEFAULT NULL COMMENT 'Reference to a locally unique object identifier of NODE table',
    importedBlocks BIGINT(20) DEFAULT NULL,
    operationInProgress ENUM('IMPORT', 'VERIFY', 'NOT_MAPPED') DEFAULT NULL COMMENT 'Current operation in progress',
    operationInProgressRaw VARCHAR(255) DEFAULT NULL COMMENT 'Current operation in progress',
    operationalStatus ENUM('IN_PROGRESS', 'FAILED', 'COMPLETED', 'STOPPED', 'PAUSED', 'NOT_MAPPED') DEFAULT NULL COMMENT 'Operational status of the import operation',
    operationalStatusRaw VARCHAR(255) DEFAULT NULL COMMENT 'Operational status of the import operation',
    path VARCHAR(255) DEFAULT NULL,
    percentComplete BIGINT(20) DEFAULT NULL,
    totalBlocks BIGINT(20) DEFAULT NULL,
    checksum INT NOT NULL,
	changeDetailChecksum INT NOT NULL,
	createtime BIGINT(20) NOT NULL,
	updatetime BIGINT(20) NOT NULL,
	objState ENUM('LIVE','DEAD'),
	KEY updatetime_index (updatetime),
	KEY clusterId_index (clusterId),
	KEY vserverId_index (vserverId),
	KEY foreignDiskId_index (foreignDiskId),
	KEY importCurrentNodeId_index (importCurrentNodeId),
	KEY importHomeNodeId_index (importHomeNodeId),
    KEY originId_index (originid),
    KEY objState_index (objState, objid),
    CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_foreignDiskId (foreignDiskId) REFERENCES netapp_model.disk (objid),
    CONSTRAINT FOREIGN KEY FK_importCurrentNodeId (importCurrentNodeId) REFERENCES netapp_model.node (objid),
    CONSTRAINT FOREIGN KEY FK_importHomeNodeId (importHomeNodeId) REFERENCES netapp_model.node (objid),
    PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;
