ALTER TABLE netapp_model.network_failover_group ADD COLUMN vserverId bigint(20) DEFAULT NULL COMMENT 'vserver that this network failover group belongs to' AFTER clusterId;
ALTER TABLE netapp_model.network_failover_group ADD CONSTRAINT FOREIGN KEY network_failover_group_vserver (vserverId) REFERENCES netapp_model.vserver (objid);

ALTER TABLE netapp_model.lif ADD failoverGroupId bigint(20) NULL DEFAULT NULL COMMENT 'failover group for this lif';
ALTER TABLE netapp_model.lif ADD CONSTRAINT FOREIGN KEY lif_failover_group (failoverGroupId) REFERENCES netapp_model.network_failover_group (objid);

UPDATE netapp_model.lif lif
	INNER JOIN netapp_model.network_failover_group nfg
		ON nfg.name = lif.failoverGroup
	SET lif.failoverGroupId = nfg.objid
  WHERE nfg.clusterId = lif.clusterId;

USE netapp_model;

DROP TABLE IF EXISTS `network_failover_group_network_port_relationship`;

CREATE TABLE `network_failover_group_network_port_relationship` (
  `objid` bigint(20) NOT NULL COMMENT 'Locally unique relationship identifier.',
  `originid` bigint(20) NOT NULL,
  `uuid` varchar(255) COLLATE utf8_bin NOT NULL COMMENT 'The uuid of this relationship.  DERIVED:{NetworkFailoverGroupUUID}.{NetworkPortUUID}',
  `resourceKey` varchar(512) COLLATE utf8_bin NOT NULL,
  `clusterId` bigint(20) NOT NULL,
  `networkFailoverGroupId` bigint(20) NOT NULL,
  `networkPortId` bigint(20) NOT NULL,
  `checksum` int(11) NOT NULL,
  `changeDetailChecksum` int(11) NOT NULL,
  `createtime` bigint(20) NOT NULL,
  `updatetime` bigint(20) NOT NULL,
  `objState` enum('LIVE','DEAD') COLLATE utf8_bin DEFAULT NULL,
  PRIMARY KEY (`objid`),
  KEY `FK_clusterId` (`clusterId`),
  KEY `FK_networkFailoverGroupId` (`networkFailoverGroupId`),
  KEY `FK_networkPortId` (`networkPortId`),
  KEY `updatetime_index` (`updatetime`),
  KEY `originId_index` (`originid`),
  KEY `objState_index` (`objState`,`objid`),
  CONSTRAINT `network_failover_group_network_port_relationship_ibfk_1` FOREIGN KEY (`clusterId`) REFERENCES `cluster` (`objid`),
  CONSTRAINT `network_failover_group_network_port_relationship_ibfk_2` FOREIGN KEY (`networkFailoverGroupId`) REFERENCES `network_failover_group` (`objid`),
  CONSTRAINT `network_failover_group_network_port_relationship_ibfk_3` FOREIGN KEY (`networkPortId`) REFERENCES `network_port` (`objid`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;

