CREATE TABLE netapp_model.metrocluster_operation (
	objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the operation',
    resourceKey VARCHAR(255) NOT NULL,
    clusterId BIGINT(20) NOT NULL COMMENT 'Reference to a locally unique object identifier CLUSTER table',
    nodeId BIGINT(20) NOT NULL COMMENT 'Reference to a locally unique object identifier of NODE table',
    startTime BIGINT(20) DEFAULT NULL COMMENT 'Time when the operation started',
    endTime BIGINT(20) DEFAULT NULL COMMENT 'Time when the operation ended',
    errorList TEXT DEFAULT NULL COMMENT 'A comma separated list of errors encountered during the operation',
    jobId BIGINT(20) DEFAULT NULL COMMENT 'Identifier for the job',
    operation ENUM('UNKNOWN', 'CONFIGURE', 'MODIFY', 'SWITCHOVER', 'HEAL_AGGREGATES', 'HEAL_ROOT_AGGREGATES', 'SWITCHBACK', 'SWITCHBACK_CONTINUATION_AGENT', 'UNCONFIGURE', 'UNCONFIGURE_CONTINUATION_AGENT', 'SWITCHOVER_SIMULATE', 'SWITCHBACK_SIMULATE', 'CHECK', 'NOT_MAPPED') DEFAULT NULL COMMENT 'Name of the operation',
    operationRaw VARCHAR(255) DEFAULT NULL COMMENT 'Name of the operation',
    state ENUM('UNKNOWN', 'IN_PROGRESS', 'PARTIALLY_SUCCESSFUL', 'COMPLETED_WITH_MANUAL_RECOVERY_NEEDED', 'COMPLETED_WITH_WARNINGS', 'SUCCESSFUL', 'FAILED', 'NOT_MAPPED') DEFAULT NULL COMMENT 'State of the operation',
    stateRaw  VARCHAR(255) DEFAULT NULL COMMENT 'State of the operation',
    checksum INT NOT NULL,
	changeDetailChecksum INT NOT NULL,
	createtime BIGINT(20) NOT NULL,
	updatetime BIGINT(20) NOT NULL,
	objState ENUM('LIVE','DEAD'),
    KEY clusterId_index (clusterId),
    KEY nodeId_index (nodeId),
    KEY updatetime_index (updatetime),
    KEY originId_index (originid),
	KEY objState_index (objState, objid),
	CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
	CONSTRAINT FOREIGN KEY FK_nodeId (nodeId) REFERENCES netapp_model.node (objid),
    PRIMARY KEY (objid)	
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;