CREATE TABLE netapp_model.cluster_peer (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the peer cluster.',
    resourceKey VARCHAR(255) NOT NULL,
    clusterId BIGINT(20) NOT NULL,
    activeAddresses TEXT DEFAULT NULL COMMENT 'The active IP addresses of the peer cluster. Available in the local cluster from a cache. ',
    addressFamily ENUM('IPV4', 'IPV6', 'NOT_MAPPED') DEFAULT NULL COMMENT 'The address family of the peer cluster connection.',
    addressFamilyRaw VARCHAR(255) DEFAULT NULL COMMENT 'The address family of the peer cluster connection. ',
    authStatusAdmin VARCHAR(255) DEFAULT NULL COMMENT 'The desired status for the authentication of the peer relationship, according to this cluster. Available in the local cluster.',
    authStatusOperational ENUM('OK', 'ABSENT','PENDING', 'EXPIRED','REVOKED', 'DECLINED','REFUSED', 'OK_AND_OFFER','ABSENT_BUT_OFFER','REVOKED_BUT_OFFER', 'INTENT_MISMATCH','KEY_MISMATCH', 'INCAPABLE', 'NOT_MAPPED') DEFAULT NULL  COMMENT 'The actual status of the authentication interaction with the peer cluster. Available in the local cluster from a cache.',
    authStatusOperationalRaw VARCHAR(255) DEFAULT NULL  COMMENT 'The actual status of the authentication interaction with the peer cluster. Available in the local cluster from a cache.',
    availability ENUM('UNAVAILABLE', 'AVAILABLE', 'PARTIAL', 'PENDING', 'NOT_MAPPED') DEFAULT NULL COMMENT 'The status of the peer cluster connection. Available in the local cluster from a cache.',
    availabilityRaw VARCHAR(255) DEFAULT NULL COMMENT 'The status of the peer cluster connection. Available in the local cluster from a cache.',
    remoteClusterName VARCHAR(255) DEFAULT NULL COMMENT 'The name of the peer cluster. Available in the local cluster. ',
    remoteClusterUUID VARCHAR(255) DEFAULT NULL COMMENT 'The UUID of the peer cluster. Available in the local cluster. ',
    remoteClusterId BIGINT(20) DEFAULT NULL COMMENT 'Foreign key to the cluster table. Available if the remote cluster is registered',
    networkIpSpaceId BIGINT(20) DEFAULT NULL COMMENT 'IPspace in which the LIFs for this relationship are to be found. Foreign key to the network_ip_space table',
    isClusterHealthy tinyint(1) DEFAULT NULL COMMENT 'The status of peer cluster health. Available in the local cluster from a cache. ',
    peerAddresses TEXT DEFAULT NULL COMMENT 'The remote IP address or host name of the peer cluster. Available in the local cluster.',
    remoteClusterNodes TEXT DEFAULT NULL COMMENT 'The list of all the nodes in the peer cluster. Available in the local cluster. ',
    serialNumber VARCHAR(255) DEFAULT NULL COMMENT 'This is an assigned serial number of peer cluster. It is assigned at the time of cluster create. ',
    unreachableLocalNodes TEXT DEFAULT NULL COMMENT 'List of local nodes not contactable from the node on which the command is executed. Available in the local cluster from a cache. ',
    checksum INT NOT NULL,
    changeDetailChecksum INT NOT NULL,
    createtime BIGINT(20) NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    objState ENUM('LIVE','DEAD'),
    KEY updatetime_index (updatetime),
    KEY clusterId_index (clusterId),
    KEY remoteClusterId_index (remoteClusterId),
    KEY networkIpSpaceId_index (networkIpSpaceId),
    KEY originId_index (originid),
    KEY objState_index (objState, objid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_remoteClusterId (remoteClusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_networkIpSpaceId (networkIpSpaceId) REFERENCES netapp_model.network_ip_space (objid),
  PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;