DROP FUNCTION IF EXISTS netapp_performance.node_utilization_calc;
DELIMITER $$

CREATE FUNCTION netapp_performance.node_utilization_calc(total_cp_msecs FLOAT, p2_flush FLOAT, total_kahuna_busy FLOAT, max_processor_elapsed_time FLOAT, processor_busy_percent FLOAT, delta_poll_time FLOAT)
	RETURNS FLOAT deterministic
BEGIN
	DECLARE node_utilization FLOAT;
	DECLARE total_kahuna_busy_percent FLOAT;
	DECLARE wafl_utilization FLOAT;
	
	IF ((max_processor_elapsed_time IS NOT NULL) or (max_processor_elapsed_time <> 0) or (total_kahuna_busy IS NOT NULL)) THEN
		SET total_kahuna_busy_percent = total_kahuna_busy / max_processor_elapsed_time;
	ELSE
		SET total_kahuna_busy_percent = 0;
	END IF;
	
	IF ((total_cp_msecs IS NOT NULL) or (total_cp_msecs <> 0) or (p2_flush IS NOT NULL) or (delta_poll_time IS NOT NULL) or (delta_poll_time <> 0)) THEN
		SET wafl_utilization = ((total_cp_msecs * (1 - (p2_flush / total_cp_msecs))/.8)/ delta_poll_time) * 100;
	ELSE
		SET wafl_utilization = 0;
	END IF;
		
	IF (total_kahuna_busy_percent > 60) THEN
		SET node_utilization = 100;
	ELSE
		IF (wafl_utilization >= processor_busy_percent) THEN
			SET node_utilization = wafl_utilization;
		ELSE
			SET node_utilization = processor_busy_percent;
		END IF;
	END IF;
	
	RETURN COALESCE(node_utilization, 0); 
END

$$
DELIMITER ;