CREATE TABLE netapp_model.vserver_peer (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of the local vserver in the relationship.',
    resourceKey VARCHAR(255) NOT NULL,
    clusterId BIGINT(20) NOT NULL COMMENT 'Foreign key to the cluster table. ',
    vServerId BIGINT(20) NOT NULL COMMENT 'Foreign key to the vserver table. ',
    peerClusterId BIGINT(20) DEFAULT NULL COMMENT 'The peer cluster ID',
    peerClusterName VARCHAR(255) DEFAULT NULL COMMENT 'The peer cluster name',
    peerVServerName VARCHAR(255) DEFAULT NULL COMMENT 'The name of the peer Vserver in the relationship. ',
    peerVServerUUID VARCHAR(255) DEFAULT NULL COMMENT 'The UUID of the peer Vserver in the relationship. ',
    peerVServerId BIGINT(20) DEFAULT NULL COMMENT 'Foreign key to the vserver table. ',
    peerState ENUM('PEERED', 'PENDING', 'INITIALING', 'INITIATED', 'REJECTED', 'SUSPENDED', 'DELETED', 'NOT_MAPPED') DEFAULT NULL COMMENT 'The state of the vserver peer relationship.',
    peerStateRaw VARCHAR(255) DEFAULT NULL COMMENT 'The state of the vserver peer relationship.',
    applications VARCHAR(255) DEFAULT NULL COMMENT 'The applications which can make use of the peering relationship. The  possible application values are: snapmirror, file_copy and lun_copy. ',
    checksum INT NOT NULL,
    changeDetailChecksum INT NOT NULL,
    createtime BIGINT(20) NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    objState ENUM('LIVE','DEAD'),
    KEY updatetime_index (updatetime),
    KEY vserverId_index (vserverId),
    KEY peerClusterId_index (peerClusterId),
    KEY peerVserverId_index (peerVserverId),
    KEY originId_index (originid),
    KEY objState_index (objState, objid),
    CONSTRAINT FOREIGN KEY FK_vserverId (vserverId) REFERENCES netapp_model.vserver (objid),
    CONSTRAINT FOREIGN KEY FK_peerClusterId (peerClusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_peerVserverId (peerVserverId) REFERENCES netapp_model.vserver (objid),
  PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;