CREATE TABLE netapp_model.network_subnet (
  objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.',
  originid BIGINT(20) NOT NULL,
  uuid VARCHAR(255) NOT NULL COMMENT 'Globally unique ID of net subnet. Derived: {ipspace UUID}.subnet-name',
  resourceKey VARCHAR(512)  NOT NULL,
  subnetName  VARCHAR(255) NOT NULL COMMENT 'subnet name',
  subnet VARCHAR(255) DEFAULT NULL COMMENT 'subnet IP value',
  clusterId BIGINT(20) NOT NULL COMMENT 'Cluseter objid hosting this subnet',
  broadcastDomainId BIGINT(20) DEFAULT NULL COMMENT 'Layer 2 broadcast domain name that the subnet belongs to',
  ipspaceId BIGINT(20) NOT NULL COMMENT 'IPspace Name that the subnet belongs to',
  availableCount INT(11) DEFAULT NULL COMMENT 'Number of available pre-allocated addresses for the subnet',
  forceUpdateLifAssociations TINYINT(1) DEFAULT NULL COMMENT 'Setting this value to true will associate any manually addressed interfaces with the subnet.',
  gateway  VARCHAR(255) DEFAULT NULL COMMENT 'The gateway for the default route of the subnet',
  ipranges TEXT DEFAULT NULL COMMENT 'The list of IP address ranges associated with the subnet',
  totalCount INT(11)  DEFAULT NULL COMMENT 'Number of addresses pre-allocated for the subnet',
  usedCount INT(11)  DEFAULT NULL COMMENT 'Number of addresses used from pre-allocated addresses for the subnet',
  checksum INT(11) NOT NULL,
  changeDetailChecksum INT(11) NOT NULL,
  createtime BIGINT(20) NOT NULL,
  updatetime BIGINT(20) NOT NULL,
  objState ENUM('LIVE','DEAD'),
  KEY FK_ipspaceId (ipspaceId),
  KEY FK_broadcastDomainId (broadcastDomainId),
  KEY updatetime_index (updatetime),
  KEY originId_index (originid),
  KEY objState_index (objState, objid),
  CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
  CONSTRAINT FOREIGN KEY FK_ipspaceId (ipspaceId) REFERENCES netapp_model.network_ip_space (objid),
  CONSTRAINT FOREIGN KEY FK_broadcastDomainId (broadcastDomainId) REFERENCES netapp_model.network_port_broadcast_domain(objid),
  PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;
