CREATE TABLE netapp_model.snap_mirror_history (
    objid BIGINT(20) NOT NULL COMMENT 'Locally unique object identifier.',
    originid BIGINT(20) NOT NULL,
    uuid VARCHAR(255) NOT NULL COMMENT 'A unique identifier assigned to a SnapMirror operation.',
    resourceKey VARCHAR(255) NOT NULL,
    clusterId BIGINT(20) NOT NULL COMMENT 'Foreign key to the cluster table',
    snapMirrorId BIGINT(20) NOT NULL COMMENT 'Foreign key to the snap_mirror table.',
    operationType ENUM('CREATE', 'MODIFY', 'QUIESCE', 'RESUME', 'DELETE', 'INITIALIZE', 'MANUAL_UPDATE', 'SCHEDULED_UPDATE', 'BREAK', 'RESYNC', 'ABORT', 'RESTORE', 'NOT_MAPPED') DEFAULT NULL COMMENT 'The type of the operation happened on the relationship. ',
    operationTypeRaw VARCHAR(255) DEFAULT NULL COMMENT 'The type of the operation happened on the relationship. ',
    additionalInfo VARCHAR(255) DEFAULT NULL COMMENT 'A message describing the cause of the failure or additional information about a successful operation.',
    startTime BIGINT(20) DEFAULT NULL COMMENT 'The timestamp of the start time of the operation.',
    endTime BIGINT(20) DEFAULT NULL COMMENT 'The timestamp of the end time of the operation.',
    result ENUM('SUCCESS', 'FAILURE', 'NOT_MAPPED') DEFAULT NULL COMMENT 'The result of the SnapMirror operation.',
    resultRaw VARCHAR(255) DEFAULT NULL COMMENT 'The result of the SnapMirror operation.',
    transferSize BIGINT(20) DEFAULT NULL COMMENT 'The total number of bytes transferred as part of the transfer operation.',
    checksum INT NOT NULL,
    changeDetailChecksum INT NOT NULL,
    createtime BIGINT(20) NOT NULL,
    updatetime BIGINT(20) NOT NULL,
    objState ENUM('LIVE','DEAD'),
    KEY updatetime_index (updatetime),
    KEY snapMirrorId_index (snapMirrorId),
    KEY originId_index (originid),
    KEY objState_index (objState, objid),
    CONSTRAINT FOREIGN KEY FK_clusterId (clusterId) REFERENCES netapp_model.cluster (objid),
    CONSTRAINT FOREIGN KEY FK_snapMirrorId (snapMirrorId) REFERENCES netapp_model.snap_mirror(objid),
  PRIMARY KEY (objid)
) ENGINE=INNODB DEFAULT CHARSET=utf8 COLLATE=utf8_bin;