package com.netapp.oci.simulation.time;

import com.netapp.oci.platform.common.util.Assert;
import com.onaro.commons.util.Pair;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * @author Anton Sergeev
 */
public class StopWatch {
    private Map<String, List<Pair<Long, Long>>> functionToDataList = new HashMap<String, List<Pair<Long, Long>>>();

    private Map<String, Long> functionToStartTime = new HashMap<String, Long>();

    public void reset() {
        functionToDataList.clear();
        functionToStartTime.clear();
    }
    
    public void startFunction(String function) {
        functionToStartTime.put(function, Long.valueOf(System.currentTimeMillis()));
    }

    public void stopFunction(String function) {
        Assert.isTrue(functionToStartTime.containsKey(function), "function is not present");
        Long startTime = functionToStartTime.get(function);

        Assert.isNotNull(startTime, "function is present, but startTime is null");
        
        long duration = System.currentTimeMillis() - startTime.longValue();
        List<Pair<Long, Long>> dataList = functionToDataList.get(function);
        if (dataList == null) {
            dataList = new LinkedList<Pair<Long, Long>>();
            functionToDataList.put(function, dataList);
        }

        dataList.add(new Pair<Long, Long>(startTime, duration));
    }

    public Map<String, List<Pair<Long, Long>>> getFunctionToDataList() {
        return functionToDataList;
    }

    public boolean isEmpty() {
        return functionToDataList.isEmpty();
    }
}
