package com.onaro.commons.prefs;

import java.util.prefs.*;
import java.io.*;

/**
 * A factory object that generates <code>PreferencesExt</code> objects encapsulating
 * <code>Preferences</code> nodes.
 */
public interface PreferencesExtFactory {

    /**
     * Gets a <code>PreferencesExt</code> object for the specified <code>Preferences</code> node.
     *
     * @param node the node
     * @return a <code>PreferencesExt</code> encapsulating this node
     */
    public PreferencesExt get(Preferences node);


    /**
     * Gets the user preferences for the package of the specified class.
     *
     * @param c the class
     * @return the user preferences associated with the pacakge of the specified class
     */
    public PreferencesExt userNodeForPackage(Class c);


    /**
     * Gets the system preferences for the package of the specified class.
     *
     * @param c the class
     * @return the system preferences associated with the pacakge of the specified class
     */
    public PreferencesExt systemNodeForPackage(Class c);


    /**
     * Gets the root node of the user preferences.
     *
     * @return the root node of the user preferences
     */
    public PreferencesExt userRoot();


    /**
     * Gets the root node of the system preferences.
     *
     * @return the root node of the system preferences
     */
    public PreferencesExt systemRoot();


    /**
     * Imports user preferences from an XML file.
     *
     * @param is an input stream reading the XML file
     * @throws IOException if failed to read
     * @throws InvalidPreferencesFormatException if the input file isn't a valid
     *         XML preferences
     */
    public void importPreferences(InputStream is)
    throws IOException, InvalidPreferencesFormatException;
}
