package com.onaro.sanscreen.acquisition.cli.commands;

import com.onaro.sanscreen.acquisition.cli.CLI;
import com.onaro.sanscreen.acquisition.sessions.AcquisitionFacadeSession;
import com.onaro.sanscreen.server.dataobject.acquisition.DataSource;
import com.onaro.sanscreen.server.dataobject.acquisition.DataSourceNew;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;

public class ExportDsCommand implements Command {
    @Override
    public String usageString() {
        return "export";
    }

    @Override
    public String usageDescription() {
        return "Return xml description of all data sources (provides all details known)";
    }

    @Override
    public void run(final CLI cli, final Map<String, String> args) throws Exception {
        if (args.size() > 0)
            throw new IllegalArgumentException("No arguments expected for the 'export' command");

        final DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
        final DocumentBuilder docBuilder = docFactory.newDocumentBuilder();

        // root elements
        final Document doc = docBuilder.newDocument();
        final Element rootElement = doc.createElement("datasources");
        doc.appendChild(rootElement);

        // Get a list of all data sources
        final AcquisitionFacadeSession facadeSession = cli.getSession(AcquisitionFacadeSession.JNDI_NAME);
        final com.onaro.sanscreen.server.dataobject.acquisition.DataSource[] dataSources = facadeSession.getAllDs();
        // Arrays.sort(dataSources);

        for (final DataSource oldDataSource : dataSources) {
            final DataSourceNew dataSource = facadeSession.getDsById(oldDataSource.getId());

            final Element dsRoot = doc.createElement("datasource");
            rootElement.appendChild(dsRoot);

            dsRoot.setAttribute("name", dataSource.getName());
            dsRoot.setAttribute("id", Long.toString(dataSource.getId()));
            dsRoot.setAttribute("au", oldDataSource.getAcquisitionUnitName() != null ? oldDataSource.getAcquisitionUnitName() : "local");
            dsRoot.setAttribute("type", oldDataSource.getType());
            dsRoot.setAttribute("vendor", dataSource.getVendor());
            dsRoot.setAttribute("model", dataSource.getModel());

            final Element propRoot = doc.createElement("properties");
            dsRoot.appendChild(propRoot);

            final SortedSet<String> propertyNames = new TreeSet<String>(dataSource.getAttrs().stringPropertyNames());

            for (final String propertyName : propertyNames) {
                final Element prop = doc.createElement("property");
                final String value = dataSource.getAttrs().getProperty(propertyName);

                prop.setAttribute("name", propertyName);
                prop.setAttribute("value", (value != null ? value : ""));

                propRoot.appendChild(prop);
            }
        }

        // write the content into xml file
        final TransformerFactory transformerFactory = TransformerFactory.newInstance();
        final Transformer transformer = transformerFactory.newTransformer();
        final DOMSource source = new DOMSource(doc);

        final StreamResult result = new StreamResult(System.out);

        transformer.transform(source, result);
    }
}
