package com.onaro.sanscreen.acquisition.cli.commands;

import com.onaro.sanscreen.acquisition.cli.CLI;
import com.onaro.sanscreen.acquisition.sessions.AcquisitionFacadeSession;
import com.onaro.sanscreen.server.dataobject.acquisition.DataSourceVendorModel;

import java.util.Map;

@SuppressWarnings({"deprecation"})
public class LsDsTypeCommand implements Command {
    @Override
    public String usageString() {
        return "lstype";
    }

    @Override
    public String usageDescription() {
        return "Lists all data source types available in the system";
    }

    @Override
    public void run(final CLI cli, final Map<String, String> args) throws Exception {
        if (args.size() > 0)
            throw new IllegalArgumentException("No arguments expected for the 'lstype' command");

        // Get a list of all data sources
        final AcquisitionFacadeSession facadeSession = cli.getSession(AcquisitionFacadeSession.JNDI_NAME);
        final DataSourceVendorModel[] vendorModels = facadeSession.findAllDataSourceVendorModels();

        // Listing all the existing data sources
        final String formatString = "%-7s\t%-12s\t%-35s\n";

        System.out.format(formatString, "Type", "Vendor", "Model");
        System.out.format(formatString, "====", "======", "=====");

        for (final DataSourceVendorModel vendorModel : vendorModels) {
            System.out.format(formatString,
                              vendorModel.getDataSourceType(),
                              vendorModel.getVendorName(),
                              vendorModel.getModelName());
        }
    }
}
