package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.common.BasePerformanceArchive;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.impl.OPMPerformanceArchive;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorType;
import com.onaro.sanscreen.acquisition.framework.mgmt.ServiceManager;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import java.io.File;

/**
 * This class is for handling multiple performance presets in OnTap. Acquisition uses only one preset now - opm.
 */
public class PerformanceArchiveManager {

    private final BasePerformanceArchive opmPerformanceArchive;


    public PerformanceArchiveManager(final NetAppOCIEPerformancePackage netAppOCIEPerformancePackage) {
        String archiveFileDownloadDir = new StringBuffer(ServiceManager.getInstance()
                .getProperty("ARCHIVE_FILE_DOWNLOAD_DIR", "/var/log/ocie/recording/archiver"))
                .append(File.separatorChar)
                .append(netAppOCIEPerformancePackage.getDataSource().getDataSourceName()).toString();
        this.opmPerformanceArchive = new OPMPerformanceArchive(netAppOCIEPerformancePackage, archiveFileDownloadDir);

    }

    public long getArchiveSampleTime(final NetAppDefaultStorageContext context, final ZAPIConnection zapiConnection)  throws DataSourceErrorException {
        final long archiveSampleTime = opmPerformanceArchive.getArchiveSampleTime(context, zapiConnection);
        if(archiveSampleTime != -1) {
            return archiveSampleTime;
        } else {
            throw new DataSourceErrorException(DataSourceErrorType.INTERNAL_ERROR,
                    "Unable to retrieve samples from the performance archive for the same file",
                    "Unable to retrieve samples from the performance archive for the same file");
        }
    }

    public void getAndReportPerformance(final NetAppDefaultStorageContext context, final ZAPIConnection zapiConnection) throws DataSourceErrorException {
        //Process & Report
        opmPerformanceArchive.getAndReportPerformance(context, zapiConnection);
    }

    public void clearAllPreviousValues() {
        this.opmPerformanceArchive.clearAllPreviousValues();
    }
}
