package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.builders;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.metrics.ArchiveCounterMetaData;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.metrics.ArchiveMetricsScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceOutputFileType;
import com.onaro.sanscreen.acquisition.framework.datasource.PackageContext;
import com.onaro.sanscreen.acquisition.framework.datasource.config.PackageConfiguration;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileWriter;

import java.io.File;
import java.io.IOException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ArchiveMetricsDebugWriter<Z extends Enum<Z> & ArchiveCounterMetaData> {
    private static final Logger logger = LoggerFactory.getLogger(ArchiveMetricsDebugWriter.class);

    private final boolean isMetricsDebugEnabled;
    /**
     * CSV file to assist in with metrics debugging
     */
    private DSFileWriter metricsDebugWriter = null;
    /**
     * Target counter object name to query
     */
    private final String counterObjectName;
    private Class<Z> metricEnumClass;

    public ArchiveMetricsDebugWriter(final NetAppOCIEPerformancePackage reportingPackage, final String counterObjectName,
                                     final Class<Z> metricEnumClass) {
        this.counterObjectName = counterObjectName;
        this.metricEnumClass = metricEnumClass;
        isMetricsDebugEnabled = reportingPackage.isMetricsDebugEnabled();
    }

    private boolean isMetricsDebugEnabled() {
        return isMetricsDebugEnabled;
    }

    private DSFileWriter getDSMetricsDebugWriter(final MetricsRowScratchPad<Z> currentValues) {
        if (metricsDebugWriter != null) {
            return metricsDebugWriter;              // Already initialized
        }
        final String metricsName = metricEnumClass.getSimpleName();
        try {
            final com.onaro.sanscreen.acquisition.framework.datasource.Package perfPackage = PackageContext.getPackage();
            final PackageConfiguration perfConfig = perfPackage.getConfiguration();
            metricsDebugWriter = perfPackage.getRecordingSession().getCurrentSample().createTextualOutputFile(perfConfig, DataSourceOutputFileType.perfdata, metricsName, "csv");
            final File tempFile = new File(metricsDebugWriter.getFileName());
            PackageContext.getPackage().getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), metricsName, "");
            final String columnHeaders = currentValues.getDebugColumnHeaders(counterObjectName);
            metricsDebugWriter.write(columnHeaders);
            metricsDebugWriter.flush();
            return metricsDebugWriter;
        } catch (IOException e) {
            logger.error(PackageContext.getLogPrefix() + " Failed to create a debug file for performance data", e);
            return null;
        }
    }

    public void writeMetricsDebugCSV(final ArchiveMetricsScratchPad<Z> currentValues) {
        if (!isMetricsDebugEnabled()) {
            return;
        }
        final String metricsDebugCSV = currentValues.toDebugCSV();
        try {
            final DSFileWriter metricsDebugWriter = getDSMetricsDebugWriter(currentValues);
            metricsDebugWriter.write(metricsDebugCSV);
            metricsDebugWriter.flush();
        } catch (IOException ioe) {
            logger.warn(PackageContext.getLogPrefix() + " - Failed to debug metrics: " + ioe, ioe);
        }
    }

    public void closeMetricDebug() {
        if (!isMetricsDebugEnabled()) {
            return;
        }
        if (metricsDebugWriter == null) {
            return;
        }
        try {
            metricsDebugWriter.close();
        } catch (IOException ignore) {
        }
        metricsDebugWriter = null;
    }
}