package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.common;

import java.util.List;
import java.util.Set;


/**
 * Information into each datastore, including a list of datafiles on disk. 
 * It is this file list which OPM will use as part of the collection process
 *
 */
public class ArchiveDataSourceInfo{
	
	/**
	 * Node name on the cluster
	 */
    private final String nodeName;
    
    /**
     * Path to the archived files location on a given node
     */
    private final String path;
    
    /**
     * list of archiver files on a given node 
     */
    private final Set<String> fileNames;
    
    /**
     * timestamp at which the data files were created on the cluster
     */
    private final long timestamp;

    public ArchiveDataSourceInfo(String nodeName, String path, Set<String> fileNames, long timestamp) {
        this.nodeName = nodeName;
        this.path = path;
        this.fileNames = fileNames;
        this.timestamp = timestamp;
    }

    public String getNodeName() {
        return nodeName;
    }

    public String getPath() {
        return path;
    }

    public Set<String> getFileNames() {
        return fileNames;
    }

    public long getTimestamp() {
        return timestamp;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        ArchiveDataSourceInfo that = (ArchiveDataSourceInfo) o;

        if (timestamp != that.timestamp) return false;
        if (nodeName != null ? !nodeName.equals(that.nodeName) : that.nodeName != null) return false;
        if (!fileNames.equals(that.fileNames)) return false;
        if (path != null ? !path.equals(that.path) : that.path != null) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result = nodeName != null ? nodeName.hashCode() : 0;
        result = 31 * result + (path != null ? path.hashCode() : 0);
        result = 31 * result + (fileNames != null ? fileNames.hashCode() : 0);
        result = 31 * result + (int) (timestamp ^ (timestamp >>> 32));
        return result;
    }

    @Override
    public String toString() {
        final StringBuilder sb = new StringBuilder("ArchiveDataSourceInfo{");
        sb.append("nodeName='").append(nodeName).append('\'');
        sb.append(", path='").append(path).append('\'');
        sb.append(", fileNames=").append(fileNames);
        sb.append(", timestamp=").append(timestamp);
        sb.append('}');
        return sb.toString();
    }
}
