package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.common;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation.BuilderUtil;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPINames;
import org.jdom.Element;

import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIUtils.children;
import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIUtils.elem;

import java.util.HashSet;
import java.util.List;
import java.util.Set;


/**
 * OnTap collects the counter values every minute and records them in performance archive files on each node. 
 * The archive files are created every five minutes and each file contains five samples, each sample at a minute apart. 
 * The archive files created for OPM, has only counters listed in the OPM preset. This builder will call the cluster wide 
 * perf-archive-datastore-get-iter ZAPI to retrieve the information on latest performance archive files created for OPM in each node. 
 *
 */
public class ArchiveDataSourceInfoBuilder extends IterBuilderBase<ArchiveDataSourceInfo> {

    private List<ArchiveDataSourceInfo> archiveDataSourceInfoList;
    private static final String PERF_ARCHIVE_DATASTORE_INFO = "perf-archive-datastore-info";
    private static final String DATASTORE = "datastore";
    private static final String PROCESS = "process";
    private static final String AGE_FILTER = "age-filter";
    private static final String IS_PARTIAL_SHOWN = "is-partial-shown"; // BURT 832174: avoids getting partially generated ccma files.

    public ArchiveDataSourceInfoBuilder(String dataStoreName, String processName) {
        super("perf-archive-datastore-get-iter", PERF_ARCHIVE_DATASTORE_INFO, getDefaultMaxRecords(),
                // desired attributes in response
                elem(ZAPINames.DESIRED_ATTRIBUTES,
                        elem(PERF_ARCHIVE_DATASTORE_INFO, children("node", "path",
                                elem("perf-archive-datafile-infos",
                                        elem("perf-archive-datafile-info",
                                                elem("filenames",
                                                        elem("filename")),
                                                elem("timestamp")))))),
                // query structure
                elem(ZAPINames.QUERY,
                        elem(PERF_ARCHIVE_DATASTORE_INFO,
                                elem(DATASTORE, dataStoreName),
                                elem(PROCESS, processName))),
                elem(AGE_FILTER, "newest"),
                elem(IS_PARTIAL_SHOWN, "false")
        );
    }

    /**
     * each iteration element in the zapi response is parsed and new Info objects created
     */
    protected ArchiveDataSourceInfo buildIterInstance(final Element perfArchiveDatastoreInfoElt) {
        ArchiveDataSourceInfo archiveDataSourceInfo = null;
        final Element perfArchiveDatafileInfos = perfArchiveDatastoreInfoElt.getChild("perf-archive-datafile-infos");
        if(perfArchiveDatafileInfos != null) {
            final Element perfArchiveDatafileInfo = perfArchiveDatafileInfos.getChild("perf-archive-datafile-info");
            if(perfArchiveDatafileInfo != null) {
                final String node = BuilderUtil.getChildTextTrim(perfArchiveDatastoreInfoElt, "node");
                final String path = BuilderUtil.getChildTextTrim(perfArchiveDatastoreInfoElt, "path");
                final List<String> fileNamesList = BuilderUtil.getListFromGrandChildren(perfArchiveDatafileInfo, "filenames", "filename");
                final Set<String> fileNamesSet = new HashSet<String> (fileNamesList);
                final Long timestamp = BuilderUtil.parseLongFromChild(perfArchiveDatafileInfo, "timestamp");
                archiveDataSourceInfo = new ArchiveDataSourceInfo(node, path, fileNamesSet, timestamp);
            }
        }
        return archiveDataSourceInfo;
    }

}
