package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.common;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.config.NetAppOCIEPerformanceConfiguration;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorType;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSInputStream;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSSessionFileReader;


public class ReplayFileDownloader extends AbstractFileDownloader implements FileDownloader {
    private final static Logger logger = LoggerFactory.getLogger(ArchiveFileDownloader.class);
    
    @Override
    public void downloadArchiveFiles(final List<ArchiveDataSourceInfo> archiveDataSourceInfoList, final String downloadDirectory,
                                     final NetAppOCIEPerformanceConfiguration configuration) throws DataSourceErrorException {
        for (ArchiveDataSourceInfo archiveDataSourceInfo : archiveDataSourceInfoList) {
            StringBuilder archiveDirectoryPath = new StringBuilder();
            archiveDirectoryPath.append(downloadDirectory).append(File.separator).append(archiveDataSourceInfo.getNodeName());
            createArchiveDirectory(archiveDirectoryPath.toString(), configuration);
  
            for (String fileName : archiveDataSourceInfo.getFileNames()) {
                try {
                    extractArchiveFile(archiveDirectoryPath.toString(), fileName, configuration.getCurrentSimulationSample());
                }
                catch (Exception e) {
                    String message = configuration.getDataSourceName()
                            + ": Failed to extract archive file. File path: " + archiveDirectoryPath.toString() + File.separator + fileName;
                    throw new DataSourceErrorException(DataSourceErrorType.INTERNAL_ERROR, message, message);
                }
            }
  
        }      
    }
    
    private void createArchiveDirectory(String directoryPath, NetAppOCIEPerformanceConfiguration configuration)
            throws DataSourceErrorException {
        File file = new File(directoryPath);
        if (!file.exists()) {
            if (!file.mkdirs()) {
                String message = configuration.getDataSourceName()
                        + ": Failed to create archive directories. Directory Path: " + file.getAbsolutePath();
                throw new DataSourceErrorException(DataSourceErrorType.INTERNAL_ERROR, message, message);
            }
        }
        else if (file.exists() && !file.isDirectory()) {
            String message = configuration.getDataSourceName() + ": A file " + file.getAbsolutePath()
                    + " does already exist, and might cannot be deleted.";
            throw new DataSourceErrorException(DataSourceErrorType.INTERNAL_ERROR, message, message);
        }

        if (file.exists() && !file.canWrite()) {
            // if directory exists but does not have write permissions
            String message = configuration.getDataSourceName()
                    + ": Archiver files directory does not have write access permissions. Directory Path: "
                    + file.getAbsolutePath();
            throw new DataSourceErrorException(DataSourceErrorType.INTERNAL_ERROR, message, message);
        }
    }
    
    private void extractArchiveFile(String directory, String fileName, DSSessionFileReader.Sample sample)
            throws DataSourceErrorException {
        for (DSInputStream stream : sample.getStreams()) {
            if (stream.getFileName().equals(fileName)) {
                File archiveFile = new File(directory + File.separator + fileName);

                try (BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(archiveFile))) {
                    int bufferSize = 8 * 1024;
                    byte[] buffer = new byte[bufferSize];
                    int count = stream.read(buffer);
                    while (count > 0) {
                        bos.write(buffer, 0, count);
                        count = stream.read(buffer);
                    }
                    bos.flush();
                }
                catch (IOException ioe) {  
                    String message = "Fail to extract archive file: " + fileName;
                    throw new DataSourceErrorException(DataSourceErrorType.INTERNAL_ERROR, message, message, ioe);
                }
            }
        }
    }
    
}
