package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.metrics;


import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;

/**
 * This is an extension of the {@link MetricsRowScratchPad} class, so it can be used with for the archiver.
 *
 * @param <C> An Enumeration, defined by implementations of {@link com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.builders.ArchiveMetricsBuilderBase}
 *           classes, that identifies the set of counter/column names requested from Archive
 */
public class ArchiveMetricsScratchPad<C extends Enum<C>> extends MetricsRowScratchPad<C> {

    private final Number archiveHigh;
    private final Number archiveLow;

    public ArchiveMetricsScratchPad(final Class enumClass, final Number archiveHi, final Number archiveLo,
                                    final String performanceInstanceUUID, final String inventoryUUID,
                                    final long serverPollStartTimeStamp, final long archiveTimeStamp,
                                    final ArchiveMetricsScratchPad previousValues,
                                    final boolean isMetricsDebugEnabled ) {
        super(enumClass, performanceInstanceUUID, inventoryUUID,
                serverPollStartTimeStamp, archiveTimeStamp, previousValues, isMetricsDebugEnabled);
        this.archiveHigh = archiveHi;
        this.archiveLow = archiveLo;
    }

    public ArchiveMetricsScratchPad(Class<C> enumClass, String perfInstanceUUID, String inventoryUUID,
                                    long serverPollStartTimeStamp, long pollStartTimeStamp,
                                    MetricsRowScratchPad<C> previousValues,
                                    final boolean isMetricsDebugEnabled) {
        super(enumClass, perfInstanceUUID, inventoryUUID, serverPollStartTimeStamp, pollStartTimeStamp, previousValues,
                isMetricsDebugEnabled);
        this.archiveHigh = null;
        this.archiveLow = null;
    }

    public ArchiveMetricsScratchPad(final Class enumClass, final Number archiveHi, final Number archiveLo,
                                    final String performanceInstanceUUID, final String inventoryUUID,
                                    final long serverPollStartTimeStamp, final long archiveTimeStamp,
                                    final ArchiveMetricsScratchPad previousValues) {
        this(enumClass, archiveHi, archiveLo, performanceInstanceUUID, inventoryUUID, serverPollStartTimeStamp,
                archiveTimeStamp, previousValues, true);
    }

    /**
     *  Returns the static column headers in the archiver recordings.
     */
    public String getStaticDebugColumnHeaders() {
        final StringBuilder debugBuilder = new StringBuilder();
        debugBuilder.append("\n");
        //Only some counter objects have archive_hi & archive_lo
        if(archiveHigh != null || archiveLow != null) {
            debugBuilder.append("archive hi, archive lo,");
        }
        debugBuilder.append("foundation UUID,performance UUID,,sample time,,archive poll start," +
                "prev archive poll start,delta archive poll start,,");
        return debugBuilder.toString();
    }

    /**
     * Returns the values for the static columns in the archiver recordings.
     */
    public String getStaticDebugColumns() {
        Long lastArchivePollStartTimeStamp = null;
        Long archiveDeltaBetweenPolls = null;
        if (previousValues != null) {
            lastArchivePollStartTimeStamp = previousValues.getPollStartTimeStamp();
            archiveDeltaBetweenPolls = pollStartTimeStamp - lastArchivePollStartTimeStamp;
        }
        final StringBuilder debugBuilder = new StringBuilder();
        if(archiveHigh != null || archiveLow != null) {
            debugBuilder.append(getDebugString(archiveHigh)).append(',');
            debugBuilder.append(getDebugString(archiveLow)).append(",");
        }
        debugBuilder.append(inventoryUUID).append(",");
        debugBuilder.append(perfInstanceUUID).append(",,");

        debugBuilder.append(getDebugString(serverPollStartTimeStamp)).append(",,");
        debugBuilder.append(getDebugString(pollStartTimeStamp)).append(',');
        debugBuilder.append(getDebugString(lastArchivePollStartTimeStamp)).append(',');
        debugBuilder.append(getDebugString(archiveDeltaBetweenPolls)).append(",,");
        return debugBuilder.toString();
    }
}
