package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.parser;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * This iterator is used to iterate through the counter data from the archive files. The class represents counter data
 * for a counter object such as workload_queue_nblade, workload_queue_dblade ect, at a given time.
 *
 * Note : This class is not thread safe. {@link #hasNext()} and {@link #next()} will not provide the expected result if
 * multiple threads access concurrently.
 */
public class CounterTable implements java.util.Iterator<Map<String, Object>> {
    /**
     * Timestamp from the archive, The time with the counter values are sampled.
     */
    private final long archiveTimeStamp;
    /**
     * Name of the counter object
     */
    private final String counterObjectName;
    /**
     * A table of counter data. Counters contain List of Rows, with each row containing a list of columns and each
     * column has a list of values of Sting or a Number.
     */
    private final List<CounterRow> counterRowList;
    /**
     * OnTap Node name from where the archiver was downloaded.
     */
    private String nodeName;
    /**
     * Pointer to the current row.
     */
    private int currentIndex = -1;


    public CounterTable(final long archiveTimestamp, final String counterObjectName, final List<CounterRow> counterRowList) {
        this.archiveTimeStamp = archiveTimestamp;
        this.counterObjectName = counterObjectName;
        this.counterRowList = counterRowList;
    }

    public boolean hasNext() {
        return (currentIndex + 1) < counterRowList.size();
    }

    public Map<String, Object> next() {
        if (hasNext()) {
            currentIndex++; //Increment current index
            return counterRowList.get(currentIndex).getCounterColumnMap();
        } else {
            throw new IndexOutOfBoundsException();
        }
    }

    @Override
    public void remove() {
        throw new UnsupportedOperationException("Cannot remove elements from Counter Table, this is a read only tables");
    }

    public String getCounterObjectName() {
        return counterObjectName;
    }

    public long getArchiveTimeStamp() {
        return archiveTimeStamp;
    }

    public String getNodeName() {
        return nodeName;
    }

    public void setNodeName(final String nodeName) {
        this.nodeName = nodeName;
    }

    @Override
    public String toString() {
        return new StringBuilder("CounterData{")
                .append("archiveTimeStamp=").append(archiveTimeStamp)
                .append(", nodeName='").append(nodeName).append('\'')
                .append(", counterObjectName='").append(counterObjectName).append('\'')
                .append(", counters=").append(counterRowList).append("} ")
                .append(super.toString()).toString();
    }
}
