package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCIFSACL;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.framework.datasource.PackageContext;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.CIFSShareACL;
import org.apache.log4j.Logger;
import org.jdom.Element;

import java.util.LinkedHashMap;

public class CIFSACLBuilder extends IterBuilderBase {

    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger logger = Logger.getLogger(CIFSACLBuilder.class);

    protected CIFSACLBuilder() {
        super("cifs-share-access-control-get-iter", "cifs-share-access-control", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"cifs-share-access-control"}, new String[]{"permission", "share", "user-or-group", "vserver",});
        }});
    }


    @Override
    protected void buildIterInstance(final CModeCluster cluster, final Element accessControlElt) {

        final String vServerName = BuilderUtil.getChildTextTrim(accessControlElt, "vserver");
        final String shareName = BuilderUtil.getChildTextTrim(accessControlElt, "share");
        final String userOrGroup = BuilderUtil.getChildTextTrim(accessControlElt, "user-or-group");

        // RR.1 does not return vServerName, required for uniqueness.
        if (vServerName == null) {
            if (logger.isDebugEnabled()) {
                logger.warn(PackageContext.getLogPrefix() + " - CIFS Share ACL " + shareName + " " + userOrGroup
                            + " failed to identify containing vServer, ignoring.");
            }
            return;
        }

        final EnumWrapper<CIFSShareACL.Permission> permissionWrapper =
                BuilderUtil.getEnumWrapperByValue(CIFSShareACL.Permission.class, accessControlElt, "permission", CIFSShareACL.Permission.NOT_MAPPED);
        final CModeCIFSACL cifsShareACL = cluster.createCIFsShareACL(vServerName, shareName, userOrGroup);
        cifsShareACL.setPermissionWrapper(permissionWrapper);
        cluster.getCIFSShareACLs().add(cifsShareACL);
    }
}
