package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapperList;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCIFSServer;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCIFSShare;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPINames;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.CIFSShare;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.VServer;
import org.apache.log4j.Logger;
import org.jdom.Element;

import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIUtils.children;
import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIUtils.elem;


import java.util.LinkedHashMap;

public class CIFSBuilder extends IterBuilderBase {

    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger logger = Logger.getLogger(LUNBuilder.class);

    public CIFSBuilder() {
        super("cifs-server-get-iter", "cifs-server-config", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"cifs-server-config"}, new String[]{"cifs-server", "vserver", "domain", "auth-style", "domain-workgroup", "administrative-status",});
        }});
    }

    @Override
    public void buildModel(final CModeCluster cluster, final ZAPIConnection zapiConnection) throws DataSourceErrorException {
        super.buildModel(cluster, zapiConnection);
        for (final CModeCIFSServer cifsServer : cluster.getCifsServers().values()) {
            new CIFSShareBuilder(cifsServer).buildModel(cluster, zapiConnection);
        }
    }

    protected void buildIterInstance(final CModeCluster cluster, final Element cifsServerAttrElt) {
        final String cifsServerName = BuilderUtil.getChildTextTrim(cifsServerAttrElt, "cifs-server");
        final String vServerName = BuilderUtil.getChildTextTrim(cifsServerAttrElt, "vserver");
        final String cifsDomain = BuilderUtil.getChildTextTrim(cifsServerAttrElt, "domain");
        final String cifsDomainWorkgroup = BuilderUtil.getChildTextTrim(cifsServerAttrElt, "domain-workgroup");
        final EnumWrapper<VServer.CifsAuthStyle> cifsAuthStyle = BuilderUtil.getEnumWrapperByValue(VServer.CifsAuthStyle.class, cifsServerAttrElt, "auth-style", VServer.CifsAuthStyle.NOT_MAPPED);
        final EnumWrapper<VServer.CifsAdminStatus> cifsServerAdminStatus = BuilderUtil.getEnumWrapperByValue(VServer.CifsAdminStatus.class, cifsServerAttrElt, "administrative-status", VServer.CifsAdminStatus.NOT_MAPPED);
        final CModeCIFSServer cifsServer = cluster.createCifsServer(cifsServerName);
        cifsServer.setVServerName(vServerName);
        cifsServer.setDomain(cifsDomain);
        cifsServer.setDomainWorkgroup(cifsDomainWorkgroup);
        cifsServer.setAuthSytle(cifsAuthStyle);
        cifsServer.setAdminStatus(cifsServerAdminStatus);
        cluster.getCifsServers().add(cifsServer);
    }

    public static final class CIFSShareBuilder extends IterBuilderBase {
        public static final String VSERVER = "vserver";
        public static final String CIFS_SHARE = "cifs-share";
        public static final String CIFS_SERVER = "cifs-server";
        public static final String SHARE_PROPERTIES = "share-properties";
        public static final String CIFS_SHARE_PROPERTIES = "cifs-share-properties";

        public CIFSShareBuilder(final CModeCIFSServer cifsServer) {
            super("cifs-share-get-iter", "cifs-share", getDefaultMaxRecords(),
                    elem(ZAPINames.DESIRED_ATTRIBUTES,
                            elem(CIFS_SHARE, children("cifs-server", "path", "share-name", "comment",
                                    elem(SHARE_PROPERTIES, elem(CIFS_SHARE_PROPERTIES))))
                            ),
                            elem(ZAPINames.QUERY,
                                elem(CIFS_SHARE,
                                        elem(VSERVER, cifsServer.getVServerName()),
                                        elem(CIFS_SERVER, cifsServer.getName()))));
        }

        protected void buildIterInstance(final CModeCluster cluster, final Element cifsShareAttrElt) {
            final String cifsServerName = BuilderUtil.getChildTextTrim(cifsShareAttrElt, "cifs-server");
            final String path = BuilderUtil.getChildTextTrim(cifsShareAttrElt, "path");
            final String shareName = BuilderUtil.getChildText(cifsShareAttrElt, "share-name"); // need to preserve leading, trailing spaces in name
            final String comment = BuilderUtil.getChildTextTrim(cifsShareAttrElt, "comment");
            final EnumWrapperList<CIFSShare.ShareProperty> shareProperties = BuilderUtil.getEnumWrapperListByValue(
                    CIFSShare.ShareProperty.class, cifsShareAttrElt, "share-properties", "cifs-share-properties");

            final CModeCIFSShare cifsShare = cluster.createCifsShare(cifsServerName, shareName);
            cifsShare.setPath(path);
            cifsShare.setComment(comment);
            cifsShare.getShareProperties().set(shareProperties);
            cluster.getCifsShares().add(cifsShare);
        }
    }
}
