package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;


import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.PackageContext;
import org.apache.log4j.Logger;

import java.util.concurrent.Callable;

/**
 * Wrapper so that Builders may participate in executor framework as Callables.
 */
public final class CallableBuilder implements Callable<CallableBuilder.BuilderResult> {

    private static final Logger logger = Logger.getLogger(ClusterBuilder.class);

    private final BuilderBase builder;
    private final CModeCluster cluster;
    private final ZAPIConnection zapiConnection;
    private final com.onaro.sanscreen.acquisition.framework.datasource.Package p;

    public CallableBuilder (final BuilderBase builder, final CModeCluster cluster, final ZAPIConnection zapiConnection) {
        this.builder = builder;
        this.cluster = cluster;
        this.zapiConnection = zapiConnection;
        //We assume the thread setting up this object is the main 'foundation' thread with the right thread local
        //for the package.  Store it here so that we can transfer it to the 'callable' thread below.
        //TODO - maybe find a more elegant way of doing this.
        p = PackageContext.getPackage();
    }


    public CModeCluster getCluster() {
        return cluster;
    }


    public ZAPIConnection getZapiConnection() {
        return zapiConnection;
    }

    /**
     * Run this builder within the context of the executor framework.
     *
     * @return builder result with simple meta data about builder.
     * @throws Exception  problems with running the builder.
     */
    public BuilderResult call() throws Exception {
        PackageContext.initialize(p);  //initialize package context thread local for the callable thread.

        final String name = builder.getClass().getSimpleName();

        if (logger.isDebugEnabled()) {
            logger.debug(p.getLogPrefix() + " Starting builder: " + name);
        }

        builder.buildModel(cluster, zapiConnection);

        if (logger.isDebugEnabled()) {
            logger.debug(p.getLogPrefix() + " Finished builder: " + name);
        }


        return new BuilderResult(name, PackageContext.secondsSinceStart());
    }

  public static class BuilderResult {
        private final String name;
        private final Long time;

        public BuilderResult(final String name, final Long time) {
            this.name = name;
            this.time = time;
        }

        /**
         * The class name of the builder.
         *
         * @return  the class name of the builder.
         */
        public String getName() {
            return name;
        }

        /**
         * Time it took this builder to run.
         *
         * @return   long time in ms it took the builder to run.
         */
        public Long getTime() {
            return time;
        }
    }

}
