package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeDisk;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeDiskPath;
import com.onaro.sanscreen.acquisition.framework.datasource.PackageContext;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Disk;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.DiskPath;
import org.apache.log4j.Logger;
import org.jdom.Element;

import java.util.LinkedHashMap;
import java.util.List;

public class DiskBuilder extends IterBuilderBase {
    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger logger = Logger.getLogger(DiskBuilder.class);

    public DiskBuilder() {
        super("storage-disk-get-iter", "storage-disk-info", 2000, new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"storage-disk-info",}, new String[]{"disk-name", "disk-type", "disk-uid",});
            put(new String[]{"storage-disk-info", "disk-inventory-info",}, new String[]{"checksum-compatibility", "shelf", "shelf-bay", "vendor", "model", "serial-number", "disk-type", "rpm", "firmware-revision", "bytes-per-sector", "capacity-sectors", "right-size-sectors", "is-shared"});
            put(new String[]{"storage-disk-info", "disk-inventory-info", "storage-ssd-info",}, new String[]{"percent-spares-consumed","percent-rated-life-used"});
            put(new String[]{"storage-disk-info", "disk-metrocluster-info",}, new String[]{"is-local-attach"});
            put(new String[]{"storage-disk-info", "disk-ownership-info",}, new String[]{"home-node-id", "home-node-name", "is-failed", "owner-node-id", "owner-node-name", "pool", "reserved-by-node-id","dr-home-node-id","dr-home-node-name",});
            put(new String[]{"storage-disk-info", "disk-raid-info",}, new String[]{"active-node-name", "container-type", "effective-disk-type", "physical-blocks", "used-blocks", "position", "spare-pool",});
            put(new String[]{"storage-disk-info", "disk-raid-info", "disk-aggregate-info",}, new String[]{"aggregate-name", "checksum-type", "is-media-scrubbing", "is-offline", "is-prefailed", "is-reconstructing", "is-replacing", "is-zeroed", "is-zeroing", "plex-name", "raid-group-name", "zeroing-percent-complete"});
            put(new String[]{"storage-disk-info", "disk-raid-info", "disk-shared-info",}, new String[]{"partitioning-type","storage-pool"});
            put(new String[]{"storage-disk-info", "disk-raid-info", "disk-spare-info",}, new String[]{"is-media-scrubbing", "is-zeroed",});
            put(new String[]{"storage-disk-info", "disk-raid-info", "disk-outage-info",}, new String[]{"is-in-fdr","reason",});
            put(new String[]{"storage-disk-info", "disk-paths", "disk-path-info"}, new String[]{"array-name", "disk-port", "disk-port-name", "disk-uuid", "initiator-port", "initiator-side-switch-port", "lun-number", "lun-path-use-state", "node", "target-side-switch-port", "target-wwpn", "tpgn", "target-wwnn"});
        }});  // Default is 2000 for disks!
    }

    protected void buildIterInstance(final CModeCluster cluster, final Element storageDiskInfoElt) {
        final String diskName = BuilderUtil.getChildTextTrim(storageDiskInfoElt, "disk-name");
        final String diskUUID = BuilderUtil.getChildTextTrim(storageDiskInfoElt, "disk-uid");

        if ((diskUUID == null) || diskUUID.isEmpty()) {
            logger.warn(PackageContext.getLogPrefix() + " - Unknown UUID for disk " + diskName + ", ignoring.");
            return;
        }

        final Element diskInventoryInfoElt = storageDiskInfoElt.getChild("disk-inventory-info");
        final String shelf = BuilderUtil.getChildTextTrim(diskInventoryInfoElt, "shelf");
        final String bay = BuilderUtil.getChildTextTrim(diskInventoryInfoElt, "shelf-bay");
        final String vendor = BuilderUtil.getChildTextTrim(diskInventoryInfoElt, "vendor");
        final String model = BuilderUtil.getChildTextTrim(diskInventoryInfoElt, "model");
        final String serialNum = BuilderUtil.getChildTextTrim(diskInventoryInfoElt, "serial-number");
        final EnumWrapper<Disk.DiskType> diskType = BuilderUtil.getEnumWrapperByValue(Disk.DiskType.class, diskInventoryInfoElt, "disk-type", Disk.DiskType.NOT_MAPPED);
        final Boolean isVirtual = (Disk.DiskType.LUN == diskType.getEnumValue());
        final Long rpm = BuilderUtil.parseLongFromChild(diskInventoryInfoElt, "rpm");
        final String firmwareRev = BuilderUtil.getChildTextTrim(diskInventoryInfoElt, "firmware-revision");
        final Long bytesPerSector = BuilderUtil.parseLongFromChild(diskInventoryInfoElt, "bytes-per-sector");
        final Long capacitySectors = BuilderUtil.parseLongFromChild(diskInventoryInfoElt, "capacity-sectors");
        final Long rightSizeSectors = BuilderUtil.parseLongFromChild(diskInventoryInfoElt, "right-size-sectors");
        final String checksumCompatibility = BuilderUtil.getChildTextTrim(diskInventoryInfoElt, "checksum-compatibility");
        final Boolean isShared = BuilderUtil.parseBooleanFromChildDefaultFalse(diskInventoryInfoElt, "is-shared");

        //storage ssd info
        final Element storageSsdInfoElt = diskInventoryInfoElt.getChild("storage-ssd-info");
        Integer percentSparesConsumed = null;
        Integer percentRatedLifeUsed = null;

        if(storageSsdInfoElt!=null) {
            percentSparesConsumed = BuilderUtil.parseIntFromChild(storageSsdInfoElt, "percent-spares-consumed");
            percentRatedLifeUsed = BuilderUtil.parseIntFromChild(storageSsdInfoElt, "percent-rated-life-used");
        }

        final Element diskMCCInfoElt = storageDiskInfoElt.getChild("disk-metrocluster-info");
        Boolean isLocalAttach = true;   //by default it is true. This applies to earlier version where is no diskMCCInfoElt.
        if(diskMCCInfoElt!=null)  {
            isLocalAttach = BuilderUtil.parseBooleanFromChild(diskMCCInfoElt, "is-local-attach");
        }

        final Element diskOwnerInfoElt = storageDiskInfoElt.getChild("disk-ownership-info");
        final String homeNodeID = BuilderUtil.getChildTextTrim(diskOwnerInfoElt, "home-node-id");
        final String homeNodeName = BuilderUtil.getChildTextTrim(diskOwnerInfoElt, "home-node-name");
        final Boolean isFailed = BuilderUtil.parseBooleanFromChildDefaultFalse(diskOwnerInfoElt, "is-failed");
        final String ownerNodeID = BuilderUtil.getChildTextTrim(diskOwnerInfoElt, "owner-node-id");
        final String ownerNodeName = BuilderUtil.getChildTextTrim(diskOwnerInfoElt, "owner-node-name");
        final String drHomeNodeId = BuilderUtil.getChildTextTrim(diskOwnerInfoElt, "dr-home-node-id");
        final Long poolID = BuilderUtil.parseLongFromChild(diskOwnerInfoElt, "pool");
        final String reservedByNodeID = BuilderUtil.getChildTextTrim(diskOwnerInfoElt, "reserved-by-node-id");

        final Element diskRAIDInfoElt = storageDiskInfoElt.getChild("disk-raid-info");
        final String activeNodeName = BuilderUtil.getChildTextTrim(diskRAIDInfoElt, "active-node-name");
        final EnumWrapper<Disk.DiskContainerType> containerType = BuilderUtil.getEnumWrapperByValue(Disk.DiskContainerType.class, diskRAIDInfoElt, "container-type", Disk.DiskContainerType.NOT_MAPPED);
        final EnumWrapper<Disk.DiskInterfaceType> effectiveDiskType = BuilderUtil.getEnumWrapperByValue(Disk.DiskInterfaceType.class, diskRAIDInfoElt, "effective-disk-type", Disk.DiskInterfaceType.NOT_MAPPED);
        final Long physicalBlocks = BuilderUtil.parseLongFromChild(diskRAIDInfoElt, "physical-blocks");
        final Long usedBlocks = BuilderUtil.parseLongFromChild(diskRAIDInfoElt, "used-blocks");
        final EnumWrapper<Disk.RaidPosition> position = BuilderUtil.getEnumWrapperByValue(Disk.RaidPosition.class, diskRAIDInfoElt, "position", Disk.RaidPosition.NOT_MAPPED);
        final String sparePool = BuilderUtil.getChildTextTrim(diskRAIDInfoElt, "spare-pool");

        // Disks in Aggregate attributes
        String aggregateName = null;
        String storagePoolName = null;
        String checkSumType = null;
        String plexName = null;
        String raidGroupName = null;
        Integer zeroingPercentComplete = null;
        Boolean isZeroing = null;
        Boolean isOffline = null;
        Boolean isPreFailed = null;
        Boolean isReconstructing = null;
        Boolean isReplacing = null;
        Boolean isZeroed = null;                    // Spare or Aggregate
        Boolean isMediaScrubbing = null;            // Spare or Aggregate
        final Boolean isInFDR;
        final String failedReason;
        EnumWrapper<Disk.PartitioningType> partitioningType=new EnumWrapper<Disk.PartitioningType>(null, null);
        if (diskRAIDInfoElt != null) {
            final Element diskSharedInfo = diskRAIDInfoElt.getChild("disk-shared-info");
            storagePoolName = BuilderUtil.getChildTextTrim(diskSharedInfo, "storage-pool");
            partitioningType = BuilderUtil.getEnumWrapperByValue(Disk.PartitioningType.class, diskSharedInfo, "partitioning-type", Disk.PartitioningType.NOT_MAPPED);
            final Element diskOutageInfo = diskRAIDInfoElt.getChild("disk-outage-info");
            isInFDR = BuilderUtil.parseBooleanFromChild(diskOutageInfo, "is-in-fdr");
            failedReason = BuilderUtil.getChildTextTrim(diskOutageInfo, "reason");
            final Disk.DiskContainerType containerTypeEnum = containerType.getEnumValue();
            if (containerTypeEnum != null) {
                if (containerTypeEnum.equals(Disk.DiskContainerType.AGGREGATE)) {
                    final Element diskAggrInfoElt = diskRAIDInfoElt.getChild("disk-aggregate-info");
                    aggregateName = BuilderUtil.getChildTextTrim(diskAggrInfoElt, "aggregate-name");
                    checkSumType = BuilderUtil.getChildTextTrim(diskAggrInfoElt, "checksum-type");
                    plexName = BuilderUtil.getChildTextTrim(diskAggrInfoElt, "plex-name");
                    raidGroupName = BuilderUtil.getChildTextTrim(diskAggrInfoElt, "raid-group-name");
                    zeroingPercentComplete = BuilderUtil.parseIntFromChild(diskAggrInfoElt, "zeroing-percent-complete");
                    isMediaScrubbing = BuilderUtil.parseBooleanFromChild(diskAggrInfoElt, "is-media-scrubbing");
                    isOffline = BuilderUtil.parseBooleanFromChild(diskAggrInfoElt, "is-offline");
                    isPreFailed = BuilderUtil.parseBooleanFromChild(diskAggrInfoElt, "is-prefailed");
                    isReconstructing = BuilderUtil.parseBooleanFromChild(diskAggrInfoElt, "is-reconstructing");
                    isReplacing = BuilderUtil.parseBooleanFromChild(diskAggrInfoElt, "is-replacing");
                    isZeroing = BuilderUtil.parseBooleanFromChild(diskAggrInfoElt, "is-zeroing");
                    isZeroed = BuilderUtil.parseBooleanFromChild(diskAggrInfoElt, "is-zeroed");
                }
                else if (containerTypeEnum.equals(Disk.DiskContainerType.SPARE)) {
                    final Element diskSpareInfoElt = diskRAIDInfoElt.getChild("disk-spare-info");
                    isMediaScrubbing = BuilderUtil.parseBooleanFromChild(diskSpareInfoElt, "is-media-scrubbing");
                    isZeroing = BuilderUtil.parseBooleanFromChild(diskSpareInfoElt, "is-zeroing");
                    zeroingPercentComplete = BuilderUtil.parseIntFromChild(diskSpareInfoElt, "zeroing-percent-complete");
                    isZeroed = BuilderUtil.parseBooleanFromChild(diskSpareInfoElt, "is-zeroed");
                }
            }
        } else {
            isInFDR = null;
            failedReason = null;
        }

        final CModeDisk cModeDisk = cluster.createDisk(diskUUID);
        if (diskName != null) {
            cModeDisk.setDiskName(diskName);
        } else {
            cModeDisk.setDiskName(serialNum);
        }

        cModeDisk.setIsLocalAttach(isLocalAttach);
        cModeDisk.setShelf(shelf);
        cModeDisk.setBay(bay);
        cModeDisk.setVendor(vendor);
        cModeDisk.setModel(model);
        cModeDisk.setSerialNum(serialNum);
        cModeDisk.setRpm(rpm);
        cModeDisk.setFirmwareRev(firmwareRev);
        cModeDisk.setBytesPerSector(bytesPerSector);
        cModeDisk.setCapacitySectors(capacitySectors);
        cModeDisk.setRightSizeSectors(rightSizeSectors);
        cModeDisk.setHomeNodeID(homeNodeID);
        cModeDisk.setHomeNodeName(homeNodeName);
        cModeDisk.setFailed(isFailed);
        cModeDisk.setOwnerNodeID(ownerNodeID);
        cModeDisk.setOwnerNodeName(ownerNodeName);
        cModeDisk.setDrHomeNodeID(drHomeNodeId);
        cModeDisk.setPoolID(poolID);
        cModeDisk.setReservedByNodeIDString(reservedByNodeID);
        cModeDisk.setActiveNodeName(activeNodeName);
        cModeDisk.setContainerType(containerType);
        cModeDisk.setDiskType(diskType);
        cModeDisk.setEffectiveDiskType(effectiveDiskType);
        cModeDisk.setPhysicalBlocks(physicalBlocks);
        cModeDisk.setUsedBlocks(usedBlocks);
        cModeDisk.setPosition(position);
        cModeDisk.setSparePool(sparePool);
        cModeDisk.setAggregateName(aggregateName);
        cModeDisk.setStoragePoolName(storagePoolName);
        cModeDisk.setCheckSumType(checkSumType);
        cModeDisk.setPlexName(plexName);
        cModeDisk.setRaidGroupName(raidGroupName);
        cModeDisk.setMediaScrubbing(isMediaScrubbing);
        cModeDisk.setZeroed(isZeroed);
        cModeDisk.setVirtual(isVirtual);
        cModeDisk.setChecksumCompatibility(checksumCompatibility);
        cModeDisk.setZeroing(isZeroing);
        cModeDisk.setZeroingPercentComplete(zeroingPercentComplete);
        cModeDisk.setOffline(isOffline);
        cModeDisk.setPreFailed(isPreFailed);
        cModeDisk.setReconstructing(isReconstructing);
        cModeDisk.setReplacing(isReplacing);
        cModeDisk.setShared(isShared);
        cModeDisk.setInFDR(isInFDR);
        cModeDisk.setFailedReason(failedReason);
        cModeDisk.setPercentSparesConsumed(percentSparesConsumed);
        cModeDisk.setPercentRatedLifeUsed(percentRatedLifeUsed);
        cModeDisk.setPartitioningType(partitioningType);
        cluster.getDisks().add(cModeDisk);

        // Paths
        final Element diskPathsElement = storageDiskInfoElt.getChild("disk-paths");
        if (diskPathsElement != null) {
            @SuppressWarnings({"unchecked"})
            final List<Element> diskPathInfoElements = diskPathsElement.getChildren("disk-path-info");

            for (final Element diskPathInfoElement : diskPathInfoElements) {

                final String diskUUIDFromPath = BuilderUtil.getChildTextTrim(diskPathInfoElement, "disk-uid");
                final String arrayName = BuilderUtil.getChildTextTrim(diskPathInfoElement, "array-name");
                final String diskPort = BuilderUtil.getChildTextTrim(diskPathInfoElement, "disk-port");
                final String diskPortName = BuilderUtil.getChildTextTrim(diskPathInfoElement, "disk-port-name");
                final String initiatorPortName = BuilderUtil.getChildTextTrim(diskPathInfoElement, "initiator-port");
                final String initiatorSideSwitchPort = BuilderUtil.getChildTextTrim(diskPathInfoElement, "initiator-side-switch-port");
                final Integer lun = BuilderUtil.parseIntFromChild(diskPathInfoElement, "lun-number");
                final EnumWrapper<DiskPath.LUNPathUseStateType> lunPathUseState = BuilderUtil.getEnumWrapperByValue(DiskPath.LUNPathUseStateType.class, diskPathInfoElement, "lun-path-use-state", DiskPath.LUNPathUseStateType.NOT_MAPPED);
                final String node = BuilderUtil.getChildTextTrim(diskPathInfoElement, "node");
                final String targetWWPN = BuilderUtil.getChildTextTrim(diskPathInfoElement, "target-wwpn");
                final String formattedWWPN = (targetWWPN == null)
                        ? null
                        : BuilderUtil.normalizeWWN(targetWWPN);
                
                final String targetWWNN = BuilderUtil.getChildTextTrim(diskPathInfoElement, "target-wwnn");
                final String formattedWWNN = (targetWWNN == null) ? null : BuilderUtil.normalizeWWN(targetWWNN);

                final String targetSideSwitchPort = BuilderUtil.getChildTextTrim(diskPathInfoElement, "target-side-switch-port");
                final Long targetPortGroupNum = BuilderUtil.parseLongFromChild(diskPathInfoElement, "tpgn");

                final CModeDiskPath cModeDiskPath = cModeDisk.createCModeDiskPath(diskUUIDFromPath, initiatorPortName, formattedWWPN);
                cModeDiskPath.setArrayName(arrayName);
                cModeDiskPath.setDiskPort(diskPort);
                cModeDiskPath.setDiskPortName(diskPortName);
                cModeDiskPath.setInitiatorSideSwitchPort(initiatorSideSwitchPort);
                cModeDiskPath.setLun(lun);
                cModeDiskPath.setLunPathUseState(lunPathUseState);
                cModeDiskPath.setNode(node);
                cModeDiskPath.setTargetSideSwitchPort(targetSideSwitchPort);
                cModeDiskPath.setTargetPortGroupNum(targetPortGroupNum);
                cModeDiskPath.setTargetWWNN(formattedWWNN);
                cModeDisk.getDiskPaths().add(cModeDiskPath);
            }
        }
    }
}
