package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeFcpAdapter;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeFcpServiceInfo;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.FcpPort;
import org.jdom.Element;

import java.util.LinkedHashMap;

public class FcpBuilder extends IterBuilderBase {

    @Override
    public void buildModel(final CModeCluster cluster, final ZAPIConnection zapiConnection) throws DataSourceErrorException {
        super.buildModel(cluster, zapiConnection);                              // fcp-adapter-get-iter
        new FcpServiceBuilder().buildModel(cluster, zapiConnection);            // fcp-service-get-iter
    }

    public FcpBuilder() {
        super("fcp-adapter-get-iter", "fcp-config-adapter-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"fcp-config-adapter-info",}, new String[]{"adapter", "data-link-rate", "max-speed", "media-type", "node", "node-name", "port-name", "physical-protocol", "speed", "state", "switch-port", "fabric-established", "info-name", "firmware-rev",});
        }});
    }

    @Override
    protected void buildIterInstance(final CModeCluster cluster, final Element fcpConfigAdapterInfoElt) {
        final String adapterName = BuilderUtil.getChildTextTrim(fcpConfigAdapterInfoElt, "adapter");
        final Integer dataLinkRate = BuilderUtil.parseIntFromChild(fcpConfigAdapterInfoElt, "data-link-rate");
        final Integer maxSpeed = BuilderUtil.parseIntFromChild(fcpConfigAdapterInfoElt, "max-speed");
        final EnumWrapper<FcpPort.MediaType> mediaType = BuilderUtil.getEnumWrapperByValue(FcpPort.MediaType.class, fcpConfigAdapterInfoElt, "media-type", FcpPort.MediaType.NOT_MAPPED);
        final String nodeName = BuilderUtil.getChildTextTrim(fcpConfigAdapterInfoElt, "node");
        final String rawNodeWWN = BuilderUtil.getChildTextTrim(fcpConfigAdapterInfoElt, "node-name");
        final String nodeWWN = BuilderUtil.normalizeWWN(rawNodeWWN);
        final String rawPortWWN = BuilderUtil.getChildTextTrim(fcpConfigAdapterInfoElt, "port-name");
        final String portWWN = BuilderUtil.normalizeWWN(rawPortWWN);
        final EnumWrapper<FcpPort.PhysicalProtocol> physicalProtocol = BuilderUtil.getEnumWrapperByValue(FcpPort.PhysicalProtocol.class, fcpConfigAdapterInfoElt, "physical-protocol", FcpPort.PhysicalProtocol.NOT_MAPPED);
        final EnumWrapper<FcpPort.Speed> speed = getFcpPortSpeed(fcpConfigAdapterInfoElt, "speed");
        final String stateRaw = BuilderUtil.getChildTextTrim(fcpConfigAdapterInfoElt, "state");
        final EnumWrapper<FcpPort.State> state = new EnumWrapper<FcpPort.State>(stateRaw, FcpPort.State.fromString(stateRaw));
        final String switchPort = BuilderUtil.getChildTextTrim(fcpConfigAdapterInfoElt, "switch-port");
        final Boolean isFabricEstablished = BuilderUtil.parseBooleanFromChild(fcpConfigAdapterInfoElt, "fabric-established");
        final String infoName = BuilderUtil.getChildTextTrim(fcpConfigAdapterInfoElt, "info-name");
        final String firmwareRev = BuilderUtil.getChildTextTrim(fcpConfigAdapterInfoElt, "firmware-rev");
        final CModeFcpAdapter fcpAdapter = cluster.createFcpAdapter(nodeName, adapterName);
        fcpAdapter.setDataLinkRate(dataLinkRate);
        fcpAdapter.setMaxSpeed(maxSpeed);
        fcpAdapter.setMediaType(mediaType);
        fcpAdapter.setNodeWWN(nodeWWN);
        fcpAdapter.setPortWWN(portWWN);
        fcpAdapter.setPhysicalProtocol(physicalProtocol);
        fcpAdapter.setSpeed(speed);
        fcpAdapter.setState(state);
        fcpAdapter.setSwitchPort(switchPort);
        fcpAdapter.setFabricEstablished(isFabricEstablished);
        fcpAdapter.setInfoName(infoName);
        fcpAdapter.setFirmwareRev(firmwareRev);
        cluster.getFcpAdapters().add(fcpAdapter);
    }

    private EnumWrapper<FcpPort.Speed> getFcpPortSpeed(final Element elt, final String targetChildName) {
        final String speedString = BuilderUtil.getChildTextTrim(elt, targetChildName);
        final Integer speedInt = BuilderUtil.parseIntFromString(speedString);
        return (speedInt == null) ? BuilderUtil.getEnumWrapperByValue(FcpPort.Speed.class, speedString, FcpPort.Speed.NOT_MAPPED) : new EnumWrapper<FcpPort.Speed>(speedString, getFcpPortSpeedEnum(speedInt));
    }

    private FcpPort.Speed getFcpPortSpeedEnum(final Integer portSpeedInt) {
        final FcpPort.Speed portSpeed;
        switch (portSpeedInt) {
            case 1:
                portSpeed = FcpPort.Speed.GIGABIT_1;
                break;
            case 4:
                portSpeed = FcpPort.Speed.GIGABIT_4;
                break;
            case 8:
                portSpeed = FcpPort.Speed.GIGABIT_8;
                break;
            case 10:
                portSpeed = FcpPort.Speed.GIGABIT_10;
                break;
            case 16:
                portSpeed = FcpPort.Speed.GIGABIT_16;
                break;
            default:
                portSpeed = FcpPort.Speed.NOT_MAPPED;
                break;
        }
        return portSpeed;
    }

    private static final class FcpServiceBuilder extends IterBuilderBase {
        private FcpServiceBuilder() {
            super("fcp-service-get-iter", "fcp-service-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
                put(new String[]{"fcp-service-info",}, new String[]{"is-available", "node-name", "vserver",});
            }});
        }

        protected void buildIterInstance(final CModeCluster cluster, final Element fcpServiceInfoElt) {
            final String vServerString = BuilderUtil.getChildTextTrim(fcpServiceInfoElt, "vserver");
            final Boolean isAvailable = BuilderUtil.parseBooleanFromChild(fcpServiceInfoElt, "is-available");
            final String nodeName = BuilderUtil.getChildTextTrim(fcpServiceInfoElt, "node-name");

            final CModeFcpServiceInfo fcpServiceInfoInfo = cluster.createFcpServiceInfo(vServerString);
            fcpServiceInfoInfo.setAvailable(isAvailable);
            fcpServiceInfoInfo.setFcpNodeName(nodeName);
            cluster.getFcpServiceInfos().add(fcpServiceInfoInfo);
        }
    }
}
