package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeIGroup;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPINames;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.IGroup;
import org.apache.log4j.Logger;
import org.jdom.Element;

import java.util.List;

import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIUtils.children;
import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIUtils.elem;

public class IGroupBuilder extends IterBuilderBase {

    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger logger = Logger.getLogger(IGroupBuilder.class);

    private static final String ZAPI_NAME_INITIATOR_GROUP_INFO = "initiator-group-info";

    protected IGroupBuilder() {
        super("igroup-get-iter", ZAPI_NAME_INITIATOR_GROUP_INFO, getDefaultMaxRecords(),
                elem(ZAPINames.DESIRED_ATTRIBUTES,
                        elem(ZAPI_NAME_INITIATOR_GROUP_INFO, children(
                                "initiator-group-alua-enabled", "initiator-group-uuid", "initiator-group-name", "initiator-group-os-type", "initiator-group-type", "initiator-group-use-partner", "initiator-group-vsa-enabled", "vserver",
                                elem("initiators",
                                        elem("initiator-info",
                                                elem("initiator-name")))))));
    }

    @Override
    protected void buildIterInstance(final CModeCluster cluster, final Element igroupInfoElt) {
        final String igroupUUID = BuilderUtil.getChildTextTrim(igroupInfoElt, "initiator-group-uuid");
        final String igroupName = BuilderUtil.getChildTextTrim(igroupInfoElt, "initiator-group-name");
        final EnumWrapper<IGroup.Type> iGroupType = BuilderUtil.getEnumWrapperByValue(IGroup.Type.class, igroupInfoElt, "initiator-group-type", IGroup.Type.NOT_MAPPED);
        final String iGroupOSType = BuilderUtil.getChildTextTrim(igroupInfoElt, "initiator-group-os-type");
        final Boolean igroupAluaEnabled = BuilderUtil.parseBooleanFromChild(igroupInfoElt, "initiator-group-alua-enabled");
        final Boolean igroupVsaEnabled = BuilderUtil.parseBooleanFromChild(igroupInfoElt, "initiator-group-vsa-enabled");
        final Boolean igroupUsePartner = BuilderUtil.parseBooleanFromChild(igroupInfoElt, "initiator-group-use-partner");
        final String vServerName = BuilderUtil.getChildTextTrim(igroupInfoElt, "vserver");
        final Element initiatorsElt = igroupInfoElt.getChild("initiators");
        final List<String> initiatorNames = BuilderUtil.getListFromChildren(initiatorsElt, "initiator-info", "initiator-name");

        final CModeIGroup iGroup = cluster.createCModeIGroup(igroupUUID);
        iGroup.setIGroupName(igroupName);
        iGroup.setType(iGroupType);
        iGroup.setOsType(iGroupOSType);
        iGroup.setVServerName(vServerName);
        iGroup.setAluaEnabled(igroupAluaEnabled);
        iGroup.setVsaEnabled(igroupVsaEnabled);
        iGroup.setUsePartner(igroupUsePartner);
        iGroup.setInitiatorNames(initiatorNames);
        cluster.getIGroups().add(iGroup);
    }
}
