package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeLicenseV2;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.framework.datasource.PackageContext;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.License;
import netapp.manage.NaElement;
import org.apache.log4j.Logger;
import org.jdom.Element;

import java.util.List;

import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIUtils.children;
import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIUtils.elem;

/**
 * Builds instances of {@link com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeLicenseV2}
 * from <a href="http://web.netapp.com/engineering/design-depot/appliance-mgmt/zephyr/zapidoc.cache/DOT/RsierranevadaN/web_ontap_cluster/license-v2/index.html#license-v2-list-info">license-v2-list-info</a>.
 */
public class LicenseV2Builder extends BuilderBase<CModeCluster> {

    private static final Logger logger = Logger.getLogger(LicenseV2Builder.class);
    private static final String ZAPI_NAME_LICENSE_V2_INFO = "license-v2-info";
    private static final String ZAPI_NAME_LICENSES = "licenses";

    @SuppressWarnings({"unchecked"})
    @Override
    public void buildModel(final CModeCluster cluster, final ZAPIConnection zapiConnection) throws DataSourceErrorException {
        assert !cluster.isPreSierraNevada();
        final NaElement request = elem("license-v2-list-info");
        // Manually record our desired attributes for logging purposes
        final NaElement attributes = elem("attributes",
                elem(ZAPI_NAME_LICENSES,
                    elem(ZAPI_NAME_LICENSE_V2_INFO, children(
                            "description", "expiration-time", "legacy", "owner", "package", "serial-number", "type"))));
        final Element licenseInfoResultsElem = zapiConnection.invokeElem(request, attributes).getElement();

        final Element licensesElem = licenseInfoResultsElem.getChild(ZAPI_NAME_LICENSES);
        if (licensesElem == null) {
            return;
        }

        final List<Element> licenseElems = licensesElem.getChildren(ZAPI_NAME_LICENSE_V2_INFO);
        for (final Element licenseElem : licenseElems) {
            final String description = BuilderUtil.getChildTextTrim(licenseElem, "description");
            final Integer expirationTime = BuilderUtil.parseIntFromChild(licenseElem, "expiration-time");
            final Boolean legacy = BuilderUtil.parseBooleanFromChild(licenseElem, "legacy");
            final String owner = BuilderUtil.getChildTextTrim(licenseElem, "owner");
            final EnumWrapper<CModeLicenseV2.LicensedPackage> licensedPackage = BuilderUtil.getEnumWrapperByValue(
                    CModeLicenseV2.LicensedPackage.class, licenseElem, "package", CModeLicenseV2.LicensedPackage.NOT_MAPPED);
            final String serialNumber = BuilderUtil.getChildTextTrim(licenseElem, "serial-number");
            if (serialNumber == null || serialNumber.isEmpty()) {
                logger.warn(PackageContext.getLogPrefix() + " - Skipping invalid license " + licensedPackage.getRawValue()
                        + " " + description);
                continue;
            }
            final EnumWrapper<License.LicenseType> type = BuilderUtil.getEnumWrapperByValue(
                    License.LicenseType.class, licenseElem, "type", License.LicenseType.NOT_MAPPED);

            final CModeLicenseV2 licenseV2 = cluster.createLicenseV2(licensedPackage, serialNumber);
            licenseV2.setDescription(description);
            licenseV2.setExpirationTime(expirationTime);
            licenseV2.setLegacy(legacy);
            licenseV2.setOwner(owner);
            licenseV2.setType(type);
            cluster.getLicenses().add(licenseV2);
        }
    }
}
