package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeMetroClusterOperation;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.MetroClusterOperation;
import org.apache.commons.lang3.StringUtils;
import org.jdom.Element;
import java.util.LinkedHashMap;
import java.util.List;

public class MetroClusterOperationBuilder extends IterBuilderBase {    
    public MetroClusterOperationBuilder() {
        super("metrocluster-operation-get-iter", "metrocluster-operation-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"metrocluster-operation-info",}, new String[]{"end-time", "job-id", "node-name", "operation", "operation-uuid", "start-time", "state"});
            put(new String[]{"metrocluster-operation-info", "error-list",}, new String[]{"string",});
        }});
    }
    
    @Override
    protected void buildIterInstance(final CModeCluster cModeCluster, final Element metroClusterOperationInfoElt) {
        final String metroClusterOperationUUID = BuilderUtil.getChildTextTrim(metroClusterOperationInfoElt,
                "operation-uuid");
        final Long endTime = BuilderUtil.parseLongFromChild(metroClusterOperationInfoElt, "end-time");
        final List<String> errorList = BuilderUtil.getListFromGrandChildren(metroClusterOperationInfoElt, "error-list",
                "string");
        final String delimitedErrors = StringUtils.join(errorList, "##");
        final Long jobId = BuilderUtil.parseLongFromChild(metroClusterOperationInfoElt, "job-id");
        final String node = BuilderUtil.getChildTextTrim(metroClusterOperationInfoElt, "node-name");
        final EnumWrapper<MetroClusterOperation.Operation> operation = BuilderUtil.getEnumWrapperByValue(
                MetroClusterOperation.Operation.class, metroClusterOperationInfoElt, "operation", MetroClusterOperation.Operation.NOT_MAPPED);
        final Long startTime = BuilderUtil.parseLongFromChild(metroClusterOperationInfoElt, "start-time");
        final EnumWrapper<MetroClusterOperation.State> state = BuilderUtil.getEnumWrapperByValue(
                MetroClusterOperation.State.class, metroClusterOperationInfoElt, "state", MetroClusterOperation.State.NOT_MAPPED);
        
        final CModeMetroClusterOperation metroClusterOperation = cModeCluster.createMetroClusterOperation(metroClusterOperationUUID);
        metroClusterOperation.setEndTime(endTime);
        metroClusterOperation.setErrors(delimitedErrors);
        metroClusterOperation.setJobId(jobId);
        metroClusterOperation.setNode(node);
        metroClusterOperation.setOperation(operation);
        metroClusterOperation.setStartTime(startTime);
        metroClusterOperation.setState(state);
        
        cModeCluster.getMetroClusterOperations().add(metroClusterOperation);
    }
}
