package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeNetInterface;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.NetworkPort;
import org.jdom.Element;

import java.util.LinkedHashMap;

public class NetPortBuilder extends IterBuilderBase {

    public NetPortBuilder() {
        super("net-port-get-iter", "net-port-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"net-port-info",}, new String[]{"port", "port-type", "role", "link-status", "mac-address", "node", "operational-speed", "administrative-speed", "mtu", "vlan-id", "vlan-node", "vlan-port", "ifgrp-node", "ifgrp-port", "operational-duplex", "ipspace", "broadcast-domain", });
        }});
    }

    @Override
    protected void buildIterInstance(final CModeCluster cluster, final Element netPortInfElt) {
        final String portName = BuilderUtil.getChildTextTrim(netPortInfElt, "port");
        final EnumWrapper<NetworkPort.PortType> portType = BuilderUtil.getEnumWrapperByValue(NetworkPort.PortType.class, netPortInfElt, "port-type", NetworkPort.PortType.NOT_MAPPED);
        final EnumWrapper<NetworkPort.Role> portRole = BuilderUtil.getEnumWrapperByValue(NetworkPort.Role.class, netPortInfElt, "role", NetworkPort.Role.NOT_MAPPED);
        final EnumWrapper<NetworkPort.LinkStatus> linkStatus = BuilderUtil.getEnumWrapperByValue(NetworkPort.LinkStatus.class, netPortInfElt, "link-status", NetworkPort.LinkStatus.NOT_MAPPED);
        final String macAddress = BuilderUtil.getChildTextTrim(netPortInfElt, "mac-address");
        final String nodeName = BuilderUtil.getChildTextTrim(netPortInfElt, "node");
        final EnumWrapper<NetworkPort.Speed> operSpeed = getNetPortSpeed(netPortInfElt, "operational-speed");
        final EnumWrapper<NetworkPort.Speed> adminSpeed = getNetPortSpeed(netPortInfElt, "administrative-speed");
        final Long mtu = BuilderUtil.parseLongFromChild(netPortInfElt, "mtu");
        final Long vlanID = BuilderUtil.parseLongFromChild(netPortInfElt, "vlan-id");
        final String vlanNode = BuilderUtil.getChildTextTrim(netPortInfElt, "vlan-node");
        final String vlanPort = BuilderUtil.getChildTextTrim(netPortInfElt, "vlan-port");
        final String ifGroupNode = BuilderUtil.getChildTextTrim(netPortInfElt, "ifgrp-node");
        final String ifGroupPort = BuilderUtil.getChildTextTrim(netPortInfElt, "ifgrp-port");
        final String broadcastDomainName = BuilderUtil.getChildTextTrim(netPortInfElt, "broadcast-domain");
        final String ipSpaceName = BuilderUtil.getChildTextTrim(netPortInfElt, "ipspace");

        final EnumWrapper<NetworkPort.OperationalDuplex> operationalDuplex = BuilderUtil.getEnumWrapperByValue(NetworkPort.OperationalDuplex.class, netPortInfElt, "operational-duplex", NetworkPort.OperationalDuplex.NOT_MAPPED);


        final CModeNetInterface cModePort = cluster.createNetInterface(nodeName, portName);
        cModePort.setPortType(portType);
        cModePort.setPortRole(portRole);
        cModePort.setLinkStatus(linkStatus);
        cModePort.setMacAddress(macAddress);
        cModePort.setOperSpeed(operSpeed);
        cModePort.setAdminSpeed(adminSpeed);
        cModePort.setMtu(mtu);
        cModePort.setVlanID(vlanID);
        cModePort.setVlanNode(vlanNode);
        cModePort.setVlanPort(vlanPort);
        cModePort.setIfGroupNode(ifGroupNode);
        cModePort.setIfGroupPort(ifGroupPort);
        cModePort.setOperationalDuplex(operationalDuplex);
        cModePort.setNetworkIpSpaceName(ipSpaceName);
        cModePort.setNetworkPortBroadcastDomainName(broadcastDomainName);
        cluster.getNetInterfaces().add(cModePort);
    }

    private EnumWrapper<NetworkPort.Speed> getNetPortSpeed(final Element elt, final String targetChildName) {
        final String speedString = BuilderUtil.getChildTextTrim(elt, targetChildName);
        final Integer speedInt = BuilderUtil.parseIntFromString(speedString);
        final NetworkPort.Speed speedEnum = (speedInt == null) ? BuilderUtil.getEnumByValue(NetworkPort.Speed.class, speedString, NetworkPort.Speed.NOT_MAPPED) : getNetPortSpeedEnum(speedInt);
        return new EnumWrapper<NetworkPort.Speed>(speedString, speedEnum);
    }

    private NetworkPort.Speed getNetPortSpeedEnum(final Integer portSpeedInt) {
        final NetworkPort.Speed portSpeed;
        switch (portSpeedInt) {
            case 10:
                portSpeed = NetworkPort.Speed.MEGABIT_10;
                break;
            case 100:
                portSpeed = NetworkPort.Speed.MEGABIT_100;
                break;
            case 1000:
                portSpeed = NetworkPort.Speed.GIGABIT_1;
                break;
            case 10000:
                portSpeed = NetworkPort.Speed.GIGABIT_10;
                break;
            default:
                portSpeed = NetworkPort.Speed.NOT_MAPPED;
                break;
        }
        return portSpeed;
    }

}
