package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;


import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeNetworkIPSpace;
import org.jdom.Element;
import java.util.LinkedHashMap;

/**
 * Builder class for generating the net-ipspaces-get-iter ZAPI request, reading the ZAPI response and creating the
 * CModeNetIPSpace objects.
 */
public class NetworkIpSpaceBuilder extends IterBuilderBase {

    /**
     * ZAPI Name : net-ipspaces-get-iter
     * Element in the
     */
    protected NetworkIpSpaceBuilder() {
        super("net-ipspaces-get-iter", "net-ipspaces-info", getDefaultMaxRecords(),
                new LinkedHashMap<String[], String[]>() {
                    {
                        put(new String[]{"net-ipspaces-info",}, new String[]{"id", "ipspace", "uuid",});
                    }
                });
    }


    /**
     * This method will be called for each net-ipspaces-info in the ZAPI response. Here we retrieve IP Space related
     * attributes from the ZAPI response and create a CModeNetIPSpace.
     */
    @Override
    protected void buildIterInstance(CModeCluster cluster, Element ipSpacesInfo) {
        final Long id = BuilderUtil.parseLongFromChild(ipSpacesInfo, "id");
        final String name = BuilderUtil.getChildTextTrim(ipSpacesInfo, "ipspace");
        String uuid = BuilderUtil.getChildTextTrim(ipSpacesInfo, "uuid");
        
        if (uuid == null) {
            uuid = CModeNetworkIPSpace.CREATED_NETWORKIPSPACE_UUID_PREFIX + "." + name;
        }
        
        final CModeNetworkIPSpace ipSpace = cluster.createNetworkIPSpace(uuid);
        ipSpace.setId(id);
        ipSpace.setIPSpaceName(name);
        cluster.getNetworkIPSpaces().add(ipSpace);
    }

}
