package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.*;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPINames;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.framework.datasource.PackageContext;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.FlashDevice;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Node;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.ServiceProcessor;
import org.apache.log4j.Logger;
import org.jdom.Element;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIUtils.children;
import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIUtils.elem;

public class NodeBuilder extends IterBuilderBase {

    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger logger = Logger.getLogger(VServerBuilder.class);

    public NodeBuilder() {
        super("system-node-get-iter", "node-details-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"node-details-info",}, new String[]{"node-uuid", "node", "node-vendor", "node-model", "node-nvram-id", "node-serial-number",
                    "node-uptime", "node-location", "env-over-temperature", "env-failed-fan-message", "env-failed-power-supply-message", "cpu-firmware-release",
                    "env-over-temperature", "env-failed-fan-count", "env-failed-power-supply-count", "nvram-battery-status", "node-owner", "is-node-healthy",
                    "product-version", "is-epsilon-node", "is-all-flash-optimized"});
        }});
    }

    public void buildModel(final CModeCluster cluster, final ZAPIConnection zapiConnection) throws DataSourceErrorException {
        // Node ZAPIs are not iterators and instead require Node name parameter.
        // This class discovers Nodes (including names) as well as additional Node details
        // from ZAPIs requiring Node name arguments.
        super.buildModel(cluster, zapiConnection);                              // calls system-node-get-iter
        new NodeDetailsInfoBuilder().buildModel(cluster, zapiConnection);       // calls system-get-node-info-iter
        new NodeFailoverBuilder().buildModel(cluster, zapiConnection);          // calls cf-get-iter
        new NodeFailoverStatusBuilder().buildModel(cluster, zapiConnection);    // calls cf-status
        new StorageShelfBuilder().buildModel(cluster, zapiConnection);          // calls storage-shelf-list-info
        new StorageShelfEnvBuilder().buildModel(cluster, zapiConnection);       // calls storage-shelf-environment-list-info
        new ServiceProcessorBuilder().buildModel(cluster, zapiConnection);      // calls service-processor-get
        new FlashDeviceBuilder().buildModel(cluster, zapiConnection);           // calls flash-device-get-iter
        new MetroclusterNodeInfoBuilder(cluster).buildModel(cluster, zapiConnection);  // calls metrocluster-node-get-iter
    }

    protected void buildIterInstance(final CModeCluster cluster, final Element nodeDetalsInfoElt) {
        final String uuid = BuilderUtil.getChildTextTrim(nodeDetalsInfoElt, "node-uuid");
        final String nodeName = BuilderUtil.getChildTextTrim(nodeDetalsInfoElt, "node");
        final String vendorID = BuilderUtil.getChildTextTrim(nodeDetalsInfoElt, "node-vendor");
        final String model = BuilderUtil.getChildTextTrim(nodeDetalsInfoElt, "node-model");
        final String serialNum = BuilderUtil.getChildTextTrim(nodeDetalsInfoElt, "node-serial-number");
        final Long uptime = BuilderUtil.parseLongFromChild(nodeDetalsInfoElt, "node-uptime");
        final String nodeLocation = BuilderUtil.getChildTextTrim(nodeDetalsInfoElt, "node-location");
        final Boolean envOverTemperature = BuilderUtil.parseBooleanFromChild(nodeDetalsInfoElt, "env-over-temperature");
        final String envFailedFanMessage = BuilderUtil.getChildTextTrim(nodeDetalsInfoElt, "env-failed-fan-message");
        final String envFailedPowerSupplyMessage = BuilderUtil.getChildTextTrim(nodeDetalsInfoElt, "env-failed-power-supply-message");
        final EnumWrapper<Node.BatteryState> nvramBatteryStatus = BuilderUtil.getEnumWrapperByValue(Node.BatteryState.class, nodeDetalsInfoElt, "nvram-battery-status", Node.BatteryState.NOT_MAPPED);
        final String owner = BuilderUtil.getChildTextTrim(nodeDetalsInfoElt, "node-owner");
        final String nvramID = BuilderUtil.getChildTextTrim(nodeDetalsInfoElt, "node-nvram-id");
        final String cpuFirmwareRelease = BuilderUtil.getChildTextTrim(nodeDetalsInfoElt, "cpu-firmware-release");
        final Integer failedFanCount = BuilderUtil.parseIntFromChild(nodeDetalsInfoElt, "env-failed-fan-count");
        final Integer failedPowerSupplyCount = BuilderUtil.parseIntFromChild(nodeDetalsInfoElt, "env-failed-power-supply-count");
        final Boolean isNodeHealthy = BuilderUtil.parseBooleanFromChild(nodeDetalsInfoElt, "is-node-healthy");
        final String productVersion = BuilderUtil.getChildTextTrim(nodeDetalsInfoElt, "product-version");
        final Boolean isEpsilonNode = BuilderUtil.parseBooleanFromChild(nodeDetalsInfoElt, "is-epsilon-node");
        final Boolean isAllFlashOptimized = BuilderUtil.parseBooleanFromChild(nodeDetalsInfoElt, "is-all-flash-optimized");

        final CModeNodeDetail cModeNodeDetail = cluster.createNodeDetail(uuid);
        cModeNodeDetail.setNodeNodeName(nodeName);
        cModeNodeDetail.setNodeVendor(vendorID);
        cModeNodeDetail.setNodeModel(model);
        cModeNodeDetail.setNodeSerialNum(serialNum);
        cModeNodeDetail.setNodeUptime(uptime);
        cModeNodeDetail.setNodeLocation(nodeLocation);
        cModeNodeDetail.setEnvOverTemperature(envOverTemperature);
        cModeNodeDetail.setEnvFailedFanMessage(envFailedFanMessage);
        cModeNodeDetail.setEnvFailedPowerSupplyMessage(envFailedPowerSupplyMessage);
        cModeNodeDetail.setNvramBatteryStatus(nvramBatteryStatus);
        cModeNodeDetail.setNvramID(nvramID);
        cModeNodeDetail.setCpuFirmwareRelease(cpuFirmwareRelease);
        cModeNodeDetail.setFailedFanCount(failedFanCount);
        cModeNodeDetail.setFailedPowerSupplyCount(failedPowerSupplyCount);
        cModeNodeDetail.setNodeOwner(owner);
        cModeNodeDetail.setHealthy(isNodeHealthy);
        cModeNodeDetail.setProductVersion(productVersion);
        cModeNodeDetail.setIsEpsilonNode(isEpsilonNode);
        cModeNodeDetail.setAllFlashOptimized(isAllFlashOptimized);


        cluster.getNodeDetails().add(cModeNodeDetail);
    }

    // Get some additional node info
    private static final class NodeDetailsInfoBuilder extends IterBuilderBase {
        private NodeDetailsInfoBuilder() {
            super("system-get-node-info-iter", "system-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
                put(new String[]{"system-info",}, new String[]{"system-name", "system-id", "partner-system-name", "partner-system-id", "number-of-processors", "cpu-processor-type", "cpu-processor-id", "memory-size", "prod-type",});
            }});
        }

        protected void buildIterInstance(final CModeCluster cluster, final Element systemInfoElt) {
            final String systemName = BuilderUtil.getChildTextTrim(systemInfoElt, "system-name");
            final String systemID = BuilderUtil.getChildTextTrim(systemInfoElt, "system-id");
            final String partnerSystemName = BuilderUtil.getChildTextTrim(systemInfoElt, "partner-system-name");
            final String partnerSystemID = BuilderUtil.getChildTextTrim(systemInfoElt, "partner-system-id");
            final Integer numCPUs = BuilderUtil.parseIntFromChild(systemInfoElt, "number-of-processors");
            final String cpuTypeString = BuilderUtil.getChildTextTrim(systemInfoElt, "cpu-processor-type");
            final String cpuIDString = BuilderUtil.getChildTextTrim(systemInfoElt, "cpu-processor-id");
            final Integer memSize = BuilderUtil.parseIntFromChild(systemInfoElt, "memory-size");
            final String productType = BuilderUtil.getChildTextTrim(systemInfoElt, "prod-type");
            final CModeNode cModeNode = cluster.createNode(systemName);
            cModeNode.setCpuTypeString(cpuTypeString);
            cModeNode.setCpuIDString(cpuIDString);
            cModeNode.setNumCPUs(numCPUs);
            cModeNode.setMemSize(memSize);
            cModeNode.setProductType(productType);
            cModeNode.setSystemID(systemID);
            cModeNode.setPartnerName(partnerSystemName);
            cModeNode.setPartnerID(partnerSystemID);
            cluster.getCModeNodes().add(cModeNode);
        }
    }

    private static final class NodeFailoverBuilder extends IterBuilderBase {
        public NodeFailoverBuilder() {
            super("cf-get-iter", "storage-failover-info", 20, new LinkedHashMap<String[], String[]>() {{
                put(new String[]{"storage-failover-info",}, new String[]{"node-name", "partner-name", "partner-aggregates", "state",});
                put(new String[]{"storage-failover-info", "sfo-node-info", "node-related-info"}, new String[]{"node", "partner-name", "state-description",});

            }});
        }

        @Override
        protected void buildIterInstance(final CModeCluster cluster, final Element failoverInfoElt) {
            final Element sfoNodeInfoElt = failoverInfoElt.getChild("sfo-node-info");
            if (sfoNodeInfoElt != null) {
                // Sierra Nevada style
                final Element nodeRelatedInfoElt = sfoNodeInfoElt.getChild("node-related-info");
                if (nodeRelatedInfoElt != null) {
                    final String nodeName = BuilderUtil.getChildTextTrim(nodeRelatedInfoElt, "node");
                    final String partnerNodeName = BuilderUtil.getChildTextTrim(nodeRelatedInfoElt, "partner-name");
                    final String state = BuilderUtil.getChildTextTrim(nodeRelatedInfoElt, "state-description");
                    final CModeNodeFailover nodeFailover = cluster.createNodeFailover(nodeName);
                    nodeFailover.setPartnerName(partnerNodeName);
                    nodeFailover.setNodeStateString(state);
                    cluster.getNodeFailovers().add(nodeFailover);
                }
            } else {
                // Rolling Rock style
                final String nodeName = BuilderUtil.getChildTextTrim(failoverInfoElt, "node");
                final String partnerNodeName = BuilderUtil.getChildTextTrim(failoverInfoElt, "partner-name");
                final String partnerAggregates = BuilderUtil.getChildTextTrim(failoverInfoElt, "partner-aggregates");
                final String state = BuilderUtil.getChildTextTrim(failoverInfoElt, "state");
                if (nodeName != null) {
                    final CModeNodeFailover nodeFailover = cluster.createNodeFailover(nodeName);
                    nodeFailover.setPartnerName(partnerNodeName);
                    nodeFailover.setPartnerAggregate(partnerAggregates);
                    nodeFailover.setNodeStateString(state);
                    cluster.getNodeFailovers().add(nodeFailover);
                }
            }
        }
    }

    private static final class NodeFailoverStatusBuilder extends BuilderBase<CModeCluster> {
        public void buildModel(final CModeCluster cluster, final ZAPIConnection zapiConnection) throws DataSourceErrorException {
            for (final CModeNode cModeNode : cluster.getCModeNodes().values()) {
                final String nodeName = cModeNode.getKey().getName();
                final Map<String, String> args = new LinkedHashMap<String, String>() {{
                    put("node", nodeName);
                }};
                final ZAPIConnection.ZapiResponse response = zapiConnection.invokeCommand("cf-status", args);
                final Element resultsElt = response.getElement();
                final Boolean isEnabled = BuilderUtil.parseBooleanFromChild(resultsElt, "is-enabled");
                final String failoverState = BuilderUtil.getChildTextTrim(resultsElt, "state");
                final String partnerName = BuilderUtil.getChildTextTrim(resultsElt, "partner-name");
                final Boolean takeoverPossible = BuilderUtil.parseBooleanFromChild(resultsElt, "takeover-possible");
                final String partnerFirmwareState = BuilderUtil.getChildTextTrim(resultsElt, "partner-firmware-state");
                final String localFirmwareState = BuilderUtil.getChildTextTrim(resultsElt, "local-firmware-state");
                final Boolean isInterconnectUp = BuilderUtil.parseBooleanFromChild(resultsElt, "is-interconnect-up");
                final String interconnectLinks = BuilderUtil.getChildTextTrim(resultsElt, "interconnect-links");
                final String givebackState = BuilderUtil.getChildTextTrim(resultsElt, "giveback-state");
                final String interconnectName = BuilderUtil.getChildTextTrim(resultsElt, "interconnect-name");
                final String interconnectType = BuilderUtil.getChildTextTrim(resultsElt, "interconnect-type");
                final String currentMode = BuilderUtil.getChildTextTrim(resultsElt, "current-mode");
                final String takeoverFailureReason = BuilderUtil.getChildTextTrim(resultsElt, "takeover-failure-reason");

                final List<String> byPartnerNotPossibleReasons;
                final Element takeoverByPartnerNotPossibleReasonElt = resultsElt.getChild("takeover-by-partner-not-possible-reason");
                if (takeoverByPartnerNotPossibleReasonElt != null) {
                    byPartnerNotPossibleReasons = BuilderUtil.getListFromChildren(takeoverByPartnerNotPossibleReasonElt, "takeover-info", "reason");
                } else {
                    byPartnerNotPossibleReasons = new ArrayList<String>(0);
                }


                final List<String> ofPartnerNotPossibleReasons;
                final Element takeoverOfPartnerNotPossibleReasonElt = resultsElt.getChild("takeover-of-partner-not-possible-reason");
                if (takeoverOfPartnerNotPossibleReasonElt != null) {
                    ofPartnerNotPossibleReasons = BuilderUtil.getListFromChildren(takeoverOfPartnerNotPossibleReasonElt, "takeover-info", "reason");
                } else {
                    ofPartnerNotPossibleReasons = new ArrayList<String>(0);
                }

                cModeNode.setFailoverEnabled(isEnabled);
                cModeNode.setFailoverState(failoverState);
                cModeNode.setPartnerName(partnerName);
                cModeNode.setTakeoverPossible(takeoverPossible);
                cModeNode.setPartnerFirmwareState(partnerFirmwareState);
                cModeNode.setLocalFirmwareState(localFirmwareState);
                cModeNode.setInterconnectUp(isInterconnectUp);
                cModeNode.setInterconnectLinks(interconnectLinks);
                cModeNode.setGivebackState(givebackState);
                cModeNode.setInterconnectName(interconnectName);
                cModeNode.setInterconnectType(interconnectType);
                cModeNode.setCurrentMode(currentMode);
                cModeNode.setTakeoverByPartnerNotPossibleReasons(byPartnerNotPossibleReasons);
                cModeNode.setTakeoverOfPartnerNotPossibleReasons(ofPartnerNotPossibleReasons);
                cModeNode.setTakeoverFailureReason(takeoverFailureReason);

                zapiConnection.recordRequestedAttribute("cf-status.is-enabled");
                zapiConnection.recordRequestedAttribute("cf-status.state");
                zapiConnection.recordRequestedAttribute("cf-status.partner-name");
                zapiConnection.recordRequestedAttribute("cf-status.takeover-possible");
                zapiConnection.recordRequestedAttribute("cf-status.partner-firmware-state");
                zapiConnection.recordRequestedAttribute("cf-status.local-firmware-state");
                zapiConnection.recordRequestedAttribute("cf-status.is-interconnect-up");
                zapiConnection.recordRequestedAttribute("cf-status.interconnect-links");
                zapiConnection.recordRequestedAttribute("cf-status.giveback-state");
                zapiConnection.recordRequestedAttribute("cf-status.interconnect-name");
                zapiConnection.recordRequestedAttribute("cf-status.interconnect-type");
                zapiConnection.recordRequestedAttribute("cf-status.current-mode");
                zapiConnection.recordRequestedAttribute("cf-status.takeover-failure-reason");
                zapiConnection.recordRequestedAttribute("cf-status.takeover-of-partner-not-possible-reason.takeover-info.reason");
                zapiConnection.recordRequestedAttribute("cf-status.takeover-by-partner-not-possible-reason.takeover-info.reason");
            }
        }
    }

    private static final class ServiceProcessorBuilder extends BuilderBase<CModeCluster> {
        public void buildModel(final CModeCluster cluster, final ZAPIConnection zapiConnection) throws DataSourceErrorException {
            for (final CModeNode cModeNode : cluster.getCModeNodes().values()) {
                final String nodeName = cModeNode.getKey().getName();
                final Map<String, String> args = new LinkedHashMap<String, String>() {{
                    put("node", nodeName);
                }};
                final ZAPIConnection.ZapiResponse response = zapiConnection.invokeCommand("service-processor-get", args);
                final Element responseElt = response.getElement();
                if (responseElt == null) {
                    continue;
                }
                final Element attributesElt = responseElt.getChild("attributes");
                if (attributesElt == null) {
                    continue;
                }
                final Element serviceProcessorInfoElt = attributesElt.getChild("service-processor-info");
                if (serviceProcessorInfoElt == null) {
                    continue;
                }
                final String deviceRevision = BuilderUtil.getChildTextTrim(serviceProcessorInfoElt, "device-revision");
                final String firmwareVersion = BuilderUtil.getChildTextTrim(serviceProcessorInfoElt, "firmware-version");
                final List<String> ipAddresses = BuilderUtil.getListFromGrandChildren(serviceProcessorInfoElt, "ip-address", "ip-address");
                final String ipmiVersion = BuilderUtil.getChildTextTrim(serviceProcessorInfoElt, "ipmi-version");
                final Boolean isAutoUpdateEnabled = BuilderUtil.parseBooleanFromChild(serviceProcessorInfoElt, "is-autoupdate-enabled");
                final Boolean isIpConfigured = BuilderUtil.parseBooleanFromChild(serviceProcessorInfoElt, "is-ip-configured");
                final String macAddress = BuilderUtil.getChildTextTrim(serviceProcessorInfoElt, "mac-address");
                final String returnedNodeName = BuilderUtil.getChildTextTrim(serviceProcessorInfoElt, "node");
                final String partNum = BuilderUtil.getChildTextTrim(serviceProcessorInfoElt, "part-num");
                final String serialNum = BuilderUtil.getChildTextTrim(serviceProcessorInfoElt, "serial-num");
                final EnumWrapper<ServiceProcessor.ServiceProcessorStatus> status = BuilderUtil.getEnumWrapperByValue(
                        ServiceProcessor.ServiceProcessorStatus.class, serviceProcessorInfoElt, "status", ServiceProcessor.ServiceProcessorStatus.NOT_MAPPED);
                final EnumWrapper<ServiceProcessor.ServiceProcessorType> type = BuilderUtil.getEnumWrapperByValue(
                        ServiceProcessor.ServiceProcessorType.class, serviceProcessorInfoElt, "type", ServiceProcessor.ServiceProcessorType.NOT_MAPPED);

                if (macAddress == null || macAddress.isEmpty()) {
                    logger.info(PackageContext.getLogPrefix() + " - Skipping service processor with no mac address " + ipAddresses);
                    continue;
                }
                final CModeServiceProcessor serviceProcessor = cModeNode.createServiceProcessor(macAddress);
                serviceProcessor.setDeviceRevision(deviceRevision);
                serviceProcessor.setFirmwareVersion(firmwareVersion);
                serviceProcessor.addIpAddresses(ipAddresses);
                serviceProcessor.setImpiVersion(ipmiVersion);
                serviceProcessor.setAutoUpdateEnabled(isAutoUpdateEnabled);
                serviceProcessor.setIpConfigured(isIpConfigured);
                serviceProcessor.setReturnedNodeName(returnedNodeName);
                serviceProcessor.setPartNum(partNum);
                serviceProcessor.setSerialNum(serialNum);
                serviceProcessor.setStatus(status);
                serviceProcessor.setType(type);
                cModeNode.getServiceProcessors().add(serviceProcessor);
            }
        }
    }

    private static final class MetroclusterNodeInfoBuilder extends IterBuilderBase {
        private static final String METROCLUSTER_NODE_INFO =  "metrocluster-node-info";

        private MetroclusterNodeInfoBuilder(final CModeCluster cModeCluster) {
            super("metrocluster-node-get-iter", "metrocluster-node-info", getDefaultMaxRecords(),
                    elem(ZAPINames.DESIRED_ATTRIBUTES,
                            elem(METROCLUSTER_NODE_INFO, children("node-uuid", "dr-partner-uuid",  "dr-partner-cluster-uuid", "dr-operation-state","automatic-uso"
                            ))),
                    elem(ZAPINames.QUERY,
                            elem(METROCLUSTER_NODE_INFO,
                                     elem("node-cluster-uuid", cModeCluster.getKey().getUUID()))
                    ));
        }

        @Override
        protected void buildIterInstance(final CModeCluster cluster, final Element mtNodeInfoElt) {
            final String nodeUUID = BuilderUtil.getChildTextTrim(mtNodeInfoElt, "node-uuid");
            final String drNodeId = BuilderUtil.getChildTextTrim(mtNodeInfoElt, "dr-partner-uuid");
            final String drClusterId = BuilderUtil.getChildTextTrim(mtNodeInfoElt, "dr-partner-cluster-uuid");
            final EnumWrapper<Node.MtDrOperationState> mtDrOperationState;
            mtDrOperationState = BuilderUtil.getEnumWrapperByValue(
                    Node.MtDrOperationState.class, mtNodeInfoElt, "dr-operation-state", Node.MtDrOperationState.NOT_MAPPED);
            final String automaticUsoTxt =  BuilderUtil.getChildText(mtNodeInfoElt, "automatic-uso");

            final CModeNodeMetrocluster nodeMetrocluster = cluster.createNodeMetrocluster(nodeUUID);
            nodeMetrocluster.setMtDrOperationState(mtDrOperationState);
            nodeMetrocluster.setMtDrPartnerNodeUUID(drNodeId);
            nodeMetrocluster.setMtDrPartnerClusterUUID(drClusterId);
            nodeMetrocluster.setMtAutomaticUso(automaticUsoTxt == null ? null : BuilderUtil.parseBooleanFromString(automaticUsoTxt));
            cluster.getNodeMetroclusters().add(nodeMetrocluster);
        }
    }
    /**
     * Calls flash-device-get-iter. Creates flash devices as children of the node they exist on.
     */
    private static final class FlashDeviceBuilder extends IterBuilderBase {
        private FlashDeviceBuilder() {
            super("flash-device-get-iter", "flash-device-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
                put(new String[]{"flash-device-info",}, new String[]{"node", "slot-number", "serial-number", "model-name", "firmware-revision", "capacity", "percent-online", "status", "service-time", "threshold-profile"});
            }});
        }

        @Override
        protected void buildIterInstance(final CModeCluster cluster, final Element clusterNodeInfoElt) {
            final String nodeName = BuilderUtil.getChildTextTrim(clusterNodeInfoElt, "node");
            final CModeNode node = cluster.getCModeNodes().get(new UUIDKey("node", nodeName));
            if (node == null) {
                logger.info(PackageContext.getLogPrefix() + " - Skipping flash device with unknown node " + nodeName);
                return;
            }
            final Integer slotNumber = BuilderUtil.parseIntFromChild(clusterNodeInfoElt, "slot-number");
            final String serialNumber = BuilderUtil.getChildTextTrim(clusterNodeInfoElt, "serial-number");
            final String modelName = BuilderUtil.getChildTextTrim(clusterNodeInfoElt, "model-name");
            final String firmwareRevision = BuilderUtil.getChildTextTrim(clusterNodeInfoElt, "firmware-revision");
            final Integer capacity = BuilderUtil.parseIntFromChild(clusterNodeInfoElt, "capacity");
            final Integer percentOnline = BuilderUtil.parseIntFromChild(clusterNodeInfoElt, "percent-online");
            final EnumWrapper<FlashDevice.Status> status = BuilderUtil.getEnumWrapperByValue(
                    FlashDevice.Status.class, clusterNodeInfoElt, "status", FlashDevice.Status.NOT_MAPPED);
            final Integer serviceTime = BuilderUtil.parseIntFromChild(clusterNodeInfoElt, "service-time");
            final String thresholdProfile = BuilderUtil.getChildTextTrim(clusterNodeInfoElt, "threshold-profile");

            final CModeFlashDevice flashDevice = node.createFlashDevice(serialNumber);
            flashDevice.setSlotNumber(slotNumber);
            flashDevice.setModelName(modelName);
            flashDevice.setFirmwareRevision(firmwareRevision);
            flashDevice.setCapacity(capacity);
            flashDevice.setPercentOnline(percentOnline);
            flashDevice.setStatus(status);
            flashDevice.setServiceTime(serviceTime);
            flashDevice.setThresholdProfile(thresholdProfile);
            node.getFlashDevices().add(flashDevice);
        }
    }

}
