package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModePortSet;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPINames;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Portset;
import org.apache.log4j.Logger;
import org.jdom.Element;

import java.util.List;

import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIUtils.children;
import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIUtils.elem;

public class PortSetBuilder extends IterBuilderBase {

    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger logger = Logger.getLogger(PortSetBuilder.class);

    private static final String ZAPI_NAME_PORTSET_INFO = "portset-info";

    public PortSetBuilder() {
        super("portset-get-iter", ZAPI_NAME_PORTSET_INFO, getDefaultMaxRecords(),
                elem(ZAPINames.DESIRED_ATTRIBUTES,
                        elem(ZAPI_NAME_PORTSET_INFO, children(
                                "portset-name", "portset-type", "vserver",
                                elem("initiator-group-info",
                                        elem("initiator-group-name")),
                                elem("portset-port-info",
                                        elem("portset-port-name"))))));
    }

    @Override
    protected void buildIterInstance(final CModeCluster cluster, final Element portsetInfoElt) {
        final String portsetName = BuilderUtil.getChildTextTrimDefaultEmpty(portsetInfoElt, "portset-name");
        final EnumWrapper<Portset.Type> portSetType = BuilderUtil.getEnumWrapperByValue(Portset.Type.class, portsetInfoElt, "portset-type", Portset.Type.NOT_MAPPED);
        final String vServerName = BuilderUtil.getChildTextTrim(portsetInfoElt, "vserver");
        final List<String> initiatorGroupNames = BuilderUtil.getListFromGrandChildren(portsetInfoElt, "initiator-group-info", "initiator-group-name");
        final List<String> portSetPortNames = BuilderUtil.getListFromGrandChildren(portsetInfoElt, "portset-port-info", "portset-port-name");
        final CModePortSet portSet = cluster.createCModePortSet(vServerName, portsetName);
        portSet.setType(portSetType);
        portSet.setInitiatorGroupNames(initiatorGroupNames);
        portSet.setPortSetPortNames(portSetPortNames);
        cluster.getPortSets().add(portSet);
    }
}
