package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeQosPolicyGroup;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.QosPolicyGroup;
import org.apache.log4j.Logger;
import org.jdom.Element;

import java.util.LinkedHashMap;

public class QosPolicyGroupBuilder extends IterBuilderBase {

    private static final Logger logger = Logger.getLogger(QosPolicyGroupBuilder.class);

    public QosPolicyGroupBuilder() {
        super("qos-policy-group-get-iter", "qos-policy-group-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"qos-policy-group-info",},
                new String[]{"max-throughput",
                             "pgid",
                             "policy-group",
                             "policy-group-class",
                             "uuid",
                             "vserver",});
        }});
    }

    @Override
    public void buildModel(final CModeCluster cluster, final ZAPIConnection zapiConnection) throws DataSourceErrorException {
        super.buildModel(cluster, zapiConnection);
    }

    @Override
    protected void buildIterInstance(final CModeCluster cluster, final Element qosPolicyGroupInfoElt) {

        final String maxThroughput = BuilderUtil.getChildTextTrim(qosPolicyGroupInfoElt, "max-throughput");
        final String pgid = BuilderUtil.getChildTextTrim(qosPolicyGroupInfoElt, "pgid");
        final String policyGroupName = BuilderUtil.getChildTextTrim(qosPolicyGroupInfoElt, "policy-group");
        final EnumWrapper<QosPolicyGroup.QosPolicyGroupClassType> policyGroupClass = BuilderUtil.getEnumWrapperByValue(QosPolicyGroup.QosPolicyGroupClassType.class,
                                                                                                                       qosPolicyGroupInfoElt,
                                                                                                                       "policy-group-class",
                                                                                                                       QosPolicyGroup.QosPolicyGroupClassType.NOT_MAPPED);
        final String uuid = BuilderUtil.getChildTextTrim(qosPolicyGroupInfoElt, "uuid");
        final String vServerName = BuilderUtil.getChildTextTrim(qosPolicyGroupInfoElt, "vserver");
        final CModeQosPolicyGroup policyGroup = cluster.createQosPolicyGroup(uuid);
        policyGroup.setVServerName(vServerName);
        policyGroup.setMaxThroughput(maxThroughput);
        policyGroup.setPgid(pgid);
        policyGroup.setPolicyGroupClass(policyGroupClass);
        policyGroup.setPolicyGroupName(policyGroupName);
        cluster.getQosPolicyGroups().add(policyGroup);
    }
}
