package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeQosServiceCenter;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.framework.datasource.PackageContext;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.QosServiceCenter;
import org.apache.log4j.Logger;
import org.jdom.Element;

import java.util.*;
import java.util.Map.Entry;

public class QosServiceCenterBuilder extends PerfInstanceBuilder {
    private static final Logger logger = Logger.getLogger(QosVolumeWorkloadBuilder.class);

    public QosServiceCenterBuilder() {
        super("resource");
    }

    @Override
    public void buildModel(final CModeCluster cluster, final ZAPIConnection zapiConnection) throws DataSourceErrorException {
        super.buildModel(cluster, zapiConnection);
        final Map<String, String> uuidNameMap = getUUIDNameMap();
        final String[] uuids = new String[uuidNameMap.size()];
        final Map<String, String> uuidToParent = getParentResources(Arrays.asList(uuidNameMap.keySet().toArray(uuids)),zapiConnection);
        for (final Entry<String, String> entry : uuidNameMap.entrySet()) {
            final CModeQosServiceCenter qosServiceCenter = cluster.createQosServiceCenter(entry.getKey());
            qosServiceCenter.setName(entry.getValue());
            qosServiceCenter.setType(QosServiceCenter.QosServiceCenterType.RESOURCE.name());
            qosServiceCenter.setParentResourceUUID(uuidToParent.get(entry.getKey()));
            cluster.getQosServiceCenters().add(qosServiceCenter);
        }
    }
    

    /**
     * 
     * @param resourceUuids
     * @param zapiConnection
     * @return
     */
    private Map<String, String> getParentResources(final List<String> resourceUuids, final ZAPIConnection zapiConnection) throws DataSourceErrorException {
        final Map<String, String> args = new LinkedHashMap<String, String>();
        args.put("objectname", "resource");
        
        // The set of counters we want
        final ZAPIConnection.ZapiParamArray zapiCounterNameArray = new ZAPIConnection.ZapiParamArray("counters", "counter",
                                                                                                     Arrays.asList("parent_resource"));

        final ZAPIConnection.ZapiParamArray[] zapiParamArrays = {new ZAPIConnection.ZapiParamArray("instance-uuids",
                                                                                                   "instance-uuid",
                                                                                                   resourceUuids),        // instance uuids
                                                                                                   zapiCounterNameArray  // counter names
        };

        final ZAPIConnection.ZapiResponse zapiResponse = zapiConnection.invokeCommandArrayArgs("perf-object-get-instances",
                                                                                               args, Arrays.asList(zapiParamArrays));
        
        final Map<String, String> uuidToParentResource = new HashMap<String, String>(resourceUuids.size());
        
        final Element perfObjectInstanceListInfoElt = zapiResponse.getElement();
        final Element instancesElt = perfObjectInstanceListInfoElt.getChild("instances");
        if (instancesElt != null) {
            @SuppressWarnings({"unchecked"})
            final List<Element> instanceDataElts = instancesElt.getChildren("instance-data");
            for (final Element instanceDataElt : instanceDataElts) {
                final String instanceUUID = instanceDataElt.getChildTextTrim("uuid");
                final Element countersElt = instanceDataElt.getChild("counters");
                if (countersElt == null) {
                    final String instanceName = instanceDataElt.getChildTextTrim("name");
                    logger.warn(PackageContext.getLogPrefix() + " - perf-object-get-instance " + instanceName
                                + " failed to retrieve any counters");
                    continue;
                }
                @SuppressWarnings({"unchecked"})
                final List<Element> counterDataElts = countersElt.getChildren("counter-data");
                for (final Element counterDataElt : counterDataElts) {
                    final String counterValue = counterDataElt.getChildTextTrim("value");
                    if (counterValue != null) {
                        String parentResource = counterValue;
                        if (parentResource.equalsIgnoreCase("none")) {
                            parentResource = null;
                        }
                        uuidToParentResource.put(instanceUUID, parentResource);
                    }
                }
            }
        } else {
            if (logger.isDebugEnabled()) {
                logger.debug("No zapi response when retriving resource parent");
            }
        }
        
        return uuidToParentResource;
    }
    
}