package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeQuota;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeQuotaUser;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPINames;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.QuotaUser;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.TreeQuota;
import org.jdom.Element;

import java.util.List;

import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIUtils.children;
import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIUtils.elem;

public class QuotaBuilder extends IterBuilderBase {

    protected static final String ZAPI_NAME_QUOTA = "quota";

    public QuotaBuilder() {
        super("quota-report-iter", ZAPI_NAME_QUOTA, getDefaultMaxRecords(),
                elem(ZAPINames.DESIRED_ATTRIBUTES,
                        elem(ZAPI_NAME_QUOTA, children(
                                "vserver", "quota-type", "quota-target", "volume", "tree", "disk-limit", "disk-used", "soft-disk-limit", "threshold", "file-limit", "files-used", "soft-file-limit",
                                elem("quota-users",
                                        elem("quota-user", children(
                                                "quota-user-type", "quota-user-id", "quota-user-name"))))))
                );
    }

    @Override
    protected void buildIterInstance(final CModeCluster cluster, final Element quotaElt) {

        final String vServerName = BuilderUtil.getChildTextTrim(quotaElt, "vserver");
        final TreeQuota.QuotaType quotaType = BuilderUtil.getEnumByValue(TreeQuota.QuotaType.class, quotaElt, "quota-type", TreeQuota.QuotaType.NOT_MAPPED);
        final String quotaTarget = BuilderUtil.getChildTextDefaultEmpty(quotaElt, "quota-target");
        final String volumeName = BuilderUtil.getChildTextTrim(quotaElt, "volume");
        final String qtreeName = BuilderUtil.getChildTextDefaultEmpty(quotaElt, "tree");   // No Trim!  Quota names may have leading spaces
        final String diskLimit = BuilderUtil.getChildTextTrim(quotaElt, "disk-limit");
        final String diskUsed = BuilderUtil.getChildTextTrim(quotaElt, "disk-used");
        final String softDiskLimit = BuilderUtil.getChildTextTrim(quotaElt, "soft-disk-limit");
        final String threshold = BuilderUtil.getChildTextTrim(quotaElt, "threshold");
        final String fileLimit = BuilderUtil.getChildTextTrim(quotaElt, "file-limit");
        final String fileUsed = BuilderUtil.getChildTextTrim(quotaElt, "files-used");
        final String softFileLimit = BuilderUtil.getChildTextTrim(quotaElt, "soft-file-limit");

        // Calculate a hash based on the quota users in order to make the quota key unique
        String quotaUserHashRaw = "";
        final Element quotaUsersElement = quotaElt.getChild("quota-users");
        if (null != quotaUsersElement) {
            @SuppressWarnings({"unchecked"})
            final List<Element> quotaUserElements = quotaUsersElement.getChildren("quota-user");
            for (final Element quotaUserElement : quotaUserElements) {
                final String quotaUserID = BuilderUtil.getChildTextTrim(quotaUserElement, "quota-user-id");
                quotaUserHashRaw += quotaUserID;
            }
        }

        // Create the quota
        final String quotaUserHash = String.valueOf(quotaUserHashRaw.hashCode());
        final CModeQuota cModeQuota = cluster.createQuota(vServerName, volumeName, qtreeName, quotaTarget, quotaUserHash);
        cModeQuota.setQuotaType(quotaType);
        cModeQuota.setDiskLimit(diskLimit);
        cModeQuota.setDiskUsed(diskUsed);
        cModeQuota.setSoftDiskLimit(softDiskLimit);
        cModeQuota.setThreshold(threshold);
        cModeQuota.setFileLimit(fileLimit);
        cModeQuota.setFileUsed(fileUsed);
        cModeQuota.setSoftFileLimit(softFileLimit);


        // add quota users to quota
        if (null != quotaUsersElement) {
            @SuppressWarnings({"unchecked"})
            final List<Element> quotaUserElements = quotaUsersElement.getChildren("quota-user");
            for (final Element quotaUserElement : quotaUserElements) {
                final QuotaUser.QuotaUserType quotaUserType = BuilderUtil.getEnumByValue(QuotaUser.QuotaUserType.class, quotaUserElement, "quota-user-type", QuotaUser.QuotaUserType.NOT_MAPPED);
                final String quotaUserID = BuilderUtil.getChildTextTrim(quotaUserElement, "quota-user-id");
                final String quotaUsername = BuilderUtil.getChildTextTrimDefaultEmpty(quotaUserElement, "quota-user-name");
                final CModeQuotaUser cModeQuotaUser = cModeQuota.createQuotaUser(quotaUserID);
                cModeQuotaUser.setQuotaUserID(quotaUserID);
                cModeQuotaUser.setQuotaUserType(quotaUserType);
                cModeQuotaUser.setQuotaUsername(quotaUsername);
                cModeQuota.getQuotaUsers().add(cModeQuotaUser);
            }
        }

        cluster.getQuotas().add(cModeQuota);
    }
}
