package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeSisPolicy;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.SisPolicy;
import org.jdom.Element;

import java.util.LinkedHashMap;

public class SisPolicyBuilder extends IterBuilderBase {

    public SisPolicyBuilder() {
        super("sis-policy-get-iter", "sis-policy-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"sis-policy-info",}, new String[]{"vserver", "policy-name", "enabled", "comment", "duration", "schedule", "qos-policy",});
        }});
    }

    @Override
    protected void buildIterInstance(final CModeCluster cluster, final Element sysPolicyInfoElt) {
        final String vServerName = BuilderUtil.getChildTextTrim(sysPolicyInfoElt, "vserver");
        final String policyName = BuilderUtil.getChildTextTrim(sysPolicyInfoElt, "policy-name");
        final Boolean enabled = BuilderUtil.parseBooleanFromChild(sysPolicyInfoElt, "enabled");
        final String comment = BuilderUtil.getChildTextTrim(sysPolicyInfoElt, "comment");
        final String duration = BuilderUtil.getChildTextTrim(sysPolicyInfoElt, "duration");  // Long?
        final String schedule = BuilderUtil.getChildTextTrim(sysPolicyInfoElt, "schedule");
        final EnumWrapper<SisPolicy.QosPolicy> qosPolicy = BuilderUtil.getEnumWrapperByValue(SisPolicy.QosPolicy.class, sysPolicyInfoElt, "qos-policy", SisPolicy.QosPolicy.NOT_MAPPED);
        final CModeSisPolicy sisPolicy = cluster.createSisPolicy(vServerName, policyName);
        sisPolicy.setEnabled(enabled);
        sisPolicy.setComment(comment);
        sisPolicy.setDuration(duration);
        sisPolicy.setSchedule(schedule);
        sisPolicy.setQosPolicy(qosPolicy);
        cluster.getSisPolicies().add(sisPolicy);
    }
}
