package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeSnapMirror;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.SnapMirror;
import org.apache.log4j.Logger;
import org.jdom.Element;

import java.util.LinkedHashMap;
import java.util.List;

public class SnapMirrorBuilder extends IterBuilderBase {
    
    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger logger = Logger.getLogger(SnapMirrorBuilder.class);
    public SnapMirrorBuilder() {
        super("snapmirror-get-iter", "snapmirror-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"snapmirror-info"}, new String[]{"current-max-transfer-rate",
                                                              "current-transfer-error",
                                                              "current-transfer-priority",
                                                              "current-transfer-type",
                                                              "destination-cluster",
                                                              "destination-volume",
                                                              "destination-volume-node",
                                                              "destination-vserver",
                                                              "destination-vserver-uuid",
                                                              "destination-location",
                                                              "lag-time",
                                                              "last-transfer-duration",
                                                              "last-transfer-end-timestamp",
                                                              "last-transfer-error",
                                                              "last-transfer-network-compression-ratio",
                                                              "last-transfer-type",
                                                              "last-transfer-size",
                                                              "network-compression-ratio",
                                                              "operation-id",
                                                              "progress-last-updated",
                                                              "relationship-control-plane",
                                                              "relationship-group-type",
                                                              "relationship-id",
                                                              "source-cluster",
                                                              "source-volume",
                                                              "source-vserver",
                                                              "source-vserver-uuid",
                                                              "source-location",
                                                              "mirror-state",
                                                              "exported-snapshot",
                                                              "newest-snapshot",
                                                              "transfer-snapshot",
                                                              "is-constituent",
                                                              "is-healthy",
                                                              "max-transfer-rate",
                                                              "relationship-progress",
                                                              "relationship-status",
                                                              "relationship-type",
                                                              "snapshot-checkpoint",
                                                              "snapshot-progress",
                                                              "tries",
                                                              "unhealthy-reason",
                                                              "policy",
                                                              "schedule", });
            put(new String[]{"snapmirror-info", "last-transfer-error-codes"}, new String[]{"integer",});
        }});
    }

    @Override
    protected void buildIterInstance(final CModeCluster cluster, final Element snapMirrorElt) {

        final Long currentMaxTransferRate = BuilderUtil.parseLongFromChild(snapMirrorElt, "current-max-transfer-rate");
        final String currentTransferError = BuilderUtil.getChildTextTrim(snapMirrorElt, "current-transfer-error");
        final EnumWrapper<SnapMirror.TransferPriorityType> currentTransferPriority = BuilderUtil.getEnumWrapperByValue(SnapMirror.TransferPriorityType.class, snapMirrorElt, "current-transfer-priority", SnapMirror.TransferPriorityType.NOT_MAPPED);
        final EnumWrapper<SnapMirror.TransferType> currentTransferType = BuilderUtil.getEnumWrapperByValue(SnapMirror.TransferType.class, snapMirrorElt, "current-transfer-type", SnapMirror.TransferType.NOT_MAPPED);
        final String destinationCluster = BuilderUtil.getChildTextTrim(snapMirrorElt, "destination-cluster");
        final String  destinationVolume = BuilderUtil.getChildTextTrim(snapMirrorElt, "destination-volume");
        final String destinationVolumeNode = BuilderUtil.getChildTextTrim(snapMirrorElt, "destination-volume-node");
        final String destinationVServer = BuilderUtil.getChildTextTrim(snapMirrorElt, "destination-vserver");
        final String destinationVServerUUID = BuilderUtil.getChildTextTrim(snapMirrorElt, "destination-vserver-uuid");
        final String destinationLocation = BuilderUtil.getChildTextTrim(snapMirrorElt, "destination-location");
        final Long lagTime = BuilderUtil.parseLongFromChild(snapMirrorElt, "lag-time");
        final Long lastTransferDuration = BuilderUtil.parseLongFromChild(snapMirrorElt, "last-transfer-duration");
        final Long lastTransferEndTimeStamp = BuilderUtil.parseLongFromChild(snapMirrorElt, "last-transfer-end-timestamp");
        final String lastTransferError = BuilderUtil.getChildTextTrim(snapMirrorElt, "last-transfer-error");
        final List<String> lastTransferErrorCodesList = BuilderUtil.getListFromGrandChildren(snapMirrorElt, "last-transfer-error-codes", "integer");
        final String lastTransferErrorCodes = BuilderUtil.getListAsCSVString(lastTransferErrorCodesList);
        final EnumWrapper<SnapMirror.TransferType> lastTransferType = BuilderUtil.getEnumWrapperByValue(SnapMirror.TransferType.class, snapMirrorElt, "last-transfer-type", SnapMirror.TransferType.NOT_MAPPED);
        final String lastTransferNetworkCompressionRatio = BuilderUtil.getChildTextTrim(snapMirrorElt, "last-transfer-network-compression-ratio");
        final Long lastTransferSize = BuilderUtil.parseLongFromChild(snapMirrorElt, "last-transfer-size");
        final String operationId = BuilderUtil.getChildTextTrim(snapMirrorElt, "operation-id");
        final Long progressLastUpdated = BuilderUtil.parseLongFromChild(snapMirrorElt, "progress-last-updated");
        final String relationshipControlPlane = BuilderUtil.getChildTextTrim(snapMirrorElt, "relationship-control-plane");
        final String relationshipGroupTypeRaw = BuilderUtil.getChildTextTrim(snapMirrorElt, "relationship-group-type");
        final EnumWrapper<SnapMirror.RelationshipGroupType> relationshipGroupType;
        if(relationshipGroupTypeRaw != null) {
            relationshipGroupType = BuilderUtil.getEnumWrapperByValue(SnapMirror.RelationshipGroupType.class, relationshipGroupTypeRaw, SnapMirror.RelationshipGroupType.NOT_MAPPED);
        } else {
            relationshipGroupType = new EnumWrapper<SnapMirror.RelationshipGroupType>(null, SnapMirror.RelationshipGroupType.NONE);
        }
        final String relationshipID =  BuilderUtil.getChildTextTrim(snapMirrorElt, "relationship-id");
        final String sourceCluster = BuilderUtil.getChildTextTrim(snapMirrorElt, "source-cluster");
        final String sourceVolume = BuilderUtil.getChildTextTrim(snapMirrorElt, "source-volume");
        final String sourceVServerUUID = BuilderUtil.getChildTextTrim(snapMirrorElt, "source-vserver-uuid");
        final String sourceVServer = BuilderUtil.getChildTextTrim(snapMirrorElt, "source-vserver");
        final String sourceLocation = BuilderUtil.getChildTextTrim(snapMirrorElt, "source-location");
        final String mirrorStateRaw = BuilderUtil.getChildTextTrim(snapMirrorElt, "mirror-state");
        final EnumWrapper<SnapMirror.MirrorState> mirrorState = new EnumWrapper<SnapMirror.MirrorState>(mirrorStateRaw, SnapMirror.MirrorState.fromString(mirrorStateRaw));
        final String networkCompressionRatio = BuilderUtil.getChildTextTrim(snapMirrorElt, "network-compression-ratio");
        final String exportedSnapshot = BuilderUtil.getChildTextTrim(snapMirrorElt, "exported-snapshot");
        final String newestSnapshot = BuilderUtil.getChildTextTrim(snapMirrorElt, "newest-snapshot");
        final String transferSnapshot = BuilderUtil.getChildTextTrim(snapMirrorElt, "transfer-snapshot");
        final Boolean isConstituent = BuilderUtil.parseBooleanFromChild(snapMirrorElt, "is-constituent");
        final Boolean isHealthy = BuilderUtil.parseBooleanFromChild(snapMirrorElt, "is-healthy");
        final Long maxTransferRate = BuilderUtil.parseLongFromChild(snapMirrorElt, "max-transfer-rate");
        final Long relationshipProgress = BuilderUtil.parseLongFromChild(snapMirrorElt, "relationship-progress");
        final EnumWrapper<SnapMirror.RelationshipStatusType> relationshipStatus = BuilderUtil.getEnumWrapperByValue(SnapMirror.RelationshipStatusType.class, snapMirrorElt, "relationship-status", SnapMirror.RelationshipStatusType.NOT_MAPPED);
        final EnumWrapper<SnapMirror.RelationshipType> relationshipType = BuilderUtil.getEnumWrapperByValue(SnapMirror.RelationshipType.class, snapMirrorElt, "relationship-type", SnapMirror.RelationshipType.NOT_MAPPED);
        final Long snapshotCheckpoint = BuilderUtil.parseLongFromChild(snapMirrorElt, "snapshot-checkpoint");
        final Long snapshotProgress = BuilderUtil.parseLongFromChild(snapMirrorElt, "snapshot-progress");
        final Long tries = BuilderUtil.parseLongFromChild(snapMirrorElt, "tries");
        final String unhealthyReason = BuilderUtil.getChildTextTrim(snapMirrorElt, "unhealthy-reason");
        final String policyName = BuilderUtil.getChildTextTrim(snapMirrorElt, "policy");
        final String schedule = BuilderUtil.getChildTextTrim(snapMirrorElt, "schedule");

        final CModeSnapMirror cModeSnapMirror = cluster.createSnapMirror(sourceLocation, destinationLocation);
        cModeSnapMirror.setCurrentMaxTransferRate(currentMaxTransferRate);
        cModeSnapMirror.setCurrentTransferError(currentTransferError);
        cModeSnapMirror.setCurrentTransferPriority(currentTransferPriority);
        cModeSnapMirror.setCurrentTransferType(currentTransferType);
        cModeSnapMirror.setLagTime(lagTime);
        cModeSnapMirror.setLastTransferDuration(lastTransferDuration);
        cModeSnapMirror.setLastTransferEndTimestamp(lastTransferEndTimeStamp);
        cModeSnapMirror.setLastTransferType(lastTransferType);
        cModeSnapMirror.setLastTransferError(lastTransferError);
        cModeSnapMirror.setLastTransferErrorCodes(lastTransferErrorCodes);
        cModeSnapMirror.setLastTransferNetworkCompressionRatio(lastTransferNetworkCompressionRatio);
        cModeSnapMirror.setLastTransferSize(lastTransferSize);
        cModeSnapMirror.setNetworkCompressionRatio(networkCompressionRatio);
        cModeSnapMirror.setOperationId(operationId);
        cModeSnapMirror.setProgressLastUpdated(progressLastUpdated);
        cModeSnapMirror.setRelationshipControlPlane(relationshipControlPlane);
        cModeSnapMirror.setRelationshipGroupType(relationshipGroupType);
        cModeSnapMirror.setRelationshipGroupTypeRaw(relationshipGroupTypeRaw);
        cModeSnapMirror.setRelationshipID(relationshipID);
        cModeSnapMirror.setSourceCluster(sourceCluster);
        cModeSnapMirror.setSourceVServer(sourceVServer);
        cModeSnapMirror.setSourceVServerUuid(sourceVServerUUID);
        cModeSnapMirror.setSourceVolume(sourceVolume);
        cModeSnapMirror.setDestinationCluster(destinationCluster);
        cModeSnapMirror.setDestinationVServer(destinationVServer);
        cModeSnapMirror.setDestinationVServerUuid(destinationVServerUUID);
        cModeSnapMirror.setDestinationVolume(destinationVolume);
        cModeSnapMirror.setDestinationVolumeNode(destinationVolumeNode);
        cModeSnapMirror.setMirrorState(mirrorState);
        cModeSnapMirror.setExportedSnapshot(exportedSnapshot);
        cModeSnapMirror.setNewestSnapshot(newestSnapshot);
        cModeSnapMirror.setTransferSnapshot(transferSnapshot);
        cModeSnapMirror.setConsituent(isConstituent);
        cModeSnapMirror.setHealthy(isHealthy);
        cModeSnapMirror.setMaxTransferRate(maxTransferRate);
        cModeSnapMirror.setRelationshipProgress(relationshipProgress);
        cModeSnapMirror.setRelationshipStatus(relationshipStatus);
        cModeSnapMirror.setRelationshipType(relationshipType);
        cModeSnapMirror.setSnapshotCheckpoint(snapshotCheckpoint);
        cModeSnapMirror.setSnapshotProgress(snapshotProgress);        
        cModeSnapMirror.setTries(tries);
        cModeSnapMirror.setUnhealthyReason(unhealthyReason);
        cModeSnapMirror.setPolicyName(policyName);
        cModeSnapMirror.setSchedule(schedule);
        cluster.getSnapMirrors().add(cModeSnapMirror);
    }
}
