package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeSnapMirrorPolicy;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeSnapMirrorRule;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.SnapMirrorPolicy;
import org.jdom.Element;

import java.util.LinkedHashMap;
import java.util.List;

public class SnapMirrorPolicyBuilder extends IterBuilderBase {

    public SnapMirrorPolicyBuilder() {
        super("snapmirror-policy-get-iter", "snapmirror-policy-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"snapmirror-policy-info",}, new String[]{"comment", "create-snapshot", "ignore-atime", "is-network-compression-enabled", "owner", "policy-name", "restart", "total-keep", "total-rules", "transfer-priority", "tries", "type"});
            put(new String[]{"snapmirror-policy-info", "snapmirror-policy-rules", "snapmirror-policy-rule-info"}, new String[]{"keep", "preserve", "snapmirror-label", "warn"});
        }});
    }

    @Override
    protected void buildIterInstance(final CModeCluster cluster, final Element snapMirrorPolicyInfoElt) {

        //        <comment>Default policy for DP relationship.</comment>
        //                <ignore-atime>false</ignore-atime>
        //                <owner>cluster_admin</owner>
        //                <policy-name>DPDefault</policy-name>
        //                <restart>always</restart>
        //                <total-keep>0</total-keep>
        //                <total-rules>0</total-rules>
        //                <transfer-priority>normal</transfer-priority>
        //                <tries>8</tries>
        //                <vserver-name>dev1n2</vserver-name>

        final String policyName = BuilderUtil.getChildTextTrim(snapMirrorPolicyInfoElt, "policy-name");
        final String vServerName = BuilderUtil.getChildTextTrim(snapMirrorPolicyInfoElt, "vserver-name");
        final Long totalKeep = BuilderUtil.parseLongFromChild(snapMirrorPolicyInfoElt, "total-keep");
        final String comment = BuilderUtil.getChildTextTrim(snapMirrorPolicyInfoElt, "comment");
        final EnumWrapper<SnapMirrorPolicy.TransferPriority> transferPriority = BuilderUtil.getEnumWrapperByValue(SnapMirrorPolicy.TransferPriority.class, snapMirrorPolicyInfoElt, "transfer-priority", SnapMirrorPolicy.TransferPriority.NOT_MAPPED);
        final String tries = BuilderUtil.getChildTextTrim(snapMirrorPolicyInfoElt, "tries");
        final EnumWrapper<SnapMirrorPolicy.Restart> restart = BuilderUtil.getEnumWrapperByValue(SnapMirrorPolicy.Restart.class,snapMirrorPolicyInfoElt, "restart",SnapMirrorPolicy.Restart.NOT_MAPPED);
        final Boolean ignoreAccessTime = BuilderUtil.parseBooleanFromChild(snapMirrorPolicyInfoElt, "ignore-atime");
        final Boolean createSnapshot = BuilderUtil.parseBooleanFromChild(snapMirrorPolicyInfoElt, "create-snapshot");
        final Boolean isNetworkCompressionEnabled = BuilderUtil.parseBooleanFromChild(snapMirrorPolicyInfoElt, "is-network-compression-enabled");
        final EnumWrapper<SnapMirrorPolicy.Type> type = BuilderUtil.getEnumWrapperByValue(SnapMirrorPolicy.Type.class, snapMirrorPolicyInfoElt, "type", SnapMirrorPolicy.Type.NOT_MAPPED);

        final CModeSnapMirrorPolicy snapMirrorPolicy = cluster.createSnapMirrorPolicy(vServerName, policyName);
        snapMirrorPolicy.setTotalKeep(totalKeep);
        snapMirrorPolicy.setComment(comment);
        snapMirrorPolicy.setTransferPriority(transferPriority);
        snapMirrorPolicy.setTries(tries);
        snapMirrorPolicy.setRestart(restart);
        snapMirrorPolicy.setIgnoreAccessTime(ignoreAccessTime);
        snapMirrorPolicy.setCreateSnapshot(createSnapshot);
        snapMirrorPolicy.setNetworkCompressionEnabled(isNetworkCompressionEnabled);
        snapMirrorPolicy.setType(type);
        cluster.getSnapMirrorPolicies().add(snapMirrorPolicy);

        final Element policyRulesElt = snapMirrorPolicyInfoElt.getChild("snapmirror-policy-rules");
        if (policyRulesElt != null) {
            int index = 0;
            @SuppressWarnings({"unchecked"})
            final List<Element> snapMirrorRuleInfoElts = policyRulesElt.getChildren("snapmirror-policy-rule-info");
            for (final Element snapMirrorRuleInfoElt : snapMirrorRuleInfoElts) {
                final String keep = (snapMirrorRuleInfoElt == null) ? null : BuilderUtil.getChildTextTrim(snapMirrorRuleInfoElt, "keep");
                final Boolean preserve = BuilderUtil.parseBooleanFromChild(snapMirrorRuleInfoElt, "preserve");
                final String  label = (snapMirrorRuleInfoElt == null) ? null : BuilderUtil.getChildTextTrim(snapMirrorRuleInfoElt, "snapmirror-label");
                final Long warn = BuilderUtil.parseLongFromChild(snapMirrorRuleInfoElt, "warn");
                final CModeSnapMirrorRule snapMirrorRule = snapMirrorPolicy.createCModeSnapMirrorRule(index++);
                snapMirrorRule.setKeep(keep);
                snapMirrorRule.setPreserve(preserve);
                snapMirrorRule.setLabel(label);
                snapMirrorRule.setWarn(warn);
                snapMirrorPolicy.getCmodeSnapMirrorRules().add(snapMirrorRule);
            }
        }
    }
}
