package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeSnapshot;
import org.apache.log4j.Logger;
import org.jdom.Element;

import java.util.LinkedHashMap;

public class SnapshotBuilder extends IterBuilderBase {

    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger logger = Logger.getLogger(SnapshotBuilder.class);

    public SnapshotBuilder() {
        super("snapshot-get-iter", "snapshot-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"snapshot-info",}, new String[]{"access-time", "busy", "cumulative-percentage-of-total-blocks", "dependency", "name", "percentage-of-total-blocks", "snapshot-instance-uuid", "snapshot-version-uuid", "volume", "vserver",});
        }});
    }

    @Override
    protected void buildIterInstance(final CModeCluster cluster, final Element snapshotInfoElt) {
        final Long accessTime = BuilderUtil.parseLongFromChild(snapshotInfoElt, "access-time");
        final Boolean isBusy = BuilderUtil.parseBooleanFromChild(snapshotInfoElt, "busy");
        final Long cumulativePercentageOfTotalBlocks = BuilderUtil.parseLongFromChild(snapshotInfoElt, "cumulative-percentage-of-total-blocks");
        final String dependency = BuilderUtil.getChildTextTrim(snapshotInfoElt, "dependency");
        final String name = BuilderUtil.getChildTextTrim(snapshotInfoElt, "name");
        final Integer percentageOfTotalBlocks = BuilderUtil.parseIntFromChild(snapshotInfoElt, "percentage-of-total-blocks");
        final String instanceUUID = BuilderUtil.getChildTextTrim(snapshotInfoElt, "snapshot-instance-uuid");
        final String versionUUID = BuilderUtil.getChildTextTrim(snapshotInfoElt, "snapshot-version-uuid");
        final String volumeName = BuilderUtil.getChildTextTrim(snapshotInfoElt, "volume");
        final String vServerName = BuilderUtil.getChildTextTrim(snapshotInfoElt, "vserver");
        final CModeSnapshot snapshot = cluster.createSnapshot(vServerName, volumeName, instanceUUID);
        snapshot.setAccessTime(accessTime);
        snapshot.setBusy(isBusy);
        snapshot.setCumulativePercentageOfTotalBlocks(cumulativePercentageOfTotalBlocks);
        snapshot.setDependency(dependency);
        snapshot.setName(name);
        snapshot.setPercentageOfTotalBlocks(percentageOfTotalBlocks);
        snapshot.setVersionUUID(versionUUID);
        cluster.getSnapshots().add(snapshot);
    }
}
