package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeSnapshotPolicy;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeSnapshotSchedule;
import org.jdom.Element;

import java.util.LinkedHashMap;
import java.util.List;

public class SnapshotPolicyBuilder extends IterBuilderBase {

    public SnapshotPolicyBuilder() {
        super("snapshot-policy-get-iter", "snapshot-policy-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"snapshot-policy-info",}, new String[]{"vserver-name", "policy", "enabled", "comment",});
            put(new String[]{"snapshot-policy-info", "snapshot-policy-schedules", "snapshot-schedule-info"}, new String[]{"schedule", "prefix", "count", "snapmirror-label"});
        }});
    }

    @Override
    protected void buildIterInstance(final CModeCluster cluster, final Element snapshotPolicyInfoElt) {
        final String vServerName = BuilderUtil.getChildTextTrim(snapshotPolicyInfoElt, "vserver-name");
        final String policyName = BuilderUtil.getChildTextTrim(snapshotPolicyInfoElt, "policy");
        final Boolean enabled = BuilderUtil.parseBooleanFromChild(snapshotPolicyInfoElt, "enabled");
        final String comment = BuilderUtil.getChildTextTrim(snapshotPolicyInfoElt, "comment");
        final CModeSnapshotPolicy snapshotPolicy = cluster.createSnapshotPolicy(vServerName, policyName);
        snapshotPolicy.setVServerName(vServerName);
        snapshotPolicy.setEnabled(enabled);
        snapshotPolicy.setComment(comment);
        cluster.getSnapshotPolicies().add(snapshotPolicy);

        final Element policySchedulesElt = snapshotPolicyInfoElt.getChild("snapshot-policy-schedules");
        if (policySchedulesElt != null) {
            @SuppressWarnings({"unchecked"})
            final List<Element> snapshotScheduleInfoElts = policySchedulesElt.getChildren("snapshot-schedule-info");
            for (final Element snapshotScheduleInfoElt : snapshotScheduleInfoElts) {
                final String scheduleName = BuilderUtil.getChildTextTrim(snapshotScheduleInfoElt, "schedule");
                final String prefix = BuilderUtil.getChildTextTrim(snapshotScheduleInfoElt, "prefix");
                final Long count = BuilderUtil.parseLongFromChild(snapshotScheduleInfoElt, "count");
                final String snapmirrorLabel = BuilderUtil.getChildTextTrim(snapshotScheduleInfoElt, "snapmirror-label");
                final CModeSnapshotSchedule shapshotSchedule = snapshotPolicy.createCModeSnapshotSchedule(scheduleName);
                shapshotSchedule.setPrefix(prefix);
                shapshotSchedule.setCount(count);
                shapshotSchedule.setSnapmirrorLabel(snapmirrorLabel);
                snapshotPolicy.getCModeSnapshotSchedules().add(shapshotSchedule);
            }
        }
    }
}
