package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.partialpoll.ChangeResult;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.partialpoll.PartialPollContext;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPINames;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIUtils;
import org.jdom.Element;

import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIUtils.elem;

/**
 * Builder that will fill in the partial poll context with the change information from the
 * cluster for the supplied ZAPIs
 */
public class SystemAPIChangeBuilder extends IterBuilderBaseImpl<PartialPollContext>
{
  /**
   * Constructor for the SystemAPIChangeBuilder
   *
   * @param context contains the list of zapi that will be supported for partial polling
   */
  public SystemAPIChangeBuilder( final PartialPollContext context )
  {
    super( "system-api-change-get-iter", "api-change-info", getDefaultMaxRecords(), null,
            elem( ZAPINames.QUERY,
                    elem( "api-change-info",
                            elem( "api-name", ZAPIUtils.formattedZapiNameList( context.getSupportedZapiNames() ) )
                    )
            )
    );
  }

  @Override
  protected void buildIterInstance(PartialPollContext context, Element iterationElt)
  {
    //get all elements for the api-change-opaque-info
    Element apiNameElt = iterationElt.getChild( "api-name" );
    Element changeDataElt = iterationElt.getChild( "change-data" );
    Element changeOpaqueInfoElt = changeDataElt.getChild( "api-change-opaque-info" );
    Element changeTimeElt = changeOpaqueInfoElt.getChild( "change-time" );
    Element changesElt = changeOpaqueInfoElt.getChild( "changes" );

    //create the result object
    String name = apiNameElt.getText().trim();
    long changeTime = Long.parseLong( changeTimeElt.getText() );
    long changes = Long.parseLong( changesElt.getText() );

    //add the result to the context
    context.addResult( new ChangeResult( name, changeTime, changes ) );
  }
}
