package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;


import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeVServerNameMapping;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.VServerNameMapping;
import org.apache.log4j.Logger;
import org.jdom.Element;

import java.util.LinkedHashMap;

public class VServerNameMapBuilder extends IterBuilderBase {

    private static final Logger logger = Logger.getLogger(VServerNameMapBuilder.class);

    protected VServerNameMapBuilder() {
        super("name-mapping-get-iter", "name-mapping-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
                    put(new String[]{"name-mapping-info",}, new String[]{"direction", "pattern", "position", "replacement", "vserver", });
        }});
    }

    @Override
    protected void buildIterInstance(final CModeCluster cluster, final Element nameMappingInfoElt) {
        final String vServerName = BuilderUtil.getChildTextTrim(nameMappingInfoElt, "vserver");
        final EnumWrapper<VServerNameMapping.Direction> directionWrapper =
                BuilderUtil.getEnumWrapperByValue(VServerNameMapping.Direction.class, nameMappingInfoElt, "direction", VServerNameMapping.Direction.NOT_MAPPED);
        final Integer position = BuilderUtil.parseIntFromChild(nameMappingInfoElt, "position");
        final String pattern = BuilderUtil.getChildTextTrim(nameMappingInfoElt, "pattern");
        final String replacement = BuilderUtil.getChildTextTrim(nameMappingInfoElt, "replacement");

        if (vServerName == null) {
            // Older ONTAP versions don't identify the vServer, so there is no point to continue
            if (logger.isDebugEnabled()) {
                logger.debug("Ignoring vServer name mapping with no vServer");
            }
            return;
        }

        final CModeVServerNameMapping nameMapping = cluster.createVServerNameMapping(vServerName, position, directionWrapper.getRawValue());
        nameMapping.setDirectionWrapper(directionWrapper);
        nameMapping.setPattern(pattern);
        nameMapping.setReplacement(replacement);
        cluster.getVServerNameMappings().add(nameMapping);
    }
}
