package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeVolumeMove;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.VolumeMove;
import org.apache.log4j.Logger;
import org.jdom.Element;

import java.util.LinkedHashMap;

public class VolumeMoveBuilder extends IterBuilderBase {

    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger logger = Logger.getLogger(VolumeMoveBuilder.class);

    public VolumeMoveBuilder() {
        super("volume-move-get-iter", "volume-move-info", getDefaultMaxRecords(), new LinkedHashMap<String[], String[]>() {{
            put(new String[]{"volume-move-info",}, new String[]{"actual-completion-timestamp", "destination-aggregate", "details", "estimated-completion-time", "job-id", "job-uuid", "percent-complete", "phase", "source-aggregate", "start-timestamp", "state", "volume", "vserver",});
        }});
    }

    @Override
    protected void buildIterInstance(final CModeCluster cluster, final Element volumeMoveInfoElt) {
        final Long actualCompletionTimestamp = BuilderUtil.parseLongFromChild(volumeMoveInfoElt, "actual-completion-timestamp");
        final String destinationAggregate = BuilderUtil.getChildTextTrim(volumeMoveInfoElt, "destination-aggregate");
        final String details = BuilderUtil.getChildTextTrim(volumeMoveInfoElt, "details");
        final Long jobID = BuilderUtil.parseLongFromChild(volumeMoveInfoElt, "job-id");
        final String jobUUID = BuilderUtil.getChildTextTrim(volumeMoveInfoElt, "job-uuid");
        final Integer percentComplete = BuilderUtil.parseIntFromChild(volumeMoveInfoElt, "percent-complete");
        final EnumWrapper<VolumeMove.Phase> phase = BuilderUtil.getEnumWrapperByValue(VolumeMove.Phase.class, volumeMoveInfoElt, "phase", VolumeMove.Phase.NOT_MAPPED);
        final String sourceAggregate = BuilderUtil.getChildTextTrim(volumeMoveInfoElt, "source-aggregate");
        final Long startTimestamp = BuilderUtil.parseLongFromChild(volumeMoveInfoElt, "start-timestamp");
        final EnumWrapper<VolumeMove.State> state = BuilderUtil.getEnumWrapperByValue(VolumeMove.State.class, volumeMoveInfoElt, "state", VolumeMove.State.NOT_MAPPED);
        final String volume = BuilderUtil.getChildTextTrim(volumeMoveInfoElt, "volume");
        final String vServer = BuilderUtil.getChildTextTrim(volumeMoveInfoElt, "vserver");
        final Long estimatedCompletionTime = BuilderUtil.parseLongFromChild(volumeMoveInfoElt, "estimated-completion-time");

        final CModeVolumeMove volumeMove = cluster.createVolumeMove(jobUUID);
        volumeMove.setActualCompletionTimestamp(actualCompletionTimestamp);
        volumeMove.setDestinationAggregate(destinationAggregate);
        volumeMove.setDetails(details);
        volumeMove.setJobID(jobID);
        volumeMove.setPercentComplete(percentComplete == null ? 0 : percentComplete); // Assume 0% complete when value is missing
        volumeMove.setPhase(phase);
        volumeMove.setSourceAggregate(sourceAggregate);
        volumeMove.setStartTimestamp(startTimestamp);
        volumeMove.setState(state);
        volumeMove.setVolume(volume);
        volumeMove.setVServer(vServer);
        volumeMove.setEstimatedCompletionTime(estimatedCompletionTime);
        cluster.getVolumeMoves().add(volumeMove);
    }
}

