package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;

import com.netapp.oci.netapp.performance.aggregate.interfaces.data.AggregateMeasurements;
import com.netapp.oci.netapp.performance.aggregate.interfaces.data.AggregateRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappAggregateMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappAggregateMetrics.AggregateCounter;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster.ClusterKey;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import java.util.Collection;
import java.util.List;
import java.util.Map;

public class AggregateMetricsBuilder extends MetricsBuilderBase<Cluster.ClusterKey,
	AggregateMeasurements,
	AggregateRawMeasurements,
	NetappAggregateMetrics,
	NetappAggregateMetrics.AggregateCounter,
	NetappAggregateMetrics.AggregateCounter>{

    public AggregateMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage,"aggregate", NetappAggregateMetrics.AggregateCounter.class);
    }

    @Override
    List<String> getTargetPerfUUIDs(NetAppDefaultStorageContext context,
    		ZAPIConnection zapiConnection) {
    	return context.getAggregatePerfUUIDs();
    }

    @Override
    ClusterKey getParentInventoryUUIDKey(NetAppDefaultStorageContext context,
    		String perfInstanceUUID) {
    	return context.getKey();
    }

    @Override
    String getIventoryUUIDFromPerfUUID(NetAppDefaultStorageContext context,
    		String perfInstanceUUID) {
    	return context.getAggregatePerfUUIDToUUID().get(perfInstanceUUID);
    }

    @Override
    NetappAggregateMetrics createMetricsFromValues(
    		MetricsRowScratchPad<AggregateCounter> currentValues,
    		MetricsRowScratchPad<AggregateCounter> previousValues) {
    	return new NetappAggregateMetrics(currentValues, previousValues, getNetAppOCIEPerformancePackage());
    }

    @Override
    void reportPerformance(ClusterKey clusterKey, Map<ClusterKey, Collection<RawSample<AggregateMeasurements>>> measurements) throws DataSourceErrorException {
	
    	for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<AggregateMeasurements>>> measurementEntry : measurements.entrySet()) {
    		final Cluster.ClusterKey key = measurementEntry.getKey();
    		final Collection<RawSample<AggregateMeasurements>> aggregateMeasurements = measurementEntry.getValue();
    		storagePerformanceReporter.reportAggregatePerformance(key, aggregateMeasurements);
    	}    	
    }

    @Override
    void reportRawPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<AggregateRawMeasurements>>> measurements) throws DataSourceErrorException {
    	for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<AggregateRawMeasurements>>> measurementEntry : measurements.entrySet()) {
    		final Cluster.ClusterKey key = measurementEntry.getKey();
    		final Collection<RawSample<AggregateRawMeasurements>> aggregateMeasurements = measurementEntry.getValue();
    		storagePerformanceReporter.reportAggregateRawPerformance(key, aggregateMeasurements);
    	}
    }

}
