package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;

import com.netapp.oci.netapp.performance.cifsvserver.interfaces.data.CifsVserverMeasurements;
import com.netapp.oci.netapp.performance.cifsvserver.interfaces.data.CifsVserverRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappCifsVserverMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappCifsVserverMetrics.CifsVserverCounter;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster.ClusterKey;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

public class CifsVserverMetricsBuilder extends MetricsBuilderBase<Cluster.ClusterKey,
	CifsVserverMeasurements,
	CifsVserverRawMeasurements,
	NetappCifsVserverMetrics,
	NetappCifsVserverMetrics.CifsVserverCounter,
	NetappCifsVserverMetrics.CifsVserverCounter>{

	public CifsVserverMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
		super(reportingPackage, "cifs:vserver", NetappCifsVserverMetrics.CifsVserverCounter.class);
	}

	@Override
	List<String> getTargetPerfUUIDs(NetAppDefaultStorageContext context, ZAPIConnection zapiConnection) {
		return new ArrayList<String>(context.getCifsVserverIDToUUIDMap().keySet());
	}

	@Override
	ClusterKey getParentInventoryUUIDKey(NetAppDefaultStorageContext context, String perfInstanceUUID) {
		return context.getKey();
	}

	@Override
	String getIventoryUUIDFromPerfUUID(NetAppDefaultStorageContext context, String perfInstanceUUID) {
		return context.getCifsVserverIDToUUIDMap().get(perfInstanceUUID);
	}

	@Override
	NetappCifsVserverMetrics createMetricsFromValues(
			MetricsRowScratchPad<CifsVserverCounter> currentValues,
			MetricsRowScratchPad<CifsVserverCounter> previousValues) {
		return new NetappCifsVserverMetrics(currentValues, previousValues);
	}
	
	@Override
	void reportPerformance(ClusterKey clusterKey, Map<ClusterKey, Collection<RawSample<CifsVserverMeasurements>>> measurements)
					throws DataSourceErrorException {
		
	    for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<CifsVserverMeasurements>>> measurementEntry : measurements.entrySet()) {
            final Cluster.ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<CifsVserverMeasurements>> cifsMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportCifsVserverPerformance(key, cifsMeasurements);
        }
	}

	@Override
	void reportRawPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<CifsVserverRawMeasurements>>> measurements) throws DataSourceErrorException {
		for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<CifsVserverRawMeasurements>>> measurementEntry : measurements.entrySet()) {
			final Cluster.ClusterKey key = measurementEntry.getKey();
			final Collection<RawSample<CifsVserverRawMeasurements>> cifsMeasurements = measurementEntry.getValue();
			storagePerformanceReporter.reportCifsVserverRawPerformance(key, cifsMeasurements);
		}
	}

}
