package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;

import com.netapp.oci.netapp.performance.cluster.interfaces.data.ClusterMeasurements;
import com.netapp.oci.netapp.performance.node.interfaces.data.NodeMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.data.QosWorkloadConstituentData;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.data.QosWorkloadDetailData;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.data.ReAttributionContext;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.impl.ReAttributionServiceFactory;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.interfaces.IReAttributionContext;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.interfaces.IReAttributionService;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import java.util.*;
import java.util.Map.Entry;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Builder class for processing and reporting cluster metrics derived from node 
 * and measurements.
 */
public class ClusterMetricsBuilder {

    private static final Logger LOG = LoggerFactory.getLogger(ClusterMetricsBuilder.class);
    private final NodeMetricsBuilder nodeMetricsBuilder;

    protected final NetAppOCIEPerformancePackage reportingPackage; 
    
    public ClusterMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage, final NodeMetricsBuilder nodeMetricsBuilder){
        this.reportingPackage =   reportingPackage;
        this.nodeMetricsBuilder =  nodeMetricsBuilder;
    }
    
    /**
     * process node  measurements to derive cluster metrics
     */
    public LinkedList<RawSample<ClusterMeasurements>> getClusterPerformanceMetrics(NetAppDefaultStorageContext context){
        LinkedList<RawSample<ClusterMeasurements>> measurementList = new LinkedList<RawSample<ClusterMeasurements>>();
        Map<String, NodeMeasurements> nodeMeasurementsMap = nodeMetricsBuilder.getNodeMeasurements();

       ClusterMeasurements clusterMeasurement = new ClusterMeasurements();
        for(Entry<String, NodeMeasurements> eachNodeMeasurement : nodeMeasurementsMap.entrySet()){
                String inventoryUUID = eachNodeMeasurement.getKey();
                NodeMeasurements nodeMeasurement = eachNodeMeasurement.getValue();
                clusterMeasurement.setClusterOps(MetricsRowScratchPad.add(clusterMeasurement.getClusterOps(), nodeMeasurement.getTotalOps()));
                clusterMeasurement.setClusterThroughput(MetricsRowScratchPad.add(clusterMeasurement.getClusterThroughput(), nodeMeasurement.getOpmSysThroughput()));
                clusterMeasurement.setClusterMaxNodeUtilization(Math.max(clusterMeasurement.getClusterMaxNodeUtilization(), nodeMeasurement.getOpmSysUtilization()));
                clusterMeasurement.setClusterMaxAggregateUtilization(Math.max(clusterMeasurement.getClusterMaxAggregateUtilization(), nodeMeasurement.getMaxAggregateUtilization()));
         }
            RawSample<ClusterMeasurements> clusterMeasurementRawSample = new RawSample<ClusterMeasurements>(context.getKey().getOcieUUID(), clusterMeasurement);
            measurementList.add(clusterMeasurementRawSample);
            return measurementList;
    }

    public void getAndReportPerformance(NetAppDefaultStorageContext context) throws DataSourceErrorException {
        Map<Cluster.ClusterKey,Collection<RawSample<ClusterMeasurements>>> metricsToReport = new HashMap<Cluster.ClusterKey, Collection<RawSample<ClusterMeasurements>>>();
        LinkedList<RawSample<ClusterMeasurements>> measurementList = getClusterPerformanceMetrics(context);
        metricsToReport.put(context.getKey(),measurementList);
        reportPerformance(metricsToReport);
    }
    
    void reportPerformance(
            final Map<Cluster.ClusterKey, Collection<RawSample<ClusterMeasurements>>> measurements)
            throws DataSourceErrorException {
        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<ClusterMeasurements>>> measurementEntry : measurements
                .entrySet()) {
            final Cluster.ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<ClusterMeasurements>> clusterMeasurements = measurementEntry
                    .getValue();
            reportingPackage.reportClusterPerformance(
                    key, clusterMeasurements);
            LOG.debug("Cluster measurements are reported successfully with current poll stats. Cluster Id: {}", measurementEntry.getKey());
        }
    }
}    
