package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;

import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifMeasurements;
import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappFcpLifMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.framework.datasource.PackageContext;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.VServer;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;
import org.apache.log4j.Logger;

import java.util.*;

public class FcpLifMetricsBuilder extends MetricsBuilderBase<VServer.VServerKey,
                                                             FcpLifMeasurements,
                                                             FcpLifRawMeasurements,
                                                             NetappFcpLifMetrics,
                                                             NetappFcpLifMetrics.FcpLifCounter,
                                                             NetappFcpLifMetrics.FcpLifCounter> {

    private static final Logger logger = Logger.getLogger(FcpLifMetricsBuilder.class);
    private final Map<String, String> fcpLifPerfUUIDToVServerUUIDMap = new HashMap<String, String>();
    private final Map<String, String> fcpLifPerfUUIDToLifFoundationUUID = new TreeMap<String, String>();

    public FcpLifMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage, "fcp_lif", NetappFcpLifMetrics.FcpLifCounter.class);
    }

    @Override
    List<String> getTargetPerfUUIDs(final NetAppDefaultStorageContext context, final ZAPIConnection zapiConnection) {
        fcpLifPerfUUIDToVServerUUIDMap.clear();
        fcpLifPerfUUIDToLifFoundationUUID.clear();
        final Map<String, String> fcpLifNameToFoundationUUIDMap = context.getFcpLifNameToFoundationUUIDMap();
        final Map<String, String> fcpLifNameToVServerUUIDMap = context.getFcpLifNameToVServerUUIDMap();

        // Perf Names are formatted VServerName:LIFName
        // Need to pull out the names so we can correlate the UUIDs
        if (context.isLessThanSierraNevada()) {
            // We need to discover the performance UUID for the LIF dynamically, it is not provided by
            // foundation.
            final Map<String, String> perfUUIDToNameMap = getPerfUUIDToNameMap(zapiConnection);
            for (final Map.Entry<String, String> entry : perfUUIDToNameMap.entrySet()) {
                final String lifPerfUUID = entry.getKey();
                final String lifPerfName = entry.getValue();
                if (lifPerfName == null) {
                    logger.warn(PackageContext.getLogPrefix() + " - Failed find name for fcp lif " + lifPerfUUID);
                    continue;
                }

                if (!fcpLifNameToFoundationUUIDMap.containsKey(lifPerfName)) {
                    logger.warn(PackageContext.getLogPrefix() + " - Failed to find UUID for fcp lif " + lifPerfName);
                } else {
                    final String lifFoundationUUID = fcpLifNameToFoundationUUIDMap.get(lifPerfName);
                    fcpLifPerfUUIDToLifFoundationUUID.put(lifPerfUUID, lifFoundationUUID);
                }

                if (!fcpLifNameToVServerUUIDMap.containsKey(lifPerfName)) {
                    logger.warn(PackageContext.getLogPrefix() + " - Failed to find vserver for fcp lif " + lifPerfName);
                } else {
                    final String vServerUUID = fcpLifNameToVServerUUIDMap.get(lifPerfName);
                    fcpLifPerfUUIDToVServerUUIDMap.put(lifPerfUUID, vServerUUID);
                }
            }
        } else {
            for (final Map.Entry<String, String> lifEntry : fcpLifNameToFoundationUUIDMap.entrySet()) {
                final String lifName = lifEntry.getKey();
                final String lifPerfUUID = lifEntry.getValue();
                fcpLifPerfUUIDToLifFoundationUUID.put(lifPerfUUID, lifPerfUUID);
                final String vServerUUID = fcpLifNameToVServerUUIDMap.get(lifName);
                fcpLifPerfUUIDToVServerUUIDMap.put(lifPerfUUID, vServerUUID);
            }
        }
        return new ArrayList<String>(fcpLifPerfUUIDToLifFoundationUUID.keySet());
    }

    /**
     * Required for 8.1, need to discovery the performance UUIDs dynamically
     * @param zapiConnection connection used to discover performance UUIDs
     * @return map of Lif performance UUID to Lif Name
     */
    private Map<String, String> getPerfUUIDToNameMap(final ZAPIConnection zapiConnection) {
        final Map<String, String> perfUUIDToNameMap = new HashMap<String, String>();
        try {
            perfUUIDToNameMap.putAll(getPerfInstanceUUIDs(targetObjectName, zapiConnection));
        }
        catch (DataSourceErrorException ex) {
            logger.error(PackageContext.getLogPrefix() + " - Failed to discover fcp lif performance UUIDs: " + ex.getLocalizedMessage(), ex);
        }
        return perfUUIDToNameMap;
    }

    @Override
    VServer.VServerKey getParentInventoryUUIDKey(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        final String vServerUUID = fcpLifPerfUUIDToVServerUUIDMap.get(perfInstanceUUID);
        if (vServerUUID == null) {
            logger.warn(PackageContext.getLogPrefix() + " - Failed to find vServer for fcp lif " + perfInstanceUUID);
            return null;
        }
        return new VServer.VServerKey(context.getKey().getOcieUUID(), vServerUUID);
    }

    @Override
    String getIventoryUUIDFromPerfUUID(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return fcpLifPerfUUIDToLifFoundationUUID.get(perfInstanceUUID);
    }

    @Override
    NetappFcpLifMetrics createMetricsFromValues(final MetricsRowScratchPad<NetappFcpLifMetrics.FcpLifCounter> currentValues, final MetricsRowScratchPad<NetappFcpLifMetrics.FcpLifCounter> previousValues) {
        return new NetappFcpLifMetrics(currentValues, previousValues);
    }

    @Override
    void reportPerformance(Cluster.ClusterKey clusterKey, final Map<VServer.VServerKey, Collection<RawSample<FcpLifMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<VServer.VServerKey, Collection<RawSample<FcpLifMeasurements>>> measurementEntry : measurements.entrySet()) {
            final VServer.VServerKey key = measurementEntry.getKey();
            final Collection<RawSample<FcpLifMeasurements>> fcpLifMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportFcpLifPerformance(clusterKey, key, fcpLifMeasurements);
        }
    }

    @Override
    void reportRawPerformance(Cluster.ClusterKey clusterKey, 
                              final Map<VServer.VServerKey, Collection<RawSample<FcpLifRawMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<VServer.VServerKey, Collection<RawSample<FcpLifRawMeasurements>>> measurementEntry : measurements.entrySet()) {
            final VServer.VServerKey key = measurementEntry.getKey();
            final Collection<RawSample<FcpLifRawMeasurements>> fcpLifMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportFcpLifRawPerformance(clusterKey, key, fcpLifMeasurements);
        }
    }

}
