package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;

import com.netapp.oci.netapp.performance.fcpport.interfaces.data.FcpPortMeasurements;
import com.netapp.oci.netapp.performance.fcpport.interfaces.data.FcpPortRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation.BuilderUtil;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappFcpPortMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.framework.datasource.PackageContext;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class FcpPortMetricsBuilder extends MetricsBuilderBase<Cluster.ClusterKey,
                                                              FcpPortMeasurements,
                                                              FcpPortRawMeasurements,
                                                              NetappFcpPortMetrics,
                                                              NetappFcpPortMetrics.FcpPortCounter,
                                                              NetappFcpPortMetrics.FcpPortCounter> {

    private static final Logger logger = Logger.getLogger(FcpPortMetricsBuilder.class);

    public FcpPortMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage, "fcp_port", NetappFcpPortMetrics.FcpPortCounter.class);
    }

    @Override
    List<String> getTargetPerfUUIDs(final NetAppDefaultStorageContext context, final ZAPIConnection zapiConnection) {
        // FCP Port NOT in foundation yet
        final Map<String, String> perfUUIDToNameMap = new HashMap<String, String>();
        try {
            perfUUIDToNameMap.putAll(getPerfInstanceUUIDs(targetObjectName, zapiConnection));
        }
        catch (DataSourceErrorException dse) {
            this.clearPreviousValues();
            logger.info(PackageContext.getLogPrefix() + " - Cleared counter values cached from previous poll for " + this.getClass().getName());
            logger.error(PackageContext.getLogPrefix() + " - Failed to discover lif performance UUIDs: " + dse.getLocalizedMessage(), dse);
        }
        return new ArrayList<String>(perfUUIDToNameMap.keySet());
    }

    @Override
    Cluster.ClusterKey getParentInventoryUUIDKey(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return context.getKey();
    }

    @Override
    String getIventoryUUIDFromPerfUUID(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        final String normalizedPerfUUID = BuilderUtil.normalizeWWN(perfInstanceUUID);
        return context.getFcpPortPerfUUIDToUUIDMap().get(normalizedPerfUUID);
    }

    @Override
    NetappFcpPortMetrics createMetricsFromValues(final MetricsRowScratchPad<NetappFcpPortMetrics.FcpPortCounter> currentValues, final MetricsRowScratchPad<NetappFcpPortMetrics.FcpPortCounter> previousValues) {
        return new NetappFcpPortMetrics(currentValues, previousValues, netAppOCIEPerformancePackage);
    }


    @Override
    void reportPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<FcpPortMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<FcpPortMeasurements>>> measureEntry : measurements.entrySet()) {
            storagePerformanceReporter.reportFcpPortPerformance(measureEntry.getKey(), measureEntry.getValue());
        }
    }
    
    @Override
    void reportRawPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<FcpPortRawMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<FcpPortRawMeasurements>>> measureEntry : measurements.entrySet()) {
            storagePerformanceReporter.reportFcpPortRawPerformance(measureEntry.getKey(), measureEntry.getValue());
        }
    }

}
