package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifVserverMeasurements;
import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifVserverRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappLifVserverMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster.ClusterKey;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

public class LifVserverMetricsBuilder extends MetricsBuilderBase<ClusterKey,
	NetworkLifVserverMeasurements,
	NetworkLifVserverRawMeasurements,
	NetappLifVserverMetrics,
	NetappLifVserverMetrics.LifVserverCounter,
	NetappLifVserverMetrics.LifVserverCounter> {

	public LifVserverMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
		super(reportingPackage, "lif:vserver", NetappLifVserverMetrics.LifVserverCounter.class);
	}

	@Override
	List<String> getTargetPerfUUIDs(final NetAppDefaultStorageContext context, final ZAPIConnection zapiConnection) {
		return new ArrayList<String>(context.getLifVserverIDToUUIDMap().keySet());
	}

	@Override
	ClusterKey getParentInventoryUUIDKey(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
		return context.getKey();
	}

	@Override
	String getIventoryUUIDFromPerfUUID(NetAppDefaultStorageContext context,	String perfInstanceUUID) {
		return context.getLifVserverIDToUUIDMap().get(perfInstanceUUID);
	} 

	@Override
	NetappLifVserverMetrics createMetricsFromValues(final MetricsRowScratchPad<NetappLifVserverMetrics.LifVserverCounter> currentValues, final MetricsRowScratchPad<NetappLifVserverMetrics.LifVserverCounter> previousValues) {
		return new NetappLifVserverMetrics(currentValues, previousValues);
	}

	@Override
	void reportPerformance(ClusterKey clusterKey, final Map<ClusterKey, Collection<RawSample<NetworkLifVserverMeasurements>>> measurements) throws DataSourceErrorException {
		for (final Map.Entry<ClusterKey, Collection<RawSample<NetworkLifVserverMeasurements>>> measurementEntry : measurements.entrySet()) {
			final ClusterKey key = measurementEntry.getKey();
			final Collection<RawSample<NetworkLifVserverMeasurements>> lifMeasurements = measurementEntry.getValue();
			storagePerformanceReporter.reportLifVserverPerformance(key, lifMeasurements);
		}
	}

	@Override 
	void reportRawPerformance(ClusterKey clusterKey, final Map<ClusterKey, Collection<RawSample<NetworkLifVserverRawMeasurements>>> rawMeasurements) throws DataSourceErrorException { 
		for (final Map.Entry<ClusterKey, Collection<RawSample<NetworkLifVserverRawMeasurements>>> measurementEntry : rawMeasurements.entrySet()) { 
			final ClusterKey key = measurementEntry.getKey(); 
			final Collection<RawSample<NetworkLifVserverRawMeasurements>> lifMeasurements = measurementEntry.getValue(); 
			storagePerformanceReporter.reportLifVserverRawPerformance(key, lifMeasurements); 
		} 
	}

}
