package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;


import com.netapp.oci.netapp.performance.node.interfaces.data.NFSv41NodeMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappNFSv41NodeMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

public class NFSv41NodeMetricsBuilder extends MetricsBuilderBase<Cluster.ClusterKey,
        NFSv41NodeMeasurements,
        NFSv41NodeMeasurements,
        NetappNFSv41NodeMetrics,
        NetappNFSv41NodeMetrics.NFSv41NodeCounter,
        NetappNFSv41NodeMetrics.OCIENFSv41NodeCounter> {


    public NFSv41NodeMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage, "nfsv4_1:node", NetappNFSv41NodeMetrics.NFSv41NodeCounter.class);
    }

    @Override
    List<String> getTargetPerfUUIDs(final NetAppDefaultStorageContext context, final ZAPIConnection zapiConnection) {
        return new ArrayList<String>(context.getNfsv4_1PerfUUIDtoNodeUUID().keySet());
    }

    @Override
    Cluster.ClusterKey getParentInventoryUUIDKey(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return context.getKey();        // Parent of each node is the cluster
    }

    @Override
    String getIventoryUUIDFromPerfUUID(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return context.getNfsv4_1PerfUUIDtoNodeUUID().get(perfInstanceUUID);
    }

    @Override
    NetappNFSv41NodeMetrics createMetricsFromValues(final MetricsRowScratchPad<NetappNFSv41NodeMetrics.NFSv41NodeCounter> currentValues,
                                                    final MetricsRowScratchPad<NetappNFSv41NodeMetrics.NFSv41NodeCounter> previousValues) {
        return new NetappNFSv41NodeMetrics(currentValues, previousValues);
    }

    @Override
    void reportPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<NFSv41NodeMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<NFSv41NodeMeasurements>>> measurementEntry : measurements.entrySet()) {
            final Cluster.ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<NFSv41NodeMeasurements>> nodeMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportNFSv41NodePerformance(key, nodeMeasurements);
        }
    }
}
