package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;

import com.netapp.oci.netapp.performance.nfs.interfaces.data.Nfsv3Measurements;
import com.netapp.oci.netapp.performance.nfs.interfaces.data.Nfsv3RawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappNfsv3Metrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappNfsv3Metrics.Nfsv3Counter;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster.ClusterKey;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

public class Nfsv3MetricsBuilder extends MetricsBuilderBase<Cluster.ClusterKey,
	Nfsv3Measurements,
	Nfsv3RawMeasurements,
	NetappNfsv3Metrics,
	NetappNfsv3Metrics.Nfsv3Counter,
	NetappNfsv3Metrics.Nfsv3Counter>{

    public Nfsv3MetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage, "nfsv3", NetappNfsv3Metrics.Nfsv3Counter.class);
    }

	@Override
	List<String> getTargetPerfUUIDs(NetAppDefaultStorageContext context, ZAPIConnection zapiConnection) {
		return new ArrayList<String>(context.getNfsv3VserverIDToUUIDMap().keySet());
	}

	@Override
	ClusterKey getParentInventoryUUIDKey(NetAppDefaultStorageContext context, String perfInstanceUUID) {
		return context.getKey();
	}

	@Override
	String getIventoryUUIDFromPerfUUID(NetAppDefaultStorageContext context, String perfInstanceUUID) {
		return context.getNfsv3VserverIDToUUIDMap().get(perfInstanceUUID);
	}

	@Override
	NetappNfsv3Metrics createMetricsFromValues(
			MetricsRowScratchPad<Nfsv3Counter> currentValues,
			MetricsRowScratchPad<Nfsv3Counter> previousValues) {
    	return new NetappNfsv3Metrics(currentValues, previousValues);
	}

	@Override
	void reportPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<Nfsv3Measurements>>> measurements) throws DataSourceErrorException {
		for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<Nfsv3Measurements>>> measurementEntry : measurements.entrySet()) {
			final Cluster.ClusterKey key = measurementEntry.getKey();
			final Collection<RawSample<Nfsv3Measurements>> nfsv3Measurements = measurementEntry.getValue();
			storagePerformanceReporter.reportNfsv3Performance(key, nfsv3Measurements);
		}
	}
	
    @Override
    void reportRawPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<Nfsv3RawMeasurements>>> measurements) throws DataSourceErrorException {
    	for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<Nfsv3RawMeasurements>>> measurementEntry : measurements.entrySet()) {
    		final Cluster.ClusterKey key = measurementEntry.getKey();
    		final Collection<RawSample<Nfsv3RawMeasurements>> nfsv3Measurements = measurementEntry.getValue();
    		storagePerformanceReporter.reportNfsv3RawPerformance(key, nfsv3Measurements);
    	}
    }

}
