package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;

import com.netapp.oci.netapp.performance.nfs.interfaces.data.Nfsv41Measurements;
import com.netapp.oci.netapp.performance.nfs.interfaces.data.Nfsv41RawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappNfsv41Metrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappNfsv41Metrics.Nfsv41Counter;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster.ClusterKey;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

public class Nfsv41MetricsBuilder extends MetricsBuilderBase<Cluster.ClusterKey,
		Nfsv41Measurements,
		Nfsv41RawMeasurements,
		NetappNfsv41Metrics,
		NetappNfsv41Metrics.Nfsv41Counter,
		NetappNfsv41Metrics.Nfsv41Counter>{

	public Nfsv41MetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage, "nfsv4_1", NetappNfsv41Metrics.Nfsv41Counter.class);
    }

    @Override
	List<String> getTargetPerfUUIDs(NetAppDefaultStorageContext context, ZAPIConnection zapiConnection) {
		return new ArrayList<String>(context.getNfsv41VserverIDToUUIDMap().keySet());
	}

	@Override
	ClusterKey getParentInventoryUUIDKey(NetAppDefaultStorageContext context, String perfInstanceUUID) {
		return context.getKey();
	}

	@Override
	String getIventoryUUIDFromPerfUUID(NetAppDefaultStorageContext context, String perfInstanceUUID) {
		return context.getNfsv41VserverIDToUUIDMap().get(perfInstanceUUID);
	}

	@Override
	NetappNfsv41Metrics createMetricsFromValues(
			MetricsRowScratchPad<Nfsv41Counter> currentValues,
			MetricsRowScratchPad<Nfsv41Counter> previousValues) {
		return new NetappNfsv41Metrics(currentValues, previousValues);	
	}

	@Override
    void reportPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<Nfsv41Measurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<Nfsv41Measurements>>> measurementEntry : measurements.entrySet()) {
            final Cluster.ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<Nfsv41Measurements>> nfsv41Measurements = measurementEntry.getValue();
            storagePerformanceReporter.reportNfsv41Performance(key, nfsv41Measurements);
        }
    }

	@Override
	void reportRawPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<Nfsv41RawMeasurements>>> measurements) throws DataSourceErrorException {
		for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<Nfsv41RawMeasurements>>> measurementEntry : measurements.entrySet()) {
			final Cluster.ClusterKey key = measurementEntry.getKey();
			final Collection<RawSample<Nfsv41RawMeasurements>> nfsv41Measurements = measurementEntry.getValue();
			storagePerformanceReporter.reportNfsv41RawPerformance(key, nfsv41Measurements);
		}
	}

}
