package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;

import java.util.Collection;
import java.util.List;
import java.util.Map;

import com.netapp.oci.netapp.performance.nic.interfaces.data.NicMeasurements;
import com.netapp.oci.netapp.performance.nic.interfaces.data.NicRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappNicCommonMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappNicCommonMetrics.NicCommonCounter;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.framework.datasource.config.PackageConfiguration;
import com.onaro.sanscreen.acquisition.framework.recordings.RecordingSession;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster.ClusterKey;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppStoragePerformanceReporter;

public class NicCommonMetricsBuilder extends MetricsBuilderBase<Cluster.ClusterKey,
        NicMeasurements,
        NicRawMeasurements,
        NetappNicCommonMetrics,
        NetappNicCommonMetrics.NicCommonCounter,
        NetappNicCommonMetrics.NicCommonCounter>{

    public NicCommonMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage,"nic_common", NetappNicCommonMetrics.NicCommonCounter.class);
    }

    @Override
    List<String> getTargetPerfUUIDs(final NetAppDefaultStorageContext context, final ZAPIConnection zapiConnection) {
        return context.getIpPortPerfUUIDs();
    }

    @Override
    Cluster.ClusterKey getParentInventoryUUIDKey(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return context.getKey();
    }

    @Override
    String getIventoryUUIDFromPerfUUID(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return context.getIpPortPerfUUIDToUUIDMap().get(perfInstanceUUID);
    }

    @Override
    NetappNicCommonMetrics createMetricsFromValues(
            MetricsRowScratchPad<NicCommonCounter> currentValues,
            MetricsRowScratchPad<NicCommonCounter> previousValues) {
        return new NetappNicCommonMetrics(currentValues, previousValues);
    }

    @Override
    void reportPerformance(ClusterKey clusterKey, Map<ClusterKey, Collection<RawSample<NicMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<NicMeasurements>>> measurementEntry : measurements.entrySet()) {
            final Cluster.ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<NicMeasurements>> nicMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportNicPerformance(key, nicMeasurements);
        }
    }

    @Override
    void reportRawPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<NicRawMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<NicRawMeasurements>>> measurementEntry : measurements.entrySet()) {
            final Cluster.ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<NicRawMeasurements>> nicRawMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportNicRawPerformance(key, nicRawMeasurements);
        }
    }
}
