package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;

import com.netapp.oci.netapp.performance.node.interfaces.data.NodeMeasurements;
import com.netapp.oci.netapp.performance.node.interfaces.data.NodeRawMeasurements;
import com.netapp.oci.netapp.performance.qosworkloaddetail.interfaces.data.QosWorkloadDetailMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappNodeMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.framework.datasource.config.PackageConfiguration;
import com.onaro.sanscreen.acquisition.framework.recordings.RecordingSession;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppStoragePerformanceReporter;
import org.apache.log4j.Logger;

import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class NodeMetricsBuilder extends MetricsBuilderBase<Cluster.ClusterKey,
        NodeMeasurements,
        NodeRawMeasurements,
        NetappNodeMetrics,
        NetappNodeMetrics.SystemCounter,
        NetappNodeMetrics.SystemCounter> {

    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger logger = Logger.getLogger(NodeMetricsBuilder.class);
    
    private Map<String, NodeMeasurements> nodeMeasurementsMap;

    public NodeMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage, NetappNodeMetrics.SystemCounter.class);
        this.nodeMeasurementsMap = new HashMap<String, NodeMeasurements>();
    }

    @Override
    protected String getObjectName(final NetAppDefaultStorageContext context){
        return context.getSystemObjectName();
    }

    @Override
    List<String> getTargetPerfUUIDs(final NetAppDefaultStorageContext context, final ZAPIConnection connection) {
        return context.getSystemPerfUUIDs();
    }

    @Override
    Cluster.ClusterKey getParentInventoryUUIDKey(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return context.getKey();
    }

    @Override
    String getIventoryUUIDFromPerfUUID(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return context.getSystemPerfUUIDtoNodeUUID().get(perfInstanceUUID);
    }

    @Override
    NetappNodeMetrics createMetricsFromValues(final MetricsRowScratchPad<NetappNodeMetrics.SystemCounter> currentValues, final MetricsRowScratchPad<NetappNodeMetrics.SystemCounter> previousValues) {
        return new NetappNodeMetrics(currentValues, previousValues, netAppOCIEPerformancePackage);
    }

    @Override
    void reportPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<NodeMeasurements>>> measurements) throws DataSourceErrorException {
        
    	nodeMeasurementsMap.clear();
    	for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<NodeMeasurements>>> measurementEntry : measurements.entrySet()) {
            final Cluster.ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<NodeMeasurements>> nodeMeasurements = measurementEntry.getValue();
            for(RawSample<NodeMeasurements> nodeMeasurement : nodeMeasurements){
            	nodeMeasurementsMap.put(nodeMeasurement.getKey(), nodeMeasurement.getCounters());
            }
            storagePerformanceReporter.reportNodePerformance(key, nodeMeasurements);
        }
    }

    @Override
    void reportRawPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<NodeRawMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<NodeRawMeasurements>>> measurementEntry : measurements.entrySet()) {
            final Cluster.ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<NodeRawMeasurements>> nodeMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportNodeRawPerformance(key, nodeMeasurements);
        }
    }
    
    public Map<String, NodeMeasurements> getNodeMeasurements(){
    	return nodeMeasurementsMap;
    }

}
