package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;

import com.netapp.oci.netapp.performance.processor.interfaces.data.ProcessorMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappProcessorMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

public class ProcessorMetricsBuilder extends MetricsBuilderBase<Cluster.ClusterKey,
                                                                ProcessorMeasurements,
                                                                ProcessorMeasurements,
                                                                NetappProcessorMetrics,
                                                                NetappProcessorMetrics.ProcessorCounter,
                                                                NetappProcessorMetrics.OCIEProcessorCounter> {

    private static final Logger logger = Logger.getLogger(ProcessorMetricsBuilder.class);

    public ProcessorMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage, "processor", NetappProcessorMetrics.ProcessorCounter.class);
    }

    @Override
    List<String> getTargetPerfUUIDs(final NetAppDefaultStorageContext context, final ZAPIConnection zapiConnection) {
        return context.getProcessorPerfUUIDs();
    }

    @Override
    Cluster.ClusterKey getParentInventoryUUIDKey(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return context.getKey();
    }

    @Override
    String getIventoryUUIDFromPerfUUID(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        // Perf:        {NodeUUID}:kernel:processor{Num}
        // Inventory:   {NodeName}.{Num}

        final String[] tokens = perfInstanceUUID.split(":");
        final String nodeName = tokens[0];
        final String processorAndNum = tokens[2];

        final String nodeUUID = context.getNodeNameToUUIDMap().get(nodeName);
        final String processorNum = processorAndNum.substring("processor".length(), processorAndNum.length());

        return nodeUUID + "." + processorNum;
    }

    @Override
    NetappProcessorMetrics createMetricsFromValues(final MetricsRowScratchPad<NetappProcessorMetrics.ProcessorCounter> currentValues, final MetricsRowScratchPad<NetappProcessorMetrics.ProcessorCounter> previousValues) {
        return new NetappProcessorMetrics(currentValues, previousValues);      
    }

    @Override
    void reportPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<ProcessorMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<ProcessorMeasurements>>> measurementEntry : measurements.entrySet()) {
            final Cluster.ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<ProcessorMeasurements>> processorMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportProcessorPerformance(key, processorMeasurements);
        }
    }

    @Override
    protected MetricsRowScratchPad<NetappProcessorMetrics.ProcessorCounter> getMetricsScratchPad(final Class<NetappProcessorMetrics.ProcessorCounter> enumClass,
                                                                                                  final String perfInstanceUUID, final String inventoryUUID,
                                                                                                  final long serverPollStartTimeStamp, final long pollStartTimeStamp,
                                                                                                  final long zapiStartTimeStamp, final long zapiResponseTimestamp,
                                                                                                  final MetricsRowScratchPad<NetappProcessorMetrics.ProcessorCounter> previousValues) {
        final List<String> cpuColumnNames = getCPUColumnNamesFromContext();
        return new NetappProcessorMetrics.ProcessorMetricsRowScratchPad(perfInstanceUUID, inventoryUUID, serverPollStartTimeStamp,
                                                                        pollStartTimeStamp, previousValues, cpuColumnNames, metricsDebugEnabled);
    }


    private List<String>  getCPUColumnNamesFromContext() {
        final List<String> cpuColumnNames;
        final NetAppDefaultStorageContext context =((NetAppOCIEPerformancePackage)storagePerformanceReporter).getNetAppDefaultStorageContext();
        if (context != null) {
            final List<String> cpuColumnNameCandidates = context.getCpuColumnNames();
            cpuColumnNames = (cpuColumnNameCandidates != null)
                ? cpuColumnNameCandidates
                : new ArrayList<String>(0);
        } else {
            // No previous foundation poll
            cpuColumnNames = new ArrayList<String>(0);
        }
        return cpuColumnNames;
    }
}
