package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;


import com.netapp.oci.netapp.performance.node.interfaces.data.SMB1NodeMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappSMB1NodeMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.framework.datasource.config.PackageConfiguration;
import com.onaro.sanscreen.acquisition.framework.recordings.RecordingSession;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppStoragePerformanceReporter;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

public class SMB1NodeMetricsBuilder extends MetricsBuilderBase<Cluster.ClusterKey,
        SMB1NodeMeasurements,
        SMB1NodeMeasurements,
        NetappSMB1NodeMetrics,
        NetappSMB1NodeMetrics.SMB1NodeCounter,
        NetappSMB1NodeMetrics.SMB1NodeCounter> {


    public SMB1NodeMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage, "smb1:node", NetappSMB1NodeMetrics.SMB1NodeCounter.class);
    }

    @Override
    List<String> getTargetPerfUUIDs(final NetAppDefaultStorageContext context, final ZAPIConnection zapiConnection) {
        return new ArrayList<String>(context.getSmb1PerfUUIDtoNodeUUID().keySet());
    }

    @Override
    Cluster.ClusterKey getParentInventoryUUIDKey(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return context.getKey();        // Parent of each node is the cluster
    }

    @Override
    String getIventoryUUIDFromPerfUUID(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return context.getSmb1PerfUUIDtoNodeUUID().get(perfInstanceUUID);
    }

    @Override
    NetappSMB1NodeMetrics createMetricsFromValues(final MetricsRowScratchPad<NetappSMB1NodeMetrics.SMB1NodeCounter> currentValues,
                                                  final MetricsRowScratchPad<NetappSMB1NodeMetrics.SMB1NodeCounter> previousValues) {
        return new NetappSMB1NodeMetrics(currentValues, previousValues);
    }

    @Override
    void reportPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<SMB1NodeMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<SMB1NodeMeasurements>>> measurementEntry : measurements.entrySet()) {
            final Cluster.ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<SMB1NodeMeasurements>> nodeMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportSMB1NodePerformance(key, nodeMeasurements);
        }
    }
}
