package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;


import com.netapp.oci.netapp.performance.node.interfaces.data.SMB2NodeMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappSMB2NodeMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.framework.datasource.config.PackageConfiguration;
import com.onaro.sanscreen.acquisition.framework.recordings.RecordingSession;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppStoragePerformanceReporter;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

public class SMB2NodeMetricsBuilder extends MetricsBuilderBase<Cluster.ClusterKey,
        SMB2NodeMeasurements,
        SMB2NodeMeasurements,
        NetappSMB2NodeMetrics,
        NetappSMB2NodeMetrics.SMB2NodeCounter,
        NetappSMB2NodeMetrics.SMB2NodeCounter> {

    public SMB2NodeMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage, "s2:node", NetappSMB2NodeMetrics.SMB2NodeCounter.class);
    }

    @Override
    List<String> getTargetPerfUUIDs(final NetAppDefaultStorageContext context, final ZAPIConnection zapiConnection) {
        return new ArrayList<String>(context.getSmb2PerfUUIDtoNodeUUID().keySet());
    }

    @Override
    Cluster.ClusterKey getParentInventoryUUIDKey(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return context.getKey();        // Parent of each node is the cluster
    }

    @Override
    String getIventoryUUIDFromPerfUUID(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return context.getSmb2PerfUUIDtoNodeUUID().get(perfInstanceUUID);
    }

    @Override
    NetappSMB2NodeMetrics createMetricsFromValues(final MetricsRowScratchPad<NetappSMB2NodeMetrics.SMB2NodeCounter> currentValues,
                                                  final MetricsRowScratchPad<NetappSMB2NodeMetrics.SMB2NodeCounter> previousValues) {
        return new NetappSMB2NodeMetrics(currentValues, previousValues);
    }

    @Override
    void reportPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<SMB2NodeMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<SMB2NodeMeasurements>>> measurementEntry : measurements.entrySet()) {
            final Cluster.ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<SMB2NodeMeasurements>> nodeMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportSMB2NodePerformance(key, nodeMeasurements);
        }
    }
}
