package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;

import com.netapp.oci.netapp.performance.volumenode.interfaces.data.VolumeNodeMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappVolumeNodeMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.framework.datasource.config.PackageConfiguration;
import com.onaro.sanscreen.acquisition.framework.recordings.RecordingSession;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppStoragePerformanceReporter;
import org.apache.log4j.Logger;

import java.util.Collection;
import java.util.List;
import java.util.Map;

public class VolumeNodeMetricsBuilder extends MetricsBuilderBase<Cluster.ClusterKey,
        VolumeNodeMeasurements,
        VolumeNodeMeasurements,
        NetappVolumeNodeMetrics,
        NetappVolumeNodeMetrics.VolumeNodeCounter,
        NetappVolumeNodeMetrics.VolumeNodeCounter> {

    private static final Logger logger = Logger.getLogger(VolumeNodeMetricsBuilder.class);

    public VolumeNodeMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage, "volume:node", NetappVolumeNodeMetrics.VolumeNodeCounter.class);
    }

    @Override
    List<String> getTargetPerfUUIDs(final NetAppDefaultStorageContext context, final ZAPIConnection connection) {
        return context.getVolumeNodePerfUUIDs();
    }

    @Override
    NetappVolumeNodeMetrics createMetricsFromValues(final MetricsRowScratchPad<NetappVolumeNodeMetrics.VolumeNodeCounter> currentValues, final MetricsRowScratchPad<NetappVolumeNodeMetrics.VolumeNodeCounter> previousValues) {
        return new NetappVolumeNodeMetrics(currentValues, previousValues);
    }

    @Override
    void reportPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<VolumeNodeMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<VolumeNodeMeasurements>>> measurementEntry : measurements.entrySet()) {
            final Cluster.ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<VolumeNodeMeasurements>> volumeMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportVolumeNodePerformance(key, volumeMeasurements);
        }
    }

    @Override
    String getIventoryUUIDFromPerfUUID(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return perfInstanceUUID; // they are the same - the node UUID
    }

    @Override
    Cluster.ClusterKey getParentInventoryUUIDKey(final NetAppDefaultStorageContext context, final String perfGlobalUUID) {
        return context.getKey();
    }
}
