package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;

import com.netapp.oci.netapp.performance.vserver.interfaces.data.VolumeVserverMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappVolumeVserverMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.framework.datasource.PackageContext;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.VServer;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;
import org.apache.log4j.Logger;

import java.util.*;

public class VolumeVserverMetricsBuilder extends MetricsBuilderBase<Cluster.ClusterKey,
                                                              VolumeVserverMeasurements,
                                                              VolumeVserverMeasurements,
                                                              NetappVolumeVserverMetrics,
                                                              NetappVolumeVserverMetrics.VServerVolumeCounters,
                                                              NetappVolumeVserverMetrics.VServerVolumeCounters> {

    private static final Logger logger = Logger.getLogger(VolumeVserverMetricsBuilder.class);

    public VolumeVserverMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage, "volume:vserver", NetappVolumeVserverMetrics.VServerVolumeCounters.class);
    }

    @Override
    public void getAndReportPerformance(final NetAppDefaultStorageContext context, final ZAPIConnection zapiConnection) throws DataSourceErrorException {
        super.getAndReportPerformance(context, zapiConnection);
    }

    @Override
    List<String> getTargetPerfUUIDs(final NetAppDefaultStorageContext context, final ZAPIConnection zapiConnection) {
        return getVServerTargetUUIDs(context, zapiConnection);
    }

    static List<String> getVServerTargetUUIDs(final NetAppDefaultStorageContext context, final ZAPIConnection zapiConnection) {
        final List<String> vServerInstanceIDs = new LinkedList<String>();
        for (final String vServerUUID : context.getVServerUUIDs()) {
            final String vServerInstanceID = vServerUUID.substring(vServerUUID.indexOf(".") + 1);
            vServerInstanceIDs.add(vServerInstanceID);
        }
        return vServerInstanceIDs;
    }

    @Override
    Cluster.ClusterKey getParentInventoryUUIDKey(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return context.getKey();
    }

    @Override
    String getIventoryUUIDFromPerfUUID(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return new VServer.VServerKey(context.getKey().getOcieUUID(), perfInstanceUUID).getOcieUUID();
    }

    @Override
    NetappVolumeVserverMetrics createMetricsFromValues(final MetricsRowScratchPad<NetappVolumeVserverMetrics.VServerVolumeCounters> currentValues, final MetricsRowScratchPad<NetappVolumeVserverMetrics.VServerVolumeCounters> previousValues) {
        return new NetappVolumeVserverMetrics(currentValues, previousValues);
    }

    @Override
    void reportPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<VolumeVserverMeasurements>>> measurements) throws DataSourceErrorException {


        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<VolumeVserverMeasurements>>> measurementEntry : measurements.entrySet()) {
            final Cluster.ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<VolumeVserverMeasurements>> volumeVserverMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportVolumeVserverPerformance(key, volumeVserverMeasurements);
        }
    }
}
