package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappstorageperformance;

import com.netapp.oci.netapp.performance.wafl.interfaces.data.WaflMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappWaflMetrics;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.framework.datasource.config.PackageConfiguration;
import com.onaro.sanscreen.acquisition.framework.recordings.RecordingSession;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppStoragePerformanceReporter;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;

public class WaflMetricsBuilder extends MetricsBuilderBase<Cluster.ClusterKey,
        WaflMeasurements,
        WaflMeasurements,
        NetappWaflMetrics,
        NetappWaflMetrics.WaflCounter,
        NetappWaflMetrics.OCIEWaflCounter> {

    public WaflMetricsBuilder(final NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage, "wafl", NetappWaflMetrics.WaflCounter.class);
    }

    @Override
    List<String> getTargetPerfUUIDs(final NetAppDefaultStorageContext context, final ZAPIConnection connection) {
        return context.getWaflPerfUUIDs();
    }

    @Override
    void reportPerformance(Cluster.ClusterKey clusterKey, final Map<Cluster.ClusterKey, Collection<RawSample<WaflMeasurements>>> measurements) throws DataSourceErrorException {
        for (final Map.Entry<Cluster.ClusterKey, Collection<RawSample<WaflMeasurements>>> measurementEntry : measurements.entrySet()) {
            final Cluster.ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<WaflMeasurements>> waflMeasurements = measurementEntry.getValue();
            storagePerformanceReporter.reportWaflPerformance(key, waflMeasurements);
        }
    }

    @Override
    Cluster.ClusterKey getParentInventoryUUIDKey(final NetAppDefaultStorageContext context, final String perfGlobalUUID) {
        return context.getKey();
    }

    @Override
    String getIventoryUUIDFromPerfUUID(final NetAppDefaultStorageContext context, final String perfInstanceUUID) {
        return context.getWaflPerfUUIDtoNodeUUID().get(perfInstanceUUID);
    }

    @Override
    NetappWaflMetrics createMetricsFromValues(final MetricsRowScratchPad<NetappWaflMetrics.WaflCounter> currentValues, final MetricsRowScratchPad<NetappWaflMetrics.WaflCounter> previousValues) {
        return new NetappWaflMetrics(currentValues, previousValues);
    }

    @Override
    protected MetricsRowScratchPad<NetappWaflMetrics.WaflCounter> getMetricsScratchPad(final Class<NetappWaflMetrics.WaflCounter> enumClass,
                                                                                                  final String perfInstanceUUID, final String inventoryUUID,
                                                                                                  final long serverPollStartTimeStamp, final long pollStartTimeStamp,
                                                                                                  final long zapiStartTimeStamp, final long zapiResponseTimestamp,
                                                                                                  final MetricsRowScratchPad<NetappWaflMetrics.WaflCounter> previousValues) {
    	List<String> waflCpPhaseTimesColumnNames = getWaflCpPhaseTimesColumnNamesFromContext();
        return new NetappWaflMetrics.WaflMetricsRowScratchPad(perfInstanceUUID, inventoryUUID, serverPollStartTimeStamp,
                pollStartTimeStamp, previousValues, waflCpPhaseTimesColumnNames, metricsDebugEnabled);
    }
    
    private List<String>  getWaflCpPhaseTimesColumnNamesFromContext() {
        final List<String> waflCpPhaseTimesColumnNames;
        final NetAppDefaultStorageContext context =((NetAppOCIEPerformancePackage)storagePerformanceReporter).getNetAppDefaultStorageContext();
        if (context != null) {
            final List<String> waflCpPhaseTimesColumnNameCandidates = context.getWaflCpPhaseTimesColumnNames();
            waflCpPhaseTimesColumnNames = (List<String>) ((waflCpPhaseTimesColumnNameCandidates != null)
                ? waflCpPhaseTimesColumnNameCandidates
                : Collections.emptyList());
        } else {
            // No previous foundation poll
        	waflCpPhaseTimesColumnNames = Collections.emptyList();
        }
        return waflCpPhaseTimesColumnNames;
    }
}
