package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.config;

/**
 * Common configuration for NetApp Filer/ZAPI data sources
 */
public interface NetAppCommonConfig {

    /**
     * IP address of target cluster's CLUSTER_MGMT lif
     * @return  IP address of target cluster's CLUSTER_MGMT lif
     */
    public String getIP();

    /**
     * ZAPI user name
     * @return ZAPI user name
     */
    public String getUser();

    /**
     * ZAPI password
     * @return ZAPI password
     */
    public String getPassword();

    /**
     * Get the port to use to connect to the filer
     *
     * @return the port to use to connect to the filer
     */
    public int getPort();

    /**
     * Get the type of connection to use (HTTP or HTTPS)
     *
     * @return the type of connection to use
     */
    public String getConnectionType();

    /**
     * Get max number of ZAPI retry invocations before we decide the ZAPI is failed
     * @return the max number of ZAPI retry invocations before we decide the ZAPI is failed
     */
    public int getMaxRetries();

    /**
     * Mandatory property
     * @return Mandatory property
     */
    public int getPollTimeMillis();

    /**
     * Whether or not snapshot discovery is enabled
     * @return whether or not snapshot discovery is enabled
     */
    public boolean isSnapshotDiscoveryEnabled();

    /**
     * Whether or not QOS discovery is enabled
     * @return hether or not QOS discovery is enabled
     */
    public boolean isQOSDiscoveryEnabled();

    /**
     * Get ZAPI timeout in seconds
     * @return ZAPI timeout in seconds
     */
    public int getTimeout();

    /**
     * Whether or not the storing of cooked counters is enabled.
     * @return true if enabled, false if not
     */
    public boolean isCookedStatisticsPersistenceEnabled();

    /**
     * Whether or not the storing of raw counters is enabled.
     * @return true if enabled, false if not
     */
    public boolean isRawStatisticsPersistenceEnabled();

}
