package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.config;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.framework.FrameworkException;
import com.onaro.sanscreen.acquisition.framework.datasource.config.DataSourceConfiguration;
import com.onaro.sanscreen.acquisition.framework.datasource.metadata.PackageMetaData;
import com.onaro.sanscreen.acquisition.netappfoundation.datasource.NetAppFoundationPackage;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Licenses;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.config.NetAppStoragePerformanceConfiguration;

import java.util.Properties;

public class NetAppOCIEPerformanceConfiguration extends NetAppStoragePerformanceConfiguration<NetAppOCIEPerformancePackage> implements NetAppCommonConfig {

    /**
     * The port to use to connect to the filer
     */
    private final int port;

    /**
     * The type of connection to use
     */
    private final String connectionType;

    /**
     * Whether or not snapshot discovery is enabled
     */
    private final boolean isSnapshotDiscoveryEnabled;

    /**
     * Whether or not QOS discovery/metrics are enabled
     */
    private final boolean isQOSDiscoveryEnabled;

    /**
     * Whether or not this datasource should store cooked data.
     */
    private final boolean isCookedStatisticsPersistenceEnabled;

    /**
     * Whether or not this datasource should store raw data.
     */
    private final boolean isRawStatisticsPersistenceEnabled;

    /**
     * Max ZAPI retries
     */
    private final int maxRetries;

    /**
     * Toggles generation of metrics debug CSV files
     */
    private final boolean metricsDebugEnabled;

    /**
     * ZAPI timeout in seconds
     */
    private final int timeoutSeconds;

    public NetAppOCIEPerformanceConfiguration(final DataSourceConfiguration dataSourceConfiguration, final PackageMetaData metaData, final String dataSourceName, final long dataSourceID, final Properties props) throws FrameworkException {
        super(dataSourceConfiguration, metaData, dataSourceName, dataSourceID, props);    //To change body of overridden methods use File | Settings | File Templates.
        if (!getDataSourceConfiguration().isPackageRegistered(NetAppFoundationPackage.PACKAGE)) {
            throw new FrameworkException("NetApp Foundation package is not registered !");
        }
        final Properties foundationProps = getNetAppFoundationConfiguration().getProperties();
        port = Integer.parseInt(foundationProps.getProperty("port", "80"));
        isSnapshotDiscoveryEnabled = Boolean.parseBoolean(properties.getProperty("snapshot.enabled", "false"));
        isQOSDiscoveryEnabled = Boolean.parseBoolean(properties.getProperty("qos.enabled", "false"));
        connectionType = foundationProps.getProperty("connectionType", "HTTP");
        maxRetries = Integer.parseInt(properties.getProperty("maxRetries", "5"));
        metricsDebugEnabled = Boolean.parseBoolean(properties.getProperty("metricsDebug", "true"));
        timeoutSeconds = Integer.parseInt(foundationProps.getProperty("timeout", "300"));
        isCookedStatisticsPersistenceEnabled = Boolean.parseBoolean(properties.getProperty("cookedpersistence.enabled", "true"));
        isRawStatisticsPersistenceEnabled = Boolean.parseBoolean(properties.getProperty("rawpersistence.enabled", "false"));
    }

    @Override
    public String getIP() {
        return getNetAppFoundationConfiguration().getIP();
    }

    @Override
    public String getUser() {
        return getNetAppFoundationConfiguration().getUser();
    }

    @Override
    public String getPassword() {
        return getNetAppFoundationConfiguration().getPassword();
    }

    @Override
    public String getConnectionType() {
        return connectionType;
    }

    @Override
    public int getPort() {
        return port;
    }

    @Override
    public int getMaxRetries() {
        return maxRetries;
    }

    /**
     * whether or not metrics debug CSV file generation is enabled
     * @return whether or not metrics debug CSV file generation is enabled
     */
    public boolean isMetricsDebugEnabled() {
        return metricsDebugEnabled;
    }

    @Override
    public boolean isSnapshotDiscoveryEnabled() {
        return isSnapshotDiscoveryEnabled;
    }

    @Override
    public boolean isQOSDiscoveryEnabled() {
        return isQOSDiscoveryEnabled;
    }

    @Override
    public int getTimeout() {
        return timeoutSeconds;
    }
    
    @Override
    public boolean isCookedStatisticsPersistenceEnabled() {
        return isCookedStatisticsPersistenceEnabled;
    }

    @Override
    public boolean isRawStatisticsPersistenceEnabled() {
        return isRawStatisticsPersistenceEnabled;
    }
}
