package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics;

import com.netapp.oci.netapp.performance.iscsilif.interfaces.data.IscsiLifMeasurements;
import com.netapp.oci.netapp.performance.iscsilif.interfaces.data.IscsiLifRawMeasurements;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import java.math.BigInteger;
import java.util.Map;

import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetAppISCSILIFMetrics.ISCSILIFCounter.*;

public class NetAppISCSILIFMetrics extends MetricsRowSample<IscsiLifMeasurements, IscsiLifRawMeasurements, NetAppISCSILIFMetrics.ISCSILIFCounter,
                                                                                  NetAppISCSILIFMetrics.ISCSILIFCounter> {

    public static enum ISCSILIFCounter {
        data_in_blocks,
        data_out_blocks,
        iscsi_read_ops,
        iscsi_write_ops,
        protocol_errors,
        avg_latency,
        avg_other_latency,
        avg_read_latency,
        avg_write_latency,
        iscsi_other_ops,
        read_data,
        write_data,
        cmd_transfered
    }

    public NetAppISCSILIFMetrics(final MetricsRowScratchPad<ISCSILIFCounter> currentValues, final MetricsRowScratchPad<ISCSILIFCounter> previousValues) {
        super(ISCSILIFCounter.class, currentValues, previousValues);
    }

    ISCSILIFCounter getDenominatorForNumerator(final ISCSILIFCounter numeratorName) {
        final ISCSILIFCounter denominatorName;
        switch (numeratorName) {
            case avg_latency:
                denominatorName = cmd_transfered;
                break;
            case avg_other_latency:
    			denominatorName = iscsi_other_ops;
    			break;
    		case avg_read_latency:
    			denominatorName = iscsi_read_ops;
    			break;
    		case avg_write_latency:
    			denominatorName = iscsi_write_ops;
    			break;
            default:
                denominatorName = null;
        }

        return denominatorName;
    }
    
    @Override
    float getNormalizedFloatMetric(final ISCSILIFCounter counterName) {
        final float normalizedMetric;
        switch (counterName) {
            case iscsi_read_ops:
            case iscsi_write_ops:
            case iscsi_other_ops:
            case protocol_errors:
            case read_data:
            case write_data:
                normalizedMetric = currentValues.getCounterRatePerSec(counterName);
                break;
            case avg_latency:
                normalizedMetric = currentValues.getLatencyPerOp(counterName, getDenominatorForNumerator(counterName));
            	break;
            case avg_other_latency:
				normalizedMetric = currentValues.getLatencyPerOp(counterName, getDenominatorForNumerator(counterName));
				break;
			case avg_read_latency:
				normalizedMetric = currentValues.getLatencyPerOp(counterName, getDenominatorForNumerator(counterName));
				break;
			case avg_write_latency:
				normalizedMetric = currentValues.getLatencyPerOp(counterName, getDenominatorForNumerator(counterName));
				break;
            default:
                normalizedMetric = NULL_FLOAT;
        }
        return normalizedMetric;
    }

    @Override
    public IscsiLifMeasurements normalizeMetrics(final NetAppDefaultStorageContext context) {
        final Map<ISCSILIFCounter,BigInteger> counterValues = getCurrentValues().getCounterValues();
        final IscsiLifMeasurements iScsiLifMeasurements = new IscsiLifMeasurements();
        iScsiLifMeasurements.setDataInBlocks(currentValues.getCounterDelta(data_in_blocks));
        iScsiLifMeasurements.setDataOutBlocks(currentValues.getCounterDelta(data_out_blocks));
        iScsiLifMeasurements.setIscsiReadOps(getNormalizedFloatMetric(iscsi_read_ops));
        iScsiLifMeasurements.setIscsiWriteOps(getNormalizedFloatMetric(iscsi_write_ops));
        iScsiLifMeasurements.setProtocolErrors(getNormalizedFloatMetric(protocol_errors));
        iScsiLifMeasurements.setReadData(getNormalizedFloatMetric(read_data));
        iScsiLifMeasurements.setWriteData(getNormalizedFloatMetric(write_data));
        iScsiLifMeasurements.setCmdTransfered(currentValues.getCounterDelta(cmd_transfered));
        iScsiLifMeasurements.setAvgLatency(getNormalizedFloatMetric(avg_latency));
        iScsiLifMeasurements.setAvgOtherLatency(getNormalizedFloatMetric(avg_other_latency));
        iScsiLifMeasurements.setAvgReadLatency(getNormalizedFloatMetric(avg_read_latency));
        iScsiLifMeasurements.setAvgWriteLatency(getNormalizedFloatMetric(avg_write_latency));
        iScsiLifMeasurements.setIscsiOtherOps(getNormalizedFloatMetric(iscsi_other_ops));
        iScsiLifMeasurements.setOps(MetricsRowScratchPad.add(iScsiLifMeasurements.getIscsiReadOps(),
                iScsiLifMeasurements.getIscsiWriteOps(), iScsiLifMeasurements.getIscsiOtherOps()));
        iScsiLifMeasurements.setThroughput(MetricsRowScratchPad.add(iScsiLifMeasurements.getReadData(),
                iScsiLifMeasurements.getWriteData()));
        return iScsiLifMeasurements;
    }
    
    @Override
    public IscsiLifRawMeasurements rawMetrics(final NetAppDefaultStorageContext context) {
        Map<ISCSILIFCounter,BigInteger> counterValues = getCurrentValues().getCounterValues();
        IscsiLifRawMeasurements iscsiLifRawMeasurements = new IscsiLifRawMeasurements();
        iscsiLifRawMeasurements .setAvgLatency(counterValues.get(avg_latency));
        iscsiLifRawMeasurements.setAvgOtherLatency(counterValues.get(avg_other_latency));
        iscsiLifRawMeasurements.setAvgReadLatency(counterValues.get(avg_read_latency));
        iscsiLifRawMeasurements.setAvgWriteLatency(counterValues.get(avg_write_latency));
        iscsiLifRawMeasurements.setIscsiOtherOps(counterValues.get(iscsi_other_ops));
        iscsiLifRawMeasurements.setIscsiReadOps(counterValues.get(iscsi_read_ops));
        iscsiLifRawMeasurements.setIscsiWriteOps(counterValues.get(iscsi_write_ops));
        iscsiLifRawMeasurements.setDataInBlocks(counterValues.get(data_in_blocks));
        iscsiLifRawMeasurements.setReadData(counterValues.get(read_data));
        iscsiLifRawMeasurements.setDataOutBlocks(counterValues.get(data_out_blocks));
        iscsiLifRawMeasurements.setWriteData(counterValues.get(write_data));
        iscsiLifRawMeasurements.setCmdTransfered(counterValues.get(cmd_transfered));
        iscsiLifRawMeasurements.setOps(MetricsRowScratchPad.add(iscsiLifRawMeasurements.getIscsiOtherOps(),
                iscsiLifRawMeasurements.getIscsiReadOps(), iscsiLifRawMeasurements.getIscsiWriteOps()));
        iscsiLifRawMeasurements.setThroughput(MetricsRowScratchPad.add(iscsiLifRawMeasurements.getReadData(),
                iscsiLifRawMeasurements.getWriteData()));
        return iscsiLifRawMeasurements;
    }
}
