package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics;

import com.netapp.oci.netapp.performance.node.interfaces.data.CIFSNodeMeasurements;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;
import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappCIFSNodeMetrics.CIFSNodeCounter.*;

public class NetappCIFSNodeMetrics extends MetricsRowSample<CIFSNodeMeasurements, CIFSNodeMeasurements, NetappCIFSNodeMetrics.CIFSNodeCounter,
        NetappCIFSNodeMetrics.CIFSNodeCounter>{

    public static enum CIFSNodeCounter {
        cifs_latency_base,
        cifs_latency,
        cifs_read_latency,
        cifs_write_latency,
        cifs_ops,
        cifs_read_ops,
        cifs_write_ops
    }

    public NetappCIFSNodeMetrics(final MetricsRowScratchPad<CIFSNodeCounter> currentValues, final MetricsRowScratchPad<CIFSNodeCounter> previousValues) {
        super(CIFSNodeCounter.class, currentValues, previousValues);
    }

    @Override
    float getNormalizedFloatMetric(final CIFSNodeCounter counterName) {
        final float normalizedMetric;
        switch (counterName) {
            case cifs_ops:
            case cifs_read_ops:
            case cifs_write_ops:
                normalizedMetric = currentValues.getCounterRatePerSec(counterName);
                break;

            case cifs_latency:
            case cifs_read_latency:
            case cifs_write_latency:
                normalizedMetric = currentValues.getLatencyPerOp(counterName,
                        getDenominatorForNumerator(counterName));
                break;

            default:
                normalizedMetric = NULL_FLOAT;
        }
        return normalizedMetric;
    }

    @Override
    CIFSNodeCounter getDenominatorForNumerator(final CIFSNodeCounter numeratorName) {
        final CIFSNodeCounter denominatorName;
        switch (numeratorName) {

            case cifs_latency:
                denominatorName = cifs_latency_base;
                break;

            case cifs_read_latency:
                denominatorName = cifs_read_ops;
                break;

            case cifs_write_latency:
                denominatorName = cifs_write_ops;
                break;

            default:
                denominatorName = null;
        }
        return denominatorName;
    }

    @Override
    public CIFSNodeMeasurements normalizeMetrics(final NetAppDefaultStorageContext context) {
        final CIFSNodeMeasurements cifsNodeMeasurements = new CIFSNodeMeasurements();
        cifsNodeMeasurements.setCifsLatency(getNormalizedFloatMetric(cifs_latency));
        cifsNodeMeasurements.setCifsReadLatency(getNormalizedFloatMetric(cifs_read_latency));
        cifsNodeMeasurements.setCifsWriteLatency(getNormalizedFloatMetric(cifs_write_latency));
        cifsNodeMeasurements.setCifsOps(getNormalizedFloatMetric(cifs_ops));
        cifsNodeMeasurements.setCifsReadOps(getNormalizedFloatMetric(cifs_read_ops));
        cifsNodeMeasurements.setCifsWriteOps(getNormalizedFloatMetric(cifs_write_ops));
        return cifsNodeMeasurements;
    }
}
