package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics;

import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifMeasurements;
import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifRawMeasurements;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import java.math.BigInteger;
import java.util.Map;

import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappFcpLifMetrics.FcpLifCounter.*;

public class NetappFcpLifMetrics extends MetricsRowSample<FcpLifMeasurements, 
                                                          FcpLifRawMeasurements, 
                                                          NetappFcpLifMetrics.FcpLifCounter,    // raw
                                                          NetappFcpLifMetrics.FcpLifCounter> {  // final

    public static enum FcpLifCounter {
        avg_latency,
        total_ops,
        avg_other_latency,
        other_ops,
        avg_read_latency,
        read_data,
        read_ops,
        avg_write_latency,
        write_data,
        write_ops 
    }

    public NetappFcpLifMetrics(final MetricsRowScratchPad<FcpLifCounter> currentValues, final MetricsRowScratchPad<FcpLifCounter> previousValues) {
        super(FcpLifCounter.class, currentValues, previousValues);
    }

    FcpLifCounter getDenominatorForNumerator(final FcpLifCounter numeratorName) {
        final FcpLifCounter denominatorName;
        switch (numeratorName) {
            case avg_latency:
                denominatorName = total_ops;
                break;
            case avg_other_latency:
                denominatorName = other_ops;
                break;
            case avg_read_latency:
                denominatorName = read_ops;
                break;
            case avg_write_latency:
                denominatorName = write_ops;
                break;
            default:
                denominatorName = null;
        }

        return denominatorName;
    }

    @Override
    float getNormalizedFloatMetric(final FcpLifCounter counterName) {
        final float normalizedMetric;
        switch (counterName) {
            case read_data:
            case write_data:
            case total_ops:
            case other_ops:
            case read_ops:
            case write_ops:
                normalizedMetric = currentValues.getCounterRatePerSec(counterName);
                break;
            case avg_latency:
            case avg_other_latency:
            case avg_read_latency:
            case avg_write_latency:
                normalizedMetric = currentValues.getLatencyPerOp(counterName,
                                                                 getDenominatorForNumerator(counterName));
                break;
            default:
                normalizedMetric = NULL_FLOAT;
        }
        return normalizedMetric;
    }

    @Override
    public FcpLifMeasurements normalizeMetrics(final NetAppDefaultStorageContext context) {
        final FcpLifMeasurements m = new FcpLifMeasurements();
        m.setAvgLatency(getNormalizedFloatMetric(avg_latency));
        m.setAvgOtherLatency(getNormalizedFloatMetric(avg_other_latency));
        m.setAvgReadLatency(getNormalizedFloatMetric(avg_read_latency));
        m.setAvgWriteLatency(getNormalizedFloatMetric(avg_write_latency));
        m.setTotalOps(getNormalizedFloatMetric(total_ops));
        m.setOtherOps(getNormalizedFloatMetric(other_ops));
        m.setReadData(getNormalizedFloatMetric(read_data));
        m.setReadOps(getNormalizedFloatMetric(read_ops));
        m.setWriteData(getNormalizedFloatMetric(write_data));
        m.setWriteOps(getNormalizedFloatMetric(write_ops));
        m.setThroughput(MetricsRowScratchPad.add(m.getReadData(), m.getWriteData()));
        return m;
    }

    @Override
    public FcpLifRawMeasurements rawMetrics(final NetAppDefaultStorageContext context) {
        Map<FcpLifCounter, BigInteger> counterValues = getCurrentValues().getCounterValues();
        return createFcpLifRawMeasurements( counterValues );
    }

    public static FcpLifRawMeasurements createFcpLifRawMeasurements( Map<FcpLifCounter, BigInteger> counterValues ) {
        final FcpLifRawMeasurements m = new FcpLifRawMeasurements();
        m.setAvgLatency(counterValues.get(avg_latency));
        m.setTotalOps(counterValues.get(total_ops));
        m.setAvgOtherLatency(counterValues.get(avg_other_latency));
        m.setOtherOps(counterValues.get(other_ops));
        m.setReadData(counterValues.get(read_data));
        m.setAvgReadLatency(counterValues.get(avg_read_latency));
        m.setReadOps(counterValues.get(read_ops));
        m.setWriteData(counterValues.get(write_data));
        m.setAvgWriteLatency(counterValues.get(avg_write_latency));
        m.setWriteOps(counterValues.get(write_ops));
        m.setThroughput(MetricsRowScratchPad.add(m.getReadData(), m.getWriteData()));
        return m;
    }
}
