package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics;

import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifVserverMeasurements;
import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifVserverRawMeasurements;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import java.math.BigInteger;
import java.util.Map;

import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappFcpLifVserverMetrics.FcpLifVserverCounter.*;

public class NetappFcpLifVserverMetrics extends MetricsRowSample<FcpLifVserverMeasurements, 
	FcpLifVserverRawMeasurements, 
	NetappFcpLifVserverMetrics.FcpLifVserverCounter,    // raw
	NetappFcpLifVserverMetrics.FcpLifVserverCounter> {  // final

	public static enum FcpLifVserverCounter {
		avg_latency,
		total_ops,
		avg_other_latency,
		other_ops,
		avg_read_latency,
		read_data,
		read_ops,
		avg_write_latency,
		write_data,
		write_ops
	}

	public NetappFcpLifVserverMetrics(final MetricsRowScratchPad<FcpLifVserverCounter> currentValues, final MetricsRowScratchPad<FcpLifVserverCounter> previousValues) {
		super(FcpLifVserverCounter.class, currentValues, previousValues);
	}

	FcpLifVserverCounter getDenominatorForNumerator(final FcpLifVserverCounter numeratorName) {
		final FcpLifVserverCounter denominatorName;
		switch (numeratorName) {
		case avg_latency:
			denominatorName = total_ops;
			break;
		case avg_other_latency:
			denominatorName = other_ops;
			break;
		case avg_read_latency:
			denominatorName = read_ops;
			break;
		case avg_write_latency:
			denominatorName = write_ops;
			break;
		default:
			denominatorName = null;
		}

		return denominatorName;
	}

	@Override
	float getNormalizedFloatMetric(final FcpLifVserverCounter counterName) {
		final float normalizedMetric;
		switch (counterName) {
		case read_data:
		case write_data:
		case total_ops:
		case other_ops:
		case read_ops:
		case write_ops:
			normalizedMetric = currentValues.getCounterRatePerSec(counterName);
			break;
		case avg_latency:
		case avg_other_latency:
		case avg_read_latency:
		case avg_write_latency:
			normalizedMetric = currentValues.getLatencyPerOp(counterName,
					getDenominatorForNumerator(counterName));
			break;
		default:
			normalizedMetric = NULL_FLOAT;
		}
		return normalizedMetric;
	}

	@Override
	public FcpLifVserverMeasurements normalizeMetrics(final NetAppDefaultStorageContext context) {
		final FcpLifVserverMeasurements m = new FcpLifVserverMeasurements();
		m.setAvgLatency(getNormalizedFloatMetric(avg_latency));
		m.setAvgOtherLatency(getNormalizedFloatMetric(avg_other_latency));
		m.setAvgReadLatency(getNormalizedFloatMetric(avg_read_latency));
		m.setAvgWriteLatency(getNormalizedFloatMetric(avg_write_latency));
		m.setTotalOps(getNormalizedFloatMetric(total_ops));
		m.setOtherOps(getNormalizedFloatMetric(other_ops));
		m.setReadData(getNormalizedFloatMetric(read_data));
		m.setReadOps(getNormalizedFloatMetric(read_ops));
		m.setWriteData(getNormalizedFloatMetric(write_data));
		m.setWriteOps(getNormalizedFloatMetric(write_ops));
		m.setThroughput(MetricsRowScratchPad.add(m.getReadData(), m.getWriteData()));
		return m;
	}

	@Override
	public FcpLifVserverRawMeasurements rawMetrics(final NetAppDefaultStorageContext context) {
		Map<FcpLifVserverCounter, BigInteger> counterValues = getCurrentValues().getCounterValues();
		return createFcpLifVserverRawMeasurements( counterValues );
	}

	public static FcpLifVserverRawMeasurements createFcpLifVserverRawMeasurements( Map<FcpLifVserverCounter, BigInteger> counterValues ) {
		final FcpLifVserverRawMeasurements m = new FcpLifVserverRawMeasurements();
		m.setAvgLatency(counterValues.get(avg_latency));
		m.setTotalOps(counterValues.get(total_ops));
		m.setAvgOtherLatency(counterValues.get(avg_other_latency));
		m.setOtherOps(counterValues.get(other_ops));
		m.setReadData(counterValues.get(read_data));
		m.setAvgReadLatency(counterValues.get(avg_read_latency));
		m.setReadOps(counterValues.get(read_ops));
		m.setWriteData(counterValues.get(write_data));
		m.setAvgWriteLatency(counterValues.get(avg_write_latency));
		m.setWriteOps(counterValues.get(write_ops));
		m.setThroughput(MetricsRowScratchPad.add(m.getReadData(), m.getWriteData()));
		return m;
	}



}
