package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics;

import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappLifVserverMetrics.LifVserverCounter.*;

import java.math.BigInteger;
import java.util.Map;

import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifVserverMeasurements;
import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifVserverRawMeasurements;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

public class NetappLifVserverMetrics extends MetricsRowSample<NetworkLifVserverMeasurements, NetworkLifVserverRawMeasurements, 
	NetappLifVserverMetrics.LifVserverCounter, NetappLifVserverMetrics.LifVserverCounter> {

	public static enum LifVserverCounter {
		vserver_name,
		recv_data,
		recv_errors,
		recv_packet,
		sent_data,
		sent_errors,
		sent_packet
	}

	public NetappLifVserverMetrics(final MetricsRowScratchPad<LifVserverCounter> currentValues, final MetricsRowScratchPad<LifVserverCounter> previousValues) {
		super(LifVserverCounter.class, currentValues, previousValues);
	}

	@Override
	float getNormalizedFloatMetric(final LifVserverCounter counterName) {
		final float normalizedMetric;
		switch (counterName) {
		case recv_data:
		case sent_data:
		case recv_errors:
		case sent_errors:
		case recv_packet:
		case sent_packet:
			normalizedMetric = currentValues.getCounterRatePerSec(counterName);
			break;

		default:
			normalizedMetric = NULL_FLOAT;
		}

		return normalizedMetric;
	}

	@Override
	public NetworkLifVserverMeasurements normalizeMetrics(final NetAppDefaultStorageContext context) {
		final NetworkLifVserverMeasurements m = new NetworkLifVserverMeasurements();
		m.setRecvData(getNormalizedFloatMetric(recv_data));
		m.setRecvErrors(getNormalizedFloatMetric(recv_errors));
		m.setRecvPacket(getNormalizedFloatMetric(recv_packet));
		m.setSentData(getNormalizedFloatMetric(sent_data));
		m.setSentErrors(getNormalizedFloatMetric(sent_errors));
		m.setSentPacket(getNormalizedFloatMetric(sent_packet));
		m.setOpmLifPackets(MetricsRowScratchPad.add(m.getRecvPacket(), m.getSentPacket()));
		m.setOpmLifThroughput(MetricsRowScratchPad.add(m.getRecvData(), m.getSentData()));
		return m;
	}

	@Override
	public NetworkLifVserverRawMeasurements rawMetrics(final NetAppDefaultStorageContext context) {
		final NetworkLifVserverRawMeasurements m = new NetworkLifVserverRawMeasurements();
		Map<LifVserverCounter,BigInteger> counterValue = getCurrentValues().getCounterValues();
		m.setRecvData(counterValue.get(recv_data));
		m.setRecvErrors(counterValue.get(recv_errors));
		m.setRecvPacket(counterValue.get(recv_packet));
		m.setSentData(counterValue.get(sent_data));
		m.setSentErrors(counterValue.get(sent_errors));
		m.setSentPacket(counterValue.get(sent_packet));
		m.setOpmLifPackets(MetricsRowScratchPad.add(m.getRecvPacket(), m.getSentPacket()));
		m.setOpmLifThroughput(MetricsRowScratchPad.add(m.getRecvData(), m.getSentData()));
		return m;
	}

}
