package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics;

import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappLunMetrics.LunCounter.avg_latency;
import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappLunMetrics.LunCounter.avg_read_latency;
import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappLunMetrics.LunCounter.avg_write_latency;
import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappLunMetrics.LunCounter.other_ops;
import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappLunMetrics.LunCounter.read_data;
import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappLunMetrics.LunCounter.read_ops;
import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappLunMetrics.LunCounter.read_partial_blocks;
import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappLunMetrics.LunCounter.total_ops;
import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappLunMetrics.LunCounter.write_data;
import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappLunMetrics.LunCounter.write_ops;
import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappLunMetrics.LunCounter.write_partial_blocks;

import java.math.BigInteger;
import java.util.Map;

import com.netapp.oci.netapp.performance.lun.interfaces.data.LunMeasurements;
import com.netapp.oci.netapp.performance.lun.interfaces.data.LunRawMeasurements;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

public class NetappLunMetrics extends MetricsRowSample<LunMeasurements, LunRawMeasurements, NetappLunMetrics.LunCounter,
                                                                        NetappLunMetrics.LunCounter> {

    public static enum LunCounter {
        avg_latency,
        avg_read_latency,
        avg_write_latency,
        other_ops,
        read_data,
        read_ops,
        read_partial_blocks,
        total_ops,
        write_data,
        write_ops,
        write_partial_blocks
    }

    public NetappLunMetrics(final MetricsRowScratchPad<LunCounter> currentValues, final MetricsRowScratchPad<LunCounter> previousValues) {
        super(LunCounter.class, currentValues, previousValues);
    }

    @Override
    LunCounter getDenominatorForNumerator(final LunCounter numeratorName) {
        final LunCounter denominatorName;
        switch (numeratorName) {
            case avg_latency:
                denominatorName = total_ops;
                break;
            case avg_read_latency:
                denominatorName = read_ops;
                break;
            case avg_write_latency:
                denominatorName = write_ops;
                break;
            case read_partial_blocks:
                denominatorName = read_ops;
                break;
            case write_partial_blocks:
                denominatorName = write_ops;
                break;
            default:
                denominatorName = null;
        }
        return denominatorName;
    }

    @Override
    float getNormalizedFloatMetric(final LunCounter counterName) {
        final float normalizedMetric;
        switch (counterName) {
            case avg_latency:
            case avg_read_latency:
            case avg_write_latency:
                normalizedMetric = currentValues.getLatencyPerOp(counterName,
                                                                 getDenominatorForNumerator(counterName));
                break;

            case read_data:
            case write_data:
            case other_ops:
            case read_ops:
            case total_ops:
            case write_ops:
                normalizedMetric = currentValues.getCounterRatePerSec(counterName);
                break;

            case read_partial_blocks:
            case write_partial_blocks:
                normalizedMetric = currentValues.getPercentFromDeltas(counterName,
                                                                      getDenominatorForNumerator(counterName));
                break;

            default:
                normalizedMetric = NULL_FLOAT;
        }
        return normalizedMetric;
    }    

    @Override
    public LunMeasurements normalizeMetrics(final NetAppDefaultStorageContext context) {
        final LunMeasurements m = new LunMeasurements();
        m.setAvgLatency(getNormalizedFloatMetric(avg_latency));
        m.setAvgReadLatency(getNormalizedFloatMetric(avg_read_latency));
        m.setAvgWriteLatency(getNormalizedFloatMetric(avg_write_latency));
        m.setOtherOps(getNormalizedFloatMetric(other_ops));
        m.setReadData(getNormalizedFloatMetric(read_data));
        m.setReadOps(getNormalizedFloatMetric(read_ops));
        m.setReadPartialBlocks(getNormalizedFloatMetric(read_partial_blocks));
        m.setTotalOps(getNormalizedFloatMetric(total_ops));
        m.setWriteData(getNormalizedFloatMetric(write_data));
        m.setWriteOps(getNormalizedFloatMetric(write_ops));
        m.setWritePartialBlocks(getNormalizedFloatMetric(write_partial_blocks));
        
        //synthetic counter
        m.setOpmLunThroughput(MetricsRowScratchPad.add(m.getReadData(), m.getWriteData()));
        return m;
    }

    @Override
    public LunRawMeasurements rawMetrics(final NetAppDefaultStorageContext context) {
        final LunRawMeasurements m = new LunRawMeasurements();
        Map<LunCounter,BigInteger> counterValue = getCurrentValues().getCounterValues();
        m.setAvgLatency(counterValue.get(avg_latency));
        m.setAvgReadLatency(counterValue.get(avg_read_latency));
        m.setAvgWriteLatency(counterValue.get(avg_write_latency));
        m.setOtherOps(counterValue.get(other_ops));
        m.setReadData(counterValue.get(read_data));
        m.setReadOps(counterValue.get(read_ops));
        m.setTotalOps(counterValue.get(total_ops));
        m.setWriteData(counterValue.get(write_data));
        m.setWriteOps(counterValue.get(write_ops));
        m.setReadPartialBlocks(counterValue.get(read_partial_blocks));
        m.setWritePartialBlocks(counterValue.get(write_partial_blocks));
        m.setOpmLunThroughput(MetricsRowScratchPad.add(m.getReadData(), m.getWriteData()));
        return m;
    }

}
