package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics;


import com.netapp.oci.netapp.performance.node.interfaces.data.NFSv3NodeMeasurements;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import java.util.EnumSet;

import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappNFSv3NodeMetrics.NFSv3NodeCounter.*;

public class NetappNFSv3NodeMetrics extends MetricsRowSample<NFSv3NodeMeasurements, NFSv3NodeMeasurements, NetappNFSv3NodeMetrics.NFSv3NodeCounter,
                                                                                    NetappNFSv3NodeMetrics.OCIENFSv3NodeCounter> {

    /**
     * Column names requested from ZAPI
     */
    public static enum NFSv3NodeCounter {
        read_avg_latency,
        read_total,
        write_avg_latency,
        write_total,
        nfsv3_ops,
        nfsv3_read_ops,
        nfsv3_write_ops,
        access_avg_latency,
        access_total,
        commit_avg_latency,
        commit_total,
        create_avg_latency,
        create_total,
        fsinfo_avg_latency,
        fsinfo_total,
        fsstat_avg_latency,
        fsstat_total,
        getattr_avg_latency,
        getattr_total,
        link_avg_latency,
        link_total,
        lookup_avg_latency,
        lookup_total,
        mkdir_avg_latency,
        mkdir_total,
        null_avg_latency,
        null_total,
        pathconf_avg_latency,
        pathconf_total,
        remove_avg_latency,
        remove_total,
        rmdir_avg_latency,
        rmdir_total,
        setattr_avg_latency,
        setattr_total,
        symlink_avg_latency,
        symlink_total
    }

    /**
     * ZAPI Columns contributing to other latency
     */
    public static final EnumSet<NFSv3NodeCounter> OTHER_LATENCIES = EnumSet.of(
        access_avg_latency,
        commit_avg_latency,
        create_avg_latency,
        fsinfo_avg_latency,
        fsstat_avg_latency,
        getattr_avg_latency,
        link_avg_latency,
        lookup_avg_latency,
        mkdir_avg_latency,
        null_avg_latency,
        pathconf_avg_latency,
        remove_avg_latency,
        rmdir_avg_latency,
        setattr_avg_latency,
        symlink_avg_latency
    );

    /**
     * Columns reported to OCIE Server
     */
    public static enum OCIENFSv3NodeCounter {
        read_avg_latency,
        read_total,
        write_avg_latency,
        write_total,
        nfsv3_ops,
        nfsv3_read_ops,
        nfsv3_write_ops,
        other_latency
    }

    public NetappNFSv3NodeMetrics(final MetricsRowScratchPad<NFSv3NodeCounter> currentValues, final MetricsRowScratchPad<NFSv3NodeCounter> previousValues) {
        super(NFSv3NodeCounter.class, currentValues, previousValues);
    }

    @Override
    float getNormalizedFloatMetric(final OCIENFSv3NodeCounter counterName) {
        final float normalizedMetric;
        switch (counterName) {
            case nfsv3_ops:
                normalizedMetric = currentValues.getCounterRatePerSec(nfsv3_ops);
                break;

            case nfsv3_read_ops:
                normalizedMetric = currentValues.getCounterRatePerSec(nfsv3_read_ops);
                break;

            case nfsv3_write_ops:
                normalizedMetric = currentValues.getCounterRatePerSec(nfsv3_write_ops);
                break;

            case read_avg_latency:
                normalizedMetric = currentValues.getLatencyPerOp(read_avg_latency, read_total);
                break;

            case write_avg_latency:
                normalizedMetric = currentValues.getLatencyPerOp(write_avg_latency, write_total);
                break;

            case other_latency:
                normalizedMetric = getNormalizedOtherLatency();
                break;

            default:
                normalizedMetric = NULL_FLOAT;
        }
        return normalizedMetric;
    }

    private float getNormalizedOtherLatency() {
        if (currentValues.isWrapResetDetected()) {
            return NULL_FLOAT;
        }
        float normalizedOtherLatency = 0.0f;
        for (final NFSv3NodeCounter latencyCounterName : OTHER_LATENCIES) {
            final float anOtherLatency = getOtherLatencyPerOp(latencyCounterName);
            if (anOtherLatency == NULL_FLOAT) {
                normalizedOtherLatency = NULL_FLOAT;
                break;
            }
            normalizedOtherLatency += anOtherLatency;
        }
        return normalizedOtherLatency;
    }


    private float getOtherLatencyPerOp(final NFSv3NodeCounter latencyCounterName) {
        final float normalizedMetric;
        switch (latencyCounterName) {
            case access_avg_latency:
                normalizedMetric = currentValues.getLatencyPerOp(access_avg_latency, access_total);
                break;

            case commit_avg_latency:
                normalizedMetric = currentValues.getLatencyPerOp(commit_avg_latency, commit_total);
                break;

            case create_avg_latency:
                normalizedMetric = currentValues.getLatencyPerOp(create_avg_latency, create_total);
                break;

            case fsinfo_avg_latency:
                normalizedMetric = currentValues.getLatencyPerOp(fsinfo_avg_latency, fsinfo_total);
                break;

            case fsstat_avg_latency:
                normalizedMetric = currentValues.getLatencyPerOp(fsstat_avg_latency,  fsstat_total);
                break;

            case getattr_avg_latency:
                normalizedMetric = currentValues.getLatencyPerOp(getattr_avg_latency,  getattr_total);
                break;

            case link_avg_latency:
                normalizedMetric = currentValues.getLatencyPerOp(link_avg_latency, link_total);
                break;

            case lookup_avg_latency:
                normalizedMetric = currentValues.getLatencyPerOp(lookup_avg_latency, lookup_total);
                break;

            case mkdir_avg_latency:
                normalizedMetric = currentValues.getLatencyPerOp(mkdir_avg_latency, mkdir_total);
                break;

            case null_avg_latency:
                normalizedMetric = currentValues.getLatencyPerOp(null_avg_latency, null_total);
                break;

            case pathconf_avg_latency:
                normalizedMetric = currentValues.getLatencyPerOp(pathconf_avg_latency, pathconf_total);
                break;

            case remove_avg_latency:
                normalizedMetric = currentValues.getLatencyPerOp(remove_avg_latency, remove_total);
                break;

            case rmdir_avg_latency:
                normalizedMetric = currentValues.getLatencyPerOp(rmdir_avg_latency, rmdir_total);
                break;

            case setattr_avg_latency:
                normalizedMetric = currentValues.getLatencyPerOp(setattr_avg_latency, setattr_total);
                break;

            case symlink_avg_latency:
                normalizedMetric = currentValues.getLatencyPerOp(symlink_avg_latency, symlink_total);
                break;

            default:
                normalizedMetric = NULL_FLOAT;
        }
        return normalizedMetric;
    }



    @Override
    public NFSv3NodeMeasurements normalizeMetrics(final NetAppDefaultStorageContext context) {
        final NFSv3NodeMeasurements nfSv3NodeMeasurements = new NFSv3NodeMeasurements();
        nfSv3NodeMeasurements.setNFSv3Ops(getNormalizedFloatMetric(OCIENFSv3NodeCounter.nfsv3_ops));
        nfSv3NodeMeasurements.setNFSv3ReadOps(getNormalizedFloatMetric(OCIENFSv3NodeCounter.nfsv3_read_ops));
        nfSv3NodeMeasurements.setNFSv3WriteOps(getNormalizedFloatMetric(OCIENFSv3NodeCounter.nfsv3_write_ops));
        nfSv3NodeMeasurements.setReadAvgLatency(getNormalizedFloatMetric(OCIENFSv3NodeCounter.read_avg_latency));
        nfSv3NodeMeasurements.setWriteAvgLatency(getNormalizedFloatMetric(OCIENFSv3NodeCounter.write_avg_latency));
        nfSv3NodeMeasurements.setOtherLatency(getNormalizedFloatMetric(OCIENFSv3NodeCounter.other_latency));
        return nfSv3NodeMeasurements;
    }
}
