package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics;

import com.netapp.oci.netapp.performance.nfs.interfaces.data.Nfsv4Measurements;
import com.netapp.oci.netapp.performance.nfs.interfaces.data.Nfsv4RawMeasurements;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

import java.math.BigInteger;
import java.util.Map;

import static com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.NetappNfsv4Metrics.Nfsv4Counter.*;


public class NetappNfsv4Metrics extends MetricsRowSample<Nfsv4Measurements, Nfsv4RawMeasurements, NetappNfsv4Metrics.Nfsv4Counter, NetappNfsv4Metrics.Nfsv4Counter>{
	public static enum Nfsv4Counter {
		read_avg_latency,
		read_total,
		nfs4_read_throughput,
		nfs4_throughput,
		write_avg_latency,
		write_total,
		nfs4_write_throughput,
		nfsv4_ops,
	}

    public NetappNfsv4Metrics(MetricsRowScratchPad<Nfsv4Counter> currentValues, MetricsRowScratchPad<Nfsv4Counter> previousValues) {
		super(Nfsv4Counter.class, currentValues, previousValues);
	}
	
    Nfsv4Counter getDenominatorForNumerator(final Nfsv4Counter numeratorName) {
        final Nfsv4Counter denominatorName;
        switch (numeratorName) {
            case read_avg_latency:
                denominatorName = read_total;
                break;
            case write_avg_latency:
                denominatorName = write_total;
                break;   
            default:
                denominatorName = null;
        }
        return denominatorName;
    }
    
	@Override
	float getNormalizedFloatMetric(Nfsv4Counter counterName) {
	    final float normalizedMetric;
	    
	    switch (counterName) {
        case nfsv4_ops:
        case nfs4_read_throughput:
        case nfs4_throughput:
        case nfs4_write_throughput:
            normalizedMetric = currentValues.getCounterRatePerSec(counterName);
            break;
        case read_avg_latency:
        case write_avg_latency:
            normalizedMetric = currentValues.getLatencyPerOp(counterName, getDenominatorForNumerator(counterName));
            break;
        default:
            normalizedMetric = NULL_FLOAT;
            break;
        }
        return normalizedMetric;
	}
	

	@Override
	public Nfsv4Measurements normalizeMetrics(NetAppDefaultStorageContext context) {
	    final Nfsv4Measurements m = new Nfsv4Measurements();
        m.setNfsv4Ops(getNormalizedFloatMetric(nfsv4_ops));
        m.setReadTotal(currentValues.getCounterDeltaFinal(read_total));
        m.setNfsv4ReadThroughput(getNormalizedFloatMetric(nfs4_read_throughput));
        m.setNfsv4Throughput(getNormalizedFloatMetric(nfs4_throughput));
        m.setWriteTotal(currentValues.getCounterDeltaFinal(write_total));
        m.setNfsv4WriteThroughput(getNormalizedFloatMetric(nfs4_write_throughput));
        m.setReadAvgLatency(getNormalizedFloatMetric(read_avg_latency));
        m.setWriteAvgLatency(getNormalizedFloatMetric(write_avg_latency));
        return m;
	};

	@Override
	public Nfsv4RawMeasurements rawMetrics(NetAppDefaultStorageContext context){
        final Nfsv4RawMeasurements m = new Nfsv4RawMeasurements();
        Map<Nfsv4Counter, String> rawValue = getCurrentValues().getRawValues();

        Map<Nfsv4Counter,BigInteger> counterValues = getCurrentValues().getCounterValues();
        m.setNfs4ReadThroughput(counterValues.get(nfs4_read_throughput));
        m.setNfs4Throughput(counterValues.get(nfs4_throughput));
        m.setNfs4WriteThroughput(counterValues.get(nfs4_write_throughput));
        m.setNfsv4Ops(counterValues.get(nfsv4_ops));
        m.setReadAvgLatency(counterValues.get(read_avg_latency));
        m.setReadTotal(counterValues.get(read_total));
        m.setWriteAvgLatency(counterValues.get(write_avg_latency));
        m.setWriteTotal(counterValues.get(write_total));
        return m;
	}
	
}
