package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model;

import java.util.*;

/**
 * List of enum wrappers. Used to represent arrays of enumerated values.
 */
public class EnumWrapperList<T extends Enum<T>> implements Iterable<EnumWrapper<T>> {

    private final List<EnumWrapper<T>> values = new LinkedList<EnumWrapper<T>>();

    public EnumWrapperList() {
    }

    public void add(final EnumWrapper<T> value) {
        if (value != null) {
            values.add(value);
        }
    }

    public void set(final EnumWrapperList<T> values) {
        this.values.clear();
        // Note: call add() to filter null values
        for (final EnumWrapper<T> value : values.values) {
            add(value);
        }
    }

    public boolean isEmpty() {
        return values.isEmpty();
    }

    public Iterator<EnumWrapper<T>> iterator() {
        return values.iterator();
    }

    /**
     * @return list of enum values sorted in canonical order.
     */
    public List<T> getValues() {
        final List<T> result = new ArrayList<T>(values.size());
        for (final EnumWrapper<T> wrapper :  values) {
            result.add(wrapper.getEnumValue());
        }
        return result;
    }

    public List<String> getRawValues() {
        final List<String> result = new ArrayList<String>(values.size());
        for (final EnumWrapper<T> wrapper : values) {
            result.add(wrapper.getRawValue());
        }
        return result;
    }

    /**
     * @return A comma-separated list of values, sorted alphabetically.
     */
    public String valuesToString() {
        final List<String> enumValues = new ArrayList<String>(values.size());
        for (final EnumWrapper<T> wrapper : values) {
            enumValues.add(wrapper.toString());
        }
        Collections.sort(enumValues);
        return toCommaSeparatedString(enumValues);
    }

    /**
     * @return A comma-separated list of raw values, sorted alphabetically.
     */
    public String rawValuesToString() {
        final List<String> rawValues = getRawValues();
        Collections.sort(rawValues);
        return toCommaSeparatedString(rawValues);
    }

    /** commons.lang3 not available (yet). */
    private static String toCommaSeparatedString(final List<String> list) {
        final StringBuilder buf = new StringBuilder();
        String separator = "";
        for (final String str : list) {
            buf.append(separator);
            buf.append(str);
            separator = ",";
        }
        return buf.toString();
    }

}
