package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model;

import com.onaro.sanscreen.acquisition.schema.KeyBase;

import java.util.EnumMap;
import java.util.EnumSet;


/**
 * Keys used in the model don't need to render OCIE UUIDs or ResourceKeys required by the Server,
 * they just have to provide uniqueness.  Keys may require multiple fields that contribute to
 * uniqueness as well as model marshalling.
 * We take advantage of KeyBase as a data structure for holding Key fields, but do not take advantage
 * of OCIE schema specific functionality defined in KeyBase.
 * getOcieUUID is not required in the model and so a default implementation is provided for all clients of ModelKeyBase
 * @param <E> The enum defining the key fields
 */
public abstract class ModelKeyBase<E extends Enum<E>> extends KeyBase<E> {

    public ModelKeyBase(final String keyType, final EnumMap<E, String> idValues) {
        super(keyType, idValues, null);
    }

    public ModelKeyBase(final String keyType, final EnumMap<E, String> idValues, final EnumSet<E> emptyAbleFields) {
        super(keyType, idValues, emptyAbleFields);
    }

    public String getOcieUUID() {
        return getFQID();       // In the model, UUIDs just have to be unique, no database requirements to define structure
    }

    // This is not really a UUID, just an ID with 1 or more fields.
    public String getID() {
        return getFQID();
    }
}
