package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;

/**
 * @author Iti Behari
 *         Date: 7/9/13
 */
public class CModeApplicationRecord extends BaseCRCXMLNode<CModeCluster> {
    private final UUIDKey applicationKey;
    private String recordName;
    private String recordValue;
    private String modifiedFrom;
    private Long lastModifiedTimestamp;

    CModeApplicationRecord(final CModeCluster parent, final String recordName, final String recordValue, final String modifiedFrom, final Long lastModifiedTimestamp) {
        super(parent, CModeApplicationRecord.class.getSimpleName());
        this.applicationKey = new UUIDKey("application_record" , recordName);

        this.recordName = recordName;
        this.recordValue = recordValue;
        this.modifiedFrom = modifiedFrom;
        this.lastModifiedTimestamp = lastModifiedTimestamp;
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public UUIDKey getKey() {
        return applicationKey;
    }

    @Override

    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "UUID", applicationKey.getUUID());
        addAttributeNoCRC(writer, "RecordName", recordName);
        addAttributeNoCRC(writer, "RecordValue", recordValue);
        addAttributeNoCRC(writer, "ModifiedFrom", modifiedFrom);
        addAttributeNoCRC(writer, "LastModifiedTimestamp", lastModifiedTimestamp);
    }

    public String getRecordName() {
        return recordName;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public void setRecordName(final String recordName) {
        this.recordName = recordName;
    }

    public String getRecordValue() {
        return recordValue;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public void setRecordValue(final String recordValue) {
        this.recordValue = recordValue;
    }

    public String getModifiedFrom() {
        return modifiedFrom;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public void setModifiedFrom(final String modifiedFrom) {
        this.modifiedFrom = modifiedFrom;
    }

    public Long getLastModifiedTimestamp() {
        return lastModifiedTimestamp;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public void setLastModifiedTimestamp(final Long lastModifiedTimestamp) {
        this.lastModifiedTimestamp = lastModifiedTimestamp;
    }


}

