package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;

/**
 * @author Iti Behari
 *         Date: 7/12/13
 */
public class CModeCIFSOption extends BaseCRCXMLNode<CModeCluster> {
    private final UUIDKey applicationKey;
    private String vserverName;
    private String defaultUnixUser;
    private String defaultUnixGroup;
    private Boolean exportPolicyEnabled;

    CModeCIFSOption(final CModeCluster parent, final String vserverName, final String defaultUnixUser, final String defaultUnixGroup, final Boolean exportPolicyEnabled) {
        super(parent, CModeCIFSOption.class.getSimpleName());
        this.applicationKey = new UUIDKey("cifs_option" , vserverName);

        this.vserverName = vserverName;
        this.defaultUnixUser = defaultUnixUser;
        this.defaultUnixGroup = defaultUnixGroup;
        this.exportPolicyEnabled = exportPolicyEnabled;
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public UUIDKey getKey() {
        return applicationKey;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "UUID", applicationKey.getUUID());
        addAttributeNoCRC(writer, "VserverName", vserverName);
        addAttributeNoCRC(writer, "DefaultUnixUser", defaultUnixUser);
        addAttributeNoCRC(writer, "DefaultUnixGroup", defaultUnixGroup);
        addAttributeNoCRC(writer, "ExportPolicyEnabled", exportPolicyEnabled);
    }

    public String getVserverName() {
        return vserverName;
    }

    public void setVserverName(String vserverName) {
        this.vserverName = vserverName;
    }

    public String getDefaultUnixUser() {
        return defaultUnixUser;
    }

    public void setDefaultUnixUser(String defaultUnixUser) {
        this.defaultUnixUser = defaultUnixUser;
    }

    public String getDefaultUnixGroup() {
        return defaultUnixGroup;
    }

    public void setDefaultUnixGroup(String defaultUnixGroup) {
        this.defaultUnixGroup = defaultUnixGroup;
    }

    public Boolean getExportPolicyEnabled() {
        return exportPolicyEnabled;
    }

    public void setExportPolicyEnabled(Boolean exportPolicyEnabled) {
        this.exportPolicyEnabled = exportPolicyEnabled;
    }
}


