package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.ClusterPeer;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;

public class CModeClusterPeer extends BaseCRCXMLNode<CModeCluster> {

    private final UUIDKey key;

    private String activeAddresses;
    private EnumWrapper<ClusterPeer.AddressFamily> addressFamily;
    private String authStatusAdmin;
    private EnumWrapper<ClusterPeer.AuthStatusOperational> authStatusOperational;
    private EnumWrapper<ClusterPeer.Availability> availability;
    private String remoteClusterName; //cluster-name
    private final String remoteClusterUUID; //cluster-uuid
    private String ipSpaceName;
    private Boolean isClusterHealthy;
    private String peerAddresses;
    private String remoteClusterNodes;
    private String serialNumber;
    private String unreachableLocalNodes;

    CModeClusterPeer(final CModeCluster parent, final String remoteClusterUUID) {
        super(parent, CModeClusterPeer.class.getSimpleName());
        this.key = new UUIDKey("cluster-peer", remoteClusterUUID);
        this.remoteClusterUUID = remoteClusterUUID;
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public UUIDKey getKey() {
        return key;
    }

    public String getActiveAddresses() {
        return activeAddresses;
    }

    public void setActiveAddresses(String activeAddresses) {
        this.activeAddresses = activeAddresses;
    }

    public EnumWrapper<ClusterPeer.AddressFamily> getAddressFamily() {
        return addressFamily;
    }

    public void setAddressFamily(EnumWrapper<ClusterPeer.AddressFamily> addressFamily) {
        this.addressFamily = addressFamily;
    }

    public String getAuthStatusAdmin() {
        return authStatusAdmin;
    }

    public void setAuthStatusAdmin(String authStatusAdmin) {
        this.authStatusAdmin = authStatusAdmin;
    }

    public EnumWrapper<ClusterPeer.AuthStatusOperational> getAuthStatusOperational() {
        return authStatusOperational;
    }

    public void setAuthStatusOperational(EnumWrapper<ClusterPeer.AuthStatusOperational> authStatusOperational) {
        this.authStatusOperational = authStatusOperational;
    }

    public EnumWrapper<ClusterPeer.Availability> getAvailability() {
        return availability;
    }

    public void setAvailability(EnumWrapper<ClusterPeer.Availability> availability) {
        this.availability = availability;
    }

    public String getRemoteClusterName() {
        return remoteClusterName;
    }

    public void setRemoteClusterName(String remoteClusterName) {
        this.remoteClusterName = remoteClusterName;
    }

    public String getRemoteClusterUUID() {
        return remoteClusterUUID;
    }

    public String getIpSpaceName() {
        return ipSpaceName;
    }

    public void setIpSpaceName(String ipSpaceName) {
        this.ipSpaceName = ipSpaceName;
    }

    public Boolean getClusterHealthy() {
        return isClusterHealthy;
    }

    public void setClusterHealthy(Boolean clusterHealthy) {
        isClusterHealthy = clusterHealthy;
    }

    public String getPeerAddresses() {
        return peerAddresses;
    }

    public void setPeerAddresses(String peerAddresses) {
        this.peerAddresses = peerAddresses;
    }

    public String getRemoteClusterNodes() {
        return remoteClusterNodes;
    }

    public void setRemoteClusterNodes(String remoteClusterNodes) {
        this.remoteClusterNodes = remoteClusterNodes;
    }

    public String getSerialNumber() {
        return serialNumber;
    }

    public void setSerialNumber(String serialNumber) {
        this.serialNumber = serialNumber;
    }

    public String getUnreachableLocalNodes() {
        return unreachableLocalNodes;
    }

    public void setUnreachableLocalNodes(String unreachableLocalNodes) {
        this.unreachableLocalNodes = unreachableLocalNodes;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "UUID", key.getID());
        addAttributeNoCRC(writer, "ActiveAddresses", activeAddresses);
        addAttributeNoCRC(writer, "AddressFamily", addressFamily.getEnumValue());
        addAttributeNoCRC(writer, "AddressFamilyRaw", addressFamily.getRawValue());
        addAttributeNoCRC(writer, "AuthStatusAdmin", authStatusAdmin);
        addAttributeNoCRC(writer, "AuthStatusOperational", authStatusOperational.getEnumValue());
        addAttributeNoCRC(writer, "AuthStatusOperationalRaw", authStatusOperational.getRawValue());
        addAttributeNoCRC(writer, "Availability", availability.getEnumValue());
        addAttributeNoCRC(writer, "AvailabilityRaw", availability.getRawValue());
        addAttributeNoCRC(writer, "RemoteClusterName", remoteClusterName);
        addAttributeNoCRC(writer, "RemoteClusterUUID", remoteClusterUUID);
        addAttributeNoCRC(writer, "IpSpaceName", ipSpaceName);
        addAttributeNoCRC(writer, "IsClusterHealthy", isClusterHealthy);
        addAttributeNoCRC(writer, "PeerAddresses", peerAddresses);
        addAttributeNoCRC(writer, "RemoteClusterNodes", remoteClusterNodes);
        addAttributeNoCRC(writer, "SerialNumber", serialNumber);
        addAttributeNoCRC(writer, "UnreachableLocalNodes", unreachableLocalNodes);
    }
}
