package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.DiskPath;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModeDiskPath extends BaseCRCXMLNode<CModeDisk> {

    private final DiskPathKey diskPathKey;
    private String arrayName;
    private String diskPort;
    private String diskPortName;
    private String initiatorSideSwitchPort;
    private Integer lun;
    private EnumWrapper<DiskPath.LUNPathUseStateType> lunPathUseState;
    private String node;
    private String targetSideSwitchPort;
    private Long targetPortGroupNum;
    private String targetWWNN;

    CModeDiskPath(final CModeDisk parent, final String diskUUID, final String initiatorPortName, final String targetWWPN) {
        super(parent, CModeDiskPath.class.getSimpleName());
        this.diskPathKey = new DiskPathKey(diskUUID, initiatorPortName, targetWWPN);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public DiskPathKey getKey() {
        return diskPathKey;
    }

    public String getDiskUUID() {
        return diskPathKey.getID(DiskPathKey.Field.disk_uuid);
    }

    public String getArrayName() {
        return arrayName;
    }

    public String getDiskPort() {
        return diskPort;
    }

    public String getDiskPortName() {
        return diskPortName;
    }

    public String getInitiatorPortName() {
        return diskPathKey.getID(DiskPathKey.Field.initiator_port_name);
    }

    public String getInitiatorSideSwitchPort() {
        return initiatorSideSwitchPort;
    }

    public Integer getLun() {
        return lun;
    }

    public EnumWrapper<DiskPath.LUNPathUseStateType> getLunPathUseState() {
        return lunPathUseState;
    }

    public String getNode() {
        return node;
    }

    public String getTargetWWPN() {
        return diskPathKey.getID(DiskPathKey.Field.target_wwpn);
    }

    public String getTargetSideSwitchPort() {
        return targetSideSwitchPort;
    }

    public Long getTargetPortGroupNum() {
        return targetPortGroupNum;
    }
    
    public String getTargetWWNN() {
        return targetWWNN;
    }

    public void setArrayName(final String arrayName) {
        this.arrayName = arrayName;
    }

    public void setDiskPort(final String diskPort) {
        this.diskPort = diskPort;
    }

    public void setDiskPortName(final String diskPortName) {
        this.diskPortName = diskPortName;
    }

    public void setInitiatorSideSwitchPort(final String initiatorSideSwitchPort) {
        this.initiatorSideSwitchPort = initiatorSideSwitchPort;
    }

    public void setLun(final Integer lun) {
        this.lun = lun;
    }

    public void setLunPathUseState(final EnumWrapper<DiskPath.LUNPathUseStateType> lunPathUseState) {
        this.lunPathUseState = lunPathUseState;
    }

    public void setNode(final String node) {
        this.node = node;
    }

    public void setTargetSideSwitchPort(final String targetSideSwitchPort) {
        this.targetSideSwitchPort = targetSideSwitchPort;
    }

    public void setTargetPortGroupNum(final Long targetPortGroupNum) {
        this.targetPortGroupNum = targetPortGroupNum;
    }

    public void setTargetWWNN(String targetWWNN) {
        this.targetWWNN = targetWWNN;
    }
    
    public final static class DiskPathKey extends ModelKeyBase<DiskPathKey.Field> {
        public static enum Field {
            disk_uuid,
            initiator_port_name,
            target_wwpn
        }

        public DiskPathKey(final String diskUUID, final String initiatorPortName, final String targetWWPN) {
            super("disk_path_key", new EnumMap<Field, String>(Field.class) {{
                put(Field.disk_uuid, diskUUID);
                put(Field.initiator_port_name, initiatorPortName);
                put(Field.target_wwpn, targetWWPN);
            }});
        }
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);

        addAttributeNoCRC(writer, "DiskUUID", getDiskUUID());
        addAttributeNoCRC(writer, "ArrayName", arrayName);
        addAttributeNoCRC(writer, "DiskPort", diskPort);
        addAttributeNoCRC(writer, "DiskPortName", diskPortName);
        addAttributeNoCRC(writer, "InitiatorPortName", diskPathKey.getID(DiskPathKey.Field.initiator_port_name));
        addAttributeNoCRC(writer, "InitiatorSideSwitchPort", initiatorSideSwitchPort);
        addAttributeNoCRC(writer, "LUN", lun);
        addAttributeNoCRC(writer, "LUNPathUseState", lunPathUseState.getEnumValue());
        addAttributeNoCRC(writer, "LUNPathUseStateRaw", lunPathUseState.getRawValue());
        addAttributeNoCRC(writer, "Node", node);
        addAttributeNoCRC(writer, "TargetWWPN", getTargetWWPN());
        addAttributeNoCRC(writer, "TargetSideSwitchPort", targetSideSwitchPort);
        addAttributeNoCRC(writer, "TGPN", targetPortGroupNum);
        addAttributeNoCRC(writer, "TargetWWNN", getTargetWWNN());
    }
}
