package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapperList;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.ExportRule;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

import static com.onaro.sanscreen.acquisition.netappschema.netappdata.ExportRule.AccessProtocol;
import static com.onaro.sanscreen.acquisition.netappschema.netappdata.ExportRule.SecurityFlavor;

public class CModeExportRule extends CModeBaseXMLNode<CModeCluster> {

    private final CModeExportRuleKey key;
    private String clientMatch;
    private final EnumWrapperList<AccessProtocol> protocol = new EnumWrapperList<AccessProtocol>();
    private final EnumWrapperList<SecurityFlavor> roRule = new EnumWrapperList<SecurityFlavor>();
    private final EnumWrapperList<SecurityFlavor> rwRule = new EnumWrapperList<SecurityFlavor>();
    private final EnumWrapperList<SecurityFlavor> superUserSecurity = new EnumWrapperList<SecurityFlavor>();


    CModeExportRule(final CModeCluster parent, final String vServerName, final String policyName, final int ruleIndex) {
        super(parent, CModeExportRule.class.getSimpleName());
        this.key = new CModeExportRuleKey(vServerName, policyName, String.valueOf(ruleIndex));
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public CModeExportRuleKey getKey() {
        return key;
    }

    public String getPolicyName() {
        return key.getID(CModeExportRuleKey.Field.policy_name);
    }

    public String getRuleIndex() {
        return key.getID(CModeExportRuleKey.Field.rule_index);
    }

    public String getClientMatch() {
        return clientMatch;
    }

    public EnumWrapperList<ExportRule.AccessProtocol> getProtocol() {
        return protocol;
    }

    public String getVServerName() {
        return key.getID(CModeExportRuleKey.Field.vserver_name);
    }

    public EnumWrapperList<SecurityFlavor> getRoRule() {
        return roRule;
    }

    public EnumWrapperList<SecurityFlavor> getRwRule() {
        return rwRule;
    }

    public EnumWrapperList<SecurityFlavor> getSuperUserSecurity() {
        return superUserSecurity;
    }

    public void setClientMatch(final String clientMatch) {
        this.clientMatch = clientMatch;
    }

    public static final class CModeExportRuleKey extends ModelKeyBase<CModeExportRuleKey.Field> {

        public static enum Field {
            vserver_name,
            policy_name,
            rule_index
        }

        public CModeExportRuleKey(final String vServerName, final String policyName, final String ruleIndex) {
            super("export_rule", new EnumMap<Field, String>(Field.class) {{
                put(Field.vserver_name, vServerName);
                put(Field.policy_name, policyName);
                put(Field.rule_index, ruleIndex);
            }});
        }
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "VServerName", getVServerName());
        addAttributeNoCRC(writer, "PolicyName", key.getID(CModeExportRuleKey.Field.policy_name));
        addAttributeNoCRC(writer, "RuleIndex", getRuleIndex());
        addAttributeNoCRC(writer, "ClientMatch", clientMatch);
    }

    @Override
    protected boolean hasChildren() {
        return !protocol.isEmpty() || !roRule.isEmpty() || !rwRule.isEmpty();
    }

    @Override
    protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs)
            throws IOException, SchemaException {
        super.marshallChildrenToFileAndCRC(cache, writer, tabs);
        marshallEnumWrapperList(writer, protocol, "Protocol", "AccessProtocol", tabs);
        marshallEnumWrapperList(writer, roRule, "RoRule", "SecurityFlavor", tabs);
        marshallEnumWrapperList(writer, rwRule, "RwRule", "SecurityFlavor", tabs);
        marshallEnumWrapperList(writer, superUserSecurity, "SuperUserSecurity", "SecurityFlavor", tabs);
    }
}
