package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.FlashDevice;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;

/**
 * Represents a C-Mode flash device.
 * @see <a href="http://web.netapp.com/engineering/design-depot/appliance-mgmt/zephyr/zapidoc.cache/DOT/mainN/web_ontap_cluster/flash/index.html#flash-device-info">ZAPI docs</a>
 */
public class CModeFlashDevice extends BaseCRCXMLNode<CModeNode> {

    private static final String TYPE_NAME = "flashdevice";

    private final UUIDKey key;

    // flash-device-info from flash-device-get-iter
    /** PCI-e slot number of the flash device.  */
    private Integer slotNumber = null;
    /** Model name of the flash device in XXXXXX-XX format. */
    private String modelName = null;
    /** Firmware revision of FPGA on the flash device.*/
    private String firmwareRevision = null;
    /** Advertised capacity of the device, in gigabytes. */
    private Integer capacity = null;
    /** Percentage of device capacity that is currently online [0..100]. */
    private Integer percentOnline = null;
    /** The current status of the device. */
    private EnumWrapper<FlashDevice.Status> status = null;
    /** The current status of the device. Possible values: "online", "offline_failed", or "offline_threshold". */
    //private String statusRaw = null;
    /** Total number of hours the device was operational. */
    private Integer serviceTime = null;
    /** The name of the threshold profile being used for this flash device. */
    private String thresholdProfile = null;

    CModeFlashDevice(final CModeNode parent, final String serialNumber) {
        super(parent, CModeFlashDevice.class.getSimpleName());
        this.key = new UUIDKey(TYPE_NAME, serialNumber);
    }

    public String getSerialNumber() {
        return key.getID();
    }

    public Integer getSlotNumber() {
        return slotNumber;
    }

    public void setSlotNumber(final int slotNumber) {
        this.slotNumber = slotNumber;
    }

    public String getModelName() {
        return modelName;
    }

    public void setModelName(final String modelName) {
        this.modelName = modelName;
    }

    public String getFirmwareRevision() {
        return firmwareRevision;
    }

    public void setFirmwareRevision(final String firmwareRevision) {
        this.firmwareRevision = firmwareRevision;
    }

    public Integer getCapacity() {
        return capacity;
    }

    public void setCapacity(final Integer capacity) {
        this.capacity = capacity;
    }

    public Integer getPercentOnline() {
        return percentOnline;
    }

    public void setPercentOnline(final Integer percentOnline) {
        this.percentOnline = percentOnline;
    }

    public EnumWrapper<FlashDevice.Status> getStatus() {
        return status;
    }

    public void setStatus(final EnumWrapper<FlashDevice.Status> status) {
        this.status = status;
    }

    public Integer getServiceTime() {
        return serviceTime;
    }

    public void setServiceTime(final Integer serviceTime) {
        this.serviceTime = serviceTime;
    }

    public String getThresholdProfile() {
        return thresholdProfile;
    }

    public void setThresholdProfile(final String thresholdProfile) {
        this.thresholdProfile = thresholdProfile;
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public UUIDKey getKey() {
        return key;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "SerialNumber", getSerialNumber());
        addAttributeNoCRC(writer, "SlotNumber", slotNumber);
        addAttributeNoCRC(writer, "ModelName", modelName);
        addAttributeNoCRC(writer, "FirmwareRevision", firmwareRevision);
        addAttributeNoCRC(writer, "Capacity", capacity);
        addAttributeNoCRC(writer, "PercentOnline", percentOnline);
        addAttributeNoCRC(writer, "Status", status.getEnumValue());
        addAttributeNoCRC(writer, "StatusRaw", status.getRawValue());
        addAttributeNoCRC(writer, "ServiceTime", serviceTime);
        addAttributeNoCRC(writer, "ThresholdProfile", thresholdProfile);
    }

}
