package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.IGroup;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.io.Writer;
import java.util.List;

/**
 * Represents a C-Mode initiator group.
 *
 * @see <a href="http://web.netapp.com/engineering/design-depot/appliance-mgmt/zephyr/zapidoc.cache/DOT/mainN/web_ontap_cluster/igroup/index.html">ZAPI docs</a>
 */
public class CModeIGroup extends BaseCRCXMLNode<CModeCluster> {

    private final UUIDKey key;
    private String iGroupName;
    private String vServerName;
    private EnumWrapper<IGroup.Type> type;
    private String osType;
    /**
     * Boolean value to indicate if this initiator group has ALUA (Asymmetric Logical Unit Access) features enabled for
     * LUNs mapped to this initiator group.
     */
    private Boolean aluaEnabled;
    /**
     * Boolean value to indicate if this initiator group has Volume Set Addressing (VSA) enabled or disabled.
     */
    private Boolean vsaEnabled;
    /**
     * Boolean value to indicate if this initiator group is configured for its luns to require the use of host
     * multi-pathing software for correct high-availability failover operation. In Data ONTAP 7-Mode, this value is
     * optional and is only returned for FCP initiator groups on an storage system in an HA pair. In Data ONTAP
     * Cluster-Mode, this field will always be <tt>true</tt>.
     */
    private Boolean usePartner;
    private List<String> initiatorNames;

    CModeIGroup(final CModeCluster parent, final String iGroupUUID) {
        super(parent, CModeIGroup.class.getSimpleName());
        this.key = new UUIDKey("igroup", iGroupUUID);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public UUIDKey getKey() {
        return key;
    }

    public String getIGroupName() {
        return iGroupName;
    }

    public String getVServerName() {
        return vServerName;
    }

    public EnumWrapper<IGroup.Type> getType() {
        return type;
    }

    public String getOSType() {
        return osType;
    }

    public List<String> getInitiatorNames() {
        return initiatorNames;
    }

    public void setIGroupName(final String iGroupName) {
        this.iGroupName = iGroupName;
    }

    public void setVServerName(final String vServerName) {
        this.vServerName = vServerName;
    }

    public void setType(final EnumWrapper<IGroup.Type> type) {
        this.type = type;
    }

    public void setOsType(final String osType) {
        this.osType = osType;
    }

    public Boolean getAluaEnabled() {
        return aluaEnabled;
    }

    public void setAluaEnabled(final Boolean aluaEnabled) {
        this.aluaEnabled = aluaEnabled;
    }

    public Boolean getVsaEnabled() {
        return vsaEnabled;
    }

    public void setVsaEnabled(final Boolean vsaEnabled) {
        this.vsaEnabled = vsaEnabled;
    }

    public Boolean getUsePartner() {
        return usePartner;
    }

    public void setUsePartner(final Boolean usePartner) {
        this.usePartner = usePartner;
    }

    public void setInitiatorNames(final List<String> initiatorNames) {
        this.initiatorNames = initiatorNames;
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "Name", iGroupName);
        addAttributeNoCRC(writer, "UUID", key.getID());
        addAttributeNoCRC(writer, "VServerName", vServerName);
        addAttributeNoCRC(writer, "Type", type);
        addAttributeNoCRC(writer, "OSType", osType);
        addAttributeNoCRC(writer, "AluaEnabled", aluaEnabled);
        addAttributeNoCRC(writer, "VsaEnabled", vsaEnabled);
        addAttributeNoCRC(writer, "UsePartner", usePartner);
    }

    @Override
    protected boolean hasChildren() {
        return !initiatorNames.isEmpty();
    }

    @Override
    protected void marshallChildren(final MarshallingCache cache, final Writer writer, final int tabs) throws IOException, SchemaException {
        final String xmlNodeName = "InitiatorNames";
        addTabulation(writer, tabs);
        writer.append("<").append(xmlNodeName).append(">").append(lineSeperator);
        for (final String initiatorName : initiatorNames) {
            addTabulation(writer, tabs + 1);
            writer.append("<InitiatorName>");
            writer.append(initiatorName);
            writer.append("</InitiatorName>").append(lineSeperator);
        }
        addTabulation(writer, tabs);
        writer.append("</").append(xmlNodeName).append(">").append(lineSeperator);
    }

}
