package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModeISCSIPortalGroup extends BaseCRCXMLNode<CModeCluster> {

    private final CModeISCSIPortalGroupKey key;
    private Integer tag;

    CModeISCSIPortalGroup(final CModeCluster parent, final String vServerName, final String name) {
        super(parent, CModeISCSIPortalGroup.class.getSimpleName());
        this.key = new CModeISCSIPortalGroupKey(vServerName, name);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public CModeISCSIPortalGroupKey getKey() {
        return key;
    }

    public String getName() {
        return key.getID(CModeISCSIPortalGroupKey.Field.portal_group_name);
    }

    public Integer getTag() {
        return tag;
    }

    public String getVServerName() {
        return key.getID(CModeISCSIPortalGroupKey.Field.vserver_name);
    }

    public void setTag(final Integer tag) {
        this.tag = tag;
    }

    public static final class CModeISCSIPortalGroupKey extends ModelKeyBase<CModeISCSIPortalGroupKey.Field> {

        public static enum Field {
            vserver_name,
            portal_group_name
        }

        public CModeISCSIPortalGroupKey(final String vServerName, final String portalGroupName) {
            super("iscsi_portal_group", new EnumMap<Field, String>(Field.class) {{
                put(Field.vserver_name, vServerName);
                put(Field.portal_group_name, portalGroupName);
            }});
        }
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "ID", key.getID());
        addAttributeNoCRC(writer, "VServer", getVServerName());
        addAttributeNoCRC(writer, "Name", getName());
        addAttributeNoCRC(writer, "Tag", tag);
    }
}
