package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModeISCSIPortalGroupToLIF extends BaseCRCXMLNode<CModeCluster> {

    private final CModeISCSIPortalGroupToLIFKey key;
    private String ipAddress;
    private String ipPort;
    private Integer portalGroupTag;

    CModeISCSIPortalGroupToLIF(final CModeCluster parent, final String vserverName, final String interfaceName, final String portalGroupName) {
        super(parent, CModeISCSIPortalGroupToLIF.class.getSimpleName());
        this.key = new CModeISCSIPortalGroupToLIFKey(vserverName, interfaceName, portalGroupName);
    }

    @SuppressWarnings({"unchecked"})
    public CModeISCSIPortalGroupToLIFKey getKey() {
        return key;
    }

    public String getInterfaceName() {
        return key.getID(CModeISCSIPortalGroupToLIFKey.Field.interface_name);
    }

    public String getIpAddress() {
        return ipAddress;
    }

    public String getIpPort() {
        return ipPort;
    }

    public String getPortalGroupName() {
        return key.getID(CModeISCSIPortalGroupToLIFKey.Field.portal_group_name);
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getPortalGroupTag() {
        return portalGroupTag;
    }

    public String getVServerName() {
        return key.getID(CModeISCSIPortalGroupToLIFKey.Field.vserver_name);
    }

    public void setIpAddress(final String ipAddress) {
        this.ipAddress = ipAddress;
    }

    public void setIpPort(final String ipPort) {
        this.ipPort = ipPort;
    }

    public void setPortalGroupTag(final Integer portalGroupTag) {
        this.portalGroupTag = portalGroupTag;
    }

    public static final class CModeISCSIPortalGroupToLIFKey extends ModelKeyBase<CModeISCSIPortalGroupToLIFKey.Field> {
        public static enum Field {
            vserver_name,
            interface_name,
            portal_group_name
        }

        public CModeISCSIPortalGroupToLIFKey(final String vServerName, final String interfaceName, final String portalGroupName) {
            super("iscsi_portal_group_to_lif", new EnumMap<Field, String>(Field.class) {{
                put(Field.vserver_name, vServerName);
                put(Field.interface_name, interfaceName);
                put(Field.portal_group_name, portalGroupName);
            }});
        }
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "ID", key.getID());
        addAttributeNoCRC(writer, "VServerName", getVServerName());
        addAttributeNoCRC(writer, "InterfaceName", key.getID(CModeISCSIPortalGroupToLIFKey.Field.interface_name));
        addAttributeNoCRC(writer, "PortalGroupName", getPortalGroupName());
        addAttributeNoCRC(writer, "PortalGroupTag", portalGroupTag);
        addAttributeNoCRC(writer, "IPAddress", ipAddress);
        addAttributeNoCRC(writer, "IPPort", ipPort);
    }
}
