package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.ISCSISecurityEntry;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModeISCSISecurityEntry extends BaseCRCXMLNode<CModeCluster> {

    private final CModeISCSISecurityEntryKey key;
    private ISCSISecurityEntry.ISCSISecurityEntryAuthenticationType authenticationType;
    private String username;
    private String outboundUsername;

    CModeISCSISecurityEntry(final CModeCluster cluster, final String vServerName, final String initiator) {
        super(cluster, CModeISCSISecurityEntry.class.getSimpleName());
        this.key = new CModeISCSISecurityEntryKey(cluster.getClusterUUID(), vServerName, initiator);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public CModeISCSISecurityEntryKey getKey() {
        return key;
    }

    public String getVServerName() {
        return key.getID(CModeISCSISecurityEntryKey.Field.vserver_name);
    }

    public String getInitiator() {
        return key.getID(CModeISCSISecurityEntryKey.Field.iscsi_initiator);
    }

    public ISCSISecurityEntry.ISCSISecurityEntryAuthenticationType getAuthenticationType() {
        return authenticationType;
    }

    public String getUsername() {
        return username;
    }

    public String getOutboundUsername() {
        return outboundUsername;
    }

    public void setAuthenticationType(final ISCSISecurityEntry.ISCSISecurityEntryAuthenticationType authenticationType) {
        this.authenticationType = authenticationType;
    }

    public void setUsername(final String username) {
        this.username = username;
    }

    public void setOutboundUsername(final String outboundUsername) {
        this.outboundUsername = outboundUsername;
    }

    public static final class CModeISCSISecurityEntryKey extends ModelKeyBase<CModeISCSISecurityEntryKey.Field> {
        public static enum Field {
            cluster_uuid,         // JSD TODO does this really need to be cluster scoped?
            vserver_name,
            iscsi_initiator
        }

        public CModeISCSISecurityEntryKey(final String clusterUUID, final String vServerName, final String iscsiInitiator) {
            super("iscsi_security_entry", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.vserver_name, vServerName);
                put(Field.iscsi_initiator, iscsiInitiator);
            }});
        }
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "ID", key.getID());
        addAttributeNoCRC(writer, "VServerName", getVServerName());
        addAttributeNoCRC(writer, "Initiator", getInitiator());
        addAttributeNoCRC(writer, "AuthenticationType", authenticationType);
        addAttributeNoCRC(writer, "Username", username);
        addAttributeNoCRC(writer, "OutboundUsername", outboundUsername);
    }
}
