package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.ISCSISessionConnection;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModeISCSISessionConnection extends BaseCRCXMLNode<CModeISCSISession> {

    private final CModeISCSISessionConnectionKey key;
    private final int connectionID;
    private String interfaceName;
    private String localIPAddress;
    private String localIPPort;
    private String remoteIPAddress;
    private ISCSISessionConnection.ISCSISessionAuthenticationStyle authenticationMethod;

    CModeISCSISessionConnection(final CModeISCSISession parentSession, final Integer connectionID) {
        super(parentSession, CModeISCSISessionConnection.class.getSimpleName());
        final CModeISCSISession.CModeISCSISessionKey sessionKey = parentSession.getKey();
        final String vServerName = sessionKey.getID(CModeISCSISession.CModeISCSISessionKey.Field.vserver_name);
        final String portalGroupName = sessionKey.getID(CModeISCSISession.CModeISCSISessionKey.Field.portal_group_name);
        final String targetSessionID = sessionKey.getID(CModeISCSISession.CModeISCSISessionKey.Field.target_session_id);
        this.key = new CModeISCSISessionConnectionKey(vServerName, portalGroupName, targetSessionID, String.valueOf(connectionID));
        this.connectionID = connectionID;
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public CModeISCSISessionConnectionKey getKey() {
        return key;
    }

    public String getInterfaceName() {
        return interfaceName;
    }

    public String getLocalIPAddress() {
        return localIPAddress;
    }

    public String getLocalIPPort() {
        return localIPPort;
    }

    public String getRemoteIPAddress() {
        return remoteIPAddress;
    }

    public Integer getConnectionID() {
        return connectionID;
    }

    public ISCSISessionConnection.ISCSISessionAuthenticationStyle getAuthenticationMethod() {
        return authenticationMethod;
    }

    public void setInterfaceName(final String interfaceName) {
        this.interfaceName = interfaceName;
    }

    public void setLocalIPAddress(final String localIPAddress) {
        this.localIPAddress = localIPAddress;
    }

    public void setLocalIPPort(final String localIPPort) {
        this.localIPPort = localIPPort;
    }

    public void setRemoteIPAddress(final String remoteIPAddress) {
        this.remoteIPAddress = remoteIPAddress;
    }

    public void setAuthenticationMethod(final ISCSISessionConnection.ISCSISessionAuthenticationStyle authenticationMethod) {
        this.authenticationMethod = authenticationMethod;
    }

    public static final class CModeISCSISessionConnectionKey extends ModelKeyBase<CModeISCSISessionConnectionKey.Field> {

        public static enum Field {
            vserver_name,
            portal_group_name,
            target_session_id,
            connection_id
        }

        public CModeISCSISessionConnectionKey(final String vServerName, final String portalGroupName, final String sessionID, final String connectionID) {
            super("iscsi_session_connection", new EnumMap<Field, String>(Field.class) {{
                put(Field.vserver_name, vServerName);
                put(Field.portal_group_name, portalGroupName);
                put(Field.target_session_id, sessionID);
                put(Field.connection_id, connectionID);
            }});
        }
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "ID", key.getID());
        addAttributeNoCRC(writer, "SessionID", key.getID(CModeISCSISessionConnectionKey.Field.target_session_id));
        addAttributeNoCRC(writer, "ConnectionID", key.getID(CModeISCSISessionConnectionKey.Field.connection_id));
        addAttributeNoCRC(writer, "InterfaceName", interfaceName);
        addAttributeNoCRC(writer, "LocalIPAddress", localIPAddress);
        addAttributeNoCRC(writer, "LocalIPPort", localIPPort);
        addAttributeNoCRC(writer, "RemoteIPAddress", remoteIPAddress);
        addAttributeNoCRC(writer, "AuthenticationMethod", authenticationMethod);
    }
}
