package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;

public class CModeKerbero extends BaseCRCXMLNode<CModeCluster> {

    private final UUIDKey kerberosKey;          // vserver-name

    private String interfaceName;
    private String ipAddress;
    private Boolean isKerberosEnabled;


    CModeKerbero(final CModeCluster parent, final String vServerName) {
        super(parent, CModeKerbero.class.getSimpleName());
        this.kerberosKey = new UUIDKey("kerberos", vServerName);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public UUIDKey getKey() {
        return kerberosKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getInterfaceName() {
        return interfaceName;
    }

    public void setInterfaceName(final String interfaceName) {
        this.interfaceName = interfaceName;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getIpAddress() {
        return ipAddress;
    }

    // JSD TODO why not used?
    public void setIpAddress(final String ipAddress) {
        this.ipAddress = ipAddress;
    }

    public boolean isKerberosEnabled() {
        return isKerberosEnabled != null && isKerberosEnabled;
    }

    public void setKerberosEnabled(final Boolean kerberosEnabled) {
        isKerberosEnabled = kerberosEnabled;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "UUID", kerberosKey.getUUID());          // key is vserver name
        addAttributeNoCRC(writer, "VServerName", kerberosKey.getUUID());   // key is vserver name
        addAttributeNoCRC(writer, "InterfaceName", interfaceName);
        addAttributeNoCRC(writer, "IpAddress", ipAddress);
        addAttributeNoCRC(writer, "IsKerberosEnabled", isKerberosEnabled);
    }
}
