package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.LUN;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;

public class CModeLUN extends BaseCRCXMLNode<CModeCluster> {

    private final UUIDKey key;                          // uuid
    private EnumWrapper<LUN.Alignment> alignment;       // alignment
    private Boolean isSpaceAllocEnabled;                // is-space-alloc-enabled
    private Boolean isSpaceReservationEnabled;          // is-space-reservation-enabled
    private Boolean mapped;                             // mapped
    private String multiprotocolTypeString;             // multiprotocol-type
    private Boolean online;                             // online
    private String path;                                // path
    private Long prefixSize;                            // prefix-size
    private String qTreeName;                           // qtree
    private String serialNumber;                        // serial-number
    private Long size;                                  // size
    private Long sizeUsed;                              // size-used
    private String volumeName;                          // volume
    private String vServerName;                         // vserver
    private String comment;                             // comment
    private Long suffixSize;                            // suffix-size
    private String qosPolicyGroupName;                  // qos-policy-group
    private EnumWrapper<LUN.LUNClass> lunClass;         // class

    CModeLUN(final CModeCluster parent, final String uuid) {
        super(parent, CModeLUN.class.getSimpleName());
        this.key = new UUIDKey("lun", uuid);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public UUIDKey getKey() {
        return key;
    }

    public EnumWrapper<LUN.Alignment> getAlignment() {
        return alignment;
    }

    public Boolean isSpaceAllocEnabled() {
        return isSpaceAllocEnabled;
    }

    public Boolean isSpaceReservationEnabled() {
        return isSpaceReservationEnabled;
    }

    public Boolean isMapped() {
        return mapped;
    }

    public String getMultiprotocolTypeString() {
        return multiprotocolTypeString;
    }

    public Boolean isOnline() {
        return online;
    }

    public String getPath() {
        return path;
    }

    public Long getPrefixSize() {
        return prefixSize;
    }

    public String getQTreeName() {
        return qTreeName;
    }

    public String getSerialNumber() {
        return serialNumber;
    }

    public Long getSize() {
        return size;
    }

    public Long getSizeUsed() {
        return sizeUsed;
    }

    public String getVolumeName() {
        return volumeName;
    }

    public String getVServerName() {
        return vServerName;
    }

    public String getComment() {
        return comment;
    }

    public Long getSuffixSize() {
        return suffixSize;
    }

    public String getQosPolicyGroupName() {
        return qosPolicyGroupName;
    }

    public void setQosPolicyGroupName(final String qosPolicyGroupName) {
        this.qosPolicyGroupName = qosPolicyGroupName;
    }

    public void setAlignment(final EnumWrapper<LUN.Alignment> alignment) {
        this.alignment = alignment;
    }

    public void setSpaceAllocEnabled(final Boolean spaceAllocEnabled) {
        isSpaceAllocEnabled = spaceAllocEnabled;
    }

    public void setSpaceReservationEnabled(final Boolean spaceReservationEnabled) {
        isSpaceReservationEnabled = spaceReservationEnabled;
    }

    public void setMapped(final Boolean mapped) {
        this.mapped = mapped;
    }

    public void setMultiprotocolTypeString(final String multiprotocolTypeString) {
        this.multiprotocolTypeString = multiprotocolTypeString;
    }

    public void setOnline(final Boolean online) {
        this.online = online;
    }

    public void setPath(final String path) {
        this.path = path;
    }

    public void setPrefixSize(final Long prefixSize) {
        this.prefixSize = prefixSize;
    }

    public void setQTreeName(final String qTreeName) {
        this.qTreeName = qTreeName;
    }

    public void setSerialNumber(final String serialNumber) {
        this.serialNumber = serialNumber;
    }

    public void setSize(final Long size) {
        this.size = size;
    }

    public void setSizeUsed(final Long sizeUsed) {
        this.sizeUsed = sizeUsed;
    }

    public void setVolumeName(final String volumeName) {
        this.volumeName = volumeName;
    }

    public void setVServerName(final String vServerName) {
        this.vServerName = vServerName;
    }

    public void setComment(final String comment) {
        this.comment = comment;
    }

    public void setSuffixSize(final Long suffixSize) {
        this.suffixSize = suffixSize;
    }


    public EnumWrapper<LUN.LUNClass> getLunClass() {
        return lunClass;
    }

    public void setLunClass(final EnumWrapper<LUN.LUNClass> lunClass) {
        this.lunClass = lunClass;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "UUID", key.getID());
        addAttributeNoCRC(writer, "QtreeRef", qTreeName);
        addAttributeNoCRC(writer, "VolumeRef", volumeName);
        addAttributeNoCRC(writer, "VserverRef", vServerName);
        addAttributeNoCRC(writer, "Path", path);
        addAttributeNoCRC(writer, "IsOnline", online);
        addAttributeNoCRC(writer, "Size", size);
        addAttributeNoCRC(writer, "SizeUsed", sizeUsed);
        addAttributeNoCRC(writer, "IsSpaceReservationEnabled", isSpaceReservationEnabled);
        addAttributeNoCRC(writer, "IsSpaceAllocEnabled", isSpaceAllocEnabled);
        addAttributeNoCRC(writer, "IsMapped", mapped);
        addAttributeNoCRC(writer, "SerialNumber", serialNumber);
        addAttributeNoCRC(writer, "MultiProtocolType", multiprotocolTypeString);
        addAttributeNoCRC(writer, "Alignment", alignment.getEnumValue());
        addAttributeNoCRC(writer, "AlignmentRaw", alignment.getRawValue());
        addAttributeNoCRC(writer, "PrefixSize", prefixSize);
        addAttributeNoCRC(writer, "SuffixSize", suffixSize);
        addAttributeNoCRC(writer, "Comment", comment);
        addAttributeNoCRC(writer, "QosPolicyGroupName", qosPolicyGroupName);
        addAttributeNoCRC(writer, "Class", lunClass.getEnumValue());
        addAttributeNoCRC(writer, "ClassRaw", lunClass.getRawValue());
    }
}
