package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.License;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Node;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.Map;

/**
 * License information for Sierranevada (SN) and later.
 */
public class CModeLicenseV2 extends CModeLicense {

    /**
     * Name of the licensed package.
     */
    private final EnumWrapper<LicensedPackage> _package;
    /**
     * Description of the licensed package.
     */
    private String description;
    /**
     * License expiration time in number of seconds since January 1, 1970. Range : [0..2^31-1].
     */
    private Integer expirationTime;
    /**
     * A legacy license indicates that the license was previously installed prior to this release. <tt>true</tt> if
     * the license was installed prior to this release and <tt>false</tt> otherwise.
     */
    private Boolean legacy;
    /**
     * Controller or Cluster that owns the serial number.
     */
    private String owner;
    /**
     * License type.
     */
    private EnumWrapper<License.LicenseType> type;

    CModeLicenseV2(final CModeCluster cluster, final EnumWrapper<LicensedPackage> _package, final String serialNumber) {
        super(cluster, CModeLicenseV2.class.getSimpleName(), _package.getRawValue(), serialNumber);
        this._package = _package;
    }

    public EnumWrapper<LicensedPackage> getPackage() {
        return _package;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(final String description) {
        this.description = description;
    }

    public Integer getExpirationTime() {
        return expirationTime;
    }

    public void setExpirationTime(final Integer expirationTime) {
        this.expirationTime = expirationTime;
    }

    public Boolean getLegacy() {
        return legacy;
    }

    public void setLegacy(final Boolean legacy) {
        this.legacy = legacy;
    }

    public String getOwner() {
        return owner;
    }

    public void setOwner(final String owner) {
        this.owner = owner;
    }

    public EnumWrapper<License.LicenseType> getType() {
        return type;
    }

    public void setType(final EnumWrapper<License.LicenseType> type) {
        this.type = type;
    }

    public static enum LicensedPackage {
        NOT_MAPPED,
        BASE,
        NFS,
        CIFS,
        ISCSI,
        FCP,
        CDMI,
        SNAPRESTORE,
        SNAPMIRROR,
        FLEXCLONE,
        SNAPVAULT,
        SNAPLOCK,
        SNAPMANAGERSUITE,
        SNAPPROTECTAPPS,
        V_STORAGEATTACH,
        SNAPLOCK_ENTERPRISE,
        INSIGHT_BALANCE,
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "Package", getPackage().getEnumValue());
        addAttributeNoCRC(writer, "PackageRaw", getPackage().getRawValue());
        addAttributeNoCRC(writer, "Description", getDescription());
        addAttributeNoCRC(writer, "ExpirationTime", getExpirationTime());
        addAttributeNoCRC(writer, "Legacy", getLegacy());
        addAttributeNoCRC(writer, "Owner", getOwner());
        addAttributeNoCRC(writer, "LicenseType", getType().getEnumValue());
        addAttributeNoCRC(writer, "LicenseTypeRaw", getType().getRawValue());
    }

    @Override
    public void toLicense(final License license, final Map<String, Node> nodesByName) {
        final String owner = getOwner();
        license.setOwner(owner);
        // Figure out whether owner is cluster or node
        if (getParent().getClusterName().equals(owner)) {
            // Owner is cluster
            license.setOwnerType(License.OwnerType.CLUSTER);
        }
        else {
            // Owner is not cluster. Find node with matching name.
            final Node node = nodesByName.get(owner);
            if (node != null) {
                license.setOwnerNode(node);
                license.setOwnerType(License.OwnerType.NODE);
            }
            else {
                license.setOwnerType(License.OwnerType.UNKNOWN);
            }
        }
        license.setExpirationTime(getExpirationTime());
        license.setDescription(getDescription());

        license.setLicenseType(getType().getEnumValue());
        license.setLicenseTypeRaw(getType().getRawValue());
    }
}
