package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Lif;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModeLif extends BaseCRCXMLNode<CModeCluster> {
    private final LifKey lifKey;
    private String lifUUID;
    private String currentNode;
    private String currentPort;
    private EnumWrapper<Lif.FailoverPolicy> failoverPolicy = null;
    private String homeNode;
    private String homePort;
    private Boolean isAutoRevert;
    private Boolean isHome;
    private String ipAddress;
    private String netMask;
    private Integer netmaskLen;
    private String dnsDomainName;
    private String routingGroupName;
    private EnumWrapper<Lif.Status> operStatus = null;
    private EnumWrapper<Lif.Status> adminStatus = null;
    private EnumWrapper<Lif.Role> role = null;
    private String failoverGroupName;
    private String dataProtocolList;
    private Boolean isDNSUpdateEnabled;
    private Boolean listenForDNSQuery;

    CModeLif(final CModeCluster parent, final String lifName, final String vServerName) {
        super(parent, CModeLif.class.getSimpleName());
        this.lifKey = new LifKey(vServerName, lifName);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public LifKey getKey() {
        return lifKey;
    }

    public String getInterfaceName() {
        return lifKey.getID(LifKey.Field.interface_name);
    }

    public String getLifUUID() {
        return lifUUID;
    }

    public void setLifUUID(final String lifUUID) {
        this.lifUUID = lifUUID;
    }

    public String getCurrentNode() {
        return currentNode;
    }

    public void setCurrentNode(final String currentNode) {
        this.currentNode = currentNode;
    }

    public String getCurrentPort() {
        return currentPort;
    }

    public void setCurrentPort(final String currentPort) {
        this.currentPort = currentPort;
    }

    public EnumWrapper<Lif.FailoverPolicy> getFailoverPolicy() {
        return failoverPolicy;
    }

    public String getHomeNode() {
        return homeNode;
    }

    public void setHomeNode(final String homeNode) {
        this.homeNode = homeNode;
    }

    public String getHomePort() {
        return homePort;
    }

    public void setHomePort(final String homePort) {
        this.homePort = homePort;
    }

    public Boolean getIsDNSUpdateEnabled() {
        return isDNSUpdateEnabled;
    }

    public void setIsDNSUpdateEnabled(final Boolean isDNSUpdateEnabled) {
        this.isDNSUpdateEnabled = isDNSUpdateEnabled;
    }
    
    public Boolean getListenForDNSQuery() {
        return listenForDNSQuery;
    }

    public void setListenForDNSQuery(final Boolean listenForDNSQuery) {
        this.listenForDNSQuery = listenForDNSQuery;
    }

    public Boolean isHome() {
        return isHome;
    }

    public void setHome(final Boolean home) {
        isHome = home;
    }

    public String getIpAddress() {
        return ipAddress;
    }

    public void setIpAddress(final String ipAddress) {
        this.ipAddress = ipAddress;
    }

    public String getNetMask() {
        return netMask;
    }

    public void setNetMask(final String netMask) {
        this.netMask = netMask;
    }

    public Integer getNetmaskLen() {
        return netmaskLen;
    }

    public void setNetmaskLen(final Integer netmaskLen) {
        this.netmaskLen = netmaskLen;
    }

    public String getDnsDomainName() {
        return dnsDomainName;
    }

    public void setDnsDomainName(final String dnsDomainName) {
        this.dnsDomainName = dnsDomainName;
    }

    public String getRoutingGroupName() {
        return routingGroupName;
    }

    public void setRoutingGroupName(final String routingGroupName) {
        this.routingGroupName = routingGroupName;
    }

    public EnumWrapper<Lif.Status> getOperStatus() {
        return operStatus;
    }

    public void setOperStatus(final EnumWrapper<Lif.Status> operStatus) {
        this.operStatus = operStatus;
    }

    public EnumWrapper<Lif.Status> getAdminStatus() {
        return adminStatus;
    }

    public void setAdminStatus(final EnumWrapper<Lif.Status> adminStatus) {
        this.adminStatus = adminStatus;
    }

    public EnumWrapper<Lif.Role> getRole() {
        return role;
    }

    public void setRole(final EnumWrapper<Lif.Role> role) {
        this.role = role;
    }

    public String getFailoverGroupName() {
        return failoverGroupName;
    }

    public void setFailoverGroupName(final String failoverGroupName) {
        this.failoverGroupName = failoverGroupName;
    }

    public String getDataProtocolList() {
        return dataProtocolList;
    }

    public void setDataProtocolList(final String dataProtocolList) {
        this.dataProtocolList = dataProtocolList;
    }

    public void setFailoverPolicy(final EnumWrapper<Lif.FailoverPolicy> failoverPolicy) {
        this.failoverPolicy = failoverPolicy;
    }
    
    public Boolean isAutoRevert() {
        return isAutoRevert;
    }

    public void setAutoRevert(final Boolean autoRevert) {
        isAutoRevert = autoRevert;
    }

    public final static class LifKey extends ModelKeyBase<LifKey.Field> {
        public static enum Field {
            vserver_name,
            interface_name
        }

        public LifKey(final String vServerName, final String lifUUID) {
            super("lif", new EnumMap<Field, String>(Field.class) {{
                put(Field.vserver_name, vServerName);
                put(Field.interface_name, lifUUID);
            }});
        }
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "VServerName", lifKey.getID(LifKey.Field.vserver_name));
        addAttributeNoCRC(writer, "LifName", getInterfaceName());
        addAttributeNoCRC(writer, "lifUUID", lifUUID);
        addAttributeNoCRC(writer, "CurrentNode", currentNode);
        addAttributeNoCRC(writer, "CurrentPort", currentPort);
        addAttributeNoCRC(writer, "FailoverPolicy", failoverPolicy.getEnumValue());
        addAttributeNoCRC(writer, "FailoverPolicyRaw", failoverPolicy.getRawValue());
        addAttributeNoCRC(writer, "HomeNode", homeNode);
        addAttributeNoCRC(writer, "HomePort", homePort);
        addAttributeNoCRC(writer, "IsAutoRevert", isAutoRevert);
        addAttributeNoCRC(writer, "IsHome", isHome);
        addAttributeNoCRC(writer, "IpAddress", ipAddress);
        addAttributeNoCRC(writer, "Netmask", netMask);
        addAttributeNoCRC(writer, "NetmaskLength", netmaskLen);
        addAttributeNoCRC(writer, "DNSDomainName", dnsDomainName);
        addAttributeNoCRC(writer, "RoutingGroupName", routingGroupName);
        addAttributeNoCRC(writer, "OperStatus", operStatus.getEnumValue());
        addAttributeNoCRC(writer, "OperStatusRaw", operStatus.getRawValue());
        addAttributeNoCRC(writer, "AdminStatus", adminStatus.getEnumValue());
        addAttributeNoCRC(writer, "AdminStatusRaw", adminStatus.getRawValue());
        addAttributeNoCRC(writer, "Role", role.getEnumValue());
        addAttributeNoCRC(writer, "RoleRaw", role.getRawValue());
        addAttributeNoCRC(writer, "FailoverGroup", failoverGroupName);
        addAttributeNoCRC(writer, "IsDNSUpdateEnabled", isDNSUpdateEnabled);
        addAttributeNoCRC(writer, "ListenForDNSQuery", listenForDNSQuery);
        addAttributeNoCRC(writer, "ProtocolList", dataProtocolList);
    }
}
