package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.LunImport;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import java.io.IOException;
import java.util.EnumMap;

public class CModeLunImport extends BaseCRCXMLNode<CModeCluster> {

    private final CModeLunImportKey key;                         
    private EnumWrapper<LunImport.AdministrativeStatus> administrativeStatus;      
    private Long comparedBlocks;                
    private Long estimatedRemainingDuration;        
    private String failureReason; 
    private String foreignDisk;           
    private String importCurrentNode;
    private String importHomeNode;
    private Long importedBlocks;
    private EnumWrapper<LunImport.OperationInProgress> operationInProgress;
    private EnumWrapper<LunImport.OperationalStatus> operationalStatus;
    private String path;                           
    private Long percentComplete;
    private Long totalBlocks;
    private String vServer;       

    CModeLunImport(final CModeCluster parent, final String vServerName, final String foreignDisk) {
        super(parent, CModeLunImport.class.getSimpleName());
        this.key = new CModeLunImportKey(vServerName, foreignDisk);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public CModeLunImportKey getKey() {
        return key;
    }

    public EnumWrapper<LunImport.AdministrativeStatus> getAdministrativeStatus() {
        return administrativeStatus;
    }
    
    public void setAdministrativeStatus(EnumWrapper<LunImport.AdministrativeStatus> administrativeStatus) {
        this.administrativeStatus = administrativeStatus;
    }
    
    public Long getComparedBlocks() {
        return comparedBlocks;
    }
    
    public void setComparedBlocks(Long comparedBlocks) {
        this.comparedBlocks = comparedBlocks;
    }
    
    public Long getEstimatedRemainingDuration() {
        return estimatedRemainingDuration;
    }
    
    public void setEstimatedRemainingDuration(Long estimatedRemainingDuration) {
        this.estimatedRemainingDuration = estimatedRemainingDuration;
    }
    
    public String getFailureReason() {
        return failureReason;
    }
    
    public void setFailureReason(String failureReason) {
        this.failureReason = failureReason;
    }
    
    public String getForeignDisk() {
        return foreignDisk;
    }
    
    public void setForeignDisk(String foreignDisk) {
        this.foreignDisk = foreignDisk;
    }
    
    public String getImportCurrentNode() {
        return importCurrentNode;
    }
    
    public void setImportCurrentNode(String importCurrentNode) {
        this.importCurrentNode = importCurrentNode;
    }
    
    public String getImportHomeNode() {
        return importHomeNode;
    }
    
    public void setImportHomeNode(String importHomeNode) {
        this.importHomeNode = importHomeNode;
    }
    
    public Long getImportedBlocks() {
        return importedBlocks;
    }
    
    public void setImportedBlocks(Long importedBlocks) {
        this.importedBlocks = importedBlocks;
    }
    
    public EnumWrapper<LunImport.OperationInProgress> getOperationInProgress() {
        return operationInProgress;
    }
    
    public void setOperationInProgress(EnumWrapper<LunImport.OperationInProgress> operationInProgress) {
        this.operationInProgress = operationInProgress;
    }
    
    public EnumWrapper<LunImport.OperationalStatus> getOperationalStatus() {
        return operationalStatus;
    }
    
    public void setOperationalStatus(EnumWrapper<LunImport.OperationalStatus> operationalStatus) {
        this.operationalStatus = operationalStatus;
    }
    
    public String getPath() {
        return path;
    }

    public void setPath(String path) {
        this.path = path;
    }
    
    public Long getPercentComplete() {
        return percentComplete;
    }
    
    public void setPercentComplete(Long percentComplete) {
        this.percentComplete = percentComplete;
    }
    
    public Long getTotalBlocks() {
        return totalBlocks;
    }
    
    public void setTotalBlocks(Long totalBlocks) {
        this.totalBlocks = totalBlocks;
    }
    
    public String getVServer() {
        return vServer;
    }
    
    public void setVServer(String vServer) {
        this.vServer = vServer;
    }

    public final static class CModeLunImportKey extends ModelKeyBase<CModeLunImportKey.Field> {
        public static enum Field {
            vserver_name,
            foreign_disk
        }

        public CModeLunImportKey(final String vserverName, final String  foreignDisk) {
            super("lunimport", new EnumMap<Field, String>(Field.class) {{
      
                put(Field.vserver_name, vserverName);
                put(Field.foreign_disk, foreignDisk);
            }});
        }
    }
    
    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "UUID", key.getID());
        addAttributeNoCRC(writer, "AdministrativeStatus", administrativeStatus.getEnumValue());
        addAttributeNoCRC(writer, "AdministrativeStatusRaw", administrativeStatus.getRawValue());
        addAttributeNoCRC(writer, "ComparedBlocks", comparedBlocks);
        addAttributeNoCRC(writer, "EstimatedRemainingDuration", estimatedRemainingDuration);
        addAttributeNoCRC(writer, "FailureReason", failureReason);
        addAttributeNoCRC(writer, "ForeignDisk", foreignDisk);
        addAttributeNoCRC(writer, "ImportCurrentNode", importCurrentNode);
        addAttributeNoCRC(writer, "ImportHomeNode", importHomeNode);
        addAttributeNoCRC(writer, "ImportedBlocks", importedBlocks);
        addAttributeNoCRC(writer, "OperationInProgress", operationInProgress.getEnumValue());
        addAttributeNoCRC(writer, "OperationInProgressRaw", operationInProgress.getRawValue());
        addAttributeNoCRC(writer, "OperationalStatus", operationalStatus.getEnumValue());
        addAttributeNoCRC(writer, "OperationalStatusRaw", operationalStatus.getRawValue());
        addAttributeNoCRC(writer, "Path", path);
        addAttributeNoCRC(writer, "PercentComplete", percentComplete);
        addAttributeNoCRC(writer, "TotalBlocks", totalBlocks);
        addAttributeNoCRC(writer, "VServer", vServer);
    }
}
