package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.NetworkPort;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModeNetInterface extends BaseCRCXMLNode<CModeCluster> {
    private final IFKey netPortKey;
    private EnumWrapper<NetworkPort.PortType> portType;
    private EnumWrapper<NetworkPort.Role> portRole;
    private EnumWrapper<NetworkPort.LinkStatus> linkStatus;
    private String macAddress;
    private EnumWrapper<NetworkPort.Speed> operSpeed;
    private EnumWrapper<NetworkPort.Speed> adminSpeed;
    private Long mtu;
    private Long vlanID;
    private String vlanNode;
    private String vlanPort;
    private String ifGroupNode;
    private String ifGroupPort;
    private String networkPortBroadcastDomainName;
    private String networkIpSpaceName;
    private EnumWrapper<NetworkPort.OperationalDuplex> operationalDuplex;

    CModeNetInterface(final CModeCluster parent, final String nodeName, final String portName) {
        super(parent, CModeNetInterface.class.getSimpleName());
        this.netPortKey = new IFKey(nodeName, portName);
    }


    @Override
    @SuppressWarnings({"unchecked"})
    public IFKey getKey() {
        return netPortKey;
    }

    public EnumWrapper<NetworkPort.PortType> getPortType() {
        return portType;
    }

    public EnumWrapper<NetworkPort.Role> getPortRole() {
        return portRole;
    }

    public EnumWrapper<NetworkPort.LinkStatus> getLinkStatus() {
        return linkStatus;
    }

    public String getMacAddress() {
        return macAddress;
    }

    public EnumWrapper<NetworkPort.Speed> getOperSpeed() {
        return operSpeed;
    }

    public EnumWrapper<NetworkPort.Speed> getAdminSpeed() {
        return adminSpeed;
    }

    public Long getMTU() {
        return mtu;
    }

    public Long getVlanID() {
        return vlanID;
    }

    public String getVlanPort() {
        return vlanPort;
    }

    public String getIfGroupPort() {
        return ifGroupPort;
    }

    public EnumWrapper<NetworkPort.OperationalDuplex> getOperationalDuplex() {
        return operationalDuplex;
    }

    public void setPortType(final EnumWrapper<NetworkPort.PortType> portType) {
        this.portType = portType;
    }

    public void setPortRole(final EnumWrapper<NetworkPort.Role> portRole) {
        this.portRole = portRole;
    }

    public void setLinkStatus(final EnumWrapper<NetworkPort.LinkStatus> linkStatus) {
        this.linkStatus = linkStatus;
    }

    public void setMacAddress(final String macAddress) {
        this.macAddress = macAddress;
    }

    public void setOperSpeed(final EnumWrapper<NetworkPort.Speed> operSpeed) {
        this.operSpeed = operSpeed;
    }

    public void setAdminSpeed(final EnumWrapper<NetworkPort.Speed> adminSpeed) {
        this.adminSpeed = adminSpeed;
    }

    public void setMtu(final Long mtu) {
        this.mtu = mtu;
    }

    public void setVlanID(final Long vlanID) {
        this.vlanID = vlanID;
    }

    public void setVlanPort(final String vlanPort) {
        this.vlanPort = vlanPort;
    }

    public void setIfGroupPort(final String ifGroupPort) {
        this.ifGroupPort = ifGroupPort;
    }

    public void setOperationalDuplex(final EnumWrapper<NetworkPort.OperationalDuplex> operationalDuplex) {
        this.operationalDuplex = operationalDuplex;
    }

    public void setVlanNode(final String vlanNode) {
        this.vlanNode = vlanNode;
    }

    public void setIfGroupNode(final String ifGroupNode) {
        this.ifGroupNode = ifGroupNode;
    }

    public String getVlanNode() {
        return vlanNode;
    }

    public String getIfGroupNode() {
        return ifGroupNode;
    }

    public String getNetworkPortBroadcastDomainName() {
        return networkPortBroadcastDomainName;
    }

    public void setNetworkPortBroadcastDomainName(String networkPortBroadcastDomainName) {
        this.networkPortBroadcastDomainName = networkPortBroadcastDomainName;
    }

    public String getNetworkIpSpaceName() {
        return networkIpSpaceName;
    }

    public void setNetworkIpSpaceName(String networkIpSpaceName) {
        this.networkIpSpaceName = networkIpSpaceName;
    }

    public final static class IFKey extends ModelKeyBase<IFKey.Field> {
        public static enum Field {
            node_name,
            interface_name
        }

        public IFKey(final String nodeName, final String interfaceName) {
            super("net_interface", new EnumMap<Field, String>(Field.class) {{
                put(Field.node_name, nodeName);
                put(Field.interface_name, interfaceName);
            }});
        }
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "ID", netPortKey.getID());
        addAttributeNoCRC(writer, "NodeName", netPortKey.getID(IFKey.Field.node_name));
        addAttributeNoCRC(writer, "Name", netPortKey.getID(IFKey.Field.interface_name));
        addAttributeNoCRC(writer, "MacAddress", macAddress);
        addAttributeNoCRC(writer, "Type", EnumWrapper.toStringOrNull(portType));
        addAttributeNoCRC(writer, "Role", EnumWrapper.toStringOrNull(portRole));
        addAttributeNoCRC(writer, "LinkStatus", EnumWrapper.toStringOrNull(linkStatus));
        addAttributeNoCRC(writer, "OperSpeed", EnumWrapper.toStringOrNull(operSpeed));
        addAttributeNoCRC(writer, "AdminSpeed", EnumWrapper.toStringOrNull(adminSpeed));
        addAttributeNoCRC(writer, "MTU", mtu);
        addAttributeNoCRC(writer, "VlanID", vlanID);
        addAttributeNoCRC(writer, "VlanNode", vlanNode);
        addAttributeNoCRC(writer, "VlanPort", vlanPort);
        addAttributeNoCRC(writer, "IfGroupNode", ifGroupNode);
        addAttributeNoCRC(writer, "IfGroupPort", ifGroupPort);
        addAttributeNoCRC(writer, "NetworkIPSpaceName", networkIpSpaceName);
        addAttributeNoCRC(writer, "NetworkPortBroadcastDomainName", networkPortBroadcastDomainName);
        addAttributeNoCRC(writer, "OperationalDuplex", EnumWrapper.toStringOrNull(operationalDuplex));
    }
}

