package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.RoutingGroup;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModeNetRoute extends BaseCRCXMLNode<CModeCluster> {

    // Represents a routing group entry

	private final NetRouteKey key;

    private EnumWrapper<RoutingGroup.AddressFamily> addressFamily;
    private String gatewayAddress;
    private Long metric;

    public CModeNetRoute(final CModeCluster parent, final String vServerUUID, final String routingGroupName, final String destinationAddress) {
        super(parent, CModeNetRoute.class.getSimpleName());
        this.key = new NetRouteKey(vServerUUID, routingGroupName, destinationAddress);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public NetRouteKey getKey() {
        return key;
    }

    public String getDestinationAddress() {
        return key.getID(NetRouteKey.Field.destination_address);
    }

    public EnumWrapper<RoutingGroup.AddressFamily> getAddressFamily() {
        return addressFamily;
    }

    public void setAddressFamily(final EnumWrapper<RoutingGroup.AddressFamily> addressFamily) {
        this.addressFamily = addressFamily;
    }

    public String getRoutingGroupName() {
        return key.getID(NetRouteKey.Field.routing_group_name);
    }

    public String getGatewayAddress() {
        return gatewayAddress;
    }

    public void setGatewayAddress(final String gatewayAddress) {
        this.gatewayAddress = gatewayAddress;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getMetric() {
        return metric;
    }

    public void setMetric(final Long metric) {
        this.metric = metric;
    }

    public final static class NetRouteKey extends ModelKeyBase<NetRouteKey.Field> {

        public static enum Field {
            vserver_name,
            routing_group_name,
            destination_address,
        }

        public NetRouteKey(final String vServerName, final String routingGroupName, final String destinationAddress) {
            super("net_route", new EnumMap<Field, String>(Field.class) {{
                put(Field.vserver_name, vServerName);
                put(Field.routing_group_name, routingGroupName);
                put(Field.destination_address, destinationAddress);
            }});
        }
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "ID", key.getID());
        addAttributeNoCRC(writer, "VServerName", key.getID(NetRouteKey.Field.vserver_name));
        addAttributeNoCRC(writer, "RoutingGroupName", getRoutingGroupName());
        addAttributeNoCRC(writer, "AddressFamily", addressFamily.getEnumValue());
        addAttributeNoCRC(writer, "AddressFamilyRaw", addressFamily.getRawValue());
        addAttributeNoCRC(writer, "DestinationAddress", getDestinationAddress());
        addAttributeNoCRC(writer, "GatewayAddress", gatewayAddress);
        addAttributeNoCRC(writer, "Metric", metric);
    }
}