package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;


import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

/**
 * Data object for holding the results form the net-failover-group-get-iter ZAPI.
 */
public class CModeNetworkFailoverGroup extends BaseCRCXMLNode<CModeCluster> {

    /**
     * Key using the failover group name and the cluster uuid.
     */
    private final NetworkFailoverGroupKey networkFailoverGroupKey;
    /**
     * Broadcast Domain Name.
     */
    private String networkPortBroadcastDomainName;

  /**
   * vserver UUID
   */
    private String vServer;

    /**
     * Network failover group name
     */
    private String networkFailoverGroupName;


    protected CModeNetworkFailoverGroup(CModeCluster parent, String failoverGroupName) {
        super(parent, CModeNetworkFailoverGroup.class.getSimpleName());
        this.networkFailoverGroupKey = new NetworkFailoverGroupKey(parent.getClusterUUID(), failoverGroupName);
        this.networkFailoverGroupName = failoverGroupName;
    }

    @Override
    public NetworkFailoverGroupKey getKey() {
        return networkFailoverGroupKey;
    }

    public String getVServer() {
        return vServer;
    }

    public void setVServer(String vServer) {
        this.vServer = vServer;
    }

    public String getNetworkPortBroadcastDomainName() {
        return networkPortBroadcastDomainName;
    }

    public void setNetworkPortBroadcastDomainName(String networkPortBroadcastDomainName) {
        this.networkPortBroadcastDomainName = networkPortBroadcastDomainName;
    }

    public String getNetworkFailoverGroupName() {
        return networkFailoverGroupName;
    }

    /**
     * OnTap does not provide a UUID for failover group. Create a new key using the cluster uuid and the failover
     * group name.
     */
    public static final class NetworkFailoverGroupKey extends ModelKeyBase<NetworkFailoverGroupKey.Field> {
        public static enum Field {
            cluster_uuid,
            failover_group_name
        }

        public NetworkFailoverGroupKey(final String clusterUUID, final String failoverGroupName) {
            super("failover_group", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.failover_group_name, failoverGroupName);
            }});
        }
    }

    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer)
            throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "Name", networkFailoverGroupKey.getID(NetworkFailoverGroupKey.Field.failover_group_name));
        addAttributeNoCRC(writer, "BroadcastDomainName", networkPortBroadcastDomainName);
        addAttributeNoCRC(writer, "VServer", vServer);
    }
}
