package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

/**
 * Data object for holding the results form the net-subnet-get-iter ZAPI.
 */
public class CModeNetworkSubnet extends BaseCRCXMLNode<CModeCluster> {
    private final NetworkSubnetKey subnetKey;

    private int availableCount;
    private String broadcastDomain;
    private Boolean forceUpdateLifAssociations;
    private String gateway;
    private String ipRangesCSVString;
    private String ipspace;
    private String subnet;
    private String name;
    private int totalCount;
    private int usedCount;

    CModeNetworkSubnet(final CModeCluster parent, final String ipspace, final String name) {
        super(parent, CModeNetworkSubnet.class.getSimpleName());
        this.subnetKey = new NetworkSubnetKey(parent.getClusterUUID(), ipspace, name);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public NetworkSubnetKey getKey() {
        return subnetKey;
    }

    public int getAvailableCount() {
        return availableCount;
    }

    public void setAvailableCount(int availableCount) {
        this.availableCount = availableCount;
    }

    public String getBroadcastDomain() {
        return broadcastDomain;
    }

    public void setBroadcastDomain(String broadcastDomain) {
        this.broadcastDomain = broadcastDomain;
    }

    public Boolean getForceUpdateLifAssociations() {
        return forceUpdateLifAssociations;
    }

    public void setForceUpdateLifAssociations(Boolean forceUpdateLifAssociations) {
        this.forceUpdateLifAssociations = forceUpdateLifAssociations;
    }

    public String getGateway() {
        return gateway;
    }

    public void setGateway(String gateway) {
        this.gateway = gateway;
    }

    public String getIpRanges() {
        return ipRangesCSVString;
    }

    public void setIpRanges(String ipRanges) {
        this.ipRangesCSVString = ipRanges;
    }

    public String getIpspace() {
        return ipspace;
    }

    public void setIpspace(String ipspace) {
        this.ipspace = ipspace;
    }

    public String getSubnet() {
        return subnet;
    }

    public void setSubnet(String subnet) {
        this.subnet = subnet;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public int getTotalCount() {
        return totalCount;
    }

    public void setTotalCount(int totalCount) {
        this.totalCount = totalCount;
    }

    public int getUsedCount() {
        return usedCount;
    }

    public void setUsedCount(int usedCount) {
        this.usedCount = usedCount;
    }

    /**
     * OnTap does not provide a UUID for subnet. Create a new key using cluster uuid, ip space name and subnetname
     *
     */
    public static final class NetworkSubnetKey extends ModelKeyBase<NetworkSubnetKey.Field> {
        public static enum Field {
            cluster_uuid,
            ip_space_name,
            name,
        }

        public NetworkSubnetKey(final String clusterUUID, final String ipSpaceName, final String subnetName) {
            super("subnet", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.ip_space_name, ipSpaceName);
                put(Field.name, subnetName);
            }});
        }
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "Name", name);
        addAttributeNoCRC(writer, "UUID", subnetKey.getOcieUUID());
        addAttributeNoCRC(writer, "AvailableCount", availableCount);
        addAttributeNoCRC(writer, "BroadcastDomain", broadcastDomain);
        addAttributeNoCRC(writer, "ForceUpdateLifAssociations", forceUpdateLifAssociations);
        addAttributeNoCRC(writer, "Gateway", gateway);
        addAttributeNoCRC(writer, "IpRanges", ipRangesCSVString);
        addAttributeNoCRC(writer, "Ipspace", ipspace);
        addAttributeNoCRC(writer, "Subnet", subnet);
        addAttributeNoCRC(writer, "TotalCount", totalCount);
        addAttributeNoCRC(writer, "UsedCount", usedCount);
    }
}
