package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation.BuilderUtil;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.List;

public class CModeNode extends BaseCRCXMLNode<CModeCluster> {

    // system-info from system-get-node-info-iter
    private final UUIDKey nodeKey;              // node-uuid
    private String cpuTypeString;               // cpu-processor-type
    private String cpuIDString;                 // cpu-processor-id             (doc'ed but not seen)
    private Integer numCPUs;                    // number-of-processors
    private Integer memSize;                    // memory-size
    private String productType;                 // prod-type                    (doc'ed but not seen)
    private String systemID;                    // system-id
    private String partnerID;                   // partner-system-id

    // cf-status
    private String partnerName;
    private Boolean isFailoverEnabled;
    private String failoverState;
    private Boolean isTakeoverPossible;
    private String partnerFirmwareState;
    private String localFirmwareState;
    private Boolean isInterconnectUp;
    private String interconnectLinks;
    private String givebackState;
    private String interconnectName;
    private String interconnectType;
    private String currentMode;
    private String takeoverFailureReason;
    private List<String> takeoverByPartnerNotPossibleReasons;
    private List<String> takeoverOfPartnerNotPossibleReasons;

    private final CModeServiceProcessors serviceProcessors;
    private final CModeStorageShelves storageShelves;
    private final CModeStorageShelfBays storageShelfBays;
    private final CModeStorageShelfChannels storageShelfChannels;
    private final CModeFlashDevices flashDevices;

    private CModeProcessors processors;


    CModeNode(final CModeCluster parent, final String systemName) {
        super(parent, CModeNode.class.getSimpleName());
        this.nodeKey = new UUIDKey("node", systemName);
        this.serviceProcessors = new CModeServiceProcessors(this);
        this.storageShelves = new CModeStorageShelves(this);
        this.storageShelfBays = new CModeStorageShelfBays(this);
        this.storageShelfChannels = new CModeStorageShelfChannels(this);
        this.flashDevices = new CModeFlashDevices(this);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public UUIDKey getKey() {
        return nodeKey;
    }

    public String getCpuTypeString() {
        return cpuTypeString;
    }

    public void setCpuTypeString(final String cpuTypeString) {
        this.cpuTypeString = cpuTypeString;
    }

    public String getCpuIDString() {
        return cpuIDString;
    }

    public void setCpuIDString(final String cpuIDString) {
        this.cpuIDString = cpuIDString;
    }

    public Integer getNumCPUs() {
        return numCPUs;
    }

    public void setNumCPUs(final Integer numCPUs) {
        this.numCPUs = numCPUs;
        this.processors = new CModeProcessors(this);
        for (int i = 0; i < numCPUs; i++) {
            this.processors.add(createProcessor(i));
        }
    }

    public Integer getMemSize() {
        return memSize;
    }

    public void setMemSize(final Integer memSize) {
        this.memSize = memSize;
    }

    public String getProductType() {
        return productType;
    }

    public void setProductType(final String productType) {
        this.productType = productType;
    }

    public String getSystemID() {
        return systemID;
    }

    public void setSystemID(final String systemID) {
        this.systemID = systemID;
    }

    public CModeProcessors getProcessors() {
        return processors;
    }

    public CModeProcessor createProcessor(final Integer processorNumber) {
        return new CModeProcessor(this, processorNumber);
    }

    public String getPartnerID() {
        return partnerID;
    }

    public void setPartnerID(final String partnerID) {
        this.partnerID = partnerID;
    }

    public String getPartnerName() {
        return partnerName;
    }

    public void setPartnerName(final String partnerName) {
        this.partnerName = partnerName;
    }

    public Boolean isFailoverEnabled() {
        return isFailoverEnabled;
    }

    public void setFailoverEnabled(final Boolean failoverEnabled) {
        isFailoverEnabled = failoverEnabled;
    }

    public String getFailoverState() {
        return failoverState;
    }

    public void setFailoverState(final String failoverState) {
        this.failoverState = failoverState;
    }

    public Boolean isTakeoverPossible() {
        return isTakeoverPossible;
    }

    public void setTakeoverPossible(final Boolean takeoverPossible) {
        isTakeoverPossible = takeoverPossible;
    }

    public String getPartnerFirmwareState() {
        return partnerFirmwareState;
    }

    public void setPartnerFirmwareState(final String partnerFirmwareState) {
        this.partnerFirmwareState = partnerFirmwareState;
    }

    public String getLocalFirmwareState() {
        return localFirmwareState;
    }

    public void setLocalFirmwareState(final String localFirmwareState) {
        this.localFirmwareState = localFirmwareState;
    }

    public Boolean isInterconnectUp() {
        return isInterconnectUp;
    }

    public void setInterconnectUp(final Boolean interconnectUp) {
        isInterconnectUp = interconnectUp;
    }

    public String getInterconnectLinks() {
        return interconnectLinks;
    }

    public void setInterconnectLinks(final String interconnectLinks) {
        this.interconnectLinks = interconnectLinks;
    }

    public String getGivebackState() {
        return this.givebackState;
    }

    public void setGivebackState(final String givebackState) {
        this.givebackState = givebackState;
    }
    
    public String getInterconnectName() {
        return interconnectName;
    }

    public void setInterconnectName(final String interconnectName) {
        this.interconnectName = interconnectName;
    }
    

    public String getInterconnectType() {
        return interconnectType;
    }

    public void setInterconnectType(final String interconnectType) {
        this.interconnectType = interconnectType;
    }

    public String getCurrentMode() {
        return currentMode;
    }

    public void setCurrentMode(final String currentMode) {
        this.currentMode = currentMode;
    }

    public List<String> getTakeoverByPartnerNotPossibleReasons() {
        return takeoverByPartnerNotPossibleReasons;
    }

    public void setTakeoverByPartnerNotPossibleReasons(final List<String> takeoverByPartnerNotPossibleReasons) {
        this.takeoverByPartnerNotPossibleReasons = takeoverByPartnerNotPossibleReasons;
    }

    public List<String> getTakeoverOfPartnerNotPossibleReasons() {
        return takeoverOfPartnerNotPossibleReasons;
    }

    public void setTakeoverOfPartnerNotPossibleReasons(List<String> takeoverOfPartnerNotPossibleReasons) {
        this.takeoverOfPartnerNotPossibleReasons = takeoverOfPartnerNotPossibleReasons;
    }

    public String getTakeoverFailureReason() {
        return takeoverFailureReason;
    }

    public void setTakeoverFailureReason(final String takeoverFailureReason) {
        this.takeoverFailureReason = takeoverFailureReason;
    }

    public CModeServiceProcessor createServiceProcessor(final String macAddress) {
        return new CModeServiceProcessor(this, macAddress);
    }

    public CModeServiceProcessors getServiceProcessors() {
        return serviceProcessors;
    }

    public CModeStorageShelf createCModeStorageShelf(final String shelfUID) {
        return new CModeStorageShelf(this, shelfUID);
    }

    public CModeStorageShelves getStorageShelves() {
        return storageShelves;
    }

    public CModeStorageShelfBay createStorageShelfBay(final String shelfUID) {
        return new CModeStorageShelfBay(this, shelfUID);
    }

    public CModeStorageShelfBays getStorageShelfBays() {
        return storageShelfBays;
    }

    public CModeStorageShelfChannel createCModeStorageShelfChannel(final String shelfUID) {
        return new CModeStorageShelfChannel(this, shelfUID);        
    }

    public CModeStorageShelfChannels getStorageShelfChannels() {
        return storageShelfChannels;
    }

    public CModeFlashDevice createFlashDevice(final String serialNumber) {
        return new CModeFlashDevice(this, serialNumber);
    }

    public CModeFlashDevices getFlashDevices() {
        return flashDevices;
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "Name", nodeKey.getName());
        addAttributeNoCRC(writer, "SystemID", systemID);
        addAttributeNoCRC(writer, "CPUProcessorType", cpuTypeString);
        addAttributeNoCRC(writer, "CPUProcessorId", cpuIDString);
        addAttributeNoCRC(writer, "NumberOfProcessors", numCPUs);
        addAttributeNoCRC(writer, "MemorySizeMB", memSize);
        addAttributeNoCRC(writer, "ProductType", productType);
        addAttributeNoCRC(writer, "PartnerName", partnerName);
        addAttributeNoCRC(writer, "IsFailoverEnabled", isFailoverEnabled);
        addAttributeNoCRC(writer, "FailoverState", failoverState);
        addAttributeNoCRC(writer, "IsTakeoverPossible", isTakeoverPossible);
        addAttributeNoCRC(writer, "PartnerFirmwareState", partnerFirmwareState);
        addAttributeNoCRC(writer, "LocalFirmwareState", localFirmwareState);
        addAttributeNoCRC(writer, "IsInterconnectUp", isInterconnectUp);
        addAttributeNoCRC(writer, "InterconnectLinks", interconnectLinks);
        addAttributeNoCRC(writer, "GivebackState", givebackState);
        addAttributeNoCRC(writer, "InterconnectName", interconnectName);
        addAttributeNoCRC(writer, "InterconnectType", interconnectType);
        addAttributeNoCRC(writer, "CurrentMode", currentMode);
        addAttributeNoCRC(writer, "TakeoverFailureReason", takeoverFailureReason);
        addAttributeNoCRC(writer, "TakeoverByPartnerNotPossibleReason", BuilderUtil.getListAsCSVString(takeoverByPartnerNotPossibleReasons));
        addAttributeNoCRC(writer, "TakeoverOfPartnerNotPossibleReason", BuilderUtil.getListAsCSVString(takeoverOfPartnerNotPossibleReasons));
    }

    @Override
    protected boolean hasChildren() {
        return !serviceProcessors.isEmpty() || !storageShelves.isEmpty() || !storageShelfBays.isEmpty() || !storageShelfChannels.isEmpty();
    }

    @Override
    protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        super.marshallChildrenToFileAndCRC(cache, writer, tabs);
        // children martial themselves
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeServiceProcessors.class.getSimpleName(), serviceProcessors.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeStorageShelves.class.getSimpleName(), storageShelves.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeStorageShelfBays.class.getSimpleName(), storageShelfBays.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeStorageShelfChannels.class.getSimpleName(), storageShelfChannels.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeFlashDevices.class.getSimpleName(), flashDevices.values(), tabs);
    }
}
