package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapperList;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Plex;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModePlex extends BaseCRCXMLNode<CModeAggregate> {
    private final PlexKey key;
    private final Boolean isOnline;
    private final CModeRaidGroups raidGroups;
    private EnumWrapper<Plex.PlexStatus> status;       //plex status, list of enum values
    private Integer resyncPercentage;

    CModePlex(final CModeAggregate parent, final String name, final Boolean isOnline) {
        super(parent, CModePlex.class.getSimpleName());
        this.key = new PlexKey(parent.getKey().getUUID(), name);
        this.isOnline = isOnline;
        this.raidGroups = new CModeRaidGroups(this);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public PlexKey getKey() {
        return key;
    }

    public Boolean isOnline() {
        return isOnline;
    }

    public CModeRaidGroup createRaidGroup(final String aggrUUID, final String raidGroupName) {
        return new CModeRaidGroup(this, aggrUUID, raidGroupName);
    }

    public CModeRaidGroups getRaidGroups() {
        return raidGroups;
    }

    public Integer getResyncPercentage() {
        return resyncPercentage;
    }

    public void setResyncPercentage(Integer resyncPercentage) {
        this.resyncPercentage = resyncPercentage;
    }

    public final static class PlexKey extends ModelKeyBase<PlexKey.Field> {
        public static enum Field {
            aggregate_uuid,
            plex_name
        }

        public PlexKey(final String aggregateUUID, final String plexName) {
            super("plex", new EnumMap<Field, String>(Field.class) {{
                put(Field.aggregate_uuid, aggregateUUID);
                put(Field.plex_name, plexName);
            }});
        }
    }

    public EnumWrapper<Plex.PlexStatus> getStatus() {
        return status;
    }

    public void setStatus(EnumWrapper<Plex.PlexStatus> status) {
        this.status = status;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "ID", key.getID());
        addAttributeNoCRC(writer, "AggregateUUID", key.getID(PlexKey.Field.aggregate_uuid));
        addAttributeNoCRC(writer, "Name", key.getID(PlexKey.Field.plex_name));
        addAttributeNoCRC(writer, "IsOnline", isOnline);
        addAttributeNoCRC(writer, "Status", getStatus().getEnumValue());
        addAttributeNoCRC(writer, "StatusRaw", getStatus().getRawValue());
        addAttributeNoCRC(writer, "ResyncPercent", getResyncPercentage());
    }

    @Override
    protected boolean hasChildren() {
        return !raidGroups.isEmpty();
    }

    @Override
    protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        addAllChildrenUnderNodeAndCRC(cache, writer, CModePlexes.class.getSimpleName(), raidGroups.values(), tabs);
    }
}
