package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.QTree;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModeQTree extends BaseCRCXMLNode<CModeCluster> {

    private final QTreeKey key;
    private String qTreeName;
    private EnumWrapper<QTree.Status> status;
    private EnumWrapper<QTree.SecurityStyle> securityStyle;
    private Boolean opLocksEnabled;
    private String exportPolicyName;
    private Boolean isExportPolicyInherited;

    CModeQTree(final CModeCluster parent, final String vServerName, final String volumeName, final String id) {
        super(parent, CModeQTree.class.getSimpleName());
        this.key = new QTreeKey(vServerName, volumeName, id);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public QTreeKey getKey() {
        return key;
    }

    public String getQTreeID() {
        return key.getID(CModeQTree.QTreeKey.Field.qtree_id);
    }

    public String getQTreeName() {
        return qTreeName;
    }

    public String getVServerName() {
        return key.getID(CModeQTree.QTreeKey.Field.vserver_name);
    }

    public String getVolumeName() {
        return key.getID(QTreeKey.Field.volume_name);
    }

    public EnumWrapper<QTree.Status> getStatus() {
        return status;
    }

    public EnumWrapper<QTree.SecurityStyle> getSecurityStyle() {
        return securityStyle;
    }

    public Boolean getOpLocksEnabled() {
        return opLocksEnabled;
    }

    public void setQTreeName(final String qTreeName) {
        this.qTreeName = qTreeName;
    }

    public void setStatus(final EnumWrapper<QTree.Status> status) {
        this.status = status;
    }

    public void setSecurityStyle(final EnumWrapper<QTree.SecurityStyle> securityStyle) {
        this.securityStyle = securityStyle;
    }

    public void setOpLocksEnabled(final Boolean opLocksEnabled) {
        this.opLocksEnabled = opLocksEnabled;
    }

    public String getExportPolicyName() {
        return exportPolicyName;
    }

    public void setExportPolicyName(String exportPolicyName) {
        this.exportPolicyName = exportPolicyName;
    }

    public Boolean getExportPolicyInherited() {
        return isExportPolicyInherited;
    }

    public void setExportPolicyInherited(Boolean exportPolicyInherited) {
        isExportPolicyInherited = exportPolicyInherited;
    }

    public final static class QTreeKey extends ModelKeyBase<QTreeKey.Field> {
        public static enum Field {
            vserver_name,
            volume_name,
            qtree_id
        }

        public QTreeKey(final String vServerName, final String volumeName, final String qTreeID) {
            super("qtrre", new EnumMap<Field, String>(Field.class) {{
                put(Field.vserver_name, vServerName);
                put(Field.volume_name, volumeName);
                put(Field.qtree_id, qTreeID);
            }});
        }
    }

    protected void marshallAttributesToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(cache, writer);
        addAttributeNoCRC(writer, "UUID", key.getID());
        addAttributeNoCRC(writer, "VServerName", getVServerName());
        addAttributeNoCRC(writer, "VolumeName", getVolumeName());
        addAttributeNoCRC(writer, "ID", getQTreeID());
        addAttributeNoCRC(writer, "Status", status);
        addAttributeNoCRC(writer, "SecurityStyle", securityStyle);
        addAttributeNoCRC(writer, "Oplocks", opLocksEnabled);
        addAttributeNoCRC(writer, "ExportPolicyName", exportPolicyName);
        addAttributeNoCRC(writer, "IsExportPolicyInherited", isExportPolicyInherited);
    }
}
