package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.QuotaUser;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;
import java.util.EnumSet;

public class CModeQuotaUser extends BaseCRCXMLNode<CModeQuota> {

    private final CModeQuotaUserKey key;
    private QuotaUser.QuotaUserType quotaUserType;
    private String quotaUserID;
    private String quotaUsername;

    CModeQuotaUser(final CModeQuota quota, final String quotaUserID) {
        super(quota, CModeQuotaUser.class.getSimpleName());
        this.key = new CModeQuotaUserKey(quota.getVServerName(), quota.getQTreeID(), quota.getKey().getID(CModeQuota.CModeQuotaKey.Field.quota_target), quotaUserID);
    }

    @SuppressWarnings({"unchecked"})
    public CModeQuotaUserKey getKey() {
        return key;
    }

    public QuotaUser.QuotaUserType getUserType() {
        return quotaUserType;
    }

    public String getQuotaUserID() {
        return quotaUserID;
    }

    public String getQuotaUsername() {
        return quotaUsername;
    }

    public void setQuotaUserType(final QuotaUser.QuotaUserType quotaUserType) {
        this.quotaUserType = quotaUserType;
    }

    public void setQuotaUserID(final String quotaUserID) {
        this.quotaUserID = quotaUserID;
    }

    public void setQuotaUsername(final String quotaUsername) {
        this.quotaUsername = quotaUsername;
    }

    public static final class CModeQuotaUserKey extends ModelKeyBase<CModeQuotaUserKey.Field> {
        public static enum Field {
            vserver_name,
            qtree_id,
            quota_target,
            quota_user_id
        }

        public CModeQuotaUserKey(final String vServerName, final String qTreeID, final String quotaTarget, final String quotaUserID) {
            super("quota_user", new EnumMap<Field, String>(Field.class) {{
                put(Field.vserver_name, vServerName);
                put(Field.qtree_id, qTreeID);
                put(Field.quota_target, quotaTarget);
                put(Field.quota_user_id, quotaUserID);
            }}, EnumSet.of(Field.qtree_id, Field.quota_target));
        }
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "UUID", key.getOcieUUID());
        addAttributeNoCRC(writer, "QuotaUserID", quotaUserID);
        addAttributeNoCRC(writer, "QuotaUserName", quotaUsername);
    }
}
