package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation.BuilderUtil;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.ServiceProcessor;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class CModeServiceProcessor extends BaseCRCXMLNode<CModeNode> {

	private final UUIDKey key;

    private String deviceRevision;
    private String firmwareVersion;
    final private List<String> ipAddresses = new ArrayList<String>();
    private String impiVersion;
    private Boolean isAutoUpdateEnabled;
    private Boolean isIpConfigured;
    private String returnedNodeName;
    private String partNum;
    private String serialNum;
    private EnumWrapper<ServiceProcessor.ServiceProcessorStatus> status;
    private EnumWrapper<ServiceProcessor.ServiceProcessorType> type;

    public CModeServiceProcessor(final CModeNode parent, final String macAddress) {
        super(parent, CModeServiceProcessor.class.getSimpleName());
        this.key = new UUIDKey("service_processor", macAddress);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public UUIDKey getKey() {
        return key;
    }

    public String getDeviceRevision() {
        return deviceRevision;
    }

    public void setDeviceRevision(final String deviceRevision) {
        this.deviceRevision = deviceRevision;
    }

    public String getFirmwareVersion() {
        return firmwareVersion;
    }

    public void setFirmwareVersion(final String firmwareVersion) {
        this.firmwareVersion = firmwareVersion;
    }

    public String getIpAddressesAsCSV() {
        return BuilderUtil.getListAsCSVString(ipAddresses);
    }

    public void addIpAddresses(final List<String> ipAddresses) {
        this.ipAddresses.addAll(ipAddresses);
    }

    public String getImpiVersion() {
        return impiVersion;
    }

    public void setImpiVersion(final String impiVersion) {
        this.impiVersion = impiVersion;
    }

    public Boolean isAutoUpdateEnabled() {
        return isAutoUpdateEnabled;
    }

    public void setAutoUpdateEnabled(final Boolean autoUpdateEnabled) {
        isAutoUpdateEnabled = autoUpdateEnabled;
    }

    public Boolean isIpConfigured() {
        return isIpConfigured;
    }

    public void setIpConfigured(final Boolean ipConfigured) {
        isIpConfigured = ipConfigured;
    }

    public String getReturnedNodeName() {
        return returnedNodeName;
    }

    public void setReturnedNodeName(final String returnedNodeName) {
        this.returnedNodeName = returnedNodeName;
    }

    public String getPartNum() {
        return partNum;
    }

    public void setPartNum(final String partNum) {
        this.partNum = partNum;
    }

    public String getSerialNum() {
        return serialNum;
    }

    public void setSerialNum(final String serialNum) {
        this.serialNum = serialNum;
    }

    public EnumWrapper<ServiceProcessor.ServiceProcessorStatus> getStatus() {
        return status;
    }

    public void setStatus(final EnumWrapper<ServiceProcessor.ServiceProcessorStatus> status) {
        this.status = status;
    }

    public EnumWrapper<ServiceProcessor.ServiceProcessorType> getType() {
        return type;
    }

    public void setType(final EnumWrapper<ServiceProcessor.ServiceProcessorType> type) {
        this.type = type;
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "MacAddress", key.getID());
        addAttributeNoCRC(writer, "DeviceRevision", deviceRevision);
        addAttributeNoCRC(writer, "FirmwareVersion", firmwareVersion);
        addAttributeNoCRC(writer, "IpAddresses", getIpAddressesAsCSV());
        addAttributeNoCRC(writer, "ImpiVersion", impiVersion);
        addAttributeNoCRC(writer, "IsAutoUpdateEnabled", isAutoUpdateEnabled);
        addAttributeNoCRC(writer, "IsIpConfigured", isIpConfigured);
        addAttributeNoCRC(writer, "NodeName", returnedNodeName);
        addAttributeNoCRC(writer, "PartNum", partNum);
        addAttributeNoCRC(writer, "SerialNum", serialNum);
        addAttributeNoCRC(writer, "Status", status.getEnumValue());
        addAttributeNoCRC(writer, "StatusRaw", status.getRawValue());
        addAttributeNoCRC(writer, "Type", type.getEnumValue());
        addAttributeNoCRC(writer, "TypeRaw", type.getRawValue());
    }
}