package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Volume;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;

public class CModeSisStatus extends BaseCRCXMLNode<CModeCluster> {

    private final SisKey sisKey;

    private String sisPolicyName = null;
    private EnumWrapper<Volume.VolumeSisStatus> status = null;
    private EnumWrapper<Volume.VolumeSisState> state = null;
    private EnumWrapper<Volume.VolumeSisType> type = null;
    private String progress = null;
    private String schedule = null;
    private Long lastOpBeginTS = null;
    private Long lastOpEndTS = null;
    private String lastOpError = null;
    private EnumWrapper<Volume.VolumeSisOpState> lastOpState = null;
    private Long lastOpSize = null;
    private Boolean isCompressionEnabled = null;
    private Boolean isInlineCompressionEnabled = null;

    CModeSisStatus(final CModeCluster parent, final String vServerName, final String volumePath) {
        super(parent, CModeSisStatus.class.getSimpleName());
        this.sisKey = new SisKey(vServerName, volumePath);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public SisKey getKey() {
        return this.sisKey;
    }

    public String getSisPolicyName() {
        return sisPolicyName;
    }

    public void setSisPolicyName(final String sisPolicyName) {
        this.sisPolicyName = sisPolicyName;
    }

    public EnumWrapper<Volume.VolumeSisStatus> getStatus() {
        return status;
    }

    public void setStatus(final EnumWrapper<Volume.VolumeSisStatus> status) {
        this.status = status;
    }

    public EnumWrapper<Volume.VolumeSisState> getState() {
        return state;
    }

    public void setState(final EnumWrapper<Volume.VolumeSisState> state) {
        this.state = state;
    }

    public EnumWrapper<Volume.VolumeSisType> getType() {
        return type;
    }

    public void setType(final EnumWrapper<Volume.VolumeSisType> type) {
        this.type = type;
    }

    public String getProgress() {
        return progress;
    }

    public void setProgress(final String progress) {
        this.progress = progress;
    }

    public String getSchedule() {
        return schedule;
    }

    public void setSchedule(final String schedule) {
        this.schedule = schedule;
    }

    public Long getLastOpBeginTS() {
        return lastOpBeginTS;
    }

    public void setLastOpBeginTS(final Long lastOpBeginTS) {
        this.lastOpBeginTS = lastOpBeginTS;
    }

    public Long getLastOpEndTS() {
        return lastOpEndTS;
    }

    public void setLastOpEndTS(final Long lastOpEndTS) {
        this.lastOpEndTS = lastOpEndTS;
    }

    public String getLastOpError() {
        return lastOpError;
    }

    public void setLastOpError(final String lastOpError) {
        this.lastOpError = lastOpError;
    }

    public EnumWrapper<Volume.VolumeSisOpState> getLastOpState() {
        return lastOpState;
    }

    public void setLastOpState(final EnumWrapper<Volume.VolumeSisOpState> lastOpState) {
        this.lastOpState = lastOpState;
    }

    public Long getLastOpSize() {
        return lastOpSize;
    }

    public void setLastOpSize(final Long lastOpSize) {
        this.lastOpSize = lastOpSize;
    }

    public Boolean isCompressionEnabled() {
        return isCompressionEnabled;
    }

    public void setCompressionEnabled(final Boolean compressionEnabled) {
        isCompressionEnabled = compressionEnabled;
    }

    public Boolean isInlineCompressionEnabled() {
        return isInlineCompressionEnabled;
    }

    public void setInlineCompressionEnabled(final Boolean inlineCompressionEnabled) {
        isInlineCompressionEnabled = inlineCompressionEnabled;
    }

    public final static class SisKey implements Key<SisKey> {
        private final String vServerName;
        private final String volumePath;

        public SisKey(final String vServerName, final String volumePath) {
            this.vServerName = vServerName;
            this.volumePath = volumePath;
        }

        @Override
        public int compareTo(final SisKey o) {
            final int nodeNameCompare = this.vServerName.compareTo(o.vServerName);
            if (nodeNameCompare != 0) {
                return nodeNameCompare;
            }
            return this.volumePath.compareTo(o.volumePath);
        }

        @Override
        public boolean equals(final Object o) {
            if (this == o) {
                return true;
            }
            if (o == null || getClass() != o.getClass()) {
                return false;
            }
            final SisKey sisKey = (SisKey) o;
            if (!vServerName.equals(sisKey.vServerName)) {
                return false;
            }
            if (!volumePath.equals(sisKey.volumePath)) {
                return false;
            }
            return true;
        }

        @Override
        public int hashCode() {
            int result = vServerName.hashCode();
            result = 31 * result + volumePath.hashCode();
            return result;
        }

        @Override
        public String toString() {
            return vServerName + "." + volumePath;
        }

        public String getVServerName() {
            return vServerName;
        }

        public String getVolumePath() {
            return volumePath;
        }
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "VServerName", sisKey.getVServerName());
        addAttributeNoCRC(writer, "VolumePath", sisKey.getVolumePath());
        addAttributeNoCRC(writer, "SisPolicyName", sisPolicyName);
        addAttributeNoCRC(writer, "VolumeSisStatus", status);
        addAttributeNoCRC(writer, "VolumeSisState", state);
        addAttributeNoCRC(writer, "VolumeSisType", type);
        addAttributeNoCRC(writer, "Progress", progress);
        addAttributeNoCRC(writer, "Schedule", schedule);
        addAttributeNoCRC(writer, "LastOpBeginTS", lastOpBeginTS);
        addAttributeNoCRC(writer, "LastOpEndTS", lastOpEndTS);
        addAttributeNoCRC(writer, "LastOpError", lastOpError);
        addAttributeNoCRC(writer, "LastOpState", lastOpState);
        addAttributeNoCRC(writer, "LastOpSize", lastOpSize);
        addAttributeNoCRC(writer, "IsCompressionEnabled", isCompressionEnabled);
        addAttributeNoCRC(writer, "IsInlineCompressionEnabled", isInlineCompressionEnabled);
    }

}
