package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.SnapMirror;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

public class CModeSnapMirror extends BaseCRCXMLNode<CModeCluster> {

    private final CModeSnapMirrorKey key;

    private String destinationCluster;
    private String destinationVolume;
    private String destinationVServer;
    private String destinationVServerUuid;
    private String sourceCluster;
    private String sourceVolume;
    private String sourceVServer;
    private String sourceVServerUuid;
    private EnumWrapper<SnapMirror.MirrorState> mirrorState;
    private String exportedSnapshot;
    private String newestSnapshot;
    private String transferSnapshot;
    private Boolean isConsituent;
    private Boolean isHealthy;
    private Long maxTransferRate;
    private Long relationshipProgress;
    private EnumWrapper<SnapMirror.RelationshipStatusType> relationshipStatus;
    private EnumWrapper<SnapMirror.RelationshipType> relationshipType;
    private Long snapshotCheckpoint;
    private Long snapshotProgress;
    private Long tries;
    private Long currentMaxTransferRate;
    private String currentTransferError;
    private EnumWrapper<SnapMirror.TransferPriorityType> currentTransferPriority;
    private EnumWrapper<SnapMirror.TransferType> currentTransferType;
    private Long lagTime;
    private Long lastTransferDuration;
    private String lastTransferError;
    private EnumWrapper<SnapMirror.TransferType> lastTransferType;
    private Long lastTransferSize;
    private Long progressLastUpdated;
    private String relationshipControlPlane;
    private String relationshipID;
    private String policyName;
    private String schedule;
    private Long lastTransferEndTimestamp;
    private String lastTransferErrorCodes;
    private String destinationVolumeNode;
    private String unhealthyReason;
    private String operationId;
    private String networkCompressionRatio;
    private String lastTransferNetworkCompressionRatio;
    private EnumWrapper<SnapMirror.RelationshipGroupType>  relationshipGroupType;
    private String  relationshipGroupTypeRaw;


    CModeSnapMirror(final CModeCluster parent, final String sourceLocation, final String destinationLocation) {
        super(parent, CModeSnapMirror.class.getSimpleName());
        this.key = new CModeSnapMirrorKey(sourceLocation, destinationLocation);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public CModeSnapMirrorKey getKey() {
        return key;
    }

    public String getDestinationCluster() {
        return destinationCluster;
    }

    public void setDestinationCluster(final String destinationCluster) {
        this.destinationCluster = destinationCluster;
    }

    public String getDestinationVolume() {
        return destinationVolume;
    }

    public void setDestinationVolume(final String destinationVolume) {
        this.destinationVolume = destinationVolume;
    }

    public String getDestinationVServer() {
        return destinationVServer;
    }

    public void setDestinationVServer(final String destinationVServer) {
        this.destinationVServer = destinationVServer;
    }

    public String getDestinationVServerUuid() {
        return destinationVServerUuid;
    }

    public void setDestinationVServerUuid(final String destinationVServerUuid) {
        this.destinationVServerUuid = destinationVServerUuid;
    }

    public String getDestinationLocation() {
        return key.getID(CModeSnapMirrorKey.Field.destination_location);
    }

    public String getSourceCluster() {
        return sourceCluster;
    }

    public void setSourceCluster(final String sourceCluster) {
        this.sourceCluster = sourceCluster;
    }

    public String getSourceVolume() {
        return sourceVolume;
    }

    public void setSourceVolume(final String sourceVolume) {
        this.sourceVolume = sourceVolume;
    }

    public String getSourceVServer() {
        return sourceVServer;
    }

    public void setSourceVServer(final String sourceVServer) {
        this.sourceVServer = sourceVServer;
    }

    public String getSourceVServerUuid() {
        return sourceVServerUuid;
    }

    public void setSourceVServerUuid(final String sourceVServerUuid) {
        this.sourceVServerUuid = sourceVServerUuid;
    }

    public String getSourceLocation() {
        return key.getID(CModeSnapMirrorKey.Field.source_location);
    }

    public EnumWrapper<SnapMirror.MirrorState> getMirrorState() {
        return mirrorState;
    }

    public void setMirrorState(final EnumWrapper<SnapMirror.MirrorState> mirrorState) {
        this.mirrorState = mirrorState;
    }

    public String getExportedSnapshot() {
        return exportedSnapshot;
    }

    public void setExportedSnapshot(final String exportedSnapshot) {
        this.exportedSnapshot = exportedSnapshot;
    }

    public String getNewestSnapshot() {
        return newestSnapshot;
    }

    public void setNewestSnapshot(final String newestSnapshot) {
        this.newestSnapshot = newestSnapshot;
    }

    public String getTransferSnapshot() {
        return transferSnapshot;
    }

    public void setTransferSnapshot(final String transferSnapshot) {
        this.transferSnapshot = transferSnapshot;
    }

    public Boolean isConsituent() {
        return isConsituent;
    }

    public void setConsituent(final Boolean consituent) {
        isConsituent = consituent;
    }

    public Boolean isHealthy() {
        return isHealthy;
    }

    public void setHealthy(final Boolean healthy) {
        isHealthy = healthy;
    }

    public Long getMaxTransferRate() {
        return maxTransferRate;
    }

    public void setMaxTransferRate(final Long maxTransferRate) {
        this.maxTransferRate = maxTransferRate;
    }

    public Long getRelationshipProgress() {
        return relationshipProgress;
    }

    public void setRelationshipProgress(final Long relationshipProgress) {
        this.relationshipProgress = relationshipProgress;
    }

    public EnumWrapper<SnapMirror.RelationshipStatusType> getRelationshipStatus() {
        return relationshipStatus;
    }

    public void setRelationshipStatus(final EnumWrapper<SnapMirror.RelationshipStatusType> relationshipStatus) {
        this.relationshipStatus = relationshipStatus;
    }

    public EnumWrapper<SnapMirror.RelationshipType> getRelationshipType() {
        return relationshipType;
    }

    public void setRelationshipType(final EnumWrapper<SnapMirror.RelationshipType> relationshipType) {
        this.relationshipType = relationshipType;
    }

    public Long getSnapshotCheckpoint() {
        return snapshotCheckpoint;
    }

    public void setSnapshotCheckpoint(final Long snapshotCheckpoint) {
        this.snapshotCheckpoint = snapshotCheckpoint;
    }

    public Long getSnapshotProgress() {
        return snapshotProgress;
    }

    public void setSnapshotProgress(final Long snapshotProgress) {
        this.snapshotProgress = snapshotProgress;
    }

    public Long getTries() {
        return tries;
    }

    public void setTries(final Long tries) {
        this.tries = tries;
    }

    public Long getCurrentMaxTransferRate() {
        return currentMaxTransferRate;
    }

    public void setCurrentMaxTransferRate(final Long currentMaxTransferRate) {
        this.currentMaxTransferRate = currentMaxTransferRate;
    }

    public String getCurrentTransferError() {
        return currentTransferError;
    }

    public void setCurrentTransferError(final String currentTransferError) {
        this.currentTransferError = currentTransferError;
    }

    public EnumWrapper<SnapMirror.TransferPriorityType> getCurrentTransferPriority() {
        return currentTransferPriority;
    }

    public void setCurrentTransferPriority(final EnumWrapper<SnapMirror.TransferPriorityType> currentTransferPriority) {
        this.currentTransferPriority = currentTransferPriority;
    }

    public EnumWrapper<SnapMirror.TransferType> getCurrentTransferType() {
        return currentTransferType;
    }

    public void setCurrentTransferType(final EnumWrapper<SnapMirror.TransferType> currentTransferType) {
        this.currentTransferType = currentTransferType;
    }

    public Long getLagTime() {
        return lagTime;
    }

    public void setLagTime(final Long lagTime) {
        this.lagTime = lagTime;
    }

    public Long getLastTransferDuration() {
        return lastTransferDuration;
    }

    public void setLastTransferDuration(final Long lastTransferDuration) {
        this.lastTransferDuration = lastTransferDuration;
    }

    public String getLastTransferError() {
        return lastTransferError;
    }

    public void setLastTransferError(final String lastTransferError) {
        this.lastTransferError = lastTransferError;
    }

    public EnumWrapper<SnapMirror.TransferType> getLastTransferType() {
        return lastTransferType;
    }

    public void setLastTransferType(final EnumWrapper<SnapMirror.TransferType> lastTransferType) {
        this.lastTransferType = lastTransferType;
    }

    public Long getLastTransferSize() {
        return lastTransferSize;
    }

    public void setLastTransferSize(final Long lastTransferSize) {
        this.lastTransferSize = lastTransferSize;
    }

    public Long getProgressLastUpdated() {
        return progressLastUpdated;
    }

    public void setProgressLastUpdated(final Long progressLastUpdated) {
        this.progressLastUpdated = progressLastUpdated;
    }

    public String getRelationshipControlPlane() {
        return relationshipControlPlane;
    }

    public void setRelationshipControlPlane(final String relationshipControlPlane) {
        this.relationshipControlPlane = relationshipControlPlane;
    }

    public String getRelationshipID() {
        return relationshipID;
    }

    public void setRelationshipID(final String relationshipID) {
        this.relationshipID = relationshipID;
    }

    public String getPolicyName() {
        return policyName;
    }

    public void setPolicyName(final String policyName) {
        this.policyName = policyName;
    }

    public String getSchedule() {
        return schedule;
    }

    public void setSchedule(final String schedule) {
        this.schedule = schedule;
    }

    public Long getLastTransferEndTimestamp() {
        return lastTransferEndTimestamp;
    }

    public void setLastTransferEndTimestamp(final Long lastTransferEndTimestamp) {
        this.lastTransferEndTimestamp = lastTransferEndTimestamp;
    }

    public String getLastTransferErrorCodes() {
        return lastTransferErrorCodes;
    }

    public void setLastTransferErrorCodes(final String lastTransferErrorCodes) {
        this.lastTransferErrorCodes = lastTransferErrorCodes;
    }

    public String getDestinationVolumeNode() {
        return destinationVolumeNode;
    }

    public void setDestinationVolumeNode(final String destinationVolumeNode) {
        this.destinationVolumeNode = destinationVolumeNode;
    }

    public String getUnhealthyReason() {
        return unhealthyReason;
    }

    public void setUnhealthyReason(final String unhealthyReason) {
        this.unhealthyReason = unhealthyReason;
    }

    public String getOperationId() {
        return operationId;
    }

    public void setOperationId(final String operationId) {
        this.operationId = operationId;
    }

    public String getNetworkCompressionRatio() {
        return networkCompressionRatio;
    }

    public void setNetworkCompressionRatio(final String networkCompressionRatio) {
        this.networkCompressionRatio = networkCompressionRatio;
    }

    public String getLastTransferNetworkCompressionRatio() {
        return lastTransferNetworkCompressionRatio;
    }

    public void setLastTransferNetworkCompressionRatio(final String lastTransferNetworkCompressionRatio) {
        this.lastTransferNetworkCompressionRatio = lastTransferNetworkCompressionRatio;
    }

    public String getRelationshipGroupTypeRaw() {
        return relationshipGroupTypeRaw;
    }

    public void setRelationshipGroupTypeRaw(String relationshipGroupTypeRaw) {
        this.relationshipGroupTypeRaw = relationshipGroupTypeRaw;
    }

    public EnumWrapper<SnapMirror.RelationshipGroupType> getRelationshipGroupType() {
        return relationshipGroupType;
    }

    public void setRelationshipGroupType(final EnumWrapper<SnapMirror.RelationshipGroupType> relationshipGroupType) {
        this.relationshipGroupType = relationshipGroupType;
    }

    public final static class CModeSnapMirrorKey extends ModelKeyBase<CModeSnapMirrorKey.Field> {
        public static enum Field {
            source_location,
            destination_location
        }

        public CModeSnapMirrorKey(final String sourceLocation, final String destinationLocation) {
            super("snapmirror", new EnumMap<Field, String>(Field.class) {{
                put(Field.source_location, sourceLocation);
                put(Field.destination_location, destinationLocation);
            }});
        }
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "ID", key.getID());
        addAttributeNoCRC(writer, "CurrentMaxTransferRate", currentMaxTransferRate);
        addAttributeNoCRC(writer, "currentTransferError", currentTransferError);
        addAttributeNoCRC(writer, "CurrentTransferPriority", EnumWrapper.toStringOrNull(currentTransferPriority));
        addAttributeNoCRC(writer, "CurrentTransferType", EnumWrapper.toStringOrNull(currentTransferType));
        addAttributeNoCRC(writer, "DestinationVolume", destinationVolume);
        addAttributeNoCRC(writer, "DestinationVserver", destinationVServer);
        addAttributeNoCRC(writer, "DestinationVserverUuid", destinationVServerUuid);
        addAttributeNoCRC(writer, "DestinationLocation", getDestinationLocation());
        addAttributeNoCRC(writer, "ExportedSnapshot", exportedSnapshot);
        addAttributeNoCRC(writer, "LagTime", lagTime);
        addAttributeNoCRC(writer, "LastTransferDuration", lastTransferDuration);
        addAttributeNoCRC(writer, "LastTransferError", lastTransferError);
        addAttributeNoCRC(writer, "LastTransferType", EnumWrapper.toStringOrNull(lastTransferType));
        addAttributeNoCRC(writer, "LastTransferSize", lastTransferSize);
        addAttributeNoCRC(writer, "LastTransferNetworkCompressionRatio", lastTransferNetworkCompressionRatio);
        addAttributeNoCRC(writer, "MaxTransferRate", maxTransferRate);
        addAttributeNoCRC(writer, "MirrorState", EnumWrapper.toStringOrNull(mirrorState));
        addAttributeNoCRC(writer, "NewestSnapshot", newestSnapshot);
        addAttributeNoCRC(writer, "NetworkCompressionRatio", networkCompressionRatio);
        addAttributeNoCRC(writer, "OperationId", operationId);
        addAttributeNoCRC(writer, "ProgressLastUpdated", progressLastUpdated);
        addAttributeNoCRC(writer, "RelationshipControlPlane", relationshipControlPlane);
        addAttributeNoCRC(writer, "RelationshipGroupType", relationshipGroupType);
        addAttributeNoCRC(writer, "RelationshipGroupTypeRaw", relationshipGroupTypeRaw);
        addAttributeNoCRC(writer, "RelationshipIdentifier", relationshipID);
        addAttributeNoCRC(writer, "RelationshipProgress", relationshipProgress);
        addAttributeNoCRC(writer, "RelationshipStatus", EnumWrapper.toStringOrNull(relationshipStatus));
        addAttributeNoCRC(writer, "RelationshipType", EnumWrapper.toStringOrNull(relationshipType));
        addAttributeNoCRC(writer, "SnapshotCheckpoint", snapshotCheckpoint);
        addAttributeNoCRC(writer, "SnapshotProgress", snapshotProgress);
        addAttributeNoCRC(writer, "SourceCluster", sourceCluster);
        addAttributeNoCRC(writer, "SourceVolume", sourceVolume);
        addAttributeNoCRC(writer, "SourceVServer", sourceVServer);
        addAttributeNoCRC(writer, "SourceVServerUuid", sourceVServerUuid);
        addAttributeNoCRC(writer, "SourceLocation", getSourceLocation());
        addAttributeNoCRC(writer, "TransferSnapshot", transferSnapshot);
        addAttributeNoCRC(writer, "Tries", tries);
        addAttributeNoCRC(writer, "PolicyName", policyName);
        addAttributeNoCRC(writer, "Schedule", schedule);
        addAttributeNoCRC(writer, "LastTransferEndTimestamp", lastTransferEndTimestamp);
        addAttributeNoCRC(writer, "LastTransferErrorCodes", lastTransferErrorCodes);
        addAttributeNoCRC(writer, "DestinationVolumeNode", destinationVolumeNode);
        addAttributeNoCRC(writer, "UnhealthyReason", unhealthyReason);
    }
}
