package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;

/**
 * Name	    Range	    Type	    Description
 * keep		            string      Specifies the Snapshot copy retention count
 * optional
 *
 * preserve		        boolean     Specifies whether Snapshot copy preserve is enabled, which determines the behavior when
 * optional    the Snapshot copy retention count is reached on the SnapMirror Vault destination.
 *
 * snapmirror-label	    string      Specifies the Snapshot copy label
 * optional
 *
 * warn	    [0..2^32-1]	integer     Specifies the warning threshold count
 * optional
 */
public class CModeSnapMirrorRule extends BaseCRCXMLNode<CModeSnapMirrorPolicy> {

    private final UUIDKey ruleKey;
    private final Integer ruleIndex;
    private String keep;
    private Boolean preserve;
    private String label;
    private Long warn;

    CModeSnapMirrorRule(final CModeSnapMirrorPolicy parent, final int index) {
        super(parent, CModeSnapMirrorRule.class.getSimpleName());
        this.ruleKey = new UUIDKey("snapmirror_rule", String.valueOf(index));
        this.ruleIndex = index;
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public UUIDKey getKey() {
        return ruleKey;
    }

    public Integer getRuleIndex() {
        return ruleIndex;
    }

    public String getKeep() {
        return keep;
    }

    public void setKeep(final String keep) {
        this.keep = keep;
    }

    public Boolean getPreserve() {
        return preserve;
    }

    public void setPreserve(final Boolean preserve) {
        this.preserve = preserve;
    }

    public String getLabel() {
        return label;
    }

    public void setLabel(final String label) {
        this.label = label;
    }

    public Long getWarn() {
        return warn;
    }

    public void setWarn(final Long warn) {
        this.warn = warn;
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "Index", ruleKey.getUUID());
        addAttributeNoCRC(writer, "Keep", keep);
        addAttributeNoCRC(writer, "Preserve", preserve);
        addAttributeNoCRC(writer, "Label", label);
        addAttributeNoCRC(writer, "Warn", warn);
    }
}
