package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;
import java.util.EnumSet;

public class CModeSnapshotPolicy extends BaseCRCXMLNode<CModeCluster> {

    private final SnapshotPolicyKey policyKey;

    private String vServerName;
    private Boolean enabled;
    private String comment;

    private final CModeSnapshotSchedules schedules;

    CModeSnapshotPolicy(final CModeCluster parent, final String vServerName, final String policyName) {
        super(parent, CModeSnapshotPolicy.class.getSimpleName());
        final String vServerNameForKey = (vServerName == null) ? "" : vServerName;
        this.policyKey = new SnapshotPolicyKey(vServerNameForKey, policyName);
        this.schedules = new CModeSnapshotSchedules(this);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public SnapshotPolicyKey getKey() {
        return policyKey;
    }

    public String getVServerName() {
        return vServerName;
    }

    public void setVServerName(final String vServerName) {
        this.vServerName = vServerName;
    }

    public Boolean isEnabled() {
        return enabled;
    }

    public void setEnabled(final Boolean enabled) {
        this.enabled = enabled;
    }

    public String getComment() {
        return comment;
    }

    public void setComment(final String comment) {
        this.comment = comment;
    }

    public CModeSnapshotSchedule createCModeSnapshotSchedule(final String scheduleName) {
        return new CModeSnapshotSchedule(this, scheduleName);
    }

    public CModeSnapshotSchedules getCModeSnapshotSchedules() {
        return schedules;
    }

    public final static class SnapshotPolicyKey extends ModelKeyBase<SnapshotPolicyKey.Field> {
        public static enum Field {
            vserver_name,
            policy_name
        }

        public SnapshotPolicyKey(final String vServerName, final String policyName) {
            super("snapshot_policy", new EnumMap<Field, String>(Field.class) {{
                put(Field.vserver_name, vServerName);
                put(Field.policy_name, policyName);
            }}, EnumSet.of(Field.vserver_name));
        }

    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "ID", policyKey.getID());
        addAttributeNoCRC(writer, "PolicyName", policyKey.getID(SnapshotPolicyKey.Field.policy_name));
        addAttributeNoCRC(writer, "VServerName", vServerName);
        addAttributeNoCRC(writer, "Enabled", enabled);
        addAttributeNoCRC(writer, "Comment", comment);
    }

    protected boolean hasChildren() {
        return !schedules.isEmpty();
    }

    @Override
    protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeSnapshotSchedules.class.getSimpleName(), schedules.values(), tabs);
    }
}
