package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.StorageShelf;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;

public class CModeStorageShelf extends BaseCRCXMLNode<CModeNode> {

	private final UUIDKey key;

    private String channelName;
    private String firmwareRevA;
    private String firmwareRevB;
    private EnumWrapper<ShelfType> shelfType;
    private String module;
    private String moduleState;
    private String nodeName;
    private String shelfName;
    private String shelfID;
    private EnumWrapper<StorageShelf.State> shelfState;
    private String terminationSwitch;
    private Integer bayCount;
    private String bayList;
    private String stackId;
    private Boolean isLocalAttach;

    public static enum ShelfType {
        NOT_MAPPED,
        edm,
        vem,
        esp,
        lrc,
        esh,
        esh2,
        esh4,
        eshfx,
        eshtx,
        emu,
        efh,
        at_fc("at-fc"),
        at_fc2("at-fc2"),
        at_fcx("at-fcx"),
        at_fcx2("at-fcx2"),
        sas,
        esas,
        sas_fc("sas-fc"),
        iom3,
        iom6;

        private final String zapiVal;
        private ShelfType() {
            this(null);
        }

        private ShelfType(final String zapiVal) {
            this.zapiVal = zapiVal;
        }
    }

    public CModeStorageShelf(final CModeNode parent, final String shelfUID) {
        super(parent, CModeStorageShelf.class.getSimpleName());
        this.key = new UUIDKey("storage_shelf", shelfUID);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public UUIDKey getKey() {
        return key;
    }

    public EnumWrapper<ShelfType> getShelfType() {
        return shelfType;
    }

    public void setShelfType(final EnumWrapper<ShelfType> shelfType) {
        this.shelfType = shelfType;
    }

    public String getChannelName() {
        return channelName;
    }

    public void setChannelName(final String channelName) {
        this.channelName = channelName;
    }

    public String getFirmwareRevA() {
        return firmwareRevA;
    }

    public void setFirmwareRevA(final String firmwareRevA) {
        this.firmwareRevA = firmwareRevA;
    }

    public String getFirmwareRevB() {
        return firmwareRevB;
    }

    public void setFirmwareRevB(final String firmwareRevB) {
        this.firmwareRevB = firmwareRevB;
    }

    public String getModule() {
        return module;
    }

    public void setModule(final String module) {
        this.module = module;
    }

    public String getModuleState() {
        return moduleState;
    }

    public void setModuleState(final String moduleState) {
        this.moduleState = moduleState;
    }

    public String getReturnedNodeName() {
        return nodeName;
    }

    public void setReturnedNodeName(final String nodeName) {
        this.nodeName = nodeName;
    }

    public String getShelfName() {
        return shelfName;
    }

    public void setShelfName(final String shelfName) {
        this.shelfName = shelfName;
    }

    public String getShelfID() {
        return shelfID;
    }

    public void setShelfID(final String shelfID) {
        this.shelfID = shelfID;
    }
    
    public String getStackId(){
    	return stackId;
    }
    
    public void setStackId(final String stackId){
    	this.stackId = stackId;
    }
    
    public EnumWrapper<StorageShelf.State> getShelfState() {
        return shelfState;
    }

    public void setShelfState(final EnumWrapper<StorageShelf.State> shelfState) {
        this.shelfState = shelfState;
    }

    public String getTerminationSwitch() {
        return terminationSwitch;
    }

    public void setTerminationSwitch(final String terminationSwitch) {
        this.terminationSwitch = terminationSwitch;
    }

    public Integer getBayCount() {
        return bayCount;
    }

    public void setBayCount(final Integer bayCount) {
        this.bayCount = bayCount;
    }

    public String getBayList() {
        return bayList;
    }

    public void setBayList(final String bayList) {
        this.bayList = bayList;
    }

    public Boolean getIsLocalAttach() {
        return isLocalAttach;
    }

    public void setIsLocalAttach(Boolean isLocalAttach) {
        this.isLocalAttach = isLocalAttach;
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "ShelfUID", key.getID());
        addAttributeNoCRC(writer, "ShelfType", shelfType.getRawValue());
        addAttributeNoCRC(writer, "ChannelName", channelName);
        addAttributeNoCRC(writer, "FirmwareRevA", firmwareRevA);
        addAttributeNoCRC(writer, "FirmwareRevB", firmwareRevB);
        addAttributeNoCRC(writer, "Module", module);
        addAttributeNoCRC(writer, "ModuleState", moduleState);
        addAttributeNoCRC(writer, "NodeName", nodeName);
        addAttributeNoCRC(writer, "ShelfName", shelfName);
        addAttributeNoCRC(writer, "ShelfID", shelfID);
        addAttributeNoCRC(writer, "ShelfState", shelfState);
        addAttributeNoCRC(writer, "TerminationSwitch", terminationSwitch);
        addAttributeNoCRC(writer, "BayCount", bayCount);
        addAttributeNoCRC(writer, "BayList", bayList);
        addAttributeNoCRC(writer, "StackID" , stackId);
        addAttributeNoCRC(writer, "IsLocalAttach" , isLocalAttach);
    }
}