package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.*;

import java.io.IOException;

public class CModeStorageShelfBay extends BaseCRCXMLNode<CModeNode> {

	private final UUIDKey key;

    private String channelName;
    private String module;
    private String nodeName;
    private Integer shelfID;
    private String shelfName;
    private String shelfState;
    private String shelfType;

    private final CModeStorageBayPorts storageBayPorts;

    public CModeStorageShelfBay(final CModeNode parent, final String shelfUID) {
        super(parent, CModeStorageShelfBay.class.getSimpleName());
        this.key = new UUIDKey("storage_shelf_bay", shelfUID);
        this.storageBayPorts = new CModeStorageBayPorts(this);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public UUIDKey getKey() {
        return key;
    }

    public String getChannelName() {
        return channelName;
    }

    public void setChannelName(final String channelName) {
        this.channelName = channelName;
    }

    public String getModule() {
        return module;
    }

    public void setModule(final String module) {
        this.module = module;
    }

    public String getReturnedName() {
        return nodeName;
    }

    public void setReturnedNodeName(final String nodeName) {
        this.nodeName = nodeName;
    }

    public Integer getShelfID() {
        return shelfID;
    }

    public void setShelfID(final Integer shelfID) {
        this.shelfID = shelfID;
    }

    public String getShelfName() {
        return shelfName;
    }

    public void setShelfName(final String shelfName) {
        this.shelfName = shelfName;
    }

    public String getShelfState() {
        return shelfState;
    }

    public void setShelfState(final String shelfState) {
        this.shelfState = shelfState;
    }

    public String getShelfType() {
        return shelfType;
    }

    public void setShelfType(final String shelfType) {
        this.shelfType = shelfType;
    }

    public CModeStorageBayPort createBayPort(final String portID) {
        return new CModeStorageBayPort(this, portID);
    }

    public CModeStorageBayPorts getStorageBayPorts() {
        return storageBayPorts;
    }



    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "ShelfUID", key.getID());
        addAttributeNoCRC(writer, "ChannelName", channelName);
        addAttributeNoCRC(writer, "Module", module);
        addAttributeNoCRC(writer, "NodeName", nodeName);
        addAttributeNoCRC(writer, "ShelfID", shelfID);
        addAttributeNoCRC(writer, "ShelfName", shelfName);
        addAttributeNoCRC(writer, "ShelfState", shelfState);
        addAttributeNoCRC(writer, "ShelfType", shelfType);
    }

    @Override
    protected boolean hasChildren() {
        return !storageBayPorts.isEmpty();
    }

    @Override
    protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        super.marshallChildrenToFileAndCRC(cache, writer, tabs);
        // children martial themselves
        addAllChildrenUnderNodeAndCRC(cache, writer, CModeStorageBayPorts.class.getSimpleName(), storageBayPorts.values(), tabs);

    }

    public final static class CModeStorageBayPorts extends NodeMap<CModeStorageShelfBay, CModeStorageBayPort.PortKey, CModeStorageBayPort> {

		CModeStorageBayPorts(final CModeStorageShelfBay modelParent) {
			super(modelParent);
		}

		public boolean add(final CModeStorageBayPort port) {
			return (put(port.getKey(), port) == null);
		}
	}

	public static final class CModeStorageBayPort extends BaseCRCXMLNode<CModeStorageShelfBay> {

		private final PortKey portKey;
        
        private Integer bayNo;
        private String diskName;
        private String diskUUID;
        private String portDesignator;
        private String portState;

		CModeStorageBayPort(final CModeStorageShelfBay parent, final String portID) {
			super(parent, CModeStorageBayPort.class.getSimpleName());
			this.portKey = new PortKey(portID);
		}

		@Override
		@SuppressWarnings({"unchecked"})
		public PortKey getKey() {
			return portKey;
		}

        public Integer getBayNo() {
            return bayNo;
        }

        public void setBayNo(final Integer bayNo) {
            this.bayNo = bayNo;
        }

        public String getDiskName() {
            return diskName;
        }

        public void setDiskName(final String diskName) {
            this.diskName = diskName;
        }

        public String getDiskUUID() {
            return diskUUID;
        }

        public void setDiskUUID(final String diskUUID) {
            this.diskUUID = diskUUID;
        }

        public String getPortDesignator() {
            return portDesignator;
        }

        public void setPortDesignator(final String portDesignator) {
            this.portDesignator = portDesignator;
        }

        public String getPortState() {
            return portState;
        }

        public void setPortState(final String portState) {
            this.portState = portState;
        }

        public final static class PortKey implements Key<PortKey> {
			private final String portID;

			public PortKey(final String portID) {
				this.portID = portID;
			}

			@Override
			public int compareTo(final PortKey o) {
				return this.portID.compareTo(o.portID);
			}

			@Override
			public boolean equals(final Object o) {
				if (this == o) {
					return true;
				}
				if (o == null || getClass() != o.getClass()) {
					return false;
				}
				final PortKey that = (PortKey) o;
				if (portID != null ? !portID.equals(that.portID) : that.portID != null) {
					return false;
				}
				return true;
			}

			@Override
			public int hashCode() {
				return portID != null ? portID.hashCode() : 0;
			}

			@Override
			public String toString() {
				return portID;
			}

			public String getID() {
				return portID;
			}
		}

        @Override
        protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
            super.marshallAttributes(marshallingCache, writer);
            addAttributeNoCRC(writer, "PortID", portKey.getID());
            addAttributeNoCRC(writer, "BayNo", bayNo);
            addAttributeNoCRC(writer, "DiskName", diskName);
            addAttributeNoCRC(writer, "DiskUUID", diskUUID);
            addAttributeNoCRC(writer, "PortDesignator", portDesignator);
            addAttributeNoCRC(writer, "PortState", portState);
        }
	}
}