/*
 * Copyright (c) 2013 NetApp
 * All rights reserved
 */

package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.builders.zapi.cmode.netappfoundation.BuilderUtil;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.StorageSwitch;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.ArrayList;
import java.util.EnumMap;
import java.util.HashSet;
import java.util.Set;

/**
 * @author Al Velella
 *         Date: 10/22/13
 *         Time: 4:05 PM
 */
public class CModeStorageSwitch extends BaseCRCXMLNode<CModeCluster> {

    public final static class StorageSwitchKey extends KeyBase<StorageSwitchKey.Field> {
        public static enum Field {
            wwn
        }

        public StorageSwitchKey(final String wwn) {
            super("storage_switch", new EnumMap<Field, String>(Field.class) {{
                put(Field.wwn, wwn);
            }});
        }

        @Override
        public String getOcieUUID() {
            return getID(Field.wwn);
        }
    }

    private final StorageSwitchKey storageSwitchKey;

    private String firmwareVersion = null;                              // firmware-version
    private String name = null;                                         // name
    private String ipAddress = null;                                    // ip-address
    private String model = null;                                        // model
    private EnumWrapper<StorageSwitch.Role> roleWrapper = null;         // role
    private EnumWrapper<StorageSwitch.Status> statusWrapper = null;     // status
    private String vendor = null;                                       // vendor
    private final Set<String> storageSwitchPeerWwns = new HashSet<String>();
    private int sensorCount = 0;
    private int powerSupplyCount = 0;
    private int fanCount = 0;

    CModeStorageSwitch(final CModeCluster parent, final String wwn) {
        super(parent, wwn);
        this.storageSwitchKey = new StorageSwitchKey(wwn);
    }

    @SuppressWarnings("unchecked")
    @Override
    public StorageSwitchKey getKey() {
        return storageSwitchKey;
    }

    public String getFirmwareVersion() {
        return firmwareVersion;
    }

    public void setFirmwareVersion(final String firmwareVersion) {
        this.firmwareVersion = firmwareVersion;
    }

    public String getName() {
        return name;
    }

    public void setName(final String name) {
        this.name = name;
    }

    public String getIpAddress() {
        return ipAddress;
    }

    public void setIpAddress(final String ipAddress) {
        this.ipAddress = ipAddress;
    }

    public String getModel() {
        return model;
    }

    public void setModel(final String model) {
        this.model = model;
    }

    public EnumWrapper<StorageSwitch.Role> getRoleWrapper() {
        return roleWrapper;
    }

    public void setRoleWrapper(final EnumWrapper<StorageSwitch.Role> roleWrapper) {
        this.roleWrapper = roleWrapper;
    }

    public EnumWrapper<StorageSwitch.Status> getStatusWrapper() {
        return statusWrapper;
    }

    public void setStatusWrapper(final EnumWrapper<StorageSwitch.Status> statusWrapper) {
        this.statusWrapper = statusWrapper;
    }

    public String getVendor() {
        return vendor;
    }

    public void setVendor(final String vendor) {
        this.vendor = vendor;
    }

    public Set<String> getStorageSwitchPeerWwns() {
        return storageSwitchPeerWwns;
    }

    public void addPeerWwn(final String peerWwn) {
        storageSwitchPeerWwns.add(peerWwn);
    }

    public int getSensorCount() {
        return sensorCount;
    }

    public void setSensorCount(final int sensorCount) {
        this.sensorCount = sensorCount;
    }

    public int getPowerSupplyCount() {
        return powerSupplyCount;
    }

    public void setPowerSupplyCount(final int powerSupplyCount) {
        this.powerSupplyCount = powerSupplyCount;
    }

    public int getFanCount() {
        return fanCount;
    }

    public void setFanCount(final int fanCount) {
        this.fanCount = fanCount;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "WWN", storageSwitchKey.getOcieUUID());
        addAttributeNoCRC(writer, "Name", name);
        addAttributeNoCRC(writer, "IpAddress", ipAddress);
        addAttributeNoCRC(writer, "Model", model);
        addAttributeNoCRC(writer, "Role", roleWrapper.getRawValue());
        addAttributeNoCRC(writer, "Status", statusWrapper.getRawValue());
        addAttributeNoCRC(writer, "Vendor", vendor);
        addAttributeNoCRC(writer, "StorageSwitchPeerWwns", BuilderUtil.getListAsCSVString(new ArrayList<String>(storageSwitchPeerWwns)));
        addAttributeNoCRC(writer, "SensorCount", sensorCount);
        addAttributeNoCRC(writer, "PowerSupplyCount", powerSupplyCount);
        addAttributeNoCRC(writer, "FanCount", fanCount);
    }
}
