package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.UUIDKey;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.VServer;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class CModeVServer extends BaseCRCXMLNode<CModeCluster> {

    private final UUIDKey key;
    private String name;
    private EnumWrapper<VServer.State> state;
    private EnumWrapper<VServer.OperationalState> operationalState;
    private EnumWrapper<VServer.Type> type;
    private EnumWrapper<VServer.Subtype> subtype;
    private List<EnumWrapper<VServer.Protocol>> allowedProtos;
    private List<EnumWrapper<VServer.Protocol>> disAllowedProtos;
    private String rootVolumeName;
    private Integer maxVolumes;
    private EnumWrapper<VServer.Language> language;
    private String nisDomain;
    private Boolean isRepository;
    private String nameServerSwitches;
    private String snapshotPolicy;
    private String aggregatesCSV;
    private String qosPolicyGroupName;
    private String networkIpSpaceName;
    private Boolean isConfigLockedForChanges;
    private Boolean isSmbEncryptionRequired;


    CModeVServer(final CModeCluster parent, final String uuID) {
        super(parent, CModeVServer.class.getSimpleName());
        this.key = new UUIDKey("vserver", uuID);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public UUIDKey getKey() {
        return key;
    }

    public String getName() {
        return name;
    }

    public void setName(final String name) {
        this.name = name;
    }

    public EnumWrapper<VServer.State> getState() {
        return state;
    }

    public void setState(final EnumWrapper<VServer.State> state) {
        this.state = state;
    }

    public EnumWrapper<VServer.OperationalState> getOperationalState() {
        return operationalState;
    }

    public void setOperationState(EnumWrapper<VServer.OperationalState> operationalState) {
        this.operationalState = operationalState;
    }

    public EnumWrapper<VServer.Type> getType() {
        return type;
    }

    public void setType(final EnumWrapper<VServer.Type> type) {
        this.type = type;
    }

    public EnumWrapper<VServer.Subtype> getSubtype() {
        return subtype;
    }

    public void setSubtype(final EnumWrapper<VServer.Subtype> subtype) {
        this.subtype = subtype;
    }

    public List<EnumWrapper<VServer.Protocol>> getWrappedAllowedProtos() {
        return allowedProtos;
    }

    public List<VServer.Protocol> getAllowedProtos() {
        return getProtocolListFromWrapperList(allowedProtos);
    }

    public void setAllowedProtos(final List<EnumWrapper<VServer.Protocol>> allowedProtos) {
        this.allowedProtos = allowedProtos;
    }

    public List<EnumWrapper<VServer.Protocol>> getWrappedDisAllowedProtos() {
        return disAllowedProtos;
    }

    public List<VServer.Protocol> getDisAllowedProtos() {
        return getProtocolListFromWrapperList(disAllowedProtos);
    }

    public void setDisAllowedProtos(final List<EnumWrapper<VServer.Protocol>> disAllowedProtos) {
        this.disAllowedProtos = disAllowedProtos;
    }

    public String getRootVolumeName() {
        return rootVolumeName;
    }

    public void setRootVolumeName(final String rootVolumeName) {
        this.rootVolumeName = rootVolumeName;
    }

    public Integer getMaxVolumes() {
        return maxVolumes;
    }

    public void setMaxVolumes(final Integer maxVolumes) {
        this.maxVolumes = maxVolumes;
    }

    public EnumWrapper<VServer.Language> getLanguage() {
        return language;
    }

    public void setLanguage(final EnumWrapper<VServer.Language> language) {
        this.language = language;
    }

    public String getNisDomain() {
        return nisDomain;
    }

    public void setNisDomain(final String nisDomain) {
        this.nisDomain = nisDomain;
    }

    public Boolean isRepository() {
        return isRepository;
    }

    public void setRepository(final Boolean repository) {
        isRepository = repository;
    }

    public String getNameServerSwitches() {
        return nameServerSwitches;
    }

    public void setNameServerSwitches(final String nameServerSwitches) {
        this.nameServerSwitches = nameServerSwitches;
    }

    public String getSnapshotPolicy() {
        return snapshotPolicy;
    }

    public void setSnapshotPolicy(final String snapshotPolicy) {
        this.snapshotPolicy = snapshotPolicy;
    }

    public String getAggregatesCSV() {
        return aggregatesCSV;
    }

    public void setAggregatesCSV(final String aggregatesCSV) {
        this.aggregatesCSV = aggregatesCSV;
    }

    public String getQosPolicyGroupName() {
        return qosPolicyGroupName;
    }

    public void setQosPolicyGroupName(final String qosPolicyGroupName) {
        this.qosPolicyGroupName = qosPolicyGroupName;
    }

    public String getNetworkIpSpaceName() {
        return networkIpSpaceName;
    }

    public void setNetworkIpSpaceName(String networkIpSpaceName) {
        this.networkIpSpaceName = networkIpSpaceName;
    }
    
    public Boolean getIsConfigLockedForChanges(){
    	return isConfigLockedForChanges;
    }
    
    public void setIsConfigLockedForChanges(Boolean isConfigLockedForChanges){
    	this.isConfigLockedForChanges = isConfigLockedForChanges;
    }

    public Boolean getIsSmbEncryptionRequired() { return isSmbEncryptionRequired; }

    public void setIsSmbEncryptionRequired(Boolean isSmbEncryptionRequired) {
        this.isSmbEncryptionRequired = isSmbEncryptionRequired;
    }


    public static String protoListToEnumCSVString(final List<EnumWrapper<VServer.Protocol>> protocols) {
        final int listLen = protocols.size();
        final StringBuilder stringBuilder = new StringBuilder();
        if (listLen > 0) {
            stringBuilder.append(protocols.get(0));
            for (int i = 1; i < listLen; i++) {
                stringBuilder.append(',').append(protocols.get(i).getEnumValue());
            }
        }
        return stringBuilder.toString();
    }

    public static String protoListToRawCSVString(final List<EnumWrapper<VServer.Protocol>> protocols) {
        final int listLen = protocols.size();
        final StringBuilder stringBuilder = new StringBuilder();
        if (listLen > 0) {
            stringBuilder.append(protocols.get(0).getRawValue());
            for (int i = 1; i < listLen; i++) {
                stringBuilder.append(',').append(protocols.get(i).getRawValue());
            }
        }
        return stringBuilder.toString();
    }

    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttributeNoCRC(writer, "UUID", key.getID());
        addAttributeNoCRC(writer, "Name", name);
        addAttributeNoCRC(writer, "State", state.getEnumValue());
        addAttributeNoCRC(writer, "StateRaw", state.getRawValue());
        addAttributeNoCRC(writer, "OperationalState", operationalState.getEnumValue());
        addAttributeNoCRC(writer, "OperationalStateRaw", operationalState.getRawValue());

        addAttributeNoCRC(writer, "Type", type.getEnumValue());
        addAttributeNoCRC(writer, "TypeRaw", type.getRawValue());
        addAttributeNoCRC(writer, "Subtype", subtype.getEnumValue());
        addAttributeNoCRC(writer, "SubtypeRaw", subtype.getRawValue());

        if (!allowedProtos.isEmpty()) {
            final String allowedProtosVal = protoListToEnumCSVString(allowedProtos);
            addAttributeNoCRC(writer, "AllowedProtocols", allowedProtosVal);

            final String allowedProtosRawVal = protoListToRawCSVString(allowedProtos);
            addAttributeNoCRC(writer, "AllowedProtocolsRaw", allowedProtosRawVal);
        }

        if (!disAllowedProtos.isEmpty()) {
            final String disAllowedProtosVal = protoListToEnumCSVString(disAllowedProtos);
            addAttributeNoCRC(writer, "DisallowedProtocols", disAllowedProtosVal);

            final String disAllowedProtosRawVal = protoListToRawCSVString(disAllowedProtos);
            addAttributeNoCRC(writer, "DisallowedProtocolsRaw", disAllowedProtosRawVal);
        }

        addAttributeNoCRC(writer, "RootVolumeName", rootVolumeName);
        addAttributeNoCRC(writer, "MaxVolumes", maxVolumes);
        addAttributeNoCRC(writer, "Language", language.getEnumValue());
        addAttributeNoCRC(writer, "LanguageRaw", language.getRawValue());
        addAttributeNoCRC(writer, "NISDomain", nisDomain);
        addAttributeNoCRC(writer, "IsRepositoryVServer", isRepository);
        addAttributeNoCRC(writer, "IPSpaceName", networkIpSpaceName);
        addAttributeNoCRC(writer, "IsConfigLockedForChanges", isConfigLockedForChanges);
        addAttributeNoCRC(writer, "NameServerSwitches", nameServerSwitches);
        addAttributeNoCRC(writer, "SnapshotPolicy", snapshotPolicy);
        addAttributeNoCRC(writer, "Aggregates", aggregatesCSV);
        addAttributeNoCRC(writer, "Aggregates", aggregatesCSV);
        addAttributeNoCRC(writer, "QosPolicyGroup", qosPolicyGroupName);
        addAttributeNoCRC(writer, "IsSmbEncryptionRequired", isSmbEncryptionRequired);

    }

    private List<VServer.Protocol> getProtocolListFromWrapperList(final List<EnumWrapper<VServer.Protocol>> enumWrapperList) {
        final List<VServer.Protocol> protocolList = new ArrayList<VServer.Protocol>(enumWrapperList.size());
        for (final EnumWrapper<VServer.Protocol> enumWrapper : enumWrapperList) {
            protocolList.add(enumWrapper.getEnumValue());
        }
        return protocolList;
    }

}
