package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.EnumWrapper;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.ModelKeyBase;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeDiskAggregateRelationship.DiskAggregateRelationshipKey;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeDiskAggregateRelationship.DiskAggregateRelationshipKey.Field;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.VServerPeer;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.VServerPeer.PeerState;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import java.io.IOException;
import java.util.EnumMap;

/**
 * NetApp Foundation Schema implementation of a VServerPeer and its attributes.
 */
public class CModeVServerPeer extends BaseCRCXMLNode<CModeCluster> {

    private final VServerPeerKey key;

    private String vServerUUID;
    private String vServerName;
    private String peerVServerUUID;
    private String peerClusterName;
    private String peerVServerName;
    private EnumWrapper<VServerPeer.PeerState> peerState;
    private String applications;

    CModeVServerPeer(final CModeCluster parent, final String peerClusterName, final String peerVServerName, final String vServerName) {
        super(parent, CModeVServerPeer.class.getSimpleName());
        this.key = new VServerPeerKey(peerClusterName, peerVServerName, vServerName);
    }
    
    public final static class VServerPeerKey extends ModelKeyBase<VServerPeerKey.Field> {
        public static enum Field {
            peer_cluster_name,   
            peer_vserver_name,  
            vserver_name          
        }

        public VServerPeerKey(final String peerClusterName, final String peerVServerName, final String vServerName) {
            super("vserver_peer", new EnumMap<Field, String>(Field.class) {{
                put(Field.peer_cluster_name, peerClusterName);
                put(Field.peer_vserver_name, peerVServerName);
                put(Field.vserver_name, vServerName);
            }});
        }
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public VServerPeerKey getKey() {
        return key;
    }
    
    public String getVServerUUID() {
        return vServerUUID;
    }
    
    public void setVServerUUID(String vServerUUID) {
        this.vServerUUID = vServerUUID;
    }
    
    public String getVServerName() {
        return vServerName;
    }
    
    public void setVServerName(String vServerName) {
        this.vServerName = vServerName;
    }

    public String getPeerVServerUUID() {
        return peerVServerUUID;
    }
    
    public void setPeerVServerUUID(String peerVServerUUID) {
        this.peerVServerUUID = peerVServerUUID;
    }
    
    public String getPeerClusterName() {
        return peerClusterName;
    }

    public void setPeerClusterName(String peerClusterName) {
        this.peerClusterName = peerClusterName;
    }

    public String getPeerVServerName() {
        return peerVServerName;
    }
    
    public void setPeerVServerName(String peerVServerName) {
        this.peerVServerName = peerVServerName;
    }
    
    public EnumWrapper<VServerPeer.PeerState> getPeerState() {
        return peerState;
    }
    
    public void setPeerState(EnumWrapper<VServerPeer.PeerState> peerState) {
        this.peerState = peerState;
    }
    
    public String getApplications() {
        return applications;
    }

    public void setApplications(String applications) {
        this.applications = applications;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addAttributeNoCRC(writer, "UUID", key.getID());
        addAttributeNoCRC(writer, "VServerUUID", vServerUUID);
        addAttributeNoCRC(writer, "PeerVServeUUID", peerVServerUUID);
        addAttributeNoCRC(writer, "PeerClusterName", peerClusterName);
        addAttributeNoCRC(writer, "PeerVServerName", peerVServerName);
        addAttributeNoCRC(writer, "PeerState", peerState.getEnumValue());
        addAttributeNoCRC(writer, "PeerStateRaw", peerState.getRawValue());
        addAttributeNoCRC(writer, "Applications", applications);
    }
}
