package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.data;


/**
 * Data object for Qos Workload Constituent.
 */
public class QosWorkloadConstituentData {

    /**
     * Node workload relationship uuid - node_uuid.workload
     */
    private final String UUID;
    /**
     * Number of dirty buffers (delta)
     */
    private final long dirtyBuffers;
    /**
     * Number of free blocks (delta)
     */
    private final long freeBlocks;
    /**
     * Number of read a head x strips (delta)
     */
    private final long readAheadXStrips;
    /**
     * Total number of dirty buffers (delta), this includes contributions from _WAFL_CP, _BACKGROUND_DELETE &
     * _RA_STRIPE_X
     */
    private double totalDirtyBuffers;
    /**
     * Total Number of free blocks (delta),this includes contributions from _WAFL_CP, _BACKGROUND_DELETE &
     * _RA_STRIPE_X
     */
    private double totalFreeBlocks;
    /**
     * Total Number of read a head x strips (delta), this includes contributions from _WAFL_CP, _BACKGROUND_DELETE &
     * _RA_STRIPE_X
     */
    private double totalReadAheadXStrips;

    /**
     * Create new workload constituent data object.
     *
     * @param uuid             Node workload relationship uuid - node_uuid.workload
     * @param dirtyBuffers     Number of dirty buffers (delta)
     * @param freeBlocks       Number of free blocks (delta)
     * @param readAheadXStrips Number of read ahead x strips (delta)
     */
    public QosWorkloadConstituentData(String uuid, long dirtyBuffers, long freeBlocks, long readAheadXStrips) {
        this.UUID = uuid;
        this.dirtyBuffers = dirtyBuffers;
        this.freeBlocks = freeBlocks;
        this.readAheadXStrips = readAheadXStrips;
        this.totalDirtyBuffers = dirtyBuffers;
        this.totalFreeBlocks = freeBlocks;
        this.totalReadAheadXStrips = readAheadXStrips;
    }

    public long getDirtyBuffers() {
        return dirtyBuffers;
    }

    public long getFreeBlocks() {
        return freeBlocks;
    }

    public long getReadAheadXStrips() {
        return readAheadXStrips;
    }

    public String getUUID() {
        return UUID;
    }

    public double getTotalDirtyBuffers() {
        return totalDirtyBuffers;
    }

    public double getTotalFreeBlocks() {
        return totalFreeBlocks;
    }

    public double getTotalReadAheadXStrips() {
        return totalReadAheadXStrips;
    }

    public void addTotalDirtyBuffers(double dirtyBuffers) {
        this.totalDirtyBuffers += dirtyBuffers;
    }

    public void addTotalFreeBlocks(double freeBlocks) {
        this.totalFreeBlocks += freeBlocks;
    }

    public void addTotalReadAheadXStrips(double readAheadXStrips) {
        this.totalReadAheadXStrips += readAheadXStrips;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        QosWorkloadConstituentData that = (QosWorkloadConstituentData) o;

        if (dirtyBuffers != that.dirtyBuffers) return false;
        if (freeBlocks != that.freeBlocks) return false;
        if (readAheadXStrips != that.readAheadXStrips) return false;
        if (Double.compare(that.totalDirtyBuffers, totalDirtyBuffers) != 0) return false;
        if (Double.compare(that.totalFreeBlocks, totalFreeBlocks) != 0) return false;
        if (Double.compare(that.totalReadAheadXStrips, totalReadAheadXStrips) != 0) return false;
        if (UUID != null ? !UUID.equals(that.UUID) : that.UUID != null) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result;
        long temp;
        result = UUID != null ? UUID.hashCode() : 0;
        result = 31 * result + (int) (dirtyBuffers ^ (dirtyBuffers >>> 32));
        result = 31 * result + (int) (freeBlocks ^ (freeBlocks >>> 32));
        result = 31 * result + (int) (readAheadXStrips ^ (readAheadXStrips >>> 32));
        temp = Double.doubleToLongBits(totalDirtyBuffers);
        result = 31 * result + (int) (temp ^ (temp >>> 32));
        temp = Double.doubleToLongBits(totalFreeBlocks);
        result = 31 * result + (int) (temp ^ (temp >>> 32));
        temp = Double.doubleToLongBits(totalReadAheadXStrips);
        result = 31 * result + (int) (temp ^ (temp >>> 32));
        return result;
    }

    @Override
    public String toString() {
        final StringBuilder sb = new StringBuilder("QosWorkloadConstituentData{");
        sb.append("UUID='").append(UUID).append('\'');
        sb.append(", dirtyBuffers=").append(dirtyBuffers);
        sb.append(", freeBlocks=").append(freeBlocks);
        sb.append(", readAheadXStrips=").append(readAheadXStrips);
        sb.append(", totalDirtyBuffers=").append(totalDirtyBuffers);
        sb.append(", totalFreeBlocks=").append(totalFreeBlocks);
        sb.append(", totalReadAheadXStrips=").append(totalReadAheadXStrips);
        sb.append('}');
        return sb.toString();
    }
}
