package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.data;

/**
 * Data object for QOS Workload Detail.
 */
public class QosWorkloadDetailData {
    /**
     * Workload Detail inventory UUID.
     */
    private final String UUID;
    /**
     * Service time delta in microseconds.
     */
    private long serviceTime;
    /**
     * Service time delta in microseconds.
     */
    private long waitTime;
    /**
     * Service time contribution from "_WAFL_CP" in microseconds.
     */
    private long serviceTimeWAFLCP;
    /**
     * Service time contribution from "_BACKGROUND_DELETE" in microseconds.
     */
    private long serviceTimeBackgroundDelete;
    /**
     * Service time contribution from "_RA_STRIPE_X" in microseconds.
     */
    private long serviceTimeReadAheadXStripe;

    /**
     * Create new workload detail data object.
     *
     * @param UUID Workload Detail inventory UUID.
     */
    public QosWorkloadDetailData(String UUID) {
        this.UUID = UUID;
    }

    public QosWorkloadDetailData(String UUID, long serviceTime, long waitTime) {
        this.UUID = UUID;
        this.serviceTime = serviceTime;
        this.waitTime = waitTime;
    }

    public long getServiceTime() {
        return serviceTime;
    }

    public QosWorkloadDetailData setServiceTime(long serviceTime) {
        this.serviceTime = serviceTime;
        return this;
    }

    public long getWaitTime() {
        return waitTime;
    }

    public QosWorkloadDetailData setWaitTime(long waitTime) {
        this.waitTime = waitTime;
        return this;
    }

    public long getServiceTimeWAFLCP() {
        return serviceTimeWAFLCP;
    }

    public long getServiceTimeBackgroundDelete() {
        return serviceTimeBackgroundDelete;
    }

    public long getServiceTimeReadAheadXStripe() {
        return serviceTimeReadAheadXStripe;
    }

    public QosWorkloadDetailData setServiceTimeWAFLCP(long serviceTimeWAFLCP) {
        this.serviceTimeWAFLCP = serviceTimeWAFLCP;
        return this;
    }

    public QosWorkloadDetailData setServiceTimeBackgroundDelete(long serviceTimeBackgroundDelete) {
        this.serviceTimeBackgroundDelete = serviceTimeBackgroundDelete;
        return this;
    }

    public QosWorkloadDetailData setServiceTimeReadAheadXStripe(long serviceTimeReadAheadXStripe) {
        this.serviceTimeReadAheadXStripe = serviceTimeReadAheadXStripe;
        return this;
    }

    public String getUUID() {
        return UUID;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }

        QosWorkloadDetailData that = (QosWorkloadDetailData) o;

        if (serviceTime != that.serviceTime) {
            return false;
        }
        if (serviceTimeBackgroundDelete != that.serviceTimeBackgroundDelete) {
            return false;
        }
        if (serviceTimeReadAheadXStripe != that.serviceTimeReadAheadXStripe) {
            return false;
        }
        if (serviceTimeWAFLCP != that.serviceTimeWAFLCP) {
            return false;
        }
        if (waitTime != that.waitTime) {
            return false;
        }
        if (UUID != null ? !UUID.equals(that.UUID) : that.UUID != null) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int result = UUID != null ? UUID.hashCode() : 0;
        result = 31 * result + (int) (serviceTime ^ (serviceTime >>> 32));
        result = 31 * result + (int) (waitTime ^ (waitTime >>> 32));
        result = 31 * result + (int) (serviceTimeWAFLCP ^ (serviceTimeWAFLCP >>> 32));
        result = 31 * result + (int) (serviceTimeBackgroundDelete ^ (serviceTimeBackgroundDelete >>> 32));
        result = 31 * result + (int) (serviceTimeReadAheadXStripe ^ (serviceTimeReadAheadXStripe >>> 32));
        return result;
    }

    @Override
    public String toString() {
        final StringBuilder sb = new StringBuilder("QosWorkloadDetailData{");
        sb.append("UUID='").append(UUID).append('\'');
        sb.append(", serviceTime=").append(serviceTime);
        sb.append(", waitTime=").append(waitTime);
        sb.append(", serviceTimeWAFLCP=").append(serviceTimeWAFLCP);
        sb.append(", serviceTimeBackgroundDelete=").append(serviceTimeBackgroundDelete);
        sb.append(", serviceTimeReadAheadXStripe=").append(serviceTimeReadAheadXStripe);
        sb.append('}');
        return sb.toString();
    }
}