package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.impl;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.data.QosWorkloadConstituentData;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.data.QosWorkloadDetailData;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.interfaces.IReAttributionContext;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.interfaces.IReAttributionContext.QosWorkloadNodeRelationshipAttributes;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * In this final step we use the output from step 1 & 2 and re attribute the volume workload details.
 */
public class ReAttributeDeferredWorkServiceStep3 {

    private static final Logger logger = LoggerFactory.getLogger(ReAttributeDeferredWorkServiceStep3.class);

    public ReAttributeDeferredWorkServiceStep3(IReAttributionContext reAttributionContext,
                                               List<QosWorkloadConstituentData> qosWorkloadConstituentDataList,
                                               List<QosWorkloadDetailData> qosWorkloadDetailData, ReAttributeDeferredWorkServiceStep1 reAttributionServiceStep1,
                                               ReAttributeDeferredWorkServiceStep2 reAttributionServiceStep2) {
        perform(reAttributionContext, qosWorkloadConstituentDataList, qosWorkloadDetailData,
                reAttributionServiceStep1, reAttributionServiceStep2);
    }

    /**
     * Use the computed values from step one and step 2 and re-attribute the client workload details.
     */
    private void perform(IReAttributionContext reAttributionContext,
                         List<QosWorkloadConstituentData> qosWorkloadConstituentDataList,
                         List<QosWorkloadDetailData> qosWorkloadDetailData,
                         ReAttributeDeferredWorkServiceStep1 reAttributionServiceStep1,
                         ReAttributeDeferredWorkServiceStep2 reAttributionServiceStep2) {

        final Map<String, HashMap<String, QosWorkloadDetailData>> nodeToWorkloadToCPUDBladeBackgroundMap
                = reAttributionServiceStep1.getNodeToWorkloadToCPUDBladeBackgroundMap();
        final Map<String, HashMap<String, QosWorkloadDetailData>> aggregateToWorkloadToDiskHDDMap
                = reAttributionServiceStep1.getAggregateToWorkloadToDiskHDDMap();
        final Map<String, HashMap<String, QosWorkloadDetailData>> aggregateToWorkloadToDiskSSDMap
                = reAttributionServiceStep1.getAggregateToWorkloadToDiskSSDMap();
        final Map<String, Map<QosWorkloadNodeRelationshipAttributes, String>> qosWorkloadNodeRelationshipMap
                = reAttributionContext.getQosWorkloadNodeRelationshipMap();
        final Map<String, Map<IReAttributionContext.AggregateAttributes, String>> qosWorkloadUUIDToAggregateUUIDMap
                = reAttributionContext.getQosVolumeWorkloadUUIDToAggregateUUIDMap();
        final String clusterUUID = reAttributionContext.getClusterUUID();

        for (QosWorkloadConstituentData workloadConstituentData : qosWorkloadConstituentDataList) {
            final Map<QosWorkloadNodeRelationshipAttributes, String>
                    qosWorkloadConstituent = qosWorkloadNodeRelationshipMap.get(workloadConstituentData.getUUID());
            if (qosWorkloadConstituent != null) {
                final String nodeName = qosWorkloadConstituent.get(QosWorkloadNodeRelationshipAttributes.NODE_NAME);
                final String workloadUUID = qosWorkloadConstituent.get(QosWorkloadNodeRelationshipAttributes.WORKLOAD_UUID);
                final String workloadName = qosWorkloadConstituent.get(QosWorkloadNodeRelationshipAttributes.WORKLOAD_NAME);
                String aggregateUUID = null;
                String aggregateName = null;
                final Map<IReAttributionContext.AggregateAttributes, String> aggregate = qosWorkloadUUIDToAggregateUUIDMap.get(workloadUUID);
                if (aggregate == null) {
                    continue; //ignore system workloads
                } else {
                    aggregateUUID = aggregate.get(IReAttributionContext.AggregateAttributes.AGGREGATE_UUID);
                    aggregateName = aggregate.get(IReAttributionContext.AggregateAttributes.AGGREGATE_NAME);
                }

                final HashMap<String, QosWorkloadDetailData> workloadUUIDToCPUDBladeDetailMap
                        = nodeToWorkloadToCPUDBladeBackgroundMap.get(nodeName);
                QosWorkloadDetailData cpuDBladeBackgroundWorkloadDetail = null;
                if (workloadUUIDToCPUDBladeDetailMap != null) {
                    cpuDBladeBackgroundWorkloadDetail = workloadUUIDToCPUDBladeDetailMap.get(workloadUUID);
                }
                //If the workload detail is not found create one. The original workload must have been zero suppressed
                if(cpuDBladeBackgroundWorkloadDetail == null) {
                    //UUID format ${cluster_uuid}.${node_name}:kernel:${workload name}.CPU_dblade_background
                    final String workloadDetailUUID = new StringBuilder(clusterUUID).append('.')
                            .append(nodeName).append(":kernel:").append(workloadName)
                            .append(".CPU_dblade_background").toString();
                    cpuDBladeBackgroundWorkloadDetail = new QosWorkloadDetailData(workloadDetailUUID,0,0);
                    qosWorkloadDetailData.add(cpuDBladeBackgroundWorkloadDetail);
                }

                QosWorkloadDetailData diskHDDWorkloadDetail = null;
                QosWorkloadDetailData diskSSDWorkloadDetail = null;
                if (aggregateUUID != null) {
                    final HashMap<String, QosWorkloadDetailData> workloadUUIDToDiskHDDDetailMap
                            = aggregateToWorkloadToDiskHDDMap.get(aggregateUUID);

                    if (workloadUUIDToDiskHDDDetailMap != null) {
                        diskHDDWorkloadDetail = workloadUUIDToDiskHDDDetailMap.get(workloadUUID);
                    }
                    //If the workload detail is not found create one. The original workload must have been zero suppressed
                    if(diskHDDWorkloadDetail == null) {
                        //UUID format ${cluster_uuid}.${node_name}:kernel:${workload name}.DISK_HDD_${aggregate}_${aggregate uuid}
                        final String workloadDetailUUID = new StringBuilder(clusterUUID).append('.')
                                .append(nodeName).append(":kernel:").append(workloadName)
                                .append(".DISK_HDD_")
                                .append(aggregateName).append('_').append(aggregateUUID).toString();
                        diskHDDWorkloadDetail = new QosWorkloadDetailData(workloadDetailUUID, 0,0);
                        qosWorkloadDetailData.add(diskHDDWorkloadDetail);
                    }

                    final HashMap<String, QosWorkloadDetailData> workloadUUIDToDiskSSDDetailMap
                            = aggregateToWorkloadToDiskSSDMap.get(aggregateUUID);

                    if (workloadUUIDToDiskSSDDetailMap != null) {
                        diskSSDWorkloadDetail = workloadUUIDToDiskSSDDetailMap.get(workloadUUID);
                    }
                    //If the workload detail is not found create one. The original workload must have been zero suppressed
                    if(diskSSDWorkloadDetail == null) {
                        //UUID format ${cluster_uuid}.${node_name}:kernel:${workload name}.DISK_SSD_${aggregate}_${aggregate uuid}
                        final String workloadDetailUUID = new StringBuilder(clusterUUID).append('.')
                                .append(nodeName).append(":kernel:").append(workloadName)
                                .append(".DISK_SSD_")
                                .append(aggregateName).append('_').append(aggregateUUID).toString();
                        diskSSDWorkloadDetail = new QosWorkloadDetailData(workloadDetailUUID, 0,0);
                        qosWorkloadDetailData.add(diskSSDWorkloadDetail);
                    }
                }

                final double dirtyBuffers = workloadConstituentData.getTotalDirtyBuffers();
                final double freeBlocks = workloadConstituentData.getTotalFreeBlocks();
                final double readAheadXStrips = workloadConstituentData.getTotalReadAheadXStrips();

                if (dirtyBuffers > 0) {
                    processDirtyBuffers(reAttributionServiceStep1, reAttributionServiceStep2, nodeName, aggregateUUID,
                            cpuDBladeBackgroundWorkloadDetail, diskHDDWorkloadDetail, diskSSDWorkloadDetail, dirtyBuffers);
                }
                if (freeBlocks > 0) {
                    processFreeBlocks(reAttributionServiceStep1, reAttributionServiceStep2, nodeName, aggregateUUID,
                            cpuDBladeBackgroundWorkloadDetail, diskHDDWorkloadDetail, diskSSDWorkloadDetail, freeBlocks);
                }
                if (readAheadXStrips > 0) {
                    processReadAheadXStrips(reAttributionServiceStep1, reAttributionServiceStep2, nodeName, aggregateUUID,
                            cpuDBladeBackgroundWorkloadDetail, diskHDDWorkloadDetail, diskSSDWorkloadDetail, readAheadXStrips);
                }
            }
        }

        logger.debug("Results after ReAttributeDeferredWorkServiceStep3 qosWorkloadDetailData {}", qosWorkloadDetailData);
    }

    /**
     * Re-attribute ReadAheadXStrips for CPU DBlade Background, Disk HDD & Disk SSD workload details.
     */
    private void processReadAheadXStrips(ReAttributeDeferredWorkServiceStep1 reAttributionServiceStep1,
                                         ReAttributeDeferredWorkServiceStep2 reAttributionServiceStep2,
                                         String nodeName, String aggregateUUID,
                                         QosWorkloadDetailData cpuDBladeBackgroundWorkloadDetail,
                                         QosWorkloadDetailData diskHDDWorkloadDetail,
                                         QosWorkloadDetailData diskSSDWorkloadDetail, double readAheadXStrips) {
        if (cpuDBladeBackgroundWorkloadDetail != null) {
            final long reAttributedValue = reAttributeDeferredWork(readAheadXStrips,
                    reAttributionServiceStep2.getTotalReadAheadXStripeByNode().get(nodeName),
                    reAttributionServiceStep1.getRaXStripeCPUdBladeByNode().get(nodeName));
            cpuDBladeBackgroundWorkloadDetail.setServiceTimeReadAheadXStripe(reAttributedValue);
        }
        if (diskHDDWorkloadDetail != null) {
            final long reAttributedValue = reAttributeDeferredWork(readAheadXStrips,
                    reAttributionServiceStep2.getTotalReadAheadXStripeByAggregate().get(aggregateUUID),
                    reAttributionServiceStep1.getRaXStripeDiskHDDByAggregate().get(aggregateUUID));
            diskHDDWorkloadDetail.setServiceTimeReadAheadXStripe(reAttributedValue);
        }
        if (diskSSDWorkloadDetail != null) {
            final long reAttributedValue = reAttributeDeferredWork(readAheadXStrips,
                    reAttributionServiceStep2.getTotalReadAheadXStripeByAggregate().get(aggregateUUID),
                    reAttributionServiceStep1.getRaXStripeDiskSSDByAggregate().get(aggregateUUID));
            diskSSDWorkloadDetail.setServiceTimeReadAheadXStripe(reAttributedValue);
        }
    }

    /**
     * Re-attribute Free Blocks for CPU DBlade Background, Disk HDD & Disk SSD workload details.
     */
    private void processFreeBlocks(ReAttributeDeferredWorkServiceStep1 reAttributionServiceStep1,
                                   ReAttributeDeferredWorkServiceStep2 reAttributionServiceStep2,
                                   String nodeName, String aggregateUUID,
                                   QosWorkloadDetailData cpuDBladeWorkloadDetail,
                                   QosWorkloadDetailData diskHDDWorkloadDetail,
                                   QosWorkloadDetailData diskSSDWorkloadDetail, double freeBlocks) {
        if (cpuDBladeWorkloadDetail != null) {
            final long reAttributedValue = reAttributeDeferredWork(freeBlocks,
                    reAttributionServiceStep2.getTotalFreedBlocksByNode().get(nodeName),
                    reAttributionServiceStep1.getBackgroundDeleteCPUdBladeByNode().get(nodeName));
            cpuDBladeWorkloadDetail.setServiceTimeBackgroundDelete(reAttributedValue);
        }
        if (diskHDDWorkloadDetail != null) {
            final long reAttributedValue = reAttributeDeferredWork(freeBlocks,
                    reAttributionServiceStep2.getTotalFreedBlocksByAggregate().get(aggregateUUID),
                    reAttributionServiceStep1.getBackgroundDeleteDiskHDDByAggregate().get(aggregateUUID));
            diskHDDWorkloadDetail.setServiceTimeBackgroundDelete(reAttributedValue);
        }
        if (diskSSDWorkloadDetail != null) {
            final long reAttributedValue = reAttributeDeferredWork(freeBlocks,
                    reAttributionServiceStep2.getTotalFreedBlocksByAggregate().get(aggregateUUID),
                    reAttributionServiceStep1.getBackgroundDeleteDiskSSDByAggregate().get(aggregateUUID));
            diskSSDWorkloadDetail.setServiceTimeBackgroundDelete(reAttributedValue);
        }
    }

    /**
     * Re-attribute dirty buffers for CPU DBlade Background, Disk HDD & Disk SSD workload details.
     */
    private void processDirtyBuffers(ReAttributeDeferredWorkServiceStep1 reAttributionServiceStep1,
                                     ReAttributeDeferredWorkServiceStep2 reAttributionServiceStep2,
                                     String nodeName, String aggregateUUID,
                                     QosWorkloadDetailData cpuDBladeWorkloadDetail,
                                     QosWorkloadDetailData diskHDDWorkloadDetail,
                                     QosWorkloadDetailData diskSSDWorkloadDetail, double dirtyBuffers) {
        if (cpuDBladeWorkloadDetail != null) {
            final long reAttributedValue = reAttributeDeferredWork(dirtyBuffers,
                    reAttributionServiceStep2.getTotalDirtyBuffersByNode().get(nodeName),
                    reAttributionServiceStep1.getWaflCPCPUdBladeByNode().get(nodeName));
            cpuDBladeWorkloadDetail.setServiceTimeWAFLCP(reAttributedValue);
        }
        if (diskHDDWorkloadDetail != null) {
            final long reAttributedValue = reAttributeDeferredWork(dirtyBuffers,
                    reAttributionServiceStep2.getTotalDirtyBuffersByAggregate().get(aggregateUUID),
                    reAttributionServiceStep1.getWaflCPDiskHDDByAggregate().get(aggregateUUID));
            diskHDDWorkloadDetail.setServiceTimeWAFLCP(reAttributedValue);
        }
        if (diskSSDWorkloadDetail != null) {
            final long reAttributedValue = reAttributeDeferredWork(dirtyBuffers,
                    reAttributionServiceStep2.getTotalDirtyBuffersByAggregate().get(aggregateUUID),
                    reAttributionServiceStep1.getWaflCPDiskSSDByAggregate().get(aggregateUUID));
            diskSSDWorkloadDetail.setServiceTimeWAFLCP(reAttributedValue);
        }
    }

    /**
     * Compute the amount of deferred work that has to be re attributed.
     */
    private long reAttributeDeferredWork(final double dbFbRaForWorkload, final Double sumOfDbFbRaByNodeOrAggregate,
                                         final Long deferredWork) {
        long reAttributedValue = 0l;
        if (deferredWork != null && sumOfDbFbRaByNodeOrAggregate != null && deferredWork != 0
                && sumOfDbFbRaByNodeOrAggregate != 0) {
            final double ratioOfDeferredWork = dbFbRaForWorkload / sumOfDbFbRaByNodeOrAggregate;
            reAttributedValue = (long) (ratioOfDeferredWork * deferredWork); //Truncate the decimals
        }
        return reAttributedValue;
    }

}
