package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.reattribution.interfaces;

import java.util.Map;

/**
 * Re Attribution context contains the foundation data required for the re attribution service.
 */
public interface IReAttributionContext {

    /**
     * Returns the UUID of the cluster.
     */
    public String getClusterUUID();

    /**
     * Map of volume workload to aggregate - cluster_uuid.workload_uuid to aggregate_uuid
     *
     * @return Map of volume workload to aggregate.
     */
    public Map<String, Map<AggregateAttributes, String>> getQosVolumeWorkloadUUIDToAggregateUUIDMap();

    /**
     * Map of workload node relationship uuid (node_uuid.workload_uuid) to {@link QosWorkloadNodeRelationshipAttributes}
     *
     * @return Map of workload node relationship uuid to {@link QosWorkloadNodeRelationshipAttributes}
     */
    public Map<String, Map<QosWorkloadNodeRelationshipAttributes, String>> getQosWorkloadNodeRelationshipMap();

    /**
     * Map of workload detail UUID (cluster_uuid.workload_detail_uuid) to {@link QosWorkloadDetailAttributes}
     *
     * @return Map of workload detail UUID  to {@link QosWorkloadDetailAttributes}
     */
    public Map<String, Map<QosWorkloadDetailAttributes, String>> getQosWorkloadDetailMap();

    public enum QosWorkloadDetailAttributes {
        /**
         * Node name associated with the workload detail.
         */
        NODE_NAME,
        /**
         * UUID of the workload (cluster_uuid.workload_uuid) associated with the workload detail
         */
        WORKLOAD_UUID,
        /**
         * UUID of the aggregate associated with the workload detail. Only available for aggregate level workloads.
         * such as DISK_HDD & DISK_SSD
         */
        AGGREGATE_UUID,
    }

    public enum QosWorkloadNodeRelationshipAttributes {
        /**
         * Node Name of the node associated with the workload constituent
         */
        NODE_NAME,
        /**
         * UUID of the workload (cluster_uuid.workload_uuid) associated with the workload constituent
         */
        WORKLOAD_UUID,
        /**
         * Name of the workload associated with the workload constituent
         */
        WORKLOAD_NAME,
    }

    public enum AggregateAttributes {
        /**
         * Aggregate UUID
         */
        AGGREGATE_UUID,
        /**
         * Aggregate Name
         */
        AGGREGATE_NAME,
    }


}
