package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.synthetic.builders;

import java.math.BigInteger;
import java.util.Collection;
import java.util.Map;

import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifVserverMeasurements;
import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifVserverRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.builders.FcpLifMetricsBuilder;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.builders.FcpLifMetricsBuilder.FcpLIFCounter;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.GlobalScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster.ClusterKey;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

public class FcpLifVserverMetricsBuilder extends SyntheticBuilderBase<FcpLifVserverMeasurements,
        FcpLifVserverRawMeasurements, Cluster.ClusterKey, FcpLifMetricsBuilder.FcpLIFCounter> {

    public FcpLifVserverMetricsBuilder(NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage);
    }

    @Override
    protected FcpLifVserverRawMeasurements rawMetrics(MetricsRowScratchPad<FcpLIFCounter> currentScratchPad,
                                                        NetAppDefaultStorageContext context) {

        final FcpLifVserverRawMeasurements m = new FcpLifVserverRawMeasurements();
        Map<FcpLIFCounter,BigInteger> counterValue = currentScratchPad.getCounterValues();
        m.setAvgLatency(counterValue.get(FcpLIFCounter.avg_latency));
        m.setAvgOtherLatency(counterValue.get(FcpLIFCounter.avg_other_latency));
        m.setAvgReadLatency(counterValue.get(FcpLIFCounter.avg_read_latency));
        m.setAvgWriteLatency(counterValue.get(FcpLIFCounter.avg_write_latency));
        m.setOtherOps(counterValue.get(FcpLIFCounter.other_ops));
        m.setReadOps(counterValue.get(FcpLIFCounter.read_ops));
        m.setWriteOps(counterValue.get(FcpLIFCounter.write_ops));
        m.setTotalOps(counterValue.get(FcpLIFCounter.total_ops));
        m.setReadData(counterValue.get(FcpLIFCounter.read_data));
        m.setWriteData(counterValue.get(FcpLIFCounter.write_data));
        m.setThroughput(MetricsRowScratchPad.add(
                m.getReadData(), m.getWriteData()));

        return m;
    }

    @Override
    public void reportRawPerformance(ClusterKey clusterKey,
                                     Map<ClusterKey, Collection<RawSample<FcpLifVserverRawMeasurements>>> rawMeasurements)
            throws DataSourceErrorException {

        for (final Map.Entry<ClusterKey, Collection<RawSample<FcpLifVserverRawMeasurements>>> measurementEntry : rawMeasurements.entrySet()) {
            final ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<FcpLifVserverRawMeasurements>> fcpLifMeasurements = measurementEntry.getValue();
            reportingPackage.reportFcpLifVserverRawPerformance(key, fcpLifMeasurements);
        }

    }

    @Override
    protected ClusterKey getParentInventoryUUIDKey(
            NetAppDefaultStorageContext context, String inventoryUUID,
            MetricsRowScratchPad<FcpLIFCounter> metricsRowScratchPad) {

        return context.getKey();
    }

    /**
     * Get the reporting object inventoryUUID. In this case it's vserver's UUID.
     */
    @Override
    protected String getInventoryUUID(NetAppDefaultStorageContext context, String vserverUuid,
                                      MetricsRowScratchPad<FcpLIFCounter> metricsRowScratchPad) {
        return vserverUuid;
    }

    @Override
    protected Map<String, MetricsRowScratchPad<FcpLIFCounter>> generateMetricsRowScratchPads(
            NetAppDefaultStorageContext context, ZAPIConnection zapiConnection,
            GlobalScratchPad globalScratchPad) {
        return globalScratchPad.merge(FcpLIFCounter.class, "fcp_lif", FcpLIFCounter.vserver_uuid);
    }

    @Override
    protected FcpLifVserverMeasurements normalizeMetrics(
            MetricsRowScratchPad<FcpLIFCounter> currentScratchPad,
            NetAppDefaultStorageContext context) {
        final FcpLifVserverMeasurements m = new FcpLifVserverMeasurements();

        m.setAvgLatency(currentScratchPad.getLatencyPerOp(FcpLIFCounter.avg_latency, FcpLIFCounter.total_ops));
        m.setAvgOtherLatency(currentScratchPad.getLatencyPerOp(FcpLIFCounter.avg_other_latency, FcpLIFCounter.other_ops));
        m.setAvgReadLatency(currentScratchPad.getLatencyPerOp(FcpLIFCounter.avg_read_latency, FcpLIFCounter.read_ops));
        m.setAvgWriteLatency(currentScratchPad.getLatencyPerOp(FcpLIFCounter.avg_write_latency, FcpLIFCounter.write_ops));
        m.setOtherOps(currentScratchPad.getCounterRatePerSec(FcpLIFCounter.other_ops));
        m.setReadOps(currentScratchPad.getCounterRatePerSec(FcpLIFCounter.read_ops));
        m.setTotalOps(currentScratchPad.getCounterRatePerSec(FcpLIFCounter.total_ops));
        m.setWriteOps(currentScratchPad.getCounterRatePerSec(FcpLIFCounter.write_ops));

        float readData = currentScratchPad.getCounterRatePerSec(FcpLIFCounter.read_data);
        float writeData = currentScratchPad.getCounterRatePerSec(FcpLIFCounter.write_data);

        m.setReadData(readData);
        m.setWriteData(writeData);
        m.setThroughput(MetricsRowScratchPad.add(readData, writeData));

        return m;
    }

    @Override
    public void reportPerformance(ClusterKey clusterKey,
                                  Map<ClusterKey, Collection<RawSample<FcpLifVserverMeasurements>>> measurements) throws DataSourceErrorException {

        for (final Map.Entry<ClusterKey, Collection<RawSample<FcpLifVserverMeasurements>>> measurementEntry : measurements.entrySet()) {
            final ClusterKey key = measurementEntry.getKey();
            final Collection<RawSample<FcpLifVserverMeasurements>> fcpLifMeasurements = measurementEntry.getValue();
            reportingPackage.reportFcpLifVserverPerformance(key, fcpLifMeasurements);
        }
    }
}
