package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.synthetic.builders;

import java.math.BigInteger;
import java.util.Collection;
import java.util.Map;

import com.netapp.oci.netapp.performance.iscsilif.interfaces.data.IscsiLifVserverMeasurements;
import com.netapp.oci.netapp.performance.iscsilif.interfaces.data.IscsiLifVserverRawMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.NetAppOCIEPerformancePackage;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.builders.ISCSILIFMetricsBuilder;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.builders.ISCSILIFMetricsBuilder.ISCSILIFCounter;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.GlobalScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.MetricsRowScratchPad;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi.ZAPIConnection;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceErrorException;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster.ClusterKey;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.NetAppDefaultStorageContext;

public class IscsiLifVserverMetricsBuilder extends SyntheticBuilderBase<IscsiLifVserverMeasurements, 
	IscsiLifVserverRawMeasurements, Cluster.ClusterKey, ISCSILIFMetricsBuilder.ISCSILIFCounter> {

    public IscsiLifVserverMetricsBuilder(NetAppOCIEPerformancePackage reportingPackage) {
        super(reportingPackage);
    }
	
	@Override
	protected IscsiLifVserverRawMeasurements rawMetrics(MetricsRowScratchPad<ISCSILIFCounter> currentScratchPad,
			NetAppDefaultStorageContext context) {
		
		final IscsiLifVserverRawMeasurements m = new IscsiLifVserverRawMeasurements();
        Map<ISCSILIFCounter,BigInteger> counterValue = currentScratchPad.getCounterValues();
        m.setAvgLatency(counterValue.get(ISCSILIFCounter.avg_latency));
        m.setAvgOtherLatency(counterValue.get(ISCSILIFCounter.avg_other_latency));
        m.setAvgReadLatency(counterValue.get(ISCSILIFCounter.avg_read_latency));
        m.setAvgWriteLatency(counterValue.get(ISCSILIFCounter.avg_write_latency));
        m.setCmdTransfered(counterValue.get(ISCSILIFCounter.cmd_transfered));
        m.setIscsiOtherOps(counterValue.get(ISCSILIFCounter.iscsi_other_ops));
        m.setIscsiReadOps(counterValue.get(ISCSILIFCounter.iscsi_read_ops));
        m.setIscsiWriteOps(counterValue.get(ISCSILIFCounter.iscsi_write_ops));
        m.setReadData(counterValue.get(ISCSILIFCounter.read_data));
        m.setWriteData(counterValue.get(ISCSILIFCounter.write_data));
        
        m.setOpmIscsiLifOps(MetricsRowScratchPad.add(m.getIscsiOtherOps(),
        		m.getIscsiReadOps(), m.getIscsiWriteOps()));
        m.setOpmIscsiLifThroughput(MetricsRowScratchPad.add(m.getReadData(), m.getWriteData()));
        
        return m;
	}

	@Override
	public void reportRawPerformance(ClusterKey clusterKey,
			Map<ClusterKey, Collection<RawSample<IscsiLifVserverRawMeasurements>>> rawMeasurements)
			throws DataSourceErrorException {

		for (final Map.Entry<ClusterKey, Collection<RawSample<IscsiLifVserverRawMeasurements>>> measurementEntry : rawMeasurements.entrySet()) { 
            final ClusterKey key = measurementEntry.getKey(); 
            final Collection<RawSample<IscsiLifVserverRawMeasurements>> lifMeasurements = measurementEntry.getValue(); 
            reportingPackage.reportIscsiLifVserverRawPerformance(key, lifMeasurements); 
        } 		
		
	}

	@Override
	protected ClusterKey getParentInventoryUUIDKey(
			NetAppDefaultStorageContext context, String inventoryUUID,
			MetricsRowScratchPad<ISCSILIFCounter> metricsRowScratchPad) {
		
		return context.getKey();
	}

	/**
	 * Get the reporting object inventoryUUID. In this case it's vserver's UUID.
	 */
	@Override
	protected String getInventoryUUID(NetAppDefaultStorageContext context, String vserverUuid, 
			MetricsRowScratchPad<ISCSILIFCounter> metricsRowScratchPad) {
		return vserverUuid;
	}
	
	@Override
	protected Map<String, MetricsRowScratchPad<ISCSILIFCounter>> generateMetricsRowScratchPads(
			NetAppDefaultStorageContext context, ZAPIConnection zapiConnection,
			GlobalScratchPad globalScratchPad) {
		return globalScratchPad.merge(ISCSILIFCounter.class, "iscsi_lif", ISCSILIFCounter.vserver_uuid);
	}

	@Override
	protected IscsiLifVserverMeasurements normalizeMetrics(
			MetricsRowScratchPad<ISCSILIFCounter> currentScratchPad,
			NetAppDefaultStorageContext context) {
        final IscsiLifVserverMeasurements m = new IscsiLifVserverMeasurements();

        m.setAvgLatency(currentScratchPad.getLatencyPerOp(ISCSILIFCounter.avg_latency, ISCSILIFCounter.cmd_transfered));
        m.setAvgOtherLatency(currentScratchPad.getLatencyPerOp(ISCSILIFCounter.avg_other_latency, ISCSILIFCounter.iscsi_other_ops));
        m.setAvgReadLatency(currentScratchPad.getLatencyPerOp(ISCSILIFCounter.avg_read_latency, ISCSILIFCounter.iscsi_read_ops));
        m.setAvgWriteLatency(currentScratchPad.getLatencyPerOp(ISCSILIFCounter.avg_write_latency, ISCSILIFCounter.iscsi_write_ops));
        m.setCmdTransfered(currentScratchPad.getCounterDeltaFinal(ISCSILIFCounter.cmd_transfered));
        m.setIscsiReadOps(currentScratchPad.getCounterRatePerSec(ISCSILIFCounter.iscsi_read_ops));
        m.setIscsiWriteOps(currentScratchPad.getCounterRatePerSec(ISCSILIFCounter.iscsi_write_ops));
        m.setIscsiOtherOps(currentScratchPad.getCounterRatePerSec(ISCSILIFCounter.iscsi_other_ops));
        m.setReadData(currentScratchPad.getCounterRatePerSec(ISCSILIFCounter.read_data));
        m.setWriteData(currentScratchPad.getCounterRatePerSec(ISCSILIFCounter.write_data));

        m.setOpmIscsiLifOps(MetricsRowScratchPad.add(m.getIscsiReadOps(),m.getIscsiWriteOps(), m.getIscsiOtherOps()));
        m.setOpmIscsiLifThroughput(MetricsRowScratchPad.add(m.getReadData(), m.getWriteData()));

        return m;
	}

	@Override
	public void reportPerformance(ClusterKey clusterKey,
			Map<ClusterKey, Collection<RawSample<IscsiLifVserverMeasurements>>> measurements) throws DataSourceErrorException {

		for (final Map.Entry<ClusterKey, Collection<RawSample<IscsiLifVserverMeasurements>>> measurementEntry : measurements.entrySet()) {
			final ClusterKey key = measurementEntry.getKey();
			final Collection<RawSample<IscsiLifVserverMeasurements>> lifMeasurements = measurementEntry.getValue();
			reportingPackage.reportIscsiLifVserverPerformance(key, lifMeasurements);
		}		
	}

}
