package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.synthetic.mergers;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.*;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.synthetic.builders.NFSMetricsBuilder;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.synthetic.builders.NFSMetricsBuilder.NFSCounter;

import java.math.BigInteger;
import java.util.EnumMap;


public class NFSMetricsMergerSN extends MergeByPerformanceUUID<NFSCounter> {

    public NFSMetricsMergerSN() {
        super(NFSCounter.class);
    }

    @Override
    public MetricsRowScratchPad<NFSMetricsBuilder.NFSCounter> mergeCounters(MetricsRowScratchPad<NFSCounter> resultScratchPad,
                                                                            String mergeWithCounterObjectName,
                                                                            MetricsRowScratchPad<? extends Enum<?>> mergeWithScratchPad) {
        switch (mergeWithCounterObjectName) {
            case "nfsv3":
                MetricsRowScratchPad<NetappNfsv3Metrics.Nfsv3Counter> nfsV3ScratchPad =
                        (MetricsRowScratchPad<NetappNfsv3Metrics.Nfsv3Counter>) mergeWithScratchPad;
                final EnumMap<NetappNfsv3Metrics.Nfsv3Counter, BigInteger> nfsV3Counters = nfsV3ScratchPad.getCounterValues();
                //Merge Current
                mergeNFSv3ScratchPad(resultScratchPad, nfsV3Counters);
                if (resultScratchPad.isPreviousPollValuesAvailable() && nfsV3ScratchPad.isPreviousPollValuesAvailable()) {
                    //Merge Previous
                    mergeNFSv3ScratchPad(resultScratchPad.getPreviousScratchPad(),
                            nfsV3ScratchPad.getPreviousScratchPad().getCounterValues());
                }
                break;
            case "nfsv4":
                MetricsRowScratchPad<NetappNfsv4Metrics.Nfsv4Counter> nfsV4ScratchPad =
                        (MetricsRowScratchPad<NetappNfsv4Metrics.Nfsv4Counter>) mergeWithScratchPad;
                final EnumMap<NetappNfsv4Metrics.Nfsv4Counter, BigInteger> nfsV4Counters = nfsV4ScratchPad.getCounterValues();
                mergeNFSv4ScratchPad(resultScratchPad, nfsV4Counters);
                if (resultScratchPad.isPreviousPollValuesAvailable() && nfsV4ScratchPad.isPreviousPollValuesAvailable()) {
                    mergeNFSv4ScratchPad(resultScratchPad.getPreviousScratchPad(),
                            nfsV4ScratchPad.getPreviousScratchPad().getCounterValues());
                }
                break;
            case "nfsv4_1":
                MetricsRowScratchPad<NetappNfsv41Metrics.Nfsv41Counter> nfsV41ScratchPad =
                        (MetricsRowScratchPad<NetappNfsv41Metrics.Nfsv41Counter>) mergeWithScratchPad;
                final EnumMap<NetappNfsv41Metrics.Nfsv41Counter, BigInteger> nfsV41Counters = nfsV41ScratchPad.getCounterValues();
                mergeNFSv41ScratchPad(resultScratchPad, nfsV41Counters);
                if (resultScratchPad.isPreviousPollValuesAvailable() && nfsV41ScratchPad.isPreviousPollValuesAvailable()) {
                    mergeNFSv41ScratchPad(resultScratchPad.getPreviousScratchPad(),
                            nfsV41ScratchPad.getPreviousScratchPad().getCounterValues());
                }
                break;
        }
        return resultScratchPad;
    }

    private void mergeNFSv41ScratchPad(MetricsRowScratchPad<NFSCounter> resultScratchPad, EnumMap<NetappNfsv41Metrics.Nfsv41Counter, BigInteger> nfsV41Counters) {
        //Add Ops
        resultScratchPad.addToCounter(NFSCounter.opm_nfs_ops, nfsV41Counters.get(NetappNfsv41Metrics.Nfsv41Counter.nfsv4_1_ops));
        resultScratchPad.addToCounter(NFSCounter.opm_nfs_read_ops, nfsV41Counters.get(NetappNfsv41Metrics.Nfsv41Counter.read_total));
        resultScratchPad.addToCounter(NFSCounter.opm_nfs_write_ops, nfsV41Counters.get(NetappNfsv41Metrics.Nfsv41Counter.write_total));
        //Add Throughput
        resultScratchPad.addToCounter(NFSCounter.opm_nfs_throughput, nfsV41Counters.get(NetappNfsv41Metrics.Nfsv41Counter.nfs41_throughput));
        resultScratchPad.addToCounter(NFSCounter.opm_nfs_read_throughput, nfsV41Counters.get(NetappNfsv41Metrics.Nfsv41Counter.nfs41_read_throughput));
        resultScratchPad.addToCounter(NFSCounter.opm_nfs_write_throughput, nfsV41Counters.get(NetappNfsv41Metrics.Nfsv41Counter.nfs41_write_throughput));
        //Add Latency
        resultScratchPad.addToCounter(NFSCounter.opm_nfs_read_latency, nfsV41Counters.get(NetappNfsv41Metrics.Nfsv41Counter.read_avg_latency));
        resultScratchPad.addToCounter(NFSCounter.opm_nfs_write_latency, nfsV41Counters.get(NetappNfsv41Metrics.Nfsv41Counter.write_avg_latency));
    }

    private void mergeNFSv4ScratchPad(MetricsRowScratchPad<NFSCounter> resultScratchPad, EnumMap<NetappNfsv4Metrics.Nfsv4Counter, BigInteger> nfsV4Counters) {
        //Add Ops
        resultScratchPad.addToCounter(NFSCounter.opm_nfs_ops, nfsV4Counters.get(NetappNfsv4Metrics.Nfsv4Counter.nfsv4_ops));
        resultScratchPad.addToCounter(NFSCounter.opm_nfs_read_ops, nfsV4Counters.get(NetappNfsv4Metrics.Nfsv4Counter.read_total));
        resultScratchPad.addToCounter(NFSCounter.opm_nfs_write_ops, nfsV4Counters.get(NetappNfsv4Metrics.Nfsv4Counter.write_total));
        //Add Throughput
        resultScratchPad.addToCounter(NFSCounter.opm_nfs_throughput, nfsV4Counters.get(NetappNfsv4Metrics.Nfsv4Counter.nfs4_throughput));
        resultScratchPad.addToCounter(NFSCounter.opm_nfs_read_throughput, nfsV4Counters.get(NetappNfsv4Metrics.Nfsv4Counter.nfs4_read_throughput));
        resultScratchPad.addToCounter(NFSCounter.opm_nfs_write_throughput, nfsV4Counters.get(NetappNfsv4Metrics.Nfsv4Counter.nfs4_write_throughput));
        //Add Latency
        resultScratchPad.addToCounter(NFSCounter.opm_nfs_read_latency, nfsV4Counters.get(NetappNfsv4Metrics.Nfsv4Counter.read_avg_latency));
        resultScratchPad.addToCounter(NFSCounter.opm_nfs_write_latency, nfsV4Counters.get(NetappNfsv4Metrics.Nfsv4Counter.write_avg_latency));
    }

    private void mergeNFSv3ScratchPad(MetricsRowScratchPad<NFSCounter> resultScratchPad, EnumMap<NetappNfsv3Metrics.Nfsv3Counter, BigInteger> nfsV3Counters) {
        //Add OPS
        resultScratchPad.addToCounter(NFSCounter.opm_nfs_ops, nfsV3Counters.get(NetappNfsv3Metrics.Nfsv3Counter.nfsv3_ops));
        resultScratchPad.addToCounter(NFSCounter.opm_nfs_read_ops, nfsV3Counters.get(NetappNfsv3Metrics.Nfsv3Counter.nfsv3_read_ops));
        resultScratchPad.addToCounter(NFSCounter.opm_nfs_write_ops, nfsV3Counters.get(NetappNfsv3Metrics.Nfsv3Counter.nfsv3_write_ops));
        //Add Throughput
        resultScratchPad.addToCounter(NFSCounter.opm_nfs_throughput, nfsV3Counters.get(NetappNfsv3Metrics.Nfsv3Counter.nfsv3_throughput));
        resultScratchPad.addToCounter(NFSCounter.opm_nfs_read_throughput, nfsV3Counters.get(NetappNfsv3Metrics.Nfsv3Counter.nfsv3_read_throughput));
        resultScratchPad.addToCounter(NFSCounter.opm_nfs_write_throughput, nfsV3Counters.get(NetappNfsv3Metrics.Nfsv3Counter.nfsv3_write_throughput));
        //Add Latency
        resultScratchPad.addToCounter(NFSCounter.opm_nfs_read_latency, nfsV3Counters.get(NetappNfsv3Metrics.Nfsv3Counter.read_avg_latency));
        resultScratchPad.addToCounter(NFSCounter.opm_nfs_write_latency, nfsV3Counters.get(NetappNfsv3Metrics.Nfsv3Counter.write_avg_latency));
    }
}
