package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.synthetic.mergers;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.builders.FcpLifMetricsBuilder;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.archiver.builders.Nfsv41MetricsBuilder;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.metrics.*;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.synthetic.builders.VserverMetricsBuilder;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.synthetic.builders.VserverMetricsBuilder.VserverCounter;

import java.math.BigInteger;
import java.util.EnumMap;
import java.util.Map;

/**
 *  This Merger merges (nfsv3, mfsv4, nfs4v_1, cifs:vserver, iscsi_lif:vseerver and fcp_lip:vserver) into
 *  Map<String, MetricsRowScratchPad<VserverMetricsBuilder.VserverCounter> , where String here is a unique key
 *  representing vserverUUID
 */
public class VserverMetricsMergerSN extends MergeByPerformanceUUID<VserverMetricsBuilder.VserverCounter> {

    public VserverMetricsMergerSN() {
        super(VserverMetricsBuilder.VserverCounter.class);
    }

    /**
     * For fcp_lif and Iscsi_lif the performanceUUD is not vserverUUID, so handle them separately so that merge
     * happens on vserverUUID and not its performacenUUID. This is done in the super class.
     * @param counterInstances
     * @param counterObjectName
     * @return
     */
    @Override
    public String retrievePerfUUID(Map.Entry<String, ? extends MetricsRowScratchPad<? extends Enum<?>>> counterInstances, String counterObjectName){
        final String uuid;
        // All the matched cases has vserver_id as the performacneUUID, we need to change it as vserverUUID which in this
        // case is inventoryUUID
        switch(counterObjectName) {
            case "nfsv3":
            case "nfsv4":
            case "nfsv4_1":
            case "cifs:vserver":
                uuid = counterInstances.getValue().getInventoryUUID();
                break;
            default:
                uuid = counterInstances.getKey();
        }
        return uuid;
    }

    /**
     * Construct Result ScratchPad by partially merging them for each counter Object type
     * @param resultScratchPad The scratch pad to be updated with the merge results.
     * @param mergeWithCounterObjectName Name of the counter object to merge with.
     * @param mergeWithScratchPad Scratchpad to merge with.
     * @return
     */
    @Override
    public MetricsRowScratchPad<VserverCounter> mergeCounters(MetricsRowScratchPad<VserverCounter> resultScratchPad,
                                                              String mergeWithCounterObjectName,
                                                              MetricsRowScratchPad<? extends Enum<?>> mergeWithScratchPad) {
        switch (mergeWithCounterObjectName) {
            case "nfsv3":
                MetricsRowScratchPad<NetappNfsv3Metrics.Nfsv3Counter> nfsV3ScratchPad =
                        (MetricsRowScratchPad<NetappNfsv3Metrics.Nfsv3Counter>) mergeWithScratchPad;
                final EnumMap<NetappNfsv3Metrics.Nfsv3Counter, BigInteger> nfsV3Counters = nfsV3ScratchPad.getCounterValues();
                //Merge Current
                mergeNFSv3ScratchPad(resultScratchPad, nfsV3Counters);
                if (resultScratchPad.isPreviousPollValuesAvailable() && nfsV3ScratchPad.isPreviousPollValuesAvailable()) {
                    //Merge Previous
                    mergeNFSv3ScratchPad(resultScratchPad.getPreviousScratchPad(),
                            nfsV3ScratchPad.getPreviousScratchPad().getCounterValues());
                }
                break;
            case "nfsv4":
                MetricsRowScratchPad<NetappNfsv4Metrics.Nfsv4Counter> nfsV4ScratchPad =
                        (MetricsRowScratchPad<NetappNfsv4Metrics.Nfsv4Counter>) mergeWithScratchPad;
                final EnumMap<NetappNfsv4Metrics.Nfsv4Counter, BigInteger> nfsV4Counters = nfsV4ScratchPad.getCounterValues();
                mergeNFSv4ScratchPad(resultScratchPad, nfsV4Counters);
                if (resultScratchPad.isPreviousPollValuesAvailable() && nfsV4ScratchPad.isPreviousPollValuesAvailable()) {
                    mergeNFSv4ScratchPad(resultScratchPad.getPreviousScratchPad(),
                            nfsV4ScratchPad.getPreviousScratchPad().getCounterValues());
                }
                break;
            case "nfsv4_1":
                MetricsRowScratchPad<NetappNfsv41Metrics.Nfsv41Counter> nfsV41ScratchPad =
                        (MetricsRowScratchPad<NetappNfsv41Metrics.Nfsv41Counter>) mergeWithScratchPad;
                final EnumMap<NetappNfsv41Metrics.Nfsv41Counter, BigInteger> nfsV41Counters = nfsV41ScratchPad.getCounterValues();
                mergeNFSv41ScratchPad(resultScratchPad, nfsV41Counters);
                if (resultScratchPad.isPreviousPollValuesAvailable() && nfsV41ScratchPad.isPreviousPollValuesAvailable()) {
                    mergeNFSv41ScratchPad(resultScratchPad.getPreviousScratchPad(),
                            nfsV41ScratchPad.getPreviousScratchPad().getCounterValues());
                }
                break;
            case "cifs:vserver":
                MetricsRowScratchPad<NetappCifsVserverMetrics.CifsVserverCounter> cifsScratchPad =
                        (MetricsRowScratchPad<NetappCifsVserverMetrics.CifsVserverCounter>) mergeWithScratchPad;
                final EnumMap<NetappCifsVserverMetrics.CifsVserverCounter, BigInteger> cifsCounters = cifsScratchPad.getCounterValues();
                mergeCifsScratchPad(resultScratchPad, cifsCounters);
                if (resultScratchPad.isPreviousPollValuesAvailable() && cifsScratchPad.isPreviousPollValuesAvailable()) {
                    mergeCifsScratchPad(resultScratchPad.getPreviousScratchPad(),
                            cifsScratchPad.getPreviousScratchPad().getCounterValues());
                }
                break;
            case "iscsi_lif:vserver":
                MetricsRowScratchPad<NetappIscsiLifVserverMetrics.IscsiLifVserverCounter> iscsiLifScratchPad =
                        (MetricsRowScratchPad<NetappIscsiLifVserverMetrics.IscsiLifVserverCounter>) mergeWithScratchPad;
                final EnumMap<NetappIscsiLifVserverMetrics.IscsiLifVserverCounter, BigInteger> iscsiLifCounters = iscsiLifScratchPad.getCounterValues();
                mergeIscsiLifScratchPad(resultScratchPad, iscsiLifCounters);
                if (resultScratchPad.isPreviousPollValuesAvailable() && iscsiLifScratchPad.isPreviousPollValuesAvailable()) {
                    mergeIscsiLifScratchPad(resultScratchPad.getPreviousScratchPad(),
                            iscsiLifScratchPad.getPreviousScratchPad().getCounterValues());
                }
                break;
            case "fcp_lif:vserver":
                MetricsRowScratchPad<NetappFcpLifVserverMetrics.FcpLifVserverCounter> fcpLifScratchPad =
                        (MetricsRowScratchPad<NetappFcpLifVserverMetrics.FcpLifVserverCounter>) mergeWithScratchPad;
                final EnumMap<NetappFcpLifVserverMetrics.FcpLifVserverCounter, BigInteger> fcpLifCounters = fcpLifScratchPad.getCounterValues();
                mergeFcpLifScratchPad(resultScratchPad, fcpLifCounters);
                if (resultScratchPad.isPreviousPollValuesAvailable() && fcpLifScratchPad.isPreviousPollValuesAvailable()) {
                    mergeFcpLifScratchPad(resultScratchPad.getPreviousScratchPad(),
                            fcpLifScratchPad.getPreviousScratchPad().getCounterValues());
                }
                break;
        }
        return resultScratchPad;
    }

    private void mergeNFSv41ScratchPad(MetricsRowScratchPad<VserverCounter> resultScratchPad, EnumMap<NetappNfsv41Metrics.Nfsv41Counter, BigInteger> nfsV41Counters) {
        //Add Ops
        resultScratchPad.addToCounter(VserverCounter.opm_ops, nfsV41Counters.get(NetappNfsv41Metrics.Nfsv41Counter.nfsv4_1_ops));
        resultScratchPad.addToCounter(VserverCounter.opm_read_ops, nfsV41Counters.get(NetappNfsv41Metrics.Nfsv41Counter.read_total));
        resultScratchPad.addToCounter(VserverCounter.opm_write_ops, nfsV41Counters.get(Nfsv41MetricsBuilder.NFSv41Counter.write_total));
        //Add Throughput
        resultScratchPad.addToCounter(VserverCounter.opm_throughput, nfsV41Counters.get(NetappNfsv41Metrics.Nfsv41Counter.nfs41_throughput));
        resultScratchPad.addToCounter(VserverCounter.opm_read_throughput, nfsV41Counters.get(NetappNfsv41Metrics.Nfsv41Counter.nfs41_read_throughput));
        resultScratchPad.addToCounter(VserverCounter.opm_write_throughput, nfsV41Counters.get(NetappNfsv41Metrics.Nfsv41Counter.nfs41_write_throughput));
        //Add Latency
        resultScratchPad.addToCounter(VserverCounter.opm_read_latency, nfsV41Counters.get(NetappNfsv41Metrics.Nfsv41Counter.read_avg_latency));
        resultScratchPad.addToCounter(VserverCounter.opm_write_latency, nfsV41Counters.get(NetappNfsv41Metrics.Nfsv41Counter.write_avg_latency));
    }

    private void mergeNFSv4ScratchPad(MetricsRowScratchPad<VserverCounter> resultScratchPad, EnumMap<NetappNfsv4Metrics.Nfsv4Counter, BigInteger> nfsV4Counters) {
        //Add Ops
        resultScratchPad.addToCounter(VserverCounter.opm_ops, nfsV4Counters.get(NetappNfsv4Metrics.Nfsv4Counter.nfsv4_ops));
        resultScratchPad.addToCounter(VserverCounter.opm_read_ops, nfsV4Counters.get(NetappNfsv4Metrics.Nfsv4Counter.read_total));
        resultScratchPad.addToCounter(VserverCounter.opm_write_ops, nfsV4Counters.get(NetappNfsv4Metrics.Nfsv4Counter.write_total));
        //Add Throughput
        resultScratchPad.addToCounter(VserverCounter.opm_throughput, nfsV4Counters.get(NetappNfsv4Metrics.Nfsv4Counter.nfs4_throughput));
        resultScratchPad.addToCounter(VserverCounter.opm_read_throughput, nfsV4Counters.get(NetappNfsv4Metrics.Nfsv4Counter.nfs4_read_throughput));
        resultScratchPad.addToCounter(VserverCounter.opm_write_throughput, nfsV4Counters.get(NetappNfsv4Metrics.Nfsv4Counter.nfs4_write_throughput));
        //Add Latency
        resultScratchPad.addToCounter(VserverCounter.opm_read_latency, nfsV4Counters.get(NetappNfsv4Metrics.Nfsv4Counter.read_avg_latency));
        resultScratchPad.addToCounter(VserverCounter.opm_write_latency, nfsV4Counters.get(NetappNfsv4Metrics.Nfsv4Counter.write_avg_latency));
    }

    private void mergeNFSv3ScratchPad(MetricsRowScratchPad<VserverCounter> resultScratchPad, EnumMap<NetappNfsv3Metrics.Nfsv3Counter, BigInteger> nfsV3Counters) {
        //Add OPS
        resultScratchPad.addToCounter(VserverCounter.opm_ops, nfsV3Counters.get(NetappNfsv3Metrics.Nfsv3Counter.nfsv3_ops));
        resultScratchPad.addToCounter(VserverCounter.opm_read_ops, nfsV3Counters.get(NetappNfsv3Metrics.Nfsv3Counter.nfsv3_read_ops));
        resultScratchPad.addToCounter(VserverCounter.opm_write_ops, nfsV3Counters.get(NetappNfsv3Metrics.Nfsv3Counter.nfsv3_write_ops));
        //Add Throughput
        resultScratchPad.addToCounter(VserverCounter.opm_throughput, nfsV3Counters.get(NetappNfsv3Metrics.Nfsv3Counter.nfsv3_throughput));
        resultScratchPad.addToCounter(VserverCounter.opm_read_throughput, nfsV3Counters.get(NetappNfsv3Metrics.Nfsv3Counter.nfsv3_read_throughput));
        resultScratchPad.addToCounter(VserverCounter.opm_write_throughput, nfsV3Counters.get(NetappNfsv3Metrics.Nfsv3Counter.nfsv3_write_throughput));
        //Add Latency
        resultScratchPad.addToCounter(VserverCounter.opm_read_latency, nfsV3Counters.get(NetappNfsv3Metrics.Nfsv3Counter.read_avg_latency));
        resultScratchPad.addToCounter(VserverCounter.opm_write_latency, nfsV3Counters.get(NetappNfsv3Metrics.Nfsv3Counter.write_avg_latency));
    }

    private void mergeCifsScratchPad(MetricsRowScratchPad<VserverCounter> resultScratchPad, EnumMap<NetappCifsVserverMetrics.CifsVserverCounter, BigInteger> cifsCounters) {
        //Add OPS
        resultScratchPad.addToCounter(VserverCounter.opm_ops, cifsCounters.get(NetappCifsVserverMetrics.CifsVserverCounter.cifs_ops));
        resultScratchPad.addToCounter(VserverCounter.opm_read_ops, cifsCounters.get(NetappCifsVserverMetrics.CifsVserverCounter.cifs_read_ops));
        resultScratchPad.addToCounter(VserverCounter.opm_write_ops, cifsCounters.get(NetappCifsVserverMetrics.CifsVserverCounter.cifs_write_ops));
        //Add Latency
        resultScratchPad.addToCounter(VserverCounter.opm_read_latency, cifsCounters.get(NetappCifsVserverMetrics.CifsVserverCounter.cifs_read_latency));
        resultScratchPad.addToCounter(VserverCounter.opm_write_latency, cifsCounters.get(NetappCifsVserverMetrics.CifsVserverCounter.cifs_write_latency));
        resultScratchPad.addToCounter(VserverCounter.opm_latency, cifsCounters.get(NetappCifsVserverMetrics.CifsVserverCounter.cifs_latency));
    }

    private void mergeIscsiLifScratchPad(MetricsRowScratchPad<VserverCounter> resultScratchPad, EnumMap<NetappIscsiLifVserverMetrics.IscsiLifVserverCounter, BigInteger> iscsiLifCounters) {
        //Add OPS
        resultScratchPad.addToCounter(VserverCounter.opm_ops, MetricsRowScratchPad.add(
                iscsiLifCounters.get(NetappIscsiLifVserverMetrics.IscsiLifVserverCounter.iscsi_other_ops), iscsiLifCounters.get(NetappIscsiLifVserverMetrics.IscsiLifVserverCounter.iscsi_read_ops),
                iscsiLifCounters.get(NetappIscsiLifVserverMetrics.IscsiLifVserverCounter.iscsi_write_ops)));
        resultScratchPad.addToCounter(VserverCounter.opm_read_ops, iscsiLifCounters.get(NetappIscsiLifVserverMetrics.IscsiLifVserverCounter.iscsi_read_ops));
        resultScratchPad.addToCounter(VserverCounter.opm_write_ops, iscsiLifCounters.get(NetappIscsiLifVserverMetrics.IscsiLifVserverCounter.iscsi_write_ops));
        //Add Throughput
        resultScratchPad.addToCounter(VserverCounter.opm_throughput, MetricsRowScratchPad.add(
                iscsiLifCounters.get(NetappIscsiLifVserverMetrics.IscsiLifVserverCounter.read_data), iscsiLifCounters.get(NetappIscsiLifVserverMetrics.IscsiLifVserverCounter.write_data)));
        resultScratchPad.addToCounter(VserverCounter.opm_read_throughput, iscsiLifCounters.get(NetappIscsiLifVserverMetrics.IscsiLifVserverCounter.read_data));
        resultScratchPad.addToCounter(VserverCounter.opm_write_throughput, iscsiLifCounters.get(NetappIscsiLifVserverMetrics.IscsiLifVserverCounter.write_data));
        //Add Latency
        resultScratchPad.addToCounter(VserverCounter.opm_read_latency, iscsiLifCounters.get(NetappIscsiLifVserverMetrics.IscsiLifVserverCounter.avg_read_latency));
        resultScratchPad.addToCounter(VserverCounter.opm_write_latency, iscsiLifCounters.get(NetappIscsiLifVserverMetrics.IscsiLifVserverCounter.avg_write_latency));
        resultScratchPad.addToCounter(VserverCounter.opm_latency, iscsiLifCounters.get(NetappIscsiLifVserverMetrics.IscsiLifVserverCounter.avg_latency));
    }

    private void mergeFcpLifScratchPad(MetricsRowScratchPad<VserverCounter> resultScratchPad, EnumMap<NetappFcpLifVserverMetrics.FcpLifVserverCounter, BigInteger> fcpLifCounters) {
        //Add OPS
        resultScratchPad.addToCounter(VserverCounter.opm_ops, fcpLifCounters.get(NetappFcpLifVserverMetrics.FcpLifVserverCounter.total_ops));
        resultScratchPad.addToCounter(VserverCounter.opm_read_ops, fcpLifCounters.get(NetappFcpLifVserverMetrics.FcpLifVserverCounter.read_ops));
        resultScratchPad.addToCounter(VserverCounter.opm_write_ops, fcpLifCounters.get(NetappFcpLifVserverMetrics.FcpLifVserverCounter.write_ops));
        //Add Throughput
        resultScratchPad.addToCounter(VserverCounter.opm_throughput, MetricsRowScratchPad.add(fcpLifCounters.get(NetappFcpLifVserverMetrics.FcpLifVserverCounter.read_data),
                fcpLifCounters.get(FcpLifMetricsBuilder.FcpLIFCounter.write_data)));
        resultScratchPad.addToCounter(VserverCounter.opm_read_throughput, fcpLifCounters.get(NetappFcpLifVserverMetrics.FcpLifVserverCounter.read_data));
        resultScratchPad.addToCounter(VserverCounter.opm_write_throughput, fcpLifCounters.get(NetappFcpLifVserverMetrics.FcpLifVserverCounter.write_data));
        //Add Latency
        resultScratchPad.addToCounter(VserverCounter.opm_read_latency, fcpLifCounters.get(NetappFcpLifVserverMetrics.FcpLifVserverCounter.avg_read_latency));
        resultScratchPad.addToCounter(VserverCounter.opm_write_latency, fcpLifCounters.get(NetappFcpLifVserverMetrics.FcpLifVserverCounter.avg_write_latency));
        resultScratchPad.addToCounter(VserverCounter.opm_latency, fcpLifCounters.get(NetappFcpLifVserverMetrics.FcpLifVserverCounter.avg_latency));
    }
}