package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transform.ocie;

import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeClusterPeer;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.ClusterPeer;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.NetworkIpSpace;
import com.onaro.util.VersionComparator;
import org.springframework.beans.BeanUtils;

import java.util.Map;

public class ClusterPeerTransform {

    public static void transform(final CModeCluster cModeCluster, final Cluster cluster,
                                 final Map<String, NetworkIpSpace.NetworkIpSpaceKey> networkIpSpaceKeyMap) {
        for (CModeClusterPeer cModeClusterPeer : cModeCluster.getClusterPeers().values()) {
            final ClusterPeer clusterPeer = cluster.createClusterPeer(cModeClusterPeer.getRemoteClusterUUID());
            //Default matching properties
            BeanUtils.copyProperties(cModeClusterPeer, clusterPeer,
                    new String[]{"ipSpaceKey", "addressFamily", "addressFamilyRaw", "authStatusOperational",
                            "authStatusOperationalRaw", "availability", "availabilityRaw"});

            NetworkIpSpace.NetworkIpSpaceKey networkIpSpaceKey = networkIpSpaceKeyMap.get(cModeClusterPeer.getIpSpaceName());
            if (networkIpSpaceKey == null) {
                final VersionComparator versionComparator = new VersionComparator();
                final String clusterVersion = cModeCluster.getVersionString();
                //OnTap X Builds have cluster version 8.3 and the release builds have 8.3.0, so check for both
                final boolean isFullSteam = versionComparator.compare(clusterVersion, Cluster.FULL_STEAM) == 0  ||
                        versionComparator.compare(clusterVersion, Cluster.FULL_STEAM_0) == 0;
                //For Full Steam set it to default if an IP Space is not set.
                if (isFullSteam) {
                    networkIpSpaceKey = networkIpSpaceKeyMap.get("Default");
                }
            }
            clusterPeer.setIpSpaceKey(networkIpSpaceKey);
            clusterPeer.setAddressFamily(cModeClusterPeer.getAddressFamily().getEnumValue());
            clusterPeer.setAddressFamilyRaw(cModeClusterPeer.getAddressFamily().getRawValue());
            clusterPeer.setAuthStatusOperational(cModeClusterPeer.getAuthStatusOperational().getEnumValue());
            clusterPeer.setAuthStatusOperationalRaw(cModeClusterPeer.getAuthStatusOperational().getRawValue());
            clusterPeer.setAvailability(cModeClusterPeer.getAvailability().getEnumValue());
            clusterPeer.setAvailabilityRaw(cModeClusterPeer.getAvailability().getRawValue());
            cluster.getClusterPeers().add(clusterPeer);
        }
    }
}
