package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transform.ocie;

import java.util.Map;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeCluster;
import com.onaro.sanscreen.acquisition.datasource.netapp_ocie.model.cmode.CModeVServerPeer;
import com.onaro.sanscreen.acquisition.framework.datasource.PackageContext;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.VServer;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.VServerPeer;
import com.onaro.util.VersionComparator;


public class VServerPeerTransformer {
    private static final Logger logger = LoggerFactory.getLogger(VServerPeerTransformer.class);
            
    private VServerPeerTransformer() {
    }

    public static void transform(final CModeCluster cModeCluster, final Cluster cluster,
            final Map<String, VServer.VServerKey> vServerKeysByName) {
        for (CModeVServerPeer cModeVServerPeer : cModeCluster.getVServerPeers().values()) {
            String clusterVersion = cModeCluster.getVersionString();
            VServerPeer vServerPeer = (VersionComparator.compareVersions(clusterVersion.toString(), Cluster.FULL_STEAM) >= 0) ? cluster
                    .createVServerPeer(cModeVServerPeer.getPeerVServerUUID(), cModeVServerPeer.getVServerUUID())
                    : cluster.createVServerPeer(cModeVServerPeer.getPeerClusterName(),
                            cModeVServerPeer.getPeerVServerName(), cModeVServerPeer.getVServerName());

            BeanUtils.copyProperties(cModeVServerPeer, vServerPeer, new String[] { "peerState", "peerStateRaw" });
            vServerPeer.setPeerState(cModeVServerPeer.getPeerState().getEnumValue());
            vServerPeer.setPeerStateRaw(cModeVServerPeer.getPeerState().getRawValue());

            if (vServerPeer.getVServerUUID() == null) {
                final String vServerName = cModeVServerPeer.getVServerName();
                final VServer.VServerKey vServerKey = vServerKeysByName.get(vServerName);
                if (vServerKey == null) {
                    logger.info(PackageContext.getLogPrefix() + " - Failed to find vServer " + vServerName
                            + " for VServerPeer.");
                    continue;
                }
                vServerPeer.setVServerUUID(vServerKey.getZapiUUID());
            }
            
            if (vServerPeer.getPeerVServerUUID() == null) {
                final String peerVServerName = cModeVServerPeer.getPeerVServerName();
                final VServer.VServerKey peerVServerKey = vServerKeysByName.get(peerVServerName);
                if (peerVServerKey != null) {
                    vServerPeer.setPeerVServerUUID(peerVServerKey.getZapiUUID());
                }
            }

            cluster.getVServerPeers().add(vServerPeer);
        }
    }

}
