package com.onaro.sanscreen.acquisition.datasource.netapp_ocie.transport.zapi;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 *
 * This class parses disable_zapis.properties file and build a set of zapis that need to be disabled
 *
 */
public class ZAPIPropertiesManager {

	private static final Logger logger = LoggerFactory.getLogger(ZAPIPropertiesManager.class);
	private static Set<String> disableZapisSet;
	private static String PROPERTIES_FILE_NAME = "preferences/disable-zapis-list.properties";

	/**
	 * Parses given file and returns a set of strings. Each entry of the set is a line in the given file.
	 * @param file File File object which needs to be parsed
	 */
	static void parseAndBuildSet(InputStream stream){
		disableZapisSet = new HashSet<String>();

		try(BufferedReader reader = new BufferedReader(new InputStreamReader(stream))){
			String line = null;

			while ((line = reader.readLine()) != null) {
				disableZapisSet.add(line.trim());
			}
		}

		catch(IOException ex){
			logger.error(" Failed to parse disable_zapis.properties file: " + ex, ex);
		}
	}

	/**
	 * Return an unmodifiable set of black listed zapis. This set contains zapi command name.
	 * @return Set<String>
	 */
	public static Set<String> getDisableZapisSet(){

		if (disableZapisSet == null){

	        final InputStream propertiesStream = ZAPIPropertiesManager.class.getClassLoader().getResourceAsStream(PROPERTIES_FILE_NAME);

        	if (propertiesStream != null){
        		parseAndBuildSet(propertiesStream);
        	}
		}

		if (disableZapisSet == null ){
			return Collections.emptySet();
		}
		else{
			return Collections.unmodifiableSet(disableZapisSet);
		}
	}

}
