package com.onaro.sanscreen.acquisition.diskmodelutils;

import java.io.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * A simple class that takes several disk model files, aggregates them, and prompts the user
 * with options to rectify the collisions.
 */
public class DiskModelConflictResolver {
    final static Map<String, List<DiskModelAttributes>> maps = new HashMap<String, List<DiskModelAttributes>>();
    public static void main(String[] args) throws Exception {
        if (args.length == 0) {
            System.err.println("Usage: application <diskmodel #1> [<diskmodel #2>] [<diskmodel #3>] ...");
            System.exit(0);
        }

        for (final String arg : args) {
            loadFile(new File(arg));
        }

        //Get the number of conflicts
        int conflicts = 0;
        for (Map.Entry<String, List<DiskModelAttributes>> kv : maps.entrySet()) {
            if (kv.getValue().size() > 1) {
                conflicts++;
            }
        }

        int conflictCount = 0;
        BufferedReader in = new BufferedReader(new InputStreamReader(System.in));

        for (Map.Entry<String, List<DiskModelAttributes>> kv : maps.entrySet()) {
            if (kv.getValue().size() > 1) {
                System.out.println("\n################ CONFLICT (" + ++conflictCount + "/" + conflicts + ") ################");

                int i = 0;
                for (final DiskModelAttributes s : kv.getValue()) {
                    System.out.println(i + ") " + s);
                    i++;
                }

                //If theres an error we want to prompt again
                while (true) {
                    try {
                        System.out.print("Resolution [#] = Select or [Disk Model] for manual: ");
                        final String response = in.readLine();
                        Integer iRep = null;
                        try {
                            iRep = Integer.parseInt(response);
                        }
                        catch (Exception e) {
                        }

                        if (iRep == null) {
                            final DiskModelAttributes newDiskModel = DiskModelAttributes.Parse(response);
                            kv.getValue().clear();
                            kv.getValue().add(newDiskModel);
                        }
                        else {
                            final DiskModelAttributes val = kv.getValue().get(iRep);
                            kv.getValue().clear();
                            kv.getValue().add(val);
                        }

                        //Leave the while loop if everything was successful!
                        break;
                    }
                    catch (final Exception e) {
                        System.err.println("ERROR: " + e.getMessage());
                    }
                }
            }
        }

        final File fNew = new File(args[0] + ".resolved");
        final OutputStream outputStream = new FileOutputStream(fNew);
        final BufferedWriter modelDescriptionWriter = new BufferedWriter(new OutputStreamWriter(outputStream, "utf-8"));
        for (Map.Entry<String, List<DiskModelAttributes>> kv : maps.entrySet()) {
            modelDescriptionWriter.write(kv.getValue().get(0).to60CSV());
        }

        modelDescriptionWriter.close();
        outputStream.close();
        System.out.println("Output saved to: " + fNew.getPath());
    }

    private static void loadFile(final File f) throws IOException {
        final InputStream inputStream = new FileInputStream(f);
        List<DiskModelAttributes> models = DiskModelFileExtractor.parseDiskModelDescriptionStream(inputStream);
        for (final DiskModelAttributes diskModelAttribute : models) {
            if (maps.containsKey(diskModelAttribute.key)) {
                maps.get(diskModelAttribute.key).add(diskModelAttribute);
            }
            else {
                final List<DiskModelAttributes> l = new ArrayList<DiskModelAttributes>();
                l.add(diskModelAttribute);
                maps.put(diskModelAttribute.key, l);
            }
        }
    }
}
