package com.onaro.sanscreen.acquisition.diskmodelutils;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

/**
 * Extracts <class>DiskModelAttributes</class> from a disk model file
 */
public class DiskModelFileExtractor {
    public static List<DiskModelAttributes> parseDiskModelDescriptionStream(final InputStream inputStream) throws IOException, IllegalArgumentException {
        final List<DiskModelAttributes> diskModels = new ArrayList<DiskModelAttributes>();

        int lineNumber = 0;
        BufferedReader modelDescriptionReader = null;
        boolean afterBanner = false;
        boolean inVendorHeader = false;
        try {
            modelDescriptionReader = new BufferedReader(new InputStreamReader(inputStream, "utf-8"));

            ArrayList<String> comments = new ArrayList<String>(5);
            while (true) {
                final String modelDescriptionLine = modelDescriptionReader.readLine();
                if (null == modelDescriptionLine) {
                    break;
                }
                ++lineNumber;

                if (modelDescriptionLine.trim().equals("")) {
                    afterBanner = true;
                    inVendorHeader = false;
                    continue;
                }

                if (modelDescriptionLine.startsWith("##")) {
                    inVendorHeader = true;
                    continue;
                }

                if (modelDescriptionLine.startsWith("#")) {
                    if ((afterBanner) && (!inVendorHeader)) {
                        comments.add(modelDescriptionLine);
                    }
                    continue;
                }

                try {
                    DiskModelAttributes modelAttributes = DiskModelAttributes.Parse(modelDescriptionLine);
                    modelAttributes.comments.addAll(comments);
                    comments = new ArrayList<String>(5);
                    diskModels.add(modelAttributes);
                }
                catch (final IllegalArgumentException ex) {
                    System.err.println("ERROR:  line " + lineNumber + ", " + modelDescriptionLine + ": " + ex.getMessage());
                }
            }
        }
        finally {
            if (null != modelDescriptionReader) {
                try {
                    modelDescriptionReader.close();
                }
                catch (IOException ignore) {
                }
            }
        }
        return diskModels;
    }
}
