package com.onaro.sanscreen.acquisition.foundation.standalone;

import com.onaro.sanscreen.acquisition.foundation.datasource.FoundationUpdatesListener;
import com.onaro.sanscreen.acquisition.foundation.datasource.OriginatorProvider;
import com.onaro.sanscreen.acquisition.framework.FrameworkException;
import com.onaro.sanscreen.acquisition.framework.datasource.*;
import com.onaro.sanscreen.acquisition.schema.odata2.OData2Root;
import org.apache.log4j.Logger;

import java.io.File;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

/**
 * Main listener for foundation events.
 */
public class FoundationStandaloneListener implements FoundationUpdatesListener, DataSourceSampleListener, OriginatorProvider, DataSourceLifeCycleListener {
    private final static Logger logger = Logger.getLogger(FoundationStandaloneListener.class);

    protected Long nextOriginatorID = (long) 0;
    protected Map<String, Long> allocatedOriginators = new HashMap<String, Long>(20);

    public FoundationStandaloneListener() {
    }

    public void dataSourceStarting(final BaseDataSource dataSource) throws FrameworkException {
        logger.info("Data Source starting");
    }

    public void dataSourceStopping(final BaseDataSource dataSource) throws FrameworkException {
        logger.info("Data Source stopping");
    }

    public void dataSourceStatusChanged(final BaseDataSource dataSource, final DataSourceStatus oldStatus, final DataSourceStatus newStatus, final String message) {
        logger.info("Data Source status changed from " + oldStatus + " to " + newStatus);
    }

    public void startingSample(final BaseDataSource dataSource, final PollingPackage pollType) {
        logger.info("Starting sample");
    }

    public void sampleSuccessfull(final BaseDataSource baseDataSource, final PollingPackage aPackage) throws FrameworkException {
        logger.info("Sample successfull");
    }

    public void finishedSample(final BaseDataSource dataSource, final PollingPackage aPackage) throws FrameworkException {
        logger.info("Finished sample");
    }

    public void sampleFailed(final BaseDataSource dataSource, final PollingPackage aPackage, final DataSourceErrorException e) {
        logger.error("Sample failed: " + e.getMessage(), e);
    }

    public void sampleFailed(final BaseDataSource dataSource, final PollingPackage aPackage, final Throwable e) {
        logger.error("Sample failed: " + e.getMessage(), e);
    }

    public long getOriginatorID(final Long datasourceID, final String originatorKey, final String originatorDisplayName) throws FrameworkException {
        // In Unit-Test we have to send the same originatorID for the old-oData as well as the new-oData.
        // otherwise we'll get a missmatch between the samples.
        Long originatorID = allocatedOriginators.get(originatorKey);
        if (originatorID == null) {
            originatorID = nextOriginatorID++;
            allocatedOriginators.put(originatorKey, originatorID);
        }

        return originatorID;
    }

    @Override
    public long getOriginatorID(final long datasourceID, final String deviceKey, final String originatorName, final boolean performDuplicateDatasourceCheck) throws FrameworkException {
        return getOriginatorID(datasourceID, deviceKey, originatorName);
    }

    public void deleteOriginatorID(final Long originatorID, final String originatorKey) throws FrameworkException {
        allocatedOriginators.remove(originatorKey);
    }

    public String[] getOriginatorCache() {
        // Do nothing...
        return new String[0];
    }

    public void cleanOriginatorCache() {
        // Do nothing...
    }

    /**
     * Called by the datasource the first time it polls to set the Management URL for the datasource itself.
     *
     * @param manageURL url
     */
    public void reportManageURL(final BaseDataSource dataSource, final String manageURL) {
        // Do nothing
    }

    public void reportOData2(final BaseDataSource dataSource,
                            final String originaorKey,
                            final String origintorDisplayName,
                            final OData2Root odataRoot,
                            final String odataXML,
                            final boolean changed,
                            final Serializable context) throws FrameworkException {
        logger.info("Sample successfull!");
    }

    public void recordingSessionDone(final BaseDataSource dataSource,
                                       final PollingPackage aPackage,
                                       final File recordingSession,
                                       final boolean shouldReport) throws FrameworkException {
        logger.info("Reported Recording Session!");
    }
}
