package com.onaro.sanscreen.acquisition.framework.datasource;

import com.onaro.sanscreen.acquisition.framework.FrameworkException;

/**
 * Listener for data source life management events. This interface defines events fired when the data source as a whole
 * unit changes status.
 */
public interface DataSourceLifeCycleListener {
    /**
     * Reported by the data source when starting
     *
     * @param dataSource
     *
     * @throws FrameworkException
     */
    public void dataSourceStarting(final BaseDataSource dataSource) throws FrameworkException;

    /**
     * Reported by the data source when stopping
     *
     * @param dataSource
     *
     * @throws FrameworkException
     */
    public void dataSourceStopping(final BaseDataSource dataSource) throws FrameworkException;

    /**
     * Update status of a data source
     *
     * @param dataSource
     * @param oldStatus
     * @param newStatus
     * @param message
     */
    public void dataSourceStatusChanged(final BaseDataSource dataSource, final DataSourceStatus oldStatus, final DataSourceStatus newStatus, final String message);
}
