package com.onaro.sanscreen.acquisition.framework.datasource;

import com.onaro.sanscreen.acquisition.framework.FrameworkException;

import java.io.File;

/**
 * Listener for sample events from data sources
 */
public interface DataSourceSampleListener {
    /**
     * Reported by the data source every time it starts an acquisition cycle.
     *
     * @param dataSource
     * @param pollType
     */
    public void startingSample(final BaseDataSource dataSource, final PollingPackage pollType);

    /**
     * Called by the data source when sample ended with no exception
     *
     * @param dataSource
     * @param aPackage
     */
    public void sampleSuccessfull(final BaseDataSource dataSource, final PollingPackage aPackage) throws FrameworkException;

    /**
     * Called by the data source when a data source error is caught during sample.
     *
     * <p/>The framework only catches these errors and
     * writes them to the log. It makes sure the system will not be affected by such errors. It does not however report these
     * errors anywhere. This is the responsibility of package specific listeners.
     *
     * @param dataSource
     * @param aPackage
     * @param e
     */
    public void sampleFailed(final BaseDataSource dataSource, final PollingPackage aPackage, final DataSourceErrorException e);

    /**
     * Called by the data source when a runtime / error / framework exception is caught during sample
     *
     * @param dataSource
     * @param aPackage
     * @param e
     */
    public void sampleFailed(final BaseDataSource dataSource, final PollingPackage aPackage, final Throwable e);

    /**
     * Called by the data source when it finished a cycle of sampling. This method is used to close audit records, etc...
     *
     * @param dataSource
     * @param aPackage
     */
    public void finishedSample(final BaseDataSource dataSource, final PollingPackage aPackage) throws FrameworkException;

    /**
     * Reported by package when a recording session is completed.
     *
     * @param dataSource Base data source handle.
     * @param aPackage current package being recorded.
     * @param recordingSession Recording file just created.
     * @param shouldReport a boolean stipulating if this recording should be "reported".   This is an indicator to
     *  implementers as to any action that should be taken to report this data to external destinations (e.g. the server).
     */
    public void recordingSessionDone(final BaseDataSource dataSource,
                                       final PollingPackage aPackage,
                                       final File recordingSession,
                                       final boolean shouldReport) throws FrameworkException;
}
