package com.onaro.sanscreen.acquisition.framework.datasource;

import com.onaro.sanscreen.acquisition.framework.FrameworkException;
import com.onaro.sanscreen.acquisition.framework.recordings.RecordingSession;
import com.onaro.sanscreen.acquisition.framework.datasource.config.PackageConfiguration;

/**
 * Base interface for all poll types
 */
public interface Package<C extends PackageConfiguration> extends PackageMBean {
    /**
     * @return A string representation of the poll type (e.g. "Foundation", "Performance", "RPO", etc...)
     */
    public String getPackageName();

    /**
     * @return A string identifying this package in the log
     */
    public String getLogPrefix();

    /**
     * The package object is being used as a key in HashMaps
     */
    public boolean equals(Object other);

    /**
     * The package object is being used as a key in HashMaps
     */
    public int hashCode();

    /**
     * @return The configuration object associated with this package instance
     */
    public C getConfiguration();

    /**
     * The recording session for this package
     */
    public RecordingSession getRecordingSession();

    public BaseDataSource getDataSource();

    /**
     * Start the package. Will be called when a data source starts
     *
     * @throws DataSourceErrorException When a package specific problem occured
     * @throws FrameworkException When there was a framework related issue
     */
    public void start() throws DataSourceErrorException, FrameworkException;

    /**
     * Stops the package. Will be called when a data source stops.
     *
     * @throws DataSourceErrorException When a package specific problem occured
     * @throws FrameworkException When there was a framework related issue
     */
    public void stop() throws DataSourceErrorException, FrameworkException;
}
