package com.onaro.sanscreen.acquisition.framework.datasource.io;

import java.io.*;

public class DSFileWriter extends BufferedWriter implements DSFile {
    private final String fileName;

    public DSFileWriter(final String fileName, final File file) throws IOException {
        // Specify large buffer size here: 500K  - 2 orders magnitude higher than default
        // (defaultCharBufferSize = 8192) * 8 * 8 == 524288
        super(new OutputStreamWriter(new FileOutputStream(file), "utf-8"), 524288);
        this.fileName = fileName;
    }

    @Override
    public String getFileName() {
        return fileName;
    }

    /**
     * Write the specified string followed by a line feed in a single atomic operation.
     *
     * @param str The string to be written.
     */
    public void writeln(final String str) throws IOException {
        synchronized (lock) {
            super.write(str);
            super.write("\n");
        }
    }

    /**
     * Writes a variable number of arguments separated by {@code delimiter}.
     *
     * @param delimiter
     * @param values
     * @throws IOException
     */
    public void writeln(String delimiter, Object... values) throws IOException {
        if (values.length > 0) {
            synchronized (lock) {
                // after the 0th element, everything can be delimiter-prefixed;
                // doing it this way avoids a within-loop IF conditional
                super.write(String.valueOf(values[0]));
                for (int i = 1; i < values.length; i++) {
                    super.write(delimiter);
                    super.write(String.valueOf(values[i]));
                }
                super.write("\n");
            }
        }
    }

    @Override
    public String toString() {
        return fileName;
    }

}
