package com.onaro.sanscreen.acquisition.framework.filecache;

import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;
import java.util.List;

/**
 *
 */
public class DirectoryCacheKey extends CacheKey
{
  private String fileName;
  private List<String> directories = new ArrayList<>();
  private FileFilter fileFilter;

  public FileFilter getFileFilter()
  {
    return fileFilter;
  }

  public void setFileFilter(FileFilter fileFilter)
  {
    this.fileFilter = fileFilter;
  }

  public DirectoryCacheKey(String baseDirectory)
  {
    directories.add( baseDirectory );
  }

  public void addDir(String directoryName)
  {
    directories.add( directoryName );
  }

  public String getFileName()
  {
    return fileName;
  }

  public void setFileName(String fileName)
  {
    this.fileName = fileName;
  }

  /**
   *
   */
  @Override
  public void validate()
  {
    if( fileName != null && fileFilter != null )
      throw new RuntimeException( "Both a filter and the file name can't be set" );
  }

  /**
   *
   * @return
   */
  @Override
  public String toString()
  {
    //validation
    if( directories == null ) throw new RuntimeException( "Null directories." );
    if( directories.isEmpty() ) throw new RuntimeException( "No directories." );

    //create full path
    StringBuilder fullPath = new StringBuilder();
    for( String dir : directories )
      fullPath.append( dir ).append( File.separator );

    //some keys have files and some are only directories
    if( fileName != null )
      fullPath.append( fileName );
    else
      //remove the extra
      fullPath.delete( fullPath.lastIndexOf( File.separator ), fullPath.length() );
    return fullPath.toString();
  }
}
