package com.onaro.sanscreen.acquisition.framework.mgmt;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

public class DeployParser {
    /**
     * Parses deployfile.dat file to generate a list of deployee XML files, one for each data source.
     *
     *
     * @param deployFileReader Reader for the deployfile.day file
     * @return List of deployee files parsed
     *
     * @throws java.io.IOException On any error reading/parsing the file
     */
    public List<String> parseDeployFile(final BufferedReader deployFileReader) throws IOException {
        final List<String> datasourceDeployFiles = new LinkedList<String>();

        while (true) {
            String line = deployFileReader.readLine();
            if (line == null)
                break;

            line = line.trim();

            if (line.length() == 0) {
                // Skip empty lines
                continue;
            }

            // Check if line contains '#' which means remark (can be either at the beginning or in the middle)
            int index = line.indexOf("#");
            if (index != -1) {
                // Remove a remark...
                line = line.substring(0, index).trim();
                if (line.length() == 0) {
                    // If the whole line was commented out, skip it
                    continue;
                }
            }

            // Check for the CatJ mark...
            if (line.endsWith("@")) {
                line = line.substring(0, line.length() - 1);
            }

            datasourceDeployFiles.add(line);
        }

        return datasourceDeployFiles;
    }
}
