package com.onaro.sanscreen.acquisition.netappfoundation.datasource;

import com.onaro.sanscreen.acquisition.framework.FrameworkException;

/**
 * Defined the interface for allocating originator IDs for data sources
 */
public interface NetAppOriginatorProvider {
    /**
     * Called by the data source to determine which originatorID it should report on based on a datasourceName and a deviceKey
     *
     * @param originatorKey
     * @param originatorDisplayName
     */
    public long getOriginatorID(final Long datasourceID, final String originatorKey, final String originatorDisplayName) throws FrameworkException;

    /**
     * Removes the specified originator ID. This will also close all the originator lines in the data base reported by
     * this originator.
     */
    public void deleteOriginatorID(final Long originatorID, final String deviceKey) throws FrameworkException;

    /**
     * For monitoring purposes
     *
     * @return A string representation of the cache contents
     */
    public String[] getOriginatorCache();

    /**
     * For monitoring purposes. Cleans the originator cache.
     */
    public void cleanOriginatorCache();
}
