package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("aggregate")
@OCIEResourceType("aggregate")
public class Aggregate extends BaseCRCXMLNode<Node> {

    public final static class AggregateKey extends KeyBase<AggregateKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "aggr-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggregate-uuid"
            )
            uuid
        }

        public AggregateKey(final String clusterUUID, final String aggregateUUID) {
            super("aggregate",
                  new EnumMap<Field, String>(Field.class) {{
                        put(Field.cluster_uuid, clusterUUID);
                        put(Field.uuid, aggregateUUID);
                  }});
        }

        @Override
        @OCIEUUID(uuidDescr = "uuid")
        public String getOcieUUID() {
            return getID(Field.uuid);
        }
    }

    private final AggregateKey key;

    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggregate-name"
    )
    @OCIEValue(
        ocieColumn = "name"
    )
    private String name = null;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-fs-attributes.type"
    )
    @OCIEValue(
        ocieColumn = "type"
    )
    private String type = null;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-raid-attributes.is-hybrid"
    )
    @OCIEValue(
        ocieColumn = "isHybrid"
    )
    private Boolean isHybrid = null;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-raid-attributes.is-hybrid-enabled"
    )
    @OCIEValue(
        ocieColumn = "isHybridEnabled"
    )
    private Boolean isHybridEnabled = null;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-raid-attributes.uses-shared-disks"
    )
    @OCIEValue(
        ocieColumn = "usesSharedDisks"
    )
    private Boolean usesSharedDisks = null;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-raid-attributes.state"
    )
    @OCIEValue(
        ocieColumn = "state",
        notes = "derived from stateRaw enum('NOT_MAPPED','CREATING','DESTROYING','FAILED','FROZEN','INCONSISTENT','IRON_RESTRICTED','MOUNTING','ONLINE','OFFLINE','PARTIAL','QUIESCED','QUIESCING','RESTRICTED','REVERTED','UNKNOWN','UNMOUNTED','UNMOUNTING')"
    )
    private AggregateState state = null;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-raid-attributes.state"
    )
    @OCIEValue(
        ocieColumn = "stateRaw"
    )
    private String stateRaw = null;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-snapshot-attributes.size-total"
    )
    @OCIEValue(
        ocieColumn = "snapshotSizeTotal"
    )
    private Long snapshotSizeTotal = null;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-snapshot-attributes.size-used"
    )
    @OCIEValue(
        ocieColumn = "snapshotSizeUsed"
    )
    private Long snapshotSizeUsed = null;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-space-attributes.hybrid-cache-size-total"
    )
    @OCIEValue(
        ocieColumn = "hybridCacheSizeTotal"
    )
    private Long hybridCacheSizeTotal = null;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-space-attributes.size-total"
    )
    @OCIEValue(
        ocieColumn = "sizeTotal"
    )
    private Long sizeTotal = null;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-space-attributes.size-used"
    )
    @OCIEValue(
        ocieColumn = "sizeUsed"
    )
    private Long sizeUsed = null;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-space-attributes.percent-used-capacity"
    )
    @OCIEValue(
        ocieColumn = "sizeUsedPercent"
    )
    private Integer sizeUsedPercent = null;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-space-attributes.percent-used-capacity"
    )
    @OCIEValue(
        ocieColumn = "sizeAvailPercent",
        notes = "derived from 100 - percent-used-capacity"
    )
    private Integer sizeAvailPercent = null;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-space-attributes.total-reserved-space"
    )
    @OCIEValue(
        ocieColumn = "totalReservedSpace"
    )
    private Long totalReservedSpace = null;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-raid-attributes.raid-status"
    )
    @OCIEValue(
        ocieColumn = "raidStatus"
    )
    private String raidStatus = null;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-raid-attributes.raid-type"
    )
    @OCIEValue(
        ocieColumn = "raidTypeRaw",
        notes = "derived from raid-type like: raid_dp, raid0, raid4"
    )
    private String raidTypeRaw = null;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-raid-attributes.raid-type"
    )
    @OCIEValue(
        ocieColumn = "raidStatus",
        notes = "derived from raidTypeRaw enum('NOT_MAPPED','RAID_DP','RAID_4','RAID_0','MIXED')"
    )
    private RaidType raidType = null;


    @OCIEZapiSource(
            sourceZapi = "aggr-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-raid-attributes.aggregate-type"
    )
    @OCIEValue(
            ocieColumn = "aggregateTypeRaw",
            notes = "derived from aggregate-type like: hdd, ssd, hybrid, lun"
    )
    private String aggregateTypeRaw = null;

    @OCIEZapiSource(
            sourceZapi = "aggr-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-raid-attributes.aggregate-type"
    )
    @OCIEValue(
            ocieColumn = "aggregateType"
    )
    private AggregateType aggregateType = null;

    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-fs-attributes.block-type"
    )
    @OCIEValue(
        ocieColumn = "blockType",
        notes = "derived from blockTypeRaw enum('32_BIT','64_BIT','NOT_MAPPED')"
    )
    private BlockType blockType = null;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-fs-attributes.block-type"
    )
    @OCIEValue(
        ocieColumn = "blockTypeRaw"
    )
    private String blockTypeRaw = null;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-raid-attributes.raid-size"
    )
    @OCIEValue(
        ocieColumn = "raidSize"
    )
    private Long raidSize = null;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-space-attributes.size-available"
    )
    @OCIEValue(
        ocieColumn = "sizeAvail"
    )
    private Long sizeAvailable = null;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-snapshot-attributes.size-available"
    )
    @OCIEValue(
        ocieColumn = "snapshotSizeAvailable"
    )
    private Long snapshotSizeAvailable = null;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-64bit-upgrade-attributes.aggr-status-attributes.is-64-bit-upgrade-in-progress"
    )
    @OCIEValue(
        ocieColumn = "is64BitUpgradeInProgress"
    )
    private Boolean is64BitUpgradeInProgress;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-raid-attributes.mirror-status"

    )
    @OCIEValue(
        ocieColumn = "mirrorStatus",
        notes = "derived from mirrorStatusRaw enum('INVALID','UNINITIALIZED','NEEDS_CP_COUNT_CHECK','CP_COUNT_CHECK_IN_PROGRESS','UNMIRRORED','MIRRORED','MIRROR_DEGRADED','MIRROR_RESYNCHRONIZING','FAILED','LIMBO','NOT_MAPPED')"
    )
    private MirrorStatus mirrorStatus;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-raid-attributes.mirror-status"
    )
    @OCIEValue(
        ocieColumn = "mirrorStatusRaw"
    )
    private String mirrorStatusRaw = null;


    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-sis-attributes.deduplication-space-saved"
    )
    @OCIEValue(
            ocieColumn = "volumeDedupSpaceSavings",
            notes = "Sum of deduplication-space-saved for volumes on this aggregate",
            defaultValue = "0"
    )
    private Long volumeDedupSpaceSavings = 0L;


    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-sis-attributes.compression-space-saved"
    )
    @OCIEValue(
        ocieColumn = "volumeCompressionSpaceSavings",
        notes = "Sum of compression-space-saved for volumes on this aggregate",
        defaultValue = "0"
    )
    private Long volumeCompressionSpaceSavings = 0L;


    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "derived, many"
    )
    @OCIEValue(
            ocieColumn = "totalCommitted",
            notes = "Sum of committed capacity for volumes on this aggregate.  Committed capacity is itself a derived attribute.",
            defaultValue = "0"
    )
    private Long totalCommitted = 0L;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-raid-attributes.has-local-root"
    )
    @OCIEValue(
        ocieColumn = "hasLocalRoot"
    )
    private Boolean hasLocalRoot = null;


    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-raid-attributes.has-partner-root"
    )
    @OCIEValue(
        ocieColumn = "hasPartnerRoot"
    )
    private Boolean hasPartnerRoot = null;

    // Children
    private final Plexes plexes;

    Aggregate(final Node parentNode, final String aggregateUUID) {
        super(parentNode, Aggregate.class.getSimpleName());
        final String clusterUUID = parentNode.getKey().getID(Node.NodeKey.Field.cluster_uuid);
        this.key = new AggregateKey(clusterUUID, aggregateUUID);
        this.plexes = new Plexes(this);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public AggregateKey getKey() {
        return key;
    }

    public Plex createPlex(final String plexName, final Boolean isOnline) {
        return new Plex(this, key.getID(AggregateKey.Field.uuid), plexName, isOnline);
    }

    public Plexes getPlexes() {
        return plexes;
    }

    public String getName() {
        return name;
    }

    public void setName(final String name) {
        this.name = name;
    }

    public String getType() {
        return type;
    }

    public void setType(final String type) {
        this.type = type;
    }

//    public Long getDiskCount() {
//        return diskCount;
//    }
//
//    public void setDiskCount(final Long diskCount) {
//        this.diskCount = diskCount;
//    }

    @SuppressWarnings({"unused declaration"})
    public Boolean isHybrid() {
        return isHybrid;
    }

    public void setHybrid(final Boolean hybrid) {
        isHybrid = hybrid;
    }

    @SuppressWarnings({"unused declaration"})
    public Boolean isHybridEnabled() {
        return isHybridEnabled;
    }

    public void setHybridEnabled(final Boolean hybridEnabled) {
        isHybridEnabled = hybridEnabled;
    }

    @SuppressWarnings({"unused declaration"})
    public Boolean getUsesSharedDisks() {
        return usesSharedDisks;
    }

    public void setUsesSharedDisks(Boolean usesSharedDisks) {
        this.usesSharedDisks = usesSharedDisks;
    }

    @SuppressWarnings({"unused declaration"})
    public AggregateState getState() {
        return state;
    }

    public void setState(final AggregateState state) {
        this.state = state;
    }

    public void setStateRaw(final String stateRaw) {
        this.stateRaw = stateRaw;
    }

    @SuppressWarnings({"unused declaration"})
    public Long getSnapshotSizeTotal() {
        return snapshotSizeTotal;
    }

    public void setSnapshotSizeTotal(final Long snapshotSizeTotal) {
        this.snapshotSizeTotal = snapshotSizeTotal;
    }

    @SuppressWarnings({"unused declaration"})
    public Long getSnapshotSizeUsed() {
        return snapshotSizeUsed;
    }

    public void setSnapshotSizeUsed(final Long snapshotSizeUsed) {
        this.snapshotSizeUsed = snapshotSizeUsed;
    }

    @SuppressWarnings({"unused declaration"})
    public Integer getSizeUsedPercent() {
        return sizeUsedPercent;
    }

    public void setSizeUsedPercent(final Integer sizeUsedPercent) {
        this.sizeUsedPercent = sizeUsedPercent;
    }

    @SuppressWarnings({"unused declaration"})
    public Integer getSizeAvailPercent() {
        return sizeAvailPercent;
    }

    public void setSizeAvailPercent(final Integer sizeAvailPercent) {
        this.sizeAvailPercent = sizeAvailPercent;
    }

    @SuppressWarnings({"unused declaration"})
    public Long getHybridCacheSizeTotal() {
        return hybridCacheSizeTotal;
    }

    public void setHybridCacheSizeTotal(final Long hybridCacheSizeTotal) {
        this.hybridCacheSizeTotal = hybridCacheSizeTotal;
    }

    @SuppressWarnings({"unused declaration"})
    public Long getSizeTotal() {
        return sizeTotal;
    }

    public void setSizeTotal(final Long sizeTotal) {
        this.sizeTotal = sizeTotal;
    }

    @SuppressWarnings({"unused declaration"})
    public Long getSizeUsed() {
        return sizeUsed;
    }

    public void setSizeUsed(final Long sizeUsed) {
        this.sizeUsed = sizeUsed;
    }

    @SuppressWarnings({"unused declaration"})
    public Long getTotalReservedSpace() {
        return totalReservedSpace;
    }

    public void setTotalReservedSpace(final Long totalReservedSpace) {
        this.totalReservedSpace = totalReservedSpace;
    }

    @SuppressWarnings({"unused declaration"})
    public String getRaidStatus() {
        return raidStatus;
    }

    public void setRaidStatus(final String raidStatus) {
        this.raidStatus = raidStatus;
    }

    public void setRaidType(final RaidType raidType) {
        this.raidType = raidType;
    }

    public void setRaidTypeRaw(String raidTypeRaw) {
        this.raidTypeRaw = raidTypeRaw;
    }

    @SuppressWarnings({"unused declaration"})
    public String getRaidTypeRaw() {
        return raidTypeRaw;
    }

    @SuppressWarnings({"unused declaration"})
    public RaidType getRaidType() {
        return raidType;
    }

    public AggregateType getAggregateType() {
        return aggregateType;
    }

    public void setAggregateType(final AggregateType aggregateType) {
        this.aggregateType = aggregateType;
    }

    public String getAggregateTypeRaw() {
        return aggregateTypeRaw;
    }

    public void setAggregateTypeRaw(final String aggregateTypeRaw) {
        this.aggregateTypeRaw = aggregateTypeRaw;
    }

    @SuppressWarnings({"unused declaration"})
    public BlockType getBlockType() {
        return blockType;
    }

    public void setBlockType(final BlockType blockType) {
        this.blockType = blockType;
    }

    public void setBlockTypeRaw(final String blockTypeRaw) {
        this.blockTypeRaw = blockTypeRaw;
    }

    @SuppressWarnings({"unused declaration"})
    public Long getRaidSize() {
        return raidSize;
    }

    public void setRaidSize(final Long raidSize) {
        this.raidSize = raidSize;
    }

    @SuppressWarnings({"unused declaration"})
    public Long getSizeAvailable() {
        return sizeAvailable;
    }

    public void setSizeAvailable(final Long sizeAvailable) {
        this.sizeAvailable = sizeAvailable;
    }

    @SuppressWarnings({"unused declaration"})
    public Long getSnapshotSizeAvailable() {
        return snapshotSizeAvailable;
    }

    public void setSnapshotSizeAvailable(final Long snapshotSizeAvailable) {
        this.snapshotSizeAvailable = snapshotSizeAvailable;
    }

    @SuppressWarnings({"unused declaration"})
    public Boolean isIs64BitUpgradeInProgress() {
        return is64BitUpgradeInProgress;
    }

    public void setIs64BitUpgradeInProgress(final Boolean is64BitUpgradeInProgress) {
        this.is64BitUpgradeInProgress = is64BitUpgradeInProgress;
    }

    @SuppressWarnings({"unused declaration"})
    public MirrorStatus getMirrorStatus() {
        return mirrorStatus;
    }

    public void setMirrorStatus(final MirrorStatus mirrorStatus) {
        this.mirrorStatus = mirrorStatus;
    }

    public void setMirrorStatusRaw(final String mirrorStatusRaw) {
        this.mirrorStatusRaw = mirrorStatusRaw;
    }

    @SuppressWarnings({"unused declaration"})
    public Long getVolumeDedupSpaceSavings() {
        if (Aggregate.shouldFreezeValues(state)) {
            return null;      // Don't overwrite previous value when offline, derived attribute normally defaults to 0.
        } else {
            return volumeDedupSpaceSavings;
        }
    }

    public void addToVolumeDedupSpaceSavings(final Long volumeDedupSpaceSavings) {
        if (volumeDedupSpaceSavings != null) {
            this.volumeDedupSpaceSavings += volumeDedupSpaceSavings;
        }
    }

    @SuppressWarnings({"unused declaration"})
    public Long getVolumeCompressionSpaceSavings() {
        if (Aggregate.shouldFreezeValues(state)) {
            return null;      // Don't overwrite previous value when offline, derived attribute normally defaults to 0.
        } else {
            return volumeCompressionSpaceSavings;
        }
    }

    public void addToVolumeCompressionSpaceSavings(final Long volumeCompressionSpaceSavings) {
        if (volumeCompressionSpaceSavings != null) {
            this.volumeCompressionSpaceSavings += volumeCompressionSpaceSavings;
        }
    }

    @SuppressWarnings({"unused declaration"})
    public Long getTotalCommitted() {
        if (Aggregate.shouldFreezeValues(state)) {
            return null;      // Don't overwrite previous value when offline, derived attribute normally defaults to 0.
        } else {
            return totalCommitted;
        }
    }

    @SuppressWarnings({"unused declaration"})
    public void setTotalCommitted(final Long totalCommitted) {
        this.totalCommitted = totalCommitted;
    }

    public void addToTotalCommitted(final Long totalCommitted) {
        if (totalCommitted != null) {
            this.totalCommitted += totalCommitted;
        }
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean hasLocalRoot() {
        return hasLocalRoot;
    }

    public void setHasLocalRoot(final Boolean hasLocalRoot) {
        this.hasLocalRoot = hasLocalRoot;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean hasPartnerRoot() {
        return hasPartnerRoot;
    }

    public void setHasPartnerRoot(final Boolean hasPartnerRoot) {
        this.hasPartnerRoot = hasPartnerRoot;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum AggregateState {
        NOT_MAPPED,
        UNKNOWN,
        CREATING,
        DESTROYING,
        FAILED,
        FROZEN,
        INCONSISTENT,
        IRON_RESTRICTED,
        MOUNTING,
        OFFLINE,
        ONLINE,
        PARTIAL,
        QUIESCED,
        QUIESCING,
        RESTRICTED,
        REVERTED,
        UNMOUNTED,
        UNMOUNTING, }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum RaidType {
        NOT_MAPPED,
        RAID_DP,
        RAID_4,
        RAID_0,
        MIXED_RAID_TYPE,
        RAID4,
        RAID0,}

    public static enum AggregateType {
        NOT_MAPPED,
        HDD,
        SSD,
        HYBRID,
        LUN,
        VMDISK,}

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum BlockType {
        NOT_MAPPED,
        BIT_32("32_BIT"),
        BIT_64("64_BIT");

        private final String xmlValue;

        private BlockType() {
            this(null);
        }

        private BlockType(final String xmlValue) {
            this.xmlValue = xmlValue;
        }

        public String toXML() {
            return (xmlValue == null ? name() : xmlValue);
        }

        public static BlockType fromString(final String stringVal) {
            if (stringVal == null) {
                return null;
            }
            for (final BlockType candidateBlockType : BlockType.values()) {
                if (candidateBlockType.toXML().equalsIgnoreCase(stringVal)) {
                    return candidateBlockType;
                }
            }
            return NOT_MAPPED;
        }

        public static String toXML(final BlockType blockType) {
            return (blockType == null) ? null : blockType.toXML();
        }
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum MirrorStatus {
        NOT_MAPPED,
        INVALID,
        UNINITIALIZED,
        NEEDS_CP_COUNT_CHECK("needs CP count check"),
        CP_COUNT_CHECK_IN_PROGRESS("CP count check in progress"),
        UNMIRRORED,
        MIRRORED,
        MIRROR_DEGRADED("mirror degraded"),
        MIRROR_RESYNCHRONIZING("mirror resynchronizing"),
        FAILED,
        LIMBO;

        final String rawValue;

        private MirrorStatus() {
            this(null);
        }

        private MirrorStatus(final String rawValue) {
            this.rawValue = rawValue;
        }

        public static MirrorStatus fromString(final String rawValue) {
            if (null == rawValue) {
                return null;
            }
            for (final MirrorStatus candidateMirrorStatus : MirrorStatus.values()) {
                if (candidateMirrorStatus.rawValue().equalsIgnoreCase(rawValue)) {
                    return candidateMirrorStatus;
                }
            }
            return NOT_MAPPED;
        }

        public String rawValue() {
            return (rawValue == null ? name() : rawValue);
        }
    }


    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Name", name);
        addRequiredAttributeAndCRC(writer, "Type", type);
        addRequiredAttributeAndCRC(writer, "IsHybrid", isHybrid);
        addRequiredAttributeAndCRC(writer, "IsHybridEnabled", isHybridEnabled);
        addRequiredAttributeAndCRC(writer, "UsesSharedDisks", usesSharedDisks);
        addRequiredAttributeAndCRC(writer, "State", state);
        addRequiredAttributeAndCRC(writer, "StateRaw", stateRaw);
        addRequiredAttributeAndCRC(writer, "SnapshotSizeTotal", snapshotSizeTotal);
        addRequiredAttributeAndCRC(writer, "SnapshotSizeUsed", snapshotSizeUsed);
        addRequiredAttributeAndCRC(writer, "SnapshotSizeAvail", snapshotSizeAvailable);
        addRequiredAttributeAndCRC(writer, "HybridCacheSizeTotal", hybridCacheSizeTotal);
        addRequiredAttributeAndCRC(writer, "SizeTotal", sizeTotal);
        addRequiredAttributeAndCRC(writer, "SizeUsed", sizeUsed);
        addRequiredAttributeAndCRC(writer, "SizeAvail", sizeAvailable);
        addRequiredAttributeAndCRC(writer, "SizeUsedPercent", sizeUsedPercent);
        addRequiredAttributeAndCRC(writer, "SizeAvailPercent", sizeAvailPercent);
        addRequiredAttributeAndCRC(writer, "TotalReservedSpace", totalReservedSpace);
        addRequiredAttributeAndCRC(writer, "RaidStatus", raidStatus);
        addRequiredAttributeAndCRC(writer, "RaidType", raidType);
        addRequiredAttributeAndCRC(writer, "RaidTypeRaw", raidTypeRaw);
        addRequiredAttributeAndCRC(writer, "RaidSize", raidSize);
        addRequiredAttributeAndCRC(writer, "BlockType", BlockType.toXML(blockType));
        addRequiredAttributeAndCRC(writer, "BlockTypeRaw", blockTypeRaw);
        addRequiredAttributeAndCRC(writer, "Is64BitUpgradeInProgress", is64BitUpgradeInProgress);
        addRequiredAttributeAndCRC(writer, "MirrorStatus", mirrorStatus);
        addRequiredAttributeAndCRC(writer, "MirrorStatusRaw", mirrorStatusRaw);
        addRequiredAttributeAndCRC(writer, "VolumeDedupeSpaceSavings", getVolumeDedupSpaceSavings());
        addRequiredAttributeAndCRC(writer, "VolumeCompressionSpaceSavings", getVolumeCompressionSpaceSavings());
        addRequiredAttributeAndCRC(writer, "TotalCommitted", getTotalCommitted());
        addRequiredAttributeAndCRC(writer, "HasLocalRoot", hasLocalRoot);
        addRequiredAttributeAndCRC(writer, "HasPartnerRoot", hasPartnerRoot);
        addRequiredAttributeAndCRC(writer, "AggregateType", aggregateType);
        addRequiredAttributeAndCRC(writer, "AggregateTypeRaw", aggregateTypeRaw);

    }

    protected boolean hasChildren() {
        return !plexes.isEmpty();
    }

    protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        addAllChildrenUnderNodeAndCRC(cache, writer, Plexes.class.getSimpleName(), plexes.values(), tabs);
    }

    public static boolean shouldFreezeValues(final AggregateState aggregateState) {
        if (Aggregate.AggregateState.OFFLINE == aggregateState
             || Aggregate.AggregateState.RESTRICTED == aggregateState
             || Aggregate.AggregateState.FAILED == aggregateState) {
            return true;
        } else {
            return false;
        }
    }
}
