package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("cifs_share_acl")
@OCIEResourceType("cifs_share_acl")
public class CIFSShareACL extends BaseCRCXMLNode<CIFSShare> {

    public final static class ACLKey extends KeyBase<ACLKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "vserver-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.vserver-info.uuid"
            )
            vserver_uuid,

            @OCIEZapiSource(
                sourceZapi = "cifs-share-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.cifs-share.share-name"
            )
            share_name,

            @OCIEZapiSource(
                sourceZapi = "cifs-share-access-control-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.cifs-share-access-control.user-or-group"
            )
            user_or_group,
        }

        public ACLKey(final CIFSShare.CIFSShareKey shareKey, final String userOrGroup) {
            super("cifs_share_acl",
                  new EnumMap<Field, String>(Field.class) {{
                        put(Field.cluster_uuid, shareKey.getID(CIFSShare.CIFSShareKey.Field.cluster_uuid));
                        put(Field.vserver_uuid, shareKey.getID(CIFSShare.CIFSShareKey.Field.vserver_uuid));
                        put(Field.share_name, shareKey.getID(CIFSShare.CIFSShareKey.Field.share_name));
                        put(Field.user_or_group , userOrGroup);
                  }});
        }

        @Override
        @OCIEUUID(uuidDescr = "vserver_uuid.share_name.user_or_group")
        public String getOcieUUID() {
            final String clusterUUID = getID(Field.cluster_uuid);
            final String vServerUUID = getID(Field.vserver_uuid);
            final String fqVServerUUID = VServer.VServerKey.getOCIEVServerUUID(clusterUUID, vServerUUID);
            final String shareName = getID(Field.share_name);
            final String userOrGroup = getID(Field.user_or_group);
            return fqVServerUUID + '.' + shareName + '.' + userOrGroup;
        }
    }

    public static enum Permission {
        NOT_MAPPED,
        NO_ACCESS,
        READ,
        CHANGE,
        FULL_CONTROL,
    }

    private final ACLKey key;

    @OCIEZapiSource(
        sourceZapi = "cifs-share-access-control-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.cifs-share-access-control.permission"
    )
    @OCIEValue(
        ocieColumn = "permission"
    )
    private Permission permission;


    @OCIEZapiSource(
        sourceZapi = "cifs-share-access-control-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.cifs-share-access-control.permission"
    )
    @OCIEValue(
        ocieColumn = "permissionRaw"
    )
    private String permissionRaw;

    CIFSShareACL(final CIFSShare cifsShare, final String userOrGroup) {
        super(cifsShare, "CIFSShareACL");
        final CIFSShare.CIFSShareKey shareKey = cifsShare.getKey();
        this.key = new ACLKey(shareKey, userOrGroup);
    }


    @Override
    @SuppressWarnings("unchecked")
    public ACLKey getKey() {
        return key;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Permission getPermission() {
        return permission;
    }

    public void setPermission(final Permission permission) {
        this.permission = permission;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getPermissionRaw() {
        return permissionRaw;
    }

    public void setPermissionRaw(final String permissionRaw) {
        this.permissionRaw = permissionRaw;
    }

    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Permission", permission);
        addRequiredAttributeAndCRC(writer, "PermissionRaw", permissionRaw);
        addRequiredAttributeAndCRC(writer, "UserOrGroup", key.getID(ACLKey.Field.user_or_group));
    }
}
