package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.util.VersionComparator;
import java.io.IOException;
import java.util.EnumMap;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@OCIETable("cluster")
@OCIEResourceType("cluster")
public class Cluster extends BaseCRCXMLNode<StorageRoot> {

    public final static class ClusterKey extends KeyBase<ClusterKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            uuid
        }

        public ClusterKey(final String clusterUUID) {
            super("cluster",
                new EnumMap<Field, String>(Field.class) {{
                    put(Field.cluster_uuid, clusterUUID);
                    put(Field.uuid, clusterUUID);
                }});
        }

        @Override
        @OCIEUUID(uuidDescr = "cluster_uuid")
        public String getOcieUUID() {
            return getID(Field.cluster_uuid);
        }
    }

    private final ClusterKey key;

    @OCIEZapiSource(
        sourceZapi = "cluster-identity-get",
        zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-name"
    )
    @OCIEValue(
        ocieColumn = "name"
    )
    private String name;


    @OCIEZapiSource(
        sourceZapi = "operator configured"
    )
    @OCIEValue(
        ocieColumn = "managementIp"
    )
    private String mangementIP;

    @OCIEZapiSource(
            sourceZapi = "cluster-ha-get",
            zapiResponsePath = "netapp.results.attributes.cluster-ha-info.ha-configured"
    )
    @OCIEValue(
            ocieColumn = "haConfigured"
    )
    private Boolean haConfigured;


    @OCIEZapiSource(
        sourceZapi = "cluster-identity-get",
        zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-serial-number"
    )
    @OCIEValue(
        ocieColumn = "serialNumber"
    )
    private String serialNum;


    @OCIEZapiSource(
        sourceZapi = "cluster-identity-get",
        zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-contact"
    )
    @OCIEValue(
        ocieColumn = "contact"
    )
    private String contact;


    @OCIEZapiSource(
        sourceZapi = "cluster-identity-get",
        zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-location"
    )
    @OCIEValue(
        ocieColumn = "location"
    )
    private String location;


    @OCIEZapiSource(
        sourceZapi = "system-get-version",
        zapiResponsePath = "netapp.results.version"
    )
    @OCIEValue(
        ocieColumn = "version"
    )
    private String versionString;


    @OCIEZapiSource(
        sourceZapi = "system-get-version",
        zapiResponsePath = "netapp.results.ersion-tuple.system-version-tuple.generation"
    )
    @OCIEValue(
        ocieColumn = "versionGeneration"
    )
    private Integer generation;


    @OCIEZapiSource(
        sourceZapi = "system-get-version",
        zapiResponsePath = "netapp.results.ersion-tuple.system-version-tuple.major"
    )
    @OCIEValue(
        ocieColumn = "versionMajor"
    )
    private Integer major;


    @OCIEZapiSource(
        sourceZapi = "system-get-version",
        zapiResponsePath = "netapp.results.ersion-tuple.system-version-tuple.minor"
    )
    @OCIEValue(
        ocieColumn = "versionMinor"
    )
    private Integer minor;


    @OCIEZapiSource(
        sourceZapi = "derived"
    )
    @OCIEValue(
        ocieColumn = "rawDiskBytesTotal",
        notes = "Sum of all disks regardless of state or role."
    )
    private Long rawBytesTotal = null;


    @OCIEZapiSource(
        sourceZapi = "derived"
    )
    @OCIEValue(
        ocieColumn = "rawDiskBytesUsed",
        notes = "Sum of all disks regardless of state or role."
    )
    private Long rawBytesUsed = null;


    @OCIEZapiSource(
        sourceZapi = "clock-get-timezone",
        zapiResponsePath = "netapp.results.timezone"
    )
    @OCIEValue(
        ocieColumn = "timeZone"
    )
    private String timezone;


    @OCIEZapiSource(
        sourceZapi = "clock-get-timezone",
        zapiResponsePath = "netapp.results.timezone-UTC"
    )
    @OCIEValue(
        ocieColumn = "timeZoneUTC"
    )
    private String timezoneUTC;


    @OCIEZapiSource(
        sourceZapi = "clock-get-timezone",
        zapiResponsePath = "netapp.results.timezone-version"
    )
    @OCIEValue(
        ocieColumn = "timeZoneVersion"
    )
    private String timezoneVersion;

    /**
     * Overall system health as determined by the diagnosis framework. diagnosis-status-get
     */
    @OCIEZapiSource(
        sourceZapi = "diagnosis-status-get",
        zapiResponsePath = "netapp.results.attributes.diagnosis-status.status"
    )
    @OCIEValue(
        ocieColumn = "diagnosisStatus",
        notes = "derived from diagnosisStatusRaw enum('NOT_MAPPED','OK','OK_WITH_SUPPRESSED','DEGRADED','UNREACHABLE','UNKNOWN')"
    )
    private HmStatus diagnosisStatus;


    @OCIEZapiSource(
        sourceZapi = "diagnosis-status-get",
        zapiResponsePath = "netapp.results.attributes.diagnosis-status.status"
    )
    @OCIEValue(
        ocieColumn = "diagnosisStatusRaw"
    )
    private String diagnosisStatusRaw;


    @OCIEZapiSource(
            sourceZapi = "metrocluster-is-configured",
            zapiResponsePath = "netapp.results.is-mcc-config"
    )
    @OCIEValue(
            ocieColumn = "isMetrocluster"
    )
    private Boolean isMetrocluster;
    
    @OCIEZapiSource(
            sourceZapi = "system-services-web-get",
            zapiResponsePath = "netapp.results.attributes.system-services-web-info.ssl-fips-enabled"
    )
    @OCIEValue(
            ocieColumn = "sslFipsEnabled"
    )
    private Boolean sslFipsEnabled;

    @OCIEZapiSource(
            sourceZapi = "metrocluster-get",
            zapiResponsePath = "netapp.results.attributes.metrocluster-info.local-configuration-state"
    )
    @OCIEValue(
            ocieColumn = "mtConfigurationState"
    )
    private MtConfigurationState mtConfigurationState;

    @OCIEZapiSource(
            sourceZapi = "metrocluster-get",
            zapiResponsePath = "netapp.results.attributes.metrocluster-info.local-configuration-state"
    )
    @OCIEValue(
            ocieColumn = "mtConfigurationStateRaw"
    )
    private String mtConfigurationStateRaw;

    @OCIEZapiSource(
            sourceZapi = "metrocluster-get",
            zapiResponsePath = "netapp.results.attributes.metrocluster-info.local-mode"
    )
    @OCIEValue(
            ocieColumn = "mtMode"
    )
    private MtMode mtMode;

    @OCIEZapiSource(
            sourceZapi = "metrocluster-get",
            zapiResponsePath = "netapp.results.attributes.metrocluster-info.local-mode"
    )
    @OCIEValue(
            ocieColumn = "mtModeRaw"
    )
    private String mtModeRaw;

    @OCIEZapiSource(
            sourceZapi = "metrocluster-get",
            zapiResponsePath = "netapp.results.attributes.metrocluster-info.remote-configuration-state"
    )
    @OCIEValue(
            ocieColumn = "mtRemoteConfigurationState"
    )
    private MtConfigurationState mtRemoteConfigurationState;

    @OCIEZapiSource(
            sourceZapi = "metrocluster-get",
            zapiResponsePath = "netapp.results.attributes.metrocluster-info.remote-configuration-state"
    )
    @OCIEValue(
            ocieColumn = "mtRemoteConfigurationStateRaw"
    )
    private String mtRemoteConfigurationStateRaw;

    @OCIEZapiSource(
            sourceZapi = "metrocluster-get",
            zapiResponsePath = "netapp.results.attributes.metrocluster-info.remote-mode"
    )
    @OCIEValue(
            ocieColumn = "mtRemoteMode"
    )
    private MtMode mtRemoteMode;

    @OCIEZapiSource(
            sourceZapi = "metrocluster-get",
            zapiResponsePath = "netapp.results.attributes.metrocluster-info.remote-mode"
    )
    @OCIEValue(
            ocieColumn = "mtRemoteModeRaw"
    )
    private String mtRemoteModeRaw;

    @OCIEZapiSource(
            sourceZapi = "metrocluster-get",
            zapiResponsePath = "netapp.results.attributes.metrocluster-info.configuration-type"
    )
    @OCIEValue(
            ocieColumn = "mtConfigurationType"
    )
    private String mtConfigurationType;

    @OCIEZapiSource(
            sourceZapi = "metrocluster-node-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.metrocluster-node-info.dr-partner-cluster-uuid"
    )
    @OCIEValue(
            ocieColumn = "mtDrPartnerClusterUUID"
    )
    private String mtDrPartnerClusterUUID;

    // Some CPU metrics are type "array", a single counter name returns a CSV
    // of sub-column values.   We must dynamically discover the sub-column
    // names in order to interpret the counters correctly in performance.
    private List<String> cpuColumns;
    
    private List<String> waflCpPhaseTimesColumns;

    // Some Volume Workload metrics are type "array", a single counter name returns a CSV
    // of sub-column values.   We must dynamically discover the sub-column
    // names in order to interpret the counters correctly in performance.
    private List<String> volumeWorkloadReadIOColumns;
    
    // Node performance UUID maps
    private final Map<String, String> cifsNodeUUIDNameMap = new HashMap<String, String>();
    private final Map<String, String> smb1NodeUUIDNameMap = new HashMap<String, String>();
    private final Map<String, String> smb2NodeUUIDNameMap = new HashMap<String, String>();
    private final Map<String, String> nfsv3NodeUUIDNameMap = new HashMap<String, String>();
    private final Map<String, String> nfsv4NodeUUIDNameMap = new HashMap<String, String>();
    private final Map<String, String> nfsv4_1NodeUUIDNameMap = new HashMap<String, String>();
    
    // VserverID to VserverName maps for cifs/nfs
    private final Map<String, String> cifsVserverIDNameMap = new HashMap<>();
    private final Map<String, String> nfsv3VserverIDNameMap = new HashMap<>();
    private final Map<String, String> nfsv4VserverIDNameMap = new HashMap<>();
    private final Map<String, String> nfsv41VserverIDNameMap = new HashMap<>();
    private final Map<String, String> lifVserverIDNameMap = new HashMap<>();

    // Children
    private final DoNotDeleteList doNotDeleteList;
    private final Nodes nodes;
    private final StorageShelves storageShelves;
    private final StorageSwitches storageSwitches;
    private final StoragePools storagePools;
    private final Disks disks;
    private final DiskAggregateRelationships diskAggregateRelationships;
    private final NetworkFailoverGroupNetworkPortRelationships networkFailoverGroupNetworkPortRelationships;
    private final VServers vServers;
    private final JobSchedules jobSchedules;
    private final SnapshotPolicies snapshotPolicies;
    private final SnapMirrorPolicies snapMirrorPolicies;
    private final HaPairs haPairs;
    private final DiskPaths diskPaths;
    private final Licenses licenses;
    private final SnapMirrors snapMirrors;
    private final SnapMirrorHistories snapMirrorHistories;
    private final SnapMirrorSelfRelationships snapMirrorSelfRelationships;
    private final NetworkPortRelationships portRelationships;
    private final QosWorkloadDetails qosWorkloadDetails;
    private final VServerAggregates vServerAggregates;
    private final SnapshotPolicyVserverRelationships snapshotPolicyVServerRelationships;
    private final QosVolumeWorkloads qosVolumeWorkloads;
    private final QosWorkloadNodeRelationships qosWorkloadNodeRelationships;
    private final QosPolicyGroups qosPolicyGroups;
    private final QosPolicyGroupVServerRelationships qosPolicyGroupVServerRelationships;
    private final ApplicationRecords applicationRecords;
    private final SystemHealthAlerts systemHealthAlerts;
    private final NetworkIpSpaces networkIpSpaces;
    private final NetworkSubnets networkSubnets;
    private final NetworkPortBroadcastDomains networkPortBroadcastDomains;
    private final NetworkFailoverGroups networkFailoverGroups;
    private final LunImports lunImports;
    private final MetroClusterOperations metroClusterOperations; 
    private final ClusterPeers clusterPeers;
    private final VServerPeers vServerPeers;

    public static final String SIERRA_NEVADA = "8.2";   // unfortunately for version specific behavior
    public static final String SIERRA_NEVADA_0 = "8.2.0";
    public static final String SIERRA_NEVADA_1 = "8.2.1";
    public static final String FULL_STEAM = "8.3";
    public static final String FULL_STEAM_0 = "8.3.0";
    public static final String FULL_STEAM_1 = "8.3.1";
    public static final String LONG_BOARD_0 = "8.4.0";

    Cluster(final StorageRoot parent, final String clusterUUID) {
        super(parent, "Cluster");
        this.key = new ClusterKey(clusterUUID);
        this.vServers = new VServers(this);
        this.jobSchedules = new JobSchedules(this);
        this.snapshotPolicies = new SnapshotPolicies(this);
        this.snapMirrorPolicies = new SnapMirrorPolicies(this);
        this.doNotDeleteList = new DoNotDeleteList(this);
        this.nodes = new Nodes(this);
        this.storageShelves = new StorageShelves(this);
        this.storageSwitches = new StorageSwitches(this);
        this.storagePools = new StoragePools(this);
        this.disks = new Disks(this);
        this.diskAggregateRelationships = new DiskAggregateRelationships(this);
        this.haPairs = new HaPairs(this);
        this.diskPaths = new DiskPaths(this);
        this.licenses = new Licenses(this);
        this.snapMirrors = new SnapMirrors(this);
        this.snapMirrorHistories = new SnapMirrorHistories(this);
        this.snapMirrorSelfRelationships = new SnapMirrorSelfRelationships(this);
        this.portRelationships = new NetworkPortRelationships(this);
        this.qosWorkloadDetails = new QosWorkloadDetails(this);
        this.vServerAggregates = new VServerAggregates(this);
        this.snapshotPolicyVServerRelationships = new SnapshotPolicyVserverRelationships(this);
        this.qosVolumeWorkloads = new QosVolumeWorkloads(this);
        this.qosWorkloadNodeRelationships = new QosWorkloadNodeRelationships(this);
        this.qosPolicyGroups = new QosPolicyGroups(this);
        this.qosPolicyGroupVServerRelationships = new QosPolicyGroupVServerRelationships(this);
        this.applicationRecords = new ApplicationRecords(this);
        this.systemHealthAlerts = new SystemHealthAlerts(this);
        this.networkIpSpaces = new NetworkIpSpaces(this);
        this.networkSubnets = new NetworkSubnets(this);
        this.networkPortBroadcastDomains = new NetworkPortBroadcastDomains(this);
        this.networkFailoverGroups= new NetworkFailoverGroups(this);
        this.networkFailoverGroupNetworkPortRelationships = new NetworkFailoverGroupNetworkPortRelationships(this);
        this.lunImports = new LunImports(this);
        this.metroClusterOperations = new MetroClusterOperations(this);
        this.clusterPeers = new ClusterPeers(this);
        this.vServerPeers = new VServerPeers(this);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public ClusterKey getKey() {
        return key;
    }

    public String getName() {
        return name;
    }

    public void setName(final String name) {
        this.name = name;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getMangementIP() {
        return mangementIP;
    }

    public void setMangementIP(final String mangementIP) {
        this.mangementIP = mangementIP;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getSerialNum() {
        return serialNum;
    }

    public void setSerialNum(final String serialNum) {
        this.serialNum = serialNum;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getVersionString() {
        return versionString;
    }

    public void setVersionString(final String versionString) {
        this.versionString = versionString;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getGeneration() {
        return generation;
    }

    public void setGeneration(final Integer generation) {
        this.generation = generation;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getMajor() {
        return major;
    }

    public void setMajor(final Integer major) {
        this.major = major;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getMinor() {
        return minor;
    }

    public void setMinor(final Integer minor) {
        this.minor = minor;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getContact() {
        return contact;
    }

    public void setContact(final String contact) {
        this.contact = contact;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getLocation() {
        return location;
    }

    public void setLocation(final String location) {
        this.location = location;
    }

//    @SuppressWarnings({"UnusedDeclaration"})
//    public Boolean isSpareLow() {
//        return isSpareLow;
//    }
//
//    public void setSpareLow(final Boolean spareLow) {
//        isSpareLow = spareLow;
//    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getRawBytesTotal() {
        return rawBytesTotal;
    }

    public void setRawBytesTotal(final Long rawBytesTotal) {
        this.rawBytesTotal = rawBytesTotal;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getRawBytesUsed() {
        return rawBytesUsed;
    }

    public void setRawBytesUsed(final Long rawBytesUsed) {
        this.rawBytesUsed = rawBytesUsed;
    }

    public void setHaConfigured(final Boolean haConfigured) {
        this.haConfigured = haConfigured;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean getHaConfigured() {
        return haConfigured;
    }

    public HmStatus getDiagnosisStatus() {
        return diagnosisStatus;
    }

    public void setDiagnosisStatus(final HmStatus diagnosisStatus) {
        this.diagnosisStatus = diagnosisStatus;
    }

    public String getDiagnosisStatusRaw() {
        return diagnosisStatusRaw;
    }

    public void setDiagnosisStatusRaw(final String diagnosisStatusRaw) {
        this.diagnosisStatusRaw = diagnosisStatusRaw;
    }

    public List<String> getCpuColumns() {
        return cpuColumns;
    }

    public void setCpuColumns(final List<String> cpuColumns) {
        this.cpuColumns = cpuColumns;
    }
    
    public List<String> getWaflPhaseTimesColumns(){
    	return waflCpPhaseTimesColumns;
    }
    
    public void setWaflPhaseTimesColumns(final List<String> waflCpPhaseTimesColumns){
    	this.waflCpPhaseTimesColumns = waflCpPhaseTimesColumns;
    }

    public Map<String, String> getCifsNodeUUIDNameMap() {
        return cifsNodeUUIDNameMap;
    }

    public Map<String, String> getSmb1NodeUUIDNameMap() {
        return smb1NodeUUIDNameMap;
    }

    public Map<String, String> getSmb2NodeUUIDNameMap() {
        return smb2NodeUUIDNameMap;
    }

    public Map<String, String> getNfsv3NodeUUIDNameMap() {
        return nfsv3NodeUUIDNameMap;
    }

    public Map<String, String> getNfsv4NodeUUIDNameMap() {
        return nfsv4NodeUUIDNameMap;
    }

    public Map<String, String> getNfsv4_1NodeUUIDNameMap() {
        return nfsv4_1NodeUUIDNameMap;
    }

    public Map<String, String> getCifsVserverIDNameMap() {
        return cifsVserverIDNameMap;
    }

    public Map<String, String> getNfsv3VserverIDNameMap() {
        return nfsv3VserverIDNameMap;
    }

    public Map<String, String> getNfsv4VserverIDNameMap() {
        return nfsv4VserverIDNameMap;
    }

    public Map<String, String> getNfsv41VserverIDNameMap() {
        return nfsv41VserverIDNameMap;
    }

    public Map<String, String> getLifVserverIDNameMap() {
        return lifVserverIDNameMap;
    }
    
    public List<String> getVolumeWorkloadReadIOColumns() {
        return volumeWorkloadReadIOColumns;
    }

    public void setVolumeWorkloadReadIOColumns(final List<String> volumeWorkloadReadIOColumns) {
        this.volumeWorkloadReadIOColumns = volumeWorkloadReadIOColumns;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getClusterUUID() {
        return key.getOcieUUID();
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getTimezone() {
        return timezone;
    }

    public void setTimezone(final String timezone) {
        this.timezone = timezone;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getTimezoneUTC() {
        return timezoneUTC;
    }

    public void setTimezoneUTC(final String timezoneUTC) {
        this.timezoneUTC = timezoneUTC;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getTimezoneVersion() {
        return timezoneVersion;
    }

    public void setTimezoneVersion(final String timezoneVersion) {
        this.timezoneVersion = timezoneVersion;
    }

    public Boolean getMetrocluster() {
        return isMetrocluster;
    }

    public void setMetrocluster(Boolean metrocluster) {
        isMetrocluster = metrocluster;
    }
    
    public Boolean getSslFipsEnabled() {
        return sslFipsEnabled;
    }

    public void setSslFipsEnabled(Boolean sslFipsEnabled) {
        this.sslFipsEnabled = sslFipsEnabled;
    }

    public MtConfigurationState getMtConfigurationState() {
        return mtConfigurationState;
    }

    public void setMtConfigurationState(MtConfigurationState mtConfigurationState) {
        this.mtConfigurationState = mtConfigurationState;
    }

    public String getMtConfigurationStateRaw() {
        return mtConfigurationStateRaw;
    }

    public void setMtConfigurationStateRaw(String mtConfigurationStateRaw) {
        this.mtConfigurationStateRaw = mtConfigurationStateRaw;
    }

    public MtMode getMtMode() {
        return mtMode;
    }

    public void setMtMode(MtMode mtMode) {
        this.mtMode = mtMode;
    }

    public String getMtModeRaw() {
        return mtModeRaw;
    }

    public void setMtModeRaw(String mtModeRaw) {
        this.mtModeRaw = mtModeRaw;
    }


    public MtConfigurationState getMtRemoteConfigurationState() {
        return mtRemoteConfigurationState;
    }

    public void setMtRemoteConfigurationState(MtConfigurationState mtRemoteConfigurationState) {
        this.mtRemoteConfigurationState = mtRemoteConfigurationState;
    }

    public String getMtRemoteConfigurationStateRaw() {
        return mtRemoteConfigurationStateRaw;
    }

    public void setMtRemoteConfigurationStateRaw(String mtRemoteConfigurationStateRaw) {
        this.mtRemoteConfigurationStateRaw = mtRemoteConfigurationStateRaw;
    }

    public MtMode getMtRemoteMode() {
        return mtRemoteMode;
    }

    public void setMtRemoteMode(MtMode mtRemoteMode) {
        this.mtRemoteMode = mtRemoteMode;
    }

    public String getMtRemoteModeRaw() {
        return mtRemoteModeRaw;
    }

    public void setMtRemoteModeRaw(String mtRemoteModeRaw) {
        this.mtRemoteModeRaw = mtRemoteModeRaw;
    }

    public String getMtConfigurationType() {
        return mtConfigurationType;
    }

    public void setMtConfigurationType(String mtConfigurationType) {
        this.mtConfigurationType = mtConfigurationType;
    }

    public String getMtDrPartnerClusterUUID() {
        return mtDrPartnerClusterUUID;
    }

    public void setMtDrPartnerClusterUUID(String mtDrClusterId) {
        this.mtDrPartnerClusterUUID = mtDrClusterId;
    }

    public DoNotDelete createDoNotDelete(final String uuid) {
        return new DoNotDelete(this, uuid);
    }
    
    public DoNotDeleteList getDoNotDeleteList() {
        return doNotDeleteList;
    }
    
    public Node createNode(final String uuid) {
        return new Node(this, uuid);
    }
    
    public Nodes getNodes() {
        return nodes;
    }

  public NetworkFailoverGroupNetworkPortRelationships getNetworkFailoverGroupNetworkPortRelationships()
  {
    return networkFailoverGroupNetworkPortRelationships;
  }

  public StorageShelves getStorageShelves() {
        return storageShelves;
    }

    public StorageShelf createStorageShelf(final String shelfUID) {
        return new StorageShelf(this, shelfUID);
    }

    public StorageSwitches getStorageSwitches() {
        return storageSwitches;
    }

    public StorageSwitch createStorageSwitch(final String wwn) {
        return new StorageSwitch(this, wwn);
    }

    public Disk createDisk(final String uuid) {
        return new Disk(this, uuid);
    }

    public DiskAggregateRelationship createDiskAggregateRelationship(final Disk.DiskKey diskKey, final Aggregate.AggregateKey aggregateKey) {
        return new DiskAggregateRelationship(this, diskKey, aggregateKey);
    }

    public NetworkFailoverGroupNetworkPortRelationship createNetworkFailoverGroupNetworkPortRelationship(final Node.NodeKey nodeKey, final NetworkFailoverGroup.NetworkFailoverGroupKey networkFailoverGroupKey, final NetworkPort.PortKey networkPortKey) {
      return new NetworkFailoverGroupNetworkPortRelationship(this, nodeKey, networkFailoverGroupKey, networkPortKey);
    }

    public DiskAggregateRelationships getDiskAggregateRelationships() {
        return diskAggregateRelationships;
    }

    public StoragePool createStoragePool(final String storagePoolName) {
        return new StoragePool(this, storagePoolName);
    }

    public Disks getDisks() {
        return disks;
    }

    public StoragePools getStoragePools() {
        return storagePools;
    }

    public VServer createVServer(final String clusterUUID, final String vServerUUID) {
        return new VServer(this, clusterUUID, vServerUUID);
    }

    public VServers getVServers() {
        return vServers;
    }

    public JobSchedule createJobSchedule(final String jobScheduleName) {
        return new JobSchedule(this, jobScheduleName);
    }

    public JobSchedules getJobSchedules() {
        return jobSchedules;
    }

    public SnapshotPolicy createSnapshotPolicy(final VServer.VServerKey vServerKey, final String policyName) {
        return new SnapshotPolicy(this, vServerKey, policyName);
    }

    public SnapshotPolicies getSnapshotPolicies() {
        return snapshotPolicies;
    }

    public HaPair createHaPair(final Node.NodeKey nodeAKey) {
        return new HaPair(this, nodeAKey);
    }

    public HaPairs getHaPairs() {
        return haPairs;
    }

    public DiskPath createDiskPath(final String targetWWPN, final Disk.DiskKey diskKey, final StorageAdapter.AdapterKey adapterKey) {
        return new DiskPath(this, targetWWPN, diskKey, adapterKey);
    }

    public DiskPaths getDiskPaths() {
        return diskPaths;
    }

    public License createLicense(final String packageName, final String serialNumber) {
        return new License(this, packageName, serialNumber);
    }

    public Licenses getLicenses() {
        return licenses;
    }

    public SnapMirrors getSnapMirrors() {
        return snapMirrors;
    }
    
    public SnapMirrorHistories getSnapMirrorHistories() {
        return snapMirrorHistories;
    }

    public SnapMirrorSelfRelationships getSnapMirrorSelfRelationships() {
        return snapMirrorSelfRelationships;
    }

    public SnapMirror createSnapMirror(final VServer.VServerKey destinationVServerKey, final Volume.VolumeKey destinationVolumeKey,
                                       final String destinationLocation) {
        return new SnapMirror(this, destinationVServerKey, destinationVolumeKey, destinationLocation);
    }

    public SnapMirror createSnapMirror(final VServer.VServerKey destinationVServerKey, final String destinationLocation) {
        return new SnapMirror(this, destinationVServerKey, destinationLocation);
    }
    
    public SnapMirrorHistory createSnapMirrorHistory(String operationId) {
        return new SnapMirrorHistory(this, operationId);
    }

    public SnapMirrorSelfRelationship createSnapMirrorSelfRelationship(final String volumeSnapMirrorUUID, final String vServerSnapMirrorUUID) {
        return new SnapMirrorSelfRelationship(this, volumeSnapMirrorUUID, vServerSnapMirrorUUID);
    }

    public SnapMirrorPolicies getSnapMirrorPolicies() {
        return snapMirrorPolicies;
    }

    public SnapMirrorPolicy createSnapMirrorPolicy(final VServer.VServerKey vServerKey, final String policyName) {
        return new SnapMirrorPolicy(this, vServerKey, policyName);
    }
    
    public LunImport createLunImport(final VServer.VServerKey vServerKey, final String foreignDisk) {
        return new LunImport(this, vServerKey, foreignDisk);
    }
    
    public MetroClusterOperation createMetroClusterOperation(final String uuid) {
        return new MetroClusterOperation(this, uuid);
    }
    
    public QosWorkloadDetails getQosWorkloadDetails() {
        return qosWorkloadDetails;
    }

    public QosWorkloadDetail createQosWorkloadDetail(final String uuid) {
        return new QosWorkloadDetail(this, uuid);
    }
    
    public NetworkPortRelationships getPortRelationships() {
        return portRelationships;
    }

    public NetworkPortRelationship createNetworkPortRelationship(final NetworkPort.PortKey sourcePortKey) {
        return new NetworkPortRelationship(this, sourcePortKey);
    }

    public NetworkIpSpace createNetworkIPSpace(final String ipSpaceUUID) {
        return new NetworkIpSpace(this, ipSpaceUUID);
    }

    public NetworkSubnet createNetworkSubnet(final String ipSpaceUUID, final String name) {
        return new NetworkSubnet(this, ipSpaceUUID, name);
    }

    public NetworkIpSpaces getNetworkIpSpaces() {
        return networkIpSpaces;
    }


    public NetworkSubnets getNetworkSubnets() {
        return networkSubnets;
    }
    
    public LunImports getLunImports() {
        return lunImports;
    }
    
    public MetroClusterOperations getMetroClusterOperations() {
        return metroClusterOperations;
    }

    public NetworkPortBroadcastDomain createNetworkPortBroadcastDomain(final String broadcastDomainName,
                                                                       final NetworkIpSpace.NetworkIpSpaceKey networkIpSpaceKey) {
        return new NetworkPortBroadcastDomain(this, broadcastDomainName, networkIpSpaceKey);
    }

    public NetworkPortBroadcastDomains getNetworkPortBroadcastDomains() {
        return networkPortBroadcastDomains;
    }

    public NetworkFailoverGroups getNetworkFailoverGroups() {
        return networkFailoverGroups;
    }

    public NetworkFailoverGroup createNetworkFailoverGroup(final String networkFailoverGroupName) {
        return new NetworkFailoverGroup(this, networkFailoverGroupName);
    }


    public VServerAggregate createVServerAggregate(final VServer.VServerKey vServerKey, final Aggregate.AggregateKey aggregateKey) {
        return new VServerAggregate(this, vServerKey, aggregateKey);
    }

    public VServerAggregates getVServerAggregates() {
        return vServerAggregates;
    }

    public SnapshotPolicyVserverRelationship createSnapshotPolicyVserverRelationship(final SnapshotPolicy.SnapshotPolicyKey snapshotPolicyKey,
                                                                                     final VServer.VServerKey vServerKey) {
        return new SnapshotPolicyVserverRelationship(this, snapshotPolicyKey, vServerKey);
    }

    public SnapshotPolicyVserverRelationships getSnapshotPolicyVserverRelationships() {
        return snapshotPolicyVServerRelationships;
    }

    public QosVolumeWorkloads getQosVolumeWorkloads(){
        return qosVolumeWorkloads;
    }

    public QosWorkloadNodeRelationships getQosWorkloadNodeRelationships() {
        return qosWorkloadNodeRelationships;
    }

    public QosWorkloadNodeRelationship createQosWorkloadNodeRelationship(Node.NodeKey nodeKey, QosVolumeWorkload.QosVolumeWorkloadKey qosVolumeWorkloadKey) {
        return new QosWorkloadNodeRelationship(this, nodeKey, qosVolumeWorkloadKey);
    }

    public QosVolumeWorkload createQosVolumeWorkload(final String uuid){
        return new QosVolumeWorkload(this, uuid);
    }

    public ApplicationRecord createAndAddApplicationRecord(final String recordName) {
        final ApplicationRecord applicationRecord = new ApplicationRecord(this, recordName);
        applicationRecords.add(applicationRecord);
        return applicationRecord;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public ApplicationRecords getApplicationRecords() {
        return applicationRecords;
    }

    public SystemHealthAlert createSystemHealthAlert(String alertId, String alertingResource) {
        return new SystemHealthAlert(this, alertId, alertingResource);
    }

    public SystemHealthAlerts getSystemHealthAlerts(){
        return systemHealthAlerts;
    }


    public QosPolicyGroup createQosPolicyGroup(final String qosPolicyGroupUUID) {
        return new QosPolicyGroup(this, qosPolicyGroupUUID);
    }

    public QosPolicyGroups getQosPolicyGroups() {
        return qosPolicyGroups;
    }

    public QosPolicyGroupVServerRelationship createQosPolicyGroupVServerRelationship(final QosPolicyGroup.QosPolicyGroupKey qosPolicyGroupKey, final VServer.VServerKey vServerKey) {
        return new QosPolicyGroupVServerRelationship(this, qosPolicyGroupKey, vServerKey);
    }

    public QosPolicyGroupVServerRelationships getQosPolicyGroupVServerRelationships() {
        return qosPolicyGroupVServerRelationships;
    }

    public ClusterPeers getClusterPeers() {
        return clusterPeers;
    }

    public ClusterPeer createClusterPeer(String remoteClusterUUID) {
        return new ClusterPeer(this, remoteClusterUUID);
    }
    
    public VServerPeers getVServerPeers() {
        return vServerPeers;
    }

    public VServerPeer createVServerPeer(String peerClusterName, String peerVServerName, String vServerName) {
        return new VServerPeer(this, peerClusterName, peerVServerName, vServerName);
    }

    public VServerPeer createVServerPeer(String peerVServerUUID, String vServerUUID) {
        return new VServerPeer(this, peerVServerUUID, vServerUUID);
    }
    
    public boolean isLessThanSierraNevada() {
        final VersionComparator versionComparator = new VersionComparator();
        return (versionComparator.compare(versionString, Cluster.SIERRA_NEVADA) < 0);
    }

    /**
     * Health Monitor Status. See
     * <a href="http://web.netapp.com/engineering/design-depot/appliance-mgmt/zephyr/zapidoc.cache/DOT/RsierranevadaN/web_ontap_cluster/diagnosis/index.html#hm-status">hm-status</a>
     */
    public static enum HmStatus {
        NOT_MAPPED,
        OK,
        OK_WITH_SUPPRESSED,
        DEGRADED,
        UNREACHABLE,
        UNKNOWN,
    }

    /**
     *  Local Configuration State. See
     * http://web.netapp.com/engineering/design-depot/appliance-mgmt/zephyr/zapidoc.cache/DOT/mainN/web_ontap_cluster/metrocluster/index.html#metrocluster-info
     */
    public static enum MtConfigurationState {
        NOT_MAPPED,
        UNKNOWN,
        NOT_REACHABLE,
        NOT_CONFIGURED,
        CONFIGURATION_ERROR,
        PARTIALLY_CONFIGURED,
        CONFIGURED,
    }

    /**
     *  Local Mode. See
     * http://web.netapp.com/engineering/design-depot/appliance-mgmt/zephyr/zapidoc.cache/DOT/mainN/web_ontap_cluster/metrocluster/index.html#metrocluster-info
     */
    public static enum MtMode {
        NOT_MAPPED,
        UNKNOWN,
        NOT_REACHABLE,
        NOT_CONFIGURED,
        NORMAL,
        SWITCHOVER,
        WAITING_FOR_SWITCHBACK,
        PARTIAL_SWITCHOVER,
        PARTIAL_SWITCHBACK,
    }

    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Name", name);
        addRequiredAttributeAndCRC(writer, "SerialNumber", serialNum);
        addRequiredAttributeAndCRC(writer, "ManagementIP", mangementIP);
        addRequiredAttributeAndCRC(writer, "Version", versionString);
        addRequiredAttributeAndCRC(writer, "VersionGeneration", generation);
        addRequiredAttributeAndCRC(writer, "VersionMajor", major);
        addRequiredAttributeAndCRC(writer, "VersionMinor", minor);
        addRequiredAttributeAndCRC(writer, "Contact", contact);
        addRequiredAttributeAndCRC(writer, "Location", location);
        // addRequiredAttributeAndCRC(writer, "IsSpareLow", isSpareLow);
        addRequiredAttributeAndCRC(writer, "RawDiskBytesTotal", rawBytesTotal);
        addRequiredAttributeAndCRC(writer, "RawDiskBytesUsed", rawBytesUsed);
        addRequiredAttributeAndCRC(writer, "DiagnosisStatus", getDiagnosisStatus());
        addRequiredAttributeAndCRC(writer, "DiagnosisStatusRaw", getDiagnosisStatusRaw());
        addRequiredAttributeAndCRC(writer, "TimeZone", timezone);
        addRequiredAttributeAndCRC(writer, "TimeZoneUTC", timezoneUTC);
        addRequiredAttributeAndCRC(writer, "TimeZoneVersion", timezoneVersion);
        addRequiredAttributeAndCRC(writer, "HaConfigured", haConfigured);
        addRequiredAttributeAndCRC(writer, "IsMetrocluster", isMetrocluster);
        addRequiredAttributeAndCRC(writer, "SslFipsEnabled", sslFipsEnabled);
        addRequiredAttributeAndCRC(writer, "MtConfigurationState", getMtConfigurationState());
        addRequiredAttributeAndCRC(writer, "MtConfigurationStateRaw", getMtConfigurationStateRaw());
        addRequiredAttributeAndCRC(writer, "MtMode", getMtMode());
        addRequiredAttributeAndCRC(writer, "MtModeRaw", getMtModeRaw());
        addRequiredAttributeAndCRC(writer, "MtDrPartnerClusterUUID", mtDrPartnerClusterUUID);
        addRequiredAttributeAndCRC(writer, "MtRemoteConfigurationState", getMtRemoteConfigurationState());
        addRequiredAttributeAndCRC(writer, "MtRemoteConfigurationStateRaw", getMtRemoteConfigurationStateRaw());
        addRequiredAttributeAndCRC(writer, "MtRemoteMode", getMtRemoteMode());
        addRequiredAttributeAndCRC(writer, "MtRemoteModeRaw", getMtRemoteModeRaw());
        addRequiredAttributeAndCRC(writer, "MtConfigurationType", mtConfigurationType);
    }

    protected boolean hasChildren() {
        return true;
    }

    @SuppressWarnings("unchecked")
    protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        addAllChildrenUnderNodeAndCRC(cache, writer, "DoNotDeleteList", doNotDeleteList.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "NetworkIpSpaces", networkIpSpaces.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "NetworkPortBroadcastDomains", networkPortBroadcastDomains.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "NetworkSubnets", networkSubnets.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "Nodes", nodes.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "StorageShelves", storageShelves.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "StorageSwitches", storageSwitches.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "StoragePools", storagePools.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "Disks", disks.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "DiskAggregateRelationships", diskAggregateRelationships.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "Licenses", licenses.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "HAPairs", haPairs.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "JobSchedules", jobSchedules.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "SnapshotPolicies", snapshotPolicies.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "QosPolicyGroups", qosPolicyGroups.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "Vservers", vServers.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "NetworkFailoverGroups", networkFailoverGroups.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "QosPolicyGroupVserverRelationships", qosPolicyGroupVServerRelationships.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "SnapMirrorPolicies", snapMirrorPolicies.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "SnapMirrors", snapMirrors.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "SnapMirrorHistories", snapMirrorHistories.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "SnapMirrorSelfRelationships", snapMirrorSelfRelationships.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "DiskPaths", diskPaths.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "NetworkPortRelationships", portRelationships.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "QosVolumeWorkloads", qosVolumeWorkloads.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "QosWorkloadDetails", qosWorkloadDetails.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "QosWorkloadNodeRelationships", qosWorkloadNodeRelationships.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "VserverAggregateRelationships", vServerAggregates.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "SnapshotPolicyVserverRelationships", snapshotPolicyVServerRelationships.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "ApplicationRecords", applicationRecords.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "SystemHealthAlerts", systemHealthAlerts.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "LunImports", lunImports.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "NetworkFailoverGroupNetworkPortRelationships", networkFailoverGroupNetworkPortRelationships.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "MetroClusterOperations", metroClusterOperations.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "ClusterPeers", clusterPeers.values(), tabs);
        addAllChildrenUnderNodeAndCRC(cache, writer, "VServerPeers", vServerPeers.values(), tabs);
        // DO NOT MARSHALL ISCSI SERVICES UNTIL SERVER WANTS THEM (MAYBE NOT EVER)
        // They are here for APP.
    }
}
