package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("disk")
@OCIEResourceType("disk")
public class Disk extends BaseCRCXMLNode<Cluster> {

    public final static class DiskKey extends KeyBase<DiskKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "storage-disk-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-uid"
            )
            uid
        }

        public DiskKey(final String clusterUUID, final String diskUID) {
            super("disk",
                  new EnumMap<Field, String>(Field.class) {{
                        put(Field.cluster_uuid, clusterUUID);
                        put(Field.uid, diskUID);
                  }});
        }

        @Override
        @OCIEUUID(uuidDescr = "uid")
        public String getOcieUUID() {
            return getID(Field.uid);
        }
    }

    private final DiskKey key;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-name"
    )
    @OCIEValue(
        ocieColumn = "name"
    )
    private String name;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info."
    )
    @OCIEValue(
        ocieColumn = "raidGroupId"
    )
    private RaidGroup.RaidGroupKey raidGroupKey;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-raid-info.disk-shared-info.shared-storage-pool"
    )
    @OCIEValue(
        ocieColumn = "storagePoolId"
    )
    private StoragePool.StoragePoolKey storagePoolKey;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-ownership-info.home-node-name"
    )
    @OCIEValue(
        ocieColumn = "homeNodeId"
    )
    private Node.NodeKey homeNodeKey;

    @OCIEZapiSource(
            sourceZapi = "storage-disk-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-ownership-info.home-node-name"
    )
    @OCIEValue(
            ocieColumn = "homeNodeName"
    )
    private String homeNodeName;

    @OCIEZapiSource(
            sourceZapi = "storage-disk-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-ownership-info.dr-home-node-name"
    )
    @OCIEValue(
            ocieColumn = "drHomeNodeId"
    )

    private String drHomeNodeId;

    @OCIEZapiSource(
            sourceZapi = "storage-disk-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-ownership-info.dr-home-node-name"
    )
    @OCIEValue(
            ocieColumn = "drHomeNodeName"
    )
    private String drHomeNodeName;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-ownership-info.owner-node-name"
    )
    @OCIEValue(
        ocieColumn = "ownerNodeId"
    )
    private Node.NodeKey ownerNodeKey;

    @OCIEZapiSource(
            sourceZapi = "storage-disk-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-ownership-info.owner-node-name"
    )
    @OCIEValue(
            ocieColumn = "ownerNodeName"
    )
    private String ownerNodeName;


    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-raid-info.active-node-name"
    )
    @OCIEValue(
        ocieColumn = "activeNodeId"
    )
    private Node.NodeKey activeNodeKey;

    @OCIEZapiSource(
            sourceZapi = "storage-disk-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-raid-info.active-node-name"
    )
    @OCIEValue(
            ocieColumn = "activeNodeName"
    )
    private String activeNodeName;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-type"
    )
    @OCIEValue(
        ocieColumn = "diskTypeRaw"
    )
    private String diskTypeRaw;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-type"
    )
    @OCIEValue(
        ocieColumn = "diskType",
        notes = "derived enum"
    )
    private DiskType diskType;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-raid-info.effective-disk-type"
    )
    @OCIEValue(
        ocieColumn = "interfaceType",
        notes = "derived enum"
    )
    private DiskInterfaceType interfaceType;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-raid-info.effective-disk-type"
    )
    @OCIEValue(
        ocieColumn = "interfaceTypeRaw"
    )
    private String interfaceTypeRaw;

    @OCIEZapiSource(
            sourceZapi = "storage-disk-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-inventory-info.model"
    )
    @OCIEValue(
            ocieColumn = "model"
    )
    private String model;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-inventory-info.firmware-revision"
    )
    @OCIEValue(
        ocieColumn = "firmwareRevision"
    )
    private String firmwareRevision;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-inventory-info.rpm"
    )
    @OCIEValue(
        ocieColumn = "rpm"
    )
    private Long rpm;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-inventory-info.serial-number"
    )
    @OCIEValue(
        ocieColumn = "serialNumber"
    )
    private String serialNumber;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-inventory-info.shelf"
    )
    @OCIEValue(
        ocieColumn = "shelf"
    )
    private String shelf;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-inventory-info.shelf-bay"
    )
    @OCIEValue(
        ocieColumn = "shelfBay"
    )
    private String bay;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-inventory-info.vendor"
    )
    @OCIEValue(
        ocieColumn = "vendor"
    )
    private String vendor;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-raid-info.container-type"
    )
    @OCIEValue(
        ocieColumn = "containerType",
        notes = "derived enum"
    )
    private DiskContainerType containerType;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-raid-info.container-type"
    )
    @OCIEValue(
        ocieColumn = "containerTypeRaw"
    )
    private String containerTypeRaw;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info."
    )
    @OCIEValue(
        ocieColumn = "isFailed"
    )
    private Boolean isFailed;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-raid-info.physical-blocks"
    )
    @OCIEValue(
        ocieColumn = "physicalBlocks"
    )
    private Long physicalBlocks;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-raid-info.physical-blocks"
    )
    @OCIEValue(
        ocieColumn = "physicalBytes",
        notes = "derived physicalBlocks * 4096"
    )
    private Long physicalBytes;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-raid-info.used-blocks"
    )
    @OCIEValue(
        ocieColumn = "usedBlocks"
    )
    private Long usedBlocks;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-raid-info.used-blocks"
    )
    @OCIEValue(
        ocieColumn = "usedBytes",
        notes = "derived usedBlocks * 4096"
    )
    private Long usedBytes;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-raid-info.disk-aggregate-info.aggregate-name"
    )
    @OCIEValue(
        ocieColumn = "aggregateId",
        notes = "correlated from aggregate-name and mapped to ocie objid"
    )
    private Aggregate.AggregateKey aggregateKey;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-raid-info.position"
    )
    @OCIEValue(
        ocieColumn = "raidPosition"
    )
    private RaidPosition raidPosition;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-raid-info.position"
    )
    @OCIEValue(
        ocieColumn = "raidPositionRaw",
        notes = "derived enum"
    )
    private String raidPositionRaw;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-inventory-info.bytes-per-sector"
    )
    @OCIEValue(
        ocieColumn = "bytesPerSector"
    )
    private Long bytesPerSector;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-inventory-info.disk-type"
    )
    @OCIEValue(
        ocieColumn = "isVirtual",
        notes = "derived disk-type != lun"
    )
    private Boolean isVirtual;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-inventory-info.checksum-compatibility"
    )
    @OCIEValue(
        ocieColumn = "checksumCompatibility"
    )
    private String checksumCompatibility;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-raid-info.disk-aggregate-info.is-media-scrubbing"
    )
    @OCIEValue(
        ocieColumn = "isMediaScrubbing"
    )
    private Boolean isMediaScrubbing;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-raid-info.disk-aggregate-info.is-offline"
    )
    @OCIEValue(
        ocieColumn = "isOffline"
    )
    private Boolean isOffline;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-raid-info.disk-aggregate-info.is-prefailed"
    )
    @OCIEValue(
        ocieColumn = "isPreFailed"
    )
    private Boolean isPreFailed;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-raid-info.disk-aggregate-info.is-reconstructing"
    )
    @OCIEValue(
        ocieColumn = "isReconstructing"
    )
    private Boolean isReconstructing;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-raid-info.disk-aggregate-info.is-replacing"
    )
    @OCIEValue(
        ocieColumn = "isReplacing"
    )
    private Boolean isReplacing;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-inventory-info.is-shared"
    )
    @OCIEValue(
        ocieColumn = "isShared"
    )
    private Boolean isShared;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-raid-info.disk-aggregate-info.is-zeroed"
    )
    @OCIEValue(
        ocieColumn = "isZeroed",
        notes = "for aggregate and spare disks"
    )
    private Boolean isZeroed;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-raid-info.disk-aggregate-info.is-zeroing"
    )
    @OCIEValue(
        ocieColumn = "isZeroing",
        notes = "for aggregate and spare disks"
    )
    private Boolean isZeroing;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-ownership-info.pool"
    )
    @OCIEValue(
        ocieColumn = "pool"
    )
    private Long poolID;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-raid-info.disk-spare-info.zeroing-percent-complete"
    )
    @OCIEValue(
        ocieColumn = "zeroingPercent"
    )
    private Integer zeroingPercentComplete;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-inventory-info.capacity-sectors"
    )
    @OCIEValue(
        ocieColumn = "capacitySectors"
    )
    private Long capacitySectors;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-inventory-info.right-size-sectors"
    )
    @OCIEValue(
        ocieColumn = "rightSizeSectors"
    )
    private Long rightSizeSectors;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-raid-info.disk-outage-info.is-in-fdr"
    )
    @OCIEValue(
        ocieColumn = "isInFdr"
    )
    private Boolean isInFDR;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-outage-info.reason"
    )
    @OCIEValue(
        ocieColumn = "failedReason"
    )
    private String failedReason;

    @OCIEZapiSource(
    	sourceZapi = "aggr-spare-get-iter",
    	zapiResponsePath = "netapp.results.attributes-list.aggr-spare-info.nodes.aggr-node-spare-info.disks.aggr-spare-disk-info[].local-usable-data-size"
    )
    @OCIEValue(
    	ocieColumn = "usableDataSize"
    )
    private Long usableDataSize;

    @OCIEZapiSource(
        sourceZapi = "aggr-spare-get-iter",
    	zapiResponsePath = "netapp.results.attributes-list.aggr-spare-info.nodes.aggr-node-spare-info.disks.aggr-spare-disk-info[].local-usable-root-size"
    )
    @OCIEValue(
    	ocieColumn = "usableRootSize"
    )
    private Long usableRootSize;
    
    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-metrocluster-info.is-local-attach"
    )
    @OCIEValue(
        ocieColumn = "isLocalAttach"
    )
    private Boolean isLocalAttach;

    @OCIEZapiSource(
            sourceZapi = "storage-disk-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-inventory-info.storage-ssd-info.percent-rated-life-used"
    )
    @OCIEValue(
            ocieColumn = "percentRatedLifeUsed"
    )
    private Integer percentRatedLifeUsed;

    @OCIEZapiSource(
            sourceZapi = "storage-disk-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-inventory-info.storage-ssd-info.percent-spares-consumed"
    )
    @OCIEValue(
            ocieColumn = "percentSparesConsumed"
    )
    private Integer percentSparesConsumed;


    @OCIEZapiSource(
            sourceZapi = "storage-disk-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-raid-info.disk-shared-info.partitioning-type"
    )
    @OCIEValue(
            ocieColumn = "partitioningType"
    )
    private PartitioningType partitioningType;


    @OCIEZapiSource(
            sourceZapi = "storage-disk-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-raid-info.disk-shared-info.partitioning-type"
    )
    @OCIEValue(
            ocieColumn = "partitioningTypeRaw"
    )
    private String partitioningTypeRaw;

    Disk(final Cluster parentCluster, final String diskUUID) {
        super(parentCluster, Disk.class.getSimpleName());
        this.key = new DiskKey(parentCluster.getClusterUUID(), diskUUID);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public DiskKey getKey() {
        return key;
    }

    public Aggregate.AggregateKey getAggregateKey() {
        return aggregateKey;
    }

    public String getName() {
        return name;
    }

    public RaidGroup.RaidGroupKey getRaidGroupKey() {
        return raidGroupKey;
    }

    public StoragePool.StoragePoolKey getStoragePoolKey() {
        return storagePoolKey;
    }

    public DiskInterfaceType getInterfaceType() {
        return interfaceType;
    }

    public String getModel() {
        return model;
    }

    public String getFirmwareRevision() {
        return firmwareRevision;
    }

    public Long getRpm() {
        return rpm;
    }

    public String getSerialNumber() {
        return serialNumber;
    }

    public String getShelf() {
        return shelf;
    }

    public String getBay() {
        return bay;
    }

    public String getVendor() {
        return vendor;
    }

    public DiskContainerType getContainerType() {
        return containerType;
    }

    public Boolean getFailed() {
        return isFailed;
    }

    public Long getPhysicalBlocks() {
        return physicalBlocks;
    }

    public Long getUsedBlocks() {
        return usedBlocks;
    }

    public RaidPosition getRaidPosition() {
        return raidPosition;
    }

    public Boolean isVirtual() {
        return isVirtual;
    }

    public void setName(final String name) {
        this.name = name;
    }

    public void setRaidGroupKey(final RaidGroup.RaidGroupKey raidGroupKey) {
        this.raidGroupKey = raidGroupKey;
    }

    public void setStoragePoolKey(StoragePool.StoragePoolKey storagePoolKey) {
        this.storagePoolKey = storagePoolKey;
    }

    public void setInterfaceType(final DiskInterfaceType interfaceType) {
        this.interfaceType = interfaceType;
    }

    public void setModel(final String model) {
        this.model = model;
    }

    public void setFirmwareRevision(final String firmwareRevision) {
        this.firmwareRevision = firmwareRevision;
    }

    public void setRpm(final Long rpm) {
        this.rpm = rpm;
    }

    public void setSerialNumber(final String serialNumber) {
        this.serialNumber = serialNumber;
    }

    public void setShelf(final String shelf) {
        this.shelf = shelf;
    }

    public void setBay(final String bay) {
        this.bay = bay;
    }

    public void setVendor(final String vendor) {
        this.vendor = vendor;
    }

    public void setContainerType(final DiskContainerType containerType) {
        this.containerType = containerType;
    }

    public void setFailed(final Boolean failed) {
        isFailed = failed;
    }

    public void setPhysicalBlocks(final Long physicalBlocks) {
        this.physicalBlocks = physicalBlocks;
    }

    public void setUsedBlocks(final Long usedBlocks) {
        this.usedBlocks = usedBlocks;
    }

    public void setAggregateKey(final Aggregate.AggregateKey aggregateKey) {
        this.aggregateKey = aggregateKey;
    }

    public void setRaidPosition(final RaidPosition raidPosition) {
        this.raidPosition = raidPosition;
    }

    public void setBytesPerSector(final Long bytesPerSector) {
        this.bytesPerSector = bytesPerSector;
    }

    public void setVirtual(final Boolean virtual) {
        isVirtual = virtual;
    }

    public String getContainerTypeRaw() {
        return containerTypeRaw;
    }

    public void setContainerTypeRaw(final String containerTypeRaw) {
        this.containerTypeRaw = containerTypeRaw;
    }

    public String getInterfaceTypeRaw() {
        return interfaceTypeRaw;
    }

    public void setInterfaceTypeRaw(final String interfaceTypeRaw) {
        this.interfaceTypeRaw = interfaceTypeRaw;
    }

    public String getRaidPositionRaw() {
        return raidPositionRaw;
    }

    public void setRaidPositionRaw(final String raidPositionRaw) {
        this.raidPositionRaw = raidPositionRaw;
    }

    public String getChecksumCompatibility() {
        return checksumCompatibility;
    }

    public void setChecksumCompatibility(final String checksumCompatibility) {
        this.checksumCompatibility = checksumCompatibility;
    }

    public Boolean isMediaScrubbing() {
        return isMediaScrubbing;
    }

    public void setMediaScrubbing(final Boolean mediaScrubbing) {
        isMediaScrubbing = mediaScrubbing;
    }

    public Boolean isOffline() {
        return isOffline;
    }

    public void setOffline(final Boolean offline) {
        isOffline = offline;
    }

    public Boolean isPreFailed() {
        return isPreFailed;
    }

    public void setPreFailed(final Boolean preFailed) {
        isPreFailed = preFailed;
    }

    public Boolean isReconstructing() {
        return isReconstructing;
    }

    public void setReconstructing(final Boolean reconstructing) {
        isReconstructing = reconstructing;
    }

    public Boolean isReplacing() {
        return isReplacing;
    }

    public void setReplacing(final Boolean replacing) {
        isReplacing = replacing;
    }

    public Boolean getShared() {
        return isShared;
    }

    public void setShared(Boolean shared) {
        isShared = shared;
    }

    public Boolean isZeroed() {
        return isZeroed;
    }

    public void setZeroed(final Boolean zeroed) {
        isZeroed = zeroed;
    }

    public Boolean isZeroing() {
        return isZeroing;
    }

    public void setZeroing(final Boolean zeroing) {
        this.isZeroing = zeroing;
    }

    public Integer getZeroingPercentComplete() {
        return zeroingPercentComplete;
    }

    public void setZeroingPercentComplete(final Integer zeroingPercentComplete) {
        this.zeroingPercentComplete = zeroingPercentComplete;
    }

    public Long getPoolID() {
        return poolID;
    }

    public void setPoolID(final Long poolID) {
        this.poolID = poolID;
    }

    public Boolean isFailed() {
        return isFailed;
    }

    public Long getBytesPerSector() {
        return bytesPerSector;
    }

    public Long getUsedBytes() {
        return usedBytes;
    }

    public Long getPhysicalBytes() {
        return physicalBytes;
    }

    public void setPhysicalBytes(final Long physicalBytes) {
        this.physicalBytes = physicalBytes;
    }

    public void setUsedBytes(final Long usedBytes) {
        this.usedBytes = usedBytes;
    }

    public Node.NodeKey getOwnerNodeKey() {
        return ownerNodeKey;
    }

    public void setOwnerNodeKey(final Node.NodeKey ownerNodeKey) {
        this.ownerNodeKey = ownerNodeKey;
    }

    public String getOwnerNodeName() {
        return ownerNodeName;
    }

    public void setOwnerNodeName(final String ownerNodeName) {
        this.ownerNodeName = ownerNodeName;
    }

    public Node.NodeKey getHomeNodeKey() {
        return homeNodeKey;
    }

    public void setHomeNodeKey(final Node.NodeKey homeNodeKey) {
        this.homeNodeKey = homeNodeKey;
    }

    public String getHomeNodeName() {
        return homeNodeName;
    }

    public void setHomeNodeName(final String homeNodeName) {
        this.homeNodeName = homeNodeName;
    }

    public Node.NodeKey getActiveNodeKey() {
        return activeNodeKey;
    }

    public void setActiveNodeKey(final Node.NodeKey activeNodeKey) {
        this.activeNodeKey = activeNodeKey;
    }

    public String getActiveNodeName() {
        return activeNodeName;
    }

    public void setActiveNodeName(final String activeNodeName) {
        this.activeNodeName = activeNodeName;
    }

    public String getDiskTypeRaw() {
        return diskTypeRaw;
    }

    public void setDiskTypeRaw(final String diskTypeRaw) {
        this.diskTypeRaw = diskTypeRaw;
    }

    public DiskType getDiskType() {
        return diskType;
    }

    public void setDiskType(final DiskType diskType) {
        this.diskType = diskType;
    }

    public Long getCapacitySectors() {
        return capacitySectors;
    }

    public void setCapacitySectors(final Long capacitySectors) {
        this.capacitySectors = capacitySectors;
    }

    public Long getRightSizeSectors() {
        return rightSizeSectors;
    }

    public void setRightSizeSectors(final Long rightSizeSectors) {
        this.rightSizeSectors = rightSizeSectors;
    }

    public Boolean isInFDR() {
        return isInFDR;
    }

    public void setInFDR(final Boolean inFDR) {
        isInFDR = inFDR;
    }

    public String getFailedReason() {
        return failedReason;
    }

    public void setFailedReason(final String failedReason) {
        this.failedReason = failedReason;
    }

    public Long getUsableDataSize() {
		return usableDataSize;
	}

	public void setUsableDataSize(final Long usableDataSize) {
		this.usableDataSize = usableDataSize;
	}

	public Long getUsableRootSize() {
		return usableRootSize;
	}

	public void setUsableRootSize(final Long usableRootSize) {
		this.usableRootSize = usableRootSize;
	}

    public Boolean getLocalAttach() {
        return isLocalAttach;
    }

    public void setIsLocalAttach(Boolean isLocalAttach) {
        this.isLocalAttach = isLocalAttach;
    }

    public Integer getPercentSparesConsumed() {
        return percentSparesConsumed;
    }

    public void setPercentSparesConsumed(Integer percentSparesConsumed) {
        this.percentSparesConsumed = percentSparesConsumed;
    }

    public Integer getPercentRatedLifeUsed() { return percentRatedLifeUsed; }

    public void setPercentRatedLifeUsed(Integer percentRatedLifeUsed) {
        this.percentRatedLifeUsed = percentRatedLifeUsed;
    }

    public PartitioningType getPartitioningType() {
        return partitioningType;
    }

    public void setPartitioningType(PartitioningType partitioningType) {
        this.partitioningType = partitioningType;
    }

    public String getPartitioningTypeRaw() {
        return partitioningTypeRaw;
    }

    public void setPartitioningTypeRaw(String partitioningTypeRaw) {
        this.partitioningTypeRaw = partitioningTypeRaw;
    }

    public String getDrHomeNodeName() {
        return drHomeNodeName;
    }

    public void setDrHomeNodeName(String drHomeNodeName) {
        this.drHomeNodeName = drHomeNodeName;
    }

    public String getDrHomeNodeId() {
        return drHomeNodeId;
    }

    public void setDrHomeNodeId(String drHomeNodeId) {
        this.drHomeNodeId = drHomeNodeId;
    }

    public static enum DiskType {
        NOT_MAPPED,
        ATA,
        BSAS,
        EATA,
        FCAL,
        LUN,
        SAS,
        SATA,
        SCSI,
        SSD,
        XATA,
        XSAS,
        FSAS,
        VMDISK,
        UNKNOWN, }

    public static enum DiskInterfaceType {
        NOT_MAPPED,
        ATA,
        BSAS,
        EATA,
        FCAL,
        LUN,
        MSATA,
        SAS,
        SATA,
        SCSI,
        SSD,
        XATA,
        XSAS,
        FSAS,
        VMDISK,
        UNKNOWN, }

    public static enum DiskContainerType {
        NOT_MAPPED,
        AGGREGATE,
        BROKEN,
        FOREIGN,
        LABELMAINT,
        MAINTAINANCE,
        SPARE,
        SHARED,
        UNASSIGNED,
        VOLUME,
        UNSUPPORTED,
        UNKNOWN, }

    public static enum RaidPosition {
        NOT_MAPPED,
        COPY,
        DATA,
        DPARITY,
        ORPHAN,
        PARITY,
        PENDING,
        PRESENT,
        SHARED,}

    public static enum PartitioningType {
        NOT_MAPPED,
        ROOT_DATA,
        STORAGE_POOL,}

    @Override
	protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Uid", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Name", name);
        addRequiredAttributeAndCRC(writer, "AggregateUUID", KeyBase.getOcieUUIDOrNull(aggregateKey));
        addRequiredAttributeAndCRC(writer, "RaidGroupUUID", KeyBase.getOcieUUIDOrNull(raidGroupKey));
        addRequiredAttributeAndCRC(writer, "StoragePoolUUID", KeyBase.getOcieUUIDOrNull(storagePoolKey));
        addRequiredAttributeAndCRC(writer, "OwnerNodeUUID", KeyBase.getOcieUUIDOrNull(ownerNodeKey));
        addRequiredAttributeAndCRC(writer, "HomeNodeUUID", KeyBase.getOcieUUIDOrNull(homeNodeKey));
        addRequiredAttributeAndCRC(writer, "ActiveNodeUUID", KeyBase.getOcieUUIDOrNull(activeNodeKey));
        addRequiredAttributeAndCRC(writer, "InterfaceType", diskType);
        addRequiredAttributeAndCRC(writer, "InterfaceTypeRaw", diskTypeRaw);
        addRequiredAttributeAndCRC(writer, "EffectiveInterfaceType", interfaceType);
        addRequiredAttributeAndCRC(writer, "EffectiveInterfaceTypeRaw", interfaceTypeRaw);
        addRequiredAttributeAndCRC(writer, "ShelfBay", bay);
        addRequiredAttributeAndCRC(writer, "Vendor", vendor);
        addRequiredAttributeAndCRC(writer, "ContainerType", containerType);
        addRequiredAttributeAndCRC(writer, "ContainerTypeRaw", containerTypeRaw);
        addRequiredAttributeAndCRC(writer, "IsFailed", isFailed);
        addRequiredAttributeAndCRC(writer, "PhysicalBlocks", physicalBlocks);
        addRequiredAttributeAndCRC(writer, "UsedBlocks", usedBlocks);
        addRequiredAttributeAndCRC(writer, "RaidPosition", raidPosition);
        addRequiredAttributeAndCRC(writer, "RaidPositionRaw", raidPositionRaw);
        addRequiredAttributeAndCRC(writer, "BytesPerSector", bytesPerSector);
        addRequiredAttributeAndCRC(writer, "ChecksumCompatibility", checksumCompatibility);
        addRequiredAttributeAndCRC(writer, "IsMediaScrubbing", isMediaScrubbing);
        addRequiredAttributeAndCRC(writer, "IsOffline", isOffline);
        addRequiredAttributeAndCRC(writer, "IsPrefailed", isPreFailed);
        addRequiredAttributeAndCRC(writer, "IsReconstructing", isReconstructing);
        addRequiredAttributeAndCRC(writer, "IsReplacing", isReplacing);
        addRequiredAttributeAndCRC(writer, "IsShared", isShared);
        addRequiredAttributeAndCRC(writer, "IsVirtual", isVirtual);
        addRequiredAttributeAndCRC(writer, "IsZeroing", isZeroing);
        addRequiredAttributeAndCRC(writer, "IsZeroed", isZeroed);
        addRequiredAttributeAndCRC(writer, "Pool", poolID);
        addRequiredAttributeAndCRC(writer, "TotalBytes", physicalBytes);
        addRequiredAttributeAndCRC(writer, "UsedBytes", usedBytes);
        addRequiredAttributeAndCRC(writer, "ZeroingPercent", zeroingPercentComplete);
        addRequiredAttributeAndCRC(writer, "FirmwareRevision", firmwareRevision);
        addRequiredAttributeAndCRC(writer, "Model", model);
        addRequiredAttributeAndCRC(writer, "Rpm", rpm);
        addRequiredAttributeAndCRC(writer, "Shelf", shelf);
        addRequiredAttributeAndCRC(writer, "SerialNumber", serialNumber);
        addRequiredAttributeAndCRC(writer, "FailedReason", failedReason);
        addRequiredAttributeAndCRC(writer, "CapacitySectors", capacitySectors);
        addRequiredAttributeAndCRC(writer, "RightSizeSectors", rightSizeSectors);
        addRequiredAttributeAndCRC(writer, "IsInFdr", isInFDR);
        addRequiredAttributeAndCRC(writer, "UsableDataSize", usableDataSize);
        addRequiredAttributeAndCRC(writer, "UsableRootSize", usableRootSize);
        addRequiredAttributeAndCRC(writer, "HomeNodeName", homeNodeName);
        addRequiredAttributeAndCRC(writer, "OwnerNodeName", ownerNodeName);
        addRequiredAttributeAndCRC(writer, "ActiveNodeName", activeNodeName);
        addRequiredAttributeAndCRC(writer, "IsLocalAttach", isLocalAttach);
        addRequiredAttributeAndCRC(writer, "PercentSparesConsumed", percentSparesConsumed);
        addRequiredAttributeAndCRC(writer, "PercentRatedLifeUsed", percentRatedLifeUsed);
        addRequiredAttributeAndCRC(writer, "PartitioningType", partitioningType);
        addRequiredAttributeAndCRC(writer, "PartitioningTypeRaw", partitioningTypeRaw);
        addRequiredAttributeAndCRC(writer, "DrHomeNodeName", drHomeNodeName);
        addRequiredAttributeAndCRC(writer, "DrHomeNodeId", drHomeNodeId);
    }
}
