package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("disk_path")
@OCIEResourceType("disk_path")
public class DiskPath extends BaseCRCXMLNode<Cluster> {

    public final static class DiskPathKey extends KeyBase<DiskPathKey.Field> {

        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "storage-disk-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-uid"
            )
            disk_uuid,

            @OCIEZapiSource(
                sourceZapi = "storage-disk-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-paths.disk-path-info.initiator-port"
            )
            initiator_port_name,

            @OCIEZapiSource(
                sourceZapi = "storage-disk-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-paths.disk-path-info.target-wwpn"
            )
            target_wwpn
        }

        private final String nodeUUID;

        public DiskPathKey(final String clusterUUID, final String nodeUUID, final String diskUUID, final String initiatorPortName, final String targetWWPN) {
            super("disk_path", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.disk_uuid, diskUUID);
                put(Field.initiator_port_name, initiatorPortName);
                put(Field.target_wwpn, targetWWPN);
            }});
            this.nodeUUID = nodeUUID;
        }

        @Override
        @OCIEUUID(uuidDescr = "disk_uuid.node_uuid.initiator_port_name.target_wwpn")
        public String getOcieUUID() {
            /**
             * UUID of the Disk Path equivalent to "{diskUUID}.{initiatorName}.{targetWWN}"
             */
            return getID(Field.disk_uuid) + '.' + nodeUUID + '.' + getID(Field.initiator_port_name) + '.' + getID(Field.target_wwpn);
        }
    }

    private final DiskPathKey key;

    @OCIEZapiSource(
            sourceZapi = "storage-disk-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-paths.disk-path-info.array-name"
    )
    @OCIEValue(
            ocieColumn = "initiatorPortId"
    )
    private final StorageAdapter.AdapterKey adapterKey;

    @OCIEZapiSource(
            sourceZapi = "storage-disk-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-paths.disk-path-info.node"
    )
    @OCIEValue(
            ocieColumn = "nodeId"
    )
    private Node.NodeKey nodeKey;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-paths.disk-path-info.initiator-port"
    )
    @OCIEValue(
        ocieColumn = "arrayName"
    )
    private String arrayName;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-paths.disk-path-info.disk-port"
    )
    @OCIEValue(
        ocieColumn = "diskPort"
    )
    private String diskPort;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-paths.disk-path-info.disk-port-name"
    )
    @OCIEValue(
        ocieColumn = "diskPortName"
    )
    private String diskPortName;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-paths.disk-path-info.initiator-side-switch-port"
    )
    @OCIEValue(
        ocieColumn = "initiatorSideSwitchPort"
    )
    private String initiatorSideSwitchPort;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-paths.disk-path-info.lun-number"
    )
    @OCIEValue(
        ocieColumn = "lun"
    )
    private Integer lun;

    @OCIEZapiSource(
        sourceZapi = "storage-disk-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-paths.disk-path-info.lun-path-use-state"
    )
    @OCIEValue(
        ocieColumn = "lunPathUseState"
    )
    private LUNPathUseStateType lunPathUseState;

    @OCIEZapiSource(
            sourceZapi = "storage-disk-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-paths.disk-path-info.lun-path-use-state"
    )
    @OCIEValue(
            ocieColumn = "lunPathUseStateRaw"
    )
    private String lunPathUseStateRaw;

    @OCIEZapiSource(
            sourceZapi = "storage-disk-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-paths.disk-path-info.target-side-switch-port"
    )
    @OCIEValue(
            ocieColumn = "targetSideSwitchPort"
    )
    private String targetSideSwitchPort;

    @OCIEZapiSource(
            sourceZapi = "storage-disk-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-paths.disk-path-info.target-side-switch-port"
    )
    @OCIEValue(
            ocieColumn = "tpgn"
    )
    private Long targetPortGroupNum;
    
    @OCIEZapiSource(
            sourceZapi = "storage-disk-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.storage-disk-info.disk-paths.disk-path-info.target-wwnn"
    )
    @OCIEValue(
            ocieColumn = "targetWWNN"
    )
    private String targetWWNN;

    DiskPath(final Cluster parentCluster, final String targetWWPN, final Disk.DiskKey diskKey, final StorageAdapter.AdapterKey adapterKey) {
        super(parentCluster, DiskPath.class.getSimpleName());
        this.adapterKey = adapterKey;
        final String clusterUUID = parentCluster.getClusterUUID();
        final String inititatorPortName = adapterKey.getID(StorageAdapter.AdapterKey.Field.adapter_name);
        // Previous release incorrectly include node UUID in initiator port name.
        // Be wrong but backwards compatible for now.
        final String nodeUUID = adapterKey.getID(StorageAdapter.AdapterKey.Field.node_uuid);
        this.key = new DiskPathKey(clusterUUID, nodeUUID, diskKey.getOcieUUID(), inititatorPortName, targetWWPN);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public DiskPathKey getKey() {
        return key;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getDiskUUID() {
        return key.getID(DiskPathKey.Field.disk_uuid);
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Node.NodeKey getNodeKey() {
        return nodeKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getArrayName() {
        return arrayName;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getDiskPort() {
        return diskPort;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getDiskPortName() {
        return diskPortName;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getInitiatorPortUUID() {
        return (adapterKey.isKnown())
                ? adapterKey.getOcieUUID()
                : null;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getInitiatorSideSwitchPort() {
        return initiatorSideSwitchPort;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getLun() {
        return lun;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public LUNPathUseStateType getLunPathUseState() {
        return lunPathUseState;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getLunPathUseStateRaw() {
        return lunPathUseStateRaw;
    }

    public void setLunPathUseStateRaw(final String lunPathUseStateRaw) {
        this.lunPathUseStateRaw = lunPathUseStateRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getTargetWWPN() {
        return key.getID(DiskPathKey.Field.target_wwpn);
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getTargetSideSwitchPort() {
        return targetSideSwitchPort;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getTargetPortGroupNum() {
        return targetPortGroupNum;
    }
    
    @SuppressWarnings({"UnusedDeclaration"})
    public String getTargetWWNN() {
        return targetWWNN;
    }

    public void setNodeKey(final Node.NodeKey nodeKey) {
        this.nodeKey = nodeKey;
    }

    public void setArrayName(final String arrayName) {
        this.arrayName = arrayName;
    }

    public void setDiskPort(final String diskPort) {
        this.diskPort = diskPort;
    }

    public void setDiskPortName(final String diskPortName) {
        this.diskPortName = diskPortName;
    }

    public void setInitiatorSideSwitchPort(final String initiatorSideSwitchPort) {
        this.initiatorSideSwitchPort = initiatorSideSwitchPort;
    }

    public void setLun(final Integer lun) {
        this.lun = lun;
    }

    public void setLunPathUseState(final LUNPathUseStateType lunPathUseState) {
        this.lunPathUseState = lunPathUseState;
    }

    public void setTargetSideSwitchPort(final String targetSideSwitchPort) {
        this.targetSideSwitchPort = targetSideSwitchPort;
    }

    public void setTargetPortGroupNum(final Long targetPortGroupNum) {
        this.targetPortGroupNum = targetPortGroupNum;
    }
    
    public void setTargetWWNN(final String targetWWNN) {
        this.targetWWNN = targetWWNN;
    }

    @SuppressWarnings("UnusedDeclaration")
    public static enum LUNPathUseStateType {
        NOT_MAPPED,
        INU,   /* In Use */
        RDY,   /* Ready */
        ERR,   /* High Error */
        QNG   /* Quiesce State */
    }


    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "DiskUUID", key.getID(DiskPathKey.Field.disk_uuid));
        addRequiredAttributeAndCRC(writer, "NodeUUID", nodeKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "ArrayName", arrayName);
        addRequiredAttributeAndCRC(writer, "DiskPort", diskPort);
        addRequiredAttributeAndCRC(writer, "DiskPortName", diskPortName);
        addRequiredAttributeAndCRC(writer, "InitiatorPortUUID", getInitiatorPortUUID());
        addRequiredAttributeAndCRC(writer, "InitiatorSideSwitchPort", initiatorSideSwitchPort);
        addRequiredAttributeAndCRC(writer, "LUN", lun);
        addRequiredAttributeAndCRC(writer, "LUNPathUseState", lunPathUseState);
        addRequiredAttributeAndCRC(writer, "LUNPathUseStateRaw", lunPathUseStateRaw);
        addRequiredAttributeAndCRC(writer, "TargetWWPN", key.getID(DiskPathKey.Field.target_wwpn));
        addRequiredAttributeAndCRC(writer, "TargetSideSwitchPort", targetSideSwitchPort);
        addRequiredAttributeAndCRC(writer, "TPGN", targetPortGroupNum);
        addRequiredAttributeAndCRC(writer, "TargetWWNN", targetWWNN);
    }
}
