package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("igroup")
@OCIEResourceType("igroup")
public class IGroup extends BaseCRCXMLNode<VServer> {


    public final static class IGroupKey extends KeyBase<IGroupKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "igroup-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.initiator-group-info.initiator-group-uuid"
            )
            uuid
        }

        public IGroupKey(final String clusterUUID, final String iGroupUUID) {
            super("igroup", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.uuid, iGroupUUID);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "uuid")
        public String getOcieUUID() {
            return getID(Field.uuid);
        }
    }

    private final IGroupKey key;

    @OCIEZapiSource(
        sourceZapi = "igroup-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.initiator-group-info.initiator-group-name"
    )
    @OCIEValue(
        ocieColumn = "name"
    )
    private String name;

    @OCIEZapiSource(
        sourceZapi = "igroup-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.initiator-group-info.initiator-group-os-type"
    )
    @OCIEValue(
        ocieColumn = "osType"
    )
    private String osType;

    @OCIEZapiSource(
        sourceZapi = "igroup-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.initiator-group-info.initiator-group-type"
    )
    @OCIEValue(
        ocieColumn = "type"
    )
    private Type type;

    @OCIEZapiSource(
        sourceZapi = "igroup-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.initiator-group-info.initiator-group-type"
    )
    @OCIEValue(
        ocieColumn = "typeRaw"
    )
    private String typeRaw;

    /**
     * Boolean value to indicate if this initiator group has ALUA (Asymmetric Logical Unit Access) features enabled for
     * LUNs mapped to this initiator group.
     */
    @OCIEZapiSource(
        sourceZapi = "igroup-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.initiator-group-info.initiator-group-alua-enabled"
    )
    @OCIEValue(
        ocieColumn = "aluaEnabled"
    )
    private Boolean aluaEnabled;

    /**
     * Boolean value to indicate if this initiator group has Volume Set Addressing (VSA) enabled or disabled.
     */
    @OCIEZapiSource(
        sourceZapi = "igroup-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.initiator-group-info.initiator-group-vsa-enabled"
    )
    @OCIEValue(
        ocieColumn = "vsaEnabled"
    )
    private Boolean vsaEnabled;

    /**
     * Boolean value to indicate if this initiator group is configured for its luns to require the use of host
     * multi-pathing software for correct high-availability failover operation. In Data ONTAP 7-Mode, this value is
     * optional and is only returned for FCP initiator groups on an storage system in an HA pair. In Data ONTAP
     * Cluster-Mode, this field will always be <tt>true</tt>.
     */
    @OCIEZapiSource(
        sourceZapi = "igroup-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.initiator-group-info.initiator-group-use-partner"
    )
    @OCIEValue(
        ocieColumn = "usePartner"
    )
    private Boolean usePartner;

    private final IGroupInitiators iGroupInitiators;

    IGroup(final VServer parent, final String uuid) {
        super(parent, "Igroup");
        this.key = new IGroupKey(parent.getKey().getID(VServer.VServerKey.Field.cluster_uuid), uuid);
        this.iGroupInitiators = new IGroupInitiators(this);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public IGroupKey getKey() {
        return key;
    }

    public String getName() {
        return name;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getOsType() {
        return osType;
    }

    public Type getType() {
        return type;
    }

    public void setName(final String name) {
        this.name = name;
    }

    public void setOsType(final String osType) {
        this.osType = osType;
    }

    public void setType(final Type type) {
        this.type = type;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getTypeRaw() {
        return typeRaw;
    }

    public void setTypeRaw(final String typeRaw) {
        this.typeRaw = typeRaw;
    }

    public Boolean getAluaEnabled() {
        return aluaEnabled;
    }

    public void setAluaEnabled(final Boolean aluaEnabled) {
        this.aluaEnabled = aluaEnabled;
    }

    public Boolean getVsaEnabled() {
        return vsaEnabled;
    }

    public void setVsaEnabled(final Boolean vsaEnabled) {
        this.vsaEnabled = vsaEnabled;
    }

    public Boolean getUsePartner() {
        return usePartner;
    }

    public void setUsePartner(final Boolean usePartner) {
        this.usePartner = usePartner;
    }

    public static enum Type {
        NOT_MAPPED,
        ISCSI,
        FCP,
        MIXED, }

    public IGroupInitiator createIGroupInitiator(final String initiatorName) {
        return new IGroupInitiator(this, getParent().getUUID(), key.getOcieUUID(), initiatorName);
    }

    public IGroupInitiators getIGroupInitiators() {
        return iGroupInitiators;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Name", name);
        addRequiredAttributeAndCRC(writer, "OSType", osType);
        addRequiredAttributeAndCRC(writer, "Type", type);
        addRequiredAttributeAndCRC(writer, "TypeRaw", typeRaw);
        addRequiredAttributeAndCRC(writer, "AluaEnabled", aluaEnabled);
        addRequiredAttributeAndCRC(writer, "VsaEnabled", vsaEnabled);
        addRequiredAttributeAndCRC(writer, "UsePartner", usePartner);
    }

    protected boolean hasChildren() {
        return !iGroupInitiators.isEmpty();
    }

    @SuppressWarnings("unchecked")
    @Override
    protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        addAllChildrenUnderNodeAndCRC(cache, writer, "IgroupInitiators", iGroupInitiators.values(), tabs);
    }
}
