package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("iscsi_portal_group")
@OCIEResourceType("iscsi_portal_group")
public class ISCSIPortalGroup extends BaseCRCXMLNode<VServer> {

    public static final class ISCSIPortalGroupKey extends KeyBase<ISCSIPortalGroupKey.Field> {

        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "vserver-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.vserver-info.uuid"
            )
            vserver_uuid,

            @OCIEZapiSource(
                sourceZapi = "iscsi-tpgroup-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.iscsi-tpgroup-list-entry-info.tpgroup-name"
            )
            @OCIEValue(
                ocieColumn = "name"
            )
            portal_group_name
        }

        public ISCSIPortalGroupKey(final String clusterUUID, final String vServerUUID, final String portalGroupName) {
            super("iscsi_portal_group", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.vserver_uuid, vServerUUID);
                put(Field.portal_group_name, portalGroupName);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "vserver_uuid.portal_group_name")
        public String getOcieUUID() {
            final String clusterUUID = getID(Field.cluster_uuid);
            final String vServerUUID = getID(Field.vserver_uuid);
            final String fqVServerUUID = VServer.VServerKey.getOCIEVServerUUID(clusterUUID, vServerUUID);
            final String portalGroupName = getID(Field.portal_group_name);
            return fqVServerUUID + '.' + portalGroupName;
        }
    }

    private final ISCSIPortalGroupKey key;
    private final ISCSIPortalGroupToLIFs iscsiPortalGroupToLIFs;
    private final ISCSISessions iscsiSessions;

    @OCIEZapiSource(
        sourceZapi = "iscsi-tpgroup-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.iscsi-tpgroup-list-entry-info.tpgroup-tag"
    )
    @OCIEValue(
        ocieColumn = "tag"
    )
    private Integer tag;

    ISCSIPortalGroup(final VServer parent, final String name) {
        super(parent, ISCSIPortalGroup.class.getSimpleName());
        final String clusterUUID = parent.getKey().getID(VServer.VServerKey.Field.cluster_uuid);
        final String vServerUUID = parent.getKey().getID(VServer.VServerKey.Field.uuid);
        this.key = new ISCSIPortalGroupKey(clusterUUID, vServerUUID, name);
        this.iscsiPortalGroupToLIFs = new ISCSIPortalGroupToLIFs(this);
        this.iscsiSessions = new ISCSISessions(this);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public ISCSIPortalGroupKey getKey() {
        return key;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getName() {
        return key.getID(ISCSIPortalGroupKey.Field.portal_group_name);
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getTag() {
        return tag;
    }

    public void setTag(final Integer tag) {
        this.tag = tag;
    }

    public ISCSIPortalGroupToLIFs getISCSIPortalGroupToLIFs() {
        return iscsiPortalGroupToLIFs;
    }

    public ISCSIPortalGroupToLIF createISCSIPortalGroupToLIF(final String lifUUID) {
        return new ISCSIPortalGroupToLIF(this, lifUUID);
    }

    public ISCSISessions getISCSISessions() {
        return iscsiSessions;
    }

    public ISCSISession createISCSISession(final String clusterUUID, final VServer.VServerKey vServerKey, final String portalGroupName, final Integer targetSessionID) {
        return new ISCSISession(this, clusterUUID, vServerKey, portalGroupName, targetSessionID);
    }

    @Override
    protected boolean hasChildren() {
        return !iscsiSessions.isEmpty() || !iscsiPortalGroupToLIFs.isEmpty();
    }

    @Override
    protected void marshallChildrenToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        super.marshallChildrenToFileAndCRC(marshallingCache, writer, tabs);
        addAllChildrenUnderNodeAndCRC(marshallingCache, writer, "ISCSIPortalGroupLIFs", iscsiPortalGroupToLIFs.values(), tabs);
        addAllChildrenUnderNodeAndCRC(marshallingCache, writer, "ISCSISessions", iscsiSessions.values(), tabs);
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Name", key.getID(ISCSIPortalGroupKey.Field.portal_group_name));
        addRequiredAttributeAndCRC(writer, "Tag", tag);
    }
}
