package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEUUID;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEZapiSource;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

// JSD TODO remove, all attributes are in vSErver
public class ISCSIService extends BaseCRCXMLNode<Cluster> {

    public final static class ISCSIServiceKey extends KeyBase<ISCSIServiceKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "iscsi-service-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.iscsi-service-info.node-name"
            )
            iqn
        }

        public ISCSIServiceKey(final String clusterUUID, final String iqn) {
            super("iscsi_service", new EnumMap<Field, String>(Field.class){{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.iqn, iqn);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "iqn")
        public String getOcieUUID() {
            return getID(Field.iqn);
        }
    }

    // JSD TODO: remove this class, all attributes are in vServer.
    private final ISCSIServiceKey serviceKey;

    @OCIEZapiSource(
        sourceZapi = "iscsi-service-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.iscsi-service-info.vserver"
    )
    private String vServerName;

    @OCIEZapiSource(
        sourceZapi = "iscsi-service-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.iscsi-service-info.alias-name"
    )
    private String aliasName;

    @OCIEZapiSource(
        sourceZapi = "iscsi-service-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.iscsi-service-info.is-available"
    )
    private boolean isAvailable;

    ISCSIService(final Cluster cluster, final String iqn) {
        super(cluster, ISCSIService.class.getSimpleName());
        final String clusterUUID = cluster.getClusterUUID();
        this.serviceKey = new ISCSIServiceKey(clusterUUID, iqn);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public ISCSIServiceKey getKey() {
        return serviceKey;
    }

    public String getVServerName() {
        return vServerName;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getAliasName() {
        return aliasName;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public boolean isAvailable() {
        return isAvailable;
    }

    public void setVServerName(final String vServerName) {
        this.vServerName = vServerName;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public void setAliasName(final String aliasName) {
        this.aliasName = aliasName;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public void setAvailable(final boolean available) {
        isAvailable = available;
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
//        addRequiredAttributeAndCRC(writer, "ResourceKey", serviceKey);
        addRequiredAttributeAndCRC(writer, "UUID", serviceKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "IQN", serviceKey.getID(ISCSIServiceKey.Field.iqn));
        addRequiredAttributeAndCRC(writer, "VServerName", vServerName);
        addRequiredAttributeAndCRC(writer, "Alias", aliasName);
        addRequiredAttributeAndCRC(writer, "IsAvailable", isAvailable);
    }
}
