package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("iscsi_session")
@OCIEResourceType("iscsi_session")
public class ISCSISessionConnection extends BaseCRCXMLNode<ISCSISession> {

    public static final class ISCSISessionConnectionKey extends KeyBase<ISCSISessionConnectionKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "vserver-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.vserver-info.uuid"
            )
            vserver_uuid,

            @OCIEZapiSource(
                sourceZapi = "iscsi-tpgroup-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.iscsi-tpgroup-list-entry-info.tpgroup-name"
            )
            portal_group_name,

            @OCIEZapiSource(
                sourceZapi = "iscsi-session-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.iscsi-session-list-entry-info.target-session-id"
            )
            target_session_id,

            @OCIEZapiSource(
                sourceZapi = "iscsi-session-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.iscsi-session-list-entry-info.iscsi-session-connection-list-entries.iscsi-session-connection-list-entry-info.connection-id"
            )
            connection_id
        }

        public ISCSISessionConnectionKey(final String clusterUUID, final String vServerUUID, final String portalGroupName, final String iscsiSessionUUID, final String connectionID) {
            super("iscsi_session_connection", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.vserver_uuid, vServerUUID);
                put(Field.portal_group_name, portalGroupName);
                put(Field.target_session_id, iscsiSessionUUID);
                put(Field.connection_id, connectionID);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "cluster_uuid.vserver_uuid.portal_group_name.target_session_id.connection_id")
        public String getOcieUUID() {
            final String clusterUUID = getID(Field.cluster_uuid);
            final String vServerUUID = getID(Field.vserver_uuid);
            final String fqVServerUUID = VServer.VServerKey.getOCIEVServerUUID(clusterUUID, vServerUUID);
            final String portalGroupName = getID(Field.portal_group_name);
            final String targetSessionID = getID(Field.target_session_id);
            final String connectionID = getID(Field.connection_id);
            return clusterUUID + '.' + fqVServerUUID + '.' + portalGroupName + '.' + targetSessionID + '.' + connectionID;
        }
    }

    private final ISCSISessionConnectionKey key;

    @OCIEZapiSource(
        sourceZapi = "iscsi-session-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.iscsi-session-list-entry-info.iscsi-session-connection-list-entries.iscsi-session-connection-list-entry-info.interface-name"
    )
    @OCIEValue(
        ocieColumn = "lifId"
    )
    private Lif.LifKey lifKey;

    @OCIEZapiSource(
        sourceZapi = "iscsi-session-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.iscsi-session-list-entry-info.iscsi-session-connection-list-entries.iscsi-session-connection-list-entry-info.local-ip-address"
    )
    @OCIEValue(
        ocieColumn = "localIpAddress"
    )
    private String localIPAddress;

    @OCIEZapiSource(
        sourceZapi = "iscsi-session-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.iscsi-session-list-entry-info.iscsi-session-connection-list-entries.iscsi-session-connection-list-entry-info.local-ip-port"
    )
    @OCIEValue(
        ocieColumn = "localIpPort"
    )
    private String localIPPort;

    @OCIEZapiSource(
        sourceZapi = "iscsi-session-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.iscsi-session-list-entry-info.iscsi-session-connection-list-entries.iscsi-session-connection-list-entry-info.remote-ip-address"
    )
    @OCIEValue(
        ocieColumn = "remoteIpAddress"
    )
    private String remoteIPAddress;

    @OCIEZapiSource(
        sourceZapi = "iscsi-session-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.iscsi-session-list-entry-info.iscsi-session-connection-list-entries.iscsi-session-connection-list-entry-info.authentication-method"
    )
    @OCIEValue(
        ocieColumn = "authenticationStyle"
    )
    private ISCSISessionAuthenticationStyle authenticationStyle;

    ISCSISessionConnection(final ISCSISession parent, final Integer connectionID) {
        super(parent, ISCSISessionConnection.class.getSimpleName());
        final ISCSISession.ISCSISessionKey parentKey = parent.getKey();
        final String clusterUUID = parentKey.getID(ISCSISession.ISCSISessionKey.Field.cluster_uuid);
        final String vServerUUID = parentKey.getID(ISCSISession.ISCSISessionKey.Field.vserver_uuid);
        final String portalGroupName = parentKey.getID(ISCSISession.ISCSISessionKey.Field.portal_group_name);
        final String targetSessionID = parentKey.getID(ISCSISession.ISCSISessionKey.Field.target_session_id);
        this.key = new ISCSISessionConnectionKey(clusterUUID, vServerUUID, portalGroupName, targetSessionID, String.valueOf(connectionID));
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public ISCSISessionConnectionKey getKey() {
        return key;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Lif.LifKey getLifKey() {
        return lifKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getConnectionID() {
        return key.getID(ISCSISessionConnectionKey.Field.connection_id);
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getLocalIPAddress() {
        return localIPAddress;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getLocalIPPort() {
        return localIPPort;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getRemoteIPAddress() {
        return remoteIPAddress;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public ISCSISessionAuthenticationStyle getAuthenticationStyle() {
        return authenticationStyle;
    }

    public void setLifKey(final Lif.LifKey lifKey) {
        this.lifKey = lifKey;
    }

    public void setLocalIPAddress(final String localIPAddress) {
        this.localIPAddress = localIPAddress;
    }

    public void setLocalIPPort(final String localIPPort) {
        this.localIPPort = localIPPort;
    }

    public void setRemoteIPAddress(final String remoteIPAddress) {
        this.remoteIPAddress = remoteIPAddress;
    }

    public void setAuthenticationStyle(final ISCSISessionAuthenticationStyle authenticationStyle) {
        this.authenticationStyle = authenticationStyle;
    }

    @SuppressWarnings({"UnusedDeclaration", "unchecked"})
    public enum ISCSISessionAuthenticationStyle {
        NOT_MAPPED,
        CHAP,
        NONE
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "LIFUUID", lifKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "ConnectionId", getConnectionID());
        addRequiredAttributeAndCRC(writer, "LocalIPAddress", localIPAddress);
        addRequiredAttributeAndCRC(writer, "LocalIPPort", localIPPort);
        addRequiredAttributeAndCRC(writer, "RemoteIPAddress", remoteIPAddress);
        addRequiredAttributeAndCRC(writer, "AuthenticationStyle", authenticationStyle);
    }
}
