package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;


@OCIETable("lun")
@OCIEResourceType("lun")
public class LUN extends BaseCRCXMLNode<Volume> {

    public static final String EMPTY_ATTRIBUTE_VALUE = "";

    public final static class LUNKey extends KeyBase<LUNKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "lun-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.lun-info.uuid"
            )
            uuid
        }

        public LUNKey(final String clusterUUID, final String uuid) {
            super("lun", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.uuid, uuid);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "uuid")
        public String getOcieUUID() {
            return getID(Field.uuid);
        }
    }

    private final LUNKey key;                                   // uuid

    @OCIEZapiSource(
        sourceZapi = "lun-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-info.alignment"
    )
    @OCIEValue(
        ocieColumn = "alignment"
    )
    private Alignment alignment;                                // alignment

    @OCIEZapiSource(
        sourceZapi = "lun-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-info.alignment"
    )
    @OCIEValue(
        ocieColumn = "alignmentRaw"
    )
    private String alignmentRaw;                                // alignment

    @OCIEZapiSource(
        sourceZapi = "lun-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-info.is-space-alloc-enabled"
    )
    @OCIEValue(
        ocieColumn = "isSpaceAllocEnabled"
    )
    private Boolean isSpaceAllocEnabled;                        // is-space-alloc-enabled

    @OCIEZapiSource(
        sourceZapi = "lun-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-info.is-space-reservation-enabled"
    )
    @OCIEValue(
        ocieColumn = "isSpaceReservationEnabled"
    )
    private Boolean isSpaceReservationEnabled;                  // is-space-reservation-enabled

    @OCIEZapiSource(
        sourceZapi = "lun-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-info.mapped"
    )
    @OCIEValue(
        ocieColumn = "isMapped"
    )
    private Boolean isMapped;                                   // mapped

    @OCIEZapiSource(
        sourceZapi = "lun-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-info.multiprotocol-type"
    )
    @OCIEValue(
        ocieColumn = "multiprotocolType"
    )
    private String multiprotocolType;                           // multiprotocol-type

    @OCIEZapiSource(
        sourceZapi = "lun-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-info.online"
    )
    @OCIEValue(
        ocieColumn = "online"
    )
    private Boolean online;                                     // online

    @OCIEZapiSource(
        sourceZapi = "lun-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-info.path"
    )
    @OCIEValue(
        ocieColumn = "path"
    )
    private String path;                                        // path

    @OCIEZapiSource(
        sourceZapi = "lun-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-info.prefix-size"
    )
    @OCIEValue(
        ocieColumn = "prefixSize"
    )
    private Long prefixSize;                                    // prefix-size

    @OCIEZapiSource(
        sourceZapi = "lun-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-info.suffix-size"
    )
    @OCIEValue(
        ocieColumn = "suffixSize"
    )
    private Long suffixSize;                                    // suffix-size

    @OCIEZapiSource(
        sourceZapi = "lun-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-info.qtree"
    )
    @OCIEValue(
        ocieColumn = "qtree"
    )
    private QTree.QTreeKey qTreeKey;                            // qtree

    @OCIEZapiSource(
        sourceZapi = "lun-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-info.serial-number"
    )
    @OCIEValue(
        ocieColumn = "serialNumber"
    )
    private String serialNumber;                                // serial-number

    @OCIEZapiSource(
        sourceZapi = "lun-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-info.size"
    )
    @OCIEValue(
        ocieColumn = "size"
    )
    private Long size;                                          // size

    @OCIEZapiSource(
        sourceZapi = "lun-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-info.size-used"
    )
    @OCIEValue(
        ocieColumn = "sizeUsed"
    )
    private Long sizeUsed;                                      // size-used

    @OCIEZapiSource(
        sourceZapi = "lun-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-info.comment"
    )
    @OCIEValue(
        ocieColumn = "comment"
    )
    private String comment;                                     // comment

    @OCIEZapiSource(
        sourceZapi = "lun-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-info.qos-policy-name"
    )
    @OCIEValue(
        ocieColumn = "qosPolicyGroupId"
    )
    private QosPolicyGroup.QosPolicyGroupKey qosPolicyGroupKey; // qos-policy-name

    @OCIEZapiSource(
        sourceZapi = "lun-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-info.class"
    )
    @OCIEValue(
        ocieColumn = "lunClass"
    )
    private LUNClass lunClass;                                  // class

    @OCIEZapiSource(
        sourceZapi = "lun-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.lun-info.class"
    )
    @OCIEValue(
        ocieColumn = "lunClassRaw"
    )
    private String lunClassRaw;                                 // class

    LUN(final Volume parent, final String uuID) {
        super(parent, LUN.class.getSimpleName());
        this.key = new LUNKey(parent.getKey().getID(Volume.VolumeKey.Field.cluster_uuid), uuID);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public LUNKey getKey() {
        return key;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Alignment getAlignment() {
        return alignment;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean getSpaceAllocEnabled() {
        return isSpaceAllocEnabled;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean getSpaceReservationEnabled() {
        return isSpaceReservationEnabled;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean getMapped() {
        return isMapped;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getMultiprotocolType() {
        return multiprotocolType;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean getOnline() {
        return online;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getPath() {
        return path;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getPrefixSize() {
        return prefixSize;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getSuffixSize() {
        return suffixSize;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public QTree.QTreeKey getqTreeKey() {
        return qTreeKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getSerialNumber() {
        return serialNumber;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getSize() {
        return size;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getSizeUsed() {
        return sizeUsed;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getComment() {
        return comment;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public QosPolicyGroup.QosPolicyGroupKey getQosPolicyKeyUUID() {
        return qosPolicyGroupKey;
    }

    public void setQosPolicyGroupKey(final QosPolicyGroup.QosPolicyGroupKey qosPolicyGroupKey) {
        this.qosPolicyGroupKey = qosPolicyGroupKey;
    }

    public void setAlignment(final Alignment alignment) {
        this.alignment = alignment;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getAlignmentRaw() {
        return alignmentRaw;
    }

    public void setAlignmentRaw(final String alignmentRaw) {
        this.alignmentRaw = alignmentRaw;
    }

    public void setSpaceAllocEnabled(final Boolean spaceAllocEnabled) {
        isSpaceAllocEnabled = spaceAllocEnabled;
    }

    public void setSpaceReservationEnabled(final Boolean spaceReservationEnabled) {
        isSpaceReservationEnabled = spaceReservationEnabled;
    }

    public void setMapped(final Boolean mapped) {
        isMapped = mapped;
    }

    public void setMultiprotocolType(final String multiprotocolType) {
        this.multiprotocolType = multiprotocolType;
    }

    public void setOnline(final Boolean online) {
        this.online = online;
    }

    public void setPath(final String path) {
        this.path = path;
    }

    public void setPrefixSize(final Long prefixSize) {
        this.prefixSize = prefixSize;
    }

    public void setSuffixSize(final Long suffixSize) {
        this.suffixSize = suffixSize;
    }

    public void setQTreeKey(final QTree.QTreeKey qTreeUUID) {
        this.qTreeKey = qTreeUUID;
    }

    public void setSerialNumber(final String serialNumber) {
        this.serialNumber = serialNumber;
    }

    public void setSize(final Long size) {
        this.size = size;
    }

    public void setSizeUsed(final Long sizeUsed) {
        this.sizeUsed = sizeUsed;
    }

    public void setComment(final String comment) {
        this.comment = comment;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public LUNClass getLunClass() {
        return lunClass;
    }

    public void setLunClass(final LUNClass lunClass) {
        this.lunClass = lunClass;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getLunClassRaw() {
        return lunClassRaw;
    }

    public void setLunClassRaw(final String lunClassRaw) {
        this.lunClassRaw = lunClassRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum LUNClass {
        NOT_MAPPED,
        REGULAR,
        PROTCOL_ENDPOINT,
        VVOL
    }


    @SuppressWarnings({"UnusedDeclaration"})
    public static enum Alignment {
        NOT_MAPPED,
        ALIGNED,
        MISALIGNED,
        POSSIBLY_MISALIGNED,
        INDETERMINATE, }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "LUNClass", (lunClass == null) ? null : lunClass);
        addRequiredAttributeAndCRC(writer, "LUNClassRaw", lunClassRaw);
        addRequiredAttributeAndCRC(writer, "QtreeUUID", (qTreeKey != null ? qTreeKey.getOcieUUID() : EMPTY_ATTRIBUTE_VALUE));
        addRequiredAttributeAndCRC(writer, "QosPolicyGroupUUID", KeyBase.getOcieUUIDOrNull(qosPolicyGroupKey));
        addRequiredAttributeAndCRC(writer, "Path", path);
        addRequiredAttributeAndCRC(writer, "IsOnline", online);
        addRequiredAttributeAndCRC(writer, "Size", size);
        addRequiredAttributeAndCRC(writer, "SizeUsed", sizeUsed);
        addRequiredAttributeAndCRC(writer, "IsSpaceReservationEnabled", isSpaceReservationEnabled);
        addRequiredAttributeAndCRC(writer, "IsSpaceAllocEnabled", isSpaceAllocEnabled);
        addRequiredAttributeAndCRC(writer, "IsMapped", isMapped);
        addRequiredAttributeAndCRC(writer, "SerialNumber", serialNumber);
        addRequiredAttributeAndCRC(writer, "MultiProtocolType", multiprotocolType);
        addRequiredAttributeAndCRC(writer, "Alignment", alignment);
        addRequiredAttributeAndCRC(writer, "AlignmentRaw", alignmentRaw);
        addRequiredAttributeAndCRC(writer, "PrefixSize", prefixSize);
        addRequiredAttributeAndCRC(writer, "SuffixSize", suffixSize);
        addRequiredAttributeAndCRC(writer, "Comment", comment);
    }
}
