package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEUUID;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEZapiSource;
import com.onaro.sanscreen.acquisition.schema.KeyBase;

import java.util.EnumMap;

/**
 * Common enums and key for FCP and Network (ip) LIF
 */
public abstract class Lif {

    public static final class LifKey extends KeyBase<LifKey.Field> {
        final String lifType;
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

//            @OCIEZapiSource(
//                sourceZapi = "vserver-get-iter",
//                zapiResponsePath = "netapp.results.attributes-list.vserver-info.uuid"
//            )
//            vserver_uuid,

            @OCIEZapiSource(
                sourceZapi = "net-interface-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.net-interface-info.lif-uuid"
            )
            lif_uuid
        }


        // JSD TODO why have vServer here if it is not used?????
        // Generated Wikidoc will be wrong.
        public LifKey(final String lifType, final String clusterUUID, final String vServerUUID, final String lifUUID) {
            super(lifType, new EnumMap<Field, String>(Field.class){{
                put(Field.cluster_uuid, clusterUUID);
                // put(Field.vserver_uuid, vServerUUID);
                put(Field.lif_uuid, lifUUID);
            }});
            this.lifType = lifType;
        }

        @Override
        @OCIEUUID(uuidDescr = "lif_uuid")
        public String getOcieUUID() {
            return getID(Field.lif_uuid);
        }

        @Override
        public String getResourceKey() {
            return getID(Field.cluster_uuid) + ":type=" + lifType + ",lif_uuid=" + getID(Field.lif_uuid);
        }
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum FailoverPolicy {
        NOT_MAPPED,
        DISABLED,
        NEXTAVAIL,
        PRIORITY
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum Role {
        NOT_MAPPED,
        UNDEF,
        CLUSTER,
        DATA,
        NODE_MGMT,
        INTERCLUSTER,
        CLUSTER_MGMT
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum Status {
        NOT_MAPPED,
        UNKNOWN,
        UP,
        DOWN, 
    }
}
