package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import java.io.IOException;
import java.util.EnumMap;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEResourceType;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIETable;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEUUID;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEValue;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEZapiSource;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

@OCIETable("metrocluster_operation")
@OCIEResourceType("metrocluster_operation")
public class MetroClusterOperation extends BaseCRCXMLNode<Cluster> {
    public final static class MetroClusterOperationKey extends KeyBase<MetroClusterOperationKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "metrocluster-operation-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.metrocluster-operation-info.operation-uuid"
            )
            uuid
        }

        public MetroClusterOperationKey(final String clusterUUID, final String diskUID) {
            super("metroclusterOperation",
                  new EnumMap<Field, String>(Field.class) {{
                        put(Field.cluster_uuid, clusterUUID);
                        put(Field.uuid, diskUID);
                  }});
        }

        @Override
        @OCIEUUID(uuidDescr = "uuid")
        public String getOcieUUID() {
            return getID(Field.uuid);
        }
    }

    private final MetroClusterOperationKey key;
    
    @OCIEZapiSource(
        sourceZapi = "metrocluster-operation-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.metrocluster-operation-info.node-name"
    )
    @OCIEValue(
        ocieColumn = "nodeId"
    )
    private Node.NodeKey nodeKey;
    
    @OCIEZapiSource(
        sourceZapi = "metrocluster-operation-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.metrocluster-operation-info.start-time"
    )
    @OCIEValue(
        ocieColumn = "startTime"
    )
    private Long startTime;
    
    @OCIEZapiSource(
        sourceZapi = "metrocluster-operation-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.metrocluster-operation-info.end-time"
    )
    @OCIEValue(
        ocieColumn = "endTime"
    )
    private Long endTime;
    
    @OCIEZapiSource(
        sourceZapi = "metrocluster-operation-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.metrocluster-operation-info.error-list"
    )
    @OCIEValue(
        ocieColumn = "errorList"
    )
    private String errors;
    
    @OCIEZapiSource(
        sourceZapi = "metrocluster-operation-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.metrocluster-operation-info.job-id"
    )
    @OCIEValue(
            ocieColumn = "jobId"
    )
    private Long jobId;
    
    @OCIEZapiSource(
        sourceZapi = "metrocluster-operation-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.metrocluster-operation-info.operation"
    )
    @OCIEValue(
        ocieColumn = "operation"
    )
    private Operation operation;
    
    @OCIEZapiSource(
        sourceZapi = "metrocluster-operation-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.metrocluster-operation-info.operation"
    )
    @OCIEValue(
        ocieColumn = "operationRaw"
    )
     private String operationRaw;
    
    @OCIEZapiSource(
        sourceZapi = "metrocluster-operation-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.metrocluster-operation-info.state"
    )
    @OCIEValue(
        ocieColumn = "state"
    )
    private State state;
    
    @OCIEZapiSource(
        sourceZapi = "metrocluster-operation-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.metrocluster-operation-info.state"
    )
    @OCIEValue(
        ocieColumn = "stateRaw"
    )
    private String stateRaw;
    
    MetroClusterOperation(final Cluster parent, final String uuid) {
        super(parent, MetroClusterOperation.class.getSimpleName());
        this.key = new MetroClusterOperationKey(parent.getClusterUUID(), uuid);
    }
    
    @Override
    @SuppressWarnings({"unchecked"})
    public MetroClusterOperationKey getKey() {
        return key;
    }
    
    public Node.NodeKey getNodeKey() {
        return nodeKey;
    }
    
    public void setNodeKey(Node.NodeKey nodeKey) {
        this.nodeKey = nodeKey;
    }
    
    public Long getStartTime() {
        return startTime;
    }
    
    public void setStartTime(Long startTime) {
        this.startTime = startTime;
    }
    
    public Long getEndTime() {
        return endTime;
    }
    
    public void setEndTime(Long endTime) {
        this.endTime = endTime;
    }
    
    public String getErrors() {
        return errors;
    }
    
    public void setErrors(String errors) {
        this.errors = errors;
    }
    
    public Long getJobId() {
        return jobId;
    }
    
    public void setJobId(Long jobId) {
        this.jobId = jobId;
    }
    
    public Operation getOperation() {
        return operation;
    }

    public void setOperation(Operation operation) {
        this.operation = operation;
    }
    
    public String getOperationRaw() {
        return operationRaw;
    }

    public void setOperationRaw(String operationRaw) {
        this.operationRaw = operationRaw;
    }
    
    public State getState() {
        return state;
    }
    
    public void setState(State state) {
        this.state = state;
    }
    
    public String getStateRaw() {
        return stateRaw;
    }
    
    public void setStateRaw(String stateRaw) {
        this.stateRaw = stateRaw;
    }

    public static enum Operation {
        NOT_MAPPED,
        UNKNOWN, 
        CONFIGURE, 
        MODIFY, 
        SWITCHOVER, 
        HEAL_AGGREGATES, 
        HEAL_ROOT_AGGREGATES, 
        SWITCHBACK, 
        SWITCHBACK_CONTINUATION_AGENT, 
        UNCONFIGURE, 
        UNCONFIGURE_CONTINUATION_AGENT, 
        SWITCHOVER_SIMULATE, 
        SWITCHBACK_SIMULATE, 
        CHECK,
    }
    
    public static enum State {
        NOT_MAPPED,
        UNKNOWN, 
        IN_PROGRESS, 
        PARTIALLY_SUCCESSFUL, 
        COMPLETED_WITH_MANUAL_RECOVERY_NEEDED, 
        COMPLETED_WITH_WARNINGS, 
        SUCCESSFUL, 
        FAILED,
    }
    
    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "NodeUUID", KeyBase.getOcieUUIDOrNull(nodeKey));
        addRequiredAttributeAndCRC(writer, "StartTime", startTime);
        addRequiredAttributeAndCRC(writer, "EndTime", endTime);       
        addRequiredAttributeAndCRC(writer, "ErrorList", errors);
        addRequiredAttributeAndCRC(writer, "JobId", jobId);
        addRequiredAttributeAndCRC(writer, "Operation", operation);
        addRequiredAttributeAndCRC(writer, "OperationRaw", operationRaw);
        addRequiredAttributeAndCRC(writer, "State", state);
        addRequiredAttributeAndCRC(writer, "StateRaw", stateRaw);               
    } 
}
