package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.*;

import java.io.IOException;
import java.util.EnumMap;

/**
 * A failover group is a list of physical ports. A LIF can subscribe to a failover group, and get automatically
 * configured with a list of failover rules for each physical port in the failover group.
 */
@OCIETable("network_failover_group")
@OCIEResourceType("network_failover_group")
public class NetworkFailoverGroup extends BaseCRCXMLNode<Cluster> {

    public final static class NetworkFailoverGroupKey extends KeyBase<NetworkFailoverGroupKey.Field> {

        public static enum Field {
            @OCIEZapiSource(
                    sourceZapi = "cluster-identity-get",
                    zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                    sourceZapi = "net-failover-group-get-iter",
                    zapiResponsePath = "netapp.results.attributes-list.net-failover-group-info.failover-group"
            )
            failover_group_name,
        }

        public NetworkFailoverGroupKey(final String clusterUUID, final String failoverGroupName) {
            super("network_failover_group", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.failover_group_name, failoverGroupName);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "cluster_uuid.failover_group_name")
        public String getOcieUUID() {
            return getFQID();
        }

    }

    @OCIEZapiSource(
            sourceZapi = "net-failover-group-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.net-failover-group-info.failover-group"
    )
    @OCIEValue(
            ocieColumn = "name"
    )
    private String failoverGroupName;

    @OCIEZapiSource(
            sourceZapi = "net-failover-group-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.net-failover-group-info.broadcast-domain"
    )
    @OCIEValue(
            ocieColumn = "broadcastDomainId"
    )
    private NetworkPortBroadcastDomain.NetworkPortBroadcastDomainKey broadcastDomainKey;

    @OCIEZapiSource(
            sourceZapi = "net-failover-group-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.net-failover-group-info.vserver"
    )
    @OCIEValue(
            ocieColumn = "vserverId"
    )
    private VServer.VServerKey vserverId;

    public VServer.VServerKey getVserverId() {
        return vserverId;
    }

    public void setVserverId(VServer.VServerKey vserverId) {
        this.vserverId = vserverId;
    }

    private final NetworkFailoverGroupKey failoverGroupKey;

    public NetworkFailoverGroup(final Cluster parent, final String failoverGroupName) {
        super(parent, "NetworkFailoverGroup");
        failoverGroupKey = new NetworkFailoverGroupKey(parent.getClusterUUID(), failoverGroupName);
        this.failoverGroupName = failoverGroupName;
    }

    @Override
    public NetworkFailoverGroupKey getKey() {
        return failoverGroupKey;
    }

    public void setBroadcastDomainKey(NetworkPortBroadcastDomain.NetworkPortBroadcastDomainKey broadcastDomainKey) {
        this.broadcastDomainKey = broadcastDomainKey;
    }

    public NetworkPortBroadcastDomain.NetworkPortBroadcastDomainKey getBroadcastDomainKey() {
        return broadcastDomainKey;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", failoverGroupKey);
        addRequiredAttributeAndCRC(writer, "UUID", failoverGroupKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Name", failoverGroupName);
        if(broadcastDomainKey != null) {
            addRequiredAttributeAndCRC(writer, "NetworkPortBroadcastDomainUUID", broadcastDomainKey.getOcieUUID());
        }

        if( vserverId != null )
            addRequiredAttributeAndCRC(writer, "VserverUUID", vserverId.getOcieUUID());
    }
}
