package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("plex")
@OCIEResourceType("plex")
public class Plex extends BaseCRCXMLNode<Aggregate> {

    public final static class PlexKey extends KeyBase<PlexKey.Field> {

        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "aggr-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggregate-uuid"
            )
            aggregate_uuid,

            @OCIEZapiSource(
                sourceZapi = "aggr-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-raid-attributes.plexes.plex-attributes.plex-name"
            )
            plex_name
        }

        public PlexKey(final String clusterUUID, final String aggrUUID, final String plexName) {
            super("plex", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.aggregate_uuid, aggrUUID);
                put(Field.plex_name, plexName);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "aggregate_uuid.plex_name")
        public String getOcieUUID() {
            return getID(Field.aggregate_uuid) + '.' + getID(Field.plex_name);
        }
    }

    private final PlexKey key;

    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-raid-attributes.plexes.plex-attributes.is-online"
    )
    @OCIEValue(
        ocieColumn = "isOnline"
    )
    private final Boolean isOnline;

    @OCIEZapiSource(
            sourceZapi = "aggr-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-raid-attributes.plexes.plex-attributes.plex-status"
    )
    @OCIEValue(
            ocieColumn = "status"
    )
    private PlexStatus status;

    @OCIEZapiSource(
            sourceZapi = "aggr-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-raid-attributes.plexes.plex-attributes.plex-status"
    )
    @OCIEValue(
            ocieColumn = "statusRaw"
    )
    private String statusRaw;


    @OCIEZapiSource(
            sourceZapi = "aggr-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-raid-attributes.plexes.plex-attributes.resyncing-percentage"
    )
    @OCIEValue(
            ocieColumn = "resyncPercent"
    )
    private Integer resyncPercent;

    // Children
    private final RaidGroups raidGroups;

    /**
     *  Plex status. See
     * http://http://web.netapp.com/engineering/design-depot/appliance-mgmt/zephyr/zapidoc.cache/DOT/mainN/web_ontap_cluster/aggr/index.html#plex-attributes
     */
    public static enum PlexStatus {
        NOT_MAPPED,
        NORMAL,
        FAILED,
        EMPTY,
        INVALID,
        UNINITIALIZED,
        FAILED_ASSIMILATION,
        LIMBO,
        ACTIVE,
        INACTIVE,
        RESYNCING,
    }

    Plex(final Aggregate parent, final String aggrUUID, final String name, final Boolean isOnline) {
        super(parent, "Plex");
        final Aggregate.AggregateKey aggregateKey = parent.getKey();
        final String clusterUUID = aggregateKey.getID(Aggregate.AggregateKey.Field.cluster_uuid);
        this.key = new PlexKey(clusterUUID, aggrUUID, name);
        this.isOnline = isOnline;
        this.raidGroups = new RaidGroups(this);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public PlexKey getKey() {
        return key;
    }

    public RaidGroup createRaidGroup(final String raidGroupName) {
        return new RaidGroup(this, raidGroupName);
    }

    public RaidGroups getRaidGroups() {
        return raidGroups;
    }

    @SuppressWarnings({"unused declaration"})
    public Boolean getOnline() {
        return isOnline;
    }

    public PlexStatus getStatus() {
        return status;
    }

    public void setStatus(PlexStatus status) {
        this.status = status;
    }

    public String getStatusRaw() {
        return statusRaw;
    }

    public void setStatusRaw(String statusRaw) {
        this.statusRaw = statusRaw;
    }

    public Integer getResyncPercent() { return resyncPercent; }

    public void setResyncPercent(Integer resyncPercent) { this.resyncPercent = resyncPercent; }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Name", key.getID(PlexKey.Field.plex_name));
        addRequiredAttributeAndCRC(writer, "IsOnline", isOnline);
        addRequiredAttributeAndCRC(writer, "Status", getStatus());
        addRequiredAttributeAndCRC(writer, "StatusRaw", getStatusRaw());
        addRequiredAttributeAndCRC(writer, "ResyncPercent", getResyncPercent());
    }

    @Override
    protected boolean hasChildren() {
        return !raidGroups.isEmpty();
    }

    @Override
    protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        addAllChildrenUnderNodeAndCRC(cache, writer, RaidGroups.class.getSimpleName(), raidGroups.values(), tabs);
    }
}
