package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("qos_policy_group")
@OCIEResourceType("qos_policy_group")
public class QosPolicyGroup extends BaseCRCXMLNode<Cluster> {

    public final static class QosPolicyGroupKey extends KeyBase<QosPolicyGroupKey.Field> {

        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "qos-policy-group-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.qos-policy-group-info.uuid"
            )
            qos_policy_group_uuid
        }

        public QosPolicyGroupKey(final String clusterUUID, final String qosPolicyGroupUUID) {
            super("qos_policy_group", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.qos_policy_group_uuid, qosPolicyGroupUUID);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "qos_policy_group_uuid")
        public String getOcieUUID() {
            return getID(Field.qos_policy_group_uuid);
        }
    }

    private final QosPolicyGroupKey key;

    @OCIEZapiSource(
        sourceZapi = "qos-policy-group-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.qos-policy-group-info.pgid"
    )
    @OCIEValue(
        ocieColumn = "pgIdentifier"
    )
    private String pgid;

    @OCIEZapiSource(
        sourceZapi = "qos-policy-group-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.qos-policy-group-info.policy-group"
    )
    @OCIEValue(
        ocieColumn = "policyGroup"
    )
    private String policyGroupName;

    @OCIEZapiSource(
        sourceZapi = "qos-policy-group-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.qos-policy-group-info.policy-group-class"
    )
    @OCIEValue(
        ocieColumn = "policyGroupClass"
    )
    private QosPolicyGroupClassType qosPolicyGroupClass;

    @OCIEZapiSource(
        sourceZapi = "qos-policy-group-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.qos-policy-group-info.policy-group-class"
    )
    @OCIEValue(
        ocieColumn = "policyGroupClassRaw"
    )
    private String qosPolicyGroupClassRaw;


    @OCIEZapiSource(
        sourceZapi = "qos-policy-group-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.qos-policy-group-info.max-throughput"
    )
    @OCIEValue(
        ocieColumn = "maxThroughput"
    )
    private String maxThroughput;


    public QosPolicyGroup(final Cluster parent, final String qosPolicyGroupUUID) {
        super(parent, "QosPolicyGroup");
        this.key = new QosPolicyGroupKey(parent.getClusterUUID(), qosPolicyGroupUUID);
    }


    @Override
    @SuppressWarnings({"unchecked"})
    public QosPolicyGroupKey getKey() {
        return key;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getMaxThroughput() {
        return maxThroughput;
    }

    public void setMaxThroughput(final String maxThroughput) {
        this.maxThroughput = maxThroughput;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getPgid() {
        return pgid;
    }

    public void setPgid(final String pgid) {
        this.pgid = pgid;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getPolicyGroupName() {
        return policyGroupName;
    }

    public void setPolicyGroupName(final String policyGroupName) {
        this.policyGroupName = policyGroupName;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public QosPolicyGroupClassType getQosPolicyGroupClass() {
        return qosPolicyGroupClass;
    }

    public void setQosPolicyGroupClass(final QosPolicyGroupClassType qosPolicyGroupClass) {
        this.qosPolicyGroupClass = qosPolicyGroupClass;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getQosPolicyGroupClassRaw() {
        return qosPolicyGroupClassRaw;
    }

    public void setQosPolicyGroupClassRaw(final String qosPolicyGroupClassRaw) {
        this.qosPolicyGroupClassRaw = qosPolicyGroupClassRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public static enum QosPolicyGroupClassType {
        NOT_MAPPED,
        PRESET,
        USER_DEFINED,
        SYSTEM_DEFINED,
    }
    
    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "PolicyGroup", policyGroupName);
        addRequiredAttributeAndCRC(writer, "PgIdentifier", pgid);
        addRequiredAttributeAndCRC(writer, "PolicyGroupClass", qosPolicyGroupClass);
        addRequiredAttributeAndCRC(writer, "PolicyGroupClassRaw", qosPolicyGroupClassRaw);
        addRequiredAttributeAndCRC(writer, "MaxThroughput", maxThroughput);
    }
}
