package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEResourceType;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIETable;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("qos_policy_group_vserver_relationship")
@OCIEResourceType("qos_policy_group_vserver_relationship")
public class QosPolicyGroupVServerRelationship extends BaseCRCXMLNode<Cluster> {

    public final static class RelationshipKey extends KeyBase<RelationshipKey.Field> {
        public static enum Field {
            qos_policy_group_uuid,
            vserver_uuid
        }

        public RelationshipKey(final String qosPolicyGroupUUID, final String vServerUUID) {
            super("qos_policy_group", new EnumMap<Field, String>(Field.class) {{
                put(Field.qos_policy_group_uuid, qosPolicyGroupUUID);
                put(Field.vserver_uuid, vServerUUID);
            }});
        }

        public String getOcieUUID() {
            return getFQID();
        }
    }

    private final RelationshipKey key;

    QosPolicyGroupVServerRelationship(final Cluster parent, final QosPolicyGroup.QosPolicyGroupKey qosPolicyGroupKey, final VServer.VServerKey vServerKey) {
        super(parent, "QosPolicyGroupVserverRelationship");
        this.key = new RelationshipKey(qosPolicyGroupKey.getOcieUUID(), vServerKey.getOcieUUID());
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public RelationshipKey getKey() {
        return key;
    }



    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "QosPolicyGroupUUID", key.getID(RelationshipKey.Field.qos_policy_group_uuid));
        addRequiredAttributeAndCRC(writer, "VserverUUID", key.getID(RelationshipKey.Field.vserver_uuid));
    }
}
