package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("qos_service_center")
@OCIEResourceType("qos_service_center")
public class QosServiceCenter extends BaseCRCXMLNode<Node> {

    /**
     * Globally unique key based on uuid.
     */
    public final static class QosResourceKey extends KeyBase<QosResourceKey.Field> {

        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "perf-object-instance-list-info-iter",
                zapiArgs = "resource",
                zapiResponsePath = "netapp.results.attributes-list.instance-info.uuid"
            )
            uuid
        }

        public QosResourceKey(final String clusterUUID, final String uuid) {
            super("qos_resource", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.uuid, uuid);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "uuid")
        public String getOcieUUID() {
            return getFQID();
        }
    }

    private final QosResourceKey key;
    @OCIEZapiSource(
        sourceZapi = "perf-object-instance-list-info-iter",
        zapiArgs = "resource",
        zapiResponsePath = "netapp.results.attributes-list.instance-info.name"
    )
    @OCIEValue(
        ocieColumn = "name"
    )
    private String name;

    @OCIEZapiSource(
        sourceZapi = "hard coded to 'resource'"
    )
    @OCIEValue(
        ocieColumn = "type"
    )
    private QosServiceCenterType type;


    @OCIEZapiSource(
        sourceZapi = "perf-object-instance-list-info-iter",
        zapiArgs = "resource",
        zapiResponsePath = "netapp.results.attributes-list.instance-info.name"
    )
    @OCIEValue(
        ocieColumn = "aggregateId",
        notes = "derived from name for some resource instances"
    )
    private Aggregate.AggregateKey aggregateKey;

    protected QosServiceCenter(final Node parent, final String uuid) {
        super(parent, QosServiceCenter.class.getSimpleName());
        final String clusterUUID = parent.getKey().getID(Node.NodeKey.Field.cluster_uuid);
        this.key = new QosResourceKey(clusterUUID, uuid);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public QosResourceKey getKey() {
        return key;
    }
    
    public String getName() {
        return name;
    }

    public void setName(final String name) {
        this.name = name;
    }
    
    public QosServiceCenterType getType(){
        return type;
    }
    
    public void setType(final QosServiceCenterType type){
        this.type = type;
    }

    @SuppressWarnings("UnusedDeclaration")
    public Aggregate.AggregateKey getAggregateKey(){
        return aggregateKey;
    }
    
    public void setAggregateKey(final Aggregate.AggregateKey aggregateKey){
        this.aggregateKey = aggregateKey;
    }

    @SuppressWarnings("UnusedDeclaration")
    public static enum QosServiceCenterType{
        NOT_MAPPED,
        RESOURCE, 
        DELAY_CENTER;
        
        public static QosServiceCenterType getServiceCenterType(final String serviceCenterTypeName) {
            for (final QosServiceCenterType value : QosServiceCenterType.values()) {
                if (value.name().equalsIgnoreCase(serviceCenterTypeName)) {
                    return value;
                }
            }
            return NOT_MAPPED;
        }
    }

    
    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Name", name);
        addRequiredAttributeAndCRC(writer, "Type", type);
        addRequiredAttributeAndCRC(writer, "AggregateUUID", KeyBase.getOcieUUIDOrNull(aggregateKey));
    }
}
