package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;
import java.util.EnumSet;

@OCIETable("quota_user")
@OCIEResourceType("quota_user")
public class QuotaUser extends BaseCRCXMLNode<UserQuota> {

    public static final class QuotaUserKey extends KeyBase<QuotaUserKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "vserver-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.vserver-info.uuid"
            )
            vserver_uuid,

            @OCIEZapiSource(
                sourceZapi = "volume-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-id-attributes.uuid"
            )
            volume_uuid,

            @OCIEZapiSource(
                sourceZapi = "qtree-list-iter",
                zapiResponsePath = "netapp.results.attributes-list.qtree-info.id"
            )
            qtree_id,

            @OCIEZapiSource(
                sourceZapi = "quota-report-iter",
                zapiResponsePath = "netapp.results.attributes-list.quota.quota-type"
            )
            quota_type,

            @OCIEZapiSource(
                sourceZapi = "quota-report-iter",
                zapiResponsePath = "netapp.results.attributes-list.quota.quota-target"
            )
            quota_target,

            @OCIEZapiSource(
                sourceZapi = "quota-report-iter",
                zapiResponsePath = "netapp.results.attributes-list.quota.quota-users.quota-user.quota-user-id"
            )
            quota_user_id
        }

        public QuotaUserKey(final String clusterUUID, final String vServerUUID, final String volumeUUID, final String qTreeID,
                            final String quotaType, final String quotaTarget, final String quotaUserID) {
            super("quota_user", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.vserver_uuid, vServerUUID);
                put(Field.volume_uuid, volumeUUID);
                put(Field.qtree_id, qTreeID);
                put(Field.quota_type, quotaType);
                put(Field.quota_target, quotaTarget);
                put(Field.quota_user_id, quotaUserID);
            }}, EnumSet.of(Field.quota_target));                      // These EnumSet<E> emptyAbleFields fields are allowed to be empty but not null
        }

        @Override
        @OCIEUUID(uuidDescr = "volume_uuid.qtree_id.volume_uuid.quota_type.quota_target.quota_user_name")
        public String getOcieUUID() {
            final String volumeUUID = getID(Field.volume_uuid);
            final String qTreeID = getID(Field.qtree_id);
            final String quotaType = getID(Field.quota_type);
            final String quotaTarget = getID(Field.quota_target);
            final String quotaUserID = getID(Field.quota_user_id);
            final String targetSuffix = (quotaTarget.isEmpty() ? "" : quotaTarget);
            // Include extra dot when target not defined for backward compatibility
            return volumeUUID + '.' + qTreeID + '.' + volumeUUID + '.' + quotaType + '.' + targetSuffix + '.' + quotaUserID;
        }
    }

    private final QuotaUserKey key;

    @OCIEZapiSource(
        sourceZapi = "quota-report-iter",
        zapiResponsePath = "netapp.results.attributes-list.quota.quota-users.quota-user.quota-user-name"
    )
    @OCIEValue(
        ocieColumn = "quotaUserName"
    )
    private String quotaUserName;

    @OCIEZapiSource(
        sourceZapi = "quota-report-iter",
        zapiResponsePath = "netapp.results.attributes-list.quota.quota-users.quota-user.quota-user-type"
    )
    @OCIEValue(
        ocieColumn = "quotaUserType"
    )
    private QuotaUserType quotaUserType;

    QuotaUser(final UserQuota parent, final String quotaUserID) {
        super(parent, QuotaUser.class.getSimpleName());
        final UserQuota.UserQuotaKey parentKey = parent.getKey();
        final String clusterUUID = parentKey.getID(UserQuota.UserQuotaKey.Field.cluster_uuid);
        final String vServerUUID = parentKey.getID(UserQuota.UserQuotaKey.Field.vserver_uuid);
        final String volumeUUID = parentKey.getID(UserQuota.UserQuotaKey.Field.volume_uuid);
        final String qTreeID = parentKey.getID(UserQuota.UserQuotaKey.Field.qtree_id);
        final String quotaType = parentKey.getID(UserQuota.UserQuotaKey.Field.quota_type);
        final String quotaTarget = parentKey.getID(UserQuota.UserQuotaKey.Field.quota_target);
        this.key = new QuotaUserKey(clusterUUID, vServerUUID, volumeUUID, qTreeID, quotaType, quotaTarget, quotaUserID);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public QuotaUserKey getKey() {
        return key;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getQuotaUserID() {
        return key.getID(QuotaUserKey.Field.quota_user_id);
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getQuotaUserName() {
        return quotaUserName;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public QuotaUserType getQuotaUserType() {
        return quotaUserType;
    }

    public void setQuotaUserName(final String quotaUserName) {
        this.quotaUserName = quotaUserName;
    }

    public void setQuotaUserType(final QuotaUserType quotaUserType) {
        this.quotaUserType = quotaUserType;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public enum QuotaUserType {
        NOT_MAPPED,
        SID,
        UID,
        GID
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "QuotaUserId", getQuotaUserID());
        addRequiredAttributeAndCRC(writer, "QuotaUserName", getQuotaUserName());
        addRequiredAttributeAndCRC(writer, "QuotaUserType", quotaUserType);
    }
}
