package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("raid_group")
@OCIEResourceType("raid_group")
public class RaidGroup extends BaseCRCXMLNode<Plex> {

    public final static class RaidGroupKey extends KeyBase<RaidGroupKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "aggr-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggregate-uuid"
            )
            aggregate_uuid,

            @OCIEZapiSource(
                sourceZapi = "aggr-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-raid-attributes.plexes.plex-attributes.plex-name"
            )
            plex_name,

            @OCIEZapiSource(
                sourceZapi = "aggr-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-raid-attributes.plexes.plex-attributes.raidgroups.raidgroup-attributes.raidgroup-name"
            )
            name
        }

        public RaidGroupKey(final String clusterUUID, final String aggregateUUID, final String plexName, final String rgName) {
            super("raid_group", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.aggregate_uuid, aggregateUUID);
                put(Field.plex_name, plexName);
                put(Field.name, rgName);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "aggregate_uuid.plex_name.name")
        public String getOcieUUID() {
            return getID(Field.aggregate_uuid) + '.' + getID(Field.plex_name) + '.' + getID(Field.name);
        }
    }

    public static enum RaidType {
        NOT_MAPPED,
        RAID_DP,
        RAID4,
        RAID0,
        MIXED_RAID_TYPE,
        INVALID,
    }


    private final RaidGroupKey key;

    @OCIEZapiSource(
        sourceZapi = "aggr-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggr-raid-attributes.plexes.plex-attributes.raidgroups.raidgroup-attributes.is-reconstructing"
    )
    @OCIEValue(
        ocieColumn = "isReconstructing"
    )
    private Boolean isReconstructing;

    @OCIEZapiSource(
            sourceZapi = "aggr-status-get-iter",
            zapiResponsePath = "netapp.results.attributes.aggr-status-info.aggr-plex-list.aggr-plex-info.aggr-raidgroup-list.aggr-raidgroup-info.aggr-raidgroup-entity-list.aggr-raidgroup-entity-info.rg-raidtype"
    )
    @OCIEValue(
            ocieColumn = "raidType"
    )
    private RaidType raidType;

    @OCIEZapiSource(
            sourceZapi = "aggr-status-get-iter",
            zapiResponsePath = "netapp.results.attributes.aggr-status-info.aggr-plex-list.aggr-plex-info.aggr-raidgroup-list.aggr-raidgroup-info.aggr-raidgroup-entity-list.aggr-raidgroup-entity-info.rg-raidtype"
    )
    @OCIEValue(
            ocieColumn = "raidTypeRaw"
    )
    private String raidTypeRaw;

    RaidGroup(final Plex parent, final String name) {
        super(parent, "RaidGroup");
        final Plex.PlexKey parentKey = parent.getKey();
        final String clusterUUID = parentKey.getID(Plex.PlexKey.Field.cluster_uuid);
        final String aggregateUUID = parentKey.getID(Plex.PlexKey.Field.aggregate_uuid);
        final String plexName = parentKey.getID(Plex.PlexKey.Field.plex_name);
        this.key = new RaidGroupKey(clusterUUID, aggregateUUID, plexName, name);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public RaidGroupKey getKey() {
        return key;
    }

    @SuppressWarnings({"unused declaration"})
    public Boolean isReconstructing() {
        return isReconstructing;
    }

    public void setReconstructing(final Boolean reconstructing) {
        isReconstructing = reconstructing;
    }

    public RaidType getRaidType() {
        return raidType;
    }

    public void setRaidType(RaidType raidType) {
        this.raidType = raidType;
    }

    public String getRaidTypeRaw() {
        return raidTypeRaw;
    }

    public void setRaidTypeRaw(String raidTypeRaw) {
        this.raidTypeRaw = raidTypeRaw;
    }

    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Name", key.getID(RaidGroupKey.Field.name));
        addRequiredAttributeAndCRC(writer, "IsReconstructing", isReconstructing);
        addRequiredAttributeAndCRC(writer, "RaidType", raidType);
        addRequiredAttributeAndCRC(writer, "RaidTypeRaw", raidTypeRaw);
    }
}
