package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("snap_mirror")
@OCIEResourceType("snap_mirror")
public class SnapMirror extends BaseCRCXMLNode<Cluster> {

    public static final class SnapMirrorKey extends KeyBase<SnapMirrorKey.Field> {

        @SuppressWarnings({"UnusedDeclaration"})
        private final String destinationLocation;

        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "snapmirror-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.destination-vserver"
            )
            destination_vserver_uuid,

            @OCIEZapiSource(
                sourceZapi = "snapmirror-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.destination-volume"
            )
            destination_volume_uuid
        }

        public SnapMirrorKey(final String clusterUUID, final String destinationVServerUUID,
                             final String destinationVolumeUUID, final String destinationLocation) {
            super("snap_mirror", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                // put(Field.source_location, sourceLocation);
                put(Field.destination_vserver_uuid, destinationVServerUUID);
                put(Field.destination_volume_uuid, destinationVolumeUUID);
            }});
            this.destinationLocation = destinationLocation;
        }

        /**
         * vServer SnapMirrors do not have destination volumes.
         */
        public SnapMirrorKey(final String clusterUUID, final String destinationVServerUUID,
                             final String destinationLocation) {
            super("snap_mirror", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.destination_vserver_uuid, destinationVServerUUID);
            }});
            this.destinationLocation = destinationLocation;
        }

        @Override
        @OCIEUUID(uuidDescr = "destination_volume_uuid")
        public String getOcieUUID() {
            return getID(Field.destination_volume_uuid) == null ? getID(Field.destination_vserver_uuid) : getID(Field.destination_volume_uuid);
        }
    }

    private final SnapMirrorKey key;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.destination-vserver"
    )
    @OCIEValue(
        ocieColumn = "destinationVServerId"
    )
    private final VServer.VServerKey destinationVServerKey;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.destination-volume"
    )
    @OCIEValue(
        ocieColumn = "destinationVolumeId"
    )
    private final Volume.VolumeKey destinationVolumeKey;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.source-cluster"
    )
    @OCIEValue(
        ocieColumn = "sourceCluster"
    )
    private String sourceCluster;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.source-volume"
    )
    @OCIEValue(
        ocieColumn = "sourceVolume"
    )
    private String sourceVolume;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.source-volume"
    )
    @OCIEValue(
        ocieColumn = "sourceVolumeId"
    )
    private Volume.VolumeKey sourceVolumeKey;        // only if local

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.source-vserver"
    )
    @OCIEValue(
        ocieColumn = "sourceVserver"
    )
    private String sourceVServer;


    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.source-vserver"
    )
    @OCIEValue(
        ocieColumn = "sourceVserverId"
    )
    private VServer.VServerKey sourceVServerKey;       // only if local


    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.exported-snapshot"
    )
    @OCIEValue(
        ocieColumn = "exportedSnapshotId"
    )
    private Snapshot.SnapshotKey exportedSnapshotKey;


    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.newest-snapshot"
    )
    @OCIEValue(
        ocieColumn = "newestSnapshotId"
    )
    private Snapshot.SnapshotKey newestSnapshotKey;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.transfer-snapshot"
    )
    @OCIEValue(
        ocieColumn = "transferSnapshot"
    )
    private String transferSnapshot;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.mirror-state"
    )
    @OCIEValue(
        ocieColumn = "mirrorState"
    )
    private MirrorState mirrorState;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.mirror-state"
    )
    @OCIEValue(
        ocieColumn = "mirrorStateRaw"
    )
    private String mirrorStateRaw;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.is-constituent"
    )
    @OCIEValue(
        ocieColumn = "isConstituent"
    )
    private Boolean isConstituent;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.is-healthy"
    )
    @OCIEValue(
        ocieColumn = "isHealthy"
    )
    private Boolean isHealthy;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.max-transfer-rate"
    )
    @OCIEValue(
        ocieColumn = "maxTransferRate"
    )
    private Long maxTransferRate;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.relationship-status"
    )
    @OCIEValue(
        ocieColumn = "relationshipStatus"
    )
    private RelationshipStatusType relationshipStatus;


    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.relationship-status"
    )
    @OCIEValue(
        ocieColumn = "relationshipStatusRaw"
    )
    private String relationshipStatusRaw;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.relationship-type"
    )
    @OCIEValue(
        ocieColumn = "relationshipType"
    )
    private SnapMirror.RelationshipType relationshipType;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.relationship-type"
    )
    @OCIEValue(
        ocieColumn = "relationshipTypeRaw"
    )
    private String relationshipTypeRaw;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.relationship-progress"
    )
    @OCIEValue(
        ocieColumn = "relationshipProgress"
    )
    private Long relationshipProgress;


    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.snapshot-checkpoint"
    )
    @OCIEValue(
        ocieColumn = "snapshotCheckpoint"
    )
    private Long snapshotCheckpoint;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.snapshot-progress"
    )
    @OCIEValue(
        ocieColumn = "snapshotProgress"
    )
    private Long snapshotProgress;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.tries"
    )
    @OCIEValue(
        ocieColumn = "tries"
    )
    private Long tries;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.tries"
    )
    @OCIEValue(
        ocieColumn = "tries"
    )
    private JobSchedule.JobScheduleKey scheduleKey;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.current-max-transfer-rate"
    )
    @OCIEValue(
        ocieColumn = "maxTransferRate"
    )
    private Long currentMaxTransferRate;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.current-transfer-error"
    )
    @OCIEValue(
        ocieColumn = "currentTransferError"
    )
    private String currentTransferError;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.current-transfer-priority"
    )
    @OCIEValue(
        ocieColumn = "currentTransferError"
    )
    private TransferPriorityType currentTransferPriority;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.current-transfer-priority"
    )
    @OCIEValue(
        ocieColumn = "currentTransferPriorityRaw"
    )
    private String currentTransferPriorityRaw;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.current-transfer-type"
    )
    @OCIEValue(
        ocieColumn = "currentTransferType"
    )
    private SnapMirror.TransferType currentTransferType;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.current-transfer-type"
    )
    @OCIEValue(
        ocieColumn = "currentTransferTypeRaw"
    )
    private String currentTransferTypeRaw;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.lag-time"
    )
    @OCIEValue(
        ocieColumn = "lagTime"
    )
    private Long lagTime;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.last-transfer-duration"
    )
    @OCIEValue(
        ocieColumn = "lastTransferDuration"
    )
    private Long lastTransferDuration;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.last-transfer-error"
    )
    @OCIEValue(
        ocieColumn = "lastTransferError"
    )
    private String lastTransferError;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.last-transfer-type"
    )
    @OCIEValue(
        ocieColumn = "lastTransferType"
    )
    private TransferType lastTransferType;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.last-transfer-type"
    )
    @OCIEValue(
        ocieColumn = "lastTransferTypeRaw"
    )
    private String lastTransferTypeRaw;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.last-transfer-size"
    )
    @OCIEValue(
        ocieColumn = "lastTransferSize"
    )
    private Long lastTransferSize;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.progress-last-updated"
    )
    @OCIEValue(
        ocieColumn = "progressLastUpdated"
    )
    private Long progressLastUpdated;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.relationship-control-plane"
    )
    @OCIEValue(
        ocieColumn = "relationshipControlPlane"
    )
    private String relationshipControlPlane;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.relationship-id"
    )
    @OCIEValue(
        ocieColumn = "relationshipIdentifier"
    )
    private String relationshipID;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.source-location"
    )
    @OCIEValue(
        ocieColumn = "sourceLocation"
    )
    private String sourceLocation;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.destination-location"
    )
    @OCIEValue(
        ocieColumn = "destinationLocation"
    )
    private String destinationLocation;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.policy"
    )
    @OCIEValue(
        ocieColumn = "snapMirrorPolicyId"
    )
    private SnapMirrorPolicy.SnapMirrorPolicyKey policyKey;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.last-transfer-end-timestamp"
    )
    @OCIEValue(
        ocieColumn = "lastTransferEndTimestamp"
    )
    private Long lastTransferEndTimestamp;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.last-transfer-end-codes"
    )
    @OCIEValue(
        ocieColumn = "lastTransferErrorCodes"
    )
    private String lastTransferEndErrorCodes;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.destination-volume-node"
    )
    @OCIEValue(
            ocieColumn = "destinationNodeId"
    )
    private Node.NodeKey destinationVolumeNodeKey;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.unhealthy-reason"
    )
    @OCIEValue(
        ocieColumn = "unhealthyReason"
    )
    private String unhealthyReason;

    @OCIEZapiSource(
        sourceZapi = "volume-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-id-attributes.style"
    )
    @OCIEValue(
        ocieColumn = "volumeStyle",
        notes = "derived enum"
    )
    private Volume.DerivedVolumeStyle destVolumeDerivedStyle;

    @OCIEZapiSource(
            sourceZapi = "snapmirror-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.operation-id"
    )
    @OCIEValue(
            ocieColumn = "operationId"
    )
    private String operationId;

    @OCIEZapiSource(
            sourceZapi = "snapmirror-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.network-compression-ratio"
    )
    @OCIEValue(
            ocieColumn = "networkCompressionRatio"
    )
    private String networkCompressionRatio;

    @OCIEZapiSource(
            sourceZapi = "snapmirror-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.last-transfer-network-compression-ratio"
    )
    @OCIEValue(
            ocieColumn = "lastTransferNetworkCompressionRatio"
    )
    private String lastTransferNetworkCompressionRatio;

    @OCIEZapiSource(
            sourceZapi = "snapmirror-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.relationship-group-type"
    )
    @OCIEValue(
            ocieColumn = "relationshipGroupType"
    )
    private SnapMirror.RelationshipGroupType  relationshipGroupType;

    @OCIEZapiSource(
            sourceZapi = "snapmirror-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.snapmirror-info.relationship-group-type"
    )
    @OCIEValue(
            ocieColumn = "relationshipGroupTypeRaw"
    )
    private String  relationshipGroupTypeRaw;

    SnapMirror(final Cluster parent, final VServer.VServerKey destinationVServerKey, final Volume.VolumeKey destinationVolumeKey,
               final String destinationLocation) {
        super(parent, SnapMirror.class.getSimpleName());
        this.key = new SnapMirrorKey(parent.getClusterUUID(), destinationVServerKey.getZapiUUID(),
                                     destinationVolumeKey.getVolumeInstanceUUID(), destinationLocation);
        this.destinationVServerKey = destinationVServerKey;
        this.destinationVolumeKey = destinationVolumeKey;
    }

    /**
     * This constructor is used for vServer Snap Mirrors. vServer SnapMirrors do not have destination volumes.
     */
    public SnapMirror(Cluster parent, VServer.VServerKey destinationVServerKey, String destinationLocation) {
        super(parent, SnapMirror.class.getSimpleName());
        this.key = new SnapMirrorKey(parent.getClusterUUID(), destinationVServerKey.getZapiUUID(), destinationLocation);
        this.destinationVServerKey = destinationVServerKey;
        this.destinationVolumeKey = null;
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public SnapMirrorKey getKey() {
        return key;
    }


    @SuppressWarnings({"UnusedDeclaration"})
    public String getDestinationLocation() {
        return destinationLocation;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getSourceCluster() {
        return sourceCluster;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getSourceVolume() {
        return sourceVolume;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getSourceVServer() {
        return sourceVServer;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getSourceLocation() {
        return sourceLocation;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public MirrorState getMirrorState() {
        return mirrorState;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getMirrorStateRaw() {
        return mirrorStateRaw;
    }

    public void setMirrorStateRaw(final String mirrorStateRaw) {
        this.mirrorStateRaw = mirrorStateRaw;
    }

    public void setSourceLocation(final String sourceLocation) {
        this.sourceLocation = sourceLocation;
    }

    public void setSourceCluster(final String sourceCluster) {
        this.sourceCluster = sourceCluster;
    }

    public void setSourceVolume(final String sourceVolume) {
        this.sourceVolume = sourceVolume;
    }

    public void setSourceVServer(final String sourceVServer) {
        this.sourceVServer = sourceVServer;
    }

    public void setMirrorState(final MirrorState mirrorState) {
        this.mirrorState = mirrorState;
    }

    public void setSourceVolumeKey(final Volume.VolumeKey sourceVolumeKey) {
        this.sourceVolumeKey = sourceVolumeKey;
    }

    public void setSourceVServerKey(final VServer.VServerKey sourceVServerKey) {
        this.sourceVServerKey = sourceVServerKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Snapshot.SnapshotKey getExportedSnapshotKey() {
        return exportedSnapshotKey;
    }

    public void setExportedSnapshotKey(final Snapshot.SnapshotKey exportedSnapshotKey) {
        this.exportedSnapshotKey = exportedSnapshotKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Snapshot.SnapshotKey getNewestSnapshotKey() {
        return newestSnapshotKey;
    }

    public void setNewestSnapshotKey(final Snapshot.SnapshotKey newestSnapshotKey) {
        this.newestSnapshotKey = newestSnapshotKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getTransferSnapshot() {
        return transferSnapshot;
    }

    public void setTransferSnapshot(final String transferSnapshot) {
        this.transferSnapshot = transferSnapshot;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isConstituent() {
        return isConstituent;
    }

    public void setConstituent(final Boolean constituent) {
        isConstituent = constituent;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isHealthy() {
        return isHealthy;
    }

    public void setHealthy(final Boolean healthy) {
        isHealthy = healthy;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getMaxTransferRate() {
        return maxTransferRate;
    }

    public void setMaxTransferRate(final Long maxTransferRate) {
        this.maxTransferRate = maxTransferRate;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public RelationshipStatusType getRelationshipStatus() {
        return relationshipStatus;
    }

    public void setRelationshipStatus(final RelationshipStatusType relationshipStatus) {
        this.relationshipStatus = relationshipStatus;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getRelationshipStatusRaw() {
        return relationshipStatusRaw;
    }

    public void setRelationshipStatusRaw(final String relationshipStatusRaw) {
        this.relationshipStatusRaw = relationshipStatusRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public RelationshipType getRelationshipType() {
        return relationshipType;
    }

    public void setRelationshipType(final RelationshipType relationshipType) {
        this.relationshipType = relationshipType;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getRelationshipTypeRaw() {
        return relationshipTypeRaw;
    }

    public void setRelationshipTypeRaw(final String relationshipTypeRaw) {
        this.relationshipTypeRaw = relationshipTypeRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getRelationshipProgress() {
        return relationshipProgress;
    }

    public void setRelationshipProgress(final Long relationshipProgress) {
        this.relationshipProgress = relationshipProgress;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getSnapshotCheckpoint() {
        return snapshotCheckpoint;
    }

    public void setSnapshotCheckpoint(final Long snapshotCheckpoint) {
        this.snapshotCheckpoint = snapshotCheckpoint;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getSnapshotProgress() {
        return snapshotProgress;
    }

    public void setSnapshotProgress(final Long snapshotProgress) {
        this.snapshotProgress = snapshotProgress;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getTries() {
        return tries;
    }

    public void setTries(final Long tries) {
        this.tries = tries;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public JobSchedule.JobScheduleKey getScheduleKey() {
        return scheduleKey;
    }

    public void setScheduleKey(final JobSchedule.JobScheduleKey scheduleKey) {
        this.scheduleKey = scheduleKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getCurrentMaxTransferRate() {
        return currentMaxTransferRate;
    }

    public void setCurrentMaxTransferRate(final Long currentMaxTransferRate) {
        this.currentMaxTransferRate = currentMaxTransferRate;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getCurrentTransferError() {
        return currentTransferError;
    }

    public void setCurrentTransferError(final String currentTransferError) {
        this.currentTransferError = currentTransferError;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public TransferPriorityType getCurrentTransferPriority() {
        return currentTransferPriority;
    }

    public void setCurrentTransferPriority(final TransferPriorityType currentTransferPriority) {
        this.currentTransferPriority = currentTransferPriority;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getCurrentTransferPriorityRaw() {
        return currentTransferPriorityRaw;
    }

    public void setCurrentTransferPriorityRaw(final String currentTransferPriorityRaw) {
        this.currentTransferPriorityRaw = currentTransferPriorityRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public TransferType getCurrentTransferType() {
        return currentTransferType;
    }

    public void setCurrentTransferType(final TransferType currentTransferType) {
        this.currentTransferType = currentTransferType;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getCurrentTransferTypeRaw() {
        return currentTransferTypeRaw;
    }

    public void setCurrentTransferTypeRaw(final String currentTransferTypeRaw) {
        this.currentTransferTypeRaw = currentTransferTypeRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getLagTime() {
        return lagTime;
    }

    public void setLagTime(final Long lagTime) {
        this.lagTime = lagTime;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getLastTransferDuration() {
        return lastTransferDuration;
    }

    public void setLastTransferDuration(final Long lastTransferDuration) {
        this.lastTransferDuration = lastTransferDuration;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getLastTransferError() {
        return lastTransferError;
    }

    public void setLastTransferError(final String lastTransferError) {
        this.lastTransferError = lastTransferError;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public TransferType getLastTransferType() {
        return lastTransferType;
    }

    public void setLastTransferType(final TransferType lastTransferType) {
        this.lastTransferType = lastTransferType;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getLastTransferTypeRaw() {
        return lastTransferTypeRaw;
    }

    public void setLastTransferTypeRaw(final String lastTransferTypeRaw) {
        this.lastTransferTypeRaw = lastTransferTypeRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getLastTransferSize() {
        return lastTransferSize;
    }

    public void setLastTransferSize(final Long lastTransferSize) {
        this.lastTransferSize = lastTransferSize;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getProgressLastUpdated() {
        return progressLastUpdated;
    }

    public void setProgressLastUpdated(final Long progressLastUpdated) {
        this.progressLastUpdated = progressLastUpdated;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getRelationshipControlPlane() {
        return relationshipControlPlane;
    }

    public void setRelationshipControlPlane(final String relationshipControlPlane) {
        this.relationshipControlPlane = relationshipControlPlane;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getRelationshipID() {
        return relationshipID;
    }

    public void setRelationshipID(final String relationshipID) {
        this.relationshipID = relationshipID;
    }


    public void setDestinationLocation(final String destinationLocation) {
        this.destinationLocation = destinationLocation;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public SnapMirrorPolicy.SnapMirrorPolicyKey getPolicyKey() {
        return policyKey;
    }

    public void setPolicyKey(final SnapMirrorPolicy.SnapMirrorPolicyKey policyKey) {
        this.policyKey = policyKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getLastTransferEndTimestamp() {
        return lastTransferEndTimestamp;
    }

    public void setLastTransferEndTimestamp(final Long lastTransferEndTimestamp) {
        this.lastTransferEndTimestamp = lastTransferEndTimestamp;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getLastTransferEndErrorCodes() {
        return lastTransferEndErrorCodes;
    }

    public void setLastTransferEndErrorCodes(final String lastTransferEndErrorCodes) {
        this.lastTransferEndErrorCodes = lastTransferEndErrorCodes;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Node.NodeKey getDestinationVolumeNodeKey() {
        return destinationVolumeNodeKey;
    }

    public void setDestinationVolumeNodeKey(final Node.NodeKey destinationVolumeNodeKey) {
        this.destinationVolumeNodeKey = destinationVolumeNodeKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getUnhealthyReason() {
        return unhealthyReason;
    }

    public void setUnhealthyReason(final String unhealthyReason) {
        this.unhealthyReason = unhealthyReason;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Volume.DerivedVolumeStyle getDestVolumeDerivedStyle() {
        return destVolumeDerivedStyle;
    }

    public void setDestVolumeDerivedStyle(final Volume.DerivedVolumeStyle destVolumeDerivedStyle) {
        this.destVolumeDerivedStyle = destVolumeDerivedStyle;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public VServer.VServerKey getDestinationVServerKey() {
        return destinationVServerKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Volume.VolumeKey getDestinationVolumeKey() {
        return destinationVolumeKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Volume.VolumeKey getSourceVolumeKey() {
        return sourceVolumeKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public VServer.VServerKey getSourceVServerKey() {
        return sourceVServerKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getOperationId() {
        return operationId;
    }

    public void setOperationId(String operationId) {
        this.operationId = operationId;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getNetworkCompressionRatio() {
        return networkCompressionRatio;
    }

    public void setNetworkCompressionRatio(String networkCompressionRatio) {
        this.networkCompressionRatio = networkCompressionRatio;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getLastTransferNetworkCompressionRatio() {
        return lastTransferNetworkCompressionRatio;
    }

    public void setLastTransferNetworkCompressionRatio(String lastTransferNetworkCompressionRatio) {
        this.lastTransferNetworkCompressionRatio = lastTransferNetworkCompressionRatio;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public RelationshipGroupType getRelationshipGroupType() {
        return relationshipGroupType;
    }

    public void setRelationshipGroupType(RelationshipGroupType relationshipGroupType) {
        this.relationshipGroupType = relationshipGroupType;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getRelationshipGroupTypeRaw() {
        return relationshipGroupTypeRaw;
    }

    public void setRelationshipGroupTypeRaw(String relationshipGroupTypeRaw) {
        this.relationshipGroupTypeRaw = relationshipGroupTypeRaw;
    }

    public enum MirrorState {
        NOT_MAPPED,
        UNINITIALIZED,
        SNAPMIRRORED,
        BROKEN_OFF("broken-off");

        private final String rawValue;

        private MirrorState() {
            this(null);
        }

        private MirrorState(final String rawValue) {
            this.rawValue = rawValue;
        }


        public static MirrorState fromString(final String stringVal) {
            if (stringVal == null) {
                return NOT_MAPPED;
            }

            for (final MirrorState candidateMirrorState : MirrorState.values()) {
                if (candidateMirrorState.name().equalsIgnoreCase(stringVal)) {
                    return candidateMirrorState;
                } else if (candidateMirrorState.rawValue != null && candidateMirrorState.rawValue.equalsIgnoreCase(stringVal)) {
                    return candidateMirrorState;
                }
            }
            return NOT_MAPPED;
        }
    }

    public enum RelationshipStatusType {
        NOT_MAPPED,
        IDLE,
        TRANSFERRING,
        CHECKING,
        QUIESCING,
        QUIESCED,
        QUEUED,
        PREPARING,
        FINALIZING,
        ABORTING,
    }

    public enum RelationshipType {
        NOT_MAPPED,
        DATA_PROTECTION,
        LOAD_SHARING,
        VAULT,
        RESTORE,
        TRANSITION_DATA_PROTECTION,
    }

    public enum TransferPriorityType {
        NOT_MAPPED,
        LOW,
        NORMAL,
    }

    public enum TransferType {
        NOT_MAPPED,
        INITIALIZE,
        UPDATE,
        RESYNC,
        RESTORE,
        CHECK,
    }               

    public enum RelationshipGroupType {
        NOT_MAPPED,
        NONE,
        VSERVER,
        INFINITEVOL,
    }



    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "CurrentMaxTransferRate", currentMaxTransferRate);
        addRequiredAttributeAndCRC(writer, "CurrentTransferError", currentTransferError);
        addRequiredAttributeAndCRC(writer, "CurrentTransferPriority", currentTransferPriority);
        addRequiredAttributeAndCRC(writer, "CurrentTransferPriorityRaw", currentTransferPriorityRaw);
        addRequiredAttributeAndCRC(writer, "CurrentTransferType", currentTransferType);
        addRequiredAttributeAndCRC(writer, "CurrentTransferTypeRaw", currentTransferTypeRaw);
        addRequiredAttributeAndCRC(writer, "DestinationVolumeUUID", Volume.VolumeKey.getVolumeInstanceUUIDOrNull(destinationVolumeKey));
        addRequiredAttributeAndCRC(writer, "DestinationVserverUUID", KeyBase.getOcieUUIDOrNull(destinationVServerKey));
        addRequiredAttributeAndCRC(writer, "DestinationNodeUUID", KeyBase.getOcieUUIDOrNull(destinationVolumeNodeKey));
        addRequiredAttributeAndCRC(writer, "DestinationLocation", destinationLocation);
        addRequiredAttributeAndCRC(writer, "LagTime", lagTime);
        addRequiredAttributeAndCRC(writer, "LastTransferNetworkCompressionRatio", lastTransferNetworkCompressionRatio);
        addRequiredAttributeAndCRC(writer, "LastTransferDuration", lastTransferDuration);
        addRequiredAttributeAndCRC(writer, "LastTransferEndTimestamp", lastTransferEndTimestamp);
        addRequiredAttributeAndCRC(writer, "LastTransferErrorCodes", lastTransferEndErrorCodes);
        addRequiredAttributeAndCRC(writer, "LastTransferError", lastTransferError);
        addRequiredAttributeAndCRC(writer, "LastTransferType", lastTransferType);
        addRequiredAttributeAndCRC(writer, "LastTransferTypeRaw", lastTransferTypeRaw);
        addRequiredAttributeAndCRC(writer, "LastTransferSize", lastTransferSize);
        addRequiredAttributeAndCRC(writer, "MirrorState", mirrorState);
        addRequiredAttributeAndCRC(writer, "MirrorStateRaw", mirrorStateRaw);
        addRequiredAttributeAndCRC(writer, "NetworkCompressionRatio", networkCompressionRatio);
        addRequiredAttributeAndCRC(writer, "OperationId", operationId);
        addRequiredAttributeAndCRC(writer, "ProgressLastUpdated", progressLastUpdated);
        addRequiredAttributeAndCRC(writer, "ExportedSnapshotUUID", KeyBase.getOcieUUIDOrNull(exportedSnapshotKey));
        addRequiredAttributeAndCRC(writer, "NewestSnapshotUUID", KeyBase.getOcieUUIDOrNull(newestSnapshotKey));
        addRequiredAttributeAndCRC(writer, "TransferSnapshot", transferSnapshot);
        addRequiredAttributeAndCRC(writer, "IsConstituent", isConstituent);
        addRequiredAttributeAndCRC(writer, "IsHealthy", isHealthy);
        addRequiredAttributeAndCRC(writer, "MaxTransferRate", maxTransferRate);
        addRequiredAttributeAndCRC(writer, "RelationshipControlPlane", relationshipControlPlane);
        addRequiredAttributeAndCRC(writer, "RelationshipGroupTypeRaw", relationshipGroupTypeRaw);
        addRequiredAttributeAndCRC(writer, "RelationshipGroupType", relationshipGroupType);
        addRequiredAttributeAndCRC(writer, "RelationshipIdentifier", relationshipID);
        addRequiredAttributeAndCRC(writer, "RelationshipProgress", relationshipProgress);
        addRequiredAttributeAndCRC(writer, "RelationshipStatus", relationshipStatus);
        addRequiredAttributeAndCRC(writer, "RelationshipStatusRaw", relationshipStatusRaw);
        addRequiredAttributeAndCRC(writer, "RelationshipType", relationshipType);
        addRequiredAttributeAndCRC(writer, "RelationshipTypeRaw", relationshipTypeRaw);
        addRequiredAttributeAndCRC(writer, "SnapshotCheckpoint", snapshotCheckpoint);
        addRequiredAttributeAndCRC(writer, "SnapshotProgress", snapshotProgress);
        addRequiredAttributeAndCRC(writer, "SourceVolumeUUID", Volume.VolumeKey.getVolumeInstanceUUIDOrNull(sourceVolumeKey));
        addRequiredAttributeAndCRC(writer, "SourceVserverUUID", KeyBase.getOcieUUIDOrNull(sourceVServerKey));
        addRequiredAttributeAndCRC(writer, "SourceCluster", sourceCluster);
        addRequiredAttributeAndCRC(writer, "SourceVolume", sourceVolume);
        addRequiredAttributeAndCRC(writer, "SourceVserver", sourceVServer);
        addRequiredAttributeAndCRC(writer, "SourceLocation", getSourceLocation());
        addRequiredAttributeAndCRC(writer, "Tries", tries);
        addRequiredAttributeAndCRC(writer, "SnapMirrorPolicyUUID", KeyBase.getOcieUUIDOrNull(policyKey));
        addRequiredAttributeAndCRC(writer, "JobScheduleUUID", KeyBase.getOcieUUIDOrNull(scheduleKey));
        addRequiredAttributeAndCRC(writer, "UnhealthyReason", unhealthyReason);
        addRequiredAttributeAndCRC(writer, "DestVolDerivedStyle", destVolumeDerivedStyle);
    }
}
