package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import java.io.IOException;
import java.util.EnumMap;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEResourceType;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIETable;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEUUID;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEValue;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEZapiSource;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

@OCIETable("snap_mirror_history")
@OCIEResourceType("snap_mirror_history")
public class SnapMirrorHistory extends BaseCRCXMLNode<Cluster> {    
    public final static class SnapMirrorHistoryKey extends KeyBase<SnapMirrorHistoryKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "snapmirror-history-get-iter",
                zapiResponsePath = "netapp.results.attributes.snapmirror-history-info.operation-id"
            )
            operation_id
        }

        public SnapMirrorHistoryKey(final String operationId) {
            super("snapMirrorHistory",
                new EnumMap<Field, String>(Field.class) {{
                    put(Field.operation_id, operationId);
                }});
        }

        @Override
        @OCIEUUID(uuidDescr = "operation_id")
        public String getOcieUUID() {
            return getID(Field.operation_id);
        }
    }

    private final SnapMirrorHistoryKey key;
    
    private String snapMirrorUUID;
    
    @OCIEZapiSource(
            sourceZapi = "snapmirror-history-get-ite",
            zapiResponsePath = "netapp.results.attributes.snapmirror-history-info.operation-id"
    )
    @OCIEValue(
            ocieColumn = "operationId"
    )
    private final String operationId;
    
    @OCIEZapiSource(
            sourceZapi = "snapmirror-history-get-ite",
            zapiResponsePath = "netapp.results.attributes.snapmirror-history-info.additional-info"
    )
    @OCIEValue(
            ocieColumn = "additionalInfo"
    )
    private String additionalInfo;
    
    @OCIEZapiSource(
            sourceZapi = "snapmirror-history-get-ite",
            zapiResponsePath = "netapp.results.attributes.snapmirror-history-info.start-time"
    )
    @OCIEValue(
            ocieColumn = "startTime"
    )
    private Long startTime;
    
    @OCIEZapiSource(
            sourceZapi = "snapmirror-history-get-ite",
            zapiResponsePath = "netapp.results.attributes.snapmirror-history-info.end-time"
    )
    @OCIEValue(
            ocieColumn = "endTime"
    )
    private Long endTime;
    
    @OCIEZapiSource(
            sourceZapi = "snapmirror-history-get-ite",
            zapiResponsePath = "netapp.results.attributes.snapmirror-history-info.transfer-size"
    )
    @OCIEValue(
            ocieColumn = "transferSize"
    )
    private Long transferSize;
    
    @OCIEZapiSource(
            sourceZapi = "snapmirror-history-get-ite",
            zapiResponsePath = "netapp.results.attributes.snapmirror-history-info.operation-type"
    )
    @OCIEValue(
            ocieColumn = "operationType"
    )
    private OperationType operationType;
    
    @OCIEZapiSource(
            sourceZapi = "snapmirror-history-get-ite",
            zapiResponsePath = "netapp.results.attributes.snapmirror-history-info.operation-type"
    )
    @OCIEValue(
            ocieColumn = "operationTypeRaw"
    )
    private String operationTypeRaw;
    
    @OCIEZapiSource(
            sourceZapi = "snapmirror-history-get-ite",
            zapiResponsePath = "netapp.results.attributes.snapmirror-history-info.result"
    )
    @OCIEValue(
            ocieColumn = "result"
    )
    private Result result;
    
    @OCIEZapiSource(
            sourceZapi = "snapmirror-history-get-ite",
            zapiResponsePath = "netapp.results.attributes.snapmirror-history-info.result"
    )
    @OCIEValue(
            ocieColumn = "resultRaw"
    )
    private String resultRaw;
    
    public enum OperationType {
        NOT_MAPPED,
        CREATE,
        MODIFY,
        QUIESCE,
        RESUME,
        DELETE,
        INITIALIZE,
        MANUAL_UPDATE,
        SCHEDULED_UPDATE,
        BREAK,
        RESYNC,
        ABORT,
        RESTORE
    }
    
    public enum Result {
        NOT_MAPPED,
        SUCCESS,
        FAILURE
    }
    
    public enum RelationshipGroupType {
        NOT_MAPPED,
        NONE,
        VSERVER,
        INFINITEVOL,
    }
    
    SnapMirrorHistory(final Cluster parent, final String operationId) {
        super(parent, SnapMirrorHistory.class.getSimpleName());
        this.key = new SnapMirrorHistoryKey(operationId);
        this.operationId = operationId;
    }
     
    @SuppressWarnings("unchecked")
    @Override
    public SnapMirrorHistoryKey getKey() {
        return key;
    }
    
    public String getAdditionalInfo() {
        return additionalInfo;
    }

    public void setAdditionalInfo(String additionalInfo) {
        this.additionalInfo = additionalInfo;
    }
    
    public Long getStartTime() {
        return startTime;
    }
    
    public void setStartTime(Long startTime) {
        this.startTime = startTime;
    }

    public Long getEndTime() {
        return endTime;
    }

    public void setEndTime(Long endTime) {
        this.endTime = endTime;
    }

    public Long getTransferSize() {
        return transferSize;
    }
    
    public void setTransferSize(Long transferSize) {
        this.transferSize = transferSize;
    }
        
    public OperationType getOperationType() {
        return operationType;
    }
   
    public void setOperationType(OperationType operationType) {
        this.operationType = operationType;
    }
   
    public String getOperationTypeRaw() {
        return operationTypeRaw;
    }
    
    public void setOperationTypeRaw(String operationTypeRaw) {
        this.operationTypeRaw = operationTypeRaw;
    }
  
    public Result getResult() {
        return result;
    }
    
    public void setResult(Result result) {
        this.result = result;
    }
    
    public String getResultRaw() {
        return resultRaw;
    }
    
    public void setResultRaw(String resultRaw) {
        this.resultRaw = resultRaw;
    }
    
    public String getSnapMirrorUUID() {
        return snapMirrorUUID;
    }
    
    public void setSnapMirrorUUID(String snapMirrorUUID) {
        this.snapMirrorUUID = snapMirrorUUID;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "SnapMirrorUUID", snapMirrorUUID);
        addRequiredAttributeAndCRC(writer, "AdditionalInfo", additionalInfo);
        addRequiredAttributeAndCRC(writer, "StartTime", startTime);
        addRequiredAttributeAndCRC(writer, "EndTime", endTime);
        addRequiredAttributeAndCRC(writer, "TransferSize", transferSize);
        addRequiredAttributeAndCRC(writer, "OperationType", operationType);
        addRequiredAttributeAndCRC(writer, "OperationTypeRaw", operationTypeRaw);
        addRequiredAttributeAndCRC(writer, "Result", result);
        addRequiredAttributeAndCRC(writer, "ResultRaw", resultRaw);
    }
    
}
