package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("snap_mirror_policy")
@OCIEResourceType("snap_mirror_policy")
public class SnapMirrorPolicy extends BaseCRCXMLNode<Cluster> {

    public final static class SnapMirrorPolicyKey extends KeyBase<SnapMirrorPolicyKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "vserver-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.vserver-info.uuid"
            )
            vserver_uuid,

            @OCIEZapiSource(
                sourceZapi = "snapmirror-policy-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.snapmirror-policy-info.policy-name"
            )
            name
        }

        public SnapMirrorPolicyKey(final String clusterUUID, final String vServerUUID, final String policyName) {
            super("snap_mirror_policy", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.vserver_uuid, vServerUUID);
                put(Field.name, policyName);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "vserver_uuid.name")
        public String getOcieUUID() {
            final String clusterUUID = getID(Field.cluster_uuid);
            final String vServerUUID = getID(Field.vserver_uuid);
            final String fqVServerUUID = VServer.VServerKey.getOCIEVServerUUID(clusterUUID, vServerUUID);
            final String name = getID(Field.name);
            return fqVServerUUID + '.' + name;
        }
    }

    private final SnapMirrorPolicyKey policyKey;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-policy-info.vserver-name"
    )
    @OCIEValue(
        ocieColumn = "vserverId"
    )
    private final VServer.VServerKey vServerKey;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-policy-info.comment"
    )
    @OCIEValue(
        ocieColumn = "commnet"
    )
    private String comment;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-policy-info.total-keep"
    )
    @OCIEValue(
        ocieColumn = "totalKeep"
    )
    private Long totalKeep;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-policy-info.transfer-priority"
    )
    @OCIEValue(
        ocieColumn = "transferPriority"
    )
    private TransferPriority transferPriority;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-policy-info.transfer-priority"
    )
    @OCIEValue(
        ocieColumn = "transferPriority"
    )
    private String transferPriorityRaw;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-policy-info.tries"
    )
    @OCIEValue(
        ocieColumn = "tries"
    )
    private String tries;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-policy-info.restart"
    )
    @OCIEValue(
        ocieColumn = "restart"
    )
    private Restart restart;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-policy-info.restart"
    )
    @OCIEValue(
        ocieColumn = "restartRaw"
    )
    private String restartRaw;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-policy-info.ignore-atime"
    )
    @OCIEValue(
        ocieColumn = "ignoreAccessTime"
    )
    private Boolean ignoreAccessTime;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-policy-info.create-snapshot"
    )
    @OCIEValue(
        ocieColumn = "createSnapshot"
    )
    private Boolean createSnapshot;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-policy-info.is-network-compression-enabled"
    )
    @OCIEValue(
        ocieColumn = "isNetworkCompressionEnabled"
    )
    private Boolean isNetworkCompressionEnabled;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-policy-info.type"
    )
    @OCIEValue(
        ocieColumn = "type"
    )
    private Type type;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-policy-info.type"
    )
    @OCIEValue(
        ocieColumn = "typeRaw"
    )
    private String typeRaw;

    private final SnapMirrorPolicyRules snapMirrorPolicyRules;

    SnapMirrorPolicy(final Cluster parent, final VServer.VServerKey vServerKey, final String policyName) {
        super(parent, "SnapMirrorPolicy");
        this.vServerKey = vServerKey;
        final String vServerUUID = vServerKey.getID(VServer.VServerKey.Field.uuid);
        this.policyKey = new SnapMirrorPolicyKey(parent.getClusterUUID(), vServerUUID, policyName);
        this.snapMirrorPolicyRules = new SnapMirrorPolicyRules(this);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public SnapMirrorPolicyKey getKey() {
        return policyKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getComment() {
        return comment;
    }

    public void setComment(final String comment) {
        this.comment = comment;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getTotalKeep() {
        return totalKeep;
    }

    public void setTotalKeep(final Long totalKeep) {
        this.totalKeep = totalKeep;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public TransferPriority getTransferPriority() {
        return transferPriority;
    }

    public void setTransferPriority(final TransferPriority transferPriority) {
        this.transferPriority = transferPriority;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getTransferPriorityRaw() {
        return transferPriorityRaw;
    }

    public void setTransferPriorityRaw(final String transferPriorityRaw) {
        this.transferPriorityRaw = transferPriorityRaw;
    }

    public SnapMirrorPolicyRule createSnapMirrorPolicyRule(final int index) {
        return new SnapMirrorPolicyRule(this, index);
    }

    public SnapMirrorPolicyRules getSnapMirrorPolicyRules() {
        return snapMirrorPolicyRules;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getTypeRaw() {
        return typeRaw;
    }

    public void setTypeRaw(String typeRaw) {
        this.typeRaw = typeRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Type getType() {
        return type;
    }

    public void setType(Type type) {
        this.type = type;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean getNetworkCompressionEnabled() {
        return isNetworkCompressionEnabled;
    }

    public void setNetworkCompressionEnabled(Boolean networkCompressionEnabled) {
        isNetworkCompressionEnabled = networkCompressionEnabled;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean getCreateSnapshot() {
        return createSnapshot;
    }

    public void setCreateSnapshot(Boolean createSnapshot) {
        this.createSnapshot = createSnapshot;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean getIgnoreAccessTime() {
        return ignoreAccessTime;
    }

    public void setIgnoreAccessTime(Boolean ignoreAccessTime) {
        this.ignoreAccessTime = ignoreAccessTime;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getRestartRaw() {
        return restartRaw;
    }

    public void setRestartRaw(String restartRaw) {
        this.restartRaw = restartRaw;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Restart getRestart() {
        return restart;
    }

    public void setRestart(Restart restart) {
        this.restart = restart;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getTries() {
        return tries;
    }

    public void setTries(String tries) {
        this.tries = tries;
    }

    /**
     * Specifies the priority at which a SnapMirror transfer runs.
     * 'normal' : By default every transfer in the system has this priority. These transfers are scheduled before most 'low' priority transfers.
     * 'low': These transfers have least priority and usually are scheduled after most 'normal' priority transfers.
     */
    @SuppressWarnings({"UnusedDeclaration"})
    public static enum TransferPriority {
        NOT_MAPPED,
        NORMAL,
        LOW
    }

    public static enum Restart {
        NOT_MAPPED,
        ALWAYS,
        NEVER,
        DEFAULT
    }

    public static enum Type {
        NOT_MAPPED,
        VAULT,
        ASYNC_MIRROR,
        MIRROR_VAULT,
        SYNC_MIRROR
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", policyKey);
        addRequiredAttributeAndCRC(writer, "UUID", policyKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "VserverUUID", vServerKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Name", policyKey.getID(SnapMirrorPolicyKey.Field.name));
        addRequiredAttributeAndCRC(writer, "Comment", comment);
        addRequiredAttributeAndCRC(writer, "CreateSnapshot", createSnapshot);
        addRequiredAttributeAndCRC(writer, "IgnoreAccessTime", ignoreAccessTime);
        addRequiredAttributeAndCRC(writer, "IsNetworkCompressionEnabled", isNetworkCompressionEnabled);
        addRequiredAttributeAndCRC(writer, "Restart", restart);
        addRequiredAttributeAndCRC(writer, "RestartRaw", restartRaw);
        addRequiredAttributeAndCRC(writer, "TotalKeep", totalKeep);
        addRequiredAttributeAndCRC(writer, "TransferPriority", transferPriority);
        addRequiredAttributeAndCRC(writer, "TransferPriorityRaw", transferPriorityRaw);
        addRequiredAttributeAndCRC(writer, "Tries", tries);
        addRequiredAttributeAndCRC(writer, "Type", type);
        addRequiredAttributeAndCRC(writer, "TypeRaw", typeRaw);
    }

    @Override
    protected boolean hasChildren() {
        return !snapMirrorPolicyRules.isEmpty();
    }

    @Override
    protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        addAllChildrenUnderNodeAndCRC(cache, writer, "SnapMirrorPolicyRules", snapMirrorPolicyRules.values(), tabs);
    }

}
