package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("snap_mirror_policy_rule")
@OCIEResourceType("snap_mirror_policy_rule")
public class SnapMirrorPolicyRule extends BaseCRCXMLNode<SnapMirrorPolicy> {

    public static final class PolicyRuleKey extends KeyBase<PolicyRuleKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "vserver-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.vserver-info.uuid"
            )
            vserver_uuid,

            @OCIEZapiSource(
                sourceZapi = "snapmirror-policy-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.snapmirror-policy-info.policy-name"
            )
            snapmirror_policy_name,

            // Index is derived
            @OCIEZapiSource(
                    sourceZapi = "snapmirror-policy-get-iter",
                    zapiResponsePath = "generated ordinal id (count) of policy"
            )
            uuid                        // is actually the index but resource key wants the field named "uuid"
        }

        public PolicyRuleKey(final String clusterUUID, final String vServerUUID, final String snapmirrorPolicyName, final String index) {
            super("snap_mirror_policy_rule", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.vserver_uuid, vServerUUID);
                put(Field.snapmirror_policy_name, snapmirrorPolicyName);
                put(Field.uuid, index);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "vserver_uuid.snapmirror_policy_name.uuid")
        public String getOcieUUID() {
            final String clusterUUID = getID(Field.cluster_uuid);
            final String vServerUUID = getID(Field.vserver_uuid);
            final String fqVServerUUID = VServer.VServerKey.getOCIEVServerUUID(clusterUUID, vServerUUID);
            final String policyName = getID(Field.snapmirror_policy_name);
            final String uuid = getID(Field.uuid);
            return fqVServerUUID + '.' + policyName + '.' + uuid;
        }
    }

    private final PolicyRuleKey ruleKey;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-policy-info.snapmirror-policy-rules.snapmirror-policy-rule-info.keep"
    )
    @OCIEValue(
        ocieColumn = "keep"
    )
    private String keep;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-policy-info.snapmirror-policy-rules.snapmirror-policy-rule-info.preserve"
    )
    @OCIEValue(
        ocieColumn = "preserve"
    )
    private Boolean preserve;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-policy-info.snapmirror-policy-rules.snapmirror-policy-rule-info.snapmirror-label"
    )
    @OCIEValue(
        ocieColumn = "snapMirrorLabel"
    )
    private String label;

    @OCIEZapiSource(
        sourceZapi = "snapmirror-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapmirror-policy-info.snapmirror-policy-rules.snapmirror-policy-rule-info.warn"
    )
    @OCIEValue(
        ocieColumn = "warn"
    )
    private Long warn;

    SnapMirrorPolicyRule(final SnapMirrorPolicy parent, final int index) {
        super(parent, "SnapMirrorPolicyRule");
        final SnapMirrorPolicy.SnapMirrorPolicyKey parentKey = parent.getKey();
        final String clusterUUID = parentKey.getID(SnapMirrorPolicy.SnapMirrorPolicyKey.Field.cluster_uuid);
        final String vServerUUID = parentKey.getID(SnapMirrorPolicy.SnapMirrorPolicyKey.Field.vserver_uuid);
        final String policyName = parentKey.getID(SnapMirrorPolicy.SnapMirrorPolicyKey.Field.name);
        this.ruleKey = new PolicyRuleKey(clusterUUID, vServerUUID, policyName, String.valueOf(index));
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public PolicyRuleKey getKey() {
        return ruleKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getKeep() {
        return keep;
    }

    public void setKeep(final String keep) {
        this.keep = keep;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean getPreserve() {
        return preserve;
    }

    public void setPreserve(final Boolean preserve) {
        this.preserve = preserve;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getLabel() {
        return label;
    }

    public void setLabel(final String label) {
        this.label = label;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getWarn() {
        return warn;
    }

    public void setWarn(final Long warn) {
        this.warn = warn;
    }



    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", ruleKey);
        addRequiredAttributeAndCRC(writer, "UUID", ruleKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Keep", keep);
        addRequiredAttributeAndCRC(writer, "Preserve", preserve);
        addRequiredAttributeAndCRC(writer, "SnapMirrorLabel", label);
        addRequiredAttributeAndCRC(writer, "Warn", warn);
    }

}
