package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("snapshot")
@OCIEResourceType("snapshot")
public class Snapshot extends BaseCRCXMLNode<VServer> {

    public final static class SnapshotKey extends KeyBase<SnapshotKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "volume-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-id-attributes.uuid"
            )
            volume_uuid,

            @OCIEZapiSource(
                sourceZapi = "snapshot-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.snapshot-info.snapshot-instance-uuid"
            )
            snapshot_uuid
        }

        public SnapshotKey(final String clusterUUID, final String volumeInstanceUUID, final String snapshotInstanceUUID) {
            super("snapshot", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.volume_uuid, volumeInstanceUUID);
                put(Field.snapshot_uuid, snapshotInstanceUUID);
            }});
        }

        @Override
        @OCIEUUID(uuidDescr = "volume_uuid.snapshot_uuid")
        public String getOcieUUID() {
            return getID(Field.volume_uuid) + '.' + getID(Field.snapshot_uuid);
        }
    }

    private final SnapshotKey snapshotKey;

    @OCIEZapiSource(
         sourceZapi = "snapshot-get-iter",
         zapiResponsePath = "netapp.results.attributes-list.snapshot-info.access-time"
    )
    @OCIEValue(
        ocieColumn = "accessTime"
    )
    private Long accessTime;

    @OCIEZapiSource(
         sourceZapi = "snapshot-get-iter",
         zapiResponsePath = "netapp.results.attributes-list.snapshot-info.busy"
    )
    @OCIEValue(
        ocieColumn = "isBusy"
    )
    private Boolean isBusy;

    @OCIEZapiSource(
         sourceZapi = "snapshot-get-iter",
         zapiResponsePath = "netapp.results.attributes-list.snapshot-info.cumulative-percentage-of-total-blocks"
    )
    @OCIEValue(
        ocieColumn = "cumulativePercentageOfTotalBlocks"
    )
    private Long cumulativePercentageOfTotalBlocks;

    @OCIEZapiSource(
         sourceZapi = "snapshot-get-iter",
         zapiResponsePath = "netapp.results.attributes-list.snapshot-info.dependency"
    )
    @OCIEValue(
        ocieColumn = "dependency"
    )
    private String dependency;

    @OCIEZapiSource(
         sourceZapi = "snapshot-get-iter",
         zapiResponsePath = "netapp.results.attributes-list.snapshot-info.name"
    )
    @OCIEValue(
        ocieColumn = "name"
    )
    private String name;

    @OCIEZapiSource(
         sourceZapi = "snapshot-get-iter",
         zapiResponsePath = "netapp.results.attributes-list.snapshot-info.percentage-of-total-blocks"
    )
    @OCIEValue(
        ocieColumn = "percentageOfTotalBlocks"
    )
    private Integer percentageOfTotalBlocks;

    @OCIEZapiSource(
         sourceZapi = "snapshot-get-iter",
         zapiResponsePath = "netapp.results.attributes-list.snapshot-info.snapshot-version-uuid"
    )
    @OCIEValue(
        ocieColumn = "versionUUID"
    )
    private String versionUUID;

    Snapshot(final VServer parent, final String volumeInstanceUUID, final String snapshotInstanceUUID) {
        super(parent, "Snapshot");
        this.snapshotKey = new SnapshotKey(parent.getKey().getID(VServer.VServerKey.Field.cluster_uuid), volumeInstanceUUID, snapshotInstanceUUID);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public SnapshotKey getKey() {
        return this.snapshotKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getAccessTime() {
        return accessTime;
    }

    public void setAccessTime(final Long accessTime) {
        this.accessTime = accessTime;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isBusy() {
        return isBusy;
    }

    public void setBusy(final Boolean busy) {
        isBusy = busy;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getCumulativePercentageOfTotalBlocks() {
        return cumulativePercentageOfTotalBlocks;
    }

    public void setCumulativePercentageOfTotalBlocks(final Long cumulativePercentageOfTotalBlocks) {
        this.cumulativePercentageOfTotalBlocks = cumulativePercentageOfTotalBlocks;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getDependency() {
        return dependency;
    }

    public void setDependency(final String dependency) {
        this.dependency = dependency;
    }

    public String getName() {
        return name;
    }

    public void setName(final String name) {
        this.name = name;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getPercentageOfTotalBlocks() {
        return percentageOfTotalBlocks;
    }

    public void setPercentageOfTotalBlocks(final Integer percentageOfTotalBlocks) {
        this.percentageOfTotalBlocks = percentageOfTotalBlocks;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getVersionUUID() {
        return versionUUID;
    }

    public void setVersionUUID(final String versionUUID) {
        this.versionUUID = versionUUID;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getVolumeUUID() {
        return snapshotKey.getID(SnapshotKey.Field.volume_uuid);
    }




    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", snapshotKey);
        addRequiredAttributeAndCRC(writer, "UUID", snapshotKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "InstanceUUID", snapshotKey.getID(SnapshotKey.Field.snapshot_uuid));   // JSD should be a key and .getTailIDs()
        addRequiredAttributeAndCRC(writer, "VolumeUUID", snapshotKey.getID(SnapshotKey.Field.volume_uuid));
        addRequiredAttributeAndCRC(writer, "VersionUUID", versionUUID);
        addRequiredAttributeAndCRC(writer, "Name", name);
        addRequiredAttributeAndCRC(writer, "IsBusy", isBusy);
        addRequiredAttributeAndCRC(writer, "AccessTime", accessTime);
        addRequiredAttributeAndCRC(writer, "Dependency", dependency);
        addRequiredAttributeAndCRC(writer, "CumulativePercentageOfTotalBlocks", cumulativePercentageOfTotalBlocks);
        addRequiredAttributeAndCRC(writer, "PercentageOfTotalBlocks", percentageOfTotalBlocks);
    }
}
