package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;
import java.util.EnumSet;


@OCIETable("snapshot_policy")
@OCIEResourceType("snapshot_policy")
public class SnapshotPolicy extends BaseCRCXMLNode<Cluster> {

    public final static class SnapshotPolicyKey extends KeyBase<SnapshotPolicyKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "volume-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-id-attributes.uuid"
            )
            vserver_uuid,

            @OCIEZapiSource(
                sourceZapi = "snapshot-policy-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.snapshot-policy-info.policy"
            )
            name
        }

        public SnapshotPolicyKey(final String clusterUUID, final String vServerUUID, final String policyName) {
            super("snapshot_policy", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.vserver_uuid, vServerUUID);
                put(Field.name, policyName);
            }}, EnumSet.of(Field.vserver_uuid));                      // These EnumSet<E> emptyAbleFields fields are allowed to be empty but not null
        }

        @Override
        @OCIEUUID(uuidDescr = "cluster_uuid.vserver_uuid.name")
        public String getOcieUUID() {
            final String clusterUUID = getID(Field.cluster_uuid);
            final String vServerUUID = getID(Field.vserver_uuid);
            final String fqVServerUUID = VServer.VServerKey.getOCIEVServerUUID(clusterUUID, vServerUUID);
            final String policyName = getID(Field.name);
            return clusterUUID + '.' + fqVServerUUID + '.' + policyName;
        }
    }


    private final SnapshotPolicyKey policyKey;

    @OCIEZapiSource(
        sourceZapi = "snapshot-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapshot-policy-info.vserver-name"
    )
    @OCIEValue(
        ocieColumn = "vserverId"
    )
    private final VServer.VServerKey vServerKey;

    @OCIEZapiSource(
        sourceZapi = "snapshot-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapshot-policy-info.vserver-name"
    )
    @OCIEValue(
        ocieColumn = "enabled"
    )
    private Boolean enabled;

    @OCIEZapiSource(
        sourceZapi = "snapshot-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapshot-policy-info.comment"
    )
    @OCIEValue(
        ocieColumn = "comment"
    )
    private String comment;
    private final SnapshotPolicySchedules snapshotPolicySchedules;

    SnapshotPolicy(final Cluster parent, final VServer.VServerKey vServerKey, final String policyName) {
        super(parent, "SnapshotPolicy");
        this.vServerKey = vServerKey;
        final String vServerUUID = (vServerKey == null) ? "" : vServerKey.getID(VServer.VServerKey.Field.uuid);
        this.policyKey = new SnapshotPolicyKey(parent.getClusterUUID(), vServerUUID, policyName);
        this.snapshotPolicySchedules = new SnapshotPolicySchedules(this);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public SnapshotPolicyKey getKey() {
        return policyKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public VServer.VServerKey getVServerKey() {
        return vServerKey;
    }


    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean isEnabled() {
        return enabled;
    }

    public void setEnabled(final Boolean enabled) {
        this.enabled = enabled;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getComment() {
        return comment;
    }

    public void setComment(final String comment) {
        this.comment = comment;
    }

    public SnapshotPolicySchedule createSnapshotPolicySchedule(final String scheduleName) {
        final String vServerUUID = policyKey.getID(SnapshotPolicyKey.Field.vserver_uuid);
        final String policyName = policyKey.getID(SnapshotPolicyKey.Field.name);
        return new SnapshotPolicySchedule(this, vServerUUID, policyName, scheduleName);
    }

    public SnapshotPolicySchedules getSnapshotPolicySchedules() {
        return snapshotPolicySchedules;
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", policyKey);
        addRequiredAttributeAndCRC(writer, "UUID", policyKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "Name", policyKey.getID(SnapshotPolicyKey.Field.name));
        // VserverUUID   Rendered later in a relationship table
        addRequiredAttributeAndCRC(writer, "Comment", comment);
        addRequiredAttributeAndCRC(writer, "IsEnabled", enabled);

    }

    @Override
    protected boolean hasChildren() {
        return !snapshotPolicySchedules.isEmpty();
    }

    @Override
    protected void marshallChildrenToFileAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        addAllChildrenUnderNodeAndCRC(cache, writer, "SnapshotPolicySchedules", snapshotPolicySchedules.values(), tabs);
    }
}
