package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;
import java.util.EnumSet;

@OCIETable("snapshot_policy_schedule")
@OCIEResourceType("snapshot_policy_schedule")
public class SnapshotPolicySchedule extends BaseCRCXMLNode<SnapshotPolicy> {

    public static final class PolicyScheduleKey extends KeyBase<PolicyScheduleKey.Field> {
        public static enum Field {

            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "volume-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-id-attributes.uuid"
            )
            vserver_uuid,

            @OCIEZapiSource(
                sourceZapi = "snapshot-policy-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.snapshot-policy-info.policy"
            )
            snapshot_policy_name,

            @OCIEZapiSource(
                sourceZapi = "snapshot-policy-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.snapshot-policy-info.snapshot-policy-schedules.snapshot-schedule-info.schedule"
            )
            name
        }

        public PolicyScheduleKey(final String clusterUUID, final String vServerUUID, final String policyName, final String scheduleName) {
            super("snapshot_policy_schedule", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.vserver_uuid, vServerUUID);
                put(Field.snapshot_policy_name, policyName);
                put(Field.name, scheduleName);                          // "prefix" from ZAPI but Resource Key wants the field named 'name'
            }}, EnumSet.of(Field.vserver_uuid));                      // These EnumSet<E> emptyAbleFields fields are allowed to be empty but not null
        }

        @Override
        @OCIEUUID(uuidDescr = "cluster_uuid.vserver_uuid.name.snapshot_policy_name.name")
        public String getOcieUUID() {
            final String clusterUUID = getID(Field.cluster_uuid);
            final String vServerUUID = getID(Field.vserver_uuid);
            final String fqVServerUUID = VServer.VServerKey.getOCIEVServerUUID(clusterUUID, vServerUUID); // may be null
            final String snapshotPolicyName = getID(Field.snapshot_policy_name);
            final String name = getID(Field.name);
            final String policySuffix = snapshotPolicyName + '.' + name;
            return (fqVServerUUID == null || fqVServerUUID.isEmpty())
                    ? clusterUUID + ".." + policySuffix
                    : fqVServerUUID + '.' + policySuffix;
        }
    }

    private final PolicyScheduleKey scheduleKey;

    @OCIEZapiSource(
        sourceZapi = "snapshot-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapshot-policy-info.policy"
    )
    @OCIEValue(
        ocieColumn = "snapshotPolicyId"
    )
    private JobSchedule.JobScheduleKey jobScheduleKey;

    @OCIEZapiSource(
        sourceZapi = "snapshot-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapshot-policy-info.vserver-name"
    )
    @OCIEValue(
        ocieColumn = "vserverId"
    )
    private VServer.VServerKey vServerKey;

    @OCIEZapiSource(
        sourceZapi = "snapshot-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapshot-policy-info.snapshot-policy-schedules.snapshot-schedule-info.prefix"
    )
    @OCIEValue(
        ocieColumn = "prefix"
    )
    private String prefix;

    @OCIEZapiSource(
        sourceZapi = "snapshot-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapshot-policy-info.snapshot-policy-schedules.snapshot-schedule-info.count"
    )
    @OCIEValue(
        ocieColumn = "count"
    )
    private Long count;

    @OCIEZapiSource(
        sourceZapi = "snapshot-policy-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.snapshot-policy-info.snapshot-policy-schedules.snapshot-schedule-info.snapmirror-label"
    )
    @OCIEValue(
        ocieColumn = "snapmirrorLabel"
    )
    private String snapmirrorLabel;

    SnapshotPolicySchedule(final SnapshotPolicy parent, final String vServerUUID, final String policyName, final String scheduleName) {
        super(parent, "SnapshotPolicySchedule");
        this.scheduleKey = new PolicyScheduleKey(parent.getKey().getID(SnapshotPolicy.SnapshotPolicyKey.Field.cluster_uuid), vServerUUID, policyName, scheduleName);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public PolicyScheduleKey getKey() {
        return scheduleKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public JobSchedule.JobScheduleKey getJobScheduleKey() {
        return jobScheduleKey;
    }

    public void setJobScheduleKey(final JobSchedule.JobScheduleKey jobScheduleKey) {
        this.jobScheduleKey = jobScheduleKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public VServer.VServerKey getVServerKey() {
        return vServerKey;
    }

    public void setVServerKey(final VServer.VServerKey vServerKey) {
        this.vServerKey = vServerKey;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getPrefix() {
        return prefix;
    }

    public void setPrefix(final String prefix) {
        this.prefix = prefix;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getCount() {
        return count;
    }

    public void setCount(final Long count) {
        this.count = count;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getSnapmirrorLabel() {
        return snapmirrorLabel;
    }

    public void setSnapmirrorLabel(final String snapmirrorLabel) {
        this.snapmirrorLabel = snapmirrorLabel;
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", scheduleKey);
        addRequiredAttributeAndCRC(writer, "UUID", scheduleKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "JobScheduleUUID", KeyBase.getOcieUUIDOrNull(jobScheduleKey));
        addRequiredAttributeAndCRC(writer, "Count", count);
        addRequiredAttributeAndCRC(writer, "Prefix", prefix);
        addRequiredAttributeAndCRC(writer, "SnapMirrorLabel", snapmirrorLabel);
    }


}
