package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.IntegrityException;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import org.apache.log4j.Logger;

import java.io.IOException;
import java.io.Writer;

/**
 * NetApp Foundation Schema implementation of a Storage Root for OCI/cmode/7mode.   It should contain one and only one Cluster.
 */
public class StorageRootImpl extends BaseRootImpl implements StorageRoot {
    private final Clusters clusters;

    StorageRootImpl(final RootFactory parent, final long originatorID, final String originatorType, final String dataSourceName) throws SchemaException {
        super(parent, originatorID, originatorType, dataSourceName);
        clusters = new Clusters(this);
    }

    @Override
    public Clusters getClusters() {
        return clusters;
    }

    @Override
    public Cluster createCluster(final String clusterUUID) {
        return new Cluster(this, clusterUUID);
    }

    /**
     * Perform integrity checks on storage device data
     *
     * @throws IntegrityException
     */
    @Override
    public void checkIntegrity() throws IntegrityException {

    }

    @Override
    protected void doMarshallChildren(final MarshallingCache cache, final Writer writer, final int tabs) throws IOException, SchemaException {
        addAllChildrenUnderNode(cache, writer, "Clusters", clusters.values(), tabs);
    }

    @Override
    protected void doMarshallChildrenAndCRC(final MarshallingCache cache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        addAllChildrenUnderNodeAndCRC(cache, writer, "Clusters", clusters.values(), tabs);
    }

}
