package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEResourceType;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIETable;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("storage_service")
@OCIEResourceType("storage_service")
public class StorageServiceVolumeRelationship extends BaseCRCXMLNode<VServer> {

    private final RelationshipKey RelationshipKey;
    private final StorageService.StorageServiceKey storageServiceKey;
    private final Volume.VolumeKey volumeKey;

    StorageServiceVolumeRelationship(final VServer parent, final StorageService.StorageServiceKey storageServiceKey, final Volume.VolumeKey volumeKey) {
        super(parent, "StorageServiceVolumeRelationship");
        this.RelationshipKey = new RelationshipKey(storageServiceKey.getOcieUUID(), volumeKey.getOcieUUID());
        this.storageServiceKey = storageServiceKey;
        this.volumeKey = volumeKey;
    }


    @Override
    @SuppressWarnings({"unchecked"})
    public RelationshipKey getKey() {
        return RelationshipKey;
    }

    public final static class RelationshipKey extends KeyBase<RelationshipKey.Field> {
        public static enum Field {
            storage_service_uuid,
            volume_uuid
        }

        public RelationshipKey(final String storageServiceUUID, final String volumeUUID) {
            super("storage_service_relationship",
                  new EnumMap<Field, String>(Field.class) {{
                        put(Field.storage_service_uuid, storageServiceUUID);
                        put(Field.volume_uuid, volumeUUID);
                  }});
        }

        public String getOcieUUID() {
            return getFQID();
        }
    }


    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "StorageServiceUUID", storageServiceKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "VolumeUUID", volumeKey.getOcieUUID());
    }
}
