package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEResourceType;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIETable;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEValue;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEZapiSource;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import org.apache.commons.lang3.StringUtils;

import java.io.IOException;
import java.util.EnumMap;
import java.util.EnumSet;

/**
 * @author Iti Behari
 *         Date: 7/19/13
 */

@OCIETable("storage_shelf_cable")
@OCIEResourceType("storage_shelf_cable")
public class StorageShelfCable extends BaseCRCXMLNode<StorageShelf> {

    public final static class StorageShelfCableKey extends KeyBase<StorageShelfCableKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                    sourceZapi = "cluster-identity-get",
                    zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

             @OCIEZapiSource(
                    sourceZapi = "storage-shelf-list-info",
                    zapiArgs = "node-name",
                    zapiResponsePath = "netapp.results.shelf-list.shelf-info.shelf-uid"
            )
            shelf_id,

            @OCIEZapiSource(
                    sourceZapi = "storage-shelf-environment-list-info",
                    zapiArgs = "node-name",
                    zapiResponsePath = "netapp.shelf-environ-channel-list.shelf-environ-channel-info.channel-name"
            )
            channel_name,

            @OCIEZapiSource(
                    sourceZapi = "storage-shelf-environment-list-info",
                    zapiArgs = "node-name",
                    zapiResponsePath = "netapp.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-list.shelf-environ-shelf-info.connector-information.connector-info.sas-connector-list.sas-connector-info.cable-part-no"
            )
            part_number,

            @OCIEZapiSource(
                    sourceZapi = "storage-shelf-environment-list-info",
                    zapiArgs = "node-name",
                    zapiResponsePath = "netapp.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-list.shelf-environ-shelf-info.connector-information.connector-info.sas-connector-list.sas-connector-info.cable-serial-no"
            )
            serial_number,

            @OCIEZapiSource(
                    sourceZapi = "storage-shelf-environment-list-info",
                    zapiArgs = "node-name",
                    zapiResponsePath = "netapp.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-list.shelf-environ-shelf-info.connector-information.connector-info.sas-connector-list.sas-connector-info.connector-no"
            )
            connector_number
        }

        public StorageShelfCableKey(final String clusterUUID, final String shelfId, final String channelName, final String partNumber, final String serialNumber, final String connectorNumber) {
            super("storage_shelf_cable", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.shelf_id, shelfId);
                put(Field.channel_name, channelName);
                put(Field.part_number, (partNumber == null? StringUtils.EMPTY: partNumber));
                put(Field.serial_number, (serialNumber ==null? StringUtils.EMPTY: serialNumber));
                put(Field.connector_number, connectorNumber);
            }}, EnumSet.of(Field.part_number, Field.serial_number));
        }

        public String getOcieUUID() {
            return getFQID();
        }
    }

    private final StorageShelfCableKey key;

    private String nodeUUID;

    @OCIEZapiSource(
            sourceZapi = "storage-shelf-environment-list-info",
            zapiArgs = "node-name",
            zapiResponsePath = "netapp.shelf-environ-channel-list.shelf-environ-channel-info.channel-name"
    )
    @OCIEValue(
            ocieColumn = "channelName"
    )
    private String channelName;

    @OCIEZapiSource(
            sourceZapi = "storage-shelf-environment-list-info",
            zapiArgs = "node-name",
            zapiResponsePath = "netapp.shelf-environ-channel-list.shelf-environ-channel-info.is-shelf-channel-failure"
    )
    @OCIEValue(
            ocieColumn = "isShelfChannelFailure"
    )
    private Boolean isShelfChannelFailure;

    @OCIEZapiSource(
            sourceZapi = "storage-shelf-environment-list-info",
            zapiArgs = "node-name",
            zapiResponsePath = "netapp.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-list.shelf-environ-shelf-info.connector-information.connector-info.sas-connector-list.sas-connector-info.cable-end-identifier"
    )
    @OCIEValue(
            ocieColumn = "endIdentifier"
    )
    private String endIdentifier;

    @OCIEZapiSource(
            sourceZapi = "storage-shelf-environment-list-info",
            zapiArgs = "node-name",
            zapiResponsePath = "netapp.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-list.shelf-environ-shelf-info.connector-information.connector-info.sas-connector-list.sas-connector-info.cable-length"
    )
    @OCIEValue(
            ocieColumn = "length"
    )
    private String length;

    @OCIEZapiSource(
            sourceZapi = "storage-shelf-environment-list-info",
            zapiArgs = "node-name",
            zapiResponsePath = "netapp.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-list.shelf-environ-shelf-info.connector-information.connector-info.sas-connector-list.sas-connector-info.cable-manufacturer"
    )
    @OCIEValue(
            ocieColumn = "manufacturer"
    )
    private String manufacturer;

    @OCIEZapiSource(
            sourceZapi = "storage-shelf-environment-list-info",
            zapiArgs = "node-name",
            zapiResponsePath = "netapp.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-list.shelf-environ-shelf-info.connector-information.connector-info.sas-connector-list.sas-connector-info.cable-part-no"
    )
    @OCIEValue(
            ocieColumn = "partNumber"
    )
    private final String partNumber;

    @OCIEZapiSource(
            sourceZapi = "storage-shelf-environment-list-info",
            zapiArgs = "node-name",
            zapiResponsePath = "netapp.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-list.shelf-environ-shelf-info.connector-information.connector-info.sas-connector-list.sas-connector-info.cable-serial-no"
    )
    @OCIEValue(
            ocieColumn = "serialNumber"
    )
    private String serialNumber;

    @OCIEZapiSource(
            sourceZapi = "storage-shelf-environment-list-info",
            zapiArgs = "node-name",
            zapiResponsePath = "netapp.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-list.shelf-environ-shelf-info.connector-information.connector-info.sas-connector-list.sas-connector-info.attached-serial-no"
    )
    @OCIEValue(
            ocieColumn = "attachedSerialNumber"
    )
    private String attachedSerialNumber;

    @OCIEZapiSource(
            sourceZapi = "storage-shelf-environment-list-info",
            zapiArgs = "node-name",
            zapiResponsePath = "netapp.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-list.shelf-environ-shelf-info.connector-information.connector-info.sas-connector-list.sas-connector-info.cable-technology"
    )
    @OCIEValue(
            ocieColumn = "technology"
    )
    private String technology;

    @OCIEZapiSource(
            sourceZapi = "storage-shelf-environment-list-info",
            zapiArgs = "node-name",
            zapiResponsePath = "netapp.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-list.shelf-environ-shelf-info.connector-information.connector-info.sas-connector-list.sas-connector-info.cable-type"
    )
    @OCIEValue(
            ocieColumn = "type"
    )
    private String type;

    @OCIEZapiSource(
            sourceZapi = "storage-shelf-environment-list-info",
            zapiArgs = "node-name",
            zapiResponsePath = "netapp.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-list.shelf-environ-shelf-info.connector-information.connector-info.sas-connector-list.sas-connector-info.connector-designator"
    )
    @OCIEValue(
            ocieColumn = "connectorDesignator"
    )
    private String connectorDesignator;

    @OCIEZapiSource(
            sourceZapi = "storage-shelf-environment-list-info",
            zapiArgs = "node-name",
            zapiResponsePath = "netapp.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-list.shelf-environ-shelf-info.connector-information.connector-info.sas-connector-list.sas-connector-info.connector-no"
    )
    @OCIEValue(
            ocieColumn = "connectorNumber"
    )
    private final Integer connectorNumber;

    @OCIEZapiSource(
            sourceZapi = "storage-shelf-environment-list-info",
            zapiArgs = "node-name",
            zapiResponsePath = "netapp.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-list.shelf-environ-shelf-info.connector-information.connector-info.sas-connector-list.sas-connector-info.is-cable-connected"
    )
    @OCIEValue(
            ocieColumn = "isCableConnected"
    )
    private Boolean isCableConnected;

    @OCIEZapiSource(
            sourceZapi = "storage-shelf-environment-list-info",
            zapiArgs = "node-name",
            zapiResponsePath = "netapp.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-list.shelf-environ-shelf-info.connector-information.connector-info.sas-connector-list.sas-connector-info.is-connector-error"
    )
    @OCIEValue(
            ocieColumn = "isCableConnected"
    )
    private Boolean isConnectorError;

    @OCIEZapiSource(
            sourceZapi = "storage-shelf-environment-list-info",
            zapiArgs = "node-name",
            zapiResponsePath = "netapp.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-list.shelf-environ-shelf-info.connector-information.connector-info.sas-connector-list.sas-connector-info.swap-count"
    )
    @OCIEValue(
            ocieColumn = "swapCount"
    )
    private Integer swapCount;

    @OCIEZapiSource(
            sourceZapi = "storage-shelf-environment-list-info",
            zapiArgs = "node-name",
            zapiResponsePath = "netapp.shelf-environ-channel-list.shelf-environ-channel-info.shelf-environ-shelf-list.shelf-environ-shelf-info.connector-information.connector-info.sas-connector-list.sas-connector-info.shelf-module"
    )
    @OCIEValue(
            ocieColumn = "shelfModule"
    )
    private String shelfModule;

    StorageShelfCable(final StorageShelf parent, final String nodeUUID, final String shelfId, final String channelName, final String partNumber, final String serialNumber, final Integer connectorNumber) {
        super(parent, StorageShelfCable.class.getSimpleName());
        final String clusterUUID = parent.getKey().getID(StorageShelf.StorageShelfKey.Field.cluster_uuid);
        this.key = new StorageShelfCableKey(clusterUUID,shelfId, channelName, partNumber, serialNumber, String.valueOf(connectorNumber));
        this.channelName = channelName;
        this.partNumber = partNumber;
        this.serialNumber = serialNumber;
        this.connectorNumber = connectorNumber;
        this.nodeUUID = nodeUUID;
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public StorageShelfCableKey getKey() {
        return key;
    }

    public String getChannelName() {
        return channelName;
    }

    public void setChannelName(final String channelName) {
        this.channelName = channelName;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean getShelfChannelFailure() {
        return isShelfChannelFailure;
    }

    public void setShelfChannelFailure(final Boolean shelfChannelFailure) {
        isShelfChannelFailure = shelfChannelFailure;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getEndIdentifier() {
        return endIdentifier;
    }

    public void setEndIdentifier(final String endIdentifier) {
        this.endIdentifier = endIdentifier;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getLength() {
        return length;
    }

    public void setLength(final String length) {
        this.length = length;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getManufacturer() {
        return manufacturer;
    }

    public void setManufacturer(final String manufacturer) {
        this.manufacturer = manufacturer;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getPartNumber() {
        return partNumber;
    }

    public String getSerialNumber() {
        return serialNumber;
    }

    public void setSerialNumber(final String serialNumber) {
        this.serialNumber = serialNumber;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getAttachedSerialNumber() {
        return attachedSerialNumber;
    }

    public void setAttachedSerialNumber(final String attachedSerialNumber) {
        this.attachedSerialNumber = attachedSerialNumber;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getTechnology() {
        return technology;
    }

    public void setTechnology(final String technology) {
        this.technology = technology;
    }

    public String getType() {
        return type;
    }

    public void setType(final String type) {
        this.type = type;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getConnectorDesignator() {
        return connectorDesignator;
    }

    public void setConnectorDesignator(final String connectorDesignator) {
        this.connectorDesignator = connectorDesignator;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getConnectorNumber() {
        return connectorNumber;
    }


    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean getCableConnected() {
        return isCableConnected;
    }

    public void setCableConnected(final Boolean cableConnected) {
        isCableConnected = cableConnected;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean getConnectorError() {
        return isConnectorError;
    }

    public void setConnectorError(final Boolean connectorError) {
        isConnectorError = connectorError;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Integer getSwapCount() {
        return swapCount;
    }

    public void setSwapCount(final Integer swapCount) {
        this.swapCount = swapCount;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getShelfModule() {
        return shelfModule;
    }

    public void setShelfModule(final String shelfModule) {
        this.shelfModule = shelfModule;
    }

    public String getNodeUUID() {
        return nodeUUID;
    }

    public void setNodeUUID(final String uuid) {
        this.nodeUUID = uuid;
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "NodeUUID", getNodeUUID());
        addRequiredAttributeAndCRC(writer, "ChannelName", channelName);
        addRequiredAttributeAndCRC(writer, "SwapCount", swapCount);
        addRequiredAttributeAndCRC(writer, "ShelfModule", shelfModule);
        addRequiredAttributeAndCRC(writer, "IsConnectorError", isConnectorError);
        addRequiredAttributeAndCRC(writer, "IsCableConnected", isCableConnected);
        addRequiredAttributeAndCRC(writer, "ConnectorNumber", connectorNumber);
        addRequiredAttributeAndCRC(writer, "ConnectorDesignator", connectorDesignator);
        addRequiredAttributeAndCRC(writer, "Type", type);
        addRequiredAttributeAndCRC(writer, "Technology", technology);
        addRequiredAttributeAndCRC(writer, "SerialNumber", serialNumber);
        addRequiredAttributeAndCRC(writer, "AttachedSerialNumber", attachedSerialNumber);
        addRequiredAttributeAndCRC(writer, "PartNumber", partNumber);
        addRequiredAttributeAndCRC(writer, "Manufacturer", manufacturer);
        addRequiredAttributeAndCRC(writer, "Length", length);
        addRequiredAttributeAndCRC(writer, "EndIdentifier", endIdentifier);
        addRequiredAttributeAndCRC(writer, "IsShelfChannelFailure", isShelfChannelFailure);
    }

}
