package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

/**
 * Created with IntelliJ IDEA.
 * User: jianwei
 * Date: 1/8/14
 * Time: 4:20 PM
 * To change this template use File | Settings | File Templates.
 */
@OCIETable("system_health_alert")
@OCIEResourceType("system_health_alert")
public class SystemHealthAlert extends BaseCRCXMLNode<Cluster> {
    public final static class SystemHealthAlertKey extends KeyBase<SystemHealthAlertKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                    sourceZapi = "cluster-identity-get",
                    zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                    sourceZapi = "diagnosis-alert-get-iter",
                    zapiResponsePath = "netapp.results.attributes-list.diagnosis-alert-info.alert-id"
            )
            alert_id,

            @OCIEZapiSource(
                    sourceZapi = "diagnosis-alert-get-iter",
                    zapiResponsePath = "netapp.results.attributes-list.diagnosis-alert-info.alerting-resource"
            )
            alerting_resource,
        }

        public SystemHealthAlertKey(final String clusterUUID, final String alertID, final String alertingResource) {
            super("system_health_alert", new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.alert_id, alertID);
                put(Field.alerting_resource, alertingResource);
            }});
        }

        public String getOcieUUID() {
            final String clusterId = getID(Field.cluster_uuid);
            final String alertID = getID(Field.alert_id);
            final String alertingResource = getID(Field.alerting_resource);
            // Include extra dot when target not defined for backward compatibility
            StringBuilder builder = new StringBuilder();
            builder.append(clusterId).append('.').append(alertID).append('.').append(alertingResource);

            return builder.toString();
        }
    }

    private final SystemHealthAlertKey key;

    @OCIEZapiSource(
            sourceZapi = "diagnosis-alert-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.diagnosis-alert-info.indication-time"
    )
    private Long indication_time;

    @OCIEZapiSource(
            sourceZapi = "diagnosis-alert-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.diagnosis-alert-info.alerting-resource-name"
    )
    private String alerting_resource_name;

    @OCIEZapiSource(
            sourceZapi = "diagnosis-alert-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.diagnosis-alert-info.monitor"
    )
    @OCIEValue(
            ocieColumn = "monitor"
    )
    private MonitorType monitor;

    @OCIEZapiSource(
            sourceZapi = "diagnosis-alert-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.diagnosis-alert-info.monitor"
    )
    @OCIEValue(
            ocieColumn = "monitorRaw"
    )
    private String monitorRaw;

    @OCIEZapiSource(
            sourceZapi = "diagnosis-alert-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.diagnosis-alert-info.node"
    )
    @OCIEValue(
            ocieColumn = "nodeID"
    )
    private String nodeID;

    @OCIEZapiSource(
            sourceZapi = "diagnosis-alert-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.diagnosis-alert-info.perceived-severity"    )
    @OCIEValue(
            ocieColumn = "perceivedSeverity"
    )
    private String perceivedSeverity;

    @OCIEZapiSource(
            sourceZapi = "diagnosis-alert-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.diagnosis-alert-info.policy"
    )
    @OCIEValue(
            ocieColumn = "policy"
    )
    private String policy;

    @OCIEZapiSource(
            sourceZapi = "diagnosis-alert-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.diagnosis-alert-info.possible-effect"
    )
    @OCIEValue(
            ocieColumn = "possibleEffect"
    )
    private String possibleEffect;

    @OCIEZapiSource(
            sourceZapi = "diagnosis-alert-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.diagnosis-alert-info.corrective-actions"
    )
    @OCIEValue(
            ocieColumn = "correctiveActions"
    )
    private String correctiveActions;

    @OCIEZapiSource(
            sourceZapi = "diagnosis-alert-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.diagnosis-alert-info.probable-cause"
    )
    @OCIEValue(
            ocieColumn = "probableCause"
    )
    private String probableCause;

    @OCIEZapiSource(
            sourceZapi = "diagnosis-alert-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.diagnosis-alert-info.probable-cause-description"
    )
    @OCIEValue(
            ocieColumn = "probableCauseDescrition"
    )
    private String probableCauseDescription;

    @OCIEZapiSource(
            sourceZapi = "diagnosis-alert-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.diagnosis-alert-info.subsystem"
    )
    @OCIEValue(
            ocieColumn = "subsystem"
    )
    private String subsystem;

    @OCIEZapiSource(
            sourceZapi = "diagnosis-alert-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.diagnosis-alert-info.additional-info"
    )
    @OCIEValue(
            ocieColumn = "additionalInfo"
    )
    private String additionalInfo;

    @OCIEZapiSource(
            sourceZapi = "diagnosis-alert-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.diagnosis-alert-info.acknowledge"
    )
    @OCIEValue(
            ocieColumn = "acknowledge"
    )
    private Boolean acknowledge;

    @OCIEZapiSource(
            sourceZapi = "diagnosis-alert-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.diagnosis-alert-info.acknowledger"
    )
    @OCIEValue(
            ocieColumn = "acknowledger"
    )
    private String acknowledger;

    @OCIEZapiSource(
            sourceZapi = "diagnosis-alert-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.diagnosis-alert-info.suppress"
    )
    @OCIEValue(
            ocieColumn = "suppress"
    )
    private Boolean suppress;

    @OCIEZapiSource(
            sourceZapi = "diagnosis-alert-get-iter",
            zapiResponsePath = "netapp.results.attributes-list.diagnosis-alert-info.suppressor"
    )
    @OCIEValue(
            ocieColumn = "suppressor"
    )
    private String suppressor;


    SystemHealthAlert(final Cluster parent, final String alertID, final String alertingResource) {
        super(parent, "SystemHealthAlert");
        final String clusterUUID = parent.getKey().getOcieUUID();
        this.key = new SystemHealthAlertKey(clusterUUID, alertID, alertingResource);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public SystemHealthAlertKey getKey() {
        return key;
    }

    public String getClusterId() {
        return key.getID(SystemHealthAlertKey.Field.cluster_uuid);
    }

    public String getAlertId() {
        return key.getID(SystemHealthAlertKey.Field.alert_id);
    }

    public Long getIndicationTime() {
        return indication_time;
    }

    public String getAlertingResourceName() {
        return alerting_resource_name;
    }

    public void setAlertingResourceName(String alertingResourceName) {
        this.alerting_resource_name = alertingResourceName;
    }

    public void setIndicationTime(Long indicationTime)  {
        this.indication_time = indicationTime;
    }

    public String getAlertingResource() {
        return key.getID(SystemHealthAlertKey.Field.alerting_resource);
    }

    public MonitorType getMonitor() {
        return monitor;
    }

    public void setMonitor(MonitorType monitor) {
        this.monitor = monitor;
    }

    public String getMonitorRaw() {
        return monitorRaw;
    }

    public void setMonitorRaw(String monitorRaw) {
        this.monitorRaw = monitorRaw;
    }



    public String getNodeID() {
        return nodeID;
    }

    public void setNodeID(String nodeID) {
        this.nodeID = nodeID;
    }

    public String getPerceivedSeverity() {
        return perceivedSeverity;
    }

    public void setPerceivedSeverity(String perceivedSeverity) {
        this.perceivedSeverity = perceivedSeverity;
    }

    public String getPolicy() {
        return policy;
    }

    public void setPolicy(String policy) {
        this.policy = policy;
    }

    public String getPossibleEffect() {
        return possibleEffect;
    }

    public void setPossibleEffect(String possibleEffect) {
        this.possibleEffect = possibleEffect;
    }

    public String getProbableCause() {
        return probableCause;
    }

    public String getCorrectiveActions() {
        return correctiveActions;
    }

    public void setCorrectiveActions(String correctiveActions) {
        this.correctiveActions = correctiveActions;
    }

    public void setProbableCause(String probableCause) {
        this.probableCause = probableCause;
    }

    public String getProbableCauseDescription() {
        return probableCauseDescription;
    }

    public void setProbableCauseDescription(String probableCauseDescrition) {
        this.probableCauseDescription = probableCauseDescrition;
    }

    public String getSubsystem() {
        return subsystem;
    }

    public void setSubsystem(String subsystem) {
        this.subsystem = subsystem;
    }

    public String getAdditionalInfo() {
        return additionalInfo;
    }

    public void setAdditionalInfo(String additionalInfo) {
        this.additionalInfo = additionalInfo;
    }

    public Boolean getAcknowledge() {
        return acknowledge;
    }

    public void setAcknowledge(Boolean acknowledge) {
        this.acknowledge = acknowledge;
    }

    public String getAcknowledger() {
        return acknowledger;
    }

    public void setAcknowledger(String acknowledger) {
        this.acknowledger = acknowledger;
    }

    public Boolean getSuppress() {
        return suppress;
    }

    public void setSuppress(Boolean suppress) {
        this.suppress = suppress;
    }

    public String getSuppressor() {
        return suppressor;
    }

    public void setSuppressor(String suppressor) {
        this.suppressor = suppressor;
    }


    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "AlertId", getAlertId());
        addRequiredAttributeAndCRC(writer, "AlertingResource", getAlertingResource());
        addRequiredAttributeAndCRC(writer, "AlertingResourceName", getAlertingResourceName());
        addRequiredAttributeAndCRC(writer, "IndicationTime", getIndicationTime());
        addRequiredAttributeAndCRC(writer, "Monitor", getMonitor());
        addRequiredAttributeAndCRC(writer, "MonitorRaw", getMonitorRaw());
        addRequiredAttributeAndCRC(writer, "NodeUUID", getNodeID());
        addRequiredAttributeAndCRC(writer, "PerceivedSeverity", perceivedSeverity);
        addRequiredAttributeAndCRC(writer, "Policy", policy);
        addRequiredAttributeAndCRC(writer, "PossibleEffect", possibleEffect);
        addRequiredAttributeAndCRC(writer, "CorrectiveActions", correctiveActions);
        addRequiredAttributeAndCRC(writer, "ProbableCause", probableCause);
        addRequiredAttributeAndCRC(writer, "ProbableCauseDescription", probableCauseDescription);
        addRequiredAttributeAndCRC(writer, "Subsystem", subsystem);
        addRequiredAttributeAndCRC(writer, "AdditionalInfo", additionalInfo);
        addRequiredAttributeAndCRC(writer, "Acknowledge", acknowledge);
        addRequiredAttributeAndCRC(writer, "Acknowledger", acknowledger);
        addRequiredAttributeAndCRC(writer, "Suppress", suppress);
        addRequiredAttributeAndCRC(writer, "Suppressor", suppressor);
    }

    /**
     *  Type of Health Monitor. See
     *  http://web.netapp.com/engineering/design-depot/appliance-mgmt/zephyr/zapidoc.cache/DOT/mainN/web/diagnosis/index.html#diagnosis-alert-info
     */
    public static enum MonitorType {
        NOT_MAPPED,
        NODE_CONNECT,
        SYSTEM_CONNECT,
        SYSTEM,
    }

}
