package com.onaro.sanscreen.acquisition.netappschema.netappdata;


import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEResourceType;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIETable;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEValue;
import com.onaro.sanscreen.acquisition.netappschema.annotations.OCIEZapiSource;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;
import java.util.EnumSet;

@OCIETable("user_quota")
@OCIEResourceType("user_quota")
public class UserQuota extends BaseCRCXMLNode<Volume> {

    public static final class UserQuotaKey extends KeyBase<UserQuotaKey.Field> {
        public enum Field {
            @OCIEZapiSource(
                    sourceZapi = "cluster-identity-get",
                    zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                    sourceZapi = "vserver-get-iter",
                    zapiResponsePath = "netapp.results.attributes-list.vserver-info.uuid"
            )
            vserver_uuid,

            @OCIEZapiSource(
                    sourceZapi = "volume-get-iter",
                    zapiResponsePath = "netapp.results.attributes-list.volume-attributes.volume-id-attributes.uuid"
            )
            volume_uuid,

            @OCIEZapiSource(
                    sourceZapi = "qtree-list-iter",
                    zapiResponsePath = "netapp.results.attributes-list.qtree-info.id"
            )
            qtree_id,

            @OCIEZapiSource(
                    sourceZapi = "quota-report-iter",
                    zapiResponsePath = "netapp.results.attributes-list.quota.quota-type"
            )
            quota_type,

            @OCIEZapiSource(
                    sourceZapi = "quota-report-iter",
                    zapiResponsePath = "netapp.results.attributes-list.quota.quota-target"
            )
            quota_target,

            @OCIEZapiSource(
                    sourceZapi = "quota-report-iter",
                    zapiResponsePath = "netapp.results.attributes-list.quota.quota-users.quota-user.quota-user-id",
                    notes = "Hash calculated from set of all quota user ids"
            )
            quota_user_hash
        }

        public UserQuotaKey(final String resourceType, final String clusterUUID, final String vServerUUID, final String volumeUUID,
                            final String qTreeID, final String quotaType, final String quotaTarget, final String userHash) {
            super(resourceType, new EnumMap<Field, String>(Field.class) {{
                put(Field.cluster_uuid, clusterUUID);
                put(Field.vserver_uuid, vServerUUID);
                put(Field.volume_uuid, volumeUUID);
                put(Field.qtree_id, qTreeID);
                put(Field.quota_type, quotaType);
                put(Field.quota_target, quotaTarget);
                put(Field.quota_user_hash, userHash);
            }}, EnumSet.of(Field.quota_target, Field.quota_user_hash));                      // These EnumSet<E> emptyAbleFields fields are allowed to be empty but not null
        }

        public String getOcieUUID() {
            final String volumeUUID = getID(Field.volume_uuid);
            final String qTreeID = getID(Field.qtree_id);
            final String quotaType = getID(Field.quota_type);
            final String quotaTarget = getID(Field.quota_target);
            final String targetSuffix = (quotaTarget.isEmpty() ? "" : quotaTarget);
            // Include extra dot when target not defined for backward compatibility
            final String userHash = getID(Field.quota_user_hash);
            return volumeUUID + '.' + qTreeID + '.' + quotaType + '.' + targetSuffix + '.' + userHash;
        }
    }

    final UserQuotaKey key;

    @OCIEZapiSource(
            sourceZapi = "quota-report-iter",
            zapiResponsePath = "netapp.results.attributes-list.quota.tree"
    )
    @OCIEValue(
            ocieColumn = "qtreeId"
    )
    private final QTree.QTreeKey qTreeKey;

    @OCIEZapiSource(
            sourceZapi = "quota-report-iter",
            zapiResponsePath = "netapp.results.attributes-list.quota.disk-limit"
    )
    @OCIEValue(
            ocieColumn = "diskLimit"
    )
    private Long diskLimit;

    @OCIEZapiSource(
            sourceZapi = "quota-report-iter",
            zapiResponsePath = "netapp.results.attributes-list.quota.disk-limit"
    )
    @OCIEValue(
            ocieColumn = "diskLimitUnlimited"
    )
    private Boolean diskLimitUnlimited;

    @OCIEZapiSource(
            sourceZapi = "quota-report-iter",
            zapiResponsePath = "netapp.results.attributes-list.quota.disk-used"
    )
    @OCIEValue(
            ocieColumn = "diskUsed"
    )
    private Long diskUsed;

    @OCIEZapiSource(
            sourceZapi = "quota-report-iter",
            zapiResponsePath = "netapp.results.attributes-list.quota.disk-used"
    )
    @OCIEValue(
            ocieColumn = "diskUsedUnlimited"
    )
    private Boolean diskUsedUnlimited;

    @OCIEZapiSource(
            sourceZapi = "quota-report-iter",
            zapiResponsePath = "netapp.results.attributes-list.quota.file-limit"
    )
    @OCIEValue(
            ocieColumn = "fileLimit"
    )
    private Long fileLimit;

    @OCIEZapiSource(
            sourceZapi = "quota-report-iter",
            zapiResponsePath = "netapp.results.attributes-list.quota.file-limit"
    )
    @OCIEValue(
            ocieColumn = "fileLimitUnlimited"
    )
    private Boolean fileLimitUnlimited;

    @OCIEZapiSource(
            sourceZapi = "quota-report-iter",
            zapiResponsePath = "netapp.results.attributes-list.quota.file-used"
    )
    @OCIEValue(
            ocieColumn = "fileUsed"
    )
    private Long fileUsed;

    @OCIEZapiSource(
            sourceZapi = "quota-report-iter",
            zapiResponsePath = "netapp.results.attributes-list.quota.file-used"
    )
    @OCIEValue(
            ocieColumn = "fileUsedUnlimited"
    )
    private Boolean fileUsedUnlimited;

    @OCIEZapiSource(
            sourceZapi = "quota-report-iter",
            zapiResponsePath = "netapp.results.attributes-list.quota.soft-disk-limit"
    )
    @OCIEValue(
            ocieColumn = "softDiskLimit"
    )
    private Long softDiskLimit;

    @OCIEZapiSource(
            sourceZapi = "quota-report-iter",
            zapiResponsePath = "netapp.results.attributes-list.quota.soft-disk-limit"
    )
    @OCIEValue(
            ocieColumn = "softDiskLimitUnlimited"
    )
    private Boolean softDiskLimitUnlimited;

    @OCIEZapiSource(
            sourceZapi = "quota-report-iter",
            zapiResponsePath = "netapp.results.attributes-list.quota.soft-file-limit"
    )
    @OCIEValue(
            ocieColumn = "softFileLimit"
    )
    private Long softFileLimit;

    @OCIEZapiSource(
            sourceZapi = "quota-report-iter",
            zapiResponsePath = "netapp.results.attributes-list.quota.soft-file-limit"
    )
    @OCIEValue(
            ocieColumn = "softFileLimitUnlimited"
    )
    private Boolean softFileLimitUnlimited;

    @OCIEZapiSource(
            sourceZapi = "quota-report-iter",
            zapiResponsePath = "netapp.results.attributes-list.quota.threshold"
    )
    @OCIEValue(
            ocieColumn = "threshold"
    )
    private Long threshold;

    @OCIEZapiSource(
            sourceZapi = "quota-report-iter",
            zapiResponsePath = "netapp.results.attributes-list.quota.threshold"
    )
    @OCIEValue(
            ocieColumn = "thresholdUnlimited"
    )
    private Boolean thresholdUnlimited;

    @OCIEZapiSource(
            sourceZapi = "quota-report-iter",
            zapiResponsePath = "netapp.results.attributes-list.quota.quota-users.quota-user.quota-user-name",
            notes = "CSV list of all quota user id names"
    )
    @OCIEValue(
            ocieColumn = "users",
            notes = "CSV list of all quota user id names"
    )
    private String users;

    private final QuotaUsers quotaUsers;

    UserQuota(final Volume parent, final QTree.QTreeKey qTreeKey, final TreeQuota.QuotaType quotaType, final String quotaTarget,
              final String userHash) {
        super(parent, "UserQuota");
        this.quotaUsers = new QuotaUsers(this);
        this.qTreeKey = qTreeKey;
        final VServer vServer = parent.getParent();
        final VServer.VServerKey vServerKey = vServer.getKey();
        final Volume.VolumeKey volumeKey = parent.getKey();
        final String clusterUUID = volumeKey.getID(Volume.VolumeKey.Field.cluster_uuid);
        final String vServerUUID = vServerKey.getID(VServer.VServerKey.Field.uuid);
        final String volumeUUID = volumeKey.getVolumeInstanceUUID();
        final String qTreeID = qTreeKey.getID(QTree.QTreeKey.Field.qtree_id);
        this.key = new UserQuotaKey("user_quota", clusterUUID, vServerUUID, volumeUUID, qTreeID, String.valueOf(quotaType), quotaTarget, userHash);
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public UserQuotaKey getKey() {
        return key;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getQtreeID() {
        return key.getID(UserQuotaKey.Field.qtree_id);
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getQuotaType() {
        return key.getID(UserQuotaKey.Field.quota_type);
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getQuotaTarget() {
        return key.getID(UserQuotaKey.Field.quota_target);
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getDiskLimit() {
        return diskLimit;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean getDiskLimitUnlimited() {
        return diskLimitUnlimited;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getDiskUsed() {
        return diskUsed;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean getDiskUsedUnlimited() {
        return diskUsedUnlimited;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getFileLimit() {
        return fileLimit;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean getFileLimitUnlimited() {
        return fileLimitUnlimited;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getFileUsed() {
        return fileUsed;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean getFileUsedUnlimited() {
        return fileUsedUnlimited;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getSoftDiskLimit() {
        return softDiskLimit;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean getSoftDiskLimitUnlimited() {
        return softDiskLimitUnlimited;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getSoftFileLimit() {
        return softFileLimit;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean getSoftFileLimitUnlimited() {
        return softFileLimitUnlimited;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Long getThreshold() {
        return threshold;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public Boolean getThresholdUnlimited() {
        return thresholdUnlimited;
    }

    public void setDiskLimit(final Long diskLimit) {
        this.diskLimit = diskLimit;
    }

    public void setDiskLimitUnlimited(final Boolean diskLimitUnlimited) {
        this.diskLimitUnlimited = diskLimitUnlimited;
    }

    public void setDiskUsed(final Long diskUsed) {
        this.diskUsed = diskUsed;
    }

    public void setDiskUsedUnlimited(final Boolean diskUsedUnlimited) {
        this.diskUsedUnlimited = diskUsedUnlimited;
    }

    public void setFileLimit(final Long fileLimit) {
        this.fileLimit = fileLimit;
    }

    public void setFileLimitUnlimited(final Boolean fileLimitUnlimited) {
        this.fileLimitUnlimited = fileLimitUnlimited;
    }

    public void setFileUsed(final Long fileUsed) {
        this.fileUsed = fileUsed;
    }

    public void setFileUsedUnlimited(final Boolean fileUsedUnlimited) {
        this.fileUsedUnlimited = fileUsedUnlimited;
    }

    public void setSoftDiskLimit(final Long softDiskLimit) {
        this.softDiskLimit = softDiskLimit;
    }

    public void setSoftDiskLimitUnlimited(final Boolean softDiskLimitUnlimited) {
        this.softDiskLimitUnlimited = softDiskLimitUnlimited;
    }

    public void setSoftFileLimit(final Long softFileLimit) {
        this.softFileLimit = softFileLimit;
    }

    public void setSoftFileLimitUnlimited(final Boolean softFileLimitUnlimited) {
        this.softFileLimitUnlimited = softFileLimitUnlimited;
    }

    public void setThreshold(final Long threshold) {
        this.threshold = threshold;
    }

    public void setThresholdUnlimited(final Boolean thresholdUnlimited) {
        this.thresholdUnlimited = thresholdUnlimited;
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public boolean isDefaultQuota() {
        return "*".equals(getQuotaTarget());
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getUsers() {
        return users;
    }

    public void setUsers(final String users) {
        this.users = users;
    }

    public QuotaUsers getQuotaUsers() {
        return quotaUsers;
    }

    public QuotaUser createQuotaUser(final String quotaUserID) {
        return new QuotaUser(this, quotaUserID);
    }

    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        addRequiredAttributeAndCRC(writer, "ResourceKey", key);
        addRequiredAttributeAndCRC(writer, "UUID", key.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "QtreeUUID", qTreeKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "QuotaTarget", getQuotaTarget());
        addRequiredAttributeAndCRC(writer, "QuotaType", getQuotaType());
        addRequiredAttributeAndCRC(writer, "DiskLimit", diskLimit);
        addRequiredAttributeAndCRC(writer, "DiskLimitUnlimited", diskLimitUnlimited);
        addRequiredAttributeAndCRC(writer, "DiskUsed", diskUsed);
        addRequiredAttributeAndCRC(writer, "DiskUsedUnlimited", diskUsedUnlimited);
        addRequiredAttributeAndCRC(writer, "FileLimit", fileLimit);
        addRequiredAttributeAndCRC(writer, "FileLimitUnlimited", fileLimitUnlimited);
        addRequiredAttributeAndCRC(writer, "FileUsed", fileUsed);
        addRequiredAttributeAndCRC(writer, "FileUsedUnlimited", fileUsedUnlimited);
        addRequiredAttributeAndCRC(writer, "SoftDiskLimit", softDiskLimit);
        addRequiredAttributeAndCRC(writer, "SoftDiskLimitUnlimited", softDiskLimitUnlimited);
        addRequiredAttributeAndCRC(writer, "SoftFileLimit", softFileLimit);
        addRequiredAttributeAndCRC(writer, "SoftFileLimitUnlimited", softFileLimitUnlimited);
        addRequiredAttributeAndCRC(writer, "Threshold", threshold);
        addRequiredAttributeAndCRC(writer, "ThresholdUnlimited", thresholdUnlimited);
        addRequiredAttributeAndCRC(writer, "Users", users);
    }

    @Override
    protected boolean hasChildren() {
        return !quotaUsers.isEmpty();
    }

    @Override
    protected void marshallChildrenToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer, final int tabs) throws IOException, SchemaException {
        super.marshallChildrenToFileAndCRC(marshallingCache, writer, tabs);
        addAllChildrenUnderNodeAndCRC(marshallingCache, writer, "QuotaUsers", quotaUsers.values(), tabs);
    }
}
