package com.onaro.sanscreen.acquisition.netappschema.netappdata;

import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileCRCWriter;
import com.onaro.sanscreen.acquisition.netappschema.annotations.*;
import com.onaro.sanscreen.acquisition.schema.BaseCRCXMLNode;
import com.onaro.sanscreen.acquisition.schema.KeyBase;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.util.EnumMap;

@OCIETable("vserver_to_aggregate")
@OCIEResourceType("vserver_to_aggregate")
public class VServerAggregate extends BaseCRCXMLNode<Cluster> {

    public final static class VServerAggrKey extends KeyBase<VServerAggrKey.Field> {
        public static enum Field {
            @OCIEZapiSource(
                sourceZapi = "cluster-identity-get",
                zapiResponsePath = "netapp.results.attributes.cluster-identity-info.cluster-uuid"
            )
            cluster_uuid,

            @OCIEZapiSource(
                sourceZapi = "vserver-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.vserver-info.uuid"
            )
            vserver_uuid,

            @OCIEZapiSource(
                sourceZapi = "aggr-get-iter",
                zapiResponsePath = "netapp.results.attributes-list.aggr-attributes.aggregate-uuid"
            )
            aggregate_uuid
        }

        public VServerAggrKey(final String clusterUUID, final String vServerUUID, final String aggrUUID) {
            super("vserver_aggregate",
                new EnumMap<Field, String>(Field.class) {{
                    put(Field.cluster_uuid, clusterUUID);
                    put(Field.vserver_uuid, vServerUUID);
                    put(Field.aggregate_uuid, aggrUUID);
                }});
        }

        @Override
        @OCIEUUID(uuidDescr = "cluster_uuid.vserver_uuid.aggregate_uuid")
        public String getOcieUUID() {
            return getFQID();
        }
    }

    private final VServerAggrKey vServerAggrKey;


    @OCIEZapiSource(
        sourceZapi = "vserver-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.vserver-info.uuid"
    )
    @OCIEValue(
        ocieColumn = "vserverId"
    )
    private final VServer.VServerKey vServerKey;

    @OCIEZapiSource(
        sourceZapi = "vserver-get-iter",
        zapiResponsePath = "netapp.results.attributes-list.vserver-info.aggr-list.aggr-name"
    )
    @OCIEValue(
        ocieColumn = "aggregateId"
    )
    private final Aggregate.AggregateKey aggregateKey;


    VServerAggregate(final Cluster parent, final VServer.VServerKey vServerKey, final Aggregate.AggregateKey aggregateKey) {
        super(parent, "VserverAggregateRelationship");
        final String clusterUUID = parent.getClusterUUID();
        final String vServerUUID = vServerKey.getZapiUUID();
        final String aggregateUUID = aggregateKey.getID(Aggregate.AggregateKey.Field.uuid);
        this.vServerAggrKey = new VServerAggrKey(clusterUUID, vServerUUID, aggregateUUID);
        this.vServerKey = vServerKey;
        this.aggregateKey = aggregateKey;
    }

    @Override
    @SuppressWarnings({"unchecked"})
    public VServerAggrKey getKey() {
        return vServerAggrKey;
    }



    @Override
    protected void marshallAttributesToFileAndCRC(final MarshallingCache marshallingCache, final DSFileCRCWriter writer) throws IOException, SchemaException {
        super.marshallAttributesToFileAndCRC(marshallingCache, writer);
        //addRequiredAttributeAndCRC(writer, "ResourceKey", vServerAggrKey);
        addRequiredAttributeAndCRC(writer, "VserverUUID", vServerKey.getOcieUUID());
        addRequiredAttributeAndCRC(writer, "AggregateUUID", aggregateKey.getOcieUUID());
    }
}
