package com.onaro.sanscreen.acquisition.netappstorageperformance.datasource;

import com.onaro.sanscreen.acquisition.netappschema.netappdata.*;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.NetworkPort.Speed;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.StorageAdapter.AdapterKey.Field;
import com.onaro.util.VersionComparator;
import org.apache.log4j.Logger;

import java.util.*;

/**
 * Contains disk, volume and internalVolume information for given storage
 */
public class NetAppDefaultStorageContext implements NetAppStorageContext {

    private static final Logger logger = Logger.getLogger(NetAppDefaultStorageContext.class);

    private final Cluster.ClusterKey key;
    private final List<String> vServerUUIDs;
    private final List<String> volumeInstanceUUIDs;
    private final Map<String, String> volumeVServerUUIDMap;
    private final Map<String, String> fcpLifNameToVServerUUIDMap;
    private final Map<String, String> fcpLifNameToFoundationUUIDMap;
    private final Map<String, Integer> fcpPortToLinkSpeedMap;
    private final Map<String, List<String>> fcpPortWWNToLifUUIDMap;
    private final Map<String, String> networkLifNameToVServerUUIDMap;
    private final Map<String, String> networkLifNameToFoundationUUIDMap;
    private final Map<String, String> cifsVserverIDToUUIDMap;
    private final Map<String, String> nfsv3VserverIDToUUIDMap;
    private final Map<String, String> nfsv4VserverIDToUUIDMap;
    private final Map<String, String> nfsv41VserverIDToUUIDMap;
    private final Map<String, String> lifVserverIDToUUIDMap;
    
    private final List<String> systemPerfUUIDs;
    private final List<String> hybridAggrUUIDs;
    private final List<String> extCachePerfObjUUIDs;
    private final List<String> waflPerfUUIDs;
    private final List<String> volumeNodePerfUUIDs;
    private final List<String> diskUUIDs;
    //key is aggregateUUID, value is List of data Disk UUID
    private final Map<String, List<String>> aggregateUUIDToDiskUUIDList;
    private final Map<String, String> aggregateUUIDToNameMap;
    private final Map<String, String> diskNameToUUIDMap;
    private final List<String> aggregatePerfUUIDs;
    private final Map<String, String> aggregatePerfUUIDToUUID;
    private final Map<String, String> systemPerfUUIDtoNodeUUID;
    private final Map<String, String> extCachePerfUUIDtoNodeUUID;
    private final Map<String, String> waflPerfUUIDtoNodeUUID;
    private final List<String> lunUUIDs;
    private final Map<String, String> lunUUIDtoVolumeUUIDMap;
    private final Map<String, String> fcpPortPerfUUIDToUUIDMap;
    private final List<String> ipPortPerfUUIDs;
    private final Map<String, String> ipPortPerfUUIDToUUIDMap;
    private final Map<String, Speed> ipPortToLinkSpeedMap;
    private final List<String> processorPerfUUIDs;
    private final Map<String, String> nodeNameToUUIDMap;
    private final Map<String, Set<String>> qosServiceCenterUUIDtoWorkloadDetailUUIDs;
    private final Map<String, List<String>> nodeUUIDToDiskUUIDs;
    private final List<String> nonDelayCenterResourceUUIDs;
    private final List<String> qosVolumeWorkloadUUIDs;
    private final Map<String,String> qosVolumeWorkloadUUIDtoNameMap;
    private final Map<String, Map<String, String>> qosVolumeWorkloadUUIDToAggregateMap;
    private final Map<String, Map<String, String>> qosWorkloadConstituentMap;
    private final List<String> qosWorkloadConstituentUUIDs;
    private final List<String> qosWorkloadDetailUUIDs;
    private final Map<String, Map<String, String>> qosWorkloadDetailMap;
    private String clusterVersion;
    private String systemObjectName;
    private boolean isLessThanSierraNevada;
    private boolean isLessThanFullSteam;
    private boolean isGreaterThanFullSteam0;
    private List<String> cpuColumnNames;
    private List<String> waflCpTimesColumnNames;
    private List<String> volumeWorkloadReadIOColumns;
    private final Map<String, String> vServerUUIDVolumeNameToVolumeUUIDMap;


    private final Map<String, String> cifsPerfUUIDtoNodeUUID;
    private final Map<String, String> smb1PerfUUIDtoNodeUUID;
    private final Map<String, String> smb2PerfUUIDtoNodeUUID;
    private final Map<String, String> nfsv3PerfUUIDtoNodeUUID;
    private final Map<String, String> nfsv4PerfUUIDtoNodeUUID;
    private final Map<String, String> nfsv4_1PerfUUIDtoNodeUUID;
    private List<String> storageAdapterPerfUUIDs;

    private final Map<String, Integer> nodeNameToCoreCount;
    
    /**
     * Flags to turn on/off cooked/raw data persistence. 
     * cookedPersistenceEnabled default is true and rawPersistenceEnabled default is false.
     */
    boolean cookedPersistenceEnabled = true;
    boolean rawPersistenceEnabled = true;

    public NetAppDefaultStorageContext(final Cluster.ClusterKey key) {
        this.key = key;
        systemObjectName = "";
        vServerUUIDs = new ArrayList<String>();
        volumeInstanceUUIDs = new ArrayList<String>();
        volumeVServerUUIDMap = new TreeMap<String, String>();
        fcpLifNameToVServerUUIDMap = new TreeMap<String, String>();
        fcpLifNameToFoundationUUIDMap = new TreeMap<String, String>();
        networkLifNameToVServerUUIDMap = new TreeMap<String, String>();
        networkLifNameToFoundationUUIDMap = new TreeMap<String, String>();
        cifsVserverIDToUUIDMap = new TreeMap<>();
        nfsv3VserverIDToUUIDMap = new TreeMap<>();
        nfsv4VserverIDToUUIDMap = new TreeMap<>();
        nfsv41VserverIDToUUIDMap = new TreeMap<>();
        lifVserverIDToUUIDMap = new TreeMap<>();
        systemPerfUUIDs = new ArrayList<String>();
        hybridAggrUUIDs = new ArrayList<String>();
        extCachePerfObjUUIDs = new ArrayList<String>();
        waflPerfUUIDs = new ArrayList<String>();
        volumeNodePerfUUIDs = new ArrayList<String>();
        diskUUIDs = new ArrayList<String>();
        diskNameToUUIDMap = new TreeMap<String, String>();
        aggregateUUIDToDiskUUIDList = new HashMap<String, List<String>>();
        aggregateUUIDToNameMap = new HashMap<>();
        aggregatePerfUUIDs = new ArrayList<String>();
        aggregatePerfUUIDToUUID = new TreeMap<String, String>();
        systemPerfUUIDtoNodeUUID = new TreeMap<String, String>();
        extCachePerfUUIDtoNodeUUID = new TreeMap<String, String>();
        waflPerfUUIDtoNodeUUID = new TreeMap<String, String>();
        lunUUIDs = new ArrayList<String>();
        lunUUIDtoVolumeUUIDMap = new TreeMap<String, String>();
        fcpPortPerfUUIDToUUIDMap = new TreeMap<String, String>();
        ipPortPerfUUIDs = new ArrayList<String>();
        ipPortPerfUUIDToUUIDMap = new TreeMap<String, String>();
        ipPortToLinkSpeedMap = new TreeMap<String,Speed>();
        processorPerfUUIDs = new ArrayList<String>();
        nodeNameToUUIDMap = new TreeMap<String, String>();
        qosServiceCenterUUIDtoWorkloadDetailUUIDs = new TreeMap<String, Set<String>>();
        nodeUUIDToDiskUUIDs = new TreeMap<String, List<String>>();
        nonDelayCenterResourceUUIDs = new ArrayList<String>();
        qosVolumeWorkloadUUIDs = new ArrayList<String>();
        qosVolumeWorkloadUUIDtoNameMap = new HashMap<>();
        qosVolumeWorkloadUUIDToAggregateMap = new HashMap<String, Map<String, String>>();
        qosWorkloadConstituentMap = new HashMap<String, Map<String, String>>();
        qosWorkloadConstituentUUIDs = new ArrayList<String>();
        qosWorkloadDetailUUIDs = new ArrayList<String>();
        qosWorkloadDetailMap = new HashMap<String, Map<String, String>>();
        nodeNameToCoreCount = new TreeMap<String, Integer>();

        cifsPerfUUIDtoNodeUUID = new TreeMap<String, String>();
        smb1PerfUUIDtoNodeUUID = new TreeMap<String, String>();
        smb2PerfUUIDtoNodeUUID = new TreeMap<String, String>();
        nfsv3PerfUUIDtoNodeUUID = new TreeMap<String, String>();
        nfsv4PerfUUIDtoNodeUUID = new TreeMap<String, String>();
        nfsv4_1PerfUUIDtoNodeUUID = new TreeMap<String, String>();
        vServerUUIDVolumeNameToVolumeUUIDMap = new HashMap<String, String>();
        fcpPortWWNToLifUUIDMap = new HashMap<String, List<String>>();
        fcpPortToLinkSpeedMap = new HashMap<String, Integer>();
        storageAdapterPerfUUIDs  = new ArrayList<String>();
    }

    public final Cluster.ClusterKey getKey() {
        return key;
    }
    
    public String getSystemObjectName() {
        return systemObjectName;
    }

    public List<String> getSystemPerfUUIDs() {
        return systemPerfUUIDs;
    }

    public List<String> getHybridAggrUUIDs() {
        return hybridAggrUUIDs;
    }

    public List<String> getExtCachePerfObjUUIDs() {
        return extCachePerfObjUUIDs;
    }

    public List<String> getWaflPerfUUIDs() {
        return waflPerfUUIDs;
    }

    public List<String> getVolumeNodePerfUUIDs() {
        return volumeNodePerfUUIDs;
    }

    public List<String> getDiskUUIDs() {
        return diskUUIDs;
    }

    public Map<String, List<String>> getAggregateUUIDToDiskUUIDList() {
        return aggregateUUIDToDiskUUIDList;
    }

    public Map<String, String> getAggregateUUIDToNameMap() {
        return aggregateUUIDToNameMap;
    }

    public List<String> getAggregatePerfUUIDs() {
        return aggregatePerfUUIDs;
    }

    public List<String> getVServerUUIDs() {
        return vServerUUIDs;
    }

    public List<String> getVolumeInstanceUUIDs() {
        return volumeInstanceUUIDs;
    }

    public Map<String, String> getVolumeVServerUUIDMap() {
        return volumeVServerUUIDMap;
    }

    public Map<String, String> getAggregatePerfUUIDToUUID() {
        return aggregatePerfUUIDToUUID;
    }

    public Map<String, String> getSystemPerfUUIDtoNodeUUID() {
        return systemPerfUUIDtoNodeUUID;
    }

    public Map<String, String> getExtCachePerfUUIDtoNodeUUID() {
        return extCachePerfUUIDtoNodeUUID;
    }

    public Map<String, String> getWaflPerfUUIDtoNodeUUID() {
        return waflPerfUUIDtoNodeUUID;
    }

    public List<String> getLunUUIDs() {
        return lunUUIDs;
    }

    public Map<String, String> getLunUUIDtoVolumeUUIDMap() {
        return lunUUIDtoVolumeUUIDMap;
    }

    public Map<String, String> getFcpLifNameToVServerUUIDMap() {
        return fcpLifNameToVServerUUIDMap;
    }

    public Map<String, String> getFcpLifNameToFoundationUUIDMap() {
        return fcpLifNameToFoundationUUIDMap;
    }

    public Map<String, String> getNetworkLifNameToVServerUUIDMap() {
        return networkLifNameToVServerUUIDMap;
    }

    public Map<String, String> getNetworkLifNameToFoundationUUIDMap() {
        return networkLifNameToFoundationUUIDMap;
    }

    public Map<String, String> getCifsVserverIDToUUIDMap() {
        return cifsVserverIDToUUIDMap;
    }

    public Map<String, String> getNfsv3VserverIDToUUIDMap() {
        return nfsv3VserverIDToUUIDMap;
    }

    public Map<String, String> getNfsv4VserverIDToUUIDMap() {
        return nfsv4VserverIDToUUIDMap;
    }

    public Map<String, String> getNfsv41VserverIDToUUIDMap() {
        return nfsv41VserverIDToUUIDMap;
    }

    public Map<String, String> getLifVserverIDToUUIDMap() {
        return lifVserverIDToUUIDMap;
    }
    
    public List<String> getIpPortPerfUUIDs() {
        return ipPortPerfUUIDs;
    }

    public Map<String, String> getIpPortPerfUUIDToUUIDMap() {
        return ipPortPerfUUIDToUUIDMap;
    }
    
    public Map<String, Speed> getIpPortToLinkSpeedMap(){
    	return ipPortToLinkSpeedMap;
    }

    public Map<String, String> getFcpPortPerfUUIDToUUIDMap() {
        return fcpPortPerfUUIDToUUIDMap;
    }

    public List<String> getProcessorPerfUUIDs() {
        return processorPerfUUIDs;
    }

    public Map<String, String> getNodeNameToUUIDMap() {
        return nodeNameToUUIDMap;
    }

    public List<String> getQosVolumeWorkloadInstanceUUIDs() {
        return qosVolumeWorkloadUUIDs;
    }

    public Map<String, String> getQosVolumeWorkloadUUIDtoNameMap() {
        return qosVolumeWorkloadUUIDtoNameMap;
    }

    public Map<String, Map<String, String>> getQosVolumeWorkloadUUIDToAggregateMap() {
        return qosVolumeWorkloadUUIDToAggregateMap;
    }

    public Map<String, Map<String, String>> getQosWorkloadConstituentMap() {
        return qosWorkloadConstituentMap;
    }

    public List<String> getQosWorkloadConstituentUUIDs() {
        return qosWorkloadConstituentUUIDs;
    }

    public List<String> getQosWorkloadDetailInstanceUUIDs() {
        return qosWorkloadDetailUUIDs;
    }

    public Map<String, Map<String, String>> getQosWorkloadDetailMap() {
        return qosWorkloadDetailMap;
    }

    public Map<String, Set<String>> getQosServiceCenterUUIDtoWorkloadDetailUUIDs() {
        return qosServiceCenterUUIDtoWorkloadDetailUUIDs;
    }

    public Map<String, List<String>> getNodeUUIDToDiskUUIDs() {
        return nodeUUIDToDiskUUIDs;
    }
    
    public List<String> getNonDelayCenterResourceUUIDs() {
        return nonDelayCenterResourceUUIDs;
    }

    public Map<String, Integer> getNodeNameToCoreCount() {
        return nodeNameToCoreCount;
    }
    
    public Map<String, String> getvServerUUIDVolumeNameToVolumeUUIDMap(){
    	return vServerUUIDVolumeNameToVolumeUUIDMap;
    }

    public Map<String, String> getDiskNameToUUIDMap(){
    	return diskNameToUUIDMap;
    }

    public Map<String, List<String>> getFcpPortWWNToLifUUIDMap() {
        return fcpPortWWNToLifUUIDMap;
    }

    public Map<String, Integer> getFcpPortToLinkSpeedMap() {
        return fcpPortToLinkSpeedMap;
    }


 	public List<String> getStorageAdapterPerfUUIDs() {
		return storageAdapterPerfUUIDs;
	}

	public void processFoundationData(final Cluster cluster) {
        volumeVServerUUIDMap.clear();
        vServerUUIDs.clear();
        volumeInstanceUUIDs.clear();
        systemPerfUUIDs.clear();
        hybridAggrUUIDs.clear();
        extCachePerfObjUUIDs.clear();
        waflPerfUUIDs.clear();
        volumeNodePerfUUIDs.clear();
        aggregatePerfUUIDToUUID.clear();
        aggregatePerfUUIDs.clear();
        diskUUIDs.clear();
        aggregateUUIDToDiskUUIDList.clear();
        aggregateUUIDToNameMap.clear();
        diskNameToUUIDMap.clear();
        systemPerfUUIDtoNodeUUID.clear();
        extCachePerfUUIDtoNodeUUID.clear();
        waflPerfUUIDtoNodeUUID.clear();
        lunUUIDs.clear();
        lunUUIDtoVolumeUUIDMap.clear();
        fcpLifNameToVServerUUIDMap.clear();
        fcpLifNameToFoundationUUIDMap.clear();
        fcpPortWWNToLifUUIDMap.clear();
        fcpPortPerfUUIDToUUIDMap.clear();
        fcpPortToLinkSpeedMap.clear();
        ipPortPerfUUIDs.clear();
        ipPortPerfUUIDToUUIDMap.clear();
        ipPortToLinkSpeedMap.clear();
        processorPerfUUIDs.clear();
        nodeNameToUUIDMap.clear();
        qosServiceCenterUUIDtoWorkloadDetailUUIDs.clear();
        nodeUUIDToDiskUUIDs.clear();
        nonDelayCenterResourceUUIDs.clear();
        qosVolumeWorkloadUUIDs.clear();
        qosVolumeWorkloadUUIDtoNameMap.clear();
        qosVolumeWorkloadUUIDToAggregateMap.clear();
        qosWorkloadConstituentMap.clear();
        qosWorkloadConstituentUUIDs.clear();
        qosWorkloadDetailUUIDs.clear();
        qosWorkloadDetailMap.clear();
        networkLifNameToFoundationUUIDMap.clear();
        cifsVserverIDToUUIDMap.clear();
        nfsv3VserverIDToUUIDMap.clear();
        nfsv4VserverIDToUUIDMap.clear();
        nfsv41VserverIDToUUIDMap.clear();
        lifVserverIDToUUIDMap.clear();
        networkLifNameToVServerUUIDMap.clear();
        cifsPerfUUIDtoNodeUUID.clear();
        smb1PerfUUIDtoNodeUUID.clear();
        smb2PerfUUIDtoNodeUUID.clear();
        nfsv3PerfUUIDtoNodeUUID.clear();
        nfsv4PerfUUIDtoNodeUUID.clear();
        nfsv4_1PerfUUIDtoNodeUUID.clear();
        clusterVersion = cluster.getVersionString();         // used this for version specific performance chunk size and column sets
        cpuColumnNames = cluster.getCpuColumns();
        waflCpTimesColumnNames = cluster.getWaflPhaseTimesColumns();
        volumeWorkloadReadIOColumns = cluster.getVolumeWorkloadReadIOColumns();
        final VersionComparator versionComparator = new VersionComparator();
        isLessThanSierraNevada = (versionComparator.compare(clusterVersion, Cluster.SIERRA_NEVADA) < 0);     // Rolling Rock  is less than Sierra Nevada
        isLessThanFullSteam = (versionComparator.compare(clusterVersion, Cluster.FULL_STEAM) < 0);     // Sierra Nevada  is less than Full Steam
        isGreaterThanFullSteam0 = (versionComparator.compare(clusterVersion, Cluster.FULL_STEAM) > 0); 
        storageAdapterPerfUUIDs.clear();
        
        if(isLessThanFullSteam){
            systemObjectName = "system";
        }
        else{
            systemObjectName = "system:constituent";
        }

        assert (cluster.getKey().compareTo(key) == 0);

        Map<String, String> vserverNameToVserverUUIDMap = new HashMap<>(); // used to construct vserverID->vServerUUID map
        for (final VServer vServer : cluster.getVServers().values()) {
            final String vServerUUID = vServer.getKey().getOcieUUID();
            final String vServerName = vServer.getName();
            vserverNameToVserverUUIDMap.put(vServerName, vServerUUID);
            
            vServerUUIDs.add(vServerUUID);
            for (final Volume volume : vServer.getVolumes().values()) {
                // Performance metrics require the volume "global" UUID argument, NOT the volume instance UUID.
                // When reporting volume metrics, provide the instance UUID
                final String volumeInstanceUUID = volume.getInstanceUUID();
                volumeInstanceUUIDs.add(volumeInstanceUUID);
                volumeVServerUUIDMap.put(volumeInstanceUUID, vServerUUID);
                vServerUUIDVolumeNameToVolumeUUIDMap.put(vServerUUIDVolNameComboKey(vServerUUID, volume.getVolumeName()),volumeInstanceUUID);
                for (final LUN lun : volume.getLUNs().values()) {
                    final String lunUUID = lun.getKey().getOcieUUID();
                    lunUUIDs.add(lunUUID);
                    lunUUIDtoVolumeUUIDMap.put(lunUUID, volumeInstanceUUID);
                }
            }
            for (final NetworkLif lif : vServer.getNetworkLifs().values()) {
                final String lifName = lif.getInterfaceName();

                final StringBuilder fqLifName = new StringBuilder();
                // to overcome duplicate lif names across vservers, tag the parent vserver name to lif name in the format vserverName:lifName
                fqLifName.append(vServerName);
                fqLifName.append(":");
                fqLifName.append(lifName);

                final String ocieUUID = lif.getKey().getOcieUUID();
                networkLifNameToVServerUUIDMap.put(fqLifName.toString(), vServerUUID);
                networkLifNameToFoundationUUIDMap.put(fqLifName.toString(), ocieUUID);
            }
            for (final FcpLif lif : vServer.getFcpLifs().values()) {
                final String fcpLifName = lif.getInterfaceName();

                final StringBuilder fqFcpLifName = new StringBuilder();
                fqFcpLifName.append(vServerName);
                // The instance UUIDs returned from perf.fcp_lif use a "." as a separator
                fqFcpLifName.append(".");
                fqFcpLifName.append(fcpLifName);

                fcpLifNameToFoundationUUIDMap.put(fqFcpLifName.toString(), lif.getKey().getOcieUUID());
                fcpLifNameToVServerUUIDMap.put(fqFcpLifName.toString(), vServerUUID);

                // Add to the map of port WWN to list of lif uuids
                final String physicalPort = lif.getHomePortKey().getID(FcpPort.FcpPortKey.Field.port_name);
                if(physicalPort != null) {
                    List<String> lifUUIDlist = fcpPortWWNToLifUUIDMap.get(physicalPort);
                    if (lifUUIDlist == null) {
                        fcpPortWWNToLifUUIDMap.put(physicalPort, lifUUIDlist = new ArrayList<String>());
                    }
                    lifUUIDlist.add(lif.getKey().getOcieUUID());
                }
            }
        }

        // construct vserserID to VserverUUID maps for CIFS/NFS
        for (final String vserverID : cluster.getCifsVserverIDNameMap().keySet()) {
            String vserverUUID = vserverNameToVserverUUIDMap.get(cluster.getCifsVserverIDNameMap().get(vserverID));
            if (vserverUUID == null) {
                logger.error("Unable to map Vserver ID " + vserverID + " to any inventory vserver for CIFS.");
                continue;
            }
            
            cifsVserverIDToUUIDMap.put(vserverID, vserverUUID);
        }
        for (final String vserverID : cluster.getNfsv3VserverIDNameMap().keySet()) {
            String vserverUUID = vserverNameToVserverUUIDMap.get(cluster.getNfsv3VserverIDNameMap().get(vserverID));
            if (vserverUUID == null) {
                logger.error("Unable to map Vserver ID " + vserverID + " to any inventory vserver for NFSV3.");
                continue;
            }
            
            nfsv3VserverIDToUUIDMap.put(vserverID, vserverUUID);
        }
        for (final String vserverID : cluster.getNfsv4VserverIDNameMap().keySet()) {
            String vserverUUID = vserverNameToVserverUUIDMap.get(cluster.getNfsv4VserverIDNameMap().get(vserverID));
            if (vserverUUID == null) {
                logger.error("Unable to map Vserver ID " + vserverID + " to any inventory vserver for NFSV4.");
                continue;
            }
            
            nfsv4VserverIDToUUIDMap.put(vserverID, vserverUUID);
        }
        for (final String vserverID : cluster.getNfsv41VserverIDNameMap().keySet()) {
            String vserverUUID = vserverNameToVserverUUIDMap.get(cluster.getNfsv41VserverIDNameMap().get(vserverID));
            if (vserverUUID == null) {
                logger.error("Unable to map Vserver ID " + vserverID + " to any inventory vserver for NFSV4_1.");
                continue;
            }
            
            nfsv41VserverIDToUUIDMap.put(vserverID, vserverUUID);
        }
        for (final String vserverID : cluster.getLifVserverIDNameMap().keySet()) {
            String vserverUUID = vserverNameToVserverUUIDMap.get(cluster.getLifVserverIDNameMap().get(vserverID));
            if (vserverUUID == null) {
                logger.error("Unable to map Vserver ID " + vserverID + " to any inventory vserver for LIF.");
                continue;
            }
            
            lifVserverIDToUUIDMap.put(vserverID, vserverUUID);
        }
        
        for (final QosVolumeWorkload qosVolumeWorkload : cluster.getQosVolumeWorkloads().values()) {
            final String qosVolumeWorkloadUUID = qosVolumeWorkload.getKey().getID(QosVolumeWorkload.QosVolumeWorkloadKey.Field.uuid);
            qosVolumeWorkloadUUIDtoNameMap.put(qosVolumeWorkloadUUID, qosVolumeWorkload.getName());

            final Volume.VolumeKey volumeKey = qosVolumeWorkload.getVolumeKey();
            final VServer.VServerKey vServerKey = qosVolumeWorkload.getVServerKey();
            if (volumeKey != null && vServerKey != null) {
                final VServer vServer = cluster.getVServers().get(vServerKey);
                if (vServer != null) {
                    final Volume volume = vServer.getVolumes().get(volumeKey);
                    if (volume != null) {
                        final Aggregate.AggregateKey aggregateKey = volume.getAggregateKey();
                        Aggregate aggregate = null;
                        for (Node node : cluster.getNodes().values()) {
                            aggregate = node.getAggregates().get(aggregateKey);
                            if (aggregate != null) {
                                break;
                            }
                        }
                        if (aggregate != null) {
                            Map<String, String> aggregateMap = new HashMap<>(2);
                            aggregateMap.put("AggregateUUID", aggregate.getKey().getOcieUUID());
                            aggregateMap.put("AggregateName", aggregate.getName());
                            qosVolumeWorkloadUUIDToAggregateMap.put(qosVolumeWorkload.getKey().getOcieUUID(), aggregateMap);
                        }
                    }
                }
            }
            qosVolumeWorkloadUUIDs.add(qosVolumeWorkloadUUID);
        }

        for(final QosWorkloadNodeRelationship qosWorkloadNodeRelationship : cluster.getQosWorkloadNodeRelationships().values()) {
                String qosWorkloadNodeRelationshipUUID = qosWorkloadNodeRelationship.getKey().getOcieUUID();
                Map<String,String> nodeNameAndWorkloadUUID = new HashMap<String, String>(3);
                String nodeName = cluster.getNodes().get(qosWorkloadNodeRelationship.getNodeKey()).getName();
                String workloadUUID = qosWorkloadNodeRelationship.getQosWorkloadKey().getOcieUUID();
                nodeNameAndWorkloadUUID.put("WorkloadConstituentNodeName", nodeName);
                nodeNameAndWorkloadUUID.put("WorkloadUUID", workloadUUID); 
                nodeNameAndWorkloadUUID.put("WorkloadName", cluster.getQosVolumeWorkloads().get(qosWorkloadNodeRelationship.getQosWorkloadKey()).getName());
                String perfUUID = nodeName + ";" + workloadUUID.substring(workloadUUID.indexOf(".")+1, workloadUUID.length()); 
                qosWorkloadConstituentMap.put(qosWorkloadNodeRelationshipUUID, nodeNameAndWorkloadUUID);
                qosWorkloadConstituentUUIDs.add(perfUUID);
        }
        // loop through qos volume workload details
        QosVolumeWorkloads qosVolumeWorkloads = cluster.getQosVolumeWorkloads();
        for (final QosWorkloadDetail qosWorkloadDetail : cluster.getQosWorkloadDetails().values()) {
            final String workloadDetailUUID = qosWorkloadDetail.getKey().getID(QosWorkloadDetail.QosWorkloadDetailKey.Field.uuid);
            
            Map<String, String> nodeNameAggregateUUIDAndWorkloadUUID = new HashMap<String, String>(3);
            nodeNameAggregateUUIDAndWorkloadUUID.put("NodeNamePrefix",qosWorkloadDetail.getNodeNamePrefix());
            nodeNameAggregateUUIDAndWorkloadUUID.put("AggregateUUIDSuffix",qosWorkloadDetail.getAggregateUUIDSuffix());
            nodeNameAggregateUUIDAndWorkloadUUID.put("WorkloadUUID",qosWorkloadDetail.getQosVolumeWorkloadKey().getOcieUUID());
            
            QosVolumeWorkload qosVolumeWorkload = qosVolumeWorkloads.get(qosWorkloadDetail.getQosVolumeWorkloadKey());
            
            if(isLessThanFullSteam){
                qosWorkloadDetailMap.put(qosWorkloadDetail.getKey().getOcieUUID(), nodeNameAggregateUUIDAndWorkloadUUID);
                qosWorkloadDetailUUIDs.add(workloadDetailUUID);
            }
            else if(qosVolumeWorkload.getVolumeKey() == null && !isLessThanFullSteam) {
                qosWorkloadDetailUUIDs.add(workloadDetailUUID);
            }

            // Keep map of parent service center to child delay centers for DELAY service centers only
            final QosServiceCenter.QosResourceKey resourceKey = qosWorkloadDetail.getQosResourceKey();
            final String resourceUUID = resourceKey.getID(QosServiceCenter.QosResourceKey.Field.uuid);
            final int nameStart = resourceUUID.lastIndexOf(':');
            final String resourceName = resourceUUID.substring(nameStart + 1);
            // DELAY CENTERS ONLY
            if (!resourceName.startsWith("DELAY")) {
                continue;
            }
            if (!qosServiceCenterUUIDtoWorkloadDetailUUIDs.containsKey(resourceUUID)) {
                final Set<String> workloadUUIDs = new HashSet<String>();
                qosServiceCenterUUIDtoWorkloadDetailUUIDs.put(resourceUUID, workloadUUIDs);
            }
            final Set<String> workloadUUIDs = qosServiceCenterUUIDtoWorkloadDetailUUIDs.get(resourceUUID);
            workloadUUIDs.add(workloadDetailUUID);
        }



        for (final Node node : cluster.getNodes().values()) {
            final String nodeUUID = node.getKey().getOcieUUID();
            volumeNodePerfUUIDs.add(nodeUUID);
            final String nodeName = node.getName();
            nodeNameToUUIDMap.put(nodeName, nodeUUID);

            final String systemPerfUUID = getNodePerfUUID(nodeName);
            systemPerfUUIDs.add(systemPerfUUID);
            systemPerfUUIDtoNodeUUID.put(systemPerfUUID, nodeUUID);

            final String extCacheObjPerfUUID = getExtCacheObjPerfUUID(nodeName);
            extCachePerfObjUUIDs.add(extCacheObjPerfUUID);
            extCachePerfUUIDtoNodeUUID.put(extCacheObjPerfUUID, nodeUUID);

            final String waflPerfUUID = getWaflPerfUUID(nodeName);
            waflPerfUUIDs.add(waflPerfUUID);
            waflPerfUUIDtoNodeUUID.put(waflPerfUUID, nodeUUID);

            for (final Processor processor : node.getProcessors().values()) {
                final String processorPerfUUID = nodeName + ":kernel:processor" + processor.getProcessorNumber();
                processorPerfUUIDs.add(processorPerfUUID);
            }

            nodeNameToCoreCount.put(node.getName(), node.getProcessors().size());

            // Why not just return foundation UUIDs and provide static translate methods to get performanece UUIDs?
            // But client can't provide nodeName
            
            
            for (final Aggregate aggregate : node.getAggregates().values()) {
                String aggrPerfUUID;
                if(isLessThanFullSteam){
                    aggrPerfUUID = getNodeKernelPerfUUID(nodeName, aggregate.getName());
                }
                else{
                    aggrPerfUUID = aggregate.getKey().getOcieUUID();
                }
                aggregatePerfUUIDs.add(aggrPerfUUID);
                aggregatePerfUUIDToUUID.put(aggrPerfUUID, aggregate.getKey().getOcieUUID());
                if (aggregate.isHybridEnabled() != null && aggregate.isHybridEnabled()) {
                    hybridAggrUUIDs.add(aggrPerfUUID);
                }
                aggregateUUIDToNameMap.put(aggregate.getKey().getOcieUUID(), aggregate.getName());
            }
            
           

            for (final NetworkPort port : node.getNetworkPorts().values()) {
                final String portName = port.getKey().getPortName();
                final String portPerfUUID = getNodeKernelPerfUUID(nodeName, portName);
                ipPortPerfUUIDs.add(portPerfUUID);
                ipPortPerfUUIDToUUIDMap.put(portPerfUUID, port.getKey().getOcieUUID());
                ipPortToLinkSpeedMap.put(port.getKey().getOcieUUID(),port.getOperSpeed());
            }

            for (final FcpPort fcpPort : node.getFcpPorts().values()) {
                final FcpPort.FcpPortKey portKey = fcpPort.getKey();
                final String foundationUUID = portKey.getOcieUUID();
                final String perfUUID = portKey.getID(FcpPort.FcpPortKey.Field.port_name);
                fcpPortPerfUUIDToUUIDMap.put(perfUUID, foundationUUID);
                fcpPortToLinkSpeedMap.put(foundationUUID, fcpPort.getDataLinkRate());
            }

            for (final QosServiceCenter qosServiceCenter : node.getQosServiceCenters().values()) {
                final String qosServerCenterName = qosServiceCenter.getName();
                final String qosServiceCenterUUID = qosServiceCenter.getKey().getID(QosServiceCenter.QosResourceKey.Field.uuid);
                if (!qosServerCenterName.startsWith("DELAY")) {
                    nonDelayCenterResourceUUIDs.add(qosServiceCenterUUID);
                }
            }
            
            for(final StorageAdapter storageAdapter: node.getStorageAdapters().values()){
            	storageAdapterPerfUUIDs.add(getNodeKernelPerfUUID(node.getName(), storageAdapter.getAdapterKey().getID(Field.adapter_name)));
            }
        }

        for (final Disk disk : cluster.getDisks().values()) {
            final String diskUUID = disk.getKey().getOcieUUID();
            diskUUIDs.add(disk.getKey().getOcieUUID());
            diskNameToUUIDMap.put(disk.getName(), diskUUID);
            final String nodeName = disk.getOwnerNodeName();
            if(nodeName != null) {
                final String nodeUUID = nodeNameToUUIDMap.get(nodeName);
                if(nodeUUID!=null) {  //burt 894335 sometimes node does not exist with the nodeName.
                    ArrayList<String> diskListUUIDs;
                    if(!nodeUUIDToDiskUUIDs.containsKey(nodeUUID)){
                        diskListUUIDs = new ArrayList<String>();
                        nodeUUIDToDiskUUIDs.put(nodeUUID, diskListUUIDs);
                    }
                    diskListUUIDs = (ArrayList<String>) nodeUUIDToDiskUUIDs.get(nodeUUID);
                    diskListUUIDs.add(diskUUID);
                }
            }
        }

        for(final DiskAggregateRelationship diskAggregateRelationship : cluster.getDiskAggregateRelationships().values()) {
            String diskUUID = null;
            Disk.DiskKey diskKey = diskAggregateRelationship.getDiskKey();
            Aggregate.AggregateKey aggregateKey = diskAggregateRelationship.getAggregateKey();
            if(aggregateKey!=null)    {
                String aggrUUID = aggregateKey.getOcieUUID();
                if(diskKey!=null) {
                    diskUUID = diskKey.getOcieUUID();
                }
                if(aggregateUUIDToDiskUUIDList.containsKey(aggrUUID))  {
                    List<String> value = aggregateUUIDToDiskUUIDList.get(aggrUUID);
                    value.add(diskUUID);
                }
                else {
                    List<String> newValue = new ArrayList<>();
                    newValue.add(diskUUID);
                    aggregateUUIDToDiskUUIDList.put(aggrUUID, newValue);
                }
            }
        }

        // Node Protocol Perf UUIDs
        updateNodeProtocolPerfUUIDMap("CIFS", cluster.getCifsNodeUUIDNameMap(), nodeNameToUUIDMap, cifsPerfUUIDtoNodeUUID);
        updateNodeProtocolPerfUUIDMap("SMB1", cluster.getSmb1NodeUUIDNameMap(), nodeNameToUUIDMap, smb1PerfUUIDtoNodeUUID);
        updateNodeProtocolPerfUUIDMap("SMB2", cluster.getSmb2NodeUUIDNameMap(), nodeNameToUUIDMap, smb2PerfUUIDtoNodeUUID);
        updateNodeProtocolPerfUUIDMap("NFSv3", cluster.getNfsv3NodeUUIDNameMap(), nodeNameToUUIDMap, nfsv3PerfUUIDtoNodeUUID);
        updateNodeProtocolPerfUUIDMap("NFSv4", cluster.getNfsv4NodeUUIDNameMap(), nodeNameToUUIDMap, nfsv4PerfUUIDtoNodeUUID);
        updateNodeProtocolPerfUUIDMap("NFSv4_1", cluster.getNfsv4_1NodeUUIDNameMap(), nodeNameToUUIDMap, nfsv4_1PerfUUIDtoNodeUUID);
    }


    private void updateNodeProtocolPerfUUIDMap(final String protocolName, final Map<String, String> protoUUIDToNodeName,
                                               final Map<String, String> nodeNameToUUID, final Map<String, String> protoUUIDToFoundationUUID) {
        for (final Map.Entry<String, String> perfEntry : protoUUIDToNodeName.entrySet()) {
            final String perfUUID = perfEntry.getKey();
            final String nodeName = perfEntry.getValue();
            if (!nodeNameToUUID.containsKey(nodeName)) {
                if (logger.isDebugEnabled()) {
                    logger.debug("Unkown node " + nodeName + " for " + protocolName + " perf uuid " + perfUUID);
                }
                logger.error("Unkown node " + nodeName + " for " + protocolName + " perf uuid " + perfUUID);
                continue;
            }
            final String nodeUUID = nodeNameToUUID.get(nodeName);
            protoUUIDToFoundationUUID.put(perfUUID, nodeUUID);
        }
    }

    private String getNodeKernelPerfUUID(final String nodeName, final String childName) {
        return nodeName + ":kernel:" + childName;
    }

    private String getNodePerfUUID(final String nodeName) {
        if(isLessThanFullSteam){
            return nodeName + ":kernel:system";
        }
        else{
            return nodeName + ":kernel:" + nodeName;
        }
    }
    
    private String vServerUUIDVolNameComboKey(String vServerUUID, String volumeName){
    	return vServerUUID + volumeName ;
    }
    

    private String getWaflPerfUUID(final String nodeName) {
        return nodeName + ":kernel:wafl";
    }

    private String getExtCacheObjPerfUUID(final String nodeName) {
        return nodeName + ":kernel:ec0";
    }

    @SuppressWarnings({"UnusedDeclaration"})
    public String getClusterVersion() {
        return clusterVersion;
    }

    public boolean isLessThanSierraNevada() {
        return isLessThanSierraNevada;
    }

    public boolean isLessThanFullSteam() {
        return isLessThanFullSteam;
    }
    
    public boolean isGreaterThanFullSteam0(){
    	return isGreaterThanFullSteam0;
    }

    public List<String> getCpuColumnNames() {
        return cpuColumnNames;
    }
    
    public List<String> getWaflCpPhaseTimesColumnNames(){
    	return waflCpTimesColumnNames;
    }

    public List<String> getVolumeWorkloadReadIOColumns() {
        return volumeWorkloadReadIOColumns;
    }
    
    public Map<String, String> getCifsPerfUUIDtoNodeUUID() {
        return cifsPerfUUIDtoNodeUUID;
    }

    public Map<String, String> getSmb1PerfUUIDtoNodeUUID() {
        return smb1PerfUUIDtoNodeUUID;
    }

    public Map<String, String> getSmb2PerfUUIDtoNodeUUID() {
        return smb2PerfUUIDtoNodeUUID;
    }

    public Map<String, String> getNfsv3PerfUUIDtoNodeUUID() {
        return nfsv3PerfUUIDtoNodeUUID;
    }

    public Map<String, String> getNfsv4PerfUUIDtoNodeUUID() {
        return nfsv4PerfUUIDtoNodeUUID;
    }

    public Map<String, String> getNfsv4_1PerfUUIDtoNodeUUID() {
        return nfsv4_1PerfUUIDtoNodeUUID;
    }
    
    public boolean isCookedPersistenceEnabled() {
        return cookedPersistenceEnabled;
    }

    public void setCookedPersistenceEnabled(boolean cookedPersistenceEnabled) {
        this.cookedPersistenceEnabled = cookedPersistenceEnabled;
    }

    public boolean isRawPersistenceEnabled() {
        return rawPersistenceEnabled;
    }

    public void setRawPersistenceEnabled(boolean rawPersistenceEnabled) {
        this.rawPersistenceEnabled = rawPersistenceEnabled;
    }

}
