package com.onaro.sanscreen.acquisition.netappstorageperformance.datasource;

import java.io.Serializable;

/**
 * This class holds the Disk Specification data
 * Source of information is hard drive manufacture
 */
public class NetAppDiskSpecification implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * the Disk model
     */
    private String model;

    /**
     * Average time to move head to a specific track (in ms)
     * for example 7.5ms
     */
    private float seekTime;

    /**
     * the disk speed RPM (in Round Per Minute)
     */
    private int speed;

    /**
     * the disk Potential Transfer Rate (in MB/s)
     */
    private long potentialTransferRate;


    public NetAppDiskSpecification(String model, float seekTime, int speed, long potentialTransferRate) {
        this.model = model;
        this.seekTime = seekTime;
        this.speed = speed;
        this.potentialTransferRate = potentialTransferRate;
    }

    public String getModel() {
        return model;
    }

    public void setModel(String model) {
        this.model = model;
    }

    public float getSeekTimeMs() {
        return seekTime;
    }

    public void setSeekTime(float seekTime) {
        this.seekTime = seekTime;
    }

    public int getSpeed() {
        return speed;
    }

    public void setSpeed(int speed) {
        this.speed = speed;
    }


    public long getPotentialTransferRate() {
        return potentialTransferRate;
    }

    public void setPotentialTransferRate(long potentialTransferRate) {
        this.potentialTransferRate = potentialTransferRate;
    }

    public boolean equals(Object o) {
        if (this == o)
            return true;
        if (o == null || getClass() != o.getClass())
            return false;

        NetAppDiskSpecification that = (NetAppDiskSpecification) o;

        if (potentialTransferRate != that.potentialTransferRate)
            return false;
        if (seekTime != that.seekTime)
            return false;
        if (speed != that.speed)
            return false;
        if (!model.equals(that.model))
            return false;

        return true;
    }


    public int hashCode() {
        int result;
        result = (model != null ? model.hashCode() : 0);
        result = 31 * result + seekTime != +0.0f ? Float.floatToIntBits(seekTime) : 0;
        result = 31 * result + (int) (speed ^ (speed >>> 32));
        result = 31 * result + (int) (potentialTransferRate ^ (potentialTransferRate >>> 32));
        return result;
    }

    public String toString() {
        return "DiskSpecification{" +
                "model='" + model + '\'' +
                ", seekTime=" + seekTime +
                ", speed=" + speed +
                ", potentialTransferRate=" + potentialTransferRate +
                '}';
    }
}
