package com.onaro.sanscreen.acquisition.schema;

import java.io.IOException;
import java.io.Serializable;
import java.io.Writer;
import java.io.Serializable;

/**
 * Data source and sample information
 * Contains information about the time & date of the sample, information about the data source generating this sample, etc...
 */
public class Info extends XMLNode<Root> implements Serializable {
    private final String dataSourceName;
    private final long originatorID;

    private String date = null;
    private Long sampleTimeSecs = null;
    private String version = null;
    private String originatorKey = null;
    private String originatorDisplayName = null;
    private CommunicationMethod communication = new CommunicationMethod();

    public Info(final Root parent, final long originatorID, final String datasourceName) throws SchemaException {
        super(parent, "Info");

        if (datasourceName == null || datasourceName.length() == 0)
            throw new SchemaException(getNodeName() + " Type requires non-null datasourceName");

        this.originatorID = originatorID;
        this.dataSourceName = datasourceName;
    }

    public Key getKey() {
        return getParent().getKey();
    }

    public String getDataSourceName() {
        return dataSourceName;
    }

    public long getOriginatorID() {
        return originatorID;
    }

    public String getDate() {
        return date;
    }

    public void setDate(String date) {
        this.date = date;
    }

    public long getSampleTimeSecs() {
        return sampleTimeSecs;
    }

    public void setSampleTimeSecs(long sampleTimeSecs) {
        this.sampleTimeSecs = sampleTimeSecs;
    }

    public String getVersion() {
        return version;
    }

    public void setVersion(String version) {
        this.version = version;
    }

    public String getOriginatorKey() {
        return originatorKey;
    }

    public void setOriginatorKey(String originatorKey) {
        this.originatorKey = originatorKey;
    }

    public void setCommunicationMethodVersion(final String version) {
        communication.setVersion(version);

    }

    public void setCommunicationMethodName(final String name) {
        communication.setName(name);
    }

    public void setCommunicationMethodDescription(final String desc) {
        communication.setDescription(desc);
    }

    public void setCommunicationMethod(final String apiName, final String apiVersion, final String desc) {
        communication = new CommunicationMethod(apiName, apiVersion, desc);
    }

    public String getOriginatorDisplayName() {
        return originatorDisplayName;
    }

    public void setOriginatorDisplayName(String originatorDisplayName) {
        this.originatorDisplayName = originatorDisplayName;
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException, SchemaException {
        addAttribute(writer, "DataSourceName", dataSourceName);
        addAttribute(writer, "OriginatorID", originatorID);
        addAttribute(writer, "Date", date);
        addAttribute(writer, "SampleTimeSecs", sampleTimeSecs);
        addAttribute(writer, "Version", version);
        addAttribute(writer, "OriginatorKey", originatorKey);
        addAttribute(writer, "ApiName", communication.getName());
        addAttribute(writer, "ApiVersion", communication.getVersion());
        addAttribute(writer, "ApiDescription", communication.getDescription());

        // TODO: Add originator display name to schema
//        addAttribute(writer, "OriginatorDisplayName", originatorDisplayName);
    }

    /**
     *  This class represents the way data source communicate with the device
     */
    private class CommunicationMethod implements Serializable {
        private String name;
        private String version;
        private String description;

        private CommunicationMethod(final String name, final String version, final String description) {
            this.name = name;
            this.version = version;
            this.description = description;
        }

        private CommunicationMethod() {
        }

        public String getName() {
            return name;
        }

        public void setName(final String name) {
            this.name = name;
        }

        public String getVersion() {
            return version;
        }

        public void setVersion(final String version) {
            this.version = version;
        }

        public String getDescription() {
            return description;
        }

        public void setDescription(final String description) {
            this.description = description;
        }
    }

}

