package com.onaro.sanscreen.acquisition.schema;

import java.io.Serializable;

/**
 * Base interface for all nodes included in the schema
 *
 * <p/>A node object has a parent (since the schema is a tree structure) which allows traversal of objects in the tree going
 * "up" and "down" the hierarchy.
 *
 * <p/>A node also has a key which is a {@link Comparable} object that can be used as a key to maps containing schema objects
 * and is used internally by the schema to avoid duplicates. 
 */
public interface Node<P> extends Serializable {
    /**
     * @return The parent object of this node (or <code>null</code> if this is the root and there is no parent)
     */
    public P getParent();

    /**
     * @return A key object representing this node.
     */
    public <K extends Key> K getKey();
}
