package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.io.Writer;

/**
 * This switch object represents chassis switches (like the CISCO chassis). In this configuration, in the fabric
 * will be the logical switches defined for this chassis.
 */
public class ChassisSwitch extends SwitchType<FabricRoot> {
    private Boolean vsanEnabled = null;
    private Boolean sanRoutingEnabled = null;

    /**
     * Use {@link FabricRoot#createChassisSwitch} to create this object
     */
    ChassisSwitch(final FabricRoot parent, final String wwn, final Type switchType) throws SchemaException {
        super(parent, "Switch", wwn, switchType);
    }

    public FabricRoot getRoot() {
        return getParent();
    }

    public boolean isVSANEnabled() {
        return (vsanEnabled != null && vsanEnabled);
    }

    public void setVSANEnabled(boolean vsanenabled) {
        vsanEnabled = vsanenabled;
    }

    public boolean isSANRoutingEnabled() {
        return (sanRoutingEnabled != null && sanRoutingEnabled);
    }

    public void setSANRoutingEnabled(final boolean sanRoutingEnabled) {
        this.sanRoutingEnabled = sanRoutingEnabled;
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException {
        super.marshallAttributes(cache, writer);

        if (vsanEnabled != null)
            addAttribute(writer, "VSANEnabled", (vsanEnabled ? "1" : "0"));

        if (sanRoutingEnabled != null)
            addAttribute(writer, "SANRouteEnabled", (sanRoutingEnabled ? "1" : "0"));
    }
}
