package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.io.Writer;

public class ConnectedPort extends PortType<Port> {
    /**
     * Use {@link Port#createConnectedPort} to create this object
     * @param parent the parent this connected port object belongs to
     * @param portWwn the port WWN of this connected port
     * @throws com.onaro.sanscreen.acquisition.schema.SchemaException throws when the port wwn is in invalid format
     */
    ConnectedPort(final Port parent, final String portWwn) throws SchemaException {
        super(parent, new PortTypeKey(portWwn));
    }

    /**
     * Construct a connected port by switch wwn and port ID when port wwn is not available.
     *
     * @param parent the parent this connected port belongs to
     * @param switchWwn the switch wwn of this connected port
     * @param portId the port ID of this connected port
     * @throws SchemaException throws when either switch wwn or port ID is invalid
     */
    ConnectedPort(final Port parent, final String switchWwn, final String portId) throws SchemaException {
        super(parent, new PortIdKey(switchWwn, portId));
    }

    public Port getPort() {
        return getParent();
    }

    public String getWWN() throws SchemaException {
        if (getKey().getPortTypeKeyType() == PortTypeKeyType.PORT_TYPE_KEY) {
            return getKey().toString();
        }
        // Should not come here
        throw new SchemaException("Method getWWN() only supports PortTypeKey but not PortIdKey", null);
    }

    public boolean hasWWN() {
        return getKey().getPortTypeKeyType() == PortTypeKeyType.PORT_TYPE_KEY;
    }

    /**
     * HEADSUP: Promote access priviledges for this method for the use of Hub#marshall}
     */
    @Override
    public void marshall(final MarshallingCache cache, final Writer writer, final int tabs) throws IOException, SchemaException {

        //if we can not get the  speed of the connected port ,we will take the speed from the parent port.
        if (getPortSpeed() == null)
            setPortSpeed(getParent().getPortSpeed());

        super.marshall(cache, writer, tabs);
    }
}
