package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;

import java.io.IOException;
import java.io.Writer;

public class DataStoreToNasShare extends XMLNode<VirtualizationRoot> {
    private final DataStoreToNasShareKey key;

    DataStoreToNasShare(final VirtualizationRoot parent, final DataStore.DataStoreKey dataStoreKey, final NasShare.NasShareKey nasShareKey) throws SchemaException {
        super(parent, "HV_DataStoreToNasShare");
        key = new DataStoreToNasShareKey(dataStoreKey, nasShareKey);
    }

    @SuppressWarnings({"unchecked"})
    @Override
    public DataStoreToNasShareKey getKey() {
        return key;
    }

    @Override
    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException, SchemaException {
        super.marshallAttributes(cache, writer);
        addAttribute(writer, "HV_DataStore_OID", key.dataStoreKey);
        addAttribute(writer, "HV_NasShare_OID", key.nasShareKey);
        addAttribute(writer, "OID", key.toString());
    }

    public static class DataStoreToNasShareKey implements Key<DataStoreToNasShareKey> {
        private final DataStore.DataStoreKey dataStoreKey;
        private final NasShare.NasShareKey nasShareKey;

        public DataStoreToNasShareKey(final DataStore.DataStoreKey dataStoreKey, final NasShare.NasShareKey nasShareKey) throws SchemaException {
            if (dataStoreKey == null || nasShareKey == null) {
                throw new SchemaException("DataStoreToNasShare must have DataStoreKey and NasShareKey");
            }
            this.dataStoreKey = dataStoreKey;
            this.nasShareKey = nasShareKey;
        }

        @Override
        public int compareTo(final DataStoreToNasShareKey o) {
            final int result = nasShareKey.compareTo(o.nasShareKey);
            if (result != 0) {
                return result;
            }
            return dataStoreKey.compareTo(o.dataStoreKey);
        }

        @Override
        public boolean equals(final Object o) {
            if (this == o) return true;
            if (!(o instanceof DataStoreToNasShareKey)) return false;

            final DataStoreToNasShareKey that = (DataStoreToNasShareKey) o;

            if (dataStoreKey != null ? !dataStoreKey.equals(that.dataStoreKey) : that.dataStoreKey != null)
                return false;
            if (nasShareKey != null ? !nasShareKey.equals(that.nasShareKey) : that.nasShareKey != null) return false;

            return true;
        }

        @Override
        public int hashCode() {
            int result = dataStoreKey != null ? dataStoreKey.hashCode() : 0;
            result = 31 * result + (nasShareKey != null ? nasShareKey.hashCode() : 0);
            return result;
        }

        @Override
        public String toString() {
            return new StringBuilder().append(dataStoreKey).append(':').append(nasShareKey).toString();
        }
    }
}
