package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;

import java.io.IOException;
import java.io.Writer;

@SuppressWarnings({"unchecked"})
public class DeviceCapability extends XMLNode<Storage>{
    private Boolean active = Boolean.FALSE;
    private Boolean used = Boolean.FALSE;
    private Boolean licensed = Boolean.FALSE;

    private final String technology;
    private final DeviceCapabilityKey key;
    //todo convert technology to enum
    DeviceCapability(final Storage parent, final String technology) throws SchemaException {
        super(parent, "Capability");

        if (technology == null || technology.length() == 0)
            throw new SchemaException("device supported technology must not be null");

        this.technology = technology;

        this.key = new DeviceCapabilityKey(parent, technology);
    }

    protected void marshallAttributes(final MarshallingCache marshallingCache, final Writer writer) throws IOException, SchemaException {
        addAttribute(writer, "Type", technology);
        addAttribute(writer, "Active", active);
        addAttribute(writer, "Licensed", licensed);
        addAttribute(writer, "Used", used);
    }

    public final DeviceCapabilityKey getKey() {
        return key;
    }

    public final String getTechnology() {
        return technology;
    }

    public final Boolean isActive() {
        return active;
    }

    public final void setActive(final Boolean active) {
        this.active = active;
    }

    public final Boolean isInUse() {
        return used;
    }

    public final void setUsed(final Boolean used) {
        this.used = used;
    }

    public final Boolean isLicensed() {
        return licensed;
    }

    public final void setLicensed(final Boolean licensed) {
        this.licensed = licensed;
    }

    public String toString() {
        final StringBuilder sb = new StringBuilder();
        sb.append("key=").append(key).append(";");
        sb.append("active=").append(active).append(";");
        sb.append("used=").append(used).append(";");
        sb.append("licensed=").append(licensed).append(";");
        return sb.toString();
    }

    public static class DeviceCapabilityKey<Parent extends XMLNode> implements Key<DeviceCapabilityKey> {
        private final Key parentKey;
        private final String technology;

        public DeviceCapabilityKey(final Parent parent, final String technology) throws SchemaException {
            if (parent == null || technology == null)
                throw new SchemaException("parent device and supported technology must not be null");

            this.parentKey = parent.getKey();
            this.technology = technology;
        }

        public int compareTo(final DeviceCapabilityKey o) {
            int i = getParentKey().compareTo(o.getParentKey());
            if (i != 0)
                return i;

            return technology.compareTo(o.technology);
        }

        public boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final DeviceCapabilityKey that = (DeviceCapabilityKey) o;

            if (parentKey != null ? !parentKey.equals(that.parentKey) : that.parentKey != null) return false;
            if (technology != null ? !technology.equals(that.technology) : that.technology != null) return false;

            return true;
        }

        public int hashCode() {
            int result;
            result = (parentKey != null ? parentKey.hashCode() : 0);
            result = 31 * result + (technology != null ? technology.hashCode() : 0);
            return result;
        }

        public Key getParentKey() {
            return parentKey;
        }

        public String getTechnology() {
            return technology;
        }

        public String toString() {
            return parentKey + ":" + technology;
        }
    }
}
