package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.io.Writer;

/**
 * Base class for switches that are associated with fabrics. This objects adds the association to a fabric to the
 * basic switch type object.
 *
 * @see Switch
 * @see LogicalSwitch
 */
public abstract class FabricSwitchType<F extends Fabric> extends SwitchType<F> {
    protected Integer domainID = null;
    protected DomainIdType domainIDType = null;
    protected Integer priority = null;
    protected Integer generated;

    protected FabricSwitchType(final F parent, final String elementName, final String wwn, final Type switchType) throws SchemaException {
        super(parent, elementName, wwn, switchType);
    }

    public F getFabric() {
        return getParent();
    }

    public final Integer getDomainID() {
        return domainID;
    }

    public void setDomainID(final Integer domainID) {
        this.domainID = domainID;
    }

    public DomainIdType getDomainIDType() {
        return domainIDType;
    }

    public void setDomainIDType(final DomainIdType domainIDType) {
        this.domainIDType = domainIDType;
    }

    public Integer getPriority() {
        return priority;
    }

    public void setPriority(final Integer priority) {
        this.priority = priority;
    }

    /**
     * If this switch represents a device that we have not directly acquired
     * and only have a partial view. Used for creating NPV switch devices
     * when we see a switch is connected to one.
     * @param generatedBoolean - true if switch has been generated without directly acquiring
     * from that device.
     */
    public void setGenerated(final boolean generatedBoolean) {
        this.generated = generatedBoolean ? 1 : 0;
    }

    /**
     *
     * @return
     */
    public int isGenerated() {
        return generated;
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException {
        super.marshallAttributes(cache, writer);

        addAttribute(writer, "DomainId", domainID);
        addAttribute(writer, "DomainIdType", domainIDType);
        addAttribute(writer, "Priority", priority);
        addAttribute(writer, "Generated", generated);

    }
}
