package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.XMLNode;
import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.util.Capacity;

import java.io.Writer;
import java.io.IOException;


public class FileSystem extends XMLNode<FsHost> {

    public static final String ELEMENT_NAME = "FileSystem" ;
    /**
     *  File system name - Drive letter for windows or mount point for unix
     */
    private String name ;
    /**
     *  Hardware ID - Disk number for windows, or disk id for Unix/Linux
     */
    private String hardwareID ;
    /**
     * Location type - SAN, NAS or Local. Optional
     */
    private LocationType locationType ;
    /**
     * Type of file system - uxfs, ntfs, ext2 etc
     */
    private String fileSystemType ;
    /**
     * Size in MB
     */
    private Capacity allocatedCapacity ;
    /**
     * Used space in MB
     */
    private Capacity   usedCapacity ;

    private FileSystemKey key ;

    FileSystem(final String name, Capacity allocatedCapacity, Capacity usedCapacity, final FsHost parent) {
        super(parent, ELEMENT_NAME);
        this.name = name ;
        key = new FileSystemKey(name, parent.getKey()) ;
        this.allocatedCapacity = allocatedCapacity ;
        this.usedCapacity = usedCapacity ;
    }

    public String getName() {
        return name;
    }

    public String getHardwareID() {
        return hardwareID;
    }

    public void setHardwareID(final String hardwareID) {
        this.hardwareID = hardwareID;
    }

    public LocationType getLocationType() {
        return locationType;
    }

    public void setLocationType(final LocationType locationType) {
        this.locationType = locationType;
    }

    public String getFileSystemType() {
        return fileSystemType;
    }

    public void setFileSystemType(final String fileSystemType) {
        this.fileSystemType = fileSystemType;
    }

    public Capacity getAllocatedCapacity() {
        return allocatedCapacity;
    }


    public Capacity getUsedCapacity() {
        return usedCapacity;
    }

    @SuppressWarnings("unchecked")
    public FileSystemKey getKey() {
        return key;
    }

    public static enum LocationType {
        SAN,
        NAS,
        DAS,
        Local
    }

    protected void marshallAttributes(final MarshallingCache marshallingCache, final Writer writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttribute(writer, "Name", name);
        addAttribute(writer, "AllocatedCapacityMB", allocatedCapacity.value(Capacity.Unit.MB_1024));
        addAttribute(writer, "UsedCapacityMB", usedCapacity.value(Capacity.Unit.MB_1024));
        addAttribute(writer, "HardwareID", hardwareID);
        addAttribute(writer, "LocationType", locationType);
        addAttribute(writer, "Type", fileSystemType);
    }

    public static class FileSystemKey implements Key<FileSystemKey> {

        private String name ;
        private FsHost.FileSystemHostKey hostKey ;

        public FileSystemKey(final String name, final FsHost.FileSystemHostKey hostKey) {
            this.name = name;
            this.hostKey = hostKey ;
        }

        public String getName() {
            return name;
        }

        public FsHost.FileSystemHostKey getHostKey() {
            return hostKey;
        }


        public boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final FileSystemKey that = (FileSystemKey) o;

            if (hostKey != null ? !hostKey.equals(that.hostKey) : that.hostKey != null) return false;
            if (name != null ? !name.equals(that.name) : that.name != null) return false;

            return true;
        }

        public int hashCode() {
            int result;
            result = (name != null ? name.hashCode() : 0);
            result = 31 * result + (hostKey != null ? hostKey.hashCode() : 0);
            return result;
        }

        public int compareTo(final FileSystemKey fileSystemKey) {
            if (fileSystemKey == this) return 0 ;

            int r = this.hostKey.compareTo(fileSystemKey.getHostKey()) ;
            if (r != 0) return r ;

            return this.name.compareTo(fileSystemKey.getName()) ;
        }
    }

}
