package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;

import java.io.IOException;
import java.io.Writer;

public class HostAdapter extends XMLNode<Host> {

    private final String nodeWWN;
    private String firmwareVersion = null;
    private String hardwareVersion = null;
    private String model = null;
    private String vendor = null;
    private String driver = null;
    private String modelDescription = null;
    private String name = null;
    private String nodeSymbolicName = null;
    private String optionROMVersion = null;
    private String serialNumber = null;
    private String manufacturer = null;
    private String firmware = null;

    private final HostPorts ports = new HostPorts(this);
    private final HostAdapterKey key;

    HostAdapter(final Host parent, final String wwn) {
        super(parent, "Adapter");
        this.nodeWWN = wwn;

        key = new HostAdapterKey(parent.getKey(), wwn);
    }

    public HostAdapterKey getKey() {
        return key;
    }

    public String getFirmwareVersion() {
        return firmwareVersion;
    }

    public void setFirmwareVersion(String firmwareVersion) {
        this.firmwareVersion = firmwareVersion;
    }

    public String getHardwareVersion() {
        return hardwareVersion;
    }

    public void setHardwareVersion(String hardwareVersion) {
        this.hardwareVersion = hardwareVersion;
    }

    public String getModel() {
        return model;
    }

    public void setModel(String model) {
        this.model = model;
    }

    public String getVendor() {
        return vendor;
    }

    public void setVendor(String vendor) {
        this.vendor = vendor;
    }

    public String getDriver() {
        return driver;
    }

    public void setDriver(String driver) {
        this.driver = driver;
    }

    public String getModelDescription() {
        return modelDescription;
    }

    public void setModelDescription(String modelDescription) {
        this.modelDescription = modelDescription;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getNodeSymbolicName() {
        return nodeSymbolicName;
    }

    public void setNodeSymbolicName(String nodeSymbolicName) {
        this.nodeSymbolicName = nodeSymbolicName;
    }

    public String getOptionROMVersion() {
        return optionROMVersion;
    }

    public void setOptionROMVersion(String optionROMVersion) {
        this.optionROMVersion = optionROMVersion;
    }

    public String getSerialNumber() {
        return serialNumber;
    }

    public void setSerialNumber(String serialNumber) {
        this.serialNumber = serialNumber;
    }

    public String getManufacturer() {
        return manufacturer;
    }

    public void setManufacturer(String manufacturer) {
        this.manufacturer = manufacturer;
    }

    public String getFirmware() {
        return firmware;
    }

    public void setFirmware(String firmware) {
        this.firmware = firmware;
    }


    public HostPort createPort(final String nodeWWN) throws SchemaException {
        return new HostPort(this, nodeWWN);
    }

    public String getNodeWWN() {
        return nodeWWN;
    }

    public HostPorts getPorts() {
        return ports;
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException, SchemaException {
        super.marshallAttributes(cache, writer);

        addAttribute(writer, "WWN", nodeWWN);
        addAttribute(writer, "FirmwareVersion", firmwareVersion);
        addAttribute(writer, "HardwareVersion", hardwareVersion);
        addAttribute(writer, "Model", model);
        addAttribute(writer, "Vendor", vendor);
        addAttribute(writer, "Driver", driver);
        addAttribute(writer, "ModelDescription", modelDescription);
        addAttribute(writer, "Name", name);
        addAttribute(writer, "NodeSymbolicName", nodeSymbolicName);
        addAttribute(writer, "OptionRomVersion", optionROMVersion);
        addAttribute(writer, "SerialNumber", serialNumber);
        addAttribute(writer, "Manufactuer", manufacturer);
        addAttribute(writer, "Firmware", firmware);
    }

    protected void marshallChildren(final MarshallingCache cache, final Writer writer, int tabs) throws IOException, SchemaException {
        addAllChildren(cache, writer, ports, tabs);
    }

    protected boolean hasChildren() {
        return (!ports.isEmpty());
    }

    public static final class HostAdapterKey implements Key<HostAdapterKey> {
        private final Host.HostKey hostKey;
        private final String wwn;

        public HostAdapterKey(final Host.HostKey hostKey, final String wwn) {
            this.hostKey = hostKey;
            this.wwn = wwn;
        }

        public int compareTo(final HostAdapterKey o) {
            int i;

            i = wwn.compareTo(o.getWWN());
            if (i != 0)
                return i;

            return hostKey.compareTo(o.getHostKey());
        }

        public String getWWN() {
            return wwn;
        }

        public Host.HostKey getHostKey() {
            return hostKey;
        }

        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final HostAdapterKey that = (HostAdapterKey) o;

            if (hostKey != null ? !hostKey.equals(that.hostKey) : that.hostKey != null) return false;
            if (wwn != null ? !wwn.equals(that.wwn) : that.wwn != null) return false;

            return true;
        }

        public int hashCode() {
            int result;
            result = (hostKey != null ? hostKey.hashCode() : 0);
            result = 29 * result + (wwn != null ? wwn.hashCode() : 0);
            return result;
        }

        public String toString() {
            return hostKey.toString() + ":" + wwn;
        }
    }
}
