package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;

import java.io.IOException;
import java.io.Writer;

public class HostToNasShare extends XMLNode<VirtualizationRoot> {

    private final Host.HostKey hostKey;
    private final NasShare.NasShareKey nasShareKey;

    private boolean readOnly;
    private HostToNasShareKey key;

    public HostToNasShare(final VirtualizationRoot parent, final Host.HostKey hostKey, final NasShare.NasShareKey nasShareKey) throws SchemaException {
        super(parent, "HV_HostToNasShare");
        if (hostKey == null || nasShareKey == null)
            throw new SchemaException("HostToNasShare must have HostKey and NasShareKey");
        this.hostKey = hostKey;
        this.nasShareKey = nasShareKey;
        key = new HostToNasShareKey(hostKey, nasShareKey);
    }

    public Host.HostKey getHostKey() {
        return hostKey;
    }

    public NasShare.NasShareKey getNasShareKey() {
        return nasShareKey;
    }

    public HostToNasShareKey getKey() {
        return key;
    }

    public boolean isReadOnly() {
        return readOnly;
    }

    public void setReadOnly(final boolean readOnly) {
        this.readOnly = readOnly;
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException, SchemaException {
        super.marshallAttributes(cache, writer);
        addAttribute(writer, "Host_OID", hostKey);
        addAttribute(writer, "HV_NasShare_OID", nasShareKey);
        addAttribute(writer, "ReadOnly", readOnly);
        addAttribute(writer, "OID", getKey());

    }

    public static class HostToNasShareKey implements Key<HostToNasShareKey> {

        private final Host.HostKey hostKey;
        private final NasShare.NasShareKey nasShareKey;

        public HostToNasShareKey(final Host.HostKey hostKey, final NasShare.NasShareKey nasShareKey) throws SchemaException {
            if (hostKey == null || nasShareKey == null)
                throw new SchemaException("HostToNasShare must have HostKey and NasShareKey");

            this.hostKey = hostKey;
            this.nasShareKey = nasShareKey;

        }

        public boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final HostToNasShareKey that = (HostToNasShareKey) o;

            return !(hostKey != null ? !hostKey.equals(that.hostKey) : that.hostKey != null) && !(nasShareKey != null ? !nasShareKey.equals(that.nasShareKey) : that.nasShareKey != null);

        }

        public int hashCode() {
            int result;
            result = (hostKey != null ? hostKey.hashCode() : 0);
            result = 31 * result + (nasShareKey != null ? nasShareKey.hashCode() : 0);
            return result;
        }

        public Host.HostKey getHostKey() {
            return hostKey;
        }

        public NasShare.NasShareKey getNasShareKey() {
            return nasShareKey;
        }

        public int compareTo(final HostToNasShareKey o) {
            int i = nasShareKey.compareTo(o.getNasShareKey());
            if (i != 0)
                return i;
            return hostKey.compareTo(o.getHostKey());
        }

        public String toString() {
            return hostKey + ":" + nasShareKey;
        }
    }

}
