package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;

import java.io.IOException;
import java.io.Writer;

public class ISCSINetworkPortal extends XMLNode<ISCSINetworkPortalGroup> {
    private final String ip;
    private String nic;

    private final ISCSINetworkPortalKey key;

    ISCSINetworkPortal(final ISCSINetworkPortalGroup groupNetwork, final String ip) throws SchemaException {
        super(groupNetwork, "ISCSINetworkPortal");

        if (ip == null)
            throw new SchemaException("ip  must not be null");

        this.ip = ip;
        key = new ISCSINetworkPortalKey(groupNetwork.getKey(), ip);
    }

    public String getNIC() {
        return nic;
    }

    public void setNIC(final String nic) {
        this.nic = nic;
    }

    public String getIP() {
        return ip;
    }

    @SuppressWarnings({"unchecked"})
    public ISCSINetworkPortalKey getKey() {
        return key;
    }

    protected void marshallAttributes(final MarshallingCache marshallingCache, final Writer writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);
        addAttribute(writer, "OID", getKey().toString());
        addAttribute(writer, "Ip", ip);
        addAttribute(writer, "Nic", nic);
    }

    public static class ISCSINetworkPortalKey implements Key<ISCSINetworkPortalKey> {
        private final String ip;
        private final Key key;

        public ISCSINetworkPortalKey(final Key key, final String ip) throws SchemaException {
            if (ip == null)
                throw new SchemaException("ip  must not be null");
            this.ip = ip;
            this.key = key;
        }

        public int compareTo(final ISCSINetworkPortalKey o) {
            int i = o.getIp().compareTo(getIp());
            if (i != 0)
                return i;

            return o.getKey().toString().compareTo(getKey().toString());
        }

        public String getIp() {
            return ip;
        }

        public Key getKey() {
            return key;
        }

        public boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final ISCSINetworkPortalKey that = (ISCSINetworkPortalKey) o;

            if (ip != null ? !ip.equals(that.ip) : that.ip != null) return false;
            if (key != null ? !key.equals(that.key) : that.key != null) return false;

            return true;
        }

        public int hashCode() {
            int result;
            result = (ip != null ? ip.hashCode() : 0);
            result = 31 * result + (key != null ? key.hashCode() : 0);
            return result;
        }

        public String toString() {
            return key + ":" + ip;
        }
    }
}
