package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;

import java.io.IOException;
import java.io.Writer;

public class ISCSIPortalGroupMap extends XMLNode<ISCSINode> {
    private final ISCSINetworkPortalGroup.ISCSIPortalGroupKey portalGroupKey;
    private final String iscsiTargetNodeName;

    private final ISCSIPortalGroupMapKey key;

    ISCSIPortalGroupMap(final ISCSINode parent, final ISCSINetworkPortalGroup.ISCSIPortalGroupKey portalGroupKey, final String iscsiTargetNodeName) throws SchemaException {
        super(parent, "ISCSINodeMap");

        if (portalGroupKey == null || iscsiTargetNodeName == null)
            throw new SchemaException("portalGroupKey and iscsiTargetNodeName  must not be null");
        this.portalGroupKey = portalGroupKey;
        this.iscsiTargetNodeName = iscsiTargetNodeName;
        key = new ISCSIPortalGroupMapKey(portalGroupKey, iscsiTargetNodeName);
    }

    public ISCSINetworkPortalGroup.ISCSIPortalGroupKey getPortalGroupKey() {
        return portalGroupKey;
    }

    public String getIscsiTargetNodeName() {
        return iscsiTargetNodeName;
    }

    public ISCSIPortalGroupMapKey getKey() {
        return key;
    }

    protected void marshallAttributes(final MarshallingCache marshallingCache, final Writer writer) throws IOException, SchemaException {
        super.marshallAttributes(marshallingCache, writer);

        addAttribute(writer, "OID", getKey().toString());
        addAttribute(writer, "PortalGroupOID", portalGroupKey);
    }

    public static class ISCSIPortalGroupMapKey implements Key<ISCSIPortalGroupMapKey> {
        final private ISCSINetworkPortalGroup.ISCSIPortalGroupKey portalGroupKey;
        final private String iscsiTargetNodeName;

        public ISCSIPortalGroupMapKey(final ISCSINetworkPortalGroup.ISCSIPortalGroupKey portalGroupKey, final String iscsiTargetNodeName) throws SchemaException {
            if (portalGroupKey == null || iscsiTargetNodeName == null)
                throw new SchemaException("portalGroupKey and iscsiTargetNodeName  must not be null");
            this.portalGroupKey = portalGroupKey;
            this.iscsiTargetNodeName = iscsiTargetNodeName;
        }

        public int compareTo(final ISCSIPortalGroupMapKey o) {
            int i = getIscsiTargetNodeName().compareTo(o.getIscsiTargetNodeName());
            if (i != 0)
                return i;

            return getPortalGroupKey().compareTo(o.getPortalGroupKey());
        }

        public ISCSINetworkPortalGroup.ISCSIPortalGroupKey getPortalGroupKey() {
            return portalGroupKey;
        }

        public String getIscsiTargetNodeName() {
            return iscsiTargetNodeName;
        }

        public String toString() {
            return iscsiTargetNodeName + ":" + portalGroupKey;
        }

        public boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final ISCSIPortalGroupMapKey that = (ISCSIPortalGroupMapKey) o;

            if (iscsiTargetNodeName != null ? !iscsiTargetNodeName.equals(that.iscsiTargetNodeName) : that.iscsiTargetNodeName != null)
                return false;
            if (portalGroupKey != null ? !portalGroupKey.equals(that.portalGroupKey) : that.portalGroupKey != null)
                return false;

            return true;
        }

        public int hashCode() {
            int result;
            result = (portalGroupKey != null ? portalGroupKey.hashCode() : 0);
            result = 31 * result + (iscsiTargetNodeName != null ? iscsiTargetNodeName.hashCode() : 0);
            return result;
        }
    }
}
