package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;
import com.onaro.util.ToString;

import java.io.IOException;
import java.io.Writer;
import java.util.SortedSet;
import java.util.TreeSet;

@SuppressWarnings({"unchecked"})
public class ISCSISession extends XMLNode<ISCSINode> {
    private final String initiatorNodeName;
    private final String targetNodeName;
    private final ISCSINetworkPortalGroup.ISCSIPortalGroupKey targetPortalGroup;

    private String initiatorNodeAlias;
    private String targetSessionId;
    private String initiatorSessionId;
    private int time2retain;
    private int numberOfConnections;
    private int maxNumberOfConnections;
    private SortedSet<String> ips;
    private SortedSet<ISCSISecurity> security;

    private ISCSISessionKey key;

    ISCSISession(final ISCSINode parent, final String initiatorNodeName, final String targetNodeName, final ISCSINetworkPortalGroup.ISCSIPortalGroupKey targetPortalGroup) throws SchemaException {
        super(parent, "ISCSISession");
        if (initiatorNodeName == null || targetNodeName == null || targetPortalGroup == null)
            throw new SchemaException("initiatorNodeName and targetNodeName and targetPortalGroup must not be null");

        this.initiatorNodeName = initiatorNodeName;
        this.targetNodeName = targetNodeName;
        this.targetPortalGroup = targetPortalGroup;
        this.ips = new TreeSet<String>();
        this.security = new TreeSet<ISCSISecurity>();

        key = new ISCSISessionKey(initiatorNodeName, targetNodeName, targetPortalGroup);
    }

    public String getTargetSessionId() {
        return targetSessionId;
    }

    public void setTargetSessionId(final String targetSessionId) {
        this.targetSessionId = targetSessionId;
    }

    public String getInitiatorSessionId() {
        return initiatorSessionId;
    }

    public void setInitiatorSessionId(final String initiatorSessionId) {
        this.initiatorSessionId = initiatorSessionId;
    }

    public int getTime2retain() {
        return time2retain;
    }

    public void setTime2retain(final int time2retain) {
        this.time2retain = time2retain;
    }

    public int getNumberOfConnections() {
        return numberOfConnections;
    }

    public void setNumberOfConnections(final int numberOfConnections) {
        this.numberOfConnections = numberOfConnections;
    }

    public int getMaxNumberOfConnections() {
        return maxNumberOfConnections;
    }

    public void setMaxNumberOfConnections(final int maxNumberOfConnections) {
        this.maxNumberOfConnections = maxNumberOfConnections;
    }public String getInitiatorNodeName() {
    return initiatorNodeName;
}

    public String getTargetNodeName() {
        return targetNodeName;
    }

    public String getInitiatorNodeAlias() {
        return initiatorNodeAlias;
    }

    public void setInitiatorNodeAlias(final String initiatorNodeAlias) {
        this.initiatorNodeAlias = initiatorNodeAlias;
    }

    public SortedSet<String> getIPs() {
        return ips;
    }

    public SortedSet<ISCSISecurity> getSecurity() {
        return security;
    }


    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException, SchemaException {
        super.marshallAttributes(cache, writer);

        addAttribute(writer, "OID", getKey().toString());

        addAttribute(writer, "InitiatorOID", initiatorNodeName);
        addAttribute(writer, "PortalGroupOID", targetPortalGroup);
        addAttribute(writer, "TargetSessionId", targetSessionId);
        addAttribute(writer, "InitiatorSessionId", initiatorSessionId);
        addAttribute(writer, "NumberOfConnections", numberOfConnections);
        addAttribute(writer, "MaxNumberOfConnections", maxNumberOfConnections);
        addAttribute(writer, "InitiatorIps", ToString.collectionToCSV(ips));
        addAttribute(writer, "Security", ToString.collectionToCSV(security));
    }

    public static enum ISCSISecurity {
        CHAP,
        NONE,
        IPCEC
    }

    public ISCSISessionKey getKey() {
        return key;
    }

    public static class ISCSISessionKey implements Key<ISCSISessionKey> {
        private final String initiatorNodeName;
        private final String targetNodeName;
        private final ISCSINetworkPortalGroup.ISCSIPortalGroupKey targetPortalGroup;

        public ISCSISessionKey(final String initiatorNodeName, final String targetNodeName, final ISCSINetworkPortalGroup.ISCSIPortalGroupKey targetPortalGroup) throws SchemaException {
            if (initiatorNodeName == null || targetNodeName == null || targetPortalGroup == null)
                throw new SchemaException("initiatorNodeName and targetNodeName and targetPortalGroup must not be null");

            this.initiatorNodeName = initiatorNodeName;
            this.targetNodeName = targetNodeName;
            this.targetPortalGroup = targetPortalGroup;
        }

        public String getInitiatorNodeName() {
            return initiatorNodeName;
        }

        public String getTargetNodeName() {
            return targetNodeName;
        }

        public ISCSINetworkPortalGroup.ISCSIPortalGroupKey getTargetPortalGroup() {
            return targetPortalGroup;
        }

        public int compareTo(final ISCSISessionKey o) {
            int i = getInitiatorNodeName().compareTo(o.getInitiatorNodeName());
            if (i != 0)
                return i;

            i = getTargetNodeName().compareTo(o.targetNodeName);
            if (i != 0)
                return i;
            return getTargetPortalGroup().compareTo(o.getTargetPortalGroup());
        }

        public String toString() {
            return initiatorNodeName + ":" + targetNodeName + ":" + targetPortalGroup;
        }

        public boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final ISCSISessionKey that = (ISCSISessionKey) o;

            if (initiatorNodeName != null ? !initiatorNodeName.equals(that.initiatorNodeName) : that.initiatorNodeName != null)
                return false;
            if (targetNodeName != null ? !targetNodeName.equals(that.targetNodeName) : that.targetNodeName != null)
                return false;
            if (targetPortalGroup != null ? !targetPortalGroup.equals(that.targetPortalGroup) : that.targetPortalGroup != null)
                return false;

            return true;
        }

        public int hashCode() {
            int result;
            result = (initiatorNodeName != null ? initiatorNodeName.hashCode() : 0);
            result = 31 * result + (targetNodeName != null ? targetNodeName.hashCode() : 0);
            result = 31 * result + (targetPortalGroup != null ? targetPortalGroup.hashCode() : 0);
            return result;
        }
    }
}
