package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;

import java.io.IOException;
import java.io.Writer;

/**
 * Represents an IVR zone
 */
public class IVRZone extends XMLNode<IVRZoneSet> {
    private final String zoneName;
    private final IVRZoneMembers ivrZoneMembers;

    private final IVRZoneKey key;

    IVRZone(final IVRZoneSet parent, final String zoneName) {
        super(parent, "IVRZone");
        this.zoneName = zoneName;
        this.ivrZoneMembers = new IVRZoneMembers(this);

        key = new IVRZoneKey(parent.getKey(), zoneName);
    }

    public final IVRZoneKey getKey() {
        return key;
    }

    public final IVRZoneSet getIVRZoneSet() {
        return getParent();
    }

    public final String getZoneName() {
        return zoneName;
    }

    public final IVRZoneMember createIVRZoneMember(final String wwn, final int vsan, final int afid) throws SchemaException {
        return new IVRZoneMember(this, wwn, vsan, afid);
    }

    public final IVRZoneMembers getIVRZoneMembers() {
        return ivrZoneMembers;
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException, SchemaException {
        addAttribute(writer, "IVRZoneName", zoneName);
    }

    protected boolean hasChildren() {
        return true;
    }

    protected void marshallChildren(final MarshallingCache cache, final Writer writer, final int tabs) throws IOException, SchemaException {
        addAllChildren(cache, writer, ivrZoneMembers, tabs);
    }

    public final static class IVRZoneKey implements Key<IVRZoneKey> {
        private final IVRZoneSet.IVRZoneSetKey ivrZoneSetKey;
        private final String zoneName;

        public IVRZoneKey(final IVRZoneSet.IVRZoneSetKey ivrZoneSetKey, final String zoneName) {
            this.ivrZoneSetKey = ivrZoneSetKey;
            this.zoneName = zoneName;
        }

        public int compareTo(final IVRZoneKey o) {
            // To be unique within the SAN, we need to include the parent key as well!
            int i = getIVRZoneSetKey().compareTo(o.getIVRZoneSetKey());
            if (i != 0)
                return i;

            return getZoneName().compareTo(o.getZoneName());
        }

        public boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final IVRZoneKey that = (IVRZoneKey) o;

            if (ivrZoneSetKey != null ? !ivrZoneSetKey.equals(that.ivrZoneSetKey) : that.ivrZoneSetKey != null)
                return false;
            if (zoneName != null ? !zoneName.equals(that.zoneName) : that.zoneName != null) return false;

            return true;
        }

        public int hashCode() {
            int result;
            result = (ivrZoneSetKey != null ? ivrZoneSetKey.hashCode() : 0);
            result = 29 * result + (zoneName != null ? zoneName.hashCode() : 0);
            return result;
        }

        public String toString() {
            return ivrZoneSetKey.toString() + ":" + zoneName;
        }

        public String getZoneName() {
            return zoneName;
        }

        public IVRZoneSet.IVRZoneSetKey getIVRZoneSetKey() {
            return ivrZoneSetKey;
        }
    }
}
