package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;

import java.io.IOException;
import java.io.Writer;

/**
 * Represents an IVR zone set
 */
public class IVRZoneSet extends XMLNode<IVRPhysicalFabric> {
    private final String name;
    private final boolean active;
    private final IVRZones ivrZones;

    private final IVRZoneSetKey key;

    IVRZoneSet(final IVRPhysicalFabric parent, final String name, boolean isActive) {
        super(parent, "IVRZoneSet");
        this.name = name;
        active = isActive;

        this.ivrZones = new IVRZones(this);

        key = new IVRZoneSetKey(name, isActive);
    }

    public final IVRZoneSetKey getKey() {
        return key;
    }

    public final IVRPhysicalFabric getIVRFabric() {
        return getParent();
    }

    public final String getName() {
        return name;
    }

    public final boolean isActive() {
        return active;
    }

    public final IVRZone createIVRZone(final String zoneName) {
        return new IVRZone(this, zoneName);
    }

    public final IVRZones getIvrZones() {
        return ivrZones;
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException, SchemaException {
        addAttribute(writer, "IVRZoneSetName", name);
        addAttribute(writer, "ActiveConfiguration", (active ? 1 : 0));
    }

    protected boolean hasChildren() {
        return true;
    }

    protected void marshallChildren(final MarshallingCache cache, final Writer writer, final int tabs) throws IOException, SchemaException {
        addAllChildren(cache, writer, ivrZones, tabs);
    }

    public static final class IVRZoneSetKey implements Key<IVRZoneSetKey> {
        private final String name;
        private final boolean active;

        public IVRZoneSetKey(final String name, final boolean active) {
            this.name = name;
            this.active = active;
        }

        public final int compareTo(final IVRZoneSetKey o) {
            int i;

            i = getName().compareTo(o.getName());
            if (i != 0)
                return i;

            return (isActive() != o.isActive() ? 1 : 0);
        }

        public final boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final IVRZoneSetKey that = (IVRZoneSetKey) o;

            if (active != that.active) return false;
            if (name != null ? !name.equals(that.name) : that.name != null) return false;

            return true;
        }

        public final int hashCode() {
            int result;
            result = (name != null ? name.hashCode() : 0);
            result = 29 * result + (active ? 1 : 0);
            return result;
        }

        public final String toString() {
            return name + ":" + active;
        }

        public final boolean isActive() {
            return active;
        }

        public final String getName() {
            return name;
        }
    }
}
