package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.io.Writer;

public abstract class InternalVolumeStorageSynchronization extends StorageSynchronization {

    private String technology;

    protected InternalVolumeStorageSynchronization(final Storage parent, final String sourceVolume, final String targetVolume) throws SchemaException {
        super(parent, "InternalVolumeStorageSynchronization", sourceVolume, targetVolume);
    }

    /**
     * What "technology" is used with this synchronization?   Vendor specific (e.g. "SnapMirror" in NetApp).   Optional
     * attribute - may be null.
     *
     * @return the vendor specific technology.   May be null.
     */
    public String getTechnology() {
        return technology;
    }

    /**
     * Set the technology type for this synchronization.  Vendor specific (e.g. "SnapMirror" in NetApp).  Optional.
     *
     * @param technology the vendor specific synchronization technology.
     */
    public void setTechnology(final String technology) {
        this.technology = technology;
    }

    @Override
    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException {
        addAttribute(writer, "SourceInternalVolume", getSource());
        addAttribute(writer, "TargetInternalVolume", getTarget());
        addAttribute(writer, "Mode", getMode());
        if (technology != null) {
            addAttribute(writer, "Technology", technology);
        }
    }

}
