package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.*;


public class LocalInternalVolumeStorageSynchronization extends InternalVolumeStorageSynchronization {
    private final LocalInternalVolumeStorageSynchronizationKey key;

    /**
     * Use {@link com.onaro.sanscreen.acquisition.schema.odata2.Storage#createLocalInternalVolumeStorageSynchronization} to create this object
     */
    LocalInternalVolumeStorageSynchronization(final Storage parent, final String sourceVolume, final String targetVolume) throws SchemaException {
        super(parent, sourceVolume, targetVolume);

        key = new LocalInternalVolumeStorageSynchronizationKey(parent.getKey(), sourceVolume, targetVolume);
    }

    public final LocalInternalVolumeStorageSynchronizationKey getKey() {
        return key;
    }


    public static final class LocalInternalVolumeStorageSynchronizationKey implements Key<LocalInternalVolumeStorageSynchronizationKey> {
        private final Storage.StorageKey storageKey;
        private final String sourceVolume;
        private final String targetVolume;

        public LocalInternalVolumeStorageSynchronizationKey(final Storage.StorageKey storageKey, final String sourceVolume, final String targetVolume) {
            this.storageKey = storageKey;
            this.sourceVolume = sourceVolume;
            this.targetVolume = targetVolume;
        }

        public final int compareTo(final LocalInternalVolumeStorageSynchronizationKey o) {
            int i;

            // Must be the same volume
            i = getStorageKey().compareTo(o.getStorageKey());
            if (i != 0)
                return i;

            i = getSourceVolume().compareTo(o.getSourceVolume());
            if (i != 0)
                return i;

            return getTargetVolume().compareTo(o.getTargetVolume());
        }

        public final boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final LocalInternalVolumeStorageSynchronizationKey that = (LocalInternalVolumeStorageSynchronizationKey) o;

            if (storageKey != null ? !storageKey.equals(that.storageKey) : that.storageKey != null) return false;
            if (sourceVolume != null ? !sourceVolume.equals(that.sourceVolume) : that.sourceVolume != null) return false;
            if (targetVolume != null ? !targetVolume.equals(that.targetVolume) : that.targetVolume != null) return false;

            return true;
        }

        public final int hashCode() {
            int result;
            result = (storageKey != null ? storageKey.hashCode() : 0);
            result = 29 * result + (sourceVolume != null ? sourceVolume.hashCode() : 0);
            result = 29 * result + (targetVolume != null ? targetVolume.hashCode() : 0);
            return result;
        }

        public final String toString() {
            return storageKey.toString() + ":" + getSourceVolume() + ":" + getTargetVolume();
        }

        public final Storage.StorageKey getStorageKey() {
            return storageKey;
        }

        public final String getSourceVolume() {
            return sourceVolume;
        }

        public final String getTargetVolume() {
            return targetVolume;
        }
    }
}
