package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

public class LocalQtreeStorageSynchronization extends QtreeStorageSynchronization {

    private final LocalQtreeStorageSynchronizationKey key;


    LocalQtreeStorageSynchronization(final Storage parent,
                                     final String sourceQtreeId,
                                     final String targetQtreeId) throws SchemaException {
        super(parent, sourceQtreeId, targetQtreeId);

        key = new LocalQtreeStorageSynchronizationKey(parent.getKey(), sourceQtreeId, targetQtreeId);
    }

    @SuppressWarnings({"unchecked"})
    public final LocalQtreeStorageSynchronizationKey getKey() {
        return key;
    }


    public static final class LocalQtreeStorageSynchronizationKey implements Key<LocalQtreeStorageSynchronizationKey> {
        private final Storage.StorageKey storageKey;
        private final String sourceQtreeId;
        private final String targetQtreeId;

        public LocalQtreeStorageSynchronizationKey(final Storage.StorageKey storageKey,
                                                   final String sourceQtreeId,
                                                   final String targetQtreeId) {
            this.storageKey = storageKey;
            this.sourceQtreeId = sourceQtreeId;
            this.targetQtreeId = targetQtreeId;
        }

        public final int compareTo(final LocalQtreeStorageSynchronizationKey o) {
            int i;

            // Must be the same qtree
            i = getStorageKey().compareTo(o.getStorageKey());
            if (i != 0)
                return i;

            i = getSourceQtreeId().compareTo(o.getSourceQtreeId());
            if (i != 0)
                return i;

            return getTargetQtreeId().compareTo(o.getTargetQtreeId());
        }

        public final boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final LocalQtreeStorageSynchronizationKey that = (LocalQtreeStorageSynchronizationKey) o;

            if (storageKey != null ? !storageKey.equals(that.storageKey) : that.storageKey != null) return false;
            if (sourceQtreeId != null ? !sourceQtreeId.equals(that.sourceQtreeId) : that.sourceQtreeId != null) return false;
            if (targetQtreeId != null ? !targetQtreeId.equals(that.targetQtreeId) : that.targetQtreeId != null) return false;

            return true;
        }

        public final int hashCode() {
            int result;
            result = (storageKey != null ? storageKey.hashCode() : 0);
            result = 29 * result + (sourceQtreeId != null ? sourceQtreeId.hashCode() : 0);
            result = 29 * result + (targetQtreeId != null ? targetQtreeId.hashCode() : 0);
            return result;
        }

        public final String toString() {
            return storageKey.toString() + ":" + getSourceQtreeId() + ":" + getTargetQtreeId();
        }

        public final Storage.StorageKey getStorageKey() {
            return storageKey;
        }

        public final String getSourceQtreeId() {
            return sourceQtreeId;
        }

        public final String getTargetQtreeId() {
            return targetQtreeId;
        }
    }
}
