package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;
import com.onaro.sanscreen.acquisition.schema.Key;

import java.io.IOException;
import java.io.Writer;

public abstract class PortType<P> extends XMLNode<P> {
    private State portState = null;
    private Status portStatus = null;
    private Integer portNumber = null;
    private Integer blade = null;
    private String name = null;
    private Speed portSpeed = null;
    private PhysicalState portPhysicalState = null;
    private String fc4protocol = null;
    private String gbicType = null;
    private String portID = null;
    private Type portType = null;
    private String rawPortStatus = null;
    private String controller = null;
    private final AbstractPortTypeKey key;

    protected PortType(final P parent, final AbstractPortTypeKey key) throws SchemaException {
        super(parent, "Port");

        if (key == null) {
            throw new SchemaException(getNodeName() + " Key required non-null");
        }
        key.checkValidity();
        this.key = key;
    }

    @SuppressWarnings({"unchecked"})
    public final AbstractPortTypeKey getKey() {
        return key;
    }

    public final Integer getBlade() {
        return blade;
    }

    public final void setBlade(final Integer blade) {
        this.blade = blade;
    }

    public final String getController() {
        return controller;
    }

    public final void setController(final String controller) {
        this.controller = controller;
    }

    public final String getFc4protocol() {
        return fc4protocol;
    }

    public final void setFC4Protocol(final String fc4protocol) {
        this.fc4protocol = fc4protocol;
    }

    public final String getGbicType() {
        return gbicType;
    }

    public final void setGbicType(final String gbictype) {
        gbicType = gbictype;
    }

    public final String getName() {
        return name;
    }

    public final void setName(final String name) {
        this.name = name;
    }

    public final Type getPortType() {
        return portType;
    }

    public final void setPortType(final Type portType) {
        this.portType = portType;
    }

    public final String getPortID() {
        return portID;
    }

    public final void setPortId(final String portid) {
        portID = portid;
    }

    public final Integer getPortNumber() {
        return portNumber;
    }

    public final void setPortNumber(final Integer portnumber) {
        portNumber = portnumber;
    }

    public final PhysicalState getPortPhysicalState() {
        return portPhysicalState;
    }

    public final void setPortPhysicalState(final PhysicalState portphysicalstate) {
        portPhysicalState = portphysicalstate;
    }

    public final Speed getPortSpeed() {
        return portSpeed;
    }

    public final void setPortSpeed(final Speed portspeed) {
        portSpeed = portspeed;
    }

    public final State getPortState() {
        return portState;
    }

    public final void setPortState(final State portstate) {
        portState = portstate;
    }

    public final Status getPortStatus() {
        return portStatus;
    }

    public final void setPortStatus(final Status portstatus) {
        portStatus = portstatus;
    }

    /**
     * @return The raw (vendor specific) status string as received from the device
     */
    public final String getRawPortStatus() {
        return rawPortStatus;
    }

    /**
     * Sets the raw (vendor specific) status string as received from the device
     * @param rawPortStatus the raw port status
     */
    public final void setRawPortStatus(final String rawPortStatus) {
        this.rawPortStatus = rawPortStatus;
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException, SchemaException {
        key.marshallAttributes(this, writer);
        addAttribute(writer, "PortState", portState);
        addAttribute(writer, "PortStatus", portStatus);
        addAttribute(writer, "PortNumber", portNumber);
        addAttribute(writer, "PortId", portID);
        addAttribute(writer, "Blade", blade);
        addAttribute(writer, "Name", name);
        addAttribute(writer, "PortSpeed", portSpeed);
        addAttribute(writer, "PortPhysicalState", portPhysicalState);
        addAttribute(writer, "FC4Protocol", fc4protocol);
        addAttribute(writer, "GBICType", gbicType);
        addAttribute(writer, "PortType", portType);
        addAttribute(writer, "Controller", controller);
        addAttribute(writer, "RawPortStatus", rawPortStatus);
    }

    public static enum State {
        INACTIVE(-1),
        UNKNOWN(1),
        ONLINE(2),
        OFFLINE(3),
        TESTING(4),
        FAULTY(5),
        NO_LIGHT(6),
        NOT_LICENSED(9);

        private final int e;

        State(final int e) {
            this.e = e;
        }

        @SuppressWarnings("unused")
        public int toEnum() {
            return e;
        }

        public String toString() {
            return Integer.toString(e);
        }
    }

    public static enum Status {
        INACTIVE(-1),
        UNKNOWN(1),
        NOT_SUPPORTED(2),
        OK(3),
        UNUSED(4),
        WARNING(5),
        FAULTY(6),
        DISABLED(7),
        SEGMENTED(8),
        OLS(9);

        private final int e;

        Status(final int e) {
            this.e = e;
        }

        @SuppressWarnings("unused")
        public int toEnum() {
            return e;
        }

        public String toString() {
            return Integer.toString(e);
        }
    }

    public static enum PhysicalState {
        UNKNOWN(1),
        NO_CARD(2),
        NO_MODULE(3),
        LASER_FLT(4),
        NO_LIGHT(5),
        NO_SYNC(6),
        IN_SYNC(7),
        PORT_FLT(8),
        DIAG_FLT(9),
        LOCK_REF(10);

        private final int e;

        PhysicalState(final int e) {
            this.e = e;
        }

        @SuppressWarnings("unused")
        public int toEnum() {
            return e;
        }

        public String toString() {
            return Integer.toString(e);
        }
    }

    @SuppressWarnings("unused")
    public static enum Type {
        /**
         * Fabric port. A port that is able to transmit under fabric protocol and interface over links. Can be used to connect an N_Port to a switch.
         */
        F_PORT("F-Port"),
        /**
         * Fabric loop port. A port that is able to transmit under fabric protocol and also has arbitrated-loop capabilities. Can be used to connect an NL_Port to a switch.
         */
        FL_PORT("FL-Port"),
        /**
         * Expansion port. A standard Fibre Channel mechanism that enables switches to network with each other, creating an ISL.
         */
        E_PORT("E-Port"),
        /**
         * Node port. A port on a node that can connect to a Fibre Channel port or to another N_Port in a point-topoint connection.
         */
        N_PORT("N-Port"),
        /**
         * Port used to connect a fabric switch to an NPV or access gateway device. The ports on both ends are NP-Ports
         */
        NP_PORT("NP-Port"),
        /**
         * Node loop port. A node port that has arbitrated-loop capabilities. Used to connect an equipment port to the fabric in a loop configuration through an FL_Port.
         */
        NL_PORT("NL-Port"),
        /**
         * Generic port. A port that can operate as either an E_Port or an F_Port. A port is defined as a G_Port when it is not yet connected or has not yet assumed a specific function in the fabric.
         */
        G_PORT("G-Port"),
        /**
         * A type of E_Port that connects a Brocade FC router to an edge fabric. EX_Ports limit the scope of fabric services scope but provide device connectivity using FC-NAT.
         */
        EX_PORT("EX-Port"),
        /**
         * Cisco switches specific port type that represents a connection between two chassis.
         */
        TE_PORT("TE-Port"),
        /**
         * Cisco switches specific port type that represents a connection between two chassis.  (VSAN trunk F port)
         */
        TF_PORT("TF-Port"),
        /**
         * Inter Chassis Link (ICL) Expansion port. Ports created by ICLs between two Brocade DCX switches, creating an ISL.
         */
        ICL_E_PORT("ICL-E-Port"),
        /**
         * Offline Inter Chassis Link (ICL) Expansion port.
         */
        ICL("ICL"),
        /**
         * Diagonotics port
         */
        D_PORT("D-Port");

        private final String t;

        Type(final String e) {
            this.t = e;
        }

        public String toEnum() {
            return t;
        }

        public String toString() {
            return t;
        }

        public boolean isPortConnectSwitches() {
            switch (this) {
                case E_PORT:
                case EX_PORT:
                case TE_PORT:
                case ICL_E_PORT:
                case NP_PORT:
                    return true;

                default:
                    return false;
            }
        }

        public boolean isConnectToNode() {
            switch (this) {
                case F_PORT:
                case TF_PORT:
                case FL_PORT:
                    return true;
                default:
                    return false;
            }
        }
    }

    /**
     * Enum for port speed.   The "enum" int is actually what is sent to the server and stored in the DB.
     * It is de-convoluted to speed for presentation in later translations.
     */
    @SuppressWarnings("unused")
    public static enum Speed {
        ONE_GPS(2, 1),
        TWO_GPS(3, 2),
        FOUR_GPS(6, 4),
        EIGHT_GPS(7, 8),
        TEN_GPS(8, 10),
        SIXTEEN_GPS(9, 16),
        TWENTY_GPS(10, 20);


        private final int e;
        private final int gbitsPerSecond;

        Speed(final int e, final int gbitsPerSecond) {
            this.e = e;
            this.gbitsPerSecond = gbitsPerSecond;
        }

        public int toEnum() {
            return e;
        }

        public int toGBitsPerSecond() {
            return gbitsPerSecond;
        }

        public String toString() {
            return Integer.toString(e);
        }

        public static Speed fromGBitsPerSecond(final int gbitsPerSecond) {
            for (final Speed speed : Speed.values()) {
                if (speed.gbitsPerSecond == gbitsPerSecond)
                    return speed;
            }

            // Not found...
            return null;
        }
    }

    public static enum PortTypeKeyType {
        PORT_TYPE_KEY,
        PORT_ID_KEY
    }

    public static abstract class AbstractPortTypeKey<K> implements Key<K> {
        private final PortTypeKeyType portTypeKeyType;

        protected AbstractPortTypeKey(final PortTypeKeyType portTypeKeyType) {
            this.portTypeKeyType = portTypeKeyType;
        }

        abstract void checkValidity() throws SchemaException;
        abstract void marshallAttributes(final PortType portType, final Writer writer) throws IOException;

        public PortTypeKeyType getPortTypeKeyType() {
            return portTypeKeyType;
        }

        @Override
        public String toString() {
            return super.toString();
        }
    }

    public static final class PortTypeKey extends AbstractPortTypeKey<PortTypeKey> {
        private final String portWwn;

        public PortTypeKey(final String portWwn) {
            super(PortTypeKeyType.PORT_TYPE_KEY);
            this.portWwn = portWwn.toUpperCase();
        }

        public final int compareTo(final PortTypeKey o) {
            return portWwn.compareTo(o.portWwn);
        }

        @Override
        public final boolean equals(final Object o) {
            return (this == o) || (o instanceof PortTypeKey) && portWwn.equals(((PortTypeKey) o).portWwn);
        }

        @Override
        public final int hashCode() {
            return portWwn.hashCode();
        }

        @Override
        public final String toString() {
            return portWwn;
        }

        @Override
        void checkValidity() throws SchemaException {
            if (portWwn == null || portWwn.length() == 0) {
                throw new SchemaException("PortTypeKey WWN is not valid");
            }
        }

        @Override
        void marshallAttributes(final PortType portType, final Writer writer) throws IOException {
            portType.addAttribute(writer, "WWN", portWwn);
        }
    }

    public static final class PortIdKey extends AbstractPortTypeKey<PortIdKey> {
        private final String switchWwn;
        private final String portId;

        public PortIdKey(final String switchWwn, final String portId) {
            super(PortTypeKeyType.PORT_ID_KEY);
            this.switchWwn = switchWwn.toUpperCase();
            this.portId = portId;
        }

        public String getSwitchWwn() {
            return switchWwn;
        }

        public String getPortId() {
            return portId;
        }

        @Override
        public int compareTo(final PortIdKey o) {
            final int result = switchWwn.compareTo(o.switchWwn);
            if (result != 0) {
                return result;
            }
            return portId.compareTo(o.portId);
        }

        @Override
        public boolean equals(final Object o) {
            return (this == o) || (o instanceof PortIdKey) && switchWwn.equals(((PortIdKey) o).switchWwn)
                && portId.equals(((PortIdKey) o).portId);
        }

        @Override
        public int hashCode() {
            return switchWwn.hashCode() + 31 * portId.hashCode();
        }

        @Override
        void checkValidity() throws SchemaException {
            if (switchWwn == null || switchWwn.length() == 0) {
                throw new SchemaException("PortIdKey switch WWN is invalid");
            }

            if (portId == null || portId.length() == 0) {
                throw new SchemaException("PortIdKey port ID is invalid");
            }
        }

        @Override
        void marshallAttributes(final PortType portType, final Writer writer) throws IOException {
            portType.addAttribute(writer, "NodeWWN", switchWwn);
            portType.addAttribute(writer, "PortId", portId);
        }

        @Override
        public String toString() {
            return new StringBuilder().append(switchWwn).append("-").append(portId).toString();
        }
    }
}
