package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;

import java.io.IOException;
import java.io.Writer;

public class ProtocolEndPointRef extends XMLNode<RemoteStorageSynchronization> {
    private final String id;

    private final ProtocolEndPointRefKey key;

    /**
     * Use {@link RemoteStorageSynchronization#createProtocolEndPointRef} to create this object
     */
    ProtocolEndPointRef(final RemoteStorageSynchronization parent, final String id) throws SchemaException {
        super(parent, "ProtocolEndPointRef");

        if (id == null || id.length() == 0)
            throw new SchemaException(getNodeName() + " Type requires non-null ID");

        this.id = id;

        key = new ProtocolEndPointRefKey(parent.getKey(), id);
    }

    public final ProtocolEndPointRefKey getKey() {
        return key;
    }

    public final StorageSynchronization getStorageSynchronization() {
        return getParent();
    }

    public final String getID() {
        return id;
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException {
        addAttribute(writer, "ID", id);
    }

    public static final class ProtocolEndPointRefKey implements Key<ProtocolEndPointRefKey> {
        private final RemoteStorageSynchronization.RemoteStorageSynchronizationKey remoteSyncKey;
        private final String id;

        public ProtocolEndPointRefKey(final RemoteStorageSynchronization.RemoteStorageSynchronizationKey remoteSyncKey, final String id) {
            this.remoteSyncKey = remoteSyncKey;
            this.id = id;
        }

        public final int compareTo(final ProtocolEndPointRefKey o) {
            int i;

            i = id.compareTo(o.getID());
            if (i != 0)
                return i;

            return remoteSyncKey.compareTo(o.getRemoteSyncKey());
        }

        public final boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final ProtocolEndPointRefKey that = (ProtocolEndPointRefKey) o;

            if (id != null ? !id.equals(that.id) : that.id != null) return false;
            if (remoteSyncKey != null ? !remoteSyncKey.equals(that.remoteSyncKey) : that.remoteSyncKey != null) return false;

            return true;
        }

        public final int hashCode() {
            int result;
            result = (remoteSyncKey != null ? remoteSyncKey.hashCode() : 0);
            result = 29 * result + (id != null ? id.hashCode() : 0);
            return result;
        }

        public final String toString() {
            return remoteSyncKey.toString() + ":" + id;
        }

        public final String getID() {
            return id;
        }

        public final RemoteStorageSynchronization.RemoteStorageSynchronizationKey getRemoteSyncKey() {
            return remoteSyncKey;
        }
    }
}
