package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;

import java.io.IOException;
import java.io.Writer;

/**
 * Represents abstract concept of a "qtree" synchronization setting, with source and destination qtrees.
 */
public abstract class QtreeStorageSynchronization extends StorageSynchronization {

    private String technology;

    /**
     * Create a new Qtree synchronization.
     *
     * @param parent parent schema element.
     * @param sourceQtreeId source qtree. This should be the *ID* of the  qtree as appears in ODATA.
     * @param targetQtreeId target qtree. This should be the *ID* of the  qtree as appears in ODATA.
     * @throws SchemaException exceptions building schema.
     */
    protected QtreeStorageSynchronization(final Storage parent,
                                          final String sourceQtreeId,
                                          final String targetQtreeId) throws SchemaException {
        super(parent, "QtreeStorageSynchronization", sourceQtreeId, targetQtreeId);

    }

    /**
     * What "technology" is used with this synchronization?   Vendor specific (e.g. "SnapMirror" in NetApp).   Optional
     * attribute - may be null.
     * 
     * @return the vendor specific technology.   May be null.
     */
    public String getTechnology() {
        return technology;
    }

    /**
     * Set the technology type for this synchronization.  Vendor specific (e.g. "SnapMirror" in NetApp).  Optional.
     *
     * @param technology  the vendor specific synchronization technology.
     */
    public void setTechnology(final String technology) {
        this.technology = technology;
    }


    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException {
        addAttribute(writer, "SourceQtreeId", getSource());
        addAttribute(writer, "TargetQtreeId", getTarget());
        addAttribute(writer, "Mode", getMode());
        if (technology != null) {
            addAttribute(writer, "Technology", technology);
        }
    }



}
