package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.io.Writer;

public class RemoteInternalVolumeStorageSynchronization extends InternalVolumeStorageSynchronization {
    private final String sourceStorageIP;
    private final String targetStorageIP;

    private final RemoteInternalVolumeStorageSynchronizationKey key;

    /**
     * Use {@link Storage#createRemoteInternalVolumeStorageSynchronization} to create this object
     */
    RemoteInternalVolumeStorageSynchronization(final Storage parent, final String sourceStorageIP, final String sourceVolume, final String targetStorageIP, final String targetVolume) throws SchemaException {
        super(parent, sourceVolume, targetVolume);

        if (sourceStorageIP == null || sourceStorageIP.length() == 0 || targetStorageIP == null || targetStorageIP.length() == 0)
            throw new SchemaException(getNodeName() + " Type requires non-null SourceStorageIP and TargetStorageIP");

        this.sourceStorageIP = sourceStorageIP;
        this.targetStorageIP = targetStorageIP;

        key = new RemoteInternalVolumeStorageSynchronizationKey(parent.getKey(), sourceStorageIP, sourceVolume, targetStorageIP, targetVolume);
    }

    public final RemoteInternalVolumeStorageSynchronizationKey getKey() {
        return key;
    }

    public final String getSourceStorageIP() {
        return sourceStorageIP;
    }

    public final String getTargetStorageIP() {
        return targetStorageIP;
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException {
        super.marshallAttributes(cache, writer);

        addAttribute(writer, "SourceStorageIp", sourceStorageIP);
        addAttribute(writer, "TargetStorageIp", targetStorageIP);
    }

    public static final class RemoteInternalVolumeStorageSynchronizationKey implements Key<RemoteInternalVolumeStorageSynchronizationKey> {
        private final Storage.StorageKey storageKey;
        private final String sourceStorageIP;
        private final String sourceVolume;
        private final String targetStorageIP;
        private final String targetVolume;

        public RemoteInternalVolumeStorageSynchronizationKey(final Storage.StorageKey storageKey, final String sourceStorageIP, final String sourceVolume, final String targetStorageIP, final String targetVolume) {
            this.storageKey = storageKey;
            this.sourceStorageIP = sourceStorageIP;
            this.sourceVolume = sourceVolume;
            this.targetStorageIP = targetStorageIP;
            this.targetVolume = targetVolume;
        }

        public final int compareTo(final RemoteInternalVolumeStorageSynchronizationKey o) {
            int i;

            // Must be the same volume
            i = storageKey.compareTo(o.getStorageKey());
            if (i != 0)
                return i;

            i = getSourceVolume().compareTo(o.getSourceVolume());
            if (i != 0)
                return i;

            i = getTargetVolume().compareTo(o.getTargetVolume());
            if (i != 0)
                return i;

            i = getSourceStorageIP().compareTo(o.getSourceStorageIP());
            if (i != 0)
                return i;

            return getTargetStorageIP().compareTo(o.getTargetStorageIP());
        }

        public final boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final RemoteInternalVolumeStorageSynchronizationKey that = (RemoteInternalVolumeStorageSynchronizationKey) o;

            if (sourceStorageIP != null ? !sourceStorageIP.equals(that.sourceStorageIP) : that.sourceStorageIP != null)
                return false;
            if (sourceVolume != null ? !sourceVolume.equals(that.sourceVolume) : that.sourceVolume != null) return false;
            if (storageKey != null ? !storageKey.equals(that.storageKey) : that.storageKey != null) return false;
            if (targetStorageIP != null ? !targetStorageIP.equals(that.targetStorageIP) : that.targetStorageIP != null)
                return false;
            if (targetVolume != null ? !targetVolume.equals(that.targetVolume) : that.targetVolume != null) return false;

            return true;
        }

        public final int hashCode() {
            int result;
            result = (storageKey != null ? storageKey.hashCode() : 0);
            result = 29 * result + (sourceStorageIP != null ? sourceStorageIP.hashCode() : 0);
            result = 29 * result + (sourceVolume != null ? sourceVolume.hashCode() : 0);
            result = 29 * result + (targetStorageIP != null ? targetStorageIP.hashCode() : 0);
            result = 29 * result + (targetVolume != null ? targetVolume.hashCode() : 0);
            return result;
        }

        public final String toString() {
            return storageKey.toString() + ":" + sourceStorageIP + ":" + sourceVolume + ":" + targetStorageIP + ":" + targetVolume;
        }

        public final Storage.StorageKey getStorageKey() {
            return storageKey;
        }

        public final String getSourceVolume() {
            return sourceVolume;
        }

        public final String getTargetVolume() {
            return targetVolume;
        }

        public final String getSourceStorageIP() {
            return sourceStorageIP;
        }

        public final String getTargetStorageIP() {
            return targetStorageIP;
        }
    }
}
