package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.io.Writer;

public class RemoteQtreeStorageSynchronization extends QtreeStorageSynchronization {

 private final String sourceStorageIP;
    private final String targetStorageIP;

    private final RemoteQtreeStorageSynchronizationKey key;


    RemoteQtreeStorageSynchronization(final Storage parent,
                                      final String sourceStorageIP,
                                      final String sourceQtreeId,
                                      final String targetStorageIP,
                                      final String targetQtreeId) throws SchemaException {
        super(parent, sourceQtreeId, targetQtreeId);

        if (sourceStorageIP == null || sourceStorageIP.length() == 0 || targetStorageIP == null || targetStorageIP.length() == 0)
            throw new SchemaException(getNodeName() + " Type requires non-null SourceStorageIP and TargetStorageIP");

        this.sourceStorageIP = sourceStorageIP;
        this.targetStorageIP = targetStorageIP;

        key = new RemoteQtreeStorageSynchronizationKey(parent.getKey(), sourceStorageIP, sourceQtreeId, targetStorageIP, targetQtreeId);
    }

    @SuppressWarnings({"unchecked"})
    public final RemoteQtreeStorageSynchronizationKey getKey() {
        return key;
    }

    public final String getSourceStorageIP() {
        return sourceStorageIP;
    }

    public final String getTargetStorageIP() {
        return targetStorageIP;
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException {
        super.marshallAttributes(cache, writer);

        addAttribute(writer, "SourceStorageIp", sourceStorageIP);
        addAttribute(writer, "TargetStorageIp", targetStorageIP);
    }

    public static final class RemoteQtreeStorageSynchronizationKey implements Key<RemoteQtreeStorageSynchronizationKey> {
        private final Storage.StorageKey storageKey;
        private final String sourceStorageIP;
        private final String sourceQtreeId;
        private final String targetStorageIP;
        private final String targetQtreeid;

        public RemoteQtreeStorageSynchronizationKey(final Storage.StorageKey storageKey, final String sourceStorageIP, final String sourceQtreeId, final String targetStorageIP, final String targetQtreeid) {
            this.storageKey = storageKey;
            this.sourceStorageIP = sourceStorageIP;
            this.sourceQtreeId = sourceQtreeId;
            this.targetStorageIP = targetStorageIP;
            this.targetQtreeid = targetQtreeid;
        }

        public final int compareTo(final RemoteQtreeStorageSynchronizationKey o) {
            int i;

            // Must be the same qtree
            i = storageKey.compareTo(o.getStorageKey());
            if (i != 0)
                return i;

            i = getSourceQtreeId().compareTo(o.getSourceQtreeId());
            if (i != 0)
                return i;

            i = getTargetQtreeid().compareTo(o.getTargetQtreeid());
            if (i != 0)
                return i;

            i = getSourceStorageIP().compareTo(o.getSourceStorageIP());
            if (i != 0)
                return i;

            return getTargetStorageIP().compareTo(o.getTargetStorageIP());
        }

        public final boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final RemoteQtreeStorageSynchronizationKey that = (RemoteQtreeStorageSynchronizationKey) o;

            if (sourceStorageIP != null ? !sourceStorageIP.equals(that.sourceStorageIP) : that.sourceStorageIP != null)
                return false;
            if (sourceQtreeId != null ? !sourceQtreeId.equals(that.sourceQtreeId) : that.sourceQtreeId != null) return false;
            if (storageKey != null ? !storageKey.equals(that.storageKey) : that.storageKey != null) return false;
            if (targetStorageIP != null ? !targetStorageIP.equals(that.targetStorageIP) : that.targetStorageIP != null)
                return false;
            if (targetQtreeid != null ? !targetQtreeid.equals(that.targetQtreeid) : that.targetQtreeid != null) return false;

            return true;
        }

        public final int hashCode() {
            int result;
            result = (storageKey != null ? storageKey.hashCode() : 0);
            result = 29 * result + (sourceStorageIP != null ? sourceStorageIP.hashCode() : 0);
            result = 29 * result + (sourceQtreeId != null ? sourceQtreeId.hashCode() : 0);
            result = 29 * result + (targetStorageIP != null ? targetStorageIP.hashCode() : 0);
            result = 29 * result + (targetQtreeid != null ? targetQtreeid.hashCode() : 0);
            return result;
        }

        public final String toString() {
            return storageKey.toString() + ":" + sourceStorageIP + ":" + sourceQtreeId + ":" + targetStorageIP + ":" + targetQtreeid;
        }

        public final Storage.StorageKey getStorageKey() {
            return storageKey;
        }

        public final String getSourceQtreeId() {
            return sourceQtreeId;
        }

        public final String getTargetQtreeid() {
            return targetQtreeid;
        }

        public final String getSourceStorageIP() {
            return sourceStorageIP;
        }

        public final String getTargetStorageIP() {
            return targetStorageIP;
        }
    }

}
