package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.BaseRootFactory;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

/**
 * This is the factory for all Root classes.
 */
public class RootFactory extends BaseRootFactory {
    public RootFactory(final String dataSourceName) {
        super(dataSourceName);
    }

    public FabricRoot createFabricRoot(final long originatorID, final String originatorType) throws SchemaException {
        if (roots.contains(originatorID))
            throw new SchemaException("Root with originator ID " + originatorID + " already created");

        return new FabricRootImpl(this, originatorID, originatorType, getDataSourceName());
    }

    public StorageRoot createStorageRoot(final long originatorID, final String originatorType) throws SchemaException {
        if (roots.contains(originatorID))
            throw new SchemaException("Root with originator ID " + originatorID + " already created");

        return new StorageRootImpl(this, originatorID, originatorType, getDataSourceName());
    }

    public SRMRoot createSRMRoot(final long originatorID, final String originatorType) throws SchemaException {
        if (roots.contains(originatorID))
            throw new SchemaException("Root with originator ID " + originatorID + " already created");

        return new SRMRootImpl(this, originatorID, originatorType, getDataSourceName());
    }

    public HostDataRoot createHostDataRoot(final long originatorID, final String originatorType) throws SchemaException {
        if (roots.contains(originatorID))
            throw new SchemaException("\"Root with originator ID \" + originatorID + \" already created\"");

        return new HostDataRootImpl(this, originatorID, originatorType, getDataSourceName());

    }

    public VirtualizationRoot createVirtualizationRoot(final long originatorID, final String originatorType) throws SchemaException {
        if (roots.contains(originatorID))
            throw new SchemaException("Root with originator ID " + originatorID + " already created");

        return new VirtualizationRootImpl(this, originatorID, originatorType, getDataSourceName());
    }

}
