package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;

import java.io.IOException;
import java.io.Writer;

public class ShareInitiator extends XMLNode<Share> {

    final String initiator;
    final String permission;
    final ShareInitiatorKey initiatorKey;

    protected ShareInitiator(final Share parent, final String initiator, final String permission) throws SchemaException {
        super(parent, "ShareInitiator");
        if (initiator == null)
            throw new SchemaException(getNodeName() + " Type requires non-null initiator.");

        this.initiator = initiator;
        this.permission = permission;
        initiatorKey = new ShareInitiatorKey(initiator);
    }

    public Key getKey() {
        return initiatorKey;
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException {
        addAttribute(writer, "Initiator", initiator);
        addAttribute(writer, "Permission", permission);
    }

    public static final class ShareInitiatorKey implements Key<ShareInitiatorKey> {
        final String initiator;

        public ShareInitiatorKey(final String initiator) throws SchemaException {
            if (initiator == null)
                throw new SchemaException(" Type requires non-null initiator.");
            this.initiator = initiator;
        }

        public String getInitiator() {
            return initiator;
        }

        public int compareTo(final ShareInitiatorKey o) {
            return getInitiator().compareTo(o.getInitiator());
        }

    }
}
