package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.NodeMap;

public class StorageEfficiencies extends NodeMap<Storage, StorageEfficiency.StorageEfficiencyKey, StorageEfficiency> {
    StorageEfficiencies(final Storage parent) {
        super(parent);
    }

    /**
     * Add an efficiency technology to the collection.
     *
     * <p/>Note that this will <u>override</u> any previously existing value
     *
     * @param efficiency Efficiency to add
     */
    public void add(final StorageEfficiency efficiency) {
        put(efficiency.getKey(), efficiency);
    }

    /**
     * Given an efficiency object, this method goes and looks up if there is already an existing object with same
     * key. If so, it will go and add all efficiency values to the original (sum).
     *
     * <p/>Used when we need to 'merge' results from different devices (like nodes on a NetApp cluster)
     *
     * @param efficiency New object to update
     */
    public void update(final StorageEfficiency efficiency) {
        final StorageEfficiency existing = get(efficiency.getKey());
        if (existing == null) {
            // This technology didn't exist before. Add it now
            add(efficiency);
        }
        else {
            // Just upgrade the existing technology values.
            existing.getGainLossMB().add(efficiency.getGainLossMB());
            existing.getPotentialGainLossMB().add(efficiency.getPotentialGainLossMB());
        }
    }
}
