package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.io.Writer;

public class VSAN extends Fabric {
    private Boolean vsanEnabled = Boolean.FALSE;
    private Integer vsanID = null;

    private final LogicalSwitches logicalSwitches;

    public VSAN(final FabricRoot parent, final String wwn) throws SchemaException {
        super(parent, wwn);

        logicalSwitches = new LogicalSwitches(this);
    }

    public LogicalSwitches getLogicalSwitches() {
        return logicalSwitches;
    }

    public LogicalSwitch createLogicalSwitch(final String wwn,
                                             final String chassisWWN,
                                             final int domainID) throws SchemaException {
        return new LogicalSwitch(this, wwn, chassisWWN, domainID, SwitchType.Type.FABRIC_LOGICAL);
    }

    public LogicalSwitch createNpvLogicalSwitch(final String wwn,
                                                final String chassisWWN,
                                                final int domainID) throws SchemaException {
        return new LogicalSwitch(this, wwn, chassisWWN, domainID, SwitchType.Type.NPV_LOGICAL);
    }

    public int getVsanID() {
        return vsanID;
    }

    public void setVsanID(final int vsanID) {
        this.vsanID = vsanID;
    }

    public Boolean isVsanEnabled() {
        return vsanEnabled;
    }

    public void setVsanEnabled(final boolean vsanEnabled) {
        this.vsanEnabled = vsanEnabled;
    }


    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException {
        super.marshallAttributes(cache, writer);

        if (vsanEnabled) {
            addAttribute(writer, "VSANEnabled", "1");
        }

        addAttribute(writer, "VSANId", vsanID);
    }

    protected boolean hasChildren() {
        if (super.hasChildren())
            return true;

        return !logicalSwitches.isEmpty();
    }

    protected void marshallChildren(final MarshallingCache cache, final Writer writer, final int tabs) throws IOException, SchemaException {
        addAllChildren(cache, writer, logicalSwitches, tabs);

        super.marshallChildren(cache, writer, tabs);
    }
}
