package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;

import java.io.IOException;
import java.io.Writer;

/**
 * Root level element representing the association between a virtual machine and the virtual disks it is using
 */
public class VirtualMachineToDisk extends XMLNode<VirtualizationRoot> {
    private final VirtualMachine.VirtualMachineKey virtualMachineKey;
    private final VirtualDisk.VirtualDiskKey virtualDiskKey;

    private final VirtualMachineToDiskKey key;

    VirtualMachineToDisk(final VirtualizationRoot parent,
                         final VirtualMachine.VirtualMachineKey virtualMachineKey,
                         final VirtualDisk.VirtualDiskKey virtualDiskKey) {
        super(parent, "HV_VirtualMachineToDisk");

        this.virtualMachineKey = virtualMachineKey;
        this.virtualDiskKey = virtualDiskKey;

        key = new VirtualMachineToDiskKey(virtualMachineKey, virtualDiskKey);
    }

    public VirtualMachineToDiskKey getKey() {
        return key;
    }

    public VirtualMachine.VirtualMachineKey getVirtualMachineKey() {
        return virtualMachineKey;
    }

    public VirtualDisk.VirtualDiskKey getVirtualDiskKey() {
        return virtualDiskKey;
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException, SchemaException {
        super.marshallAttributes(cache, writer);

        addAttribute(writer, "OID", getKey().toString());

        addAttribute(writer, "VirtualMachine_OID", virtualMachineKey.toString());
        addAttribute(writer, "VirtualDisk_OID", virtualDiskKey.toString());
    }

    public final static class VirtualMachineToDiskKey implements Key<VirtualMachineToDiskKey> {
        private final VirtualMachine.VirtualMachineKey virtualMachineKey;
        private final VirtualDisk.VirtualDiskKey virtualDiskKey;

        public VirtualMachineToDiskKey(final VirtualMachine.VirtualMachineKey virtualMachineKey, final VirtualDisk.VirtualDiskKey virtualDiskKey) {
            this.virtualMachineKey = virtualMachineKey;
            this.virtualDiskKey = virtualDiskKey;
        }

        public int compareTo(final VirtualMachineToDiskKey o) {
            int i = getVirtualMachineKey().compareTo(o.getVirtualMachineKey());
            if (i != 0)
                return i;

            return getVirtualDiskKey().compareTo(o.getVirtualDiskKey());
        }

        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final VirtualMachineToDiskKey that = (VirtualMachineToDiskKey) o;

            if (virtualDiskKey != null ? !virtualDiskKey.equals(that.virtualDiskKey) : that.virtualDiskKey != null)
                return false;
            if (virtualMachineKey != null ? !virtualMachineKey.equals(that.virtualMachineKey) : that.virtualMachineKey != null)
                return false;

            return true;
        }

        public int hashCode() {
            int result;
            result = (virtualMachineKey != null ? virtualMachineKey.hashCode() : 0);
            result = 29 * result + (virtualDiskKey != null ? virtualDiskKey.hashCode() : 0);
            return result;
        }

        public VirtualMachine.VirtualMachineKey getVirtualMachineKey() {
            return virtualMachineKey;
        }

        public VirtualDisk.VirtualDiskKey getVirtualDiskKey() {
            return virtualDiskKey;
        }

        public String toString() {
            return virtualMachineKey.toString() + ":" + virtualDiskKey.toString();
        }
    }
}
