package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.SchemaException;

/**
 * Root of virtualization ODATA used for virtualization technologies (e.g. VMWare, PowerVM, etc.).
 */
public interface VirtualizationRoot extends OData2Root {

    public VirtualServers getVirtualServers();

    public VirtualServer createVirtualServer(final String virtualCenterIP, final Host.HostKey host) throws SchemaException;

    /**
     * Get all defined virtual machine instances.
     *
     * @return  virtual machines map.
     */
    public VirtualMachines getVirtualMachines();

    /**
     * Create a nwe virtual machine instance.   This is NOT automatically stored on the internal list of VMs - it must
     * be added manually.
     *
     * @param moid  object ID.  An internal reference of the VM, unique in the scope of the management IP.
     * @param managementIp the IP of the management station for this VM, e.g. a virtual center or HMC instance, etc.
     * @return the newly created VM.
     * @throws SchemaException for an internal model building error.
     */
    public VirtualMachine createVirtualMachine(final String moid, final String managementIp) throws SchemaException;

    public DataStores getDataStores();

    public DataStore createVirtualDataStore(final String oid, final String virtualCenterIP) throws SchemaException;

    public VmLuns getVirtualLuns();

    public VmLun createVmLun(final String oid, final String virtualCenterIP) throws SchemaException;

    public VirtualDisks getVirtualDisks();

    public VirtualDisk createVirtualDisk(final String moid, final String virtualCenterIP) throws SchemaException;

    public VirtualMachineToDisks getVirtualMachineToDisks();

    public VirtualMachineToDisk createVirtualMachineToDisk(final VirtualMachine.VirtualMachineKey vmkey, VirtualDisk.VirtualDiskKey vdiskKey) throws SchemaException;

    public VirtualPaths getVirtualPaths();

    public VirtualPath createVirtualPath(final String oid, final VmLun.VMLunKey lunOid, final String virtualCenterIP) throws SchemaException;

    /**
     * Get "hosts" defined on this virtualization root.  A host is really a Hypervisor for the purposes of
     * virtualization.
     *
     * @return the list of hosts (as a map).
     */
    public Hosts getHosts();

    /**
     * Create a new host (hypervisor). This is NOT automatically stored on the internal list of hosts - it must
     * be added manually.
     *
     * @param ips comma separated list (ordered) of IP addresses for this host.  Used to uniquely identify the host
     *  in server and odata.
     * @return the newly created host.
     * @throws SchemaException for an internal error building the schema.
     */
    public Host createHost(final String ips) throws SchemaException;

    public NasShare createNasShare(final String virtualCenetrIp, final String filerIp, final String shareIP, final NasShare.Type type) throws SchemaException;

    public NasShares getNasShares();

    public HostToNasShare createHostToNasShare(final Host.HostKey hostKey, final NasShare.NasShareKey nasShareKey) throws SchemaException;

    public HostToNasShares getHostToNasShares();

    public DataStoreToNasShare createDataStoreToNasShare(final DataStore.DataStoreKey dataStoreKey, final NasShare.NasShareKey nasShareKey) throws SchemaException;

    public DataStoreToNasShares getDataStoreToNasShares();
}
