package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;

import java.io.IOException;
import java.io.Writer;

/**
 * Represents the properties of a LUN which is accessed by the same host and is part of a datastore.
 */
public class VmLun extends XMLNode<VirtualizationRoot> {
    private final String moid;
    private final String managementIp;

    private DataStore.DataStoreKey dataStoreKey;
    private Host.HostKey hostKey;
    private int number;
    private String policy;
    private String diskName;
    private String tid;

    private final VMLunKey key;

    VmLun(final VirtualizationRoot parent, final String moid, final String managementIp) throws SchemaException {
        super(parent, "HV_Lun");

        if (moid == null || moid.length() == 0)
            throw new SchemaException("moid is mandatory!");

        if (managementIp == null || managementIp.length() == 0)
            throw new SchemaException("virtualCenterIP is mandatory!");

        this.moid = moid;
        this.managementIp = managementIp;
        this.key=new VMLunKey(moid, managementIp);
    }

    public VMLunKey getKey() {
        return key;
    }

    public String getMOID() {
        return moid;
    }

    public String getManagementIp() {
        return managementIp;
    }

    public DataStore.DataStoreKey getDataStore() {
        return dataStoreKey;
    }

    public void setDataStore(final DataStore.DataStoreKey dataStoreOid) {
        this.dataStoreKey = dataStoreOid;
    }

    public Host.HostKey getHost() {
        return hostKey;
    }

    public void setHost(final Host.HostKey hostKey) {
        this.hostKey = hostKey;
    }

    public int getNumber() {
        return number;
    }

    public void setNumber(final int number) {
        this.number = number;
    }

    public String getPolicy() {
        return policy;
    }

    public void setPolicy(final String policy) {
        this.policy = policy;
    }

    public String getDiskName() {
        return diskName;
    }

    public void setDiskName(final String diskName) {
        this.diskName = diskName;
    }

    public String getTID() {
        return tid;
    }

    public void setTID(final String tid) {
        this.tid = tid;
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException, SchemaException {
        super.marshallAttributes(cache, writer);

        addAttribute(writer, "OID", getKey().toString());

        addAttribute(writer, "Number", number);
        addAttribute(writer, "Policy", policy);
        addAttribute(writer, "HV_DataStore_OID", dataStoreKey);
        addAttribute(writer, "Host_OID", hostKey);
        addAttribute(writer, "DiskName", diskName);
        addAttribute(writer, "TID", tid);
    }

    public static class VMLunKey implements Key<VMLunKey> {
        final String moid;
        final String managementIp;

        public VMLunKey(String moid, String managementIp) {
            this.moid = moid;
            this.managementIp = managementIp;
        }

        public int compareTo(final VMLunKey o) {
            int i = moid.compareTo(o.getMOID());
            if (i != 0)
                return i;

            return managementIp.compareTo(o.getManagementIp());
        }

        public boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final VMLunKey vmLunKey = (VMLunKey) o;

            if (moid != null ? !moid.equals(vmLunKey.moid) : vmLunKey.moid != null) return false;
            if (managementIp != null ? !managementIp.equals(vmLunKey.managementIp) : vmLunKey.managementIp != null)
                return false;

            return true;
        }

        public int hashCode() {
            int result;
            result = (moid != null ? moid.hashCode() : 0);
            result = 31 * result + (managementIp != null ? managementIp.hashCode() : 0);
            return result;
        }

        private String getManagementIp() {
            return managementIp;
        }

        public String getMOID() {
            return moid;
        }

        public String toString() {
            return getMOID() + ":" + getManagementIp();
        }
    }
}
