package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.util.Capacity;

import java.io.IOException;
import java.io.Writer;

public class Volume extends BaseVolume<Storage> {

    protected final Maps maps;
    protected final Masks masks;
    protected final VolumeCapacity capacity;

    Volume(final Storage parent, final String name, final StoragePool storagePool, final Qtree qtree, final boolean isDrPackageInstalled) throws SchemaException {
        super(parent, parent, "Volume", name, storagePool, qtree, isDrPackageInstalled);

        maps = new Maps(this);
        masks = new Masks(this);

        capacity = new VolumeCapacity(this);
    }

    /**
     * Only used by Symmetrix to allow it to create a volume
     * without a storage pool, setting it later.
     * @param parent
     * @param name
     * @param isDrPackageInstalled
     * @throws SchemaException
     */
    @Deprecated
    public Volume(final Storage parent, final String name, final boolean isDrPackageInstalled) throws SchemaException {
        super(parent, parent, "Volume", name, null, null, isDrPackageInstalled);

        maps = new Maps(this);
        masks = new Masks(this);

        capacity = new VolumeCapacity(this);

    }

    public VolumeMap createVolumeMap(final String storagePortWWN, final String protocolControler) throws SchemaException {
        return new VolumeMap(this, storagePortWWN, protocolControler);
    }

    public VolumeMask createVolumeMask(final String storagePortWWN, final String initiatorWWN, final String protocolController) throws SchemaException {
        return new VolumeMask(this, storagePortWWN, initiatorWWN, protocolController);
    }

    public Storage getStorage() {
        return getParent();
    }

    public Masks getMasks() {
        return masks;
    }

    public Maps getMaps() {
        return maps;
    }

    public VolumeCapacity getCapacity() {
        return capacity;
    }

    public InternalVolume getInternalVolume() {
        return getQtree().getParent();
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException, SchemaException {
        super.marshallAttributes(cache, writer);

        addAttribute(writer, "Type", type);
        addAttribute(writer, "DiskType", diskType);
        addAttribute(writer, "DiskGroup", diskGroup);
        addAttribute(writer, "Virtual", virtual);
        addAttribute(writer, "ThinProvisioned", thinProvisioned);
        // Disk speed is a comma seperated list of sizes
        addAttribute(writer, "DiskSpeed", convertToCSV(diskSpeeds));
        // Disk size is a comma seperated list of sizes
        addAttribute(writer, "DiskSize", convertToCSV(diskSizes));

        if (isDrPackageInstalled()) {
            addAttribute(writer, "ReplicaSource", replicaSource);
            addAttribute(writer, "ReplicaTarget", replicaTarget);
            addAttribute(writer, "Snapshot", snapshot);
        }

        addAttribute(writer, "LastKnownAccessTime", lastKnownAccessTime);

        addAttribute(writer, "CapacityMB", getCapacity().getCapacity(), Capacity.Unit.MB_1024);
        //provide raw capacity using the storage pool ratio
        addAttribute(writer, "RawCapacityMB", getCapacity().getRawCapacity(), Capacity.Unit.MB_1024);
        addAttribute(writer, "UsedCapacityMB", getCapacity().getUsedCapacity(), Capacity.Unit.MB_1024);
        addAttribute(writer, "Redundancy", getCapacity().getRedundancy());
    }

    protected void marshallChildren(final MarshallingCache cache, final Writer writer, final int tabs) throws IOException, SchemaException {

        addAllChildrenUnderNode(cache, writer, "Maps", maps, tabs);

        addAllChildrenUnderNode(cache, writer, "Masks", masks, tabs);

    }

    protected boolean hasChildren() {
        return (maps.size() > 0) || (masks.size() > 0) || super.hasChildren();
    }

    public static enum MapMaskType {
        ISCSI,
        FC,
    }

}
