package com.onaro.sanscreen.acquisition.schema.odata2;

import com.onaro.sanscreen.acquisition.schema.Key;
import com.onaro.sanscreen.acquisition.schema.MarshallingCache;
import com.onaro.sanscreen.acquisition.schema.SchemaException;
import com.onaro.sanscreen.acquisition.schema.XMLNode;

import java.io.IOException;
import java.io.Writer;

public class VolumeMask extends XMLNode<Volume> {
    private final String initiatorWWN;
    private final String storagePortWWN;
    private final String protocolController;
    private Volume.MapMaskType type = null;

    private final VolumeMaskKey key;

    /**
     * Use {@link Volume#createVolumeMask} to create this object
     *
     * @param initiatorWWN
     * @param storagePortWWN The storage port WWN through which this volume is mapped (if null, it means "map through all ports!")
     * @param protocolController The protocol controller associated with this mapping (if null, no protocol controllers are supported in this storage)
     */
    VolumeMask(final Volume parent, final String storagePortWWN, final String initiatorWWN, final String protocolController) throws SchemaException {
        super(parent, "VolumeMask");

        if (initiatorWWN == null || initiatorWWN.length() == 0)
            throw new SchemaException(getNodeName() + " Type requires non-null OID and InitiatorWWN");

        this.storagePortWWN = storagePortWWN;
        this.initiatorWWN = initiatorWWN;
        this.protocolController = protocolController;

        key = new VolumeMaskKey(parent.getKey(), storagePortWWN, initiatorWWN, protocolController);
    }

    public final VolumeMaskKey getKey() {
        return key;
    }

    public final Volume getVolume() {
        return getParent();
    }

    public final String getStoragePortWWN() {
        return storagePortWWN;
    }

    public final String getInitiatorWWN() {
        return initiatorWWN;
    }

    public final String getProtocolController() {
        return protocolController;
    }


    public Volume.MapMaskType getType() {
        return type;
    }

    public void setType(final Volume.MapMaskType type) {
        this.type = type;
    }

    protected void marshallAttributes(final MarshallingCache cache, final Writer writer) throws IOException {
        addAttribute(writer, "StoragePort", storagePortWWN);
        addAttribute(writer, "Initiator", initiatorWWN);
        addAttribute(writer, "ProtocolController", getProtocolController());
        addAttribute(writer, "Type", type);

    }

    public static final class VolumeMaskKey implements Key<VolumeMaskKey> {
        private final BaseVolume.VolumeKey volumeKey;
        private final String storagePortWWN;
        private final String initiatorWWN;
        private final String protocolController;

        public VolumeMaskKey(final BaseVolume.VolumeKey volumeKey, final String storagePortWWN, final String initiatorWWN, final String protocolController) {
            this.volumeKey = volumeKey;
            this.storagePortWWN = storagePortWWN;
            this.initiatorWWN = initiatorWWN;
            this.protocolController = protocolController;
        }

        public final int compareTo(final VolumeMaskKey o) {
            int i;

            // Must be the same volume
            i = volumeKey.compareTo(o.getVolumeKey());
            if (i != 0)
                return i;

            i = compareObjects(getStoragePortWWN(), o.getStoragePortWWN());
            if (i != 0)
                return i;

            i = compareObjects(getProtocolController(), o.getProtocolController());
            if (i != 0)
                return i;

            return compareObjects(getInitiatorWWN(), o.getInitiatorWWN());
        }

        public final boolean equals(final Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            final VolumeMaskKey that = (VolumeMaskKey) o;

            if (initiatorWWN != null ? !initiatorWWN.equals(that.initiatorWWN) : that.initiatorWWN != null)
                return false;
            if (protocolController != null ? !protocolController.equals(that.protocolController) : that.protocolController != null)
                return false;
            if (storagePortWWN != null ? !storagePortWWN.equals(that.storagePortWWN) : that.storagePortWWN != null)
                return false;
            if (volumeKey != null ? !volumeKey.equals(that.volumeKey) : that.volumeKey != null) return false;

            return true;
        }

        public final int hashCode() {
            int result;
            result = (volumeKey != null ? volumeKey.hashCode() : 0);
            result = 29 * result + (storagePortWWN != null ? storagePortWWN.hashCode() : 0);
            result = 29 * result + (initiatorWWN != null ? initiatorWWN.hashCode() : 0);
            result = 29 * result + (protocolController != null ? protocolController.hashCode() : 0);
            return result;
        }

        public final String toString() {
            return volumeKey.toString() + ":" + storagePortWWN + ":" + initiatorWWN + ":" + protocolController;
        }

        public final Volume.VolumeKey getVolumeKey() {
            return volumeKey;
        }

        public final String getProtocolController() {
            return protocolController;
        }

        public final String getStoragePortWWN() {
            return storagePortWWN;
        }

        public final String getInitiatorWWN() {
            return initiatorWWN;
        }
    }

}
