package com.onaro.sanscreen.acquisition.schema2;

import com.onaro.sanscreen.acquisition.schema.SchemaException;

import java.io.IOException;
import java.io.Writer;

/**
 * Defines the basic functionality of a odata root object.
 * <p/>
 * <p/>Each acquisition report is sent to the SANscreen server in XML format. On
 * the data source package implementation side, building this report is done by
 * utilizing the odata schema library (<code>com.onaro.sanscreen.acquisition.schema.odata2.*</code>).
 * <p/>
 * <p/>To start building an <code>odata</code> object tree, a
 * <code>RootNode</code> object has to be created. Using the {@link
 * com.onaro.sanscreen.acquisition.schema.BaseRootFactory} factory class, root
 * of different kinds can be created, depending on the type of report the data
 * source will generate. There is a specific root object for every type of
 * report that can be generated by the package <code>BaseStorageRoot</code> for
 * example is used to report about a storage array to the server).
 * <p/>
 * <p/>A package implementation can choose to report more than one roots in a
 * single sample (this is achieved by construction a "Multiple Originator ID
 * (MOID)" package).
 */
public interface RootNode extends Node {

    /**
     * Used by the framework to convert the schema object hierarchy into XML.
     * This is done when sending the report to the server.
     *
     * @param writer Writer to dump XML to
     * @throws SchemaException If an XML integrity error occurs
     * @throws java.io.IOException If an IO error occurs
     */
    public void marshall(final Writer writer)
            throws IOException, SchemaException;
}
