package com.onaro.sanscreen.acquisition.schema2.collect;

import com.onaro.sanscreen.acquisition.schema2.XmlChildNode;
import com.onaro.sanscreen.acquisition.schema2.XmlNode;
import com.onaro.util.collect.IndexedSet;

/**
 * Set of XML nodes indexed by key. Ensures that all nodes in this set have
 * the same (fixed) parent.
 */
public abstract class NodeSet<N extends XmlChildNode<P>, P extends XmlNode>
        extends IndexedSet<N> {

    private final P parent;

    public NodeSet(final P parent) {
        this.parent = parent;
    }

    public P getParent() {
        return parent;
    }

    @Override
    public boolean add(final N node) {
        if (node.getParent() != parent) {
            throw new IllegalArgumentException(
                    "Node parent does not match map parent");
        }
        return super.add(node);
    }

}
