package com.onaro.sanscreen.acquisition.schema2.collect;

import com.onaro.sanscreen.acquisition.schema2.XmlChildNode;
import com.onaro.sanscreen.acquisition.schema2.XmlNode;
import com.onaro.util.collect.Tuple;

import java.io.Serializable;

/**
 * Set of XML nodes indexed by a single key of type <tt>T1</tt>.
 */
public class NodeSet1<N extends XmlChildNode<P> & Keyable1<T1>,
        P extends XmlNode,
        T1 extends Comparable<? super T1> & Serializable>
        extends NodeSet<N, P> {

    public NodeSet1(final P parent) {
        super(parent);
    }

    /**
     * @return The node with the specified key.
     */
    public N get(final T1 key1) {
        return map.get(createTuple(key1));
    }

    /**
     * Return <tt>true</tt> if this set contains the node with the specified
     * key, <tt>false</tt> otherwise.
     */
    public boolean contains(final T1 key1) {
        return map.containsKey(createTuple(key1));
    }

    /** Remove the node with the specified key from this set. */
    public N remove(final T1 key1) {
        return map.remove(createTuple(key1));
    }

    @Override
    protected Tuple createKey(final N node) {
        return createTuple(node.getKey1());
    }

    /** Create a tuple representing the specified key. */
    private Tuple createTuple(final T1 key1) {
        return new Tuple(key1);
    }

}
