package com.onaro.sanscreen.acquisition.schema2.collect;

import com.onaro.sanscreen.acquisition.schema2.XmlChildNode;
import com.onaro.sanscreen.acquisition.schema2.XmlNode;
import com.onaro.util.collect.Tuple;

import java.io.Serializable;

/**
 * Set of XML child nodes indexed by two keys of type <tt>T1</tt> and
 * <tt>T2</tt>.
 */
public class NodeSet2<N extends XmlChildNode<P> & Keyable2<T1, T2>,
        P extends XmlNode,
        T1 extends Comparable<? super T1> & Serializable,
        T2 extends Comparable<? super T2> & Serializable>
        extends NodeSet<N, P> {

    public NodeSet2(final P parent) {
        super(parent);
    }

    /**
     * @return The node with the specified keys.
     */
    public N get(final T1 key1, final T2 key2) {
        return map.get(createTuple(key1, key2));
    }

    /**
     * Return <tt>true</tt> if this set contains a node with the specified key,
     * <tt>false</tt> otherwise.
     */
    public boolean contains(final T1 key1, final T2 key2) {
        return map.containsKey(createTuple(key1, key2));
    }

    /** Remove the node with the specified keys from this set. */
    public N remove(final T1 key1, final T2 key2) {
        return map.remove(createTuple(key1, key2));
    }

    @Override
    protected Tuple createKey(final N node) {
        return createTuple(node.getKey1(), node.getKey2());
    }

    private Tuple createTuple(final T1 key1, final T2 key2) {
        return new Tuple(key1, key2);
    }

}
