/*
 * Copyright (c) 2010 NetApp
 * All rights reserved
 */

package com.onaro.sanscreen.acquisition.sessions;

import com.netapp.oci.platform.common.ServerConstant;
import com.netapp.oci.platform.common.table.ConnectionHelper;
import com.netapp.oci.platform.common.table.TableException;
import com.onaro.sanscreen.server.dataobject.acquisition.AcquisitionUnitAudit;
import com.onaro.sanscreen.server.tables.acquisition.AcquisitionUnitAuditTable;
import org.jboss.ejb3.annotation.RemoteBinding;
import org.jboss.ejb3.annotation.RemoteBindings;

import javax.annotation.Resource;
import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import java.sql.Connection;
import java.util.Collection;

/**
 * @author Asaf Yigal
 * @version 1.0
 */
@Stateless
@RemoteBindings({
        @RemoteBinding(jndiBinding = AcquisitionUnitAuditSession.JNDI_NAME),
        @RemoteBinding(jndiBinding = ServerConstant.HTTPS_JNDI_BINDING_PREFIX + AcquisitionUnitAuditSession.JNDI_NAME)})
@TransactionAttribute(TransactionAttributeType.REQUIRED)
@Resource(name = "jdbc/OnaroDataSource", mappedName = "java:/onaro_pool", type = javax.sql.DataSource.class, authenticationType = Resource.AuthenticationType.CONTAINER)
@RolesAllowed({"guest", "scheduler", "acquisition", "admin"})
public class AcquisitionUnitAuditSessionEJB implements AcquisitionUnitAuditSession {


    public void removeAllAudit(String auditName) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnectionNoEjb();
            AcquisitionUnitAuditTable auditTable = new AcquisitionUnitAuditTable(connection);
            auditTable.deleteByName(auditName);
        } catch (TableException e) {
            throw new AcquisitionUnitException("Failed to remove", e);
        } finally {
            ConnectionHelper.close(connection);
        }
    }


    public Collection<AcquisitionUnitAudit> getAcquisitionUnitStatus(final String auditName) throws AcquisitionUnitException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnectionNoEjb();
            AcquisitionUnitAuditTable auditTable = new AcquisitionUnitAuditTable(connection);
            return auditTable.selectByName(auditName);
        } catch (TableException e) {
            throw new AcquisitionUnitException("Failed to get staus", e);
        } finally {
            ConnectionHelper.close(connection);
        }

    }
}
