/*
 * Copyright (c) 2010 NetApp
 * All rights reserved
 */

package com.onaro.sanscreen.acquisition.sessions;

import com.netapp.oci.platform.common.table.ConnectionHelper;
import org.jboss.ejb3.annotation.ResourceAdapter;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.StatefulJob;

import javax.annotation.Resource;
import javax.annotation.security.RunAs;
import javax.ejb.ActivationConfigProperty;
import javax.ejb.MessageDriven;
import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.SQLException;


/**
 * @author Glen Nuckolls
 */
@MessageDriven(
        activationConfig = {
                // scheduled every day at 1:15 am
                @ActivationConfigProperty(propertyName = "cronTrigger", propertyValue = "0 15 1 1/1 * ? *"),
                @ActivationConfigProperty(propertyName = "jobName", propertyValue = "DSAuditsPurgeOldEntries")})
@ResourceAdapter("quartz-ra.rar")
@RunAs("admin")
public class DSAuditsPurgeOldEntries implements StatefulJob {

    @Resource(name = "jdbc/OnaroDataSource",
            mappedName = "java:/onaro_pool",
            type = DataSource.class,
            authenticationType = Resource.AuthenticationType.CONTAINER)
    private DataSource dataSource;

    public void execute(final JobExecutionContext jobExecutionContext) throws JobExecutionException {
        Connection connection = null;
        try {
            connection = ConnectionHelper.newConnection(dataSource);

            try {

                DataSourceAuditManager.clearOldAudits(60, connection);

                // reset the audit cache
                final DataSourceAuditManager dsAuditManager = new DataSourceAuditManager();
                dsAuditManager.initialize();

            }
            catch (SQLException e) {
                throw new JobExecutionException(e);
            }

        }
        finally {
            ConnectionHelper.close(connection);
        }
    }
}
