package com.onaro.sanscreen.acquisition.util.common;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class ParseUtils {
    private final static long serialVersionUID = 1;

    /**
     * @param s         A string that represents a single item or list of items
     * @param delimiter The string that separates the items in <code>s</code>
     *
     * @return An array of String objects. May be of size zero. Never returns null;
     */
    public static String[] parseStringList(final String s, final String delimiter) {
        if (s != null) {
            final String[] split = s.split(delimiter);
            final List<String> ipList = new ArrayList<String>();
            for (final String ip : split) {
                ipList.add(ip.trim());
            }
            return ipList.toArray(new String[ipList.size()]);
        }
        else {
            return new String[0];
        }
    }

    /**
     * builds a comma separated list of toString() values of objects in the collection <code>c</code>
     *
     * @param c a collection of objects
     * @return a String. Never returns null
     */
    public static String toStringList(final Collection c){
        return toSeparatedString(c, ',');
    }

    /**
     * builds a space separated list of toString() values of objects in the collection <code>c</code>
     *
     * @param c a collection of objects
     *
     * @return a String. Never returns null
     */
    public static String toSpaceSeparatedString(final Collection c) {
        return toSeparatedString(c, ' ');
    }


    /**
     * builds a single string from the toString() value of the objects in the <code>collection</code>, separating each
     * value by the provided <code>separator</code>
     *
     * @param c a collection of objects
     *
     * @return a String. Never returns null
     */
    public static String toSeparatedString(final Collection c, final char separator) {
        final StringBuilder b = new StringBuilder();
        if (c != null) {
            for (final Object o : c) {
                b.append(o).append(separator);
            }
            if (b.length() > 0) {
                // remove the separator at the end
                b.deleteCharAt(b.length() - 1);
            }
        }
        return b.toString();
    }


}
