package com.onaro.sanscreen.acquisition.util.reader;

import org.apache.xerces.util.XMLChar;

import java.io.FilterReader;
import java.io.IOException;
import java.io.Reader;

/**
 * This filter makes sure that only XML valid chars are parsed
 */
public class BadXmlFilterReader extends FilterReader {
    /**
     * Constructor
     *
     * @param in The reader to use to read XML into the filter
     */
    public BadXmlFilterReader(final Reader in) {
        super(in);
    }

    /**
     * Reads a single character.  This method will block until a character is
     * available, an I/O error occurs, or the end of the stream is reached.
     *
     * <p> Subclasses that intend to support efficient single-character input
     * should override this method.
     *
     * @return The character read, as an integer in the range 0 to 65535
     *         (<tt>0x00-0xffff</tt>), or -1 if the end of the stream has
     *         been reached
     *
     * @throws IOException If an I/O error occurs
     */
    public int read() throws IOException {
        final int b = in.read();
        if (b == -1) {
            return b;
        }
        
        return XMLChar.isValid(b) ? b : '?';
    }

    /**
     * Reads characters into a portion of an array.  This method will block
     * until some input is available, an I/O error occurs, or the end of the
     * stream is reached.
     *
     * @param cbuf Destination buffer
     * @param off  Offset at which to start storing characters
     * @param len  Maximum number of characters to read
     *
     * @return The number of characters read, or -1 if the end of the
     *         stream has been reached
     *
     * @throws IOException If an I/O error occurs
     * @throws IndexOutOfBoundsException If read location is out of passed in array's bounds
     */
    public int read(final char[] cbuf, final int off, final int len) throws IOException, IndexOutOfBoundsException {
        //Make sure our passing in values are valid
        if (len <= 0) {
            if (len < 0) {
                throw new IndexOutOfBoundsException();
            }
            return 0;
        }

        //Check to make sure we wont run into array out of bounds exceptions during reading
        if (off + len > cbuf.length) {
            throw new IndexOutOfBoundsException();
        }

        int i;
        for (i = 0; i < len; i++) {
            final int b = read();

            if (b == -1) {
                if (i == 0) {
                    return -1;
                }
                else {
                    break;
                }
            }

            cbuf[off + i] = (char) b;
        }
        return i;
    }
}

