/**
 * Created by Onaro Inc.
 *
 * User: ophir.horn
 * Date: Nov 8, 2004 2:13:19 PM
 */

package com.onaro.sanscreen.acquisition.util.resolver;

class BrocadePortGenerator extends VendorPortGenerator {
    // http://en.wikipedia.org/wiki/World_Wide_Name
    // http://standards.ieee.org/develop/regauth/oui/oui.txt
    private static final String brocadeID[] = {
            "00:05:1E", "00:60:69", "00:05:33", "00:27:F8", "00:00:88",
            "00:01:0F", "00:04:80", "00:05:33", "00:0C:DB", "00:12:F2",
            "00:14:C9", "00:1B:ED", "00:24:38", "00:27:F8", "00:60:DF",
            "00:E0:52", "50:EB:1A", "74:8E:F8", "8C:7C:FF", "C4:F5:7C",
            "CC:4E:24"};

    public String getVendorName() {
        return "Brocade";
    }

    public boolean checkVendor(final String chassisWWN) {
        final String vendorID = extractVendorFromWWN(chassisWWN);
        for (final String id : brocadeID) {
            if (vendorID.equalsIgnoreCase(id)) {
                return true;
            }
        }
        return false;
    }

 /**
     * The way brocade encodes WWNs is the following:
     * <p/>For port number &lt; 256:  <code>20:[port number]:[... (Brocade OUI), (Switch identifier)]</code>
     * <p/>For port number &gt;= 256: <code>2E:[port number - 256]:[... (Brocade OUI), (Switch identifier)]</code>
     *
     * @param chassisWWN    chassis wwn
     * @param bladeNumber   blade number
     * @param portNumber    port number
     *
     * @return The generated WWN
     */
    public String getPortWWN(final String chassisWWN, final int bladeNumber, final int portNumber)  {
        if (chassisWWN == null)
            throw new IllegalArgumentException("Chassis WWN wasn't set");

        if (bladeNumber != 0)
            throw new IllegalArgumentException("Can't generate the port when blade provided");

        final StringBuilder sb = new StringBuilder(24);
        if (portNumber > 255)
            sb.append("2E:");
        else
            sb.append("20:");

        final String port = Integer.toString(portNumber % 256, 16);
        if (port.length() < 2)
            sb.append(0);

        sb.append(port);
        sb.append(chassisWWN.substring(5));

        return sb.toString().toUpperCase();
    }
}
