package com.onaro.sanscreen.acquisition.util.servercache;

import com.onaro.sanscreen.acquisition.framework.FrameworkException;
import com.onaro.sanscreen.acquisition.framework.datasource.BasePackage;
import com.onaro.sanscreen.acquisition.framework.datasource.DataSourceOutputFileType;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileWriter;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSInputStream;
import com.onaro.sanscreen.acquisition.framework.recordings.RecordingSample;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.util.HashSet;
import java.util.Set;

class Utility {
    static Set<Long> getSetUnion(final Set<Long> set1, final Set<Long> set2) {
        final Set<Long> set = new HashSet<Long>(set1);
        for (final Long id : set2) {
            if (!set.contains(id)) {
                set.add(id);
            }
        }
        return set;
    }

    static Set<Long> getSetSubtract(final Set<Long> set1, final Set<Long> set2) {
        final Set<Long> set = new HashSet<Long>(set1);
        for (final Long id : set2) {
            if (set.contains(id)) {
                set.remove(id);
            }
        }
        return set;
    }

    static void recordServerCache(
            final BasePackage basePackage, final String key, final Object object) throws FrameworkException {
        final Writer writer = getSimulationOutputStream(basePackage, key);
        marshallObject(object, writer);
        try {
            writer.close();
        } catch (IOException ignored) {
        }
    }

    static <T> T simulateServerCache(
            final BasePackage basePackage, final String key, final Class<T> clazz) throws FrameworkException {
        final DSInputStream serverCacheStream = getSimulationInputStream(basePackage, key);

        if (serverCacheStream == null) {
            return null;
        } else {
            return unmarshallXml(serverCacheStream, clazz);
        }
    }

    static <T> T unmarshallXml(final InputStream inputStream, final Class<T> clazz) throws FrameworkException {
        try {
            final JAXBContext jaxbContext = JAXBContext.newInstance(
                    clazz, ServerVolumeMap.class, ServerVolume.class, ServerPort.class, ServerVolumeMask.class);
            final Unmarshaller unmarshaller = jaxbContext.createUnmarshaller();
            final BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream, "UTF-8"));
            //noinspection unchecked
            return (T) unmarshaller.unmarshal(reader);
        } catch (JAXBException ex) {
            throw new FrameworkException("Failed to simulate server storage cache from recording.", ex);
        } catch (UnsupportedEncodingException ex) {
            throw new FrameworkException("Failed to simulate server storage cache from recording.", ex);
        }
    }

    static void marshallObject(final Object object, final Writer writer) throws FrameworkException {
        try {
            final JAXBContext jaxbContext = JAXBContext.newInstance(
                    object.getClass(), ServerVolumeMap.class, ServerVolume.class, ServerPort.class, ServerVolumeMask.class);
            final Marshaller marshaller = jaxbContext.createMarshaller();
            marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);
            marshaller.setProperty(Marshaller.JAXB_SCHEMA_LOCATION, "");
            marshaller.marshal(object, writer);
        } catch (JAXBException ex) {
            throw new FrameworkException("Failed to generate XML from the server cache", ex);
        }
    }

    private static DSInputStream getSimulationInputStream(
            final BasePackage basePackage, final String key) throws FrameworkException {
        return basePackage.getConfiguration().getSimulationSession().
                getCurrentSample().getStreamByDescription(DataSourceOutputFileType.servercache, key, key);
    }

    private static Writer getSimulationOutputStream(
            final BasePackage basePackage, final String key) throws FrameworkException {
        final RecordingSample recordingSample = basePackage.getRecordingSession().getCurrentSample();
        try {
            final DSFileWriter recordingWriter = recordingSample.createTextualOutputFile(
                    basePackage.getConfiguration(), DataSourceOutputFileType.servercache, key, "xml");
            recordingSample.addFileDescription(new File(recordingWriter.getFileName()).getName(), key, key);
            return recordingWriter;
        } catch (IOException ex) {
            throw new FrameworkException("Failed to generate recording output stream file.", ex);
        }
    }
}
