package com.onaro.sanscreen.server.interfaces.impl;

import com.netapp.oci.platform.common.ServerConstant;
import com.onaro.sanscreen.server.interfaces.remote.SchedulerManagement;
import org.jboss.ejb3.annotation.RemoteBinding;
import org.jboss.ejb3.annotation.RemoteBindings;
import org.quartz.SchedulerException;
import org.quartz.impl.SchedulerRepository;

import javax.annotation.security.RolesAllowed;
import javax.ejb.Stateless;
import java.util.Arrays;
import java.util.Collection;

/**
 * Manages quartz scheduler
 * <p/>
 * Manages 'JBossQuartzScheduler'. JBossQuartzScheduler is used by JBoss and all the OCI jobs are managed by this scheduler.
 * If you try to access the scheduler out of JBoss JVM, for example while running unit tests, you won't be able to access to
 * this scheduler. So, this ejb needs to be used in order to access the scheduler.
 * JBossQuartzScheduler is shipped with JBoss and configured in
 *    quartz-ra.rar\quartz-ra.jar\org\jboss\resource\adapter\quartz\inflow\quartz.properties.
 * quartz-ra.rar is located in the deploy folder of JBoss.
 *
 * @author Onur Keskin
 */
@Stateless
@RemoteBindings({
        @RemoteBinding(jndiBinding = SchedulerManagement.JNDI_NAME),
        @RemoteBinding(jndiBinding = ServerConstant.HTTPS_JNDI_BINDING_PREFIX + SchedulerManagement.JNDI_NAME)})
@RolesAllowed({"guest", "user", "admin"})
public class SchedulerManagementBean implements SchedulerManagement {

    public static final String DEFAULT_SCHEDULER_NAME = "JBossQuartzScheduler";

    public void pauseScheduler() throws SchedulerException {
        SchedulerRepository.getInstance().lookup(DEFAULT_SCHEDULER_NAME).pauseAll();
    }

    public void resumeScheduler() throws SchedulerException {
        SchedulerRepository.getInstance().lookup(DEFAULT_SCHEDULER_NAME).resumeAll();
    }

    public void shutdown(boolean waitForJobsToComplete) throws Exception {
        SchedulerRepository.getInstance().lookup(DEFAULT_SCHEDULER_NAME).shutdown(waitForJobsToComplete);
    }

    public void triggerJob(String jobName, String groupName) throws SchedulerException {
        SchedulerRepository.getInstance().lookup(DEFAULT_SCHEDULER_NAME).triggerJobWithVolatileTrigger(jobName, groupName);
    }

    public Collection<String> getJobGroups() throws SchedulerException {
        return Arrays.asList(SchedulerRepository.getInstance().lookup(DEFAULT_SCHEDULER_NAME).getJobGroupNames());
    }

    public Collection<String> getJobs(String group) throws SchedulerException {
        return Arrays.asList(SchedulerRepository.getInstance().lookup(DEFAULT_SCHEDULER_NAME).getJobNames(group));
    }
}
