package com.onaro.sanscreen.server.sessions;

import com.netapp.oci.platform.common.param.ParamException;
import org.jboss.ejb3.annotation.ResourceAdapter;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.StatefulJob;
import org.apache.log4j.Logger;

import javax.ejb.ActivationConfigProperty;
import javax.ejb.EJB;
import javax.ejb.MessageDriven;
import javax.annotation.security.RunAs;

/**
 * Triggers every day at 7AM
 *
 * @author carl.chan-aldebol
 */
@MessageDriven(activationConfig = {
    @ActivationConfigProperty(propertyName = "cronTrigger", propertyValue = "0 0 7 * * ?"),
    @ActivationConfigProperty(propertyName = "jobName", propertyValue = "LogSchedulerJob")})
@ResourceAdapter("quartz-ra.rar")
@RunAs("admin")
public class LogSchedulerJob implements StatefulJob {
    @EJB
    private LogSchedulerSession logSchedulerSession;
    private static final Logger logger = Logger.getLogger(LogSchedulerJob.class);

    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        try {
            logSchedulerSession.checkLogs(new Long(System.currentTimeMillis()));
        } catch (ParamException pe) {
            logger.error("Param Exception", pe);
            throw new JobExecutionException(pe);
        }
    }
}