package com.onaro.sanscreen.util;

import org.jboss.security.auth.callback.UsernamePasswordHandler;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.security.auth.login.Configuration;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;
import java.util.HashMap;
import java.util.Map;

public class JndiHelper {
    static Map interfaces;
    static protected Context initialContext = null;
    static LoginContext lc = null;

    static {
        interfaces = new HashMap();
    }


    /**
     * Looks up a EJB interface bound to a name
     * @param name the JNDI name to be looked up
     * @return The interface bound to the specified name
     * @throws NamingException if the lookup failed
     * @deprecated Does exactly the same thing as getEJB(). Use getEJB() instead.
     */
    public static Object getLocalEJB(String name) throws NamingException {
        if (interfaces.containsKey(name)) {
            return interfaces.get(name);
        }
        if (initialContext == null) {
            initialContext = new InitialContext();

        }


        Object obj = initialContext.lookup(name);
        interfaces.put(name, obj);
        return obj;
    }

    /**
     * Looks up a EJB interface bound to a name
     * @param name the JNDI name to be looked up
     * @return The interface bound to the specified name
     * @throws NamingException if the lookup failed
     */
    public static Object getEJB(String name) throws NamingException {

        if (interfaces.containsKey(name)) {
            return interfaces.get(name);
        }

        if (initialContext == null) {
            initialContext = new InitialContext();
        }

        Object obj = initialContext.lookup(name);
        interfaces.put(name, obj);
        return obj;
    }

    /**
     * Looks up a EJB interface bound to a name. Similar to getEJB, but uses the no-parameter login() method to log in
     * as "guest", and checks for a LoginException.
     *
     * @param name the JNDI name to be looked up
     * @return The interface bound to the specified name
     * @throws NamingException if the lookup failed, or the login failed
     */
    public static Object getGuestEJB(String name) throws NamingException {

        if (interfaces.containsKey(name)) {
            return interfaces.get(name);
        }
        if (initialContext == null) {
            try {
                login();
                initialContext = new InitialContext();
            } catch (LoginException e) {
                throw new NamingException("Got login Exception " + e);
            }
        }

        Object obj = initialContext.lookup(name);
        interfaces.put(name, obj);
        return obj;
    }

    public static void login() throws LoginException {
        Configuration.setConfiguration(new LoginConfiguration());
        UsernamePasswordHandler handler = new UsernamePasswordHandler("guest", "".toCharArray());
        lc = new LoginContext("onaro-login", handler);
        lc.login();
    }

    public static void login(String user, String password) throws LoginException {
        Configuration.setConfiguration(new LoginConfiguration());
        UsernamePasswordHandler handler = new UsernamePasswordHandler(user, password.toCharArray());
        lc = new LoginContext("onaro-login", handler);
        lc.login();
    }

    public static void adminLogin() throws LoginException {
        Configuration.setConfiguration(new LoginConfiguration());
        UsernamePasswordHandler handler = new UsernamePasswordHandler("admin", "admin123".toCharArray());
        lc = new LoginContext("onaro-login", handler);
        lc.login();
    }

    public static void logout() throws LoginException {
        initialContext = null;
        interfaces = new HashMap();
        if (lc != null)
            lc.logout();
        lc = null;
    }
}
