package com.onaro.util;

/**
 * This class maintains average of samples
 */
public class Average {
    private final long[] samples;
    private int index = 0;

    public Average(int numOfSamples) {
        this.samples = new long[numOfSamples];

        for (int i = 0; i < samples.length; i++)
            samples[i] = 0;
    }

    public synchronized void add(long num) {
        samples[index++ % samples.length] = num;
    }

    /**
     * Calculates the average based on the last samples
     *
     * @return the average or null if no samples were recorded
     */
    public synchronized Long calculateAverage() {
        int total = 0;

        if (index == 0)
            return null;

        final long max = Math.min(samples.length, index);
        for (int i = 0; i < max; i++)
            total += samples[i];

        return total / max;
    }
}
