package com.onaro.util;

import org.apache.log4j.Logger;

import java.util.Comparator;
import java.io.Serializable;

/**
 * A class that compares two WWNs by their numerical value. All WWNs are expected to be of the format:
 *    XX:XX:XX:XX:XX:XX:XX:XX:XX where XX is a hex digit (capital letters)
 */
public class WWNComparator implements Comparator<String>, Serializable {
    private final static long serialVersionUID = 1;

    public int compare(final String wwn1, final String wwn2) throws IllegalArgumentException {
        final String[] nibbles1 = wwn1.split(":");
        if (nibbles1.length != 8)
            throw new IllegalArgumentException("Invalid WWN format " + wwn1);

        final String[] nibbles2 = wwn2.split(":");
        if (nibbles2.length != 8)
            throw new IllegalArgumentException("Invalid WWN format " + wwn2);

        for (int i = 0 ; i < 8 ; i++) {
            int c = Integer.parseInt(nibbles1[i], 16) - Integer.parseInt(nibbles2[i], 16);
            if (c != 0)
                return c;
        }

        return 0;
    }
}
