package com.onaro.sanscreen.acquisition.netappstorageperformance.datasource;

import com.netapp.oci.netapp.performance.aggregate.interfaces.data.AggregateMeasurements;
import com.netapp.oci.netapp.performance.aggregate.interfaces.data.AggregateRawMeasurements;
import com.netapp.oci.netapp.performance.aggregate.interfaces.data.WaflHyaAggrMeasurements;
import com.netapp.oci.netapp.performance.cifsvserver.interfaces.data.CifsVserverMeasurements;
import com.netapp.oci.netapp.performance.cifsvserver.interfaces.data.CifsVserverRawMeasurements;
import com.netapp.oci.netapp.performance.cluster.interfaces.data.ClusterMeasurements;
import com.netapp.oci.netapp.performance.disk.interfaces.data.DiskMeasurements;
import com.netapp.oci.netapp.performance.disk.interfaces.data.DiskRawMeasurements;
import com.netapp.oci.netapp.performance.ext_cache_obj.interfaces.data.ExtCacheObjMeasurements;
import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifMeasurements;
import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifRawMeasurements;
import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifVserverMeasurements;
import com.netapp.oci.netapp.performance.fcplif.interfaces.data.FcpLifVserverRawMeasurements;
import com.netapp.oci.netapp.performance.fcpport.interfaces.data.FcpPortMeasurements;
import com.netapp.oci.netapp.performance.fcpport.interfaces.data.FcpPortRawMeasurements;
import com.netapp.oci.netapp.performance.hostadapter.interfaces.data.HostadapterMeasurements;
import com.netapp.oci.netapp.performance.iscsilif.interfaces.data.IscsiLifMeasurements;
import com.netapp.oci.netapp.performance.iscsilif.interfaces.data.IscsiLifRawMeasurements;
import com.netapp.oci.netapp.performance.iscsilif.interfaces.data.IscsiLifVserverMeasurements;
import com.netapp.oci.netapp.performance.iscsilif.interfaces.data.IscsiLifVserverRawMeasurements;
import com.netapp.oci.netapp.performance.lun.interfaces.data.LunMeasurements;
import com.netapp.oci.netapp.performance.lun.interfaces.data.LunRawMeasurements;
import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifMeasurements;
import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifRawMeasurements;
import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifVserverMeasurements;
import com.netapp.oci.netapp.performance.networklif.interfaces.data.NetworkLifVserverRawMeasurements;
import com.netapp.oci.netapp.performance.networkport.interfaces.data.NetworkPortMeasurements;
import com.netapp.oci.netapp.performance.nfs.interfaces.data.*;
import com.netapp.oci.netapp.performance.nic.interfaces.data.NicMeasurements;
import com.netapp.oci.netapp.performance.nic.interfaces.data.NicRawMeasurements;
import com.netapp.oci.netapp.performance.node.interfaces.data.*;
import com.netapp.oci.netapp.performance.processor.interfaces.data.ProcessorMeasurements;
import com.netapp.oci.netapp.performance.qosservicecenter.interfaces.data.QosServiceCenterMeasurements;
import com.netapp.oci.netapp.performance.qosworkload.interfaces.data.QosVolumeWorkloadMeasurements;
import com.netapp.oci.netapp.performance.qosworkloaddblade.interfaces.data.QosWorkloadQueueDBladeMeasurements;
import com.netapp.oci.netapp.performance.qosworkloaddetail.interfaces.data.QosWorkloadDetailMeasurements;
import com.netapp.oci.netapp.performance.qosworkloadnblade.interfaces.data.QosWorkloadQueueNBladeMeasurements;
import com.netapp.oci.netapp.performance.volume.interfaces.data.VolumeMeasurements;
import com.netapp.oci.netapp.performance.volume.interfaces.data.VolumeRawMeasurements;
import com.netapp.oci.netapp.performance.volumenode.interfaces.data.VolumeNodeMeasurements;
import com.netapp.oci.netapp.performance.vserver.interfaces.data.VolumeVserverMeasurements;
import com.netapp.oci.netapp.performance.vserver.interfaces.data.VserverMeasurements;
import com.netapp.oci.netapp.performance.wafl.interfaces.data.WaflMeasurements;
import com.netapp.oci.platform.common.interfaces.data.performance.RawSample;
import com.onaro.sanscreen.acquisition.framework.FrameworkException;
import com.onaro.sanscreen.acquisition.framework.datasource.*;
import com.onaro.sanscreen.acquisition.framework.datasource.Package;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSFileWriter;
import com.onaro.sanscreen.acquisition.framework.datasource.io.DSSessionFileReader;
import com.onaro.sanscreen.acquisition.netappfoundation.datasource.NetAppClockSynchronizer;
import com.onaro.sanscreen.acquisition.netappfoundation.datasource.NetAppFoundationUpdatesListener;
import com.onaro.sanscreen.acquisition.netappfoundation.datasource.config.NetAppFoundationConfiguration;
import com.onaro.sanscreen.acquisition.netappfoundation.datasource.dependant.NetAppFoundationDependantPackage;
import com.onaro.sanscreen.acquisition.netappfoundation.datasource.dependant.config.NetAppFoundationDependantConfiguration;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.Cluster.ClusterKey;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.StorageRoot;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.VServer;
import com.onaro.sanscreen.acquisition.netappschema.netappdata.VServer.VServerKey;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.config.NetAppStoragePerformanceConfiguration;
import com.onaro.sanscreen.acquisition.netappstorageperformance.datasource.integrity.*;
import com.onaro.sanscreen.acquisition.schema.Root;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.util.*;

/**
 * Base class for all storage performance packages.
 *
 * <p/>The package class takes care of the interface with the server.
 */
public abstract class NetAppStoragePerformancePackage<C extends NetAppFoundationDependantConfiguration, CTX extends NetAppStorageContext> extends NetAppFoundationDependantPackage<C> implements NetAppFoundationUpdatesListener, NetAppStoragePerformancePackageMBean, NetAppStoragePerformanceReporter {
    private final static Logger LOG = LoggerFactory.getLogger(NetAppStoragePerformancePackage.class);
    public final static String PACKAGE = "netappstorageperformance";

    private final static String crlf = System.getProperty("line.separator");

    private final NetAppStoragePerformanceUpdatesListener storagePerformanceListener;

    private NetAppClockSynchronizer clock = null;

    // One originator may report XXXs
    private final Map<String, Set<Cluster.ClusterKey>> originatorLastReports = new HashMap<String, Set<Cluster.ClusterKey>>();

    // Each storage has a context - data that we gathered in foundation that we need in performance
    private final SortedMap<Cluster.ClusterKey, CTX> storageContexts = new TreeMap<Cluster.ClusterKey, CTX>();

    //A bit to determine if disk samples were reported on a given performance run
    private boolean reportedDisks = false;
    private boolean reportedRawDisks = false;

    //A bit to determine if volume samples were reported on a given performance run
    private boolean reportedVolumes = false; 
    private boolean reportedRawVolumes = false;
    private boolean reportedRawAggregates = false;
    private boolean reportedAggregates = false;
    private boolean reportedNfsv3 = false;
    private boolean reportedRawNfsv3 = false;
    private boolean reportedNfsv4 = false;
    private boolean reportedRawNfsv4 = false;
    private boolean reportedNfsv41 = false;
    private boolean reportedRawNfsv41 = false;
    private boolean reportedCifsVserver = false;
    private boolean reportedRawCifsVserver = false;
    private boolean reportedLifVserver = false;
    private boolean reportedRawLifVserver = false;
    private boolean reportedLuns = false;
    private boolean reportedRawLuns = false;
    private boolean reportedNodes = false;
    private boolean reportedRawNodes = false;
    private boolean reportedLifs = false;
    private boolean reportedRawLifs = false;
    private boolean reportedIscsiLifs = false;
    private boolean reportedRawIscsiLifs = false;
    private boolean reportedIscsiLifVservers = false;
    private boolean reportedRawIscsiLifVservers = false;
    private boolean reportedFcpLifs = false;
    private boolean reportedRawFcpLifs = false;
    private boolean reportedFcpLifVservers = false;
    private boolean reportedRawFcpLifVservers = false;    
    private boolean reportedFcpPorts = false;
    private boolean reportedRawFcpPorts = false;
    private boolean reportedIpPorts = false;
    private boolean reportedRawNic = false;
    private boolean reportedNic = false;
    private boolean reportedVolumeVserver = false;
    private boolean reportedVserver = false;
    private boolean reportedProcessors = false;
    private boolean reportedWorkloads = false;
    private boolean reportedWorkloadDetails = false;
    private boolean reportedServiceCenters = false;
    private boolean reportedWafl = false;
    private boolean reportedWaflHyaAggr = false;
    private boolean reportedExtCache = false;
    private boolean reportedVolumeNodeCache = false;

    private boolean reportedCIFSNode = false;
    private boolean reportedSMB1Node = false;
    private boolean reportedSMB2Node = false;
    private boolean reportedNFSv3 = false;
    private boolean reportedNFSv4 = false;
    private boolean reportedNFSv41 = false;
    private boolean reportedHostadapter = false;

    //How many polls have gone by with a no reports of any data to the server?
    private int consecutiveNonReports = 0;

    private final long leaseTimeMS = 30 * 60 * 1000;                // 30 minutes TODO: better lease time

    private long cachedServerTime = 0;

    protected NetAppStoragePerformancePackage(final BaseDataSource dataSource, final C configuration, final DataSourceSampleListener dataSourceSampleListener, final NetAppStoragePerformanceUpdatesListener storagePerformanceListener, final NetAppClockSynchronizer clockSynchronizer) {
        super(dataSource, configuration, dataSourceSampleListener);
        this.storagePerformanceListener = storagePerformanceListener;
        this.clock = clockSynchronizer;
    }

    protected void doFoundationDependantPollNow() throws DataSourceErrorException, FrameworkException {
        cachedServerTime = getCurrentServerTime();
        resetReportedBits();
        doArrayPerformance(dataSource);

        //We want to assert that samples from at least one class of objects was reported on each performance poll.
        //If not, we throw an exception to notify people of this possible problem.
        final Package performancePackage = PackageContext.getPackage();
        final NetAppStoragePerformanceConfiguration performanceConfiguration = (NetAppStoragePerformanceConfiguration) performancePackage.getConfiguration();
        final NetAppFoundationConfiguration foundationConfiguration = performanceConfiguration.getNetAppFoundationConfiguration();
        final NetAppFoundationConfiguration.AcquisitionPollStatus pollStatus = foundationConfiguration.getAcquisitionPollStatus(performancePackage.getPackageName());
        if (!checkReportedBits()) {
            pollStatus.setDataReported(false);
            final int limit = getNumberOfAllowedConsecutiveEmptySamples();
            if (limit >= 0) {
                consecutiveNonReports = consecutiveNonReports == Integer.MAX_VALUE
                                        ? Integer.MAX_VALUE
                                        : consecutiveNonReports + 1;
                if (consecutiveNonReports > limit) {
                    throw getExceptionForConsecutiveEmptySamples();
                }
            }
        }
        else {
            pollStatus.setDataReported(true);
            consecutiveNonReports = 0;
        }
    }

    /**
     * Restore all "report bits" to their default value (false).   This will allow tracking to begin anew on
     * each performance poll.
     */
    private void resetReportedBits() {
        reportedDisks = false;
        reportedRawDisks = false;
        reportedVolumes = false;
        reportedRawVolumes = false;
        reportedRawAggregates = false;
        reportedAggregates = false;    
        reportedNfsv3 = false;
        reportedRawNfsv3 = false;
        reportedNfsv4 = false;
        reportedRawNfsv4 = false;
        reportedNfsv41 = false;
        reportedRawNfsv41 = false;
        reportedCifsVserver = false;
        reportedRawCifsVserver = false;
        reportedLifVserver = false;
        reportedRawLifVserver = false;        
        reportedLuns = false;
        reportedRawLuns = false;
        reportedNodes = false;
        reportedRawNodes = false;
        reportedLifs = false;
        reportedRawLifs = false;
        reportedIscsiLifs = false;
        reportedRawIscsiLifs = false;
        reportedIscsiLifVservers = false;
        reportedRawIscsiLifVservers = false;
        reportedRawFcpLifs = false;
        reportedFcpLifs = false;
        reportedRawFcpLifVservers = false;
        reportedFcpLifVservers = false;
        reportedFcpPorts = false;
        reportedFcpPorts = false;
        reportedRawFcpPorts = false;
        reportedIpPorts = false;
        reportedRawNic = false;
        reportedNic = false;
        reportedVolumeVserver = false;
        reportedVserver = false;
        reportedProcessors = false;
        reportedWorkloads = false;
        reportedWorkloadDetails = false;
        reportedServiceCenters = false;
        reportedWafl = false;
        reportedWaflHyaAggr = false;
        reportedExtCache = false;
        reportedVolumeNodeCache = false;
        reportedCIFSNode = false;
        reportedSMB1Node = false;
        reportedSMB2Node = false;
        reportedNFSv3 = false;
        reportedNFSv4 = false;
        reportedNFSv41 = false;
        reportedHostadapter = false;
    }

    /**
     * Were samples of any object type (disk, volume, internal volume) reported on the latest poll run?  Returns
     * true if at least one type was reported.
     *
     * @return true if samples were reported from at least one object type to the server.
     */
    private boolean checkReportedBits() {
        return reportedDisks || reportedRawDisks || reportedVolumes || reportedRawVolumes || reportedRawAggregates || reportedAggregates || reportedNfsv3 || reportedRawNfsv3 || reportedRawCifsVserver || reportedCifsVserver
        	   || reportedNfsv4 || reportedRawNfsv4 || reportedNfsv41 || reportedRawNfsv41 || reportedLuns || reportedRawLuns || reportedNodes || reportedRawNodes || reportedLifs || reportedRawLifs
               || reportedIscsiLifs || reportedRawIscsiLifs || reportedFcpLifs || reportedRawFcpLifs || reportedFcpLifVservers || reportedRawFcpLifVservers || reportedFcpPorts || reportedRawFcpPorts || reportedIpPorts || reportedRawNic || reportedNic || reportedVolumeVserver
               || reportedProcessors || reportedWorkloads || reportedWorkloadDetails || reportedServiceCenters || reportedIscsiLifVservers || reportedRawIscsiLifVservers || reportedVserver
               || reportedWafl || reportedExtCache || reportedVolumeNodeCache || reportedCIFSNode || reportedSMB1Node || reportedLifVserver || reportedRawLifVserver
               || reportedSMB2Node || reportedNFSv3 || reportedNFSv4 || reportedNFSv41 || reportedWaflHyaAggr || reportedHostadapter;
    }

    /**
     * Generate the exception to be thrown when performance package encounters consecutive empty samples.
     *
     * <p>Splitting this knowledge out into a separate method allows derived packages instances to override
     * and provide their own customized message in the event more specific triage information is available.
     *
     * @return DataSourceErrorException to be thrown in the event of reports of consecutive empty samples.
     */
    protected DataSourceErrorException getExceptionForConsecutiveEmptySamples() {
        return new DataSourceErrorException(DataSourceErrorType.RETRIEVE_DATA_FROM_DEVICE,
                                            "Performance sample failed to send any data to server on " + consecutiveNonReports + " consecutive updates.",
                                            "Performance sample failed to send any data to server on " + consecutiveNonReports + "consecutive updates.");
    }

    /**
     * Specify number of allowed empty samples before a failure is thrown by package level.  Defaults to 1.
     *
     * <p> Instances may override to customize.   Any value < 0 will cause this type of error to be surpressed
     * entirely.   A value of 0 will lead to an error on any empty sample report.
     *
     * @return number of consecutive empty samples before a failure is thrown.  Defaults to 1.
     */
    protected int getNumberOfAllowedConsecutiveEmptySamples() {
        // We allow 2 non-reports and expect valid performance data at the 3rd poll.
        // Traditionally, we only allow 1 sample to be empty; the first perf poll has no previous value
        // to compare to.
        // The second poll allows delta & rate calculation and produces data.
        //
        // Now with TA23247    Acquisition adjustment work    US10489: OPM - Report time adjusted performance samples,
        // we estimate the value of counters at the time of the poll start, and perform all performance calculations
        // on these estimated values.   The estimated value requires 2 consecutive performance polls to estimate.
        // Thus we can't perform calculations on estimates until we have 2 (current & previous) estimates, which
        // require 3 polls total.
        return 2;
    }

    public final void start() throws DataSourceErrorException, FrameworkException {
        doStart();
    }

    public final void stop() throws DataSourceErrorException, FrameworkException {
        doStop();

        consecutiveNonReports = 0;
        storageContexts.clear();
        originatorLastReports.clear();
    }

    public SortedMap<Cluster.ClusterKey, CTX> getStorageContexts() {
        return storageContexts;
    }

    /**
     * @return The set of storage arrays that performance should be collected for
     */
    // TODO
    public Set<Cluster.ClusterKey> getClustersToReport() {
        Set<Cluster.ClusterKey> resultSet = new HashSet<Cluster.ClusterKey>(storageContexts.keySet());
        //        final NetAppStoragePerformanceConfiguration.DeviceFilterType filterType = getConfiguration().getFilterType();
        //        final Set<String> filterSet = getConfiguration().getFilter().getDevices();
        //        if (filterType.equals(NetAppStoragePerformanceConfiguration.DeviceFilterType.EXCLUDE)) {
        //            for (final String ip : filterSet) {
        //                resultSet.remove(new Cluster.ClusterKey(ip));
        //            }
        //        }
        //        else {
        //            // for an include filter, remove anything not in the filter
        //            if (filterType.equals(NetAppStoragePerformanceConfiguration.DeviceFilterType.INCLUDE)) {
        //                final Set<Cluster.ClusterKey> storagesToInclude = new HashSet<Cluster.ClusterKey>();
        //                for (final Cluster.ClusterKey result : resultSet) {
        //                    if (filterSet.contains(result.getIP())) {
        //                        storagesToInclude.add(result);
        //                    }
        //                }
        //                resultSet = storagesToInclude;
        //            }
        //            else {
        //                // No other options for filter type
        //            }
        //        }
        return resultSet;
    }

    /**
     * We will use this update to get a list of volumes to query. Only the volumes that are mapped are collected for performance
     * (along with their meta members).
     */
    protected void doUpdateFromFoundation(final String originatorKey, final Root odataRoot, final boolean changed, final Serializable context) {
        if (getDataSource().isStopped()) {
            return;
        }

        if (!(odataRoot instanceof StorageRoot)) {
            return;
        }

        if (!changed && storageContexts.size() > 0) {
            return;
        }

        final StorageRoot storageRoot = (StorageRoot) odataRoot;

        // Store a set of storages reported in previous run
        final Set<Cluster.ClusterKey> thisOriginatorsLastReport = getOrCreateLastOriginatorReport(originatorKey);

        // Store a set of storages reported in this run. It will be used as "last report" in the next run
        final Set<Cluster.ClusterKey> currentReport = new TreeSet<Cluster.ClusterKey>();

        // (Re)create StorageContext for each reported storage
        for (final Cluster cluster : storageRoot.getClusters().values()) {
            // Accumulate storages from current odata
            currentReport.add(cluster.getKey());

            //Always create a new storage context. The existing storage context will be used by the current performance
            //poll.
            final CTX storageContext = createStorageContext(this, cluster.getKey());
            storageContexts.put(cluster.getKey(), storageContext);
            storageContext.processFoundationData(cluster);
        }

        // the "stopped reporting" list starts with everything in the last report
        // and we remove everything that is currently reported by foundation. This
        // way, we can know which storage were removed
        final Set<Cluster.ClusterKey> stoppedReporting = new HashSet<Cluster.ClusterKey>(thisOriginatorsLastReport);
        stoppedReporting.removeAll(currentReport);

        // For all the arrays that we stopped discovering, we have to stop the scheduled timer and remove them from
        // the context list
        for (final Cluster.ClusterKey storageKey : stoppedReporting) {
            // remove this storage from the cache
            storageContexts.remove(storageKey);
        }

        // create the new "last report" from current report
        thisOriginatorsLastReport.clear();
        thisOriginatorsLastReport.addAll(currentReport);
    }

    /**
     * Returns set of storages reported by a given originator.
     * Creates new entry in originatorLastReports if needed.
     *
     * @param originatorKey Originator key to look for
     *
     * @return Set of storage (keys) reported by this originator
     */
    private Set<Cluster.ClusterKey> getOrCreateLastOriginatorReport(final String originatorKey) {
        Set<Cluster.ClusterKey> thisOriginatorsLastReport = originatorLastReports.get(originatorKey);

        // if there is no last report, add an empty last report for convenience
        if (thisOriginatorsLastReport == null) {
            thisOriginatorsLastReport = new TreeSet<Cluster.ClusterKey>();
            originatorLastReports.put(originatorKey, thisOriginatorsLastReport);
        }

        return thisOriginatorsLastReport;
    }

    /**
     * Check disk performance integrity on attached samples.  Throws an exception in the event on an integrity failure.
     *
     * @param key     the key to the storage being reported
     * @param samples samples being reported.
     *
     * @throws DataSourceErrorException in the event of an integrity failure.
     */
    private void checkDiskPerformanceIntegrity(final Cluster.ClusterKey key,
                                               final Collection<RawSample<DiskMeasurements>> samples)
            throws DataSourceErrorException {

        final NetAppDiskPerformanceIntegrityChecker checker = new NetAppDiskPerformanceIntegrityChecker(samples,
                                                                                                        key.getResourceKey(),
                                                                                                        logPrefix);
        try {
            checker.checkIntegrity();
        }
        catch (final IntegrityException ie) {
            throw new DataSourceErrorException(DataSourceErrorType.INCONSISTENT_DATA, ie.getMessage(),
                                               ie.getMessage(), ie);
        }
    }

    /**
     * Check volume performance integrity on attached samples.  Throws an exception in the event on an integrity failure.
     *
     * @param key     the key to the storage being reported
     * @param samples samples being reported.
     *
     * @throws DataSourceErrorException in the event of an integrity failure.
     */
    private void checkVolumePerformanceIntegrity(final VServer.VServerKey key,
                                                 final Collection<RawSample<VolumeMeasurements>> samples)
            throws DataSourceErrorException {

        final NetAppVolumePerformanceIntegrityChecker checker = new NetAppVolumePerformanceIntegrityChecker(samples,
                                                                                                            key.getResourceKey(),
                                                                                                            logPrefix);
        try {
            checker.checkIntegrity();
        }
        catch (final IntegrityException ie) {
            throw new DataSourceErrorException(DataSourceErrorType.INCONSISTENT_DATA, ie.getMessage(),
                                               ie.getMessage(), ie);
        }

    }

    /**
     * Check disk performance integrity on attached samples.  Throws an exception in the event on an integrity failure.
     *
     * @param key     the key to the storage being reported
     * @param samples samples being reported.
     *
     * @throws DataSourceErrorException in the event of an integrity failure.
     */
    private void checkNodePerformanceIntegrity(final Cluster.ClusterKey key,
                                               final Collection<RawSample<NodeMeasurements>> samples)
            throws DataSourceErrorException {

        final NetAppNodePerformanceIntegrityChecker checker = new NetAppNodePerformanceIntegrityChecker(samples,
                                                                                                        key.getResourceKey(),
                                                                                                        logPrefix);
        try {
            checker.checkIntegrity();
        }
        catch (final IntegrityException ie) {
            throw new DataSourceErrorException(DataSourceErrorType.INCONSISTENT_DATA, ie.getMessage(),
                                               ie.getMessage(), ie);
        }
    }

    /**
     * Check lun performance integrity on attached samples.  Throws an exception in the event on an integrity failure.
     *
     * @param key     the key to the storage being reported
     * @param samples samples being reported.
     *
     * @throws DataSourceErrorException in the event of an integrity failure.
     */
    private void checkLunPerformanceIntegrity(final VServer.VServerKey key,
                                              final Collection<RawSample<LunMeasurements>> samples)
            throws DataSourceErrorException {

        final NetAppLunPerformanceIntegrityChecker checker = new NetAppLunPerformanceIntegrityChecker(samples,
                                                                                                      key.getResourceKey(),
                                                                                                      logPrefix);
        try {
            checker.checkIntegrity();
        }
        catch (final IntegrityException ie) {
            throw new DataSourceErrorException(DataSourceErrorType.INCONSISTENT_DATA, ie.getMessage(),
                                               ie.getMessage(), ie);
        }
    }

    /**
     * Check fcp lif performance integrity on attached samples.  Throws an exception in the event on an integrity failure.
     *
     * @param key     the key to the storage being reported
     * @param samples samples being reported.
     *
     * @throws DataSourceErrorException in the event of an integrity failure.
     */
    private void checkFcpLifPerformanceIntegrity(final VServer.VServerKey key,
                                                 final Collection<RawSample<FcpLifMeasurements>> samples)
            throws DataSourceErrorException {

        final NetAppFcpLifPerformanceIntegrityChecker checker = new NetAppFcpLifPerformanceIntegrityChecker(samples,
                                                                                                            key.getResourceKey(),
                                                                                                            logPrefix);
        try {
            checker.checkIntegrity();
        }
        catch (final IntegrityException ie) {
            throw new DataSourceErrorException(DataSourceErrorType.INCONSISTENT_DATA, ie.getMessage(),
                                               ie.getMessage(), ie);
        }
    }

    /**
     * Check fcp port performance integrity on attached samples.  Throws an exception in the event on an integrity failure.
     *
     * @param key     the key to the storage being reported
     * @param samples samples being reported.
     *
     * @throws DataSourceErrorException in the event of an integrity failure.
     */
    private void checkFcpPortPerformanceIntegrity(final Cluster.ClusterKey key,
                                                  final Collection<RawSample<FcpPortMeasurements>> samples)
            throws DataSourceErrorException {

        final NetAppFcpPortPerformanceIntegrityChecker checker = new NetAppFcpPortPerformanceIntegrityChecker(samples,
                                                                                                              key.getResourceKey(),
                                                                                                              logPrefix);
        try {
            checker.checkIntegrity();
        }
        catch (final IntegrityException ie) {
            throw new DataSourceErrorException(DataSourceErrorType.INCONSISTENT_DATA, ie.getMessage(),
                                               ie.getMessage(), ie);
        }
    }

    /**
     * Check fcp port performance integrity on attached samples.  Throws an exception in the event on an integrity failure.
     *
     * @param key     the key to the storage being reported
     * @param samples samples being reported.
     *
     * @throws DataSourceErrorException in the event of an integrity failure.
     */
    private void checkIpPortPerformanceIntegrity(final Cluster.ClusterKey key,
                                                 final Collection<RawSample<NetworkPortMeasurements>> samples)
            throws DataSourceErrorException {

        final NetAppIpPortPerformanceIntegrityChecker checker = new NetAppIpPortPerformanceIntegrityChecker(samples,
                                                                                                            key.getResourceKey(),
                                                                                                            logPrefix);
        try {
            checker.checkIntegrity();
        }
        catch (final IntegrityException ie) {
            throw new DataSourceErrorException(DataSourceErrorType.INCONSISTENT_DATA, ie.getMessage(),
                                               ie.getMessage(), ie);
        }
    }

    /**
     * Check workload performance integrity on attached samples.  Throws an exception in the event on an integrity failure.
     *
     * @param key     the key to the storage being reported
     * @param samples samples being reported.
     *
     * @throws DataSourceErrorException in the event of an integrity failure.
     */
    private void checkQosVolumeWorkloadPerformanceIntegrity(final Cluster.ClusterKey key,
                                                            final Collection<RawSample<QosVolumeWorkloadMeasurements>> samples)
            throws DataSourceErrorException {

        final NetAppQosVolumeWorkloadPerformanceIntegrityChecker checker = new NetAppQosVolumeWorkloadPerformanceIntegrityChecker(samples,
                                                                                                                                  key.getResourceKey(),
                                                                                                                                  logPrefix);
        try {
            checker.checkIntegrity();
        }
        catch (final IntegrityException ie) {
            throw new DataSourceErrorException(DataSourceErrorType.INCONSISTENT_DATA, ie.getMessage(),
                                               ie.getMessage(), ie);
        }
    }

    private void checkQosServiceCenterPerformanceIntegrity(final Cluster.ClusterKey key,
                                                           final Collection<RawSample<QosServiceCenterMeasurements>> samples) throws DataSourceErrorException {

        final NetAppQosServiceCenterPerformanceIntegrityChecker checker = new NetAppQosServiceCenterPerformanceIntegrityChecker(samples,
                                                                                                                                key.getResourceKey(),
                                                                                                                                logPrefix);
        try {
            checker.checkIntegrity();
        }
        catch (final IntegrityException ie) {
            throw new DataSourceErrorException(DataSourceErrorType.INCONSISTENT_DATA, ie.getMessage(),
                                               ie.getMessage(), ie);
        }
    }

    private void checkWaflPerformanceIntegrity(final Cluster.ClusterKey key,
                                               final Collection<RawSample<WaflMeasurements>> samples) throws DataSourceErrorException {
        // TODO
        //        final NetAppWaflPerformanceIntegrityChecker checker = new NetAppWaflPerformanceIntegrityChecker(samples,
        //                                                                                                        key.getResourceKey(),
        //                                                                                                        logPrefix);
        //        try {
        //            checker.checkIntegrity();
        //        }
        //        catch (final IntegrityException ie) {
        //            throw new DataSourceErrorException(DataSourceErrorType.INCONSISTENT_DATA, ie.getMessage(),
        //                                               ie.getMessage(), ie);
        //        }
    }

    private void checkWaflHyaAggrPerformanceIntegrity(final Cluster.ClusterKey key,
                                               final Collection<RawSample<WaflHyaAggrMeasurements>> samples) throws DataSourceErrorException {
        // TODO
    }

    private void checkExtCacheObjPerformanceIntegrity(final Cluster.ClusterKey key,
                                                      final Collection<RawSample<ExtCacheObjMeasurements>> samples) throws DataSourceErrorException {
        // TODO
        //        final NetAppExtCacheObjlPerformanceIntegrityChecker checker = new NetAppExtCacheObjlPerformanceIntegrityChecker(samples,
        //                                                                                                                        key.getResourceKey(),
        //                                                                                                                        logPrefix);
        //        try {
        //            checker.checkIntegrity();
        //        }
        //        catch (final IntegrityException ie) {
        //            throw new DataSourceErrorException(DataSourceErrorType.INCONSISTENT_DATA, ie.getMessage(),
        //                                               ie.getMessage(), ie);
        //        }
    }

    public void reportAggregatePerformance(final Cluster.ClusterKey key, final Collection<RawSample<AggregateMeasurements>> samples) throws DataSourceErrorException{

    	if (getDataSource().isStopped()) {
    		return;
    	}

    	if (isRecording() && samples.size() > 0) {
    		dumpAggregateData(key, samples);
    	}

    	if (samples.size() == 0) {
    		return;
    	}

    	//checkAggregatePerformanceIntegrity(key, samples);

    	getStoragePerformanceListener().reportAggregatePerformance(this, key, getSampleTime(), samples, leaseTimeMS);
    	reportedAggregates = true;
    }
    
    public void reportAggregateRawPerformance(final Cluster.ClusterKey key, final Collection<RawSample<AggregateRawMeasurements>> samples) throws DataSourceErrorException{

    	if (getDataSource().isStopped()) {
    		return;
    	}

    	if (isRecording() && samples.size() > 0) {
    		dumpRawAggregateData(key, samples);
    	}

    	if (samples.size() == 0) {
    		return;
    	}

    	//checkAggregatePerformanceIntegrity(key, samples);

    	getStoragePerformanceListener().reportAggregateRawPerformance(this, key, getSampleTime(), samples, leaseTimeMS);
    	reportedRawAggregates = true;
    }
    
    public void reportNfsv3Performance(final Cluster.ClusterKey key, final Collection<RawSample<Nfsv3Measurements>> samples) throws DataSourceErrorException{
    	if (getDataSource().isStopped()) {
    		return;
    	}

    	if (isRecording() && samples.size() > 0) {
    		dumpNfsv3Data(key, samples);
    	}

    	if (samples.size() == 0) {
    		return;
    	}

    	getStoragePerformanceListener().reportNfsv3Performance(this, key, getSampleTime(), samples, leaseTimeMS);
    	reportedNfsv3 = true;

    }
    
    public void reportNfsv3RawPerformance(final Cluster.ClusterKey key, final Collection<RawSample<Nfsv3RawMeasurements>> samples) throws DataSourceErrorException{
    	if (getDataSource().isStopped()) {
    		return;
    	}

    	if (isRecording() && samples.size() > 0) {
    		dumpRawNfsv3Data(key, samples);
    	}

    	if (samples.size() == 0) {
    		return;
    	}

    	getStoragePerformanceListener().reportNfsv3RawPerformance(this, key, getSampleTime(), samples, leaseTimeMS);
    	reportedRawNfsv3 = true;

    }

    public void reportNfsv4Performance(final Cluster.ClusterKey key, final Collection<RawSample<Nfsv4Measurements>> samples) throws DataSourceErrorException{
    	if (getDataSource().isStopped()) {
    		return;
    	}

    	if (isRecording() && samples.size() > 0) {
    		dumpNfsv4Data(key, samples);
    	}

    	if (samples.size() == 0) {
    		return;
    	}

    	getStoragePerformanceListener().reportNfsv4Performance(this, key, getSampleTime(), samples, leaseTimeMS);
    	reportedNfsv4 = true;

    }
    
    public void reportNfsv4RawPerformance(final Cluster.ClusterKey key, final Collection<RawSample<Nfsv4RawMeasurements>> samples) throws DataSourceErrorException{
    	if (getDataSource().isStopped()) {
    		return;
    	}

    	if (isRecording() && samples.size() > 0) {
    		dumpRawNfsv4Data(key, samples);
    	}

    	if (samples.size() == 0) {
    		return;
    	}

    	getStoragePerformanceListener().reportNfsv4RawPerformance(this, key, getSampleTime(), samples, leaseTimeMS);
    	reportedRawNfsv4 = true;

    }

    public void reportNfsv41Performance(final Cluster.ClusterKey key, final Collection<RawSample<Nfsv41Measurements>> samples) throws DataSourceErrorException{
    	if (getDataSource().isStopped()) {
    		return;
    	}

    	if (isRecording() && samples.size() > 0) {
    		dumpNfsv41Data(key, samples);
    	}

    	if (samples.size() == 0) {
    		return;
    	}

    	getStoragePerformanceListener().reportNfsv41Performance(this, key, getSampleTime(), samples, leaseTimeMS);
    	reportedNfsv41 = true;

    }
    
    public void reportNfsv41RawPerformance(final Cluster.ClusterKey key, final Collection<RawSample<Nfsv41RawMeasurements>> samples) throws DataSourceErrorException{
    	if (getDataSource().isStopped()) {
    		return;
    	}

    	if (isRecording() && samples.size() > 0) {
    		dumpRawNfsv41Data(key, samples);
    	}

    	if (samples.size() == 0) {
    		return;
    	}

    	getStoragePerformanceListener().reportNfsv41RawPerformance(this, key, getSampleTime(), samples, leaseTimeMS);
    	reportedRawNfsv41 = true;

    }

    public void reportNfsPerformance(final Cluster.ClusterKey key, final Collection<RawSample<NfsMeasurements>> samples) throws DataSourceErrorException{
        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpNfsData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        getStoragePerformanceListener().reportNfsPerformance(this, key, getSampleTime(), samples, leaseTimeMS);
        reportedNfsv41 = true;

    }

    public void reportVserverPerformance(final Cluster.ClusterKey key, final Collection<RawSample<VserverMeasurements>> samples) throws DataSourceErrorException{
        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpVserverData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        getStoragePerformanceListener().reportVserverPerformance(this, key, getSampleTime(), samples, leaseTimeMS);
        reportedVserver = true;

    }

    public void reportCifsVserverRawPerformance(final Cluster.ClusterKey key, final Collection<RawSample<CifsVserverRawMeasurements>> samples) throws DataSourceErrorException{
    	if (getDataSource().isStopped()) {
    		return;
    	}

    	if (isRecording() && samples.size() > 0) {
    		dumpRawCifsVserverData(key, samples);
    	}

    	if (samples.size() == 0) {
    		return;
    	}

    	getStoragePerformanceListener().reportCifsVserverRawPerformance(this, key, getSampleTime(), samples, leaseTimeMS);
    	reportedRawCifsVserver = true;
    }
    
    public void reportCifsVserverPerformance(final Cluster.ClusterKey key, final Collection<RawSample<CifsVserverMeasurements>> samples) throws DataSourceErrorException{
        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpCifsVserverData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        getStoragePerformanceListener().reportCifsVserverPerformance(this, key, getSampleTime(), samples, leaseTimeMS);
        reportedCifsVserver = true;

    }
    
    public void reportLifVserverPerformance(final Cluster.ClusterKey key, final Collection<RawSample<NetworkLifVserverMeasurements>> samples) throws DataSourceErrorException{
    	if (getDataSource().isStopped()) {
    		return;
    	}

    	if (isRecording() && samples.size() > 0) {
    		dumpLifVserverData(key, samples);
    	}

    	if (samples.size() == 0) {
    		return;
    	}

    	getStoragePerformanceListener().reportLifVserverPerformance(this, key, getSampleTime(), samples, leaseTimeMS);
    	reportedLifVserver = true;

    }
    
    public void reportLifVserverRawPerformance(final Cluster.ClusterKey key, final Collection<RawSample<NetworkLifVserverRawMeasurements>> samples) throws DataSourceErrorException{
    	if (getDataSource().isStopped()) {
    		return;
    	}

    	if (isRecording() && samples.size() > 0) {
    		dumpLifVserverRawData(key, samples);
    	}

    	if (samples.size() == 0) {
    		return;
    	}

    	getStoragePerformanceListener().reportLifVserverRawPerformance(this, key, getSampleTime(), samples, leaseTimeMS);
    	reportedRawLifVserver = true;

    }
    
    /**
     * Used by the data source package implementation to report back storage performance information.
     *
     * @param key     Key of storage report about.
     * @param samples List of samples
     *
     * @throws DataSourceErrorException in event of an integrity check failure.
     */
    public final void reportDiskPerformance(final Cluster.ClusterKey key, final Collection<RawSample<DiskMeasurements>> samples)
            throws DataSourceErrorException {

        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpDiskData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        checkDiskPerformanceIntegrity(key, samples);

        getStoragePerformanceListener().reportDiskPerformance(this, key, getSampleTime(), samples, leaseTimeMS);
        reportedDisks = true;
    }

    public final void reportDiskRawPerformance(final Cluster.ClusterKey key, final Collection<RawSample<DiskRawMeasurements>> samples)
            throws DataSourceErrorException {

        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpRawDiskData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        //checkDiskPerformanceIntegrity(key, samples);

        getStoragePerformanceListener().reportDiskRawPerformance(this, key, getSampleTime(), samples, leaseTimeMS);
        reportedRawDisks = true;
    }

    /**
     * Used by the data source package implementation to report back storage performance information.
     * <p/>
     * <p/>This method reports data to the server seperately for each storage. It aggregates consecutive reports and makes
     * sure that each storage does not report faster than one minute.
     *
     * @param key     Key of storage report about.
     * @param samples List of samples
     *
     * @throws DataSourceErrorException in event of an integrity check failure.
     */
    public final void reportVolumePerformance(final Cluster.ClusterKey clusterKey, final VServer.VServerKey key, final Collection<RawSample<VolumeMeasurements>> samples)
            throws DataSourceErrorException {

        reportVolumePerformance(clusterKey, key, samples, leaseTimeMS);
    }

    /**
     * Used by the data source package implementation to report back storage performance information.
     *
     * <p/>This method reports data to the server separately for each storage. It aggregates consecutive reports and makes
     * sure that each storage does not report faster than one minute.
     *
     * @param key       Key of storage report about.
     * @param samples   List of samples
     * @param leaseTime Time (milli seconds) of validity of sample (-1 will use default value).
     *
     * @throws DataSourceErrorException in event of an integrity check failure.
     */
    public final void reportVolumePerformance(final Cluster.ClusterKey clusterKey, final VServer.VServerKey key, final Collection<RawSample<VolumeMeasurements>> samples, final long leaseTime)
            throws DataSourceErrorException {
        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpVolumeData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        checkVolumePerformanceIntegrity(key, samples);

        if (leaseTime == -1) {
            getStoragePerformanceListener().reportVolumePerformance(this, clusterKey, key, getSampleTime(), samples);
        }
        else {
            getStoragePerformanceListener().reportVolumePerformance(this, clusterKey, key, getSampleTime(), samples, leaseTime);
        }

        reportedVolumes = true;
    }

    /**
     * Used by the data source package implementation to report back storage performance information.
     *
     * <p/>This method reports data to the server separately for each storage. It aggregates consecutive reports and makes
     * sure that each storage does not report faster than one minute.
     *
     * @param key       Key of storage report about.
     * @param samples   List of samples
     * @param clusterKey Cluster Key.
     *
     * @throws DataSourceErrorException in event of an integrity check failure.
     */
    public final void reportVolumeRawPerformance(final Cluster.ClusterKey clusterKey, final VServer.VServerKey key, final Collection<RawSample<VolumeRawMeasurements>> samples)
            throws DataSourceErrorException {
    	
        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpRawVolumeData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        //checkDiskPerformanceIntegrity(key, samples);

        getStoragePerformanceListener().reportVolumeRawPerformance(this, clusterKey, key, getSampleTime(), samples, leaseTimeMS);
        reportedRawVolumes = true;
    }
    
    /**
     * Used by the data source package implementation to report back storage performance information.
     *
     * <p/>This method reports data to the server separately for each storage. It aggregates consecutive reports and makes
     * sure that each storage does not report faster than one minute.
     *
     * @param key     Key of storage report about.
     * @param samples List of samples
     *
     * @throws DataSourceErrorException in event of an integrity check failure.
     */
    public final void reportNodePerformance(final Cluster.ClusterKey key, final Collection<RawSample<NodeMeasurements>> samples)
            throws DataSourceErrorException {
        reportNodePerformance(key, samples, leaseTimeMS);
    }

    /**
     * Used by the data source package implementation to report back storage performance information.
     *
     * <p/>This method reports data to the server separately for each storage. It aggregates consecutive reports and makes
     * sure that each storage does not report faster than one minute.
     *
     * @param key       Key of storage report about.
     * @param samples   List of samples
     * @param leaseTime Time (milli seconds) of validity of sample (-1 will use default value).
     *
     * @throws DataSourceErrorException in event of an integrity check failure.
     */
    public final void reportNodePerformance(final Cluster.ClusterKey key, final Collection<RawSample<NodeMeasurements>> samples, final long leaseTime)
            throws DataSourceErrorException {
        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpNodeData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        checkNodePerformanceIntegrity(key, samples);

        if (leaseTime == -1) {
            getStoragePerformanceListener().reportNodePerformance(this, key, getSampleTime(), samples);
        }
        else {
            getStoragePerformanceListener().reportNodePerformance(this, key, getSampleTime(), samples, leaseTime);
        }

        reportedNodes = true;
    }
    
    public final void reportNodeRawPerformance(final Cluster.ClusterKey key, final Collection<RawSample<NodeRawMeasurements>> samples)
            throws DataSourceErrorException {
        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpRawNodeData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }
        
        getStoragePerformanceListener().reportNodeRawPerformance(this, key, getSampleTime(), samples, leaseTimeMS);

        reportedRawNodes = true;
    }

    public final void reportLunPerformance(final Cluster.ClusterKey clusterKey, final VServer.VServerKey key, final Collection<RawSample<LunMeasurements>> samples)
            throws DataSourceErrorException {

        reportLunPerformance(clusterKey, key, samples, leaseTimeMS);
    }

    public final void reportLunPerformance(final Cluster.ClusterKey clusterKey, final VServer.VServerKey key, final Collection<RawSample<LunMeasurements>> samples, final long leaseTime)
            throws DataSourceErrorException {

        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpLunData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        checkLunPerformanceIntegrity(key, samples);
        getStoragePerformanceListener().reportLunPerformance(this, clusterKey, key, getSampleTime(), samples, leaseTime);
        reportedLuns = true;
    }

	public final void reportLunRawPerformance(ClusterKey clusterKey, VServerKey key, Collection<RawSample<LunRawMeasurements>> samples)
			throws DataSourceErrorException {

        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpLunRawData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        getStoragePerformanceListener().reportLunRawPerformance(this, clusterKey, key, getSampleTime(), samples, leaseTimeMS);
        reportedRawLuns = true;

	}
    
    
    public final void reportFcpLifPerformance(final Cluster.ClusterKey clusterKey, 
                                              final VServer.VServerKey key, 
                                              final Collection<RawSample<FcpLifMeasurements>> samples)
            throws DataSourceErrorException {
        reportFcpLifPerformance(clusterKey, key, samples, leaseTimeMS);
    }

    public final void reportFcpLifPerformance(final Cluster.ClusterKey clusterKey, 
                                              final VServer.VServerKey key, 
                                              final Collection<RawSample<FcpLifMeasurements>> samples, 
                                              final long leaseTime)
            throws DataSourceErrorException {

        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpFcpLifData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        checkFcpLifPerformanceIntegrity(key, samples);
        getStoragePerformanceListener().reportFcpLifPerformance(this, clusterKey, key, getSampleTime(), samples, leaseTime);
        reportedFcpLifs = true;
    }

    public final void reportFcpLifRawPerformance(final Cluster.ClusterKey clusterKey, 
                                                 final VServer.VServerKey key, 
                                                 final Collection<RawSample<FcpLifRawMeasurements>> samples) throws DataSourceErrorException {
        reportFcpLifRawPerformance(clusterKey, key, samples, leaseTimeMS);
    }

    public final void reportFcpLifRawPerformance(final Cluster.ClusterKey clusterKey, 
                                                 final VServer.VServerKey key, 
                                                 final Collection<RawSample<FcpLifRawMeasurements>> samples, 
                                                 final long leaseTime)
            throws DataSourceErrorException {

        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpRawFcpLifData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        // Not needed
        //checkFcpLifPerformanceIntegrity(key, samples);
        //
        getStoragePerformanceListener().reportFcpLifRawPerformance(this, clusterKey, key, getSampleTime(), samples, leaseTime);
        reportedRawFcpLifs = true;
    }

    public final void reportFcpLifVserverPerformance(final Cluster.ClusterKey clusterKey, 
            final Collection<RawSample<FcpLifVserverMeasurements>> samples)
            		throws DataSourceErrorException {
    	reportFcpLifVserverPerformance(clusterKey,samples, leaseTimeMS);
    }

    public final void reportFcpLifVserverPerformance(final Cluster.ClusterKey clusterKey, 
            final Collection<RawSample<FcpLifVserverMeasurements>> samples, 
            final long leaseTime) throws DataSourceErrorException {

    	if (getDataSource().isStopped()) {
    		return;
    	}

    	if (isRecording() && samples.size() > 0) {
    		dumpFcpLifVserverData(clusterKey, samples);
    	}

    	if (samples.size() == 0) {
    		return;
    	}

    	getStoragePerformanceListener().reportFcpLifVserverPerformance(this, clusterKey, getSampleTime(), samples, leaseTime);
    	reportedFcpLifVservers = true;
    }

    public final void reportFcpLifVserverRawPerformance(final Cluster.ClusterKey clusterKey, 
               final Collection<RawSample<FcpLifVserverRawMeasurements>> samples) throws DataSourceErrorException {
    	reportFcpLifVserverRawPerformance(clusterKey, samples, leaseTimeMS);
    }

    public final void reportFcpLifVserverRawPerformance(final Cluster.ClusterKey clusterKey, 
               final Collection<RawSample<FcpLifVserverRawMeasurements>> samples, 
               final long leaseTime) throws DataSourceErrorException {

    	if (getDataSource().isStopped()) {
    		return;
    	}

    	if (isRecording() && samples.size() > 0) {
    		dumpRawFcpLifVserverData(clusterKey, samples);
    	}

    	if (samples.size() == 0) {
    		return;
    	}

    	getStoragePerformanceListener().reportFcpLifVserverRawPerformance(this, clusterKey, getSampleTime(), samples, leaseTime);
    	reportedRawFcpLifVservers = true;
    }

    public final void reportFcpPortPerformance(final Cluster.ClusterKey key, final Collection<RawSample<FcpPortMeasurements>> samples)
            throws DataSourceErrorException {
        reportFcpPortPerformance(key, samples, leaseTimeMS);
    }

    public final void reportFcpPortPerformance(final Cluster.ClusterKey key, final Collection<RawSample<FcpPortMeasurements>> samples, final long leaseTime)
            throws DataSourceErrorException {

        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpFcpPortData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        checkFcpPortPerformanceIntegrity(key, samples);
        getStoragePerformanceListener().reportFcpPortPerformance(this, key, getSampleTime(), samples, leaseTime);
        reportedFcpPorts = true;
    }

	public final void reportFcpPortRawPerformance(ClusterKey clusterKey, Collection<RawSample<FcpPortRawMeasurements>> samples)
			throws DataSourceErrorException {

        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpFcpPortRawData(clusterKey, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        getStoragePerformanceListener().reportFcpPortRawPerformance(this, clusterKey, getSampleTime(), samples, leaseTimeMS);
        reportedRawFcpPorts = true;

	}
    
    public final void reportIpPortPerformance(final Cluster.ClusterKey key, final Collection<RawSample<NetworkPortMeasurements>> samples)
            throws DataSourceErrorException {
        reportIpPortPerformance(key, samples, leaseTimeMS);
    }

    public final void reportIpPortPerformance(final Cluster.ClusterKey key, final Collection<RawSample<NetworkPortMeasurements>> samples, final long leaseTime)
            throws DataSourceErrorException {

        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpIpPortData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        checkIpPortPerformanceIntegrity(key, samples);
        getStoragePerformanceListener().reportIpPortPerformance(this, key, getSampleTime(), samples, leaseTime);
        reportedIpPorts = true;
    }
    
    public final void reportNicRawPerformance(final Cluster.ClusterKey key, final Collection<RawSample<NicRawMeasurements>> samples){

        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpRawNicData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        //checkNicPerformanceIntegrity(key, samples);
        getStoragePerformanceListener().reportNicRawPerformance(this, key, getSampleTime(), samples, leaseTimeMS);
        reportedRawNic = true;
    }
    
    public final void reportNicPerformance(final Cluster.ClusterKey key, final Collection<RawSample<NicMeasurements>> samples){

        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpNicData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        getStoragePerformanceListener().reportNicPerformance(this, key, getSampleTime(), samples, leaseTimeMS);
        reportedNic = true;
    }


    public final void reportVolumeVserverPerformance(final Cluster.ClusterKey key, final Collection<RawSample<VolumeVserverMeasurements>> samples)
            throws DataSourceErrorException {
        reportVolumeVserverPerformance(key, samples, leaseTimeMS);
    }

    public final void reportVolumeVserverPerformance(final Cluster.ClusterKey key, final Collection<RawSample<VolumeVserverMeasurements>> samples, final long leaseTime)
            throws DataSourceErrorException {

        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpVolumeVServerData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        getStoragePerformanceListener().reportVolumeVServerPerformance(this, key, getSampleTime(), samples, leaseTime);
        reportedVolumeVserver = true;
    }

    @Override
    public void reportVolumeNodePerformance(final Cluster.ClusterKey key, final Collection<RawSample<VolumeNodeMeasurements>> samples) throws DataSourceErrorException {
        reportVolumeNodePerformance(key, samples, -1L);
    }

    public final void reportVolumeNodePerformance(final Cluster.ClusterKey key, final Collection<RawSample<VolumeNodeMeasurements>> samples, final long leaseTime)
            throws DataSourceErrorException {
        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            //            dumpVolumeData(key, samples);       // TODO
        }

        if (samples.size() == 0) {
            return;
        }

        //        checkVolumePerformanceIntegrity(key, samples); // TODO

        if (leaseTime == -1) {
            getStoragePerformanceListener().reportVolumeNodePerformance(this, key, getSampleTime(), samples);
        }
        else {
            getStoragePerformanceListener().reportVolumeNodePerformance(this, key, getSampleTime(), samples, leaseTime);
        }

        reportedVolumes = true;

    }

    public final void reportLifPerformance(final Cluster.ClusterKey clusterKey, final VServer.VServerKey key, final Collection<RawSample<NetworkLifMeasurements>> samples)
            throws DataSourceErrorException {
        reportLifPerformance(clusterKey, key, samples, leaseTimeMS);
    }

    public final void reportLifPerformance(final Cluster.ClusterKey clusterKey, final VServer.VServerKey key, final Collection<RawSample<NetworkLifMeasurements>> samples, final long leaseTime)
            throws DataSourceErrorException {

        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpLifData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        getStoragePerformanceListener().reportLifPerformance(this, clusterKey, key, getSampleTime(), samples, leaseTime);
        reportedLifs = true;
    }

    public final void reportLifRawPerformance(final Cluster.ClusterKey clusterKey, final VServer.VServerKey key, final Collection<RawSample<NetworkLifRawMeasurements>> samples)
            throws DataSourceErrorException {
        reportLifRawPerformance(clusterKey, key, samples, leaseTimeMS);
    }

    public final void reportLifRawPerformance(final Cluster.ClusterKey clusterKey, final VServer.VServerKey key, final Collection<RawSample<NetworkLifRawMeasurements>> samples, final long leaseTime)
            throws DataSourceErrorException {

        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpLifRawData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        getStoragePerformanceListener().reportLifRawPerformance(this, clusterKey, key, getSampleTime(), samples, leaseTime);
        reportedRawLifs = true;
    }
    
    public final void reportIscsiLifPerformance(final Cluster.ClusterKey clusterKey, final VServer.VServerKey key, final Collection<RawSample<IscsiLifMeasurements>> samples)
            throws DataSourceErrorException {
        reportIscsiLifPerformance(clusterKey, key, samples, leaseTimeMS);
    }

    public final void reportIscsiLifPerformance(final Cluster.ClusterKey clusterKey, final VServer.VServerKey key, final Collection<RawSample<IscsiLifMeasurements>> samples, final long leaseTime)
            throws DataSourceErrorException {

        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpIscsiLifData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        getStoragePerformanceListener().reportIscsiLifPerformance(this, clusterKey, key, getSampleTime(), samples, leaseTime);
        reportedIscsiLifs = true;
    }

    public final void reportIscsiLifVserverPerformance(final Cluster.ClusterKey clusterKey, final Collection<RawSample<IscsiLifVserverMeasurements>> samples)
            throws DataSourceErrorException {
        reportIscsiLifVserverPerformance(clusterKey, samples, leaseTimeMS);
    }

    public final void reportIscsiLifVserverPerformance(final Cluster.ClusterKey clusterKey, final Collection<RawSample<IscsiLifVserverMeasurements>> samples, final long leaseTime)
            throws DataSourceErrorException {

        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpIscsiLifVserverData(clusterKey, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        getStoragePerformanceListener().reportIscsiLifVserverPerformance(this, clusterKey, getSampleTime(), samples, leaseTime);
        reportedIscsiLifVservers = true;
    }
    
    public final void reportIscsiLifRawPerformance(final Cluster.ClusterKey clusterKey, final VServer.VServerKey key, final Collection<RawSample<IscsiLifRawMeasurements>> samples)
            throws DataSourceErrorException {

        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpIscsiLifRawData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        getStoragePerformanceListener().reportIscsiLifRawPerformance(this, clusterKey, key, getSampleTime(), samples, leaseTimeMS);
        reportedRawIscsiLifs = true;
    }

    public final void reportIscsiLifVserverRawPerformance(final Cluster.ClusterKey clusterKey, final Collection<RawSample<IscsiLifVserverRawMeasurements>> samples)
            throws DataSourceErrorException {

        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpIscsiLifVserverRawData(clusterKey, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        getStoragePerformanceListener().reportIscsiLifVserverRawPerformance(this, clusterKey, getSampleTime(), samples, leaseTimeMS);
        reportedRawIscsiLifVservers = true;
    }
    
    public final void reportProcessorPerformance(final Cluster.ClusterKey key, final Collection<RawSample<ProcessorMeasurements>> samples) throws DataSourceErrorException {

        reportProcessorPerformance(key, samples, leaseTimeMS);
    }

    public final void reportProcessorPerformance(final Cluster.ClusterKey key, final Collection<RawSample<ProcessorMeasurements>> samples, final long leaseTime) throws DataSourceErrorException {

        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpProcessorData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        getStoragePerformanceListener().reportProcessorPerformance(this, key, getSampleTime(), samples, leaseTime);
        reportedProcessors = true;

    }

    /**
     * Used by the data source package implementation to report back storage performance information.
     * <p/>
     * <p/>This method reports data to the server seperately for each storage. It aggregates consecutive reports and makes
     * sure that each storage does not report faster than one minute.
     *
     * @param key     Key of storage report about.
     * @param samples List of samples
     *
     * @throws DataSourceErrorException in event of an integrity check failure.
     */
    public final void reportQosVolumeWorkloadPerformance(final Cluster.ClusterKey key, final Collection<RawSample<QosVolumeWorkloadMeasurements>> samples)
            throws DataSourceErrorException {

        reportQosVolumeWorkloadPerformance(key, samples, leaseTimeMS);
    }

    /**
     * Used by the data source package implementation to report back storage performance information.
     *
     * <p/>This method reports data to the server separately for each storage. It aggregates consecutive reports and makes
     * sure that each storage does not report faster than one minute.
     *
     * @param key       Key of storage report about.
     * @param samples   List of samples
     * @param leaseTime Time (milli seconds) of validity of sample (-1 will use default value).
     *
     * @throws DataSourceErrorException in event of an integrity check failure.
     */
    public final void reportQosVolumeWorkloadPerformance(final Cluster.ClusterKey key, final Collection<RawSample<QosVolumeWorkloadMeasurements>> samples, final long leaseTime)
            throws DataSourceErrorException {
        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpQosVolumeWorkloadData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        checkQosVolumeWorkloadPerformanceIntegrity(key, samples);

        if (leaseTime == -1) {
            getStoragePerformanceListener().reportQosVolumeWorkloadPerformance(this, key, getSampleTime(), samples);
        }
        else {
            getStoragePerformanceListener().reportQosVolumeWorkloadPerformance(this, key, getSampleTime(), samples, leaseTime);
        }

        reportedWorkloads = true;
    }

    /**
     * Used by the data source package implementation to report back storage performance information.
     * <p/>
     * <p/>This method reports data to the server seperately for each storage. It aggregates consecutive reports and makes
     * sure that each storage does not report faster than one minute.
     *
     * @param key     Key of storage report about.
     * @param samples List of samples
     * @throws DataSourceErrorException in event of an integrity check failure.
     */
    public final void reportQosWorkloadQueueDBladePerformance(final Cluster.ClusterKey key, final Collection<RawSample<QosWorkloadQueueDBladeMeasurements>> samples)
            throws DataSourceErrorException {
        reportQosWorkloadQueueDBladePerformance(key, samples, leaseTimeMS);
    }

    /**
     * Used by the data source package implementation to report back storage performance information.
     * <p/>
     * <p/>This method reports data to the server separately for each storage. It aggregates consecutive reports and makes
     * sure that each storage does not report faster than one minute.
     *
     * @param key       Key of storage report about.
     * @param samples   List of samples
     * @param leaseTime Time (milli seconds) of validity of sample (-1 will use default value).
     * @throws DataSourceErrorException in event of an integrity check failure.
     */
    public final void reportQosWorkloadQueueDBladePerformance(final Cluster.ClusterKey key, final Collection<RawSample<QosWorkloadQueueDBladeMeasurements>> samples, final long leaseTime)
            throws DataSourceErrorException {
        if (getDataSource().isStopped()) {
            return;
        }
        if (isRecording() && samples.size() > 0) {
            dumpQosWorkloadQueueDBladeData(key, samples);
        }
        if (samples.size() == 0) {
            return;
        }
        if (leaseTime == -1) {
            getStoragePerformanceListener().reportQosWorkloadQueueDBladePerformance(this, key, getSampleTime(), samples);
        } else {
            getStoragePerformanceListener().reportQosWorkloadQueueDBladePerformance(this, key, getSampleTime(), samples, leaseTime);
        }
        reportedWorkloads = true;
    }

    public final void reportQosWorkloadDetailPerformance(final Cluster.ClusterKey key, final Collection<RawSample<QosWorkloadDetailMeasurements>> samples)
            throws DataSourceErrorException {
    	reportQosWorkloadDetailPerformance(key, samples, leaseTimeMS);
    }

    public final void reportQosWorkloadDetailPerformance(final Cluster.ClusterKey key, final Collection<RawSample<QosWorkloadDetailMeasurements>> samples, final long leaseTime) throws DataSourceErrorException{
    	if (getDataSource().isStopped()) {
            return;
        }
        if (isRecording() && samples.size() > 0) {
            dumpQosWorkloadDetailData(key, samples);
        }
        if (samples.size() == 0) {
            return;
        }
        
        checkQosWorkloadDetailPerformanceIntegrity(key, samples);
        
        if (leaseTime == -1) {
            getStoragePerformanceListener().reportQosWorkloadDetailPerformance(this, key, getSampleTime(), samples);
        } else {
            getStoragePerformanceListener().reportQosWorkloadDetailPerformance(this, key, getSampleTime(), samples, leaseTime);
        }
        reportedWorkloads = true;
    }
    
    /**
     * Check workload detail performance integrity on attached samples.  Throws an exception in the event on an integrity failure.
     *
     * @param key     the key to the storage being reported
     * @param samples samples being reported.
     *                                                                                                                                                                                                              * @throws DataSourceErrorException in the event of an integrity failure.                                                                                                                                       */
    private void checkQosWorkloadDetailPerformanceIntegrity(final Cluster.ClusterKey key,
                                                            final Collection<RawSample<QosWorkloadDetailMeasurements>> samples) throws DataSourceErrorException {

        final NetAppQosWorkloadDetailPerformanceIntegrityChecker checker = new NetAppQosWorkloadDetailPerformanceIntegrityChecker(samples,
                                                                                                                                  key.getResourceKey(),
                                                                                                                                  logPrefix);                                                                          try {                                                                                                                                                                                                              checker.checkIntegrity();
        }
        catch (final IntegrityException ie) {
            throw new DataSourceErrorException(DataSourceErrorType.INCONSISTENT_DATA, ie.getMessage(),
                                               ie.getMessage(), ie);
        }
    }


    
    /**
     * Used by the data source package implementation to report back storage performance information.
     * <p/>
     * <p/>This method reports data to the server seperately for each storage. It aggregates consecutive reports and makes
     * sure that each storage does not report faster than one minute.
     *
     * @param key     Key of storage report about.
     * @param samples List of samples
     * @throws DataSourceErrorException in event of an integrity check failure.
     */
    public final void reportQosWorkloadQueueNBladePerformance(final Cluster.ClusterKey key, final Collection<RawSample<QosWorkloadQueueNBladeMeasurements>> samples)
            throws DataSourceErrorException {
        reportQosWorkloadQueueNBladePerformance(key, samples, leaseTimeMS);
    }

    /**
     * Used by the data source package implementation to report back storage performance information.
     * <p/>
     * <p/>This method reports data to the server separately for each storage. It aggregates consecutive reports and makes
     * sure that each storage does not report faster than one minute.
     *
     * @param key       Key of storage report about.
     * @param samples   List of samples
     * @param leaseTime Time (milli seconds) of validity of sample (-1 will use default value).
     * @throws DataSourceErrorException in event of an integrity check failure.
     */
    public final void reportQosWorkloadQueueNBladePerformance(final Cluster.ClusterKey key, final Collection<RawSample<QosWorkloadQueueNBladeMeasurements>> samples, final long leaseTime)
            throws DataSourceErrorException {
        if (getDataSource().isStopped()) {
            return;
        }
        if (isRecording() && samples.size() > 0) {
            dumpQosWorkloadQueueNBladeData(key, samples);
        }
        if (samples.size() == 0) {
            return;
        }
        if (leaseTime == -1) {
            getStoragePerformanceListener().reportQosWorkloadQueueNBladePerformance(this, key, getSampleTime(), samples);
        } else {
            getStoragePerformanceListener().reportQosWorkloadQueueNBladePerformance(this, key, getSampleTime(), samples, leaseTime);
        }
        reportedWorkloads = true;
    }

    @Override
    public void reportQosServiceCenterPerformance(final Cluster.ClusterKey key, final Collection<RawSample<QosServiceCenterMeasurements>> samples) throws DataSourceErrorException {
        reportQosServiceCenterPerformance(key, samples, leaseTimeMS);
    }

    public void reportQosServiceCenterPerformance(final Cluster.ClusterKey key, final Collection<RawSample<QosServiceCenterMeasurements>> samples,
                                                  final long leaseTime) throws DataSourceErrorException {
        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpQosServiceCenterData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        checkQosServiceCenterPerformanceIntegrity(key, samples);

        if (leaseTime == -1) {
            getStoragePerformanceListener().reportQosServiceCenterPerformance(this, key, getSampleTime(), samples);
        }
        else {
            getStoragePerformanceListener().reportQosServiceCenterPerformance(this, key, getSampleTime(), samples, leaseTime);
        }

        reportedServiceCenters = true;
    }

    @Override
    public void reportWaflPerformance(final Cluster.ClusterKey key, final Collection<RawSample<WaflMeasurements>> samples) throws DataSourceErrorException {
        reportWaflPerformance(key, samples, leaseTimeMS);
    }

    public void reportWaflPerformance(final Cluster.ClusterKey key, final Collection<RawSample<WaflMeasurements>> samples, final long leaseTime) throws DataSourceErrorException {
        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpWaflData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        checkWaflPerformanceIntegrity(key, samples);

        if (leaseTime == -1) {
            getStoragePerformanceListener().reportWaflPerformance(this, key, getSampleTime(), samples);
        }
        else {
            getStoragePerformanceListener().reportWaflPerformance(this, key, getSampleTime(), samples, leaseTime);
        }

        reportedWafl = true;
    }

    @Override
    public void reportExtCachePerformance(final Cluster.ClusterKey key, final Collection<RawSample<ExtCacheObjMeasurements>> samples) throws DataSourceErrorException {
        reportExtCachePerformance(key, samples, -1L);
    }

    public void reportExtCachePerformance(final Cluster.ClusterKey key, final Collection<RawSample<ExtCacheObjMeasurements>> samples, final long leaseTime) throws DataSourceErrorException {
        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpExtCacheObjData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        checkExtCacheObjPerformanceIntegrity(key, samples);

        if (leaseTime == -1) {
            getStoragePerformanceListener().reportExtCachePerformance(this, key, getSampleTime(), samples);
        }
        else {
            getStoragePerformanceListener().reportExtCachePerformance(this, key, getSampleTime(), samples, leaseTime);
        }

        reportedExtCache = true;

    }


    @Override
    public void reportWaflHyaAggrPerformance(final Cluster.ClusterKey key, final Collection<RawSample<WaflHyaAggrMeasurements>> samples) throws DataSourceErrorException {
        reportWaflHyaAggrPerformance(key, samples, leaseTimeMS);
    }

    public void reportWaflHyaAggrPerformance(final Cluster.ClusterKey key, final Collection<RawSample<WaflHyaAggrMeasurements>> samples, final long leaseTime) throws DataSourceErrorException {
        {
            if (getDataSource().isStopped()) {
                return;
            }

            if (isRecording() && samples.size() > 0) {
                dumpWaflHyaAggrData(key, samples);
            }

            if (samples.size() == 0) {
                return;
            }

            checkWaflHyaAggrPerformanceIntegrity(key, samples);

            if (leaseTime == -1) {
                getStoragePerformanceListener().reportWaflHyaAggrPerformance(this, key, getSampleTime(), samples);
            }
            else {
                getStoragePerformanceListener().reportWaflHyaAggrPerformance(this, key, getSampleTime(), samples, leaseTime);
            }

            reportedWaflHyaAggr = true;
        }
    }

    /**
     * @return the server time (calculated based on offset between acquisition clock and server clock)
     */
    public final long getCurrentServerTime() {
        try {
            return clock.getServerTime();
        }
        catch (FrameworkException fe) {
            LOG.warn("Could not retrieve server time.  Using local time for sample.");
            return System.currentTimeMillis();
        }
    }

    /**
     * This method lets a datasource to create its own StorageContext - a class that used to
     * store and process foundation data used in performance
     *
     * <p/>Usually a derived class would simply use either {@link NetAppDefaultStorageContext} or {@link NetAppEmptyStorageContext},
     * but it may return a different class, with its own foundation data processing.
     *
     * @param thePackage Package associated with this context
     * @param key        Storage key
     *
     * @return Context created for this storage
     */
    protected abstract CTX createStorageContext(final NetAppStoragePerformancePackage thePackage, final Cluster.ClusterKey key);

    @Override
    public void finishedSample(BaseDataSource dataSource, PollingPackage aPackage) throws FrameworkException {
        super.finishedSample(dataSource, aPackage);
        // Foundation re-runs the same sample over and over,
        // Performance should increment sample after reading.
        if (!isSimulating()) {
            return;
        }
        if (!PACKAGE.equals(aPackage.getPackageName())) {
            return;
        }
        if (isWaitingForFoundationInitialization()) {
            // Waiting for foundation, haven't read any perf yet, don't increment
            return;
        }
        final DSSessionFileReader simulationReader = aPackage.getConfiguration().getSimulationSession();
        simulationReader.nextSample();
    }

    /**
     * Method for polling performance related data. This method is called by the base class on a designated thread and after
     * proper class loader initialization took place.
     *
     * <p/>Note: When a data source encounters an error during the acquisition, it should throw a {@link DataSourceErrorException}.
     *
     * @param dataSource Data source associated with this package
     *
     * @throws FrameworkException       when an unexpected error occurred (e.g. communication problem).
     * @throws DataSourceErrorException when an error occurs during the sample (this error contains information needed by the framework to create an audit update)
     */
    protected abstract void doArrayPerformance(final BaseDataSource dataSource) throws FrameworkException, DataSourceErrorException;

    /**
     * Called by the framework when package is required to initialize (data source is starting)
     *
     * @throws FrameworkException       when an unexpected error occurred (e.g. communication problem).
     * @throws DataSourceErrorException when an error occurs during the start (this error contains information needed by the framework to create an audit update)
     */
    protected abstract void doStart() throws FrameworkException, DataSourceErrorException;

    /**
     * Called by the framework when the package is supposed to stop (data source is stopping)
     *
     * @throws FrameworkException       when an unexpected error occurred (e.g. communication problem).
     * @throws DataSourceErrorException when an error occurs during the stop (this error contains information needed by the framework to create an audit update)
     */
    protected abstract void doStop() throws FrameworkException, DataSourceErrorException;

    public NetAppStoragePerformanceUpdatesListener getStoragePerformanceListener() {
        return storagePerformanceListener;
    }

    void dumpRawAggregateData(final Cluster.ClusterKey key, final Collection<RawSample<AggregateRawMeasurements>> samples) {
        DSFileWriter aggregateRawDataFile = null;
        try {
            aggregateRawDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                            DataSourceOutputFileType.perfdata,
                                                                                            key.getOcieUUID() + "_aggregateRawData",
                                                                                            "txt");
            final File tempFile = new File(aggregateRawDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "aggregateRawData", key.toString());
            aggregateRawDataFile.writeln("AggregateID;cp_read_blocks;cp_reads;total_transfers;user_read_blocks;user_reads;user_write_blocks;user_writes;" +
                    "total_transfers_hdd ;total_transfers_ssd; user_read_blocks_hdd;user_read_blocks_ssd;user_reads_hdd;user_reads_ssd;user_write_blocks_hdd;user_write_blocks_ssd;" +
                    "user_writes_hdd; user_writes_ssd;");

            for (final RawSample<AggregateRawMeasurements> sample : samples) {
                aggregateRawDataFile.append(sample.getKey()).append(';');
                final AggregateRawMeasurements aggregateRawMeasurements = sample.getCounters();
                if (aggregateRawMeasurements != null) {
                	aggregateRawDataFile.append(String.valueOf(aggregateRawMeasurements.getCpReadBlocks())).append(';');
                	aggregateRawDataFile.append(String.valueOf(aggregateRawMeasurements.getCpReads())).append(';');
                	aggregateRawDataFile.append(String.valueOf(aggregateRawMeasurements.getTotalTransfers())).append(';');
                	aggregateRawDataFile.append(String.valueOf(aggregateRawMeasurements.getUserReadBlocks())).append(';');
                	aggregateRawDataFile.append(String.valueOf(aggregateRawMeasurements.getUserReads())).append(';');
                	aggregateRawDataFile.append(String.valueOf(aggregateRawMeasurements.getUserWriteBlocks())).append(';');
                	aggregateRawDataFile.append(String.valueOf(aggregateRawMeasurements.getUserWrites())).append(';');
                    aggregateRawDataFile.append(String.valueOf(aggregateRawMeasurements.getTotalTransfersHdd())).append(';');
                    aggregateRawDataFile.append(String.valueOf(aggregateRawMeasurements.getTotalTransfersSsd())).append(';');
                    aggregateRawDataFile.append(String.valueOf(aggregateRawMeasurements.getUserReadBlocksHdd())).append(';');
                    aggregateRawDataFile.append(String.valueOf(aggregateRawMeasurements.getUserReadBlocksSsd())).append(';');
                    aggregateRawDataFile.append(String.valueOf(aggregateRawMeasurements.getUserReadsHdd())).append(';');
                    aggregateRawDataFile.append(String.valueOf(aggregateRawMeasurements.getUserReadsSsd())).append(';');
                    aggregateRawDataFile.append(String.valueOf(aggregateRawMeasurements.getUserWriteBlocksHdd())).append(';');
                    aggregateRawDataFile.append(String.valueOf(aggregateRawMeasurements.getUserWriteBlocksSsd())).append(';');
                    aggregateRawDataFile.append(String.valueOf(aggregateRawMeasurements.getUserWritesHdd())).append(';');
                    aggregateRawDataFile.append(String.valueOf(aggregateRawMeasurements.getUserWritesSsd())).append(';');
                }
                aggregateRawDataFile.append(crlf);
            }
            aggregateRawDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (aggregateRawDataFile != null) {
                try {
                    aggregateRawDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }

    void dumpAggregateData(final Cluster.ClusterKey key, final Collection<RawSample<AggregateMeasurements>> samples) {
        DSFileWriter aggregateDataFile = null;
        try {
        	aggregateDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                            DataSourceOutputFileType.perfdata,
                                                                                            key.getOcieUUID() + "_aggregateData",
                                                                                            "txt");
            final File tempFile = new File(aggregateDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "aggregateData", key.toString());
            aggregateDataFile.writeln("AggregateID;cp_read_blocks;cp_reads;total_transfers;user_read_blocks;user_reads;user_write_blocks;user_writes" +
                    "total_transfers_hdd ;total_transfers_ssd; user_read_blocks_hdd;user_read_blocks_ssd;user_reads_hdd;user_reads_ssd;user_write_blocks_hdd;user_write_blocks_ssd;" +
                    "user_writes_hdd; user_writes_ssd;");
            for (final RawSample<AggregateMeasurements> sample : samples) {
                aggregateDataFile.append(sample.getKey()).append(';');
                final AggregateMeasurements aggregateMeasurements = sample.getCounters();
                if (aggregateMeasurements != null) {
                	aggregateDataFile.append(String.valueOf(aggregateMeasurements.getCpReadBlocks())).append(';');
                	aggregateDataFile.append(String.valueOf(aggregateMeasurements.getCpReads())).append(';');
                	aggregateDataFile.append(String.valueOf(aggregateMeasurements.getTotalTransfers())).append(';');
                	aggregateDataFile.append(String.valueOf(aggregateMeasurements.getUserReadBlocks())).append(';');
                	aggregateDataFile.append(String.valueOf(aggregateMeasurements.getUserReads())).append(';');
                	aggregateDataFile.append(String.valueOf(aggregateMeasurements.getUserWriteBlocks())).append(';');
                	aggregateDataFile.append(String.valueOf(aggregateMeasurements.getUserWrites())).append(';');
                    aggregateDataFile.append(String.valueOf(aggregateMeasurements.getTotalTransfersHdd())).append(';');
                    aggregateDataFile.append(String.valueOf(aggregateMeasurements.getTotalTransfersSsd())).append(';');
                    aggregateDataFile.append(String.valueOf(aggregateMeasurements.getUserReadBlocksHdd())).append(';');
                    aggregateDataFile.append(String.valueOf(aggregateMeasurements.getUserReadBlocksSsd())).append(';');
                    aggregateDataFile.append(String.valueOf(aggregateMeasurements.getUserReadsHdd())).append(';');
                    aggregateDataFile.append(String.valueOf(aggregateMeasurements.getUserReadsSsd())).append(';');
                    aggregateDataFile.append(String.valueOf(aggregateMeasurements.getUserWriteBlocksHdd())).append(';');
                    aggregateDataFile.append(String.valueOf(aggregateMeasurements.getUserWriteBlocksSsd())).append(';');
                    aggregateDataFile.append(String.valueOf(aggregateMeasurements.getUserWritesHdd())).append(';');
                    aggregateDataFile.append(String.valueOf(aggregateMeasurements.getUserWritesSsd())).append(';');
                }
                aggregateDataFile.append(crlf);
            }
            aggregateDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (aggregateDataFile != null) {
                try {
                    aggregateDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }
    
    void dumpRawNfsv3Data(final Cluster.ClusterKey key, final Collection<RawSample<Nfsv3RawMeasurements>> samples) {
        DSFileWriter nfsv3RawDataFile = null;
        try {
            nfsv3RawDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                            DataSourceOutputFileType.perfdata,
                                                                                            key.getOcieUUID() + "_nfsv3RawData",
                                                                                            "txt");
            final File tempFile = new File(nfsv3RawDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "nfsv3RawData", key.toString());
            nfsv3RawDataFile.writeln("Nfsv3ID;nfsv3_latency_hist;nfsv3_ops;nfsv3_read_ops;nfsv3_write_ops;read_total;nfsv3_read_throughput;nfsv3_throughput;write_total;nfsv3_write_throughput;read_avg_latency;write_avg_latency;latency;total_ops");

            for (final RawSample<Nfsv3RawMeasurements> sample : samples) {
                nfsv3RawDataFile.append(sample.getKey()).append(';');
                final Nfsv3RawMeasurements rawMeasurements = sample.getCounters();
                if (rawMeasurements != null) {
                    nfsv3RawDataFile.append(String.valueOf(rawMeasurements.getNfsv3Ops())).append(';');
                    nfsv3RawDataFile.append(String.valueOf(rawMeasurements.getNfsv3ReadOps())).append(';');
                    nfsv3RawDataFile.append(String.valueOf(rawMeasurements.getNfsv3WriteOps())).append(';');
                    nfsv3RawDataFile.append(String.valueOf(rawMeasurements.getReadTotal())).append(';');
                    nfsv3RawDataFile.append(String.valueOf(rawMeasurements.getNfsv3ReadThroughput())).append(';');
                    nfsv3RawDataFile.append(String.valueOf(rawMeasurements.getNfsv3Throughput())).append(';');
                    nfsv3RawDataFile.append(String.valueOf(rawMeasurements.getWriteTotal())).append(';');
                    nfsv3RawDataFile.append(String.valueOf(rawMeasurements.getNfsv3WriteThroughput())).append(';');
                    nfsv3RawDataFile.append(String.valueOf(rawMeasurements.getReadAvgLatency())).append(';');
                    nfsv3RawDataFile.append(String.valueOf(rawMeasurements.getWriteAvgLatency())).append(';');
                    nfsv3RawDataFile.append(String.valueOf(rawMeasurements.getLatency())).append(';');
                    nfsv3RawDataFile.append(String.valueOf(rawMeasurements.getTotalOps())).append(';');
                }
                nfsv3RawDataFile.append(crlf);
            }
            nfsv3RawDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            try {
                if (nfsv3RawDataFile != null) {
                    nfsv3RawDataFile.close();
                }
            }
            catch (IOException ioe) {
                //fall through
            }
        }
    }

    void dumpNfsv3Data(final Cluster.ClusterKey key, final Collection<RawSample<Nfsv3Measurements>> samples) {
        DSFileWriter nfsv3DataFile = null;
        try {
            nfsv3DataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                            DataSourceOutputFileType.perfdata,
                                                                                            key.getOcieUUID() + "_nfsv3Data",
                                                                                            "txt");
            final File tempFile = new File(nfsv3DataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "nfsv3Data", key.toString());
            nfsv3DataFile.writeln("Nfsv3ID;nfsv3_ops;nfsv3_read_ops;nfsv3_write_ops;read_total;nfsv3_read_throughput;nfsv3_throughput;write_total;nfsv3_write_throughput;read_avg_latency;write_avg_latency;latency;total_ops");

            for (final RawSample<Nfsv3Measurements> sample : samples) {
                nfsv3DataFile.append(sample.getKey()).append(';');
                final Nfsv3Measurements measurements = sample.getCounters();
                if (measurements != null) {
                    nfsv3DataFile.append(String.valueOf(measurements.getNfsv3Ops())).append(';');
                    nfsv3DataFile.append(String.valueOf(measurements.getNfsv3ReadOps())).append(';');
                    nfsv3DataFile.append(String.valueOf(measurements.getNfsv3WriteOps())).append(';');
                    nfsv3DataFile.append(String.valueOf(measurements.getReadTotal())).append(';');
                    nfsv3DataFile.append(String.valueOf(measurements.getNfsv3ReadThroughput())).append(';');
                    nfsv3DataFile.append(String.valueOf(measurements.getNfsv3Throughput())).append(';');
                    nfsv3DataFile.append(String.valueOf(measurements.getWriteTotal())).append(';');
                    nfsv3DataFile.append(String.valueOf(measurements.getNfsv3WriteThroughput())).append(';');
                    nfsv3DataFile.append(String.valueOf(measurements.getReadAvgLatency())).append(';');
                    nfsv3DataFile.append(String.valueOf(measurements.getWriteAvgLatency())).append(';');
                    nfsv3DataFile.append(String.valueOf(measurements.getLatency())).append(';');
                    nfsv3DataFile.append(String.valueOf(measurements.getTotalOps())).append(';');
                }
                nfsv3DataFile.append(crlf);
            }
            nfsv3DataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            try {
                if (nfsv3DataFile != null) {
                    nfsv3DataFile.close();
                }
            }
            catch (IOException ioe) {
                //fall through
            }
        }
    }

    void dumpRawNfsv4Data(final Cluster.ClusterKey key, final Collection<RawSample<Nfsv4RawMeasurements>> samples) {
        DSFileWriter nfsv4RawDataFile = null;
        try {
            nfsv4RawDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                            DataSourceOutputFileType.perfdata,
                                                                                            key.getOcieUUID() + "_nfsv4RawData",
                                                                                            "txt");
            final File tempFile = new File(nfsv4RawDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "nfsv4RawData", key.toString());
            nfsv4RawDataFile.writeln("Nfsv4ID;nfs4_latency_hist;read_avg_latency;read_total;nfs4_read_throughput;nfs4_throughput;write_avg_latency;write_total;nfs4_write_throughput;nfsv4_ops;latency;total_ops");

            for (final RawSample<Nfsv4RawMeasurements> sample : samples) {
                nfsv4RawDataFile.append(sample.getKey()).append(';');
                final Nfsv4RawMeasurements rawMeasurements = sample.getCounters();
                if (rawMeasurements != null) {
                    nfsv4RawDataFile.append(String.valueOf(rawMeasurements.getReadAvgLatency())).append(';');
                    nfsv4RawDataFile.append(String.valueOf(rawMeasurements.getReadTotal())).append(';');
                    nfsv4RawDataFile.append(String.valueOf(rawMeasurements.getNfs4ReadThroughput())).append(';');
                    nfsv4RawDataFile.append(String.valueOf(rawMeasurements.getNfs4Throughput())).append(';');
                    nfsv4RawDataFile.append(String.valueOf(rawMeasurements.getWriteAvgLatency())).append(';');
                    nfsv4RawDataFile.append(String.valueOf(rawMeasurements.getWriteTotal())).append(';');
                    nfsv4RawDataFile.append(String.valueOf(rawMeasurements.getNfs4WriteThroughput())).append(';');
                    nfsv4RawDataFile.append(String.valueOf(rawMeasurements.getNfsv4Ops())).append(';');
                    nfsv4RawDataFile.append(String.valueOf(rawMeasurements.getLatency())).append(';');
                    nfsv4RawDataFile.append(String.valueOf(rawMeasurements.getTotalOps())).append(';');
                }
                nfsv4RawDataFile.append(crlf);
            }
            nfsv4RawDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            try {
                if (nfsv4RawDataFile != null) {
                    nfsv4RawDataFile.close();
                }
            }
            catch (IOException ioe) {
                //fall through
            }
        }
    }
    
    void dumpNfsv4Data(final Cluster.ClusterKey key, final Collection<RawSample<Nfsv4Measurements>> samples) {
        DSFileWriter nfsv4DataFile = null;
        try {
            nfsv4DataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                            DataSourceOutputFileType.perfdata,
                                                                                            key.getOcieUUID() + "_nfsv4Data",
                                                                                            "txt");
            final File tempFile = new File(nfsv4DataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "nfsv4Data", key.toString());
            nfsv4DataFile.writeln("Nfsv4ID;read_avg_latency;read_total;nfs4_read_throughput;nfs4_throughput;write_avg_latency;write_total;nfs4_write_throughput;nfsv4_ops;latency;total_ops");

            for (final RawSample<Nfsv4Measurements> sample : samples) {
                nfsv4DataFile.append(sample.getKey()).append(';');
                final Nfsv4Measurements measurements = sample.getCounters();
                if (measurements != null) {
                    nfsv4DataFile.append(String.valueOf(measurements.getReadAvgLatency())).append(';');
                    nfsv4DataFile.append(String.valueOf(measurements.getReadTotal())).append(';');
                    nfsv4DataFile.append(String.valueOf(measurements.getNfsv4ReadThroughput())).append(';');
                    nfsv4DataFile.append(String.valueOf(measurements.getNfsv4Throughput())).append(';');
                    nfsv4DataFile.append(String.valueOf(measurements.getWriteAvgLatency())).append(';');
                    nfsv4DataFile.append(String.valueOf(measurements.getWriteTotal())).append(';');
                    nfsv4DataFile.append(String.valueOf(measurements.getNfsv4WriteThroughput())).append(';');
                    nfsv4DataFile.append(String.valueOf(measurements.getNfsv4Ops())).append(';');
                    nfsv4DataFile.append(String.valueOf(measurements.getLatency())).append(';');
                    nfsv4DataFile.append(String.valueOf(measurements.getTotal_ops())).append(';');
                }
                nfsv4DataFile.append(crlf);
            }
            nfsv4DataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            try {
                if (nfsv4DataFile != null) {
                    nfsv4DataFile.close();
                }
            }
            catch (IOException ioe) {
                //fall through
            }
        }
    }


    void dumpRawNfsv41Data(final Cluster.ClusterKey key, final Collection<RawSample<Nfsv41RawMeasurements>> samples) {
        DSFileWriter nfsv41RawDataFile = null;
        try {
            nfsv41RawDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                            DataSourceOutputFileType.perfdata,
                                                                                            key.getOcieUUID() + "_nfsv41RawData",
                                                                                            "txt");
            final File tempFile = new File(nfsv41RawDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "nfsv41RawData", key.toString());
            nfsv41RawDataFile.writeln("Nfsv41ID;nfs41_latency_hist;read_avg_latency;read_total;nfs41_read_throughput;nfs41_throughput;write_avg_latency;write_total;nfs41_write_throughput;nfsv41_ops;latency;total_ops");

            for (final RawSample<Nfsv41RawMeasurements> sample : samples) {
                nfsv41RawDataFile.append(sample.getKey()).append(';');
                final Nfsv41RawMeasurements rawMeasurements = sample.getCounters();
                if (rawMeasurements != null) {
                    nfsv41RawDataFile.append(String.valueOf(rawMeasurements.getReadAvgLatency())).append(';');
                    nfsv41RawDataFile.append(String.valueOf(rawMeasurements.getReadTotal())).append(';');
                    nfsv41RawDataFile.append(String.valueOf(rawMeasurements.getNfs41ReadThroughput())).append(';');
                    nfsv41RawDataFile.append(String.valueOf(rawMeasurements.getNfs41Throughput())).append(';');
                    nfsv41RawDataFile.append(String.valueOf(rawMeasurements.getWriteAvgLatency())).append(';');
                    nfsv41RawDataFile.append(String.valueOf(rawMeasurements.getWriteTotal())).append(';');
                    nfsv41RawDataFile.append(String.valueOf(rawMeasurements.getNfs41WriteThroughput())).append(';');
                    nfsv41RawDataFile.append(String.valueOf(rawMeasurements.getNfsv41Ops())).append(';');
                    nfsv41RawDataFile.append(String.valueOf(rawMeasurements.getLatency())).append(';');
                    nfsv41RawDataFile.append(String.valueOf(rawMeasurements.getTotalOps())).append(';');
                }
                nfsv41RawDataFile.append(crlf);
            }
            nfsv41RawDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            try {
                if (nfsv41RawDataFile != null) {
                    nfsv41RawDataFile.close();
                }
            }
            catch (IOException ioe) {
                //fall through
            }
        }
    }
    
    void dumpNfsv41Data(final Cluster.ClusterKey key, final Collection<RawSample<Nfsv41Measurements>> samples) {
        DSFileWriter nfsv41DataFile = null;
        try {
            nfsv41DataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                            DataSourceOutputFileType.perfdata,
                                                                                            key.getOcieUUID() + "_nfsv41Data",
                                                                                            "txt");
            final File tempFile = new File(nfsv41DataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "nfsv41Data", key.toString());
            nfsv41DataFile.writeln("Nfsv41ID;nfs41_latency_hist;read_avg_latency;read_total;nfs41_read_throughput;nfs41_throughput;write_avg_latency;write_total;nfs41_write_throughput;nfsv41_ops;latency;total_ops");

            for (final RawSample<Nfsv41Measurements> sample : samples) {
                nfsv41DataFile.append(sample.getKey()).append(';');
                final Nfsv41Measurements measurements = sample.getCounters();
                if (measurements != null) {
                    nfsv41DataFile.append(String.valueOf(measurements.getReadAvgLatency())).append(';');
                    nfsv41DataFile.append(String.valueOf(measurements.getReadTotal())).append(';');
                    nfsv41DataFile.append(String.valueOf(measurements.getNfsv41ReadThroughput())).append(';');
                    nfsv41DataFile.append(String.valueOf(measurements.getNfsv41Throughput())).append(';');
                    nfsv41DataFile.append(String.valueOf(measurements.getWriteAvgLatency())).append(';');
                    nfsv41DataFile.append(String.valueOf(measurements.getWriteTotal())).append(';');
                    nfsv41DataFile.append(String.valueOf(measurements.getNfsv41WriteThroughput())).append(';');
                    nfsv41DataFile.append(String.valueOf(measurements.getNfsv41Ops())).append(';');
                    nfsv41DataFile.append(String.valueOf(measurements.getLatency())).append(';');
                    nfsv41DataFile.append(String.valueOf(measurements.getTotalOps())).append(';');
                }
                nfsv41DataFile.append(crlf);
            }
            nfsv41DataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            try {
                if (nfsv41DataFile != null) {
                    nfsv41DataFile.close();
                }
            }
            catch (IOException ioe) {
                //fall through
            }
        }
    }

    void dumpNfsData(final Cluster.ClusterKey key, final Collection<RawSample<NfsMeasurements>> samples) {
        DSFileWriter nfsDataFile = null;
        try {
            nfsDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                            DataSourceOutputFileType.perfdata,
                                                                                            key.getOcieUUID() + "_nfsData",
                                                                                            "txt");
            final File tempFile = new File(nfsDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "nfsData", key.toString());
            nfsDataFile.writeln("vServerUUID;nfs_ops;nfs_read_ops;nfs_write_ops;nfs_throughput;nfs_read_throughput;" +
                    "nfs_write_throughput;nfs_read_latency;nfs_write_latency");

            for (final RawSample<NfsMeasurements> sample : samples) {
                nfsDataFile.append(sample.getKey()).append(';');
                final NfsMeasurements rawMeasurements = sample.getCounters();
                if (rawMeasurements != null) {
                    nfsDataFile.append(String.valueOf(rawMeasurements.getOpmNfsOps())).append(';');
                    nfsDataFile.append(String.valueOf(rawMeasurements.getOpmNfsReadOps())).append(';');
                    nfsDataFile.append(String.valueOf(rawMeasurements.getOpmNfsWriteOps())).append(';');
                    nfsDataFile.append(String.valueOf(rawMeasurements.getOpmNfsThroughput())).append(';');
                    nfsDataFile.append(String.valueOf(rawMeasurements.getOpmNfsReadThroughput())).append(';');
                    nfsDataFile.append(String.valueOf(rawMeasurements.getOpmNfsWriteThroughput())).append(';');
                    nfsDataFile.append(String.valueOf(rawMeasurements.getOpmNfsReadLatency())).append(';');
                    nfsDataFile.append(String.valueOf(rawMeasurements.getOpmNfsWriteLatency())).append(';');
                }
                nfsDataFile.append(crlf);
            }
            nfsDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            try {
                if (nfsDataFile != null) {
                    nfsDataFile.close();
                }
            }
            catch (IOException ioe) {
                //fall through
            }
        }
    }

    void dumpVserverData(final Cluster.ClusterKey key, final Collection<RawSample<VserverMeasurements>> samples) {
        DSFileWriter vserverDataFile = null;
        try {
            vserverDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                    DataSourceOutputFileType.perfdata,
                    key.getOcieUUID() + "_vserverData",
                    "txt");
            final File tempFile = new File(vserverDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "nfsData", key.toString());
            vserverDataFile.writeln("vServerUUID;ops;read_ops;write_ops;throughput;read_throughput;" +
                    "write_throughput;read_latency;write_latency");

            for (final RawSample<VserverMeasurements> sample : samples) {
                vserverDataFile.append(sample.getKey()).append(';');
                final VserverMeasurements rawMeasurements = sample.getCounters();
                if (rawMeasurements != null) {
                    vserverDataFile.append(String.valueOf(rawMeasurements.getOpmOps())).append(';');
                    vserverDataFile.append(String.valueOf(rawMeasurements.getOpmReadOps())).append(';');
                    vserverDataFile.append(String.valueOf(rawMeasurements.getOpmWriteOps())).append(';');
                    vserverDataFile.append(String.valueOf(rawMeasurements.getOpmThroughput())).append(';');
                    vserverDataFile.append(String.valueOf(rawMeasurements.getOpmReadThroughput())).append(';');
                    vserverDataFile.append(String.valueOf(rawMeasurements.getOpmWriteThroughput())).append(';');
                    vserverDataFile.append(String.valueOf(rawMeasurements.getOpmReadLatency())).append(';');
                    vserverDataFile.append(String.valueOf(rawMeasurements.getOpmWriteLatency())).append(';');
                }
                vserverDataFile.append(crlf);
            }
            vserverDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            try {
                if (vserverDataFile != null) {
                    vserverDataFile.close();
                }
            }
            catch (IOException ioe) {
                //fall through
            }
        }
    }

    void dumpRawCifsVserverData(final Cluster.ClusterKey key, final Collection<RawSample<CifsVserverRawMeasurements>> samples) {
        DSFileWriter rawDataFile = null;
        try {
        	rawDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                            DataSourceOutputFileType.perfdata,
                                                                                            key.getOcieUUID() + "_cifsVserverRawData",
                                                                                            "txt");
            final File tempFile = new File(rawDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "cifsVserverRawData", key.toString());
            rawDataFile.writeln("CifsVserverID;cifs_latency;cifs_latency_base;cifs_ops;cifs_read_latency;cifs_read_ops;cifs_write_latency;cifs_write_ops;cifs_read_size_histo;cifs_write_size_histo");

            for (final RawSample<CifsVserverRawMeasurements> sample : samples) {
            	rawDataFile.append(sample.getKey()).append(';');
                final CifsVserverRawMeasurements rawMeasurements = sample.getCounters();
                if (rawMeasurements != null) {
                	rawDataFile.append(String.valueOf(rawMeasurements.getCifsLatency())).append(';');
                	rawDataFile.append(String.valueOf(rawMeasurements.getCifsLatencyBase())).append(';');
                	rawDataFile.append(String.valueOf(rawMeasurements.getCifsOps())).append(';');
                	rawDataFile.append(String.valueOf(rawMeasurements.getCifsReadLatency())).append(';');
                	rawDataFile.append(String.valueOf(rawMeasurements.getCifsReadOps())).append(';');
                	rawDataFile.append(String.valueOf(rawMeasurements.getCifsWriteLatency())).append(';');
                	rawDataFile.append(String.valueOf(rawMeasurements.getCifsWriteOps())).append(';');
                	rawDataFile.append(String.valueOf(rawMeasurements.getCifsReadSizeHisto())).append(';');
                	rawDataFile.append(String.valueOf(rawMeasurements.getCifsWriteSizeHisto())).append(';');
                }
                rawDataFile.append(crlf);
            }
            rawDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            try {
                if (rawDataFile != null) {
                	rawDataFile.close();
                }
            }
            catch (IOException ioe) {
                //fall through
            }
        }
    }
    
    void dumpCifsVserverData(final Cluster.ClusterKey key, final Collection<RawSample<CifsVserverMeasurements>> samples) {
        DSFileWriter rawDataFile = null;
        try {
            rawDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                            DataSourceOutputFileType.perfdata,
                                                                                            key.getOcieUUID() + "_cifsVserverData",
                                                                                            "txt");
            final File tempFile = new File(rawDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "cifsVserverRawData", key.toString());
            rawDataFile.writeln("CifsVserverID;cifs_latency;cifs_ops;cifs_read_latency;cifs_read_ops;cifs_write_latency;cifs_write_ops");

            for (final RawSample<CifsVserverMeasurements> sample : samples) {
                rawDataFile.append(sample.getKey()).append(';');
                final CifsVserverMeasurements measurements = sample.getCounters();
                if (measurements != null) {
                    rawDataFile.append(String.valueOf(measurements.getCifsLatency())).append(';');
                    rawDataFile.append(String.valueOf(measurements.getCifsOps())).append(';');
                    rawDataFile.append(String.valueOf(measurements.getCifsReadLatency())).append(';');
                    rawDataFile.append(String.valueOf(measurements.getCifsReadOps())).append(';');
                    rawDataFile.append(String.valueOf(measurements.getCifsWriteLatency())).append(';');
                    rawDataFile.append(String.valueOf(measurements.getCifsWriteOps())).append(';');
                }
                rawDataFile.append(crlf);
            }
            rawDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            try {
                if (rawDataFile != null) {
                    rawDataFile.close();
                }
            }
            catch (IOException ioe) {
                //fall through
            }
        }
    }
    
    void dumpDiskData(final Cluster.ClusterKey key, final Collection<RawSample<DiskMeasurements>> samples) {
        DSFileWriter diskDataFile = null;
        try {
            diskDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                            DataSourceOutputFileType.perfdata,
                                                                                            key.getOcieUUID() + "_diskdata",
                                                                                            "txt");
            final File tempFile = new File(diskDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "diskData", key.toString());
            diskDataFile.writeln("DiskID;base_for_disk_busy;cp_read_blocks;cp_read_latency;cp_reads;disk_busy;guarenteed_read_blocks;" +
                                 "guarenteed_read_latency;guarenteed_reads;guarenteed_write_blocks;guarenteed_write_latency;guarenteed_writes;" +
                                 "total_transfers;user_read_blocks;user_read_latency;user_reads;user_write_blocks;" +
                                 "user_write_latency;user_writes;"
            );
            for (final RawSample<DiskMeasurements> sample : samples) {
                diskDataFile.append(sample.getKey()).append(';');
                final DiskMeasurements diskMeasurements = sample.getCounters();
                if (diskMeasurements != null) {
                    diskDataFile.append(String.valueOf(diskMeasurements.getCpReadBlocks())).append(';');
                    diskDataFile.append(String.valueOf(diskMeasurements.getCpReadLatency())).append(';');
                    diskDataFile.append(String.valueOf(diskMeasurements.getCpReads())).append(';');
                    diskDataFile.append(String.valueOf(diskMeasurements.getDiskBusy())).append(';');
                    diskDataFile.append(String.valueOf(diskMeasurements.getGuaranteedReadBlocks())).append(';');
                    diskDataFile.append(String.valueOf(diskMeasurements.getGuaranteedReadLatency())).append(';');
                    diskDataFile.append(String.valueOf(diskMeasurements.getGuaranteedReads())).append(';');
                    diskDataFile.append(String.valueOf(diskMeasurements.getGuaranteedWriteBlocks())).append(';');
                    diskDataFile.append(String.valueOf(diskMeasurements.getGuaranteedWriteLatency())).append(';');
                    diskDataFile.append(String.valueOf(diskMeasurements.getGuaranteedWrites())).append(';');
                    diskDataFile.append(String.valueOf(diskMeasurements.getTotalTransfers())).append(';');
                    diskDataFile.append(String.valueOf(diskMeasurements.getUserReadBlocks())).append(';');
                    diskDataFile.append(String.valueOf(diskMeasurements.getUserReadLatency())).append(';');
                    diskDataFile.append(String.valueOf(diskMeasurements.getUserReads())).append(';');
                    diskDataFile.append(String.valueOf(diskMeasurements.getUserWriteBlocks())).append(';');
                    diskDataFile.append(String.valueOf(diskMeasurements.getUserWriteLatency())).append(';');
                    diskDataFile.append(String.valueOf(diskMeasurements.getUserWrites())).append(';');
                }
                diskDataFile.append(crlf);
            }
            diskDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (diskDataFile != null) {
                try {
                    diskDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }
    
    @SuppressWarnings({"ConstantConditions"})
    void dumpRawDiskData(final Cluster.ClusterKey key, final Collection<RawSample<DiskRawMeasurements>> samples) {
        DSFileWriter diskDataFile = null;
        try {
            diskDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                            DataSourceOutputFileType.perfdata,
                                                                                            key.getOcieUUID() + "_diskrawdata",
                                                                                            "txt");
            final File tempFile = new File(diskDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "diskRawData", key.toString());
            diskDataFile.writeln("DiskID;base_for_disk_busy;cp_read_blocks;cp_read_latency;cp_reads;disk_busy;guarenteed_read_blocks;" +
                                 "guarenteed_read_latency;guarenteed_reads;guarenteed_write_blocks;guarenteed_write_latency;guarenteed_writes;" +
                                 "total_transfers;user_read_blocks;user_read_latency;user_reads;user_write_blocks;" +
                                 "user_write_latency;user_writes;"
            );
            for (final RawSample<DiskRawMeasurements> sample : samples) {
                diskDataFile.append(sample.getKey()).append(';');
                final DiskRawMeasurements diskRawMeasurements = sample.getCounters();
                if (diskRawMeasurements != null) {
                    diskDataFile.append(String.valueOf(diskRawMeasurements.getBaseForDiskBusy())).append(';');
                    diskDataFile.append(String.valueOf(diskRawMeasurements.getCpReadBlocks())).append(';');
                    diskDataFile.append(String.valueOf(diskRawMeasurements.getCpReadLatency())).append(';');
                    diskDataFile.append(String.valueOf(diskRawMeasurements.getCpReads())).append(';');
                    diskDataFile.append(String.valueOf(diskRawMeasurements.getDiskBusy())).append(';');
                    diskDataFile.append(String.valueOf(diskRawMeasurements.getGuarenteedReadBlocks())).append(';');
                    diskDataFile.append(String.valueOf(diskRawMeasurements.getGuarenteedReadLatency())).append(';');
                    diskDataFile.append(String.valueOf(diskRawMeasurements.getGuarenteedReads())).append(';');
                    diskDataFile.append(String.valueOf(diskRawMeasurements.getGuarenteedWriteBlocks())).append(';');
                    diskDataFile.append(String.valueOf(diskRawMeasurements.getGuarenteedWriteLatency())).append(';');
                    diskDataFile.append(String.valueOf(diskRawMeasurements.getGuarenteedWrites())).append(';');
                    diskDataFile.append(String.valueOf(diskRawMeasurements.getTotalTransfers())).append(';');
                    diskDataFile.append(String.valueOf(diskRawMeasurements.getUserReadBlocks())).append(';');
                    diskDataFile.append(String.valueOf(diskRawMeasurements.getUserReadLatency())).append(';');
                    diskDataFile.append(String.valueOf(diskRawMeasurements.getUserReads())).append(';');
                    diskDataFile.append(String.valueOf(diskRawMeasurements.getUserWriteBlocks())).append(';');
                    diskDataFile.append(String.valueOf(diskRawMeasurements.getUserWriteLatency())).append(';');
                    diskDataFile.append(String.valueOf(diskRawMeasurements.getUserWrites())).append(';');
                }
                diskDataFile.append(crlf);
            }
            diskDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (diskDataFile != null) {
                try {
                    diskDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }

    void dumpVolumeData(final VServer.VServerKey key, final Collection<RawSample<VolumeMeasurements>> samples) {
        if (samples.size() == 0) {
            return;
        }
        DSFileWriter volumeDataFile = null;
        try {
            volumeDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                              DataSourceOutputFileType.perfdata,
                                                                                              key.getOcieUUID() + "_volumedata",
                                                                                              "txt");
            final File tempFile = new File(volumeDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "volumeData", key.toString());
            // NetappVolumeMetrics 
            volumeDataFile.writeln("volumeID;avg_latency;cifs_other_latency;cifs_other_ops;cifs_read_data;cifs_read_latency;cifs_read_ops;cifs_write_data;cifs_write_latency;"
                                   + "cifs_write_ops;fcp_other_latency;fcp_other_ops;fcp_read_data;fcp_read_latency;fcp_read_ops;fcp_write_data;fcp_write_latency;fcp_write_ops;"
                                   + "iscsi_other_latency;iscsi_other_ops;iscsi_read_data;iscsi_read_latency;iscsi_read_ops;iscsi_write_data;iscsi_write_latency;iscsi_write_ops;"
                                   + "nfs_other_latency;nfs_other_ops;nfs_read_data;nfs_read_latency;nfs_read_ops;nfs_write_data;nfs_write_latency;nfs_write_ops;other_latency;"
                                   + "other_ops;read_data;read_latency;read_ops;san_other_latency;san_other_ops;san_read_data;san_read_latency;san_read_ops;san_write_data;"
                                   + "san_write_latency;san_write_ops;total_ops;write_data;write_latency;write_ops");

            for (final RawSample<VolumeMeasurements> sample : samples) {
                volumeDataFile.append(sample.getKey()).append(';');
                final VolumeMeasurements volumeMeasurements = sample.getCounters();
                if (volumeMeasurements != null) {
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getAvgLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getCifsOtherLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getCifsOtherOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getCifsReadData())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getCifsReadLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getCifsReadOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getCifsWriteData())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getCifsWriteLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getCifsWriteOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getFcpOtherLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getFcpOtherOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getFcpReadData())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getFcpReadLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getFcpReadOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getFcpWriteData())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getFcpWriteLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getFcpWriteOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getIscsiOtherLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getIscsiOtherOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getIscsiReadData())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getIscsiReadLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getIscsiReadOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getIscsiWriteData())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getIscsiWriteLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getIscsiWriteOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getNfsOtherLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getNfsOtherOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getNfsReadData())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getNfsReadLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getNfsReadOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getNfsWriteData())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getNfsWriteLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getNfsWriteOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getOtherLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getOtherOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getReadData())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getReadLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getReadOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getSanOtherLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getSanOtherOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getSanReadData())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getSanReadLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getSanReadOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getSanWriteData())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getSanWriteLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getSanWriteOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getTotalOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getWriteData())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getWriteLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getWriteOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getOpmVolNfsOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getOpmVolCifsOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getOpmVolIscsiOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getOpmVolFcpOps())).append(';');
                }
                volumeDataFile.append(crlf);
            }
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (volumeDataFile != null) {
                try {
                    volumeDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }

    void dumpRawVolumeData(final VServer.VServerKey key, final Collection<RawSample<VolumeRawMeasurements>> samples) {
        if (samples.size() == 0) {
            return;
        }
        DSFileWriter volumeDataFile = null;
        try {
            volumeDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                              DataSourceOutputFileType.perfdata,
                                                                                              key.getOcieUUID() + "_volumerawdata",
                                                                                              "txt");
            final File tempFile = new File(volumeDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "volumeData", key.toString());
            // NetappVolumeMetrics 
            volumeDataFile.writeln("volumeID;avg_latency;cifs_other_latency;cifs_other_ops;cifs_read_latency;cifs_read_ops;cifs_write_latency;"
                                   + "cifs_write_ops;fcp_other_latency;fcp_other_ops;fcp_read_latency;fcp_read_ops;fcp_write_latency;fcp_write_ops;"
                                   + "iscsi_other_latency;iscsi_other_ops;iscsi_read_latency;iscsi_read_ops;iscsi_write_latency;iscsi_write_ops;"
                                   + "nfs_other_latency;nfs_other_ops;nfs_read_latency;nfs_read_ops;nfs_write_latency;nfs_write_ops;other_latency;"
                                   + "other_ops;read_latency;read_ops;total_protocol_other_latency;total_protocol_read_latency;total_protocol_write_latency"
                                   + "total_ops;write_latency;write_ops");

            for (final RawSample<VolumeRawMeasurements> sample : samples) {
                volumeDataFile.append(sample.getKey()).append(';');
                final VolumeRawMeasurements volumeMeasurements = sample.getCounters();
                if (volumeMeasurements != null) {
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getAvgLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getCifsOtherLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getCifsOtherOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getCifsReadLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getCifsReadOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getCifsWriteLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getCifsWriteOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getFcpOtherLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getFcpOtherOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getFcpReadLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getFcpReadOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getFcpWriteLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getFcpWriteOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getIscsiOtherLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getIscsiOtherOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getIscsiReadLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getIscsiReadOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getIscsiWriteLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getIscsiWriteOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getNfsOtherLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getNfsOtherOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getNfsReadLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getNfsReadOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getNfsWriteLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getNfsWriteOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getOtherLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getOtherOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getReadLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getReadOps())).append(';');
                    volumeDataFile.append(volumeMeasurements.getTotalProtocolOtherLatency()).append(';');
                    volumeDataFile.append(volumeMeasurements.getTotalProtocolReadLatency()).append(';');
                    volumeDataFile.append(volumeMeasurements.getTotalProtocolWriteLatency()).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getTotalOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getWriteLatency())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getWriteOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getOpmVolNfsOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getOpmVolCifsOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getOpmVolIscsiOps())).append(';');
                    volumeDataFile.append(String.valueOf(volumeMeasurements.getOpmVolFcpOps())).append(';');
                    
                }
                volumeDataFile.append(crlf);
            }
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (volumeDataFile != null) {
                try {
                    volumeDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }
    
    void dumpLunData(final VServer.VServerKey key, final Collection<RawSample<LunMeasurements>> samples) {
        if (samples.size() == 0) {
            return;
        }
        DSFileWriter lunDataFile = null;
        try {
            lunDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                           DataSourceOutputFileType.perfdata,
                                                                                           key.getOcieUUID() + "_lundata",
                                                                                           "txt");
            final File tempFile = new File(lunDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "lunData", key.toString());
            lunDataFile.writeln("VolumeID;avg_latency;avg_read_latency;avg_write_latency;other_ops;read_data;read_ops;read_partial_blocks;total_ops;write_data;write_ops;write_partial_blocks;opm_lun_throughput");
            for (final RawSample<LunMeasurements> sample : samples) {
                lunDataFile.append(sample.getKey()).append(';');
                final LunMeasurements lunMeasurements = sample.getCounters();
                if (lunMeasurements != null) {
                    lunDataFile.append(String.valueOf(lunMeasurements.getAvgLatency())).append(';');
                    lunDataFile.append(String.valueOf(lunMeasurements.getAvgReadLatency())).append(';');
                    lunDataFile.append(String.valueOf(lunMeasurements.getAvgWriteLatency())).append(';');
                    lunDataFile.append(String.valueOf(lunMeasurements.getOtherOps())).append(';');
                    lunDataFile.append(String.valueOf(lunMeasurements.getReadData())).append(';');
                    lunDataFile.append(String.valueOf(lunMeasurements.getReadOps())).append(';');
                    lunDataFile.append(String.valueOf(lunMeasurements.getReadPartialBlocks())).append(';');
                    lunDataFile.append(String.valueOf(lunMeasurements.getTotalOps())).append(';');
                    lunDataFile.append(String.valueOf(lunMeasurements.getWriteData())).append(';');
                    lunDataFile.append(String.valueOf(lunMeasurements.getWriteOps())).append(';');
                    lunDataFile.append(String.valueOf(lunMeasurements.getWritePartialBlocks())).append(';');
                    lunDataFile.append(String.valueOf(lunMeasurements.getOpmLunThroughput())).append(';');                    
                }
                lunDataFile.append(crlf);
            }
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (lunDataFile != null) {
                try {
                    lunDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }

    void dumpLunRawData(final VServer.VServerKey key, final Collection<RawSample<LunRawMeasurements>> samples) {
        if (samples.size() == 0) {
            return;
        }
        DSFileWriter lunDataFile = null;
        try {
            lunDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                           DataSourceOutputFileType.perfdata,
                                                                                           key.getOcieUUID() + "_lunRawdata",
                                                                                           "txt");
            final File tempFile = new File(lunDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "lunRawData", key.toString());
            lunDataFile.writeln("VolumeID;avg_latency;avg_read_latency;avg_write_latency;other_ops;read_data;read_ops;total_ops;write_data;write_ops;opm_lun_throughput");
            for (final RawSample<LunRawMeasurements> sample : samples) {
                lunDataFile.append(sample.getKey()).append(';');
                final LunRawMeasurements lunMeasurements = sample.getCounters();
                if (lunMeasurements != null) {
                    lunDataFile.append(String.valueOf(lunMeasurements.getAvgLatency())).append(';');
                    lunDataFile.append(String.valueOf(lunMeasurements.getAvgReadLatency())).append(';');
                    lunDataFile.append(String.valueOf(lunMeasurements.getAvgWriteLatency())).append(';');
                    lunDataFile.append(String.valueOf(lunMeasurements.getOtherOps())).append(';');
                    lunDataFile.append(String.valueOf(lunMeasurements.getReadData())).append(';');
                    lunDataFile.append(String.valueOf(lunMeasurements.getReadOps())).append(';');
                    lunDataFile.append(String.valueOf(lunMeasurements.getTotalOps())).append(';');
                    lunDataFile.append(String.valueOf(lunMeasurements.getWriteData())).append(';');
                    lunDataFile.append(String.valueOf(lunMeasurements.getWriteOps())).append(';');
                    lunDataFile.append(String.valueOf(lunMeasurements.getOpmLunThroughput())).append(';');
                }
                lunDataFile.append(crlf);
            }
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (lunDataFile != null) {
                try {
                    lunDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }
    
    void dumpNodeData(final Cluster.ClusterKey key, final Collection<RawSample<NodeMeasurements>> samples) {
        if (samples.size() == 0) {
            return;
        }
        DSFileWriter nodeDataFile = null;
        try {
            nodeDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                            DataSourceOutputFileType.perfdata,
                                                                                            key.getOcieUUID() + "_nodedata",
                                                                                            "txt");
            final File tempFile = new File(nodeDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "nodeData", key.toString());
            nodeDataFile.writeln("NodeID;cpu_busy;avg_processor_busy;read_opds;write_ops;total_ops;other_ops;partner_ops;cifs_ops;fcp_ops;iscsi_ops;nfs_ops;sys_read_latency;sys_write_latency;system_ops");
            for (final RawSample<NodeMeasurements> sample : samples) {
                nodeDataFile.append(sample.getKey()).append(';');
                final NodeMeasurements nodeMeasurements = sample.getCounters();
                if (nodeMeasurements != null) {
                    nodeDataFile.append(String.valueOf(nodeMeasurements.getCpuBusy())).append(';');
                    nodeDataFile.append(String.valueOf(nodeMeasurements.getAvgProcessorBusy())).append(';');
                    nodeDataFile.append(String.valueOf(nodeMeasurements.getReadOps())).append(';');
                    nodeDataFile.append(String.valueOf(nodeMeasurements.getWriteOps())).append(';');
                    nodeDataFile.append(String.valueOf(nodeMeasurements.getTotalOps())).append(';');
                    nodeDataFile.append(String.valueOf(nodeMeasurements.getOtherOps())).append(';');
                    nodeDataFile.append(String.valueOf(nodeMeasurements.getCifsOps())).append(';');
                    nodeDataFile.append(String.valueOf(nodeMeasurements.getFcpOps())).append(';');
                    nodeDataFile.append(String.valueOf(nodeMeasurements.getIscsiOps())).append(';');
                    nodeDataFile.append(String.valueOf(nodeMeasurements.getNfsOps())).append(';');
                    nodeDataFile.append(String.valueOf(nodeMeasurements.getSysReadLatency())).append(';');
                    nodeDataFile.append(String.valueOf(nodeMeasurements.getSysWriteLatency())).append(';');
                    nodeDataFile.append(String.valueOf(nodeMeasurements.getSysAvgLatency())).append(';');
                    nodeDataFile.append(String.valueOf(nodeMeasurements.getFcpDataRecv())).append(';');
                    nodeDataFile.append(String.valueOf(nodeMeasurements.getFcpDataSent())).append(';');
                    nodeDataFile.append(String.valueOf(nodeMeasurements.getOpmSysReadThroughput())).append(';');
                    nodeDataFile.append(String.valueOf(nodeMeasurements.getOpmSysWriteThroughput())).append(';');
                    nodeDataFile.append(String.valueOf(nodeMeasurements.getOpmSysThroughput())).append(';');
                    
                }
                nodeDataFile.append(crlf);
            }
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (nodeDataFile != null) {
                try {
                    nodeDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }
    
    void dumpRawNodeData(final Cluster.ClusterKey key, final Collection<RawSample<NodeRawMeasurements>> samples) {
        if (samples.size() == 0) {
            return;
        }
        DSFileWriter nodeDataFile = null;
        try {
            nodeDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                            DataSourceOutputFileType.perfdata,
                                                                                            key.getOcieUUID() + "_noderawdata",
                                                                                            "txt");
            final File tempFile = new File(nodeDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "nodeRawData", key.toString());
            nodeDataFile.writeln("NodeID;avg_processor_busy;cpu_busy;cpu_elapsed_time;cpu_elapsed_time1;net_data_recv;" + 
            "net_data_sent;cifs_ops;fcp_ops;http_ops;iscsi_ops;nfs_ops;other_ops;read_ops;total_ops;write_ops;system_ops");
            
            for (final RawSample<NodeRawMeasurements> sample : samples) {
                nodeDataFile.append(sample.getKey()).append(';');
                final NodeRawMeasurements nodeRawMeasurements = sample.getCounters();
                if (nodeRawMeasurements != null) {
                    nodeDataFile.append(String.valueOf(nodeRawMeasurements.getAvgProcessorBusy())).append(';');
                    nodeDataFile.append(String.valueOf(nodeRawMeasurements.getCpuBusy())).append(';');
                    nodeDataFile.append(String.valueOf(nodeRawMeasurements.getCpuElapsedTime())).append(';');
                    nodeDataFile.append(String.valueOf(nodeRawMeasurements.getNetDataRecv())).append(';');
                    nodeDataFile.append(String.valueOf(nodeRawMeasurements.getNetDataSent())).append(';');
                    nodeDataFile.append(String.valueOf(nodeRawMeasurements.getCifsOps())).append(';');
                    nodeDataFile.append(String.valueOf(nodeRawMeasurements.getFcpOps())).append(';');
                    nodeDataFile.append(String.valueOf(nodeRawMeasurements.getIscsiOps())).append(';');
                    nodeDataFile.append(String.valueOf(nodeRawMeasurements.getNfsOps())).append(';');
                    nodeDataFile.append(String.valueOf(nodeRawMeasurements.getOtherOps())).append(';');
                    nodeDataFile.append(String.valueOf(nodeRawMeasurements.getReadOps())).append(';');
                    nodeDataFile.append(String.valueOf(nodeRawMeasurements.getTotalOps())).append(';');
                    nodeDataFile.append(String.valueOf(nodeRawMeasurements.getWriteOps())).append(';');
                }
                nodeDataFile.append(crlf);
            }
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (nodeDataFile != null) {
                try {
                    nodeDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }

    void dumpFcpLifData(final VServer.VServerKey key, final Collection<RawSample<FcpLifMeasurements>> samples) {
        DSFileWriter fcpLifDataFile = null;
        try {
            fcpLifDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                              DataSourceOutputFileType.perfdata,
                                                                                              key.getOcieUUID() + "_fcplifdata",
                                                                                              "txt");
            final File tempFile = new File(fcpLifDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "fcpLifData", key.toString());
            fcpLifDataFile.writeln("LifID;other_ops;read_data;read_ops;write_data;write_ops;");
            for (final RawSample<FcpLifMeasurements> sample : samples) {
                final FcpLifMeasurements fcpLifMeasurements = sample.getCounters();
                if (fcpLifMeasurements != null) {
                    fcpLifDataFile.append(sample.getKey()).append(';');
                    fcpLifDataFile.append(String.valueOf(fcpLifMeasurements.getOtherOps())).append(';');
                    fcpLifDataFile.append(String.valueOf(fcpLifMeasurements.getReadData())).append(';');
                    fcpLifDataFile.append(String.valueOf(fcpLifMeasurements.getReadOps())).append(';');
                    fcpLifDataFile.append(String.valueOf(fcpLifMeasurements.getWriteData())).append(';');
                    fcpLifDataFile.append(String.valueOf(fcpLifMeasurements.getWriteOps())).append(';');
                }
                fcpLifDataFile.append(crlf);
            }
            fcpLifDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (fcpLifDataFile != null) {
                try {
                    fcpLifDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }

    void dumpFcpLifVserverData(final Cluster.ClusterKey key, final Collection<RawSample<FcpLifVserverMeasurements>> samples) {
        DSFileWriter fcpLifDataFile = null;
        try {
            fcpLifDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                              DataSourceOutputFileType.perfdata,
                                                                                              key.getOcieUUID() + "_fcplifVserverdata",
                                                                                              "txt");
            final File tempFile = new File(fcpLifDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "fcpLifVserverData", key.toString());
            fcpLifDataFile.writeln("LifID;avg_latency;other_ops;read_data;read_ops;write_data;write_ops;total_ops;");
            for (final RawSample<FcpLifVserverMeasurements> sample : samples) {
                final FcpLifVserverMeasurements fcpLifMeasurements = sample.getCounters();
                if (fcpLifMeasurements != null) {
                    fcpLifDataFile.append(sample.getKey()).append(';');
                    fcpLifDataFile.append(String.valueOf(fcpLifMeasurements.getAvgLatency())).append(';');
                    fcpLifDataFile.append(String.valueOf(fcpLifMeasurements.getOtherOps())).append(';');
                    fcpLifDataFile.append(String.valueOf(fcpLifMeasurements.getReadData())).append(';');
                    fcpLifDataFile.append(String.valueOf(fcpLifMeasurements.getReadOps())).append(';');
                    fcpLifDataFile.append(String.valueOf(fcpLifMeasurements.getWriteData())).append(';');
                    fcpLifDataFile.append(String.valueOf(fcpLifMeasurements.getWriteOps())).append(';');
                    fcpLifDataFile.append(String.valueOf(fcpLifMeasurements.getTotalOps())).append(';');
                }
                fcpLifDataFile.append(crlf);
            }
            fcpLifDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (fcpLifDataFile != null) {
                try {
                    fcpLifDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }
    
    void dumpRawFcpLifData(final VServer.VServerKey key, final Collection<RawSample<FcpLifRawMeasurements>> samples) {
        DSFileWriter fcpLifDataFile = null;
        try {
            fcpLifDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                              DataSourceOutputFileType.perfdata,
                                                                                              key.getOcieUUID() + "_fcplifrawdata",
                                                                                              "txt");
            final File tempFile = new File(fcpLifDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "fcpRawLifData", key.toString());
            fcpLifDataFile.writeln("LifID;other_ops;read_data;read_ops;write_data;write_ops;");
            for (final RawSample<FcpLifRawMeasurements> sample : samples) {
                final FcpLifRawMeasurements fcpLifMeasurements = sample.getCounters();
                if (fcpLifMeasurements != null) {
                    fcpLifDataFile.append(sample.getKey()).append(";");
                    fcpLifDataFile.append(String.valueOf(fcpLifMeasurements.getOtherOps())).append(";");
                    fcpLifDataFile.append(String.valueOf(fcpLifMeasurements.getReadData())).append(";");
                    fcpLifDataFile.append(String.valueOf(fcpLifMeasurements.getReadOps())).append(";");
                    fcpLifDataFile.append(String.valueOf(fcpLifMeasurements.getWriteData())).append(";");
                    fcpLifDataFile.append(String.valueOf(fcpLifMeasurements.getWriteOps())).append(";");
                }
                fcpLifDataFile.append(crlf);
            }
            fcpLifDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            try {
                if (fcpLifDataFile != null) {
                    fcpLifDataFile.close();
                }
            }
            catch (IOException ioe) {
                //fall through
            }
        }
    }

    void dumpRawFcpLifVserverData(final Cluster.ClusterKey key, final Collection<RawSample<FcpLifVserverRawMeasurements>> samples) {
        DSFileWriter fcpLifDataFile = null;
        try {
            fcpLifDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                              DataSourceOutputFileType.perfdata,
                                                                                              key.getOcieUUID() + "_fcpLifVserverRawData",
                                                                                              "txt");
            final File tempFile = new File(fcpLifDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "fcpLifVserverRawData", key.toString());
            fcpLifDataFile.writeln("LifID;avg_latency;other_ops;read_data;read_ops;write_data;write_ops;total_ops");
            for (final RawSample<FcpLifVserverRawMeasurements> sample : samples) {
                final FcpLifVserverRawMeasurements fcpLifMeasurements = sample.getCounters();
                if (fcpLifMeasurements != null) {
                    fcpLifDataFile.append(sample.getKey()).append(";");
                    fcpLifDataFile.append(String.valueOf(fcpLifMeasurements.getAvgLatency())).append(";");
                    fcpLifDataFile.append(String.valueOf(fcpLifMeasurements.getOtherOps())).append(";");
                    fcpLifDataFile.append(String.valueOf(fcpLifMeasurements.getReadData())).append(";");
                    fcpLifDataFile.append(String.valueOf(fcpLifMeasurements.getReadOps())).append(";");
                    fcpLifDataFile.append(String.valueOf(fcpLifMeasurements.getWriteData())).append(";");
                    fcpLifDataFile.append(String.valueOf(fcpLifMeasurements.getWriteOps())).append(";");
                    fcpLifDataFile.append(String.valueOf(fcpLifMeasurements.getTotalOps())).append(";");
                }
                fcpLifDataFile.append(crlf);
            }
            fcpLifDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            try {
                if (fcpLifDataFile != null) {
                    fcpLifDataFile.close();
                }
            }
            catch (IOException ioe) {
                //fall through
            }
        }
    }

    @SuppressWarnings({"ConstantConditions"})
    void dumpLifData(final VServer.VServerKey key, final Collection<RawSample<NetworkLifMeasurements>> samples) {
        DSFileWriter lifDataFile = null;
        try {
            lifDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                           DataSourceOutputFileType.perfdata,
                                                                                           key.getOcieUUID() + "_lifdata",
                                                                                           "txt");
            final File tempFile = new File(lifDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "lifData", key.toString());
            lifDataFile.writeln("LifID;recv_data;recv_errors;recv_packet;sent_data;sent_errors;sent_packet");
            for (final RawSample<NetworkLifMeasurements> sample : samples) {
                lifDataFile.append(sample.getKey()).append(';');
                final NetworkLifMeasurements networkLifMeasurements = sample.getCounters();
                if (networkLifMeasurements != null) {
                    lifDataFile.append(String.valueOf(networkLifMeasurements.getRecvData())).append(';');
                    lifDataFile.append(String.valueOf(networkLifMeasurements.getRecvErrors())).append(';');
                    lifDataFile.append(String.valueOf(networkLifMeasurements.getRecvPacket())).append(';');
                    lifDataFile.append(String.valueOf(networkLifMeasurements.getSentData())).append(';');
                    lifDataFile.append(String.valueOf(networkLifMeasurements.getSentErrors())).append(';');
                    lifDataFile.append(String.valueOf(networkLifMeasurements.getSentPacket())).append(';');
                    lifDataFile.append(String.valueOf(networkLifMeasurements.getOpmLifPackets())).append(';');
                    lifDataFile.append(String.valueOf(networkLifMeasurements.getOpmLifThroughput())).append(';');
                }
                lifDataFile.append(crlf);
            }
            lifDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for LIF performance data", e);
        }
        finally {
            if (lifDataFile != null) {
                try {
                    lifDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }

    void dumpLifRawData(final VServer.VServerKey key, final Collection<RawSample<NetworkLifRawMeasurements>> samples) {
        DSFileWriter lifDataFile = null;
        try {
            lifDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                           DataSourceOutputFileType.perfdata,
                                                                                           key.getOcieUUID() + "_lifRawdata",
                                                                                           "txt");
            final File tempFile = new File(lifDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "lifData", key.toString());
            lifDataFile.writeln("LifID;recv_data;recv_errors;recv_packet;sent_data;sent_errors;sent_packet");
            for (final RawSample<NetworkLifRawMeasurements> sample : samples) {
                lifDataFile.append(sample.getKey()).append(';');
                final NetworkLifRawMeasurements networkLifRawMeasurements = sample.getCounters();
                if (networkLifRawMeasurements != null) {
                    lifDataFile.append(String.valueOf(networkLifRawMeasurements.getRecvData())).append(';');
                    lifDataFile.append(String.valueOf(networkLifRawMeasurements.getRecvErrors())).append(';');
                    lifDataFile.append(String.valueOf(networkLifRawMeasurements.getRecvPacket())).append(';');
                    lifDataFile.append(String.valueOf(networkLifRawMeasurements.getSentData())).append(';');
                    lifDataFile.append(String.valueOf(networkLifRawMeasurements.getSentErrors())).append(';');
                    lifDataFile.append(String.valueOf(networkLifRawMeasurements.getSentPacket())).append(';');
                    lifDataFile.append(String.valueOf(networkLifRawMeasurements.getOpmLifPackets())).append(';');
                    lifDataFile.append(String.valueOf(networkLifRawMeasurements.getOpmLifThroughput())).append(';');
                }
                lifDataFile.append(crlf);
            }
            lifDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for LIF raw performance data", e);
        }
        finally {
            if (lifDataFile != null) {
                try {
                    lifDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }

    void dumpLifVserverData(final Cluster.ClusterKey key, final Collection<RawSample<NetworkLifVserverMeasurements>> samples) {
        DSFileWriter lifDataFile = null;
        try {
            lifDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                           DataSourceOutputFileType.perfdata,
                                                                                           key.getOcieUUID() + "_lifVserverdata",
                                                                                           "txt");
            final File tempFile = new File(lifDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "lifVserverData", key.toString());
            lifDataFile.writeln("LifID;recv_data;recv_errors;recv_packet;sent_data;sent_errors;sent_packet;opm_lif_packets;opm_lif_throughput");
            for (final RawSample<NetworkLifVserverMeasurements> sample : samples) {
                lifDataFile.append(sample.getKey()).append(';');
                final NetworkLifVserverMeasurements networkLifMeasurements = sample.getCounters();
                if (networkLifMeasurements != null) {
                    lifDataFile.append(String.valueOf(networkLifMeasurements.getRecvData())).append(';');
                    lifDataFile.append(String.valueOf(networkLifMeasurements.getRecvErrors())).append(';');
                    lifDataFile.append(String.valueOf(networkLifMeasurements.getRecvPacket())).append(';');
                    lifDataFile.append(String.valueOf(networkLifMeasurements.getSentData())).append(';');
                    lifDataFile.append(String.valueOf(networkLifMeasurements.getSentErrors())).append(';');
                    lifDataFile.append(String.valueOf(networkLifMeasurements.getSentPacket())).append(';');
                    lifDataFile.append(String.valueOf(networkLifMeasurements.getOpmLifPackets())).append(';');
                    lifDataFile.append(String.valueOf(networkLifMeasurements.getOpmLifThroughput())).append(';');                    
                }
                lifDataFile.append(crlf);
            }
            lifDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for LifVserver performance data", e);
        }
        finally {
            if (lifDataFile != null) {
                try {
                    lifDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }

    void dumpLifVserverRawData(final Cluster.ClusterKey key, final Collection<RawSample<NetworkLifVserverRawMeasurements>> samples) {
        DSFileWriter lifDataFile = null;
        try {
            lifDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                           DataSourceOutputFileType.perfdata,
                                                                                           key.getOcieUUID() + "_lifVserverRawdata",
                                                                                           "txt");
            final File tempFile = new File(lifDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "lifVserverData", key.toString());
            lifDataFile.writeln("LifID;recv_data;recv_errors;recv_packet;sent_data;sent_errors;sent_packet;opm_lif_packets;opm_lif_throughput");
            for (final RawSample<NetworkLifVserverRawMeasurements> sample : samples) {
                lifDataFile.append(sample.getKey()).append(';');
                final NetworkLifVserverRawMeasurements networkLifRawMeasurements = sample.getCounters();
                if (networkLifRawMeasurements != null) {
                    lifDataFile.append(String.valueOf(networkLifRawMeasurements.getRecvData())).append(';');
                    lifDataFile.append(String.valueOf(networkLifRawMeasurements.getRecvErrors())).append(';');
                    lifDataFile.append(String.valueOf(networkLifRawMeasurements.getRecvPacket())).append(';');
                    lifDataFile.append(String.valueOf(networkLifRawMeasurements.getSentData())).append(';');
                    lifDataFile.append(String.valueOf(networkLifRawMeasurements.getSentErrors())).append(';');
                    lifDataFile.append(String.valueOf(networkLifRawMeasurements.getSentPacket())).append(';');
                    lifDataFile.append(String.valueOf(networkLifRawMeasurements.getOpmLifPackets())).append(';');
                    lifDataFile.append(String.valueOf(networkLifRawMeasurements.getOpmLifThroughput())).append(';');                    
                }
                lifDataFile.append(crlf);
            }
            lifDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for LifVserver raw performance data", e);
        }
        finally {
            if (lifDataFile != null) {
                try {
                    lifDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }
    
    @SuppressWarnings({"ConstantConditions"})
    void dumpIscsiLifData(final VServer.VServerKey key, final Collection<RawSample<IscsiLifMeasurements>> samples) {
        DSFileWriter iscisLifDataFile = null;
        try {
            iscisLifDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                                DataSourceOutputFileType.perfdata,
                                                                                                key.getOcieUUID() + "_iscsilifdata",
                                                                                                "txt");
            final File tempFile = new File(iscisLifDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "iscsiLifData", key.toString());
            iscisLifDataFile.writeln("IscsiLifID;data_in_blocks;data_out_blocks;iscsi_read_ops;iscsi_write_ops;protocol_errors;read_data;write_data;cmd_transfered;avg_latency;avg_other_latency;avg_read_latency;avg_write_latency;opm_iscsilif_ops;opm_iscsilif_throughput");
            for (final RawSample<IscsiLifMeasurements> sample : samples) {
                iscisLifDataFile.append(sample.getKey()).append(';');
                final IscsiLifMeasurements iscsiLifMeasurements = sample.getCounters();
                if (iscsiLifMeasurements != null) {
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getDataInBlocks())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getDataOutBlocks())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getIscsiReadOps())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getIscsiWriteOps())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getProtocolErrors())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getReadData())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getWriteData())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getCmdTransfered())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getAvgLatency())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getAvgOtherLatency())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getAvgReadLatency())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getAvgWriteLatency())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getOps())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getThroughput())).append(';');
                }
                iscisLifDataFile.append(crlf);
            }
            iscisLifDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for iSCSI LIF performance data", e);
        }
        finally {
            if (iscisLifDataFile != null) {
                try {
                    iscisLifDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }

    void dumpIscsiLifVserverData(final Cluster.ClusterKey key, final Collection<RawSample<IscsiLifVserverMeasurements>> samples) {
        DSFileWriter iscisLifDataFile = null;
        try {
            iscisLifDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                                DataSourceOutputFileType.perfdata,
                                                                                                key.getOcieUUID() + "_iscsilifvserverdata",
                                                                                                "txt");
            final File tempFile = new File(iscisLifDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "iscsiLifVserverData", key.toString());
            iscisLifDataFile.writeln("IscsiLifID;data_in_blocks;data_out_blocks;iscsi_read_ops;iscsi_write_ops;iscsi_write_ops;protocol_errors;" + ""
            		+ "read_data;write_data;cmd_transfered;avg_latency;avg_read_latency;avg_write_latency;avg_other_latency;opm_iscsilif_ops;opm_iscsilif_throughput");
            for (final RawSample<IscsiLifVserverMeasurements> sample : samples) {
                iscisLifDataFile.append(sample.getKey()).append(';');
                final IscsiLifVserverMeasurements iscsiLifMeasurements = sample.getCounters();
                if (iscsiLifMeasurements != null) {
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getDataInBlocks())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getDataOutBlocks())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getIscsiReadOps())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getIscsiWriteOps())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getIscsiOtherOps())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getProtocolErrors())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getReadData())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getWriteData())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getCmdTransfered())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getAvgLatency())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getAvgReadLatency())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getAvgWriteLatency())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getAvgOtherLatency())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getOpmIscsiLifOps())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifMeasurements.getOpmIscsiLifThroughput())).append(';');
                }
                iscisLifDataFile.append(crlf);
            }
            iscisLifDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for iscsiLifVserver performance data", e);
        }
        finally {
            if (iscisLifDataFile != null) {
                try {
                    iscisLifDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }
    
    @SuppressWarnings({"ConstantConditions"})
    void dumpIscsiLifRawData(final VServer.VServerKey key, final Collection<RawSample<IscsiLifRawMeasurements>> samples) {
        DSFileWriter iscisLifDataFile = null;
        try {
            iscisLifDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                                DataSourceOutputFileType.perfdata,
                                                                                                key.getOcieUUID() + "_iscsilifrawdata",
                                                                                                "txt");
            final File tempFile = new File(iscisLifDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "iscsiLifRawData", key.toString());
            iscisLifDataFile.writeln("IscsiLifID;avg_latency;avg_other_latency;avg_read_latency;avg_write_latency;iscsi_other_ops;iscsi_read_ops;iscsi_write_ops;data_in_blocks;read_data;data_out_blocks;write_data;cmd_transfered;opm_iscsilif_ops;opm_iscsilif_throughput");
            for (final RawSample<IscsiLifRawMeasurements> sample : samples) {
                iscisLifDataFile.append(sample.getKey()).append(';');
                final IscsiLifRawMeasurements iscsiLifRawMeasurements = sample.getCounters();
                if (iscsiLifRawMeasurements != null) {
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getAvgLatency())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getAvgOtherLatency())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getAvgReadLatency())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getAvgWriteLatency())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getIscsiOtherOps())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getIscsiReadOps())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getIscsiWriteOps())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getDataInBlocks())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getReadData())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getDataOutBlocks())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getWriteData())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getCmdTransfered())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getOps())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getThroughput())).append(';');
                }
                iscisLifDataFile.append(crlf);
            }
            iscisLifDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for iSCSI LIF performance data", e);
        }
        finally {
            if (iscisLifDataFile != null) {
                try {
                    iscisLifDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }

    void dumpIscsiLifVserverRawData(final Cluster.ClusterKey key, final Collection<RawSample<IscsiLifVserverRawMeasurements>> samples) {
        DSFileWriter iscisLifDataFile = null;
        try {
            iscisLifDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                                DataSourceOutputFileType.perfdata,
                                                                                                key.getOcieUUID() + "_iscsilifvserverrawdata",
                                                                                                "txt");
            final File tempFile = new File(iscisLifDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "iscsiLifVserverRawData", key.toString());
            iscisLifDataFile.writeln("IscsiLifID;avg_latency;avg_other_latency;avg_read_latency;avg_write_latency;iscsi_other_ops;iscsi_read_ops;" + 
            		"iscsi_write_ops;data_in_blocks;read_data;data_out_blocks;write_data;cmd_transfered;opm_iscsilif_ops;opm_iscsilif_throughput");
            for (final RawSample<IscsiLifVserverRawMeasurements> sample : samples) {
                iscisLifDataFile.append(sample.getKey()).append(';');
                final IscsiLifVserverRawMeasurements iscsiLifRawMeasurements = sample.getCounters();
                if (iscsiLifRawMeasurements != null) {
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getAvgLatency())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getAvgOtherLatency())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getAvgReadLatency())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getAvgWriteLatency())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getIscsiOtherOps())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getIscsiReadOps())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getIscsiWriteOps())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getDataInBlocks())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getReadData())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getDataOutBlocks())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getWriteData())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getCmdTransfered())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getOpmIscsiLifOps())).append(';');
                    iscisLifDataFile.append(String.valueOf(iscsiLifRawMeasurements.getOpmIscsiLifThroughput())).append(';');
                }
                iscisLifDataFile.append(crlf);
            }
            iscisLifDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for iscsiLifVserver performance data", e);
        }
        finally {
            if (iscisLifDataFile != null) {
                try {
                    iscisLifDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }
    
    @SuppressWarnings({"ConstantConditions"})
    void dumpFcpPortData(final Cluster.ClusterKey key, final Collection<RawSample<FcpPortMeasurements>> samples) {
        DSFileWriter fcpPortDataFile = null;
        try {
            fcpPortDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                               DataSourceOutputFileType.perfdata,
                                                                                               key.getOcieUUID() + "_fcpportdata",
                                                                                               "txt");
            final File tempFile = new File(fcpPortDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "fcpPortData", key.toString());
            fcpPortDataFile.writeln("PortID;other_ops;read_data;read_ops;write_data;write_ops;total_ops;avg_other_latency;avg_read_latency;avg_write_latency");
            for (final RawSample<FcpPortMeasurements> sample : samples) {
                fcpPortDataFile.append(sample.getKey()).append(';');
                final FcpPortMeasurements fcpPortMeasurements = sample.getCounters();
                if (fcpPortMeasurements != null) {
                    fcpPortDataFile.append(String.valueOf(fcpPortMeasurements.getOtherOps())).append(';');
                    fcpPortDataFile.append(String.valueOf(fcpPortMeasurements.getReadData())).append(';');
                    fcpPortDataFile.append(String.valueOf(fcpPortMeasurements.getReadOps())).append(';');
                    fcpPortDataFile.append(String.valueOf(fcpPortMeasurements.getWriteData())).append(';');
                    fcpPortDataFile.append(String.valueOf(fcpPortMeasurements.getWriteOps())).append(';');
                    fcpPortDataFile.append(String.valueOf(fcpPortMeasurements.getTotalOps())).append(';');
                    fcpPortDataFile.append(String.valueOf(fcpPortMeasurements.getAvgOtherLatency())).append(';');
                    fcpPortDataFile.append(String.valueOf(fcpPortMeasurements.getAvgReadLatency())).append(';');
                    fcpPortDataFile.append(String.valueOf(fcpPortMeasurements.getAvgWriteLatency())).append(';');
                }
                fcpPortDataFile.append(crlf);
            }
            fcpPortDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (fcpPortDataFile != null) {
                try {
                    fcpPortDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }

    void dumpFcpPortRawData(final Cluster.ClusterKey key, final Collection<RawSample<FcpPortRawMeasurements>> samples) {
        DSFileWriter fcpPortDataFile = null;
        try {
            fcpPortDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                               DataSourceOutputFileType.perfdata,
                                                                                               key.getOcieUUID() + "_fcpportrawdata",
                                                                                               "txt");
            final File tempFile = new File(fcpPortDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "fcpPortRawData", key.toString());
            fcpPortDataFile.writeln("PortID;avg_other_latency;avg_read_latency;avg_write_latency;other_ops;read_ops;write_ops;total_ops");
            for (final RawSample<FcpPortRawMeasurements> sample : samples) {
                fcpPortDataFile.append(sample.getKey()).append(';');
                final FcpPortRawMeasurements fcpPortMeasurements = sample.getCounters();
                if (fcpPortMeasurements != null) {
                    fcpPortDataFile.append(String.valueOf(fcpPortMeasurements.getAvgOtherLatency())).append(';');
                    fcpPortDataFile.append(String.valueOf(fcpPortMeasurements.getAvgReadLatency())).append(';');
                    fcpPortDataFile.append(String.valueOf(fcpPortMeasurements.getAvgWriteLatency())).append(';');                	
                    fcpPortDataFile.append(String.valueOf(fcpPortMeasurements.getOtherOps())).append(';');
                    fcpPortDataFile.append(String.valueOf(fcpPortMeasurements.getReadOps())).append(';');
                    fcpPortDataFile.append(String.valueOf(fcpPortMeasurements.getWriteOps())).append(';');
                    fcpPortDataFile.append(String.valueOf(fcpPortMeasurements.getTotalOps())).append(';');
                }
                fcpPortDataFile.append(crlf);
            }
            fcpPortDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (fcpPortDataFile != null) {
                try {
                    fcpPortDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }
    
    @SuppressWarnings({"ConstantConditions"})
    void dumpIpPortData(final Cluster.ClusterKey key, final Collection<RawSample<NetworkPortMeasurements>> samples) {
        DSFileWriter ipPortDataFile = null;
        try {
            ipPortDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                              DataSourceOutputFileType.perfdata,
                                                                                              key.getOcieUUID() + "_ipportdata",
                                                                                              "txt");
            final File tempFile = new File(ipPortDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "ipPortData", key.toString());
            ipPortDataFile.writeln("PortID;recv_data;recv_packets;send_data;send_packets;");
            for (final RawSample<NetworkPortMeasurements> sample : samples) {
                ipPortDataFile.append(sample.getKey()).append(';');
                final NetworkPortMeasurements networkPortMeasurements = sample.getCounters();
                if (networkPortMeasurements != null) {
                    ipPortDataFile.append(String.valueOf(networkPortMeasurements.getRecvData())).append(';');
                    ipPortDataFile.append(String.valueOf(networkPortMeasurements.getRecvPackets())).append(';');
                    ipPortDataFile.append(String.valueOf(networkPortMeasurements.getSendData())).append(';');
                    ipPortDataFile.append(String.valueOf(networkPortMeasurements.getSendPackets())).append(';');
                }
                ipPortDataFile.append(crlf);
            }
            ipPortDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (ipPortDataFile != null) {
                try {
                    ipPortDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }
    


    @SuppressWarnings({"ConstantConditions"})
    void dumpRawNicData(final Cluster.ClusterKey key, final Collection<RawSample<NicRawMeasurements>> samples) {
        DSFileWriter nicRawDataFile = null;
        try {
            nicRawDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                              DataSourceOutputFileType.perfdata,
                                                                                              key.getOcieUUID() + "_nicrawdata",
                                                                                              "txt");
            final File tempFile = new File(nicRawDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "nicRawData", key.toString());
            nicRawDataFile.writeln("NicID;rxBytesPerSec;rxErrorsPerSec;txBytesPerSec;txErrorsPerSec; opmNicTotalBytePerSec;opmNicTotalErrorRate");
            for (final RawSample<NicRawMeasurements> sample : samples) {
                nicRawDataFile.append(sample.getKey()).append(';');
                final NicRawMeasurements nicRawMeasurements = sample.getCounters();
                if (nicRawMeasurements != null) {
                    nicRawDataFile.append(String.valueOf(nicRawMeasurements.getRxBytesRate())).append(';');
                    nicRawDataFile.append(String.valueOf(nicRawMeasurements.getRxErrorsRate())).append(';');
                    nicRawDataFile.append(String.valueOf(nicRawMeasurements.getTxBytesRate())).append(';');
                    nicRawDataFile.append(String.valueOf(nicRawMeasurements.getTxErrorsRate())).append(';');
                    nicRawDataFile.append(String.valueOf(nicRawMeasurements.getOpmNicTotalBytePersec())).append(';');
                    nicRawDataFile.append(String.valueOf(nicRawMeasurements.getOpmNicTotalErrorRate())).append(';');
                }
                nicRawDataFile.append(crlf);
            }
            nicRawDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for raw nic performance data", e);
        }
        finally {
            if (nicRawDataFile != null) {
                try {
                    nicRawDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }
    
    @SuppressWarnings({"ConstantConditions"})
    void dumpNicData(final Cluster.ClusterKey key, final Collection<RawSample<NicMeasurements>> samples) {
        DSFileWriter nicDataFile = null;
        try {
            nicDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                              DataSourceOutputFileType.perfdata,
                                                                                              key.getOcieUUID() + "_nicdata",
                                                                                              "txt");
            final File tempFile = new File(nicDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "nicData", key.toString());
            nicDataFile.writeln("NicID;rxBytesPerSec;rxErrorsPerSec;txBytesPerSec;txErrorsPerSec;txBytesPerSec;txErrorsPerSec; opmNicTotalBytePerSec;opmNicTotalErrorRate; opmNicUtilization");
            for (final RawSample<NicMeasurements> sample : samples) {
                nicDataFile.append(sample.getKey()).append(';');
                final NicMeasurements nicMeasurements = sample.getCounters();
                if (nicMeasurements != null) {
                    nicDataFile.append(String.valueOf(nicMeasurements.getRxBytesRate())).append(';');
                    nicDataFile.append(String.valueOf(nicMeasurements.getRxErrorsRate())).append(';');
                    nicDataFile.append(String.valueOf(nicMeasurements.getTxBytesRate())).append(';');
                    nicDataFile.append(String.valueOf(nicMeasurements.getTxErrorsRate())).append(';');
                    nicDataFile.append(String.valueOf(nicMeasurements.getOpmNicTotalBytePerSec())).append(';');
                    nicDataFile.append(String.valueOf(nicMeasurements.getOpmNicTotalErrorRate())).append(';');
                    nicDataFile.append(String.valueOf(nicMeasurements.getOpmNicUtilization())).append(';');
                }
                nicDataFile.append(crlf);
            }
            nicDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for nic performance data", e);
        }
        finally {
            if (nicDataFile != null) {
                try {
                    nicDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }

    @SuppressWarnings({"ConstantConditions"})
    void dumpVolumeVServerData(final Cluster.ClusterKey key, final Collection<RawSample<VolumeVserverMeasurements>> samples) {
        DSFileWriter vserverDataFile = null;
        try {
            vserverDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                               DataSourceOutputFileType.perfdata,
                                                                                               key.getOcieUUID() + "_volumeVserverdata",
                                                                                               "txt");
            final File tempFile = new File(vserverDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "volumeVserverData", key.toString());
            vserverDataFile.writeln("VserverID;avg_latency;" +
                                    "cifs_other_latency;cifs_other_ops;cifs_read_data;cifs_read_latency;cifs_read_ops;cifs_write_data;cifs_write_latency;cifs_write_ops;" +
                                    "fcp_other_latency;fcp_other_ops;fcp_read_data;fcp_read_latency;fcp_read_ops;fcp_write_data;fcp_write_latency;fcp_write_ops;" +
                                    "iscsi_other_latency;iscsi_other_ops;iscsi_read_data;iscsi_read_latency;iscsi_read_ops;iscsi_write_data;iscsi_write_latency;iscsi_write_ops;" +
                                    "nfs_other_latency;nfs_other_ops;nfs_read_data;nfs_read_latency;nfs_read_ops;nfs_write_data;nfs_write_latency;nfs_write_ops;" +
                                    "san_other_latency;san_other_ops;san_read_data;san_read_latency;san_read_ops;san_write_data;san_write_latency;san_write_ops;" +
                                    "other_latency;other_ops;read_data;read_latency;read_ops;write_data;write_latency;write_ops;total_ops;" +
                                    "fcpLif_avg_other_latency;fcpLif_avg_read_latency;fcpLif_avg_write_latency;fcpLif_other_ops;fcpLif_read_data;fcpLif_read_ops;fcpLif_total_ops;fcpLif_write_data;fcpLif_write_ops;" +
                                    "iscsiLif_avg_other_latency;iscsiLif_avg_read_latency;iscsiLif_avg_write_latency;iscsiLif_data_in_blocks;iscsiLif_data_out_blocks;iscsiLif_other_ops;iscsiLif_read_ops;iscsiLif_write_ops;");
            for (final RawSample<VolumeVserverMeasurements> sample : samples) {
                vserverDataFile.append(sample.getKey()).append(';');
                final VolumeVserverMeasurements vServerMeasurements = sample.getCounters();
                if (vServerMeasurements != null) {
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getAvgLatency())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getCifsOtherLatency())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getCifsOtherOps())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getCifsReadData())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getCifsReadLatency())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getCifsReadOps())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getCifsWriteData())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getCifsWriteLatency())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getCifsWriteOps())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getFcpOtherLatency())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getFcpOtherOps())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getFcpReadData())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getFcpReadLatency())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getFcpReadOps())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getFcpWriteData())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getFcpWriteLatency())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getFcpWriteOps())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getIscsiOtherLatency())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getIscsiOtherOps())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getIscsiReadData())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getIscsiReadLatency())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getIscsiReadOps())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getIscsiWriteData())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getIscsiWriteLatency())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getIscsiWriteOps())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getNfsOtherLatency())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getNfsOtherOps())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getNfsReadData())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getNfsReadLatency())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getNfsReadOps())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getNfsWriteData())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getNfsWriteLatency())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getNfsWriteOps())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getSanOtherLatency())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getSanOtherOps())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getSanReadData())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getSanReadLatency())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getSanReadOps())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getSanWriteData())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getSanWriteLatency())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getSanWriteOps())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getOtherLatency())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getOtherOps())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getReadData())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getReadLatency())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getReadOps())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getWriteData())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getWriteLatency())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getWriteOps())).append(';');
                    vserverDataFile.append(String.valueOf(vServerMeasurements.getTotalOps())).append(';');
                }

                vserverDataFile.append(crlf);
            }
            vserverDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (vserverDataFile != null) {
                try {
                    vserverDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }

    @SuppressWarnings({"ConstantConditions"})
    void dumpProcessorData(final Cluster.ClusterKey key, final Collection<RawSample<ProcessorMeasurements>> samples) {
        DSFileWriter dataFile = null;
        try {
            dataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                        DataSourceOutputFileType.perfdata,
                                                                                        key.getOcieUUID() + "_processordata",
                                                                                        "txt");
            final File tempFile = new File(dataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "processorData", key.toString());
            dataFile.writeln("ProcessorID;processor_busy;nblade_busy;dblade_busy;");
            for (final RawSample<ProcessorMeasurements> sample : samples) {
                dataFile.append(sample.getKey()).append(';');
                final ProcessorMeasurements processorMeasurements = sample.getCounters();
                if (processorMeasurements != null) {
                    dataFile.append(String.valueOf(processorMeasurements.getProcessorBusy())).append(';');
                    dataFile.append(String.valueOf(processorMeasurements.getNBladeBusy())).append(';');
                    dataFile.append(String.valueOf(processorMeasurements.getDBladeBusy())).append(';');
                }
                dataFile.append(crlf);
            }
            dataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for Processor performance data", e);
        }
        finally {
            if (dataFile != null) {
                try {
                    dataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }

    @SuppressWarnings({"ConstantConditions"})
    void dumpQosVolumeWorkloadData(final Cluster.ClusterKey key, final Collection<RawSample<QosVolumeWorkloadMeasurements>> samples) {
        if (samples.size() == 0) {
            return;
        }
        DSFileWriter workloadDataFile = null;
        try {
            workloadDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                                DataSourceOutputFileType.perfdata,
                                                                                                key.getOcieUUID() + "_workloaddata",
                                                                                                "txt");
            final File tempFile = new File(workloadDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "workloadData", key.toString());
            // NetappVolumeMetrics 
            workloadDataFile.writeln("workloadID;latency;ops;total_data;read_data;read_latency;read_ops;write_data;write_latency;write_ops;" +
                                     "cache_miss_rate;");

            for (final RawSample<QosVolumeWorkloadMeasurements> sample : samples) {
                workloadDataFile.append(sample.getKey()).append(';');
                final QosVolumeWorkloadMeasurements workloadMeasurements = sample.getCounters();
                if (workloadMeasurements != null) {
                    workloadDataFile.append(String.valueOf(workloadMeasurements.getLatency())).append(';');
                    workloadDataFile.append(String.valueOf(workloadMeasurements.getOps())).append(';');
                    workloadDataFile.append(String.valueOf(workloadMeasurements.getTotalData())).append(';');
                    workloadDataFile.append(String.valueOf(workloadMeasurements.getReadData())).append(';');
                    workloadDataFile.append(String.valueOf(workloadMeasurements.getReadLatency())).append(';');
                    workloadDataFile.append(String.valueOf(workloadMeasurements.getReadOps())).append(';');
                    workloadDataFile.append(String.valueOf(workloadMeasurements.getWriteData())).append(';');
                    workloadDataFile.append(String.valueOf(workloadMeasurements.getWriteLatency())).append(';');
                    workloadDataFile.append(String.valueOf(workloadMeasurements.getWriteOps())).append(';');
                    workloadDataFile.append(String.valueOf(workloadMeasurements.getCacheMissRate())).append(';');
                }
                workloadDataFile.append(crlf);
            }
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (workloadDataFile != null) {
                try {
                    workloadDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }
    
    	  @SuppressWarnings({"ConstantConditions"})
    	    void dumpQosWorkloadDetailData(final Cluster.ClusterKey key, final Collection<RawSample<QosWorkloadDetailMeasurements>> samples) {
    	        if (samples.size() == 0) {
    	            return;
    	        }
    	        DSFileWriter workloadDetailDataFile = null;
    	        try {
    	            workloadDetailDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
    	                                                                                                      DataSourceOutputFileType.perfdata,
    	                                                                                                      key.getOcieUUID() + "_workloaddetaildata",
    	                                                                                                      "txt");
    	            final File tempFile = new File(workloadDetailDataFile.getFileName());
    	            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "workloadDetailData", key.toString());
    	            // NetappVolumeMetrics                                                                                                                                                                                         workloadDetailDataFile.writeln("workloadDetailID;service_time;wait_time;visits;in_latency_path");
    	            workloadDetailDataFile.writeln("workloadDetailID;service_time;wait_time;collection_period");
    	            
    	            for (final RawSample<QosWorkloadDetailMeasurements> sample : samples) {
    	                workloadDetailDataFile.append(sample.getKey()).append(";");
    	                final QosWorkloadDetailMeasurements workloadDetailMeasurements = sample.getCounters();
    	                if (workloadDetailMeasurements != null) {
    	                    workloadDetailDataFile.append(String.valueOf(workloadDetailMeasurements.getServiceTime())).append(";");
    	                    workloadDetailDataFile.append(String.valueOf(workloadDetailMeasurements.getWaitTime())).append(";");
    	                    workloadDetailDataFile.append(String.valueOf(workloadDetailMeasurements.getCollectionPeriod())).append(";");
    	                }
    	                workloadDetailDataFile.append(crlf);
    	            }
    	        }
    	        catch (IOException e) {
    	            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
    	        }
    	        finally {
    	            try {
    	                if (workloadDetailDataFile != null) {
    	                    workloadDetailDataFile.close();
    	                }
    	            }
    	            catch (IOException ioe) {
    	                //fall through
    	            }
    	        }
    	    }

    @SuppressWarnings({"ConstantConditions"})
    void dumpQosWorkloadQueueDBladeData(final Cluster.ClusterKey key, final Collection<RawSample<QosWorkloadQueueDBladeMeasurements>> samples) {
        if (samples.size() == 0) {
            return;
        }
        DSFileWriter workloadQueueDBladeDataFile = null;
        try {
            workloadQueueDBladeDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                    DataSourceOutputFileType.perfdata,
                    key.getOcieUUID() + "_workload_queue_dblade_data",
                    "txt");
            final File tempFile = new File(workloadQueueDBladeDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "workloadQueueDBladeData", key.toString());
            workloadQueueDBladeDataFile.writeln("workloadQueueDBladeID;dirty_buffers;freed_blocks;read_done;" +
                    "readahead_x_stripe;cpu_dblade_residence_time;cpu_dblade_util_service_time;" +
                    "cpu_dblade_background_service_time;disk_hdd_background_service_time;" +
                    "disk_ssd_background_service_time;delay_wafl_susp_cp_wait_time;" +
                    "delay_disk_io_wait_time;delay_nvlog_transfer_wait_time;cache_miss_rate;CollectionPeriod");

            for (final RawSample<QosWorkloadQueueDBladeMeasurements> sample : samples) {
                workloadQueueDBladeDataFile.append(sample.getKey()).append(';');
                final QosWorkloadQueueDBladeMeasurements workloadQueueDBladeMeasurements = sample.getCounters();
                if (workloadQueueDBladeMeasurements != null) {
                    workloadQueueDBladeDataFile.append(String.valueOf(workloadQueueDBladeMeasurements.getCpuDBladeResidenceTime())).append(';');
                    workloadQueueDBladeDataFile.append(String.valueOf(workloadQueueDBladeMeasurements.getCpuDBladeUtilServiceTime())).append(';');
                    workloadQueueDBladeDataFile.append(String.valueOf(workloadQueueDBladeMeasurements.getCpuDBladeBackgroundServiceTime())).append(';');
                    workloadQueueDBladeDataFile.append(String.valueOf(workloadQueueDBladeMeasurements.getDiskHDDBackgroundServiceTime())).append(';');
                    workloadQueueDBladeDataFile.append(String.valueOf(workloadQueueDBladeMeasurements.getDiskSSDBackgroundServiceTime())).append(';');
                    workloadQueueDBladeDataFile.append(String.valueOf(workloadQueueDBladeMeasurements.getDelayWAFLSuspCPWaitTime())).append(';');
                    workloadQueueDBladeDataFile.append(String.valueOf(workloadQueueDBladeMeasurements.getDelayDiskIOWaitTime())).append(';');
                    workloadQueueDBladeDataFile.append(String.valueOf(workloadQueueDBladeMeasurements.getDelayNvlogTransferWaitTime())).append(';');
                    workloadQueueDBladeDataFile.append(String.valueOf(workloadQueueDBladeMeasurements.getCacheMissRate())).append(';');
                    workloadQueueDBladeDataFile.append(String.valueOf(workloadQueueDBladeMeasurements.getCollectionPeriod())).append(';');
                }
                workloadQueueDBladeDataFile.append(crlf);
            }
        } catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        } finally {
            if (workloadQueueDBladeDataFile != null) {
                try {
                    workloadQueueDBladeDataFile.close();
                } catch (IOException ioe) {
                }
            }
        }
    }

    @SuppressWarnings({"ConstantConditions"})
    void dumpQosWorkloadQueueNBladeData(final Cluster.ClusterKey key, final Collection<RawSample<QosWorkloadQueueNBladeMeasurements>> samples) {
        if (samples.size() == 0) {
            return;
        }
        DSFileWriter workloadQueueNBladeDataFile = null;
        try {
            workloadQueueNBladeDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                    DataSourceOutputFileType.perfdata,
                    key.getOcieUUID() + "_workload_queue_nblade_data",
                    "txt");
            final File tempFile = new File(workloadQueueNBladeDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "workloadQueueNBladeData", key.toString());
            workloadQueueNBladeDataFile.writeln("workloadQueueNBladeID;read_iops;" +
                    "write_iops;other_iops;read_latency;write_latency;other_latency;read_data;write_data;" +
                    "cpu_nblade_residence_time;cpu_nblade_util_service_time;delay_cluster_interconnect_wait_time;" +
                    "delay_qos_limit_wait_time;delay_network_wait_time;CollectionPeriod");

            for (final RawSample<QosWorkloadQueueNBladeMeasurements> sample : samples) {
                workloadQueueNBladeDataFile.append(sample.getKey()).append(';');
                final QosWorkloadQueueNBladeMeasurements workloadQueueNBladeMeasurements = sample.getCounters();
                if (workloadQueueNBladeMeasurements != null) {
                    workloadQueueNBladeDataFile.append(String.valueOf(workloadQueueNBladeMeasurements.getReadOps())).append(';');
                    workloadQueueNBladeDataFile.append(String.valueOf(workloadQueueNBladeMeasurements.getWriteOps())).append(';');
                    workloadQueueNBladeDataFile.append(String.valueOf(workloadQueueNBladeMeasurements.getOtherOps())).append(';');
                    workloadQueueNBladeDataFile.append(String.valueOf(workloadQueueNBladeMeasurements.getReadLatency())).append(';');
                    workloadQueueNBladeDataFile.append(String.valueOf(workloadQueueNBladeMeasurements.getWriteLatency())).append(';');
                    workloadQueueNBladeDataFile.append(String.valueOf(workloadQueueNBladeMeasurements.getOtherLatency())).append(';');
                    workloadQueueNBladeDataFile.append(String.valueOf(workloadQueueNBladeMeasurements.getReadData())).append(';');
                    workloadQueueNBladeDataFile.append(String.valueOf(workloadQueueNBladeMeasurements.getWriteData())).append(';');
                    workloadQueueNBladeDataFile.append(String.valueOf(workloadQueueNBladeMeasurements.getCpuNBladeResidenceTime())).append(';');
                    workloadQueueNBladeDataFile.append(String.valueOf(workloadQueueNBladeMeasurements.getCpuNBladeUtilServiceTime())).append(';');
                    workloadQueueNBladeDataFile.append(String.valueOf(workloadQueueNBladeMeasurements.getDelayClusterInterconnectWaitTime())).append(';');
                    workloadQueueNBladeDataFile.append(String.valueOf(workloadQueueNBladeMeasurements.getDelayQosLimitWaitTime())).append(';');
                    workloadQueueNBladeDataFile.append(String.valueOf(workloadQueueNBladeMeasurements.getDelayNetworkWaitTime())).append(';');
                    workloadQueueNBladeDataFile.append(String.valueOf(workloadQueueNBladeMeasurements.getCollectionPeriod())).append(';');
                }
                workloadQueueNBladeDataFile.append(crlf);
            }
        } catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        } finally {
            if (workloadQueueNBladeDataFile != null) {
                try {
                    workloadQueueNBladeDataFile.close();
                } catch (IOException ioe) {
                }
            }
        }
    }

    @SuppressWarnings({"ConstantConditions"})
    void dumpQosServiceCenterData(final Cluster.ClusterKey key, final Collection<RawSample<QosServiceCenterMeasurements>> samples) {
        if (samples.size() == 0) {
            return;
        }
        DSFileWriter resourceDataFile = null;
        try {
            resourceDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                                DataSourceOutputFileType.perfdata,
                                                                                                key.getOcieUUID() + "_servicecenterdata",
                                                                                                "txt");
            final File tempFile = new File(resourceDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "resourceData", key.toString());
            resourceDataFile.writeln("serviceCenterID;utilization;wait_time");

            for (final RawSample<QosServiceCenterMeasurements> sample : samples) {
                resourceDataFile.append(sample.getKey()).append(';');
                final QosServiceCenterMeasurements resourceMeasurements = sample.getCounters();
                if (resourceMeasurements != null) {
                    resourceDataFile.append(String.valueOf(resourceMeasurements.getUtilization())).append(';');
                    resourceDataFile.append(String.valueOf(resourceMeasurements.getWaitTime())).append(';');
                }
                resourceDataFile.append(crlf);
            }
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (resourceDataFile != null) {
                try {
                    resourceDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }

    @SuppressWarnings({"ConstantConditions"})
    void dumpWaflData(final Cluster.ClusterKey key, final Collection<RawSample<WaflMeasurements>> samples) {
        if (samples.size() == 0) {
            return;
        }
        DSFileWriter resourceDataFile = null;
        try {
            resourceDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                                DataSourceOutputFileType.perfdata,
                                                                                                key.getOcieUUID() + "_wafldata",
                                                                                                "txt");
            final File tempFile = new File(resourceDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "resourceData", key.toString());
            resourceDataFile.writeln("nodeID;totalCpMsecs;p2Flush;waflCpBlksRead;cpCount");

            for (final RawSample<WaflMeasurements> sample : samples) {
                resourceDataFile.append(sample.getKey()).append(';');
                final WaflMeasurements resourceMeasurements = sample.getCounters();
                if (resourceMeasurements != null) {
                    resourceDataFile.append(String.valueOf(resourceMeasurements.getTotalCpMsecs())).append(';');
                    resourceDataFile.append(String.valueOf(resourceMeasurements.getP2Flush())).append(';');
                    resourceDataFile.append(String.valueOf(resourceMeasurements.getWaflCpBlksRead())).append(';');
                    resourceDataFile.append(String.valueOf(resourceMeasurements.getCpCount())).append(';');
                }
                resourceDataFile.append(crlf);
            }
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (resourceDataFile != null) {
                try {
                    resourceDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }

    @SuppressWarnings({"ConstantConditions"})
    void dumpWaflHyaAggrData(final Cluster.ClusterKey key, final Collection<RawSample<WaflHyaAggrMeasurements>> samples) {
        if (samples.size() == 0) {
            return;
        }
        DSFileWriter resourceDataFile = null;
        try {
            resourceDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                    DataSourceOutputFileType.perfdata,
                    key.getOcieUUID() + "_wafl_hya_aggr_data",
                    "txt");
            final File tempFile = new File(resourceDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "resourceData", key.toString());
            resourceDataFile.writeln("nodeID;hya_read_hit_latency_average;hya_read_miss_latency_average;hya_write_hdd_latency_average;hya_write_ssd_latency_average;" +
                    "read_miss_blks_rate;read_ops_replaced;read_ops_replaced_percent;ssd_read_cached;ssd_total;ssd_total_used;ssd_write_cached;");

            for (final RawSample<WaflHyaAggrMeasurements> sample : samples) {
                resourceDataFile.append(sample.getKey()).append(';');
                final WaflHyaAggrMeasurements measurements = sample.getCounters();
                if (measurements != null) {
                    resourceDataFile.append(String.valueOf(measurements.getHyaReadHitLatencyAverage())).append(';');
                    resourceDataFile.append(String.valueOf(measurements.getHyaReadMissLatencyAverage())).append(';');
                    resourceDataFile.append(String.valueOf(measurements.getHyaWriteHddLatencyAverage())).append(';');
                    resourceDataFile.append(String.valueOf(measurements.getHyaWriteSsdLatencyAverage())).append(';');
                    resourceDataFile.append(String.valueOf(measurements.getReadMissBlksRate())).append(';');
                    resourceDataFile.append(String.valueOf(measurements.getReadMissBlksRate())).append(';');
                    resourceDataFile.append(String.valueOf(measurements.getReadOpsReplaced())).append(';');
                    resourceDataFile.append(String.valueOf(measurements.getReadOpsReplacedPercent())).append(';');
                    resourceDataFile.append(String.valueOf(measurements.getSsdReadCached())).append(';');
                    resourceDataFile.append(String.valueOf(measurements.getSsdTotal())).append(';');
                    resourceDataFile.append(String.valueOf(measurements.getSsdTotalUsed())).append(';');
                    resourceDataFile.append(String.valueOf(measurements.getSsdWriteCached())).append(';');
                }
                resourceDataFile.append(crlf);
            }
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (resourceDataFile != null) {
                try {
                    resourceDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }


    @SuppressWarnings({"ConstantConditions"})
    void dumpExtCacheObjData(final Cluster.ClusterKey key, final Collection<RawSample<ExtCacheObjMeasurements>> samples) {
        if (samples.size() == 0) {
            return;
        }
        DSFileWriter resourceDataFile = null;
        try {
            resourceDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                                DataSourceOutputFileType.perfdata,
                                                                                                key.getOcieUUID() + "_extcacheobjdata",
                                                                                                "txt");
            final File tempFile = new File(resourceDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "resourceData", key.toString());
            resourceDataFile.writeln("portId;diskReadsReplaced;hitPercent");

            for (final RawSample<ExtCacheObjMeasurements> sample : samples) {
                resourceDataFile.append(sample.getKey()).append(';');
                final ExtCacheObjMeasurements resourceMeasurements = sample.getCounters();
                if (resourceMeasurements != null) {
                    resourceDataFile.append(String.valueOf(resourceMeasurements.getDiskReadsReplaced())).append(';');
                    resourceDataFile.append(String.valueOf(resourceMeasurements.getHitPercent())).append(';');
                }
                resourceDataFile.append(crlf);
            }
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (resourceDataFile != null) {
                try {
                    resourceDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }

    public long getCachedServerTime() {
        return cachedServerTime;
    }

    /**
     * This method returns the time that will used to report the sample.
     *
     *  @return The server time when acquisition started polling.
     */
    public long getSampleTime() {
        return cachedServerTime;
    }

    public void reportCIFSNodePerformance(final Cluster.ClusterKey key, final Collection<RawSample<CIFSNodeMeasurements>> samples) throws DataSourceErrorException {
        reportCIFSNodePerformance(key, samples, -1L);
    }

    public void reportCIFSNodePerformance(final Cluster.ClusterKey key, final Collection<RawSample<CIFSNodeMeasurements>> samples, final long leaseTime) throws DataSourceErrorException {
        if (getDataSource().isStopped()) {
            return;
        }
        if (isRecording() && samples.size() > 0) {
            dumpCIFSNodeData(key, samples);
        }
        if (samples.size() == 0) {
            return;
        }
        checkCIFSNodePerformanceIntegrity(key, samples);
        if (leaseTime == -1) {
            getStoragePerformanceListener().reportCIFSNodePerformance(this, key, getSampleTime(), samples);
        }
        else {
            getStoragePerformanceListener().reportCIFSNodePerformance(this, key, getSampleTime(), samples, leaseTime);
        }
        reportedCIFSNode = true;
    }

    private void checkCIFSNodePerformanceIntegrity(final Cluster.ClusterKey key, final Collection<RawSample<CIFSNodeMeasurements>> samples) throws DataSourceErrorException {
        // No integrity requirements
    }

    @SuppressWarnings({"ConstantConditions"})
    void dumpCIFSNodeData(final Cluster.ClusterKey key, final Collection<RawSample<CIFSNodeMeasurements>> samples) {
        DSFileWriter nodeProtoDataFile = null;
        try {
            nodeProtoDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                    DataSourceOutputFileType.perfdata,
                    key.getOcieUUID() + "_cifsnode_data",
                    "txt");
            final File tempFile = new File(nodeProtoDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "diskData", key.toString());
            nodeProtoDataFile.writeln("NodeID;cifs_latency;cifs_read_latency;cifs_write_latency;cifs_ops;cifs_read_ops;cifs_write_ops;");
            for (final RawSample<CIFSNodeMeasurements> sample : samples) {
                nodeProtoDataFile.append(sample.getKey()).append(';');
                final CIFSNodeMeasurements measurements = sample.getCounters();
                if (measurements != null) {
                    nodeProtoDataFile.append(String.valueOf(measurements.getCifsLatency())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getCifsReadLatency())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getCifsWriteLatency())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getCifsOps())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getCifsReadOps())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getCifsWriteOps())).append(';');
                }
                nodeProtoDataFile.append(crlf);
            }
            nodeProtoDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (nodeProtoDataFile != null) {
                try {
                    nodeProtoDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }

    public void reportSMB1NodePerformance(final Cluster.ClusterKey key, final Collection<RawSample<SMB1NodeMeasurements>> samples) throws DataSourceErrorException {
        reportSMB1NodePerformance(key, samples, -1L);
    }

    public void reportSMB1NodePerformance(final Cluster.ClusterKey key, final Collection<RawSample<SMB1NodeMeasurements>> samples, final long leaseTime) throws DataSourceErrorException {
        if (getDataSource().isStopped()) {
            return;
        }
        if (isRecording() && samples.size() > 0) {
            dumpSMB1NodeData(key, samples);
        }
        if (samples.size() == 0) {
            return;
        }
        checkSMB1NodePerformanceIntegrity(key, samples);
        if (leaseTime == -1) {
            getStoragePerformanceListener().reportSMB1NodePerformance(this, key, getSampleTime(), samples);
        }
        else {
            getStoragePerformanceListener().reportSMB1NodePerformance(this, key, getSampleTime(), samples, leaseTime);
        }
        reportedSMB1Node = true;
    }

    private void checkSMB1NodePerformanceIntegrity(final Cluster.ClusterKey key, final Collection<RawSample<SMB1NodeMeasurements>> samples) throws DataSourceErrorException {
        // No integrity requirements
    }

    void dumpSMB1NodeData(final Cluster.ClusterKey key, final Collection<RawSample<SMB1NodeMeasurements>> samples) {
        DSFileWriter nodeProtoDataFile = null;
        try {
            nodeProtoDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                    DataSourceOutputFileType.perfdata,
                    key.getOcieUUID() + "_smb1node_data",
                    "txt");
            final File tempFile = new File(nodeProtoDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "diskData", key.toString());
            nodeProtoDataFile.writeln("NodeID;latency;read_latency;write_latency;ops;read_ops;write_ops;");
            for (final RawSample<SMB1NodeMeasurements> sample : samples) {
                nodeProtoDataFile.append(sample.getKey()).append(';');
                final SMB1NodeMeasurements measurements = sample.getCounters();
                if (measurements != null) {
                    nodeProtoDataFile.append(String.valueOf(measurements.getLatency())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getReadLatency())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getWriteLatency())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getOps())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getReadOps())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getWriteOps())).append(';');
                }
                nodeProtoDataFile.append(crlf);
            }
            nodeProtoDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (nodeProtoDataFile != null) {
                try {
                    nodeProtoDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }



    public void reportSMB2NodePerformance(final Cluster.ClusterKey key, final Collection<RawSample<SMB2NodeMeasurements>> samples) throws DataSourceErrorException {
        reportSMB2NodePerformance(key, samples, -1L);
    }

    public void reportSMB2NodePerformance(final Cluster.ClusterKey key, final Collection<RawSample<SMB2NodeMeasurements>> samples, final long leaseTime) throws DataSourceErrorException {
        if (getDataSource().isStopped()) {
            return;
        }
        if (isRecording() && samples.size() > 0) {
            dumpSMB2NodeData(key, samples);
        }
        if (samples.size() == 0) {
            return;
        }
        checkSMB2NodePerformanceIntegrity(key, samples);
        if (leaseTime == -1) {
            getStoragePerformanceListener().reportSMB2NodePerformance(this, key, getSampleTime(), samples);
        }
        else {
            getStoragePerformanceListener().reportSMB2NodePerformance(this, key, getSampleTime(), samples, leaseTime);
        }
        reportedSMB2Node = true;
    }

    private void checkSMB2NodePerformanceIntegrity(final Cluster.ClusterKey key, final Collection<RawSample<SMB2NodeMeasurements>> samples) throws DataSourceErrorException {
        // No integrity requirements
    }

    void dumpSMB2NodeData(final Cluster.ClusterKey key, final Collection<RawSample<SMB2NodeMeasurements>> samples) {
        DSFileWriter nodeProtoDataFile = null;
        try {
            nodeProtoDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                    DataSourceOutputFileType.perfdata,
                    key.getOcieUUID() + "_smb2node_data",
                    "txt");
            final File tempFile = new File(nodeProtoDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "diskData", key.toString());
            nodeProtoDataFile.writeln("NodeID;latency;read_latency;write_latency;ops;read_ops;write_ops;");
            for (final RawSample<SMB2NodeMeasurements> sample : samples) {
                nodeProtoDataFile.append(sample.getKey()).append(';');
                final SMB2NodeMeasurements measurements = sample.getCounters();
                if (measurements != null) {
                    nodeProtoDataFile.append(String.valueOf(measurements.getLatency())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getReadLatency())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getWriteLatency())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getOps())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getReadOps())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getWriteOps())).append(';');
                }
                nodeProtoDataFile.append(crlf);
            }
            nodeProtoDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (nodeProtoDataFile != null) {
                try {
                    nodeProtoDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }


    public void reportNFSv3NodePerformance(final Cluster.ClusterKey key, final Collection<RawSample<NFSv3NodeMeasurements>> samples) throws DataSourceErrorException {
        reportNFSv3NodePerformance(key, samples, -1L);
    }

    public void reportNFSv3NodePerformance(final Cluster.ClusterKey key, final Collection<RawSample<NFSv3NodeMeasurements>> samples, final long leaseTime) throws DataSourceErrorException {
        if (getDataSource().isStopped()) {
            return;
        }
        if (isRecording() && samples.size() > 0) {
            dumpNFSv3NodeData(key, samples);
        }
        if (samples.size() == 0) {
            return;
        }
        checkNFSv3NodePerformanceIntegrity(key, samples);
        if (leaseTime == -1) {
            getStoragePerformanceListener().reportNFSv3NodePerformance(this, key, getSampleTime(), samples);
        }
        else {
            getStoragePerformanceListener().reportNFSv3NodePerformance(this, key, getSampleTime(), samples, leaseTime);
        }
        reportedNFSv3 = true;
    }

    private void checkNFSv3NodePerformanceIntegrity(final Cluster.ClusterKey key, final Collection<RawSample<NFSv3NodeMeasurements>> samples) throws DataSourceErrorException {
        // No integrity requirements
    }

    void dumpNFSv3NodeData(final Cluster.ClusterKey key, final Collection<RawSample<NFSv3NodeMeasurements>> samples) {
        DSFileWriter nodeProtoDataFile = null;
        try {
            nodeProtoDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                    DataSourceOutputFileType.perfdata,
                    key.getOcieUUID() + "_nfsv3node_data",
                    "txt");
            final File tempFile = new File(nodeProtoDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "diskData", key.toString());
            nodeProtoDataFile.writeln("NodeID;read_avg_latency;write_avg_latency;nfsv3_ops;nfsv3_read_ops;nfsv3_write_ops;");
            for (final RawSample<NFSv3NodeMeasurements> sample : samples) {
                nodeProtoDataFile.append(sample.getKey()).append(';');
                final NFSv3NodeMeasurements measurements = sample.getCounters();
                if (measurements != null) {
                    nodeProtoDataFile.append(String.valueOf(measurements.getReadAvgLatency())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getWriteAvgLatency())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getNFSv3Ops())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getNFSv3ReadOps())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getNFSv3WriteOps())).append(';');
                }
                nodeProtoDataFile.append(crlf);
            }
            nodeProtoDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (nodeProtoDataFile != null) {
                try {
                    nodeProtoDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }



    public void reportNFSv4NodePerformance(final Cluster.ClusterKey key, final Collection<RawSample<NFSv4NodeMeasurements>> samples) throws DataSourceErrorException {
        reportNFSv4NodePerformance(key, samples, -1L);
    }

    public void reportNFSv4NodePerformance(final Cluster.ClusterKey key, final Collection<RawSample<NFSv4NodeMeasurements>> samples, final long leaseTime) throws DataSourceErrorException {
        if (getDataSource().isStopped()) {
            return;
        }
        if (isRecording() && samples.size() > 0) {
            dumpNFSv4NodeData(key, samples);
        }
        if (samples.size() == 0) {
            return;
        }
        checkNFSv4NodePerformanceIntegrity(key, samples);
        if (leaseTime == -1) {
            getStoragePerformanceListener().reportNFSv4NodePerformance(this, key, getSampleTime(), samples);
        }
        else {
            getStoragePerformanceListener().reportNFSv4NodePerformance(this, key, getSampleTime(), samples, leaseTime);
        }
        reportedNFSv4 = true;
    }

    private void checkNFSv4NodePerformanceIntegrity(final Cluster.ClusterKey key, final Collection<RawSample<NFSv4NodeMeasurements>> samples) throws DataSourceErrorException {
        // No integrity requirements
    }

    void dumpNFSv4NodeData(final Cluster.ClusterKey key, final Collection<RawSample<NFSv4NodeMeasurements>> samples) {
        DSFileWriter nodeProtoDataFile = null;
        try {
            nodeProtoDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                    DataSourceOutputFileType.perfdata,
                    key.getOcieUUID() + "_nfsv4node_data",
                    "txt");
            final File tempFile = new File(nodeProtoDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "diskData", key.toString());
            nodeProtoDataFile.writeln("NodeID;read_avg_latency;write_avg_latency;nfsv4_ops;read_total;write_total;");
            for (final RawSample<NFSv4NodeMeasurements> sample : samples) {
                nodeProtoDataFile.append(sample.getKey()).append(';');
                final NFSv4NodeMeasurements measurements = sample.getCounters();
                if (measurements != null) {
                    nodeProtoDataFile.append(String.valueOf(measurements.getReadAvgLatency())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getWriteAvgLatency())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getNFSv4Ops())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getReadTotal())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getWriteTotal())).append(';');
                }
                nodeProtoDataFile.append(crlf);
            }
            nodeProtoDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (nodeProtoDataFile != null) {
                try {
                    nodeProtoDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }


    public void reportNFSv41NodePerformance(final Cluster.ClusterKey key, final Collection<RawSample<NFSv41NodeMeasurements>> samples) throws DataSourceErrorException  {
        reportNFSv41NodePerformance(key, samples, -1L);
    }

    public void reportNFSv41NodePerformance(final Cluster.ClusterKey key, final Collection<RawSample<NFSv41NodeMeasurements>> samples, final long leaseTime) throws DataSourceErrorException  {
        if (getDataSource().isStopped()) {
            return;
        }
        if (isRecording() && samples.size() > 0) {
            dumpNFSv41NodeData(key, samples);
        }
        if (samples.size() == 0) {
            return;
        }
        checkNFSv41NodePerformanceIntegrity(key, samples);
        if (leaseTime == -1) {
            getStoragePerformanceListener().reportNFSv41NodePerformance(this, key, getSampleTime(), samples);
        }
        else {
            getStoragePerformanceListener().reportNFSv41NodePerformance(this, key, getSampleTime(), samples, leaseTime);
        }
        reportedNFSv41 = true;
    }

    private void checkNFSv41NodePerformanceIntegrity(final Cluster.ClusterKey key, final Collection<RawSample<NFSv41NodeMeasurements>> samples) throws DataSourceErrorException {
        // No integrity requirements
    }

    void dumpNFSv41NodeData(final Cluster.ClusterKey key, final Collection<RawSample<NFSv41NodeMeasurements>> samples) {
        DSFileWriter nodeProtoDataFile = null;
        try {
            nodeProtoDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                    DataSourceOutputFileType.perfdata,
                    key.getOcieUUID() + "_nfsv4node_data",
                    "txt");
            final File tempFile = new File(nodeProtoDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "diskData", key.toString());
            nodeProtoDataFile.writeln("NodeID;read_avg_latency;write_avg_latency;nfsv4_1_ops;read_total;write_total;");
            for (final RawSample<NFSv41NodeMeasurements> sample : samples) {
                nodeProtoDataFile.append(sample.getKey()).append(';');
                final NFSv41NodeMeasurements measurements = sample.getCounters();
                if (measurements != null) {
                    nodeProtoDataFile.append(String.valueOf(measurements.getReadAvgLatency())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getWriteAvgLatency())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getNFSv41Ops())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getReadTotal())).append(';');
                    nodeProtoDataFile.append(String.valueOf(measurements.getWriteTotal())).append(';');
                }
                nodeProtoDataFile.append(crlf);
            }
            nodeProtoDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (nodeProtoDataFile != null) {
                try {
                    nodeProtoDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }
   
    public final void reportHostadapterPerformance(final Cluster.ClusterKey key, final Collection<RawSample<HostadapterMeasurements>> samples){

        if (getDataSource().isStopped()) {
            return;
        }

        if (isRecording() && samples.size() > 0) {
            dumpHostadapterData(key, samples);
        }

        if (samples.size() == 0) {
            return;
        }

        getStoragePerformanceListener().reportHostadapterPerformance(this, key, getSampleTime(), samples, leaseTimeMS);
        reportedHostadapter = true;
    }
    
    @SuppressWarnings({"ConstantConditions"})
    void dumpHostadapterData(final Cluster.ClusterKey key, final Collection<RawSample<HostadapterMeasurements>> samples) {
        DSFileWriter hostadapterDataFile = null;
        try {
            hostadapterDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                                                                                              DataSourceOutputFileType.perfdata,
                                                                                              key.getOcieUUID() + "_hostadapterdata",
                                                                                              "txt");
            final File tempFile = new File(hostadapterDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "hostadapterData", key.toString());
            hostadapterDataFile.writeln("bytes_read,bytes_written,max_link_datarate");
            for (final RawSample<HostadapterMeasurements> sample : samples) {
                hostadapterDataFile.append(sample.getKey()).append(';');
                final HostadapterMeasurements hostadapterMeasurements = sample.getCounters();
                if (hostadapterMeasurements != null) {
                    hostadapterDataFile.append(String.valueOf(hostadapterMeasurements.getBytesRead())).append(';');
                    hostadapterDataFile.append(String.valueOf(hostadapterMeasurements.getBytesWritten())).append(';');
                    hostadapterDataFile.append(String.valueOf(hostadapterMeasurements.getMaxLinkDataRate())).append(';');
                }
                hostadapterDataFile.append(crlf);
            }
            hostadapterDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for  hostadapter performance data", e);
        }
        finally {
            if (hostadapterDataFile != null) {
                try {
                    hostadapterDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }
    
    public void reportClusterPerformance(final Cluster.ClusterKey key, final Collection<RawSample<ClusterMeasurements>> samples) throws DataSourceErrorException {
        reportClusterPerformance(key, samples, leaseTimeMS);
    }

    public void reportClusterPerformance(final Cluster.ClusterKey key, final Collection<RawSample<ClusterMeasurements>> samples, final long leaseTime) throws DataSourceErrorException {
        if (getDataSource().isStopped()) {
            return;
        }
        if (isRecording() && samples.size() > 0) {
            dumpClusterData(key, samples);
        }
        if (samples.size() == 0) {
            return;
        }
        checkClusterPerformanceIntegrity(key, samples);
        if (leaseTime == -1) {
            getStoragePerformanceListener().reportClusterPerformance(this, key, getSampleTime(), samples);
        }
        else {
            getStoragePerformanceListener().reportClusterPerformance(this, key, getSampleTime(), samples, leaseTime);
        }
        reportedNFSv3 = true;
    }

    private void checkClusterPerformanceIntegrity(final Cluster.ClusterKey key, final Collection<RawSample<ClusterMeasurements>> samples) throws DataSourceErrorException {
        // No integrity requirements
    }

    void dumpClusterData(final Cluster.ClusterKey key, final Collection<RawSample<ClusterMeasurements>> samples) {
        DSFileWriter clusterProtoDataFile = null;
        try {
            clusterProtoDataFile = getRecordingSession().getCurrentSample().createTextualOutputFile(getConfiguration(),
                    DataSourceOutputFileType.perfdata,
                    key.getOcieUUID() + "_cluster_data",
                    "txt");
            final File tempFile = new File(clusterProtoDataFile.getFileName());
            getRecordingSession().getCurrentSample().addFileDescription(tempFile.getName(), "clusterData", key.toString());
            clusterProtoDataFile.writeln("clusterID;cluster_ops;cluster_throughput;cluster_max_node_utilization;cluster_max_aggregate_utilization;");
            for (final RawSample<ClusterMeasurements> sample : samples) {
                clusterProtoDataFile.append(sample.getKey()).append(';');
                final ClusterMeasurements measurements = sample.getCounters();
                if (measurements != null) {
                    clusterProtoDataFile.append(String.valueOf(measurements.getClusterOps())).append(';');
                    clusterProtoDataFile.append(String.valueOf(measurements.getClusterThroughput())).append(';');
                    clusterProtoDataFile.append(String.valueOf(measurements.getClusterMaxNodeUtilization())).append(';');
                    clusterProtoDataFile.append(String.valueOf(measurements.getClusterMaxAggregateUtilization())).append(';');
                }
                clusterProtoDataFile.append(crlf);
            }
            clusterProtoDataFile.close();
        }
        catch (IOException e) {
            LOG.error(getLogPrefix() + " Failed to create a recording file for performance data", e);
        }
        finally {
            if (clusterProtoDataFile != null) {
                try {
                    clusterProtoDataFile.close();
                }
                catch (IOException ioe) {
                    //fall through
                }
            }
        }
    }




}

